#include "preproc.h"
c     ******************************************************************
c     * MCCCS - Towhee: A Monte Carlo molecular simulation program     *
c     * Copyright (C) 2006-2011 Marcus G. Martin                       *
c     * see the file license.gpl for the full license information      *
c     *                                                                *
c     * This program is free software; you can redistribute it and/or  *
c     * modify it under the terms of the GNU General Public License    *
c     * as published by the Free Software Foundation; either version 2 *
c     * of the License, or (at your option) any later version.         *
c     *                                                                *
c     * This program is distributed in the hope that it will be useful,*
c     * but WITHOUT ANY WARRANTY; without even the implied warranty of *
c     * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  *
c     * GNU General Public License for more details.                   *
c     *                                                                *
c     * You should have received a copy of the GNU General Public      *
c     * License along with this program; if not, write to the Free     *
c     * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,*
c     * MA  02111-1307, USA.                                           *
c     ******************************************************************
      subroutine twh_ffmmff94(lfailure,ntypes,nbondtypes,nangletypes
     &     ,ndihedtypes,nimprotypes,nbitypes)
c     ******************************************************************
c     * sets up all of the force field parameters for the mmff94       *
c     * forcefield                                                     *
c     *                                                                *
c     * originally written 12-07-2006 by M.G. Martin                   *
c     * last modified 08-04-2011 by M.G. Martin                        *
c     ******************************************************************
      implicit none
c     --- variables passed to/from the subroutine
      logical lfailure
      integer ntypes,nbondtypes,nangletypes,ndihedtypes,nimprotypes
     &     ,nbitypes
c     --- local variables
      character*(FFNAMELEN) ffname

      ffname = 'MMFF94'

      call twh_classical_potential(GLB_SET,'Buffered 14-7')
      call twh_classical_mixrule(GLB_SET,'MMFF')

      call twh_ffmmff94_nonbond(lfailure,ffname,ntypes)
      if ( lfailure ) return
      call twh_ffmmff94_bond(lfailure,ffname,nbondtypes)
      if ( lfailure ) return
      call twh_ffmmff94_angle(lfailure,ffname,nangletypes)
      if ( lfailure ) return
      call twh_ffmmff94_dihedral(lfailure,ffname,ndihedtypes)
      if ( lfailure ) return
      call twh_ffmmff94_improper(lfailure,ffname,nimprotypes)
      if ( lfailure ) return
      call twh_ffmmff94_bondincrement(lfailure,ffname,nbitypes)
      if ( lfailure ) return

      return
      end


      subroutine twh_ffmmff94_nonbond(lfailure,ffname,ntypes)
c     ******************************************************************
c     * sets up the MMFF94 nonbonded parameters                        *
c     *                                                                *
c     * originally written 12-07-2006 by M.G. Martin                   *
c     * last modified 10-14-2008 by M.G. Martin                        *
c     ******************************************************************
      implicit none
#define FUNCTION_GETNBTYPE
#include "functions.h"
c     --- variables passed to/from the subroutine
      logical lfailure
      character*(FFNAMELEN) ffname
      integer ntypes
c     --- local variables
      integer type,ii,idum
      double precision nbclocal
      dimension nbclocal(MINNBCOEFF:MAXNBCOEFF)
      double precision masslocal,charge
      character*1 daflag
      character*2 elelocal 
      character*5 bplocal
      character*(FFNAMELEN) nbnlocal,canlocal,cbnlocal,ctnlocal

c     --- initialize type count
      type = 0

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.050d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     --- ALKYL CARBON, SP3                                 
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CR  '
      cbnlocal = ' 1'
      canlocal = ' 1 1 1 1 0'
      ctnlocal = ' 1 1 1 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.350d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  VINYLIC CARBON, SP2                              
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'C=C '
      cbnlocal = ' 2'
      canlocal = ' 2 2 2 1 0'
      ctnlocal = ' 2 2 2 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.350d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  GENERIC SP2 CARBON                               
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CSP2'
      cbnlocal = ' 2'
      canlocal = ' 2 2 2 1 0'
      ctnlocal = ' 2 2 2 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  GENERAL CARBONYL CARBON                          
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'C=O '
      cbnlocal = ' 3'
      canlocal = ' 3 3 3 1 0'
      ctnlocal = ' 3 3 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  SP2 CARBON IN C=N                                
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'C=N '
      cbnlocal = ' 3'
      canlocal = ' 3 3 3 1 0'
      ctnlocal = ' 3 3 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  GUANIDINE CARBON, DOUBLY BONDED TO N             
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CGD '
      cbnlocal = ' 3'
      canlocal = ' 3 3 3 1 0'
      ctnlocal = ' 3 3 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  KETONE OR ALDEHYDE CARBONYL CARBON               
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'C=OR'
      cbnlocal = ' 3'
      canlocal = ' 3 3 3 1 0'
      ctnlocal = ' 3 3 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  AMIDE CARBONYL CARBON                            
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'C=ON'
      cbnlocal = ' 3'
      canlocal = ' 3 3 3 1 0'
      ctnlocal = ' 3 3 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  UREA CARBONYL CARBON                             
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CONN'
      cbnlocal = ' 3'
      canlocal = ' 3 3 3 1 0'
      ctnlocal = ' 3 3 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  CARBOXYLIC ACID OR ESTER CARBONYL CARBON         
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'COO '
      cbnlocal = ' 3'
      canlocal = ' 3 3 3 1 0'
      ctnlocal = ' 3 3 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  CARBAMATE CARBONYL CARBON                        
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'COON'
      cbnlocal = ' 3'
      canlocal = ' 3 3 3 1 0'
      ctnlocal = ' 3 3 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  C ARBONIC ACID OR ESTER CARBONYL CARBON          
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'COOO'
      cbnlocal = ' 3'
      canlocal = ' 3 3 3 1 0'
      ctnlocal = ' 3 3 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  THIOESTER CARBONYL CARBON, DOUBLE BONDED TO O    
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'C=OS'
      cbnlocal = ' 3'
      canlocal = ' 3 3 3 1 0'
      ctnlocal = ' 3 3 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  THIOESTER CARBON, DOUBLY BONDED TO S             
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'C=S '
      cbnlocal = ' 3'
      canlocal = ' 3 3 3 1 0'
      ctnlocal = ' 3 3 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  THIOAMIDE, CARBON, DOUBLY BONDED TO S            
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'C=SN'
      cbnlocal = ' 3'
      canlocal = ' 3 3 3 1 0'
      ctnlocal = ' 3 3 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  CARBON IN >C=SO2                                 
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CSO2'
      cbnlocal = ' 3'
      canlocal = ' 3 3 3 1 0'
      ctnlocal = ' 3 3 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  CARBON IN >C=S=O (SULFINYL GROUP)                
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CS=O'
      cbnlocal = ' 3'
      canlocal = ' 3 3 3 1 0'
      ctnlocal = ' 3 3 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  THIOCARBOXYLIC ACID OR ESTER CARBONYL CARBON     
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CSS '
      cbnlocal = ' 3'
      canlocal = ' 3 3 3 1 0'
      ctnlocal = ' 3 3 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  CARBON DOUBLE BONDED TO PHOSPHOROUS              
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'C=P '
      cbnlocal = ' 3'
      canlocal = ' 3 3 3 1 0'
      ctnlocal = ' 3 3 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.300d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  ACETYLENIC CARBON                                
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CSP '
      cbnlocal = ' 4'
      canlocal = ' 4 4 4 1 0'
      ctnlocal = ' 4 4 4 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.300d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  ALLENIC CARBON                                   
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = '=C= '
      cbnlocal = ' 4'
      canlocal = ' 4 4 4 1 0'
      ctnlocal = ' 4 4 4 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.250d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H  ATTACHED TO C                                 
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HC  '
      cbnlocal = ' 5'
      canlocal = ' 5 5 5 5 0'
      ctnlocal = ' 5 5 5 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.250d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H ATTACHED TO SI                                 
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HSI '
      cbnlocal = ' 5'
      canlocal = ' 5 5 5 5 0'
      ctnlocal = ' 5 5 5 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  ALCOHOL OR ETHER OXYGEN                          
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OR  '
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  ESTER OR CARBOXYLIC ACID -O-                     
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OC=O'
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  ENOLIC OR PHENOLIC OXYGEN                        
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OC=C'
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  DIVALENT OXYGEN                                  
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OC=N'
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  THIOESTER OR THIOACID -O-                        
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OC=S'
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  DIVALENT NITRATE "ETHER" OXYGEN                  
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'ONO2'
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  DIVALENT NITRITE "ETHER" OXYGEN                  
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'ON=O'
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  DIVALENT OXYGEN ATTACHED TO SULFUR               
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OSO3'
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  DIVALENT OXYGEN ATTACHED TO SULFUR               
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OSO2'
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  DIVALENT OXYGEN ATTACHED TO SULFUR               
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OSO '
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  DIVALENT OXYGEN ATTACHED TO SULFOXIDE SULFUR     
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OS=O'
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  GENERAL DIVALENT OX ATTACHED TO S                
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = '-OS '
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  DIVALENT OXYGEN ATTACHED TO PHOSPHOROUS          
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OPO3'
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  DIVALENT OXYGEN ATTACHED TO PHOSPHOROUS          
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OPO2'
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  DIVALENT OXYGEN ATTACHED TO PHOSPHOROUS          
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OPO '
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  DIVALENT OXYGEN ATTACHED TO PHOSPHOROUS          
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = '-OP '
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.700d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  GENERAL DIVALENT O                               
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = '-O- '
      cbnlocal = ' 6'
      canlocal = ' 6 6 6 6 0'
      ctnlocal = ' 6 6 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.650d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  GENERAL C=O                                      
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O=C '
      cbnlocal = ' 7'
      canlocal = ' 7 7 7 6 0'
      ctnlocal = ' 7 7 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.650d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  CARBONYL OXYGEN, AMIDES                          
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O=CN'
      cbnlocal = ' 7'
      canlocal = ' 7 7 7 6 0'
      ctnlocal = ' 7 7 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.650d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  CARBONYL OXYGEN, ALDEHYDES AND KETONES           
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O=CR'
      cbnlocal = ' 7'
      canlocal = ' 7 7 7 6 0'
      ctnlocal = ' 7 7 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.650d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  CARBONYL OXYGEN, CARBOXYLIC ACIDS AND ESTERS     
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O=CO'
      cbnlocal = ' 7'
      canlocal = ' 7 7 7 6 0'
      ctnlocal = ' 7 7 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.650d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  NITROSO OXYGEN                                   
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O=N '
      cbnlocal = ' 7'
      canlocal = ' 7 7 7 6 0'
      ctnlocal = ' 7 7 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.650d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  O=S IN SULFOXIDES                                
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O=S '
      cbnlocal = ' 7'
      canlocal = ' 7 7 7 6 0'
      ctnlocal = ' 7 7 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.650d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  O=S ON SULFUR DOUBLY BONDED TO, E.G., CARBON     
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O=S='
      cbnlocal = ' 7'
      canlocal = ' 7 7 7 6 0'
      ctnlocal = ' 7 7 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.150d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  NITROGEN IN ALIPHATIC AMINES                     
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NR  '
      cbnlocal = ' 8'
      canlocal = ' 8 8 8 8 0'
      ctnlocal = ' 8 8 8 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.900d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  NITROGEN IN IMINES                               
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N=C '
      cbnlocal = ' 9'
      canlocal = ' 9 9 9 8 0'
      ctnlocal = ' 9 9 9 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.900d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  NITROGEN IN AZO COMPOUNDS                        
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N=N '
      cbnlocal = ' 9'
      canlocal = ' 9 9 9 8 0'
      ctnlocal = ' 9 9 9 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITROGEN IN AMIDES                               
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NC=O'
      cbnlocal = '10'
      canlocal = '101010 8 0'
      ctnlocal = '101010 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITROGEN IN N-C=S, THIOAMIDE                     
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NC=S'
      cbnlocal = '10'
      canlocal = '101010 8 0'
      ctnlocal = '101010 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITROGEN IN N-N=C                                
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NN=C'
      cbnlocal = '10'
      canlocal = '101010 8 0'
      ctnlocal = '101010 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITROGEN IN N-N=N                                
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NN=N'
      cbnlocal = '10'
      canlocal = '101010 8 0'
      ctnlocal = '101010 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.350d0
      nbclocal(2) = 3.480d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  FLUORINE                                         
      elelocal = ' F'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'F   '
      cbnlocal = '11'
      canlocal = '11111111 0'
      ctnlocal = '11111111 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 2.300d0
      nbclocal(2) = 5.100d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  CHLORINE                                         
      elelocal = 'Cl'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CL  '
      cbnlocal = '12'
      canlocal = '12121212 0'
      ctnlocal = '12121212 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 3.400d0
      nbclocal(2) = 6.000d0
      nbclocal(3) = 3.190d0
      nbclocal(4) = 1.359d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  BROMINE                                          
      elelocal = 'Br'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'BR  '
      cbnlocal = '13'
      canlocal = '13131313 0'
      ctnlocal = '13131313 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 5.500d0
      nbclocal(2) = 6.950d0
      nbclocal(3) = 3.080d0
      nbclocal(4) = 1.404d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  IODINE                                           
      elelocal = ' I'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'I   '
      cbnlocal = '14'
      canlocal = '14141414 0'
      ctnlocal = '14141414 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 3.000d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  SULFUR IN THIOETHERS AND MERCAPTANS              
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'S   '
      cbnlocal = '15'
      canlocal = '15151515 0'
      ctnlocal = '15151515 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 3.900d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  TERMINAL SULFUR DOUBLY BONDED TO CARBON          
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'S=C '
      cbnlocal = '16'
      canlocal = '16161615 0'
      ctnlocal = '16161615 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 2.700d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  SULFUR IN SULFOXIDES                             
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'S=O '
      cbnlocal = '17'
      canlocal = '17171715 0'
      ctnlocal = '17171715 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 2.700d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  SULFUR, TRICOORD, DOUBLY BONDED TO N             
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = '>S=N'
      cbnlocal = '17'
      canlocal = '17171715 0'
      ctnlocal = '17171715 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 2.100d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  SULFUR IN SULFONES                               
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'SO2 '
      cbnlocal = '18'
      canlocal = '18181815 0'
      ctnlocal = '18181815 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 2.100d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  SULFUR IN SULFONAMIDES                           
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'SO2N'
      cbnlocal = '18'
      canlocal = '18181815 0'
      ctnlocal = '18181815 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 2.100d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  SULFONATE SULFUR                                 
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'SO3 '
      cbnlocal = '18'
      canlocal = '18181815 0'
      ctnlocal = '18181815 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 2.100d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  SULFATE SULFUR                                   
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'SO4 '
      cbnlocal = '18'
      canlocal = '18181815 0'
      ctnlocal = '18181815 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 2.100d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  SULFONE SULPHER DOUBLY BONDED TO CARBON          
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = '=SO2'
      cbnlocal = '18'
      canlocal = '18181815 0'
      ctnlocal = '18181815 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 2.100d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  SULFUR IN NITROGEN ANALOG OF A SULFONE           
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'SNO '
      cbnlocal = '18'
      canlocal = '18181815 0'
      ctnlocal = '18181815 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 4.500d0
      nbclocal(2) = 4.200d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  SILICON                                          
      elelocal = 'Si'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'SI  '
      cbnlocal = '19'
      canlocal = '19191919 0'
      ctnlocal = '19191919 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.050d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  CARBON IN 4-MEMBERED RINGS                       
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CR4R'
      cbnlocal = '20'
      canlocal = '2020 1 1 0'
      ctnlocal = '2020 1 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  HYDROGEN IN ALCOHOLS                             
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HOR '
      cbnlocal = '21'
      canlocal = '212121 5 0'
      ctnlocal = '212121 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  GENERAL H ON OXYGEN                              
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HO  '
      cbnlocal = '21'
      canlocal = '212121 5 0'
      ctnlocal = '212121 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  HYDROGEN IN HYDROXIDE ANION                      
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HOM '
      cbnlocal = '21'
      canlocal = '212121 5 0'
      ctnlocal = '212121 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  CARBON IN A 3-MEMBERED RING                      
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CR3R'
      cbnlocal = '22'
      canlocal = '222222 1 0'
      ctnlocal = '222222 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H-N(SP3)                                         
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HNR '
      cbnlocal = '23'
      canlocal = '232323 5 0'
      ctnlocal = '232323 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H-N(SP3), AMMONIA                                
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'H3N '
      cbnlocal = '23'
      canlocal = '232323 5 0'
      ctnlocal = '232323 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H-N IN PYRROLE                                   
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HPYL'
      cbnlocal = '23'
      canlocal = '232323 5 0'
      ctnlocal = '232323 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H-N IN IN A N-OXIDE                              
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HNOX'
      cbnlocal = '23'
      canlocal = '232323 5 0'
      ctnlocal = '232323 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H ON DICOORD, NEGATIVELY CHARGED NITROGEN        
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HNM '
      cbnlocal = '23'
      canlocal = '232323 5 0'
      ctnlocal = '232323 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  GENERAL H ON NITROGEN                            
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HN  '
      cbnlocal = '23'
      canlocal = '232323 5 0'
      ctnlocal = '232323 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H-O IN CARBOXYLIC ACIDS                          
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HOCO'
      cbnlocal = '24'
      canlocal = '242424 5 0'
      ctnlocal = '242424 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  HYDROGEN ON OXYGEN ATTACHED TO PHOSPHOROUS       
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HOP '
      cbnlocal = '24'
      canlocal = '242424 5 0'
      ctnlocal = '242424 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.600d0
      nbclocal(2) = 4.500d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  PHOSPHOROUS IN PHOSPHATES AND PHOSPHODIESTERS    
      elelocal = ' P'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'PO4 '
      cbnlocal = '25'
      canlocal = '25252525 0'
      ctnlocal = '25252525 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.600d0
      nbclocal(2) = 4.500d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  TETRACOORDINATE P WITH THREE ATTACHED OXYGENS    
      elelocal = ' P'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'PO3 '
      cbnlocal = '25'
      canlocal = '25252525 0'
      ctnlocal = '25252525 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.600d0
      nbclocal(2) = 4.500d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  TETRACOORDINATE P WITH TWO ATTACHED OXYGENS      
      elelocal = ' P'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'PO2 '
      cbnlocal = '25'
      canlocal = '25252525 0'
      ctnlocal = '25252525 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.600d0
      nbclocal(2) = 4.500d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  TETRACOORDINATE P WITH ONE ATTACHED OXYGEN       
      elelocal = ' P'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'PO  '
      cbnlocal = '25'
      canlocal = '25252525 0'
      ctnlocal = '25252525 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.600d0
      nbclocal(2) = 4.500d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  GENERAL TETRACOORDINATE PHOSPHORUS               
      elelocal = ' P'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'PTET'
      cbnlocal = '25'
      canlocal = '25252525 0'
      ctnlocal = '25252525 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 3.600d0
      nbclocal(2) = 4.500d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  TRICOORDINATE P, AS IN PHOSPHINES                
      elelocal = ' P'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'P   '
      cbnlocal = '26'
      canlocal = '26262625 0'
      ctnlocal = '26262625 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  AZO HYDROGEN                                     
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HN=N'
      cbnlocal = '27'
      canlocal = '272728 5 0'
      ctnlocal = '272728 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  IMINE HYDROGEN                                   
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HN=C'
      cbnlocal = '27'
      canlocal = '272728 5 0'
      ctnlocal = '272728 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  AMIDE HYDROGEN                                   
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HNCO'
      cbnlocal = '28'
      canlocal = '282828 5 0'
      ctnlocal = '282828 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  THIOAMIDE HYDROGEN                               
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HNCS'
      cbnlocal = '28'
      canlocal = '282828 5 0'
      ctnlocal = '282828 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H-N IN ENAMINES                                  
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HNCC'
      cbnlocal = '28'
      canlocal = '282828 5 0'
      ctnlocal = '282828 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H-N IN H-N-C=N                                   
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HNCN'
      cbnlocal = '28'
      canlocal = '282828 5 0'
      ctnlocal = '282828 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H-N IN H-N-N=C                                   
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HNNC'
      cbnlocal = '28'
      canlocal = '282828 5 0'
      ctnlocal = '282828 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H-N IN H-N-N=N                                   
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HNNN'
      cbnlocal = '28'
      canlocal = '282828 5 0'
      ctnlocal = '282828 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H-N IN SULFONAMIDE                               
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HNSO'
      cbnlocal = '28'
      canlocal = '282828 5 0'
      ctnlocal = '282828 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H-N IN PHOSPHONAMIDE                             
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HNPO'
      cbnlocal = '28'
      canlocal = '282828 5 0'
      ctnlocal = '282828 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  HYDROGEN ON N ATTACHED TO TRIPLY BONDED CARBON   
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HNC%'
      cbnlocal = '28'
      canlocal = '282828 5 0'
      ctnlocal = '282828 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  GENERAL H ON SP2 NITROGEN                        
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HSP2'
      cbnlocal = '28'
      canlocal = '282828 5 0'
      ctnlocal = '282828 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H-O IN ENOLS AND PHENOLS                         
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HOCC'
      cbnlocal = '29'
      canlocal = '292929 5 0'
      ctnlocal = '292929 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H-O IN HO-C=N                                    
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HOCN'
      cbnlocal = '29'
      canlocal = '292929 5 0'
      ctnlocal = '292929 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.350d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  OLEFINIC CARBON IN 4-MEMBERED RINGS              
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CE4R'
      cbnlocal = '30'
      canlocal = '3030 2 1 0'
      ctnlocal = '3030 2 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  HYDROGEN IN H2O                                  
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HOH '
      cbnlocal = '31'
      canlocal = '31313131 0'
      ctnlocal = '31313131 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.750d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  OXYGEN IN CARBOXYLATE ANION                      
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O2CM'
      cbnlocal = '32'
      canlocal = '3232 7 6 0'
      ctnlocal = '3232 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.750d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  N-OXIDE OXYGEN                                   
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OXN '
      cbnlocal = '32'
      canlocal = '3232 7 6 0'
      ctnlocal = '3232 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.750d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  NITRO OXYGEN                                     
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O2N '
      cbnlocal = '32'
      canlocal = '3232 7 6 0'
      ctnlocal = '3232 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.750d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  NITRO-GROUP OXYGEN IN NITRATE                    
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O2NO'
      cbnlocal = '32'
      canlocal = '3232 7 6 0'
      ctnlocal = '3232 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.750d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  NITRATE ANION OXYGEN                             
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O3N '
      cbnlocal = '32'
      canlocal = '3232 7 6 0'
      ctnlocal = '3232 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.750d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  SINGLE TERMINAL OXYGEN ON TETRACOORD SULFUR      
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O-S '
      cbnlocal = '32'
      canlocal = '3232 7 6 0'
      ctnlocal = '3232 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.750d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  TERMINAL O-S IN SULFONES AND SULFONAMIDES        
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O2S '
      cbnlocal = '32'
      canlocal = '3232 7 6 0'
      ctnlocal = '3232 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.750d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  TERMINAL O IN SULFONATES                         
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O3S '
      cbnlocal = '32'
      canlocal = '3232 7 6 0'
      ctnlocal = '3232 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.750d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  TERMINAL O IN SO4(-3)                            
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O4S '
      cbnlocal = '32'
      canlocal = '3232 7 6 0'
      ctnlocal = '3232 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.750d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  TERM O IN THIOSULFINATE ANION - FORMAL CHARGE=-0.5
      charge = -0.5d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OSMS'
      cbnlocal = '32'
      canlocal = '3232 7 6 0'
      ctnlocal = '3232 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.750d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  TERMINAL O IN PHOSPHOXIDES                       
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OP  '
      cbnlocal = '32'
      canlocal = '3232 7 6 0'
      ctnlocal = '3232 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.750d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  TERMINAL O IN PHOSPHINATES                       
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O2P '
      cbnlocal = '32'
      canlocal = '3232 7 6 0'
      ctnlocal = '3232 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.750d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  TERMINAL OXYGEN IN PHOSPHONATES                  
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O3P '
      cbnlocal = '32'
      canlocal = '3232 7 6 0'
      ctnlocal = '3232 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.750d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  TERMINAL OXYGEN IN PHOSPHATES AND PHOSPHODIESTERS
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O4P '
      cbnlocal = '32'
      canlocal = '3232 7 6 0'
      ctnlocal = '3232 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.750d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  OXYGEN IN CLO4(-) ANION - FORMAL CHARGE=-0.25    
      charge = -0.25d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O4CL'
      cbnlocal = '32'
      canlocal = '3232 7 6 0'
      ctnlocal = '3232 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H ON OXYGEN ATTACHED TO SULFUR                   
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HOS '
      cbnlocal = '33'
      canlocal = '333321 5 0'
      ctnlocal = '333321 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  QUATERNARY NITROGEN, SP3, POSITIVELY CHARGED     
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NR+ '
      cbnlocal = '34'
      canlocal = '3434 8 8 0'
      ctnlocal = '3434 8 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.500d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  ALKOXIDE OXYGEN, NEGATIVELY CHARGED              
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OM  '
      cbnlocal = '35'
      canlocal = '3535 6 6 0'
      ctnlocal = '3535 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.500d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  OXIDE OXYGEN ON SP2 CARBON, NEGATIVELY CHARGED   
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OM2 '
      cbnlocal = '35'
      canlocal = '3535 6 6 0'
      ctnlocal = '3535 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H ON QUATERNARY NITROGEN                         
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HNR+'
      cbnlocal = '36'
      canlocal = '363636 5 0'
      ctnlocal = '363636 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H ON IMIDAZOLIUM-TYPE NITROGEN                   
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HIM+'
      cbnlocal = '36'
      canlocal = '363636 5 0'
      ctnlocal = '363636 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H ON PROTONATED PYRIDINE NITROGEN                
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HPD+'
      cbnlocal = '36'
      canlocal = '363636 5 0'
      ctnlocal = '363636 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H ON AMIDINIUM-TYPE NITROGEN                     
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HNN+'
      cbnlocal = '36'
      canlocal = '363636 5 0'
      ctnlocal = '363636 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H ON PROTONATED IMINE NITROGEN                   
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HNC+'
      cbnlocal = '36'
      canlocal = '363636 5 0'
      ctnlocal = '363636 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H ON GUANIDINIUM-TYPE NITROGEN                   
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HGD+'
      cbnlocal = '36'
      canlocal = '363636 5 0'
      ctnlocal = '363636 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H ON N5+, N5A+ OR N5B+                           
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HN5+'
      cbnlocal = '36'
      canlocal = '363636 5 0'
      ctnlocal = '363636 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.350d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  CARBON AS IN BENZENE, PYRROLE                    
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CB  '
      cbnlocal = '37'
      canlocal = '3737 2 1 0'
      ctnlocal = '3737 2 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.850d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  NITROGEN, AS IN PYRIDINE                         
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NPYD'
      cbnlocal = '38'
      canlocal = '3838 9 8 0'
      ctnlocal = '3838 9 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  NITROGEN, AS IN PYRROLE                          
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NPYL'
      cbnlocal = '39'
      canlocal = '393910 8 0'
      ctnlocal = '393910 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITROGEN ON N-C=C                                
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NC=C'
      cbnlocal = '40'
      canlocal = '404010 8 0'
      ctnlocal = '404010 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITROGEN IN N-C=N                                
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NC=N'
      cbnlocal = '40'
      canlocal = '404010 8 0'
      ctnlocal = '404010 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITROGEN IN N-C=P                                
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NC=P'
      cbnlocal = '40'
      canlocal = '404010 8 0'
      ctnlocal = '404010 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITROGEN ATTACHED TO C-C TRIPLE BOND             
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NC%C'
      cbnlocal = '40'
      canlocal = '404010 8 0'
      ctnlocal = '404010 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  CARBOXYLATE ANION CARBON                         
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CO2M'
      cbnlocal = '41'
      canlocal = '4141 3 1 0'
      ctnlocal = '4141 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.100d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  CARBON IN THIOCARBOXYLATE ANION                  
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CS2M'
      cbnlocal = '41'
      canlocal = '4141 3 1 0'
      ctnlocal = '4141 3 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITROGEN, TRIPLE BONDED                          
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NSP '
      cbnlocal = '42'
      canlocal = '424242 8 0'
      ctnlocal = '424242 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITROGEN IN SULFONAMIDES                         
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NSO2'
      cbnlocal = '43'
      canlocal = '434310 8 0'
      ctnlocal = '434310 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITROGEN IN SULFONAMIDES, THREE O'S ON S         
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NSO3'
      cbnlocal = '43'
      canlocal = '434310 8 0'
      ctnlocal = '434310 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITROGEN IN PHOSPHONAMIDES                       
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NPO2'
      cbnlocal = '43'
      canlocal = '434310 8 0'
      ctnlocal = '434310 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITROGEN IN PHOSPHONAMIDES, THREE O'S ON P       
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NPO3'
      cbnlocal = '43'
      canlocal = '434310 8 0'
      ctnlocal = '434310 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITROGEN ATTACHED TO CYANO GROUP                 
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NC%N'
      cbnlocal = '43'
      canlocal = '434310 8 0'
      ctnlocal = '434310 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 3.000d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  SULFUR AS IN THIOPHENE                           
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'STHI'
      cbnlocal = '44'
      canlocal = '44441615 0'
      ctnlocal = '44441615 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.150d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITRO GROUP NITROGEN                             
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NO2 '
      cbnlocal = '45'
      canlocal = '454510 8 0'
      ctnlocal = '454510 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.150d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITRATE GROUP NITROGEN                           
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NO3 '
      cbnlocal = '45'
      canlocal = '454510 8 0'
      ctnlocal = '454510 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.300d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  NITROSO NITROGEN                                 
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N=O '
      cbnlocal = '46'
      canlocal = '4646 9 8 0'
      ctnlocal = '4646 9 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  TERMINAL NITROGEN IN AZIDO OR DIAZO GROUP        
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NAZT'
      cbnlocal = '47'
      canlocal = '474742 8 0'
      ctnlocal = '474742 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.200d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  DIVALENT NITROGEN REPLACING MONOVALENT O IN SO2 G
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NSO '
      cbnlocal = '48'
      canlocal = '4848 9 8 0'
      ctnlocal = '4848 9 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  POSITIVELY CHARGED OXONIUM (TRICOORDINATE) OXYGEN
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O+  '
      cbnlocal = '49'
      canlocal = '4949 6 6 0'
      ctnlocal = '4949 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  HYDROGEN ON O+ OXYGEN                            
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HO+ '
      cbnlocal = '50'
      canlocal = '505021 5 0'
      ctnlocal = '505021 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.400d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  POSITIVELY CHARGED OXENIUM (DICOORDINATE) OXYGEN 
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'O=+ '
      cbnlocal = '51'
      canlocal = '5151 7 6 0'
      ctnlocal = '5151 7 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  HYDROGEN ON OXENIUM OXYGEN                       
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HO=+'
      cbnlocal = '52'
      canlocal = '525221 5 0'
      ctnlocal = '525221 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  NITROGEN IN C=N=N OR -N=N=N                      
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = '=N= '
      cbnlocal = '53'
      canlocal = '535342 8 0'
      ctnlocal = '535342 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.300d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  POSITIVELY CHARGED IMINIUM NITROGEN              
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N+=C'
      cbnlocal = '54'
      canlocal = '5454 9 8 0'
      ctnlocal = '5454 9 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.300d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  POSITIVELY CHARGED NITROGEN DOUBLE-BONDED TO N   
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N+=N'
      cbnlocal = '54'
      canlocal = '5454 9 8 0'
      ctnlocal = '5454 9 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.800d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  N IN +N=C-N RESONANCE STRUCTURES - FORMAL CHARGE=1/2
      charge = 0.5d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NCN+'
      cbnlocal = '55'
      canlocal = '555510 8 0'
      ctnlocal = '555510 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.800d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  GUANIDINIUM-TYPE NITROGEN - FORMAL CHARGE=1/3    
      charge = 1.0d0/3.0d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NGD+'
      cbnlocal = '56'
      canlocal = '565610 8 0'
      ctnlocal = '565610 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  GUANIDINIUM CARBON                               
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CGD+'
      cbnlocal = '57'
      canlocal = '5757 2 1 0'
      ctnlocal = '5757 2 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  C IN +N=C-N RESONANCE STRUCTURES                 
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CNN+'
      cbnlocal = '57'
      canlocal = '5757 2 1 0'
      ctnlocal = '5757 2 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.800d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  PYRIDINIUM-TYPE NITROGEN - FORMAL CHARGE=1       
      charge = 1.0d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NPD+'
      cbnlocal = '58'
      canlocal = '585810 8 0'
      ctnlocal = '585810 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.650d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  AROMATIC OXYGEN AS IN FURAN                      
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OFUR'
      cbnlocal = '59'
      canlocal = '5959 6 6 0'
      ctnlocal = '5959 6 6 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.800d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  ISONITRILE CARBON                                
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'C%  '
      cbnlocal = '60'
      canlocal = '6060 4 1 0'
      ctnlocal = '6060 4 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.800d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  ISONITRILE NITROGEN [FC = 0] OR DIAZO NITROGEN [F
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NR% '
      cbnlocal = '61'
      canlocal = '616142 8 0'
      ctnlocal = '616142 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.300d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  DEPROTONATED SULFONAMIDE N-; FORMAL CHARGE=-1    
      charge = -1.0d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NM  '
      cbnlocal = '62'
      canlocal = '626210 8 0'
      ctnlocal = '626210 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.350d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  ALPHA CARBON IN 5-MEMBERED HETEROAROMATIC RING   
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'C5A '
      cbnlocal = '63'
      canlocal = '6363 2 1 0'
      ctnlocal = '6363 2 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.350d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  BETA CARBON IN 5-MEMBERED HETEROAROMATIC RING    
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'C5B '
      cbnlocal = '64'
      canlocal = '6464 2 1 0'
      ctnlocal = '6464 2 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: E94
c     ---  ALPHA AROM HETEROCYCLIC 5-RING  NITROGEN         
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N5A '
      cbnlocal = '65'
      canlocal = '6565 9 8 0'
      ctnlocal = '6565 9 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.750d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  BETA AROM HETEROCYCLIC 5-RING  NITROGEN          
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N5B '
      cbnlocal = '66'
      canlocal = '6666 9 8 0'
      ctnlocal = '6666 9 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.950d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  SP2-HYDRIDIZED N-OXIDE NITROGEN                  
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N2OX'
      cbnlocal = '67'
      canlocal = '6767 9 8 0'
      ctnlocal = '6767 9 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.900d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  SP3-HYDRIDIZED N-OXIDE NITROGEN                  
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N3OX'
      cbnlocal = '68'
      canlocal = '6868 8 8 0'
      ctnlocal = '6868 8 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.950d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  PYRIDINE N-OXIDE NITROGEN                        
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NPOX'
      cbnlocal = '69'
      canlocal = '6969 9 8 0'
      ctnlocal = '6969 9 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.870d0
      nbclocal(2) = 3.150d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  OXYGEN ON WATER                                  
      elelocal = ' O'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'OH2 '
      cbnlocal = '70'
      canlocal = '7070707070'
      ctnlocal = '7070707070'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H ATTACHED TO DIVALENT, DICOORDINATE S           
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HS  '
      cbnlocal = '71'
      canlocal = '7171 5 5 0'
      ctnlocal = '7171 5 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H ATTACHED TO TETRAVALENT, TRICOODR S DBL BONDED 
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HS=N'
      cbnlocal = '71'
      canlocal = '7171 5 5 0'
      ctnlocal = '7171 5 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 0.800d0
      nbclocal(3) = 4.200d0
      nbclocal(4) = 1.209d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  H ATTACHED TO TRI- OR TETRACOORDINATE PHOSPHORUS 
      elelocal = ' H'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'D'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'HP  '
      cbnlocal = '71'
      canlocal = '7171 5 5 0'
      ctnlocal = '7171 5 5 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 4.000d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  TERMINAL SULFUR BONDED TO PHOSPHORUS             
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'S-P '
      cbnlocal = '72'
      canlocal = '72721615 0'
      ctnlocal = '72721615 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 4.000d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  TERMINAL SULFUR IN THIOCARBOXYLATE ANION         
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'S2CM'
      cbnlocal = '72'
      canlocal = '72721615 0'
      ctnlocal = '72721615 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 4.000d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  TERMINAL SULFUR - FORMAL CHARGE=-1               
      charge = -1.0d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'SM  '
      cbnlocal = '72'
      canlocal = '72721615 0'
      ctnlocal = '72721615 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 4.000d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  TERMINAL SULFUR IN THIOSULFINATE GROUP           
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'SSMO'
      cbnlocal = '72'
      canlocal = '72721615 0'
      ctnlocal = '72721615 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 3.000d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  SULFUR IN NEGATIVELY CHARGED SULFINATE GROUP     
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'SO2M'
      cbnlocal = '73'
      canlocal = '73731815 0'
      ctnlocal = '73731815 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 3.000d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  TRICOORD SULFUR IN THIOSULFINATE GROUP           
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'SSOM'
      cbnlocal = '73'
      canlocal = '73731815 0'
      ctnlocal = '73731815 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 3.000d0
      nbclocal(2) = 4.800d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  SULFINYL SULFUR, EG. IN C=S=O                    
      elelocal = ' S'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = '=S=O'
      cbnlocal = '74'
      canlocal = '74741715 0'
      ctnlocal = '74741715 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 4.000d0
      nbclocal(2) = 4.500d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  PHOSPHOROUS DOUBLY BONDED TO CARBON              
      elelocal = ' P'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = '-P=C'
      cbnlocal = '75'
      canlocal = '75752625 0'
      ctnlocal = '75752625 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.200d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  NEGATIVELY CHARGED N IN, E.G, TRI- OR TETRAZOLE A
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N5M '
      cbnlocal = '76'
      canlocal = '7676 9 8 0'
      ctnlocal = '7676 9 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.500d0
      nbclocal(2) = 5.100d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  CHLORINE IN PERCHLORATE ANION, CLO4(-)           
      elelocal = 'Cl'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CLO4'
      cbnlocal = '77'
      canlocal = '77771212 0'
      ctnlocal = '77771212 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.350d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  GENERAL CARBON IN 5-MEMBERED HETEROAROMATIC RING 
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'C5  '
      cbnlocal = '78'
      canlocal = '7878 2 1 0'
      ctnlocal = '7878 2 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  GENERAL NITROGEN IN 5-MEMBERED HETEROCYCLIC RING 
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N5  '
      cbnlocal = '79'
      canlocal = '7979 9 8 0'
      ctnlocal = '7979 9 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 2.490d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  C IN N-C-N IN IMIDAZOLIUM ION                    
      elelocal = ' C'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CIM+'
      cbnlocal = '80'
      canlocal = '8080 2 1 0'
      ctnlocal = '8080 2 1 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.800d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  IMIDAZOLIUM-TYPE NITROGEN - FORMAL CHARGE=1/2    
      charge = 0.5d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NIM+'
      cbnlocal = '81'
      canlocal = '818110 8 0'
      ctnlocal = '818110 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.800d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  POSITIVE N5A NITROGEN - FORMAL CHARGE=1          
      charge = 1.0d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N5A+'
      cbnlocal = '81'
      canlocal = '818110 8 0'
      ctnlocal = '818110 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.800d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  POSITIVE N5B NITROGEN - FORMAL CHARGE=1          
      charge = 1.0d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N5B+'
      cbnlocal = '81'
      canlocal = '818110 8 0'
      ctnlocal = '818110 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.800d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: C94
c     ---  POSITIVE N5 NITROGEN - FORMAL CHARGE=1           
      charge = 1.0d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N5+ '
      cbnlocal = '81'
      canlocal = '818110 8 0'
      ctnlocal = '818110 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.950d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  N-OXIDE NITROGEN IN 5-RING ALPHA POSITION        
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N5AX'
      cbnlocal = '82'
      canlocal = '8282 9 8 0'
      ctnlocal = '8282 9 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.950d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  N-OXIDE NITROGEN IN 5-RING BETA POSITION         
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N5BX'
      cbnlocal = '82'
      canlocal = '8282 9 8 0'
      ctnlocal = '8282 9 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.950d0
      nbclocal(2) = 2.820d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  N-OXIDE NITROGEN IN GENERAL 5-RING POSITION      
      elelocal = ' N'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'N5OX'
      cbnlocal = '82'
      canlocal = '8282 9 8 0'
      ctnlocal = '8282 9 8 0'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.450d0
      nbclocal(2) = 6.000d0
      nbclocal(3) = 4.000d0
      nbclocal(4) = 1.400d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  IRON +2 CATION                                   
      elelocal = 'Fe'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'FE+2'
      cbnlocal = '87'
      canlocal = '8787878787'
      ctnlocal = '8787878787'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.550d0
      nbclocal(2) = 6.000d0
      nbclocal(3) = 4.000d0
      nbclocal(4) = 1.400d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  IROM +3 CATION                                   
      elelocal = 'Fe'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'FE+3'
      cbnlocal = '88'
      canlocal = '8888888888'
      ctnlocal = '8888888888'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.400d0
      nbclocal(2) = 3.480d0
      nbclocal(3) = 3.890d0
      nbclocal(4) = 1.282d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  FLUORIDE ANION                                   
      elelocal = ' F'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'F-  '
      cbnlocal = '89'
      canlocal = '8989898989'
      ctnlocal = '8989898989'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 4.500d0
      nbclocal(2) = 5.100d0
      nbclocal(3) = 3.320d0
      nbclocal(4) = 1.345d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  CHLORIDE ANION                                   
      elelocal = 'Cl'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CL- '
      cbnlocal = '90'
      canlocal = '9090909090'
      ctnlocal = '9090909090'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 6.000d0
      nbclocal(2) = 6.000d0
      nbclocal(3) = 3.190d0
      nbclocal(4) = 1.359d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  BROMIDE ANION                                    
      elelocal = 'Br'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = 'A'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'BR- '
      cbnlocal = '91'
      canlocal = '9191919191'
      ctnlocal = '9191919191'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.150d0
      nbclocal(2) = 2.000d0
      nbclocal(3) = 4.000d0
      nbclocal(4) = 1.300d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  LITHIUM CATION                                   
      elelocal = 'Li'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'LI+ '
      cbnlocal = '92'
      canlocal = '9292929292'
      ctnlocal = '9292929292'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.400d0
      nbclocal(2) = 3.500d0
      nbclocal(3) = 4.000d0
      nbclocal(4) = 1.300d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  SODIUM CATION                                    
      elelocal = 'Na'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'NA+ '
      cbnlocal = '93'
      canlocal = '9393939393'
      ctnlocal = '9393939393'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 1.000d0
      nbclocal(2) = 5.000d0
      nbclocal(3) = 4.000d0
      nbclocal(4) = 1.300d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  POTASSIUM CATION                                 
      elelocal = ' K'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'K+  '
      cbnlocal = '94'
      canlocal = '9494949494'
      ctnlocal = '9494949494'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.430d0
      nbclocal(2) = 6.000d0
      nbclocal(3) = 4.000d0
      nbclocal(4) = 1.400d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  DIPOSITIVE ZINC                                  
      charge = 2.0d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = 'Zn'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'ZINC'
      cbnlocal = '95'
      canlocal = '9595959595'
      ctnlocal = '9595959595'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.430d0
      nbclocal(2) = 6.000d0
      nbclocal(3) = 4.000d0
      nbclocal(4) = 1.400d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  DIPOSITIVE ZINC                                  
      charge = 2.0d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = 'Zn'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'ZN+2'
      cbnlocal = '95'
      canlocal = '9595959595'
      ctnlocal = '9595959595'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.900d0
      nbclocal(2) = 5.000d0
      nbclocal(3) = 4.000d0
      nbclocal(4) = 1.400d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  DIPOSITIVE CALCIUM                               
      charge = 2.0d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = 'Ca'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CA+2'
      cbnlocal = '96'
      canlocal = '9696969696'
      ctnlocal = '9696969696'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.350d0
      nbclocal(2) = 6.000d0
      nbclocal(3) = 4.000d0
      nbclocal(4) = 1.400d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  MONOPOSITIVE COPPER                              
      charge = 1.0d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = 'Cu'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CU+1'
      cbnlocal = '97'
      canlocal = '9797979797'
      ctnlocal = '9797979797'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.400d0
      nbclocal(2) = 6.000d0
      nbclocal(3) = 4.000d0
      nbclocal(4) = 1.400d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  DIPOSITIVE COPPER          
      charge = 2.0d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = 'Cu'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'CU+2'
      cbnlocal = '98'
      canlocal = '9898989898'
      ctnlocal = '9898989898'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)

      type = type + 1
      ii = twh_getnbtype(type,type)
      nbclocal(1) = 0.350d0
      nbclocal(2) = 3.500d0
      nbclocal(3) = 4.000d0
      nbclocal(4) = 1.300d0
      call twh_set_nbcoeff(ii,nbclocal)
c     --- origin: X94
c     ---  DIPOSITIVE MAGNESIUM CATION                      
      charge = 2.0d0
      call twh_qbasevalue(GLB_SET,type,charge)
      elelocal = 'Mg'
      call twh_elementname(GLB_SET,type,elelocal)
      call twh_getelement(lfailure,'getmass',masslocal,elelocal,idum)
      if ( lfailure ) return
      call twh_mass(GLB_SET,type,masslocal)
      bplocal = ''
      call twh_bondpatt(GLB_SET,type,bplocal)
      daflag = '-'
      call twh_donor_acceptor(GLB_SET,type,daflag)
      call twh_nbff(GLB_SET,type,ffname)
      nbnlocal = 'MG+2'
      cbnlocal = '99'
      canlocal = '9999999999'
      ctnlocal = '9999999999'
      call twh_set_names(type,nbnlocal,cbnlocal,canlocal,ctnlocal)


      ntypes = type
      call twh_ffcheck(lfailure,0,type)
      if ( lfailure ) return

      return
      end

      subroutine twh_ffmmff94_bond(lfailure,ffname,nbondtypes)
c     ******************************************************************
c     * sets up the MMFF94 bond potentials                             *
c     *                                                                *
c     * originally written 12-09-2006 by M.G. Martin                   *
c     * last modified 08-04-2011 by M.G. Martin                        *
c     ******************************************************************
      implicit none
c     --- variables passed to/from the subroutine
      character*(FFNAMELEN) ffname
      logical lfailure
      integer nbondtypes
c     --- local variables
c     --- integer scalars
      integer type,isame
c     --- character string scalars
      character*(FFNAMELEN) order
c     --- character string arrays
      character*(FFNAMELEN) vnlocal
      dimension vnlocal(MAXSAMEV,MAXVNAME) 
c     --- double precision scalars
      double precision force,cs
c     --- double precision arrays
      double precision vclocal
      dimension vclocal(MINVIBCOEFF:MAXVIBCOEFF)

c     --- initialize type count
      type = 0

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.508d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.258d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = ' 1'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.482d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.539d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = ' 2'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.492d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.190d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = ' 3'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.459d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.707d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = ' 4'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.093d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.766d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = ' 5'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.418d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.047d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = ' 6'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.451d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.084d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = ' 8'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.458d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.763d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = ' 9'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.436d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.664d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '10'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.360d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.011d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #C94 
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '11'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.773d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.974d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '12'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.949d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.529d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '13'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.090d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 1.706d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '14'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.805d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.893d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '15'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.813d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.841d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '17'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.772d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.258d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '18'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.830d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.866d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '19'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.504d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.650d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.482d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.286d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '22'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.810d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.980d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '25'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.830d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.790d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '26'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.480d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.844d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #C94 
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '34'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.307d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.915d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '35'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.486d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.957d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.445d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.114d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '39'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.446d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.922d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.510d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.830d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #C94 
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '41'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.472d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.971d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '43'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.480d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.844d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '45'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.482d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.813d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '46'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.461d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.267d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '54'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.454d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.646d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '55'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.453d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.166d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '56'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.461d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.669d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '57'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.451d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.329d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '58'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.424d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.845d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '61'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.444d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.456d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '62'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.471d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.481d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.469d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.518d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.459d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.188d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '67'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.479d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.217d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '68'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.801d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.956d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '72'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.839d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.608d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '73'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.858d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.547d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '75'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.465d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.593d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '78'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.477d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.373d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '80'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.441d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.512d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 1'
      vnlocal(isame,2) = '81'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.333d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 9.505d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = ' 2'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.430d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.310d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #C94 
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = ' 2'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.468d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.565d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = ' 3'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.297d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 9.538d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = ' 4'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.415d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.657d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = ' 4'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.083d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.170d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = ' 5'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.373d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.520d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = ' 6'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.360d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.385d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = ' 9'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.362d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.329d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '10'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.350d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.283d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '11'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.720d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.390d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '12'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.854d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.413d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '13'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.025d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.062d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '14'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.720d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.896d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '15'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.773d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.247d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '17'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.728d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.789d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '18'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.811d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.052d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '19'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.465d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.593d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.448d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.926d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '22'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.742d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.750d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '25'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.331d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 8.166d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '30'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.407d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.207d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '34'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.250d0
      force = CON_KCALTOK*143.9825d0*0.5d0*10.343d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '35'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.449d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.007d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '37'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.368d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.164d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '39'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.370d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.110d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #C94 
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.505d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.746d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '41'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.420d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.928d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '43'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.430d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.725d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '45'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.325d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.466d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '46'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.368d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.164d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '55'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.365d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.246d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '56'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.336d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.105d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '62'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.400d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.030d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '63'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.411d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.754d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '64'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.432d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.685d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '67'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.700d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.179d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '72'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.361d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.357d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 2'
      vnlocal(isame,2) = '81'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.489d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.418d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = ' 3'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.438d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.135d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = ' 4'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.101d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.650d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = ' 5'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.355d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.801d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = ' 6'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.222d0
      force = CON_KCALTOK*143.9825d0*0.5d0*12.950d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = ' 7'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.290d0
      force = CON_KCALTOK*143.9825d0*0.5d0*10.077d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = ' 9'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.364d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.273d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = ' 9'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.369d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.829d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '10'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.340d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.570d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '11'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.715d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.449d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '12'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.748d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.536d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '15'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.665d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.735d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '16'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.808d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.888d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '17'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.760d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.394d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '18'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.530d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.298d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.465d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.593d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '22'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.792d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.164d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '25'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.471d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.481d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '30'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.237d0
      force = CON_KCALTOK*143.9825d0*0.5d0*11.012d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '35'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.457d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.488d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '37'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.375d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.978d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '39'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.370d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.110d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #C94 
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.482d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.286d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '41'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.420d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.928d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '43'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.440d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.531d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '45'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.398d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.412d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '48'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.290d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 8.562d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '51'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.320d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.637d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '53'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.280d0
      force = CON_KCALTOK*143.9825d0*0.5d0*10.333d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '54'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.563d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.771d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '54'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.422d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.886d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '55'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.421d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.907d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '56'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.422d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.492d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '57'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.409d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.163d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '58'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.322d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.568d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '62'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.423d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.468d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '63'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.431d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.288d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '64'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.304d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 8.217d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '67'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.639d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.204d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '74'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.710d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.191d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '75'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.413d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.705d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '78'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.375d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.719d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 3'
      vnlocal(isame,2) = '80'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.200d0
      force = CON_KCALTOK*143.9825d0*0.5d0*15.206d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 4'
      vnlocal(isame,2) = ' 4'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.065d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.726d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 4'
      vnlocal(isame,2) = ' 5'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.328d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.193d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 4'
      vnlocal(isame,2) = ' 6'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.176d0
      force = CON_KCALTOK*143.9825d0*0.5d0*14.916d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 4'
      vnlocal(isame,2) = ' 7'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.172d0
      force = CON_KCALTOK*143.9825d0*0.5d0*15.589d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 4'
      vnlocal(isame,2) = ' 9'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.338d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.041d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 4'
      vnlocal(isame,2) = ' 9'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.345d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.824d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 4'
      vnlocal(isame,2) = '10'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.690d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.330d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 4'
      vnlocal(isame,2) = '15'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.436d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.178d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 4'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.426d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.400d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 4'
      vnlocal(isame,2) = '22'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.282d0
      force = CON_KCALTOK*143.9825d0*0.5d0*10.227d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 4'
      vnlocal(isame,2) = '30'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.424d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.445d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 4'
      vnlocal(isame,2) = '37'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.160d0
      force = CON_KCALTOK*143.9825d0*0.5d0*16.582d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 4'
      vnlocal(isame,2) = '42'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.341d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.947d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 4'
      vnlocal(isame,2) = '43'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.416d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.633d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 4'
      vnlocal(isame,2) = '63'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.422d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.492d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 4'
      vnlocal(isame,2) = '64'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.485d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.254d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 5'
      vnlocal(isame,2) = '19'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.093d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.852d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 5'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.082d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.191d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 5'
      vnlocal(isame,2) = '22'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.086d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.176d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 5'
      vnlocal(isame,2) = '30'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.084d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.306d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 5'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.144d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.256d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 5'
      vnlocal(isame,2) = '41'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.076d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.633d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 5'
      vnlocal(isame,2) = '57'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.080d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.531d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 5'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.080d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.506d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 5'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.080d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.506d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 5'
      vnlocal(isame,2) = '78'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.076d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.633d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 5'
      vnlocal(isame,2) = '80'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.449d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.088d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = ' 6'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.450d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.059d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = ' 8'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.395d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.491d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = ' 9'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.410d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.982d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '10'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.661d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.757d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '15'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.608d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.779d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '17'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.630d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.326d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '18'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.660d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.661d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '19'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.433d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.623d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 0.972d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.794d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '21'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.433d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.556d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '22'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 0.981d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.403d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '24'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.630d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.243d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '25'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.618d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.481d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '26'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 0.973d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.839d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '29'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.271d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 9.359d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '30'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 0.986d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.143d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '33'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.376d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.614d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.388d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.629d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '39'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.389d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.609d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.342d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.754d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '41'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.426d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.937d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '43'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.404d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.321d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '45'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.365d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.117d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '54'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.381d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.772d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '55'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.330d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.128d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '57'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.380d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.792d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '58'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.324d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.324d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.345d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.664d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 6'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.500d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 8.770d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 7'
      vnlocal(isame,2) = '17'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.235d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 9.329d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 7'
      vnlocal(isame,2) = '46'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.490d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 9.129d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = ' 7'
      vnlocal(isame,2) = '74'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.420d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.264d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = ' 8'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.342d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.581d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = ' 9'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.378d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.909d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '10'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.761d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.371d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '12'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.652d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.060d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '15'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.663d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.901d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '17'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.700d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.254d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '19'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.456d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.107d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.457d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.223d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '22'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.019d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.490d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '23'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.660d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.629d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '25'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.699d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.027d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '26'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.386d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.775d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '34'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.408d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.435d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '39'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.390d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.710d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.374d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.977d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '43'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.358d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.267d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '45'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.301d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.519d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '46'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.360d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.229d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '55'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.373d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.995d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 8'
      vnlocal(isame,2) = '56'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.243d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.256d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = ' 9'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.384d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.808d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = ' 9'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.347d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.480d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '10'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.739d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.635d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '12'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.671d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.791d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '15'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.626d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.465d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '18'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.741d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.687d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '19'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.447d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.401d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.619d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.379d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '25'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.026d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.230d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '27'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.423d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.223d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '34'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.366d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.095d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '35'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.393d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.529d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '37'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.337d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.685d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '39'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.352d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.382d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.388d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.650d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '41'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.329d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.857d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '45'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.242d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.291d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '53'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.323d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.991d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '54'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.383d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.825d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '55'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.341d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.602d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '56'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.345d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.824d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '57'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.334d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.749d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '62'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.345d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.824d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '63'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.396d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.458d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '64'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.258d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.752d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '67'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.351d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.644d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '78'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.378d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.909d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = ' 9'
      vnlocal(isame,2) = '81'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.374d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.977d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '10'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.878d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.110d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '13'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.029d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 1.967d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '14'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.686d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.593d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '15'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.661d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.930d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '17'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.456d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.240d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.418d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.970d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '22'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.714d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.820d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '25'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.727d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.651d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '26'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.015d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.663d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '28'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.375d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.960d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '34'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.375d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.898d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '35'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.395d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.482d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.352d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.382d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '39'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.382d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.841d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.325d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.466d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '41'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.402d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.524d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '45'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.369d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.137d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.376d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.952d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '10'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.348d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.339d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '11'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.389d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.296d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '11'
      vnlocal(isame,2) = '22'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.583d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.019d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '11'
      vnlocal(isame,2) = '25'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.575d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.204d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '11'
      vnlocal(isame,2) = '26'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.342d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.511d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '11'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.440d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.187d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '11'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.031d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.978d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '12'
      vnlocal(isame,2) = '15'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.051d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.808d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '12'
      vnlocal(isame,2) = '18'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.050d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.838d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = '12'
      vnlocal(isame,2) = '19'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.751d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.859d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '12'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.750d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.056d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = '12'
      vnlocal(isame,2) = '22'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.023d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.063d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '12'
      vnlocal(isame,2) = '25'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.100d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.448d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = '12'
      vnlocal(isame,2) = '26'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.721d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.378d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '12'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.731d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.737d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '12'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.694d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.714d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '12'
      vnlocal(isame,2) = '57'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.718d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.413d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '12'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.699d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.649d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '12'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.920d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.767d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '13'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.902d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.928d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '13'
      vnlocal(isame,2) = '22'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.891d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.031d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '13'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.891d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.031d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '13'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.332d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 0.884d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '14'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.075d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 1.781d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '14'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.050d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.531d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '15'
      vnlocal(isame,2) = '15'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.094d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.214d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '15'
      vnlocal(isame,2) = '18'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.146d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.022d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '15'
      vnlocal(isame,2) = '19'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.822d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.757d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '15'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.727d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.802d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '15'
      vnlocal(isame,2) = '22'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.112d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.319d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '15'
      vnlocal(isame,2) = '25'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.106d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.359d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '15'
      vnlocal(isame,2) = '26'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.731d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.750d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '15'
      vnlocal(isame,2) = '30'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.765d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.565d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '15'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.666d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.859d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '15'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.717d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.221d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '15'
      vnlocal(isame,2) = '43'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.713d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.993d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '15'
      vnlocal(isame,2) = '57'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.733d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.724d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '15'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.747d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.548d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '15'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.341d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.014d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '15'
      vnlocal(isame,2) = '71'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.865d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.397d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '17'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.844d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.566d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '17'
      vnlocal(isame,2) = '22'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.787d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.098d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '17'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.601d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.900d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '17'
      vnlocal(isame,2) = '43'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.780d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.172d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '18'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.822d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.757d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '18'
      vnlocal(isame,2) = '22'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.450d0
      force = CON_KCALTOK*143.9825d0*0.5d0*10.748d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = '18'
      vnlocal(isame,2) = '32'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.770d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.281d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '18'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.693d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.504d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '18'
      vnlocal(isame,2) = '39'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.710d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.301d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = '18'
      vnlocal(isame,2) = '43'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.540d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.186d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '18'
      vnlocal(isame,2) = '48'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.628d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.432d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '18'
      vnlocal(isame,2) = '55'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.783d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.568d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '18'
      vnlocal(isame,2) = '58'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.570d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.510d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = '18'
      vnlocal(isame,2) = '62'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.749d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.524d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '18'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.723d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.856d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '18'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.702d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.150d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '18'
      vnlocal(isame,2) = '80'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.900d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.288d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '19'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.809d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.072d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '19'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.686d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.470d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '19'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.795d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.219d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '19'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.226d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 1.600d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '19'
      vnlocal(isame,2) = '75'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.526d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.663d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '20'
      vnlocal(isame,2) = '20'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.484d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.251d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '20'
      vnlocal(isame,2) = '22'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.838d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.718d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '20'
      vnlocal(isame,2) = '25'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.853d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.588d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '20'
      vnlocal(isame,2) = '26'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.507d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.977d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '20'
      vnlocal(isame,2) = '30'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.460d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.171d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '20'
      vnlocal(isame,2) = '34'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.516d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.740d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '20'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.427d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.784d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '20'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.482d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.286d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '20'
      vnlocal(isame,2) = '41'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.487d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.737d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '20'
      vnlocal(isame,2) = '43'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.480d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.844d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '20'
      vnlocal(isame,2) = '45'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.499d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.969d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '22'
      vnlocal(isame,2) = '22'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.513d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.785d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '22'
      vnlocal(isame,2) = '30'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.464d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.103d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '22'
      vnlocal(isame,2) = '34'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.471d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.481d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '22'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.459d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.188d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '22'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.441d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.071d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '22'
      vnlocal(isame,2) = '41'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.466d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.070d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '22'
      vnlocal(isame,2) = '43'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.452d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.311d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '22'
      vnlocal(isame,2) = '45'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.012d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.112d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '23'
      vnlocal(isame,2) = '39'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.026d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.339d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '23'
      vnlocal(isame,2) = '62'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.019d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.610d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #E94 
      isame = isame + 1
      vnlocal(isame,1) = '23'
      vnlocal(isame,2) = '67'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.038d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.899d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '23'
      vnlocal(isame,2) = '68'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.253d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 1.514d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '25'
      vnlocal(isame,2) = '25'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.510d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 8.296d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = '25'
      vnlocal(isame,2) = '32'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.755d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.586d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '25'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.676d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.370d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '25'
      vnlocal(isame,2) = '39'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.660d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.629d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '25'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.762d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.237d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '25'
      vnlocal(isame,2) = '43'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.699d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.356d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '25'
      vnlocal(isame,2) = '57'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.745d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.711d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '25'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.411d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.001d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '25'
      vnlocal(isame,2) = '71'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.950d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.744d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = '25'
      vnlocal(isame,2) = '72'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.279d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 1.414d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '26'
      vnlocal(isame,2) = '26'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.748d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.395d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '26'
      vnlocal(isame,2) = '34'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.788d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.207d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '26'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.646d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.870d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '26'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.415d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.959d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '26'
      vnlocal(isame,2) = '71'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.018d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.576d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '28'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.028d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.265d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '28'
      vnlocal(isame,2) = '43'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.024d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.413d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '28'
      vnlocal(isame,2) = '48'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.343d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 9.579d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '30'
      vnlocal(isame,2) = '30'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.428d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.355d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '30'
      vnlocal(isame,2) = '30'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.298d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 8.447d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '30'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.404d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.274d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '30'
      vnlocal(isame,2) = '67'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 0.969d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.880d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '31'
      vnlocal(isame,2) = '70'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.261d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 9.756d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '32'
      vnlocal(isame,2) = '41'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.233d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 9.420d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '32'
      vnlocal(isame,2) = '45'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.269d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.926d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '32'
      vnlocal(isame,2) = '67'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.348d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.398d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '32'
      vnlocal(isame,2) = '68'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.261d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.098d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '32'
      vnlocal(isame,2) = '69'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.510d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 8.427d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = '32'
      vnlocal(isame,2) = '73'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.450d0
      force = CON_KCALTOK*143.9825d0*0.5d0*10.648d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = '32'
      vnlocal(isame,2) = '77'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.252d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 8.594d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '32'
      vnlocal(isame,2) = '82'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.028d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.163d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '34'
      vnlocal(isame,2) = '36'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.450d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.347d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '34'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.351d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.401d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '34'
      vnlocal(isame,2) = '43'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.262d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 9.767d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '35'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.207d0
      force = CON_KCALTOK*143.9825d0*0.5d0*12.760d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '35'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.022d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.529d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '36'
      vnlocal(isame,2) = '54'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.014d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.744d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '36'
      vnlocal(isame,2) = '55'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.017d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.490d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '36'
      vnlocal(isame,2) = '56'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.019d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.610d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #E94 
      isame = isame + 1
      vnlocal(isame,1) = '36'
      vnlocal(isame,2) = '58'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.016d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.980d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '36'
      vnlocal(isame,2) = '81'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.374d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.573d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '37'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.436d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.178d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '37'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.333d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.737d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '38'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.375d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.978d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '39'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.388d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.650d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '39'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.398d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.168d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.468d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.537d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '41'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.428d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.764d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '43'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.431d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.705d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '45'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.367d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.191d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '46'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.352d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.615d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '55'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.414d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.055d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '56'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.440d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.092d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '57'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.326d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.432d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '58'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.414d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.055d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '58'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.385d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.724d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '61'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.335d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.137d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '62'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.372d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.095d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.436d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.178d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '63'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.379d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.161d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.432d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.265d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '64'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.430d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.725d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '67'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.352d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.396d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '69'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.375d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.719d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '78'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.471d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.987d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '81'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.440d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.531d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '37'
      vnlocal(isame,2) = '81'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.246d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.002d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '38'
      vnlocal(isame,2) = '38'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.330d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.299d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '38'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.340d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.978d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '38'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.321d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.036d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '38'
      vnlocal(isame,2) = '69'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.366d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.218d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '38'
      vnlocal(isame,2) = '78'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.367d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.101d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '39'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.402d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.524d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '39'
      vnlocal(isame,2) = '45'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.364d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.301d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '39'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.369d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.137d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '39'
      vnlocal(isame,2) = '63'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.361d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.357d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '39'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.395d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.482d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '39'
      vnlocal(isame,2) = '64'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.339d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.513d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '39'
      vnlocal(isame,2) = '65'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.369d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.137d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '39'
      vnlocal(isame,2) = '78'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.359d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.248d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '40'
      vnlocal(isame,2) = '40'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.356d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.305d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '40'
      vnlocal(isame,2) = '45'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.335d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.727d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '40'
      vnlocal(isame,2) = '46'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.256d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.817d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '40'
      vnlocal(isame,2) = '54'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.348d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.733d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '40'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.351d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.644d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '40'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.378d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.900d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '40'
      vnlocal(isame,2) = '78'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.443d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.029d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '41'
      vnlocal(isame,2) = '41'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.391d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.577d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '41'
      vnlocal(isame,2) = '55'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.335d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.137d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '41'
      vnlocal(isame,2) = '62'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.678d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.519d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '41'
      vnlocal(isame,2) = '72'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.434d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.222d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '41'
      vnlocal(isame,2) = '80'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.087d0
      force = CON_KCALTOK*143.9825d0*0.5d0*16.223d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '42'
      vnlocal(isame,2) = '61'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.361d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.211d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '43'
      vnlocal(isame,2) = '43'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.390d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.710d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '43'
      vnlocal(isame,2) = '45'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.399d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.389d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '43'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.717d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.589d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '44'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.684d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.374d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '44'
      vnlocal(isame,2) = '65'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.734d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.711d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '44'
      vnlocal(isame,2) = '78'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.719d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.910d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '44'
      vnlocal(isame,2) = '80'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.411d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.119d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '45'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.413d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.076d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '45'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.385d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.724d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '45'
      vnlocal(isame,2) = '78'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.140d0
      force = CON_KCALTOK*143.9825d0*0.5d0*12.192d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = '47'
      vnlocal(isame,2) = '53'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 0.991d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.812d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '49'
      vnlocal(isame,2) = '50'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 0.987d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.100d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '51'
      vnlocal(isame,2) = '52'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.319d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.227d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '55'
      vnlocal(isame,2) = '57'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.374d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.977d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '55'
      vnlocal(isame,2) = '62'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.393d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.529d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '55'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.324d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.500d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '55'
      vnlocal(isame,2) = '80'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.383d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.137d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '56'
      vnlocal(isame,2) = '57'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.378d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.900d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '56'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.357d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.470d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '56'
      vnlocal(isame,2) = '80'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.426d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.400d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '57'
      vnlocal(isame,2) = '63'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.438d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.135d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '57'
      vnlocal(isame,2) = '64'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.346d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.794d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '58'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.368d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.164d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '58'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.360d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.787d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '59'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.388d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.756d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '59'
      vnlocal(isame,2) = '65'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.364d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.127d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '59'
      vnlocal(isame,2) = '78'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.332d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.064d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '59'
      vnlocal(isame,2) = '80'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.431d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.855d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '59'
      vnlocal(isame,2) = '82'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.170d0
      force = CON_KCALTOK*143.9825d0*0.5d0*15.749d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: #X94 
      isame = isame + 1
      vnlocal(isame,1) = '60'
      vnlocal(isame,2) = '61'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.341d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.947d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '62'
      vnlocal(isame,2) = '63'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.364d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.273d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '62'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.412d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.729d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '63'
      vnlocal(isame,2) = '63'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.377d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.118d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '63'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.313d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 8.326d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '63'
      vnlocal(isame,2) = '66'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.679d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.503d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '63'
      vnlocal(isame,2) = '72'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.352d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.434d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '63'
      vnlocal(isame,2) = '78'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.316d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.778d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '63'
      vnlocal(isame,2) = '81'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.418d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.313d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '64'
      vnlocal(isame,2) = '64'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.448d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.926d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '64'
      vnlocal(isame,2) = '64'
      order = 'single'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.335d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 8.258d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '64'
      vnlocal(isame,2) = '65'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.369d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.456d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '64'
      vnlocal(isame,2) = '66'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.422d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.492d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '64'
      vnlocal(isame,2) = '78'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.381d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.824d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '64'
      vnlocal(isame,2) = '81'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.346d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.794d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '64'
      vnlocal(isame,2) = '82'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.323d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 7.243d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '65'
      vnlocal(isame,2) = '66'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.298d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 8.447d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '65'
      vnlocal(isame,2) = '78'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.313d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.223d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '65'
      vnlocal(isame,2) = '81'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.297d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.622d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '65'
      vnlocal(isame,2) = '82'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.368d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.874d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '66'
      vnlocal(isame,2) = '66'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.360d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.385d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '66'
      vnlocal(isame,2) = '78'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.375d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 3.960d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '66'
      vnlocal(isame,2) = '81'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.280d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.085d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '67'
      vnlocal(isame,2) = '67'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.423d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.852d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '71'
      vnlocal(isame,2) = '75'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 2.035d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 2.628d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '72'
      vnlocal(isame,2) = '73'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.357d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.286d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '76'
      vnlocal(isame,2) = '76'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.345d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.824d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: X94  
      isame = isame + 1
      vnlocal(isame,1) = '76'
      vnlocal(isame,2) = '78'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.374d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.573d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '78'
      vnlocal(isame,2) = '78'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.287d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 8.890d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '78'
      vnlocal(isame,2) = '79'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.381d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 5.046d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '78'
      vnlocal(isame,2) = '81'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.269d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 6.408d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '79'
      vnlocal(isame,2) = '79'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.356d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 4.305d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: E94  
      isame = isame + 1
      vnlocal(isame,1) = '79'
      vnlocal(isame,2) = '81'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)

      type = type + 1
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) = 1.335d0
      force = CON_KCALTOK*143.9825d0*0.5d0* 8.237d0
      cs = -2.0d0
      vclocal(1) = force
      vclocal(2) = force*cs
      vclocal(3) = force*(cs**2)*7.0d0/12.0d0
      call twh_set_vibcoeffs(type,vclocal)
      call twh_vibff(GLB_SET,type,ffname)
      isame = 0
c     --- origin: C94  
      isame = isame + 1
      vnlocal(isame,1) = '80'
      vnlocal(isame,2) = '81'
      order = 'wild'
      call twh_viborder(GLB_SET,type,order)
      call twh_set_vibnames(type,isame,vnlocal)


      nbondtypes = type
      call twh_ffcheck(lfailure,1,type)
      if ( lfailure ) return

      return
      end

      subroutine twh_ffmmff94_angle(lfailure,ffname,nangletypes)
c     ******************************************************************
c     * sets up the MMFF94 angle terms                                 *
c     *                                                                *
c     * originally written 12-11-2006 by M.G. Martin                   *
c     * last modified 08-04-2011 by M.G. Martin                        *
c     ******************************************************************
      implicit none
c     --- variables passed to/from the subroutine
      character*(FFNAMELEN) ffname
      logical lfailure
      integer nangletypes
c     --- local variables
c     --- character string scalars
      character*15 orderang
c     --- character string arrays
      character*(FFNAMELEN) bnlocal 
      dimension bnlocal(MAXSAMEB,MAXBNAME)
c     --- integer scalars
      integer type,style
c     --- double precision scalars
      double precision cb,radtodegsq,onepi,force,radtodeg
c     --- double precisino arrays
      double precision bclocal
      dimension bclocal(MINBENDCOEFF:MAXBENDCOEFF)

c     --- there are two angle types for this forcefield
c     --- the first, and most common, is a combination of a cubic
c     --- term with some bond-stretch terms

c     --- the second is used for angles with a linear geometry 
c     --- (those with an equilibrium angle of 180).  It does not have
c     --- any bond-stretch terms and uses a relatively simple cosine

c     --- set some useful constants
      cb = -0.4d0
c     --- retrieve constant
      call twh_constant_pi(GLB_GET,onepi)
      call twh_constant_radtodeg(GLB_GET,radtodeg)
      radtodegsq = radtodeg**2

c     --- initialize type count
      type = 0

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-1-* MMFF94 DEF  
      bclocal(0) = 108.900d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 109.608d0
      force = ( 0.851d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.206d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.206d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 109.445d0
      force = ( 0.736d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.136d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.197d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 107.517d0
      force = ( 0.777d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.211d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.092d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.265d0
      force = ( 1.006d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 110.549d0
      force = ( 0.636d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.227d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.070d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.133d0
      force = ( 0.992d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.173d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.417d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.290d0
      force = ( 0.777d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.136d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.282d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.194d0
      force = ( 1.136d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 109.960d0
      force = ( 1.050d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.187d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.338d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.313d0
      force = ( 1.225d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.209d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.633d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.679d0
      force = ( 1.056d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.176d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.386d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.820d0
      force = ( 1.078d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '13'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.945d0
      force = ( 0.980d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '14'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 107.397d0
      force = ( 0.743d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.139d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.217d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.578d0
      force = ( 1.089d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.315d0
      force = ( 1.093d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.436d0
      force = ( 0.755d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.659d0
      force = ( 1.021d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.125d0
      force = ( 1.001d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 112.356d0
      force = ( 0.803d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.879d0
      force = ( 0.833d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 106.493d0
      force = ( 1.179d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.236d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.436d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.617d0
      force = ( 0.756d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.152d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.260d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 109.170d0
      force = ( 0.927d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.144d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.595d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.678d0
      force = ( 1.130d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  98.422d0
      force = ( 0.330d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.122d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.051d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.019d0
      force = ( 1.135d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.028d0
      force = ( 1.197d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.424d0
      force = ( 1.173d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '54'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.604d0
      force = ( 1.150d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 110.371d0
      force = ( 1.199d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.262d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.451d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '56'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.900d0
      force = ( 1.012d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.327d0
      force = ( 1.179d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.311d0
      force = ( 1.125d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '61'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.058d0
      force = ( 1.006d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.064d0
      force = ( 0.988d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.557d0
      force = ( 1.216d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 107.195d0
      force = ( 1.018d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.186d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.125d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '68'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.658d0
      force = ( 1.160d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '73'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.850d0
      force = ( 1.012d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.327d0
      force = ( 0.947d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.837d0
      force = ( 1.108d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.453d0
      force = ( 1.113d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.282d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.282d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 104.829d0
      force = ( 0.667d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.206d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.022d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.873d0
      force = ( 1.022d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 110.292d0
      force = ( 0.632d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.234d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.088d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.699d0
      force = ( 1.074d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.183d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.387d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.553d0
      force = ( 0.884d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.214d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.363d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.577d0
      force = ( 1.118d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.963d0
      force = ( 1.160d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.419d0
      force = ( 1.192d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.410d0
      force = ( 1.070d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.560d0
      force = ( 1.078d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.434d0
      force = ( 1.077d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.110d0
      force = ( 1.188d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.448d0
      force = ( 1.053d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.020d0
      force = ( 0.942d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.815d0
      force = ( 0.893d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  99.065d0
      force = ( 1.029d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.817d0
      force = ( 1.066d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.446d0
      force = ( 0.985d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.513d0
      force = ( 1.124d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.270d0
      force = ( 1.149d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.978d0
      force = ( 1.232d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.692d0
      force = ( 0.935d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.687d0
      force = ( 1.224d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.746d0
      force = ( 0.974d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.850d0
      force = ( 1.019d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.385d0
      force = ( 0.650d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.157d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.115d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 104.112d0
      force = ( 0.528d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.036d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.456d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.837d0
      force = ( 1.197d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.535d0
      force = ( 1.201d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 102.655d0
      force = ( 0.634d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.038d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.195d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.328d0
      force = ( 1.189d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.064d0
      force = ( 1.136d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.645d0
      force = ( 1.147d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '13'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.404d0
      force = ( 1.048d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '14'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.192d0
      force = ( 1.125d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.602d0
      force = ( 1.092d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.119d0
      force = ( 1.120d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.830d0
      force = ( 0.969d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.522d0
      force = ( 0.999d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.555d0
      force = ( 0.742d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.871d0
      force = ( 1.141d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.833d0
      force = ( 1.011d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.751d0
      force = ( 1.136d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.941d0
      force = ( 1.174d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.216d0
      force = ( 1.033d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.281d0
      force = ( 1.221d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.077d0
      force = ( 1.069d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.186d0
      force = ( 1.028d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.327d0
      force = ( 1.167d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.186d0
      force = ( 0.954d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 111.417d0
      force = ( 0.615d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.977d0
      force = ( 1.273d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.063d0
      force = ( 1.099d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.750d0
      force = ( 1.187d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.488d0
      force = ( 1.117d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.047d0
      force = ( 1.021d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '13'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.432d0
      force = ( 1.028d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.351d0
      force = ( 1.187d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.556d0
      force = ( 1.174d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.999d0
      force = ( 0.853d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.160d0
      force = ( 1.148d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.424d0
      force = ( 0.993d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.836d0
      force = ( 0.516d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.115d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.115d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.577d0
      force = ( 0.781d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.013d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.436d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 110.297d0
      force = ( 0.653d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.027d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.358d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 109.894d0
      force = ( 0.733d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.040d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.418d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 107.646d0
      force = ( 0.740d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.043d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.261d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 107.897d0
      force = ( 0.875d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.003d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.452d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.162d0
      force = ( 0.698d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.018d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.380d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.049d0
      force = ( 0.613d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '13'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.019d0
      force = ( 0.508d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '14'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 109.609d0
      force = ( 0.576d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.018d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.255d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 107.944d0
      force = ( 0.634d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 106.855d0
      force = ( 0.663d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:X94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.121d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.218d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 113.195d0
      force = ( 0.450d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.000d0
      force = ( 0.706d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.069d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.327d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.380d0
      force = ( 0.618d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:X94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.055d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.267d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 109.486d0
      force = ( 0.487d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 111.172d0
      force = ( 0.466d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 106.224d0
      force = ( 0.872d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.003d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.342d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 125.663d0
      force = ( 0.644d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '35'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 109.491d0
      force = ( 0.627d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.074d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.287d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 106.299d0
      force = ( 0.811d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.092d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.607d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 109.870d0
      force = ( 0.719d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.023d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.335d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.904d0
      force = ( 0.525d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.093d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.118d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 109.083d0
      force = ( 0.692d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 105.197d0
      force = ( 0.741d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 106.735d0
      force = ( 0.719d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '46'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 106.973d0
      force = ( 0.874d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.016d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.343d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '54'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.507d0
      force = ( 0.861d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.030d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.397d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.223d0
      force = ( 0.814d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.031d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.384d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '56'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.420d0
      force = ( 0.626d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.481d0
      force = ( 0.750d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 109.227d0
      force = ( 0.710d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '61'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 113.035d0
      force = ( 0.655d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.467d0
      force = ( 0.621d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.457d0
      force = ( 0.622d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.474d0
      force = ( 0.732d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 103.817d0
      force = ( 0.748d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.041d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.216d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '68'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 116.576d0
      force = ( 0.547d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 107.153d0
      force = ( 0.633d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '73'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.078d0
      force = ( 0.640d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.144d0
      force = ( 0.684d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.870d0
      force = ( 0.721d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.368d0
      force = ( 1.156d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.320d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.320d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.223d0
      force = ( 1.333d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.950d0
      force = ( 1.224d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.568d0
      force = ( 1.432d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.900d0
      force = ( 1.593d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.012d0
      force = ( 1.273d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.655d0
      force = ( 1.348d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.214d0
      force = ( 0.906d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.202d0
      force = ( 1.293d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.913d0
      force = ( 1.287d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.598d0
      force = ( 1.171d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.433d0
      force = ( 0.888d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.975d0
      force = ( 1.257d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 107.978d0
      force = ( 0.878d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.310d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.160d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.464d0
      force = ( 1.485d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.779d0
      force = ( 1.371d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.467d0
      force = ( 1.333d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.438d0
      force = ( 1.523d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.467d0
      force = ( 1.308d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.535d0
      force = ( 1.351d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.308d0
      force = ( 1.238d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.856d0
      force = ( 1.203d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.080d0
      force = ( 1.133d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.683d0
      force = ( 1.258d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.251d0
      force = ( 1.217d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.356d0
      force = ( 1.120d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.353d0
      force = ( 1.116d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.698d0
      force = ( 1.143d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.412d0
      force = ( 1.138d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.992d0
      force = ( 1.090d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.193d0
      force = ( 1.364d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.962d0
      force = ( 0.964d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.868d0
      force = ( 1.234d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.596d0
      force = ( 1.137d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  96.139d0
      force = ( 1.583d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.266d0
      force = ( 1.038d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.598d0
      force = ( 1.104d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.127d0
      force = ( 1.156d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.720d0
      force = ( 1.209d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.152d0
      force = ( 1.173d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.465d0
      force = ( 1.024d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.432d0
      force = ( 1.060d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.565d0
      force = ( 1.077d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.728d0
      force = ( 1.084d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.509d0
      force = ( 1.163d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.995d0
      force = ( 1.191d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.502d0
      force = ( 1.161d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.509d0
      force = ( 1.269d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.838d0
      force = ( 1.220d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.262d0
      force = ( 1.132d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.822d0
      force = ( 1.015d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.423d0
      force = ( 1.107d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.536d0
      force = ( 1.264d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.961d0
      force = ( 1.087d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.622d0
      force = ( 1.268d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 106.081d0
      force = ( 1.638d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.586d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.586d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.517d0
      force = ( 1.254d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.637d0
      force = ( 1.243d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  97.532d0
      force = ( 1.244d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.669d0
      force = ( 1.338d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.367d0
      force = ( 1.556d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '35'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.278d0
      force = ( 1.151d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.053d0
      force = ( 1.301d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 100.991d0
      force = ( 1.550d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.569d0
      force = ( 1.303d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '73'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.378d0
      force = ( 0.884d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '75'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 110.422d0
      force = ( 1.096d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.508d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.508d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.064d0
      force = ( 1.146d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.827d0
      force = ( 1.299d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.971d0
      force = ( 0.932d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.605d0
      force = ( 1.081d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.028d0
      force = ( 1.097d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.118d0
      force = ( 0.989d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.030d0
      force = ( 1.076d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.359d0
      force = ( 1.150d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.430d0
      force = ( 1.353d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.474d0
      force = ( 1.071d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.338d0
      force = ( 1.093d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.645d0
      force = ( 1.093d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '13'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '13'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.534d0
      force = ( 1.084d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '13'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.469d0
      force = ( 1.068d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '13'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.383d0
      force = ( 1.305d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '13'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.718d0
      force = ( 1.021d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '14'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.896d0
      force = ( 1.147d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.308d0
      force = ( 1.059d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.318d0
      force = ( 1.222d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.959d0
      force = ( 1.051d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.005d0
      force = ( 1.149d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 100.981d0
      force = ( 1.263d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.596d0
      force = ( 1.060d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.703d0
      force = ( 1.059d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.029d0
      force = ( 1.289d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '73'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.049d0
      force = ( 1.065d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '17'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.960d0
      force = ( 1.121d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.125d0
      force = ( 1.283d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.390d0
      force = ( 1.203d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.273d0
      force = ( 1.287d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.683d0
      force = ( 1.093d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.506d0
      force = ( 0.772d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '19'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '54'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  99.084d0
      force = ( 1.229d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.428d0
      force = ( 1.052d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.335d0
      force = ( 1.169d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.226d0
      force = ( 0.990d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.293d0
      force = ( 0.885d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.940d0
      force = ( 1.045d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.586d0
      force = ( 1.037d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.181d0
      force = ( 1.182d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.138d0
      force = ( 0.551d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.271d0
      force = ( 0.779d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.945d0
      force = ( 0.784d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.417d0
      force = ( 1.062d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.234d0
      force = ( 0.916d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.700d0
      force = ( 0.625d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '26'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.167d0
      force = ( 1.216d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '34'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.275d0
      force = ( 1.075d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '34'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.238d0
      force = ( 1.048d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '34'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.412d0
      force = ( 1.077d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '34'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.240d0
      force = ( 1.142d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '34'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '73'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.315d0
      force = ( 0.986d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.188d0
      force = ( 1.129d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.478d0
      force = ( 1.074d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.800d0
      force = ( 1.259d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.047d0
      force = ( 0.981d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.239d0
      force = ( 1.175d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.983d0
      force = ( 1.100d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '68'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.638d0
      force = ( 1.005d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.040d0
      force = ( 1.176d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.547d0
      force = ( 1.260d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.005d0
      force = ( 1.182d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.786d0
      force = ( 1.322d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.505d0
      force = ( 1.032d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.376d0
      force = ( 1.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.400d0
      force = ( 1.082d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '41'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.112d0
      force = ( 1.061d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '41'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.553d0
      force = ( 1.093d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '41'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.088d0
      force = ( 1.391d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = ' 1'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-2-* MMFF94 DEF  
      bclocal(0) = 119.400d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-2-* MMFF94 DEF  
      bclocal(0) = 118.200d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:2::*-2-* MMFF94 DEF 
      bclocal(0) = 120.800d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:3::*-2-* MMFF94 DEF 
      bclocal(0) =  62.600d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:6:*-2-* MMFF94 DEF  
      bclocal(0) =  60.500d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:6 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 118.043d0
      force = ( 0.752d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.250d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.250d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 122.141d0
      force = ( 0.672d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.203d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.207d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 116.929d0
      force = ( 0.684d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 116.104d0
      force = ( 0.698d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.045d0
      force = ( 0.828d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.613d0
      force = ( 0.846d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 120.108d0
      force = ( 0.446d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.215d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.128d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.518d0
      force = ( 1.160d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.707d0
      force = ( 1.015d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.343d0
      force = ( 0.983d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.395d0
      force = ( 0.964d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '13'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.465d0
      force = ( 0.939d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.868d0
      force = ( 0.883d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.918d0
      force = ( 0.961d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.310d0
      force = ( 0.880d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.114d0
      force = ( 0.873d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.605d0
      force = ( 0.826d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 116.064d0
      force = ( 0.721d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.515d0
      force = ( 0.982d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.921d0
      force = ( 1.121d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.192d0
      force = ( 1.006d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '56'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.945d0
      force = ( 0.768d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.884d0
      force = ( 0.966d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.185d0
      force = ( 1.115d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 121.550d0
      force = ( 0.747d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.250d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.219d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 1 a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.284d0
      force = ( 0.796d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  60.549d0
      force = ( 0.173d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:6 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.297d0
      force = ( 0.545d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.456d0
      force = ( 0.893d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.155d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.112d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 2 a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  59.145d0
      force = ( 0.184d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:5 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.053d0
      force = ( 0.902d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.794d0
      force = ( 0.889d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 121.004d0
      force = ( 0.535d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.207d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.157d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 118.442d0
      force = ( 0.463d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.267d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.159d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 1 a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 121.267d0
      force = ( 1.117d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.118d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.576d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.538d0
      force = ( 1.204d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.536d0
      force = ( 0.960d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.273d0
      force = ( 1.045d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.828d0
      force = ( 1.003d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.324d0
      force = ( 1.026d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 119.100d0
      force = ( 1.089d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.828d0
      force = ( 1.090d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 120.132d0
      force = ( 0.931d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.526d0
      force = ( 0.957d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.717d0
      force = ( 0.867d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '13'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.584d0
      force = ( 0.818d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '14'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.344d0
      force = ( 0.819d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '14'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.553d0
      force = ( 0.931d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.466d0
      force = ( 0.949d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.167d0
      force = ( 0.977d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.561d0
      force = ( 1.044d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.721d0
      force = ( 0.668d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.784d0
      force = ( 0.931d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.820d0
      force = ( 0.809d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  66.165d0
      force = ( 0.149d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.830d0
      force = ( 0.700d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.151d0
      force = ( 1.066d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 137.103d0
      force = ( 0.911d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '35'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 117.508d0
      force = ( 0.598d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.229d0
      force = ( 0.817d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.143d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.172d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 2 a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.360d0
      force = ( 0.976d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 126.830d0
      force = ( 0.773d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.289d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.390d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.132d0
      force = ( 0.976d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 110.442d0
      force = ( 0.432d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.191d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.047d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.808d0
      force = ( 1.144d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 109.231d0
      force = ( 1.194d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.984d0
      force = ( 1.062d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.534d0
      force = ( 1.005d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '46'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.154d0
      force = ( 0.995d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.879d0
      force = ( 1.234d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '56'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 135.269d0
      force = ( 0.808d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.277d0
      force = ( 0.948d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.528d0
      force = ( 0.866d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.998d0
      force = ( 0.859d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.136d0
      force = ( 1.132d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 134.269d0
      force = ( 0.770d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.541d0
      force = ( 1.078d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.370d0
      force = ( 0.853d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.739d0
      force = ( 0.878d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 117.291d0
      force = ( 0.487d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.264d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.156d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 1 a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.738d0
      force = ( 1.142d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.648d0
      force = ( 1.005d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.698d0
      force = ( 1.039d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.876d0
      force = ( 1.150d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.732d0
      force = ( 0.997d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.643d0
      force = ( 0.946d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '13'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.111d0
      force = ( 0.891d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '14'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.635d0
      force = ( 1.023d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.265d0
      force = ( 0.870d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.510d0
      force = ( 0.816d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.209d0
      force = ( 1.025d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.723d0
      force = ( 1.099d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.767d0
      force = ( 1.141d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '35'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.758d0
      force = ( 0.868d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.408d0
      force = ( 1.024d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.505d0
      force = ( 0.855d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.257d0
      force = ( 1.046d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.401d0
      force = ( 1.077d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.841d0
      force = ( 1.066d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '46'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.158d0
      force = ( 0.832d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 121.000d0
      force = ( 0.573d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 120.000d0
      force = ( 0.545d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.845d0
      force = ( 0.973d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.447d0
      force = ( 0.906d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.537d0
      force = ( 0.947d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.938d0
      force = ( 0.819d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.093d0
      force = ( 0.864d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.355d0
      force = ( 1.083d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.426d0
      force = ( 1.158d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.442d0
      force = ( 0.860d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 119.523d0
      force = ( 0.365d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.140d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.140d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.757d0
      force = ( 0.589d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.213d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.502d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 117.000d0
      force = ( 0.643d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.859d0
      force = ( 0.667d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 108.186d0
      force = ( 0.795d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 110.650d0
      force = ( 0.622d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.513d0
      force = ( 0.566d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '13'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.562d0
      force = ( 0.546d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 124.000d0
      force = ( 0.492d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.053d0
      force = ( 0.548d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 120.000d0
      force = ( 0.534d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 124.000d0
      force = ( 0.395d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 120.000d0
      force = ( 0.572d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 124.164d0
      force = ( 0.682d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '35'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 117.423d0
      force = ( 0.491d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.724d0
      force = ( 0.655d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 112.322d0
      force = ( 0.568d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.070d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.463d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 123.706d0
      force = ( 0.294d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.191d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.005d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 107.774d0
      force = ( 0.728d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 116.000d0
      force = ( 0.651d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 125.344d0
      force = ( 0.568d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 120.000d0
      force = ( 0.550d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 120.000d0
      force = ( 0.546d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 122.009d0
      force = ( 0.531d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 115.000d0
      force = ( 0.665d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.520d0
      force = ( 1.214d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.921d0
      force = ( 1.311d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.560d0
      force = ( 1.080d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 132.391d0
      force = ( 1.172d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '35'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.441d0
      force = ( 1.198d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.073d0
      force = ( 1.239d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.438d0
      force = ( 1.637d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.802d0
      force = ( 1.098d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.574d0
      force = ( 0.915d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.536d0
      force = ( 0.981d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 130.521d0
      force = ( 0.922d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.723d0
      force = ( 1.144d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.519d0
      force = ( 1.078d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 100.818d0
      force = ( 1.144d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.139d0
      force = ( 1.021d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.034d0
      force = ( 0.988d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.000d0
      force = ( 0.951d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.105d0
      force = ( 1.012d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.206d0
      force = ( 1.110d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.553d0
      force = ( 1.201d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.646d0
      force = ( 0.704d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.563d0
      force = ( 0.903d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.753d0
      force = ( 0.892d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.136d0
      force = ( 0.976d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.543d0
      force = ( 1.076d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.616d0
      force = ( 1.132d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '13'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.027d0
      force = ( 0.996d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 133.654d0
      force = ( 0.950d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '35'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.757d0
      force = ( 1.007d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.924d0
      force = ( 0.895d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.955d0
      force = ( 1.051d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '17'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.608d0
      force = ( 1.183d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.108d0
      force = ( 0.841d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  58.963d0
      force = ( 0.180d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.693d0
      force = ( 0.806d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.816d0
      force = ( 0.849d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '30'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.032d0
      force = ( 0.991d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '35'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.436d0
      force = ( 0.949d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.987d0
      force = ( 1.072d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '56'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.268d0
      force = ( 0.922d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.881d0
      force = ( 0.955d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 135.317d0
      force = ( 0.820d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.095d0
      force = ( 1.284d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.342d0
      force = ( 0.888d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '64'
      bnlocal(1,2) = ' 2'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-3-* MMFF94 DEF  
      bclocal(0) = 117.300d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:1:*-3-* MMFF94 DEF  
      bclocal(0) = 115.800d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:4:*-3-* MMFF94 DEF  
      bclocal(0) =  90.800d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:7:*-3-* MMFF94 DEF  
      bclocal(0) =  91.100d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:7 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:7:*-3-* MMFF94 DEF  
      bclocal(0) =  88.900d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:8 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 118.016d0
      force = ( 1.151d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.358d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.358d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 116.853d0
      force = ( 1.106d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 114.612d0
      force = ( 1.214d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 117.280d0
      force = ( 0.808d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.321d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.183d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 109.716d0
      force = ( 1.043d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.338d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.732d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 124.410d0
      force = ( 0.938d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.154d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.856d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 7'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.788d0
      force = ( 0.978d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.132d0
      force = ( 1.038d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 112.735d0
      force = ( 0.984d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.223d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.732d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.972d0
      force = ( 1.007d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.612d0
      force = ( 1.024d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.986d0
      force = ( 0.949d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '16'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 134.097d0
      force = ( 0.732d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.312d0
      force = ( 0.830d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.001d0
      force = ( 0.928d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.808d0
      force = ( 1.058d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '35'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 115.191d0
      force = ( 1.051d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.895d0
      force = ( 1.178d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.457d0
      force = ( 0.979d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.681d0
      force = ( 0.897d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 113.731d0
      force = ( 1.046d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.019d0
      force = ( 1.132d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 116.573d0
      force = ( 1.160d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '51'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 115.065d0
      force = ( 1.052d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '53'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.322d0
      force = ( 1.135d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '54'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.129d0
      force = ( 1.162d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.523d0
      force = ( 1.119d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.001d0
      force = ( 0.909d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.253d0
      force = ( 0.887d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.666d0
      force = ( 1.142d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 116.851d0
      force = ( 1.010d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '74'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 128.037d0
      force = ( 0.646d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '75'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.562d0
      force = ( 0.976d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  62.792d0
      force = ( 0.157d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:6 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.239d0
      force = ( 0.957d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 115.350d0
      force = ( 0.901d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.407d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.159d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 1 a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 106.510d0
      force = ( 0.932d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.429d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.473d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 1 a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 122.623d0
      force = ( 0.936d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.214d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.794d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 1 a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 7'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 122.253d0
      force = ( 0.831d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.227d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.610d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 1 a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.408d0
      force = ( 1.120d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.721d0
      force = ( 1.042d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.298d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.600d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 1 a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.769d0
      force = ( 0.901d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.105d0
      force = ( 1.057d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.850d0
      force = ( 0.881d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '16'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.027d0
      force = ( 0.969d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.794d0
      force = ( 0.853d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.935d0
      force = ( 0.973d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.592d0
      force = ( 1.197d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.437d0
      force = ( 0.910d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.169d0
      force = ( 1.105d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.032d0
      force = ( 1.082d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '53'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.588d0
      force = ( 1.012d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '54'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.278d0
      force = ( 1.186d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.909d0
      force = ( 1.151d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '56'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.947d0
      force = ( 0.918d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.084d0
      force = ( 1.033d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.597d0
      force = ( 1.022d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.775d0
      force = ( 0.822d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  89.965d0
      force = ( 1.280d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:8 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 113.762d0
      force = ( 0.943d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.251d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.133d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 1 a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 103.030d0
      force = ( 0.935d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.066d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.668d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 1 a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 117.024d0
      force = ( 0.919d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.093d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.866d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 1 a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 7'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.704d0
      force = ( 1.050d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.421d0
      force = ( 1.129d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.492d0
      force = ( 1.053d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  97.562d0
      force = ( 1.390d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.888d0
      force = ( 1.092d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '16'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.910d0
      force = ( 0.977d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.295d0
      force = ( 1.010d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  87.789d0
      force = ( 1.353d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:8 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.949d0
      force = ( 0.932d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.384d0
      force = ( 1.237d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.124d0
      force = ( 1.003d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.361d0
      force = ( 0.790d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.023d0
      force = ( 0.919d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.169d0
      force = ( 1.170d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '53'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.685d0
      force = ( 0.981d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.840d0
      force = ( 0.880d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.860d0
      force = ( 1.119d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.750d0
      force = ( 1.269d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.852d0
      force = ( 1.126d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 7'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.833d0
      force = ( 1.192d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.081d0
      force = ( 0.964d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 116.699d0
      force = ( 0.594d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.126d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.126d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.253d0
      force = ( 0.819d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.174d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.734d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 123.439d0
      force = ( 0.670d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.032d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.805d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 7'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 119.491d0
      force = ( 0.623d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.037d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.669d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.168d0
      force = ( 0.638d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.761d0
      force = ( 0.874d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.169d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.619d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.405d0
      force = ( 0.522d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '16'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.400d0
      force = ( 0.564d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.684d0
      force = ( 0.959d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.087d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.685d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 118.000d0
      force = ( 0.644d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '53'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 115.471d0
      force = ( 0.816d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.098d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.210d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '54'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 118.000d0
      force = ( 0.559d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 117.000d0
      force = ( 0.566d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.698d0
      force = ( 0.700d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.094d0
      force = ( 1.678d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 124.425d0
      force = ( 1.155d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.494d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.578d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 7'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.478d0
      force = ( 1.275d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.868d0
      force = ( 1.416d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.187d0
      force = ( 1.405d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.317d0
      force = ( 1.269d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '16'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.581d0
      force = ( 1.182d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  93.130d0
      force = ( 1.495d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:X94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 1.179d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.752d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 4 a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.826d0
      force = ( 1.276d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  93.191d0
      force = ( 1.530d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:7 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 102.881d0
      force = ( 0.808d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.655d0
      force = ( 1.611d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.565d0
      force = ( 1.371d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.658d0
      force = ( 1.477d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.183d0
      force = ( 1.330d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.328d0
      force = ( 1.315d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '48'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.427d0
      force = ( 1.409d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '51'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.510d0
      force = ( 1.495d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '54'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.542d0
      force = ( 1.421d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.082d0
      force = ( 1.339d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.993d0
      force = ( 1.267d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.698d0
      force = ( 1.256d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.084d0
      force = ( 1.147d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 127.152d0
      force = ( 0.907d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.771d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.353d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 130.049d0
      force = ( 0.984d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.313d0
      force = ( 1.101d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 129.492d0
      force = ( 0.713d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.865d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.181d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.851d0
      force = ( 1.093d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 129.010d0
      force = ( 0.972d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 119.968d0
      force = ( 0.734d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.727d0
      force = ( 1.352d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.087d0
      force = ( 1.281d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 124.549d0
      force = ( 1.163d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.879d0
      force = ( 1.114d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '48'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.184d0
      force = ( 1.288d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '54'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.056d0
      force = ( 1.258d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.854d0
      force = ( 1.175d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '56'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.081d0
      force = ( 1.323d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 129.349d0
      force = ( 1.129d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.456d0
      force = ( 1.036d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.133d0
      force = ( 1.071d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 132.047d0
      force = ( 0.955d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.094d0
      force = ( 1.119d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.131d0
      force = ( 1.021d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.697d0
      force = ( 1.105d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.608d0
      force = ( 1.154d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.046d0
      force = ( 1.056d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.679d0
      force = ( 1.036d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.787d0
      force = ( 1.042d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.665d0
      force = ( 0.936d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '16'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.902d0
      force = ( 1.035d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.698d0
      force = ( 1.121d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.437d0
      force = ( 0.951d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.861d0
      force = ( 1.040d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.442d0
      force = ( 0.955d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 134.470d0
      force = ( 1.054d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '35'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.569d0
      force = ( 0.997d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.740d0
      force = ( 1.060d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 128.078d0
      force = ( 0.844d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.680d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.260d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.152d0
      force = ( 1.018d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.513d0
      force = ( 1.114d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.140d0
      force = ( 1.497d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.056d0
      force = ( 1.244d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '54'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.096d0
      force = ( 1.038d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.054d0
      force = ( 1.004d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.060d0
      force = ( 1.053d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.150d0
      force = ( 0.959d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 114.923d0
      force = ( 1.612d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 1.050d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 1.050d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.206d0
      force = ( 1.167d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.150d0
      force = ( 1.005d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '16'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.052d0
      force = ( 1.299d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.213d0
      force = ( 1.019d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  92.724d0
      force = ( 1.338d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.651d0
      force = ( 1.076d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  90.508d0
      force = ( 1.438d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:7 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.649d0
      force = ( 1.223d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '35'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.495d0
      force = ( 1.101d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.419d0
      force = ( 1.434d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.697d0
      force = ( 1.093d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.929d0
      force = ( 1.144d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.685d0
      force = ( 1.375d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '51'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.590d0
      force = ( 1.286d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.168d0
      force = ( 1.200d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '56'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.623d0
      force = ( 1.075d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.233d0
      force = ( 1.098d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.543d0
      force = ( 1.182d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.244d0
      force = ( 1.296d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.964d0
      force = ( 0.850d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '75'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.284d0
      force = ( 1.095d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.217d0
      force = ( 0.965d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.502d0
      force = ( 1.110d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '74'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.620d0
      force = ( 1.109d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.329d0
      force = ( 0.981d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '16'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.607d0
      force = ( 1.191d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.034d0
      force = ( 1.061d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  91.041d0
      force = ( 1.345d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.753d0
      force = ( 1.026d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.305d0
      force = ( 1.037d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.388d0
      force = ( 1.066d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.260d0
      force = ( 0.896d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.583d0
      force = ( 1.407d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.117d0
      force = ( 1.076d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '74'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.695d0
      force = ( 0.991d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '16'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 130.230d0
      force = ( 1.030d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '16'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '35'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.415d0
      force = ( 0.934d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '16'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.196d0
      force = ( 1.004d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '16'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.347d0
      force = ( 0.963d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '16'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.454d0
      force = ( 1.006d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '16'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.110d0
      force = ( 1.064d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '16'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.528d0
      force = ( 0.939d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '17'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.188d0
      force = ( 0.948d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  94.800d0
      force = ( 1.495d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.536d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.536d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 4 a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  89.459d0
      force = ( 1.286d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  89.733d0
      force = ( 1.282d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:7 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  90.526d0
      force = ( 1.384d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.334d0
      force = ( 0.932d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  83.915d0
      force = ( 1.496d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.995d0
      force = ( 0.940d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.404d0
      force = ( 0.677d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 131.520d0
      force = ( 0.661d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.566d0
      force = ( 0.933d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.790d0
      force = ( 0.987d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.565d0
      force = ( 0.864d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 110.383d0
      force = ( 1.125d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.268d0
      force = ( 1.120d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.645d0
      force = ( 1.033d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '54'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.132d0
      force = ( 1.085d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.163d0
      force = ( 0.934d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.701d0
      force = ( 0.955d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.460d0
      force = ( 1.084d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.582d0
      force = ( 1.231d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 117.002d0
      force = ( 1.146d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.482d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.482d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.089d0
      force = ( 0.888d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.889d0
      force = ( 1.145d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.163d0
      force = ( 0.790d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '75'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.849d0
      force = ( 1.382d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '53'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.747d0
      force = ( 1.267d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '55'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.280d0
      force = ( 0.989d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:2 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '64'
      bnlocal(1,2) = ' 3'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-4-* MMFF94 DEF  
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.000d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:1:*-4-* MMFF94 DEF  
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.000d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.423d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.463d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = '42'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.442d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.432d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.444d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.474d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = '42'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.427d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.469d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = '42'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.281d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.551d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.486d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.430d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.648d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.537d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = '42'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.487d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = '42'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.469d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = '42'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.472d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = '42'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.472d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = '42'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.541d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '42'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.474d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '42'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.473d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '42'
      bnlocal(1,2) = ' 4'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-6-* MMFF94 DEF  
      bclocal(0) = 110.400d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:3::*-6-* MMFF94 DEF 
      bclocal(0) =  57.900d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:4:*-6-* MMFF94 DEF  
      bclocal(0) =  90.200d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 106.926d0
      force = ( 1.197d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.309d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.309d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 103.614d0
      force = ( 0.967d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.157d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.375d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.055d0
      force = ( 0.923d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.153d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.252d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.905d0
      force = ( 1.884d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.422d0
      force = ( 1.629d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.496d0
      force = ( 1.628d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.317d0
      force = ( 1.656d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.230d0
      force = ( 1.480d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.951d0
      force = ( 1.493d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.346d0
      force = ( 1.370d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.943d0
      force = ( 1.093d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.833d0
      force = ( 1.316d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 106.503d0
      force = ( 0.793d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.256d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.143d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '21'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.759d0
      force = ( 1.391d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 115.581d0
      force = ( 1.095d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.081d0
      force = ( 1.170d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 102.846d0
      force = ( 1.075d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.163d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.375d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.733d0
      force = ( 1.719d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.046d0
      force = ( 1.454d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.462d0
      force = ( 1.642d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 105.875d0
      force = ( 1.642d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.545d0
      force = ( 1.449d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.848d0
      force = ( 1.512d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.339d0
      force = ( 1.354d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  98.438d0
      force = ( 0.671d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.228d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.052d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.169d0
      force = ( 1.365d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.078d0
      force = ( 1.025d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 105.727d0
      force = ( 0.816d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.259d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.163d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '29'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.694d0
      force = ( 1.418d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.785d0
      force = ( 1.341d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.067d0
      force = ( 1.455d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.404d0
      force = ( 1.409d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.872d0
      force = ( 1.648d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.437d0
      force = ( 1.596d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.468d0
      force = ( 1.274d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.840d0
      force = ( 1.019d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.381d0
      force = ( 1.379d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  91.216d0
      force = ( 1.748d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:X94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.456d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.379d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 4 a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.491d0
      force = ( 1.328d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.948d0
      force = ( 0.583d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.215d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.064d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '24'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.410d0
      force = ( 1.006d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.417d0
      force = ( 0.876d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '29'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  95.300d0
      force = ( 0.614d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.225d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.320d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.483d0
      force = ( 1.424d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.233d0
      force = ( 1.423d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  95.697d0
      force = ( 1.362d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '21'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  99.409d0
      force = ( 0.832d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.304d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.055d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '21'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.592d0
      force = ( 1.115d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '21'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  99.688d0
      force = ( 0.923d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.419d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.158d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '21'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.242d0
      force = ( 1.334d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 115.364d0
      force = ( 0.812d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:X94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.309d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.120d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '33'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.473d0
      force = ( 1.429d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.152d0
      force = ( 1.558d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.479d0
      force = ( 1.710d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 141.096d0
      force = ( 0.642d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '19'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 118.204d0
      force = ( 0.597d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '19'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '21'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.421d0
      force = ( 0.941d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '19'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  89.100d0
      force = ( 1.339d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.739d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.739d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 4 a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.587d0
      force = ( 0.944d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '21'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.394d0
      force = ( 1.394d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.417d0
      force = ( 1.124d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '21'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.253d0
      force = ( 1.058d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '21'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 100.000d0
      force = ( 1.175d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '21'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '54'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 101.000d0
      force = ( 1.139d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '21'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  58.680d0
      force = ( 0.242d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  57.087d0
      force = ( 0.279d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 118.533d0
      force = ( 0.607d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '24'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 129.375d0
      force = ( 0.777d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.923d0
      force = ( 1.099d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.692d0
      force = ( 1.090d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '26'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 108.000d0
      force = ( 0.986d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '29'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 105.409d0
      force = ( 0.726d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.130d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.241d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '29'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.922d0
      force = ( 0.923d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '29'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.967d0
      force = ( 1.462d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.274d0
      force = ( 1.607d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 6'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-8-* MMFF94 DEF  
      bclocal(0) = 110.400d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:3::*-8-* MMFF94 DEF 
      bclocal(0) =  58.500d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:4:*-8-* MMFF94 DEF  
      bclocal(0) =  95.000d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 107.018d0
      force = ( 1.090d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.312d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.312d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 102.829d0
      force = ( 1.297d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.212d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.354d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.708d0
      force = ( 1.347d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.240d0
      force = ( 1.182d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.079d0
      force = ( 1.307d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.283d0
      force = ( 1.085d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.478d0
      force = ( 1.096d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.759d0
      force = ( 0.779d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.873d0
      force = ( 1.221d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.200d0
      force = ( 1.147d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 109.062d0
      force = ( 0.763d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.309d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.135d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '23'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.482d0
      force = ( 0.865d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.630d0
      force = ( 0.926d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.609d0
      force = ( 1.363d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.149d0
      force = ( 1.266d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.092d0
      force = ( 1.265d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '46'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.296d0
      force = ( 1.776d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.334d0
      force = ( 1.664d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.100d0
      force = ( 1.456d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 100.510d0
      force = ( 0.861d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.418d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.020d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '23'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  60.000d0
      force = ( 0.230d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.917d0
      force = ( 0.792d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '23'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.595d0
      force = ( 1.068d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.816d0
      force = ( 1.047d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.864d0
      force = ( 0.832d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '23'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  84.690d0
      force = ( 1.805d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.788d0
      force = ( 0.846d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '23'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.439d0
      force = ( 1.227d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.674d0
      force = ( 0.845d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.056d0
      force = ( 1.198d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '17'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.842d0
      force = ( 0.647d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '17'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '23'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 112.000d0
      force = ( 0.542d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '19'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '23'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  90.370d0
      force = ( 1.103d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.653d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.653d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 4 a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 113.359d0
      force = ( 0.684d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.128d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.122d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '23'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  57.087d0
      force = ( 0.209d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.033d0
      force = ( 0.697d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '23'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.361d0
      force = ( 0.896d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 105.998d0
      force = ( 0.595d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.190d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.190d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '23'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '23'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 117.000d0
      force = ( 0.510d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '23'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.959d0
      force = ( 0.553d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '23'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 109.000d0
      force = ( 0.808d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '23'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.820d0
      force = ( 0.757d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '23'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.120d0
      force = ( 0.819d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '23'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.222d0
      force = ( 0.857d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '23'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 106.000d0
      force = ( 0.868d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '23'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.092d0
      force = ( 0.876d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '23'
      bnlocal(1,2) = ' 8'
      bnlocal(1,3) = '56'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-9-* MMFF94 DEF  
      bclocal(0) = 111.500d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:1:*-9-* MMFF94 DEF  
      bclocal(0) = 109.100d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 106.409d0
      force = ( 0.878d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.326d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.580d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.005d0
      force = ( 1.306d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 113.995d0
      force = ( 1.216d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '53'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.413d0
      force = ( 1.391d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.856d0
      force = ( 1.242d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.528d0
      force = ( 1.306d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.488d0
      force = ( 1.204d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.272d0
      force = ( 1.194d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.872d0
      force = ( 1.579d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.822d0
      force = ( 1.386d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.355d0
      force = ( 1.390d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.548d0
      force = ( 1.365d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.303d0
      force = ( 1.373d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.780d0
      force = ( 1.265d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.743d0
      force = ( 1.205d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.751d0
      force = ( 1.198d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.927d0
      force = ( 0.873d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.779d0
      force = ( 0.818d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.464d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.222d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '27'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.199d0
      force = ( 1.355d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.907d0
      force = ( 1.511d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '35'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.663d0
      force = ( 1.185d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.538d0
      force = ( 1.396d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.440d0
      force = ( 1.365d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.551d0
      force = ( 1.169d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.796d0
      force = ( 1.369d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.578d0
      force = ( 1.351d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '53'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.943d0
      force = ( 1.643d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '54'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.195d0
      force = ( 1.431d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.289d0
      force = ( 1.375d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '56'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.780d0
      force = ( 1.125d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.989d0
      force = ( 1.247d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.461d0
      force = ( 1.302d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.641d0
      force = ( 1.323d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.581d0
      force = ( 1.567d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 161.741d0
      force = ( 0.456d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.868d0
      force = ( 1.402d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.043d0
      force = ( 1.794d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.154d0
      force = ( 1.518d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.014d0
      force = ( 1.397d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.413d0
      force = ( 1.594d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.417d0
      force = ( 1.390d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.325d0
      force = ( 1.320d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.711d0
      force = ( 1.352d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.162d0
      force = ( 1.343d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '53'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.871d0
      force = ( 1.296d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.056d0
      force = ( 1.538d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.149d0
      force = ( 1.318d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '53'
      bnlocal(1,2) = ' 9'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-10-* MMFF94 DEF 
      bclocal(0) = 117.500d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:3::*-10-* MMFF94 DEF
      bclocal(0) =  58.900d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:4:*-10-* MMFF94 DEF 
      bclocal(0) =  92.900d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 117.909d0
      force = ( 1.117d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.063d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.063d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.916d0
      force = ( 1.004d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 119.600d0
      force = ( 0.821d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.021d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.340d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.865d0
      force = ( 1.179d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.024d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.374d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.189d0
      force = ( 1.137d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.005d0
      force = ( 1.132d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.009d0
      force = ( 1.247d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.388d0
      force = ( 1.014d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.679d0
      force = ( 0.960d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.390d0
      force = ( 0.745d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 120.066d0
      force = ( 0.552d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.155d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.051d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.332d0
      force = ( 1.038d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.838d0
      force = ( 1.060d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.314d0
      force = ( 1.194d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.033d0
      force = ( 1.031d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.599d0
      force = ( 1.268d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.185d0
      force = ( 0.949d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.315d0
      force = ( 0.960d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.878d0
      force = ( 1.146d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.703d0
      force = ( 1.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.609d0
      force = ( 1.405d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.544d0
      force = ( 1.132d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.553d0
      force = ( 0.638d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.506d0
      force = ( 0.977d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 120.274d0
      force = ( 0.709d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.219d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.219d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 130.236d0
      force = ( 0.864d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 110.133d0
      force = ( 0.960d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.497d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.513d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.075d0
      force = ( 1.168d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  93.608d0
      force = ( 1.527d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.443d0
      force = ( 1.174d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.377d0
      force = ( 1.184d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.867d0
      force = ( 0.998d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '13'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.162d0
      force = ( 0.871d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '14'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.969d0
      force = ( 1.076d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.612d0
      force = ( 1.132d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.540d0
      force = ( 0.936d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  93.349d0
      force = ( 1.371d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.929d0
      force = ( 0.975d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.157d0
      force = ( 0.794d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.912d0
      force = ( 0.848d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 120.277d0
      force = ( 0.575d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.137d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.066d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.201d0
      force = ( 1.251d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.633d0
      force = ( 1.395d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '35'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.596d0
      force = ( 1.023d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.680d0
      force = ( 1.216d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.913d0
      force = ( 1.098d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.447d0
      force = ( 1.212d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.381d0
      force = ( 1.091d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.574d0
      force = ( 1.048d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 131.702d0
      force = ( 0.816d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.214d0
      force = ( 0.829d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.476d0
      force = ( 1.393d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.160d0
      force = ( 0.703d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.599d0
      force = ( 1.167d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.206d0
      force = ( 0.847d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.501d0
      force = ( 0.751d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.553d0
      force = ( 1.222d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.309d0
      force = ( 1.310d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.715d0
      force = ( 0.735d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.743d0
      force = ( 1.237d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.033d0
      force = ( 0.614d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  91.694d0
      force = ( 1.381d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.394d0
      force = ( 0.555d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.703d0
      force = ( 1.006d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  58.894d0
      force = ( 0.202d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.583d0
      force = ( 0.605d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.785d0
      force = ( 0.447d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 115.630d0
      force = ( 0.435d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.081d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.081d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 113.000d0
      force = ( 0.757d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 114.000d0
      force = ( 0.836d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '35'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.227d0
      force = ( 0.628d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 113.000d0
      force = ( 0.754d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.067d0
      force = ( 0.560d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.099d0
      force = ( 0.640d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.575d0
      force = ( 0.643d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.412d0
      force = ( 1.232d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '10'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-15-* MMFF94 DEF 
      bclocal(0) =  97.900d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:4:*-15-* MMFF94 DEF 
      bclocal(0) =  80.200d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  97.335d0
      force = ( 1.654d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.125d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.125d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  97.853d0
      force = ( 1.321d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  97.326d0
      force = ( 1.325d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  97.370d0
      force = ( 1.344d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  89.814d0
      force = ( 1.725d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 100.316d0
      force = ( 1.377d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.012d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.238d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.641d0
      force = ( 1.309d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.069d0
      force = ( 1.007d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  94.913d0
      force = ( 1.366d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  99.768d0
      force = ( 1.268d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.732d0
      force = ( 0.967d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  95.613d0
      force = ( 1.379d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  97.111d0
      force = ( 1.439d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.048d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.229d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  94.643d0
      force = ( 1.555d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.686d0
      force = ( 1.301d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.330d0
      force = ( 1.304d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.066d0
      force = ( 1.306d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  96.494d0
      force = ( 0.931d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.080d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.012d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '71'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  95.108d0
      force = ( 1.434d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.813d0
      force = ( 1.318d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  95.780d0
      force = ( 1.426d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  97.789d0
      force = ( 1.457d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  96.942d0
      force = ( 1.362d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  90.872d0
      force = ( 1.709d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  95.424d0
      force = ( 1.402d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  94.075d0
      force = ( 1.804d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  99.399d0
      force = ( 1.403d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  79.842d0
      force = ( 1.666d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.541d0
      force = ( 1.308d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  96.051d0
      force = ( 1.390d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) =  97.000d0
      force = ( 0.830d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '71'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  97.231d0
      force = ( 1.679d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.143d0
      force = ( 1.444d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.976d0
      force = ( 1.446d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.524d0
      force = ( 1.626d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  97.105d0
      force = ( 1.504d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.715d0
      force = ( 1.415d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  97.534d0
      force = ( 1.428d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.893d0
      force = ( 1.413d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  99.173d0
      force = ( 1.563d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 100.790d0
      force = ( 1.361d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.040d0
      force = ( 1.332d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  99.239d0
      force = ( 0.787d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.172d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.068d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '71'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  73.428d0
      force = ( 1.978d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  95.589d0
      force = ( 1.361d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  99.505d0
      force = ( 0.947d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  87.982d0
      force = ( 1.030d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  96.851d0
      force = ( 1.002d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  95.428d0
      force = ( 1.172d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  96.710d0
      force = ( 1.144d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '26'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  79.546d0
      force = ( 1.732d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '30'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.802d0
      force = ( 1.295d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  96.197d0
      force = ( 1.379d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  99.423d0
      force = ( 1.286d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  96.222d0
      force = ( 0.813d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.187d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.027d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '71'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  93.377d0
      force = ( 0.734d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.045d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.045d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '71'
      bnlocal(1,2) = '15'
      bnlocal(1,3) = '71'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-17-* MMFF94 DEF 
      bclocal(0) =  99.400d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:4:*-17-* MMFF94 DEF 
      bclocal(0) =  78.400d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) =  93.266d0
      force = ( 1.415d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  94.732d0
      force = ( 1.387d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  92.852d0
      force = ( 1.430d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  92.132d0
      force = ( 1.863d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 107.104d0
      force = ( 1.408d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = ' 7'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  91.498d0
      force = ( 1.661d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  94.839d0
      force = ( 1.547d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  91.368d0
      force = ( 1.453d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  92.591d0
      force = ( 1.423d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  94.911d0
      force = ( 1.376d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  97.901d0
      force = ( 1.313d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.412d0
      force = ( 1.478d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = ' 7'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.882d0
      force = ( 1.207d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.431d0
      force = ( 1.513d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = ' 7'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  97.766d0
      force = ( 2.164d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.431d0
      force = ( 1.850d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = ' 7'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.808d0
      force = ( 1.438d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.549d0
      force = ( 1.525d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.737d0
      force = ( 1.442d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.928d0
      force = ( 1.449d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.313d0
      force = ( 1.500d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  78.354d0
      force = ( 1.891d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  91.169d0
      force = ( 1.687d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  91.633d0
      force = ( 1.487d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '17'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-18-* MMFF94 DEF 
      bclocal(0) = 104.600d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:4:*-18-* MMFF94 DEF 
      bclocal(0) =  80.300d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 101.166d0
      force = ( 1.230d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:X94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.023d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.023d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 100.420d0
      force = ( 1.264d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 100.883d0
      force = ( 1.242d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) =  95.671d0
      force = ( 1.744d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:X94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.003d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.213d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  99.465d0
      force = ( 1.438d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.315d0
      force = ( 1.224d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.417d0
      force = ( 1.207d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 107.066d0
      force = ( 1.446d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:X94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.091d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.390d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.070d0
      force = ( 1.234d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) =  98.014d0
      force = ( 1.449d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:X94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.008d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.607d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 106.586d0
      force = ( 1.277d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '48'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 102.402d0
      force = ( 1.374d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.492d0
      force = ( 1.254d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.668d0
      force = ( 1.664d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  96.849d0
      force = ( 1.539d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.979d0
      force = ( 1.422d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 100.489d0
      force = ( 1.263d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.668d0
      force = ( 1.083d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '48'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 100.361d0
      force = ( 1.418d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.453d0
      force = ( 1.557d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.747d0
      force = ( 1.350d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 103.052d0
      force = ( 1.922d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:X94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.088d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.088d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  97.446d0
      force = ( 1.916d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 108.063d0
      force = ( 1.837d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:X94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.123d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.369d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.229d0
      force = ( 1.528d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.815d0
      force = ( 1.644d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.180d0
      force = ( 1.464d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.945d0
      force = ( 1.583d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.378d0
      force = ( 1.358d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.227d0
      force = ( 1.323d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.959d0
      force = ( 1.584d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.976d0
      force = ( 1.376d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.170d0
      force = ( 1.497d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.399d0
      force = ( 1.324d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.292d0
      force = ( 1.383d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.508d0
      force = ( 1.108d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  80.297d0
      force = ( 1.831d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.247d0
      force = ( 1.465d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 120.924d0
      force = ( 1.569d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:X94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.404d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.404d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 105.280d0
      force = ( 1.497d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 101.600d0
      force = ( 1.804d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 108.548d0
      force = ( 1.569d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:X94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.384d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.281d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 126.841d0
      force = ( 1.229d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '48'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.548d0
      force = ( 1.509d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.139d0
      force = ( 1.592d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 121.426d0
      force = ( 1.326d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.212d0
      force = ( 1.571d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.771d0
      force = ( 1.634d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.401d0
      force = ( 1.400d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.380d0
      force = ( 1.157d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) =  99.854d0
      force = ( 1.404d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) =  99.200d0
      force = ( 1.416d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.466d0
      force = ( 1.330d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '48'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 100.926d0
      force = ( 1.397d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.665d0
      force = ( 1.178d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.735d0
      force = ( 1.202d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) =  99.905d0
      force = ( 1.545d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:X94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.428d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.428d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '43'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.868d0
      force = ( 1.285d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '43'
      bnlocal(1,2) = '18'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-19-* MMFF94 DEF 
      bclocal(0) = 108.700d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:4:*-19-* MMFF94 DEF 
      bclocal(0) =  89.900d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.339d0
      force = ( 0.616d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 110.795d0
      force = ( 0.390d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 113.958d0
      force = ( 0.777d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.521d0
      force = ( 0.716d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.380d0
      force = ( 0.779d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 108.947d0
      force = ( 0.729d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.828d0
      force = ( 0.656d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.858d0
      force = ( 0.754d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.924d0
      force = ( 0.699d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.633d0
      force = ( 0.530d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = '75'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.981d0
      force = ( 0.819d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 108.699d0
      force = ( 0.258d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 109.677d0
      force = ( 0.520d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.070d0
      force = ( 0.461d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 106.756d0
      force = ( 0.446d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.280d0
      force = ( 1.051d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.022d0
      force = ( 0.968d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.096d0
      force = ( 0.870d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.099d0
      force = ( 0.862d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.683d0
      force = ( 0.786d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.597d0
      force = ( 0.879d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.681d0
      force = ( 0.816d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  89.931d0
      force = ( 0.802d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.045d0
      force = ( 0.726d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '19'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-20-* MMFF94 DEF 
      bclocal(0) = 113.200d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:4:*-20-* MMFF94 DEF 
      bclocal(0) =  88.800d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.131d0
      force = ( 0.943d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.940d0
      force = ( 0.906d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 114.057d0
      force = ( 0.417d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.290d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.098d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.677d0
      force = ( 1.231d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.090d0
      force = ( 1.080d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.057d0
      force = ( 1.100d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.993d0
      force = ( 1.173d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.773d0
      force = ( 0.976d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.226d0
      force = ( 1.035d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.383d0
      force = ( 0.978d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 113.313d0
      force = ( 0.502d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.179d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.004d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.201d0
      force = ( 0.915d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.096d0
      force = ( 0.744d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.611d0
      force = ( 0.721d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.220d0
      force = ( 0.908d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.505d0
      force = ( 1.090d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.650d0
      force = ( 0.947d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.787d0
      force = ( 0.973d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.187d0
      force = ( 1.087d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.074d0
      force = ( 1.132d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.060d0
      force = ( 0.982d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.035d0
      force = ( 0.596d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.851d0
      force = ( 1.139d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.750d0
      force = ( 0.951d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.138d0
      force = ( 0.931d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.919d0
      force = ( 0.982d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 112.989d0
      force = ( 0.624d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.049d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.171d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.611d0
      force = ( 1.157d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  87.271d0
      force = ( 1.473d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.988d0
      force = ( 1.016d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.849d0
      force = ( 1.184d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.891d0
      force = ( 0.969d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.951d0
      force = ( 1.008d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '13'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.273d0
      force = ( 0.849d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  88.961d0
      force = ( 1.524d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.607d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.437d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 4 a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.667d0
      force = ( 1.137d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  85.619d0
      force = ( 1.382d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.707d0
      force = ( 0.960d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.078d0
      force = ( 0.584d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.312d0
      force = ( 0.920d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 109.107d0
      force = ( 0.439d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.182d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.182d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.352d0
      force = ( 0.818d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.051d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.312d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 114.011d0
      force = ( 0.728d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.072d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.226d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.826d0
      force = ( 0.657d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.010d0
      force = ( 0.663d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 114.117d0
      force = ( 0.339d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.014d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.597d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.339d0
      force = ( 0.562d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 113.000d0
      force = ( 0.561d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.570d0
      force = ( 0.605d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 113.940d0
      force = ( 0.564d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.101d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.079d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.722d0
      force = ( 0.472d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 116.038d0
      force = ( 0.688d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.108d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.123d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 112.000d0
      force = ( 0.661d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.670d0
      force = ( 0.552d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.331d0
      force = ( 0.682d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.686d0
      force = ( 0.655d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.408d0
      force = ( 1.443d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.666d0
      force = ( 1.225d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.868d0
      force = ( 1.162d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '13'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.117d0
      force = ( 1.109d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  93.413d0
      force = ( 1.433d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.823d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.396d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 4 a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.205d0
      force = ( 1.106d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.705d0
      force = ( 1.144d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  87.873d0
      force = ( 1.658d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.606d0
      force = ( 1.185d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  91.244d0
      force = ( 1.486d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.701d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.369d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 4 a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.782d0
      force = ( 0.874d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.640d0
      force = ( 1.103d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.525d0
      force = ( 1.170d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.564d0
      force = ( 1.127d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 100.845d0
      force = ( 1.404d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.170d0
      force = ( 1.032d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  87.497d0
      force = ( 1.468d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  86.657d0
      force = ( 1.507d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.360d0
      force = ( 0.963d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.020d0
      force = ( 1.504d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.460d0
      force = ( 1.221d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.673d0
      force = ( 1.051d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.309d0
      force = ( 0.997d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.603d0
      force = ( 1.020d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.821d0
      force = ( 0.973d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 118.108d0
      force = ( 0.866d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.310d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.000d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.399d0
      force = ( 0.887d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.361d0
      force = ( 1.077d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '13'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '13'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.037d0
      force = ( 0.938d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '13'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.888d0
      force = ( 0.837d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '14'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.048d0
      force = ( 1.094d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.793d0
      force = ( 1.058d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  90.483d0
      force = ( 1.324d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.468d0
      force = ( 0.960d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  86.726d0
      force = ( 1.447d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  94.977d0
      force = ( 1.309d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '17'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.108d0
      force = ( 0.930d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '17'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.480d0
      force = ( 1.007d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  90.185d0
      force = ( 1.355d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.656d0
      force = ( 1.241d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.298d0
      force = ( 0.567d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '19'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  88.477d0
      force = ( 0.921d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '19'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.644d0
      force = ( 1.008d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  90.294d0
      force = ( 1.149d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.283d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.283d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 4 a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.817d0
      force = ( 0.840d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  86.669d0
      force = ( 1.364d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  84.818d0
      force = ( 1.181d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.745d0
      force = ( 0.994d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  85.303d0
      force = ( 1.399d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.340d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.529d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 4 a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.143d0
      force = ( 1.069d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  90.128d0
      force = ( 1.382d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.709d0
      force = ( 0.833d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  86.810d0
      force = ( 1.346d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.254d0
      force = ( 1.097d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.408d0
      force = ( 0.922d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.540d0
      force = ( 0.964d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  92.879d0
      force = ( 1.290d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.090d0
      force = ( 1.083d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.829d0
      force = ( 0.866d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  79.399d0
      force = ( 1.649d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  96.811d0
      force = ( 0.789d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '26'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.805d0
      force = ( 0.843d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '26'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.943d0
      force = ( 1.070d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '34'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.365d0
      force = ( 0.954d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '20'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-22-* MMFF94 DEF 
      bclocal(0) = 116.100d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:3::*-22-* MMFF94 DEF
      bclocal(0) =  59.400d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:4:*-22-* MMFF94 DEF 
      bclocal(0) =  91.600d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.483d0
      force = ( 0.903d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.360d0
      force = ( 0.884d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.424d0
      force = ( 0.836d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.720d0
      force = ( 0.900d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.788d0
      force = ( 0.604d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:X94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.067d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.174d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.545d0
      force = ( 1.179d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.469d0
      force = ( 0.973d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.087d0
      force = ( 1.070d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.265d0
      force = ( 1.097d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.246d0
      force = ( 0.871d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:X94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.199d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.039d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.041d0
      force = ( 0.882d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.899d0
      force = ( 1.014d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  48.820d0
      force = ( 0.263d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.147d0
      force = ( 0.956d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.957d0
      force = ( 0.784d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.869d0
      force = ( 0.573d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.319d0
      force = ( 1.012d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.260d0
      force = ( 0.880d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  60.845d0
      force = ( 0.166d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.146d0
      force = ( 1.009d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.977d0
      force = ( 0.819d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.718d0
      force = ( 0.876d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.738d0
      force = ( 0.559d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.646d0
      force = ( 1.184d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.261d0
      force = ( 1.072d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.750d0
      force = ( 0.987d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.047d0
      force = ( 0.930d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  90.869d0
      force = ( 1.267d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.252d0
      force = ( 0.861d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  93.287d0
      force = ( 1.196d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  89.217d0
      force = ( 1.301d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.464d0
      force = ( 0.852d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.288d0
      force = ( 1.033d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.441d0
      force = ( 1.124d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.033d0
      force = ( 1.117d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 118.000d0
      force = ( 0.560d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.650d0
      force = ( 1.200d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.034d0
      force = ( 0.966d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.455d0
      force = ( 0.931d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.890d0
      force = ( 0.877d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.227d0
      force = ( 1.089d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 114.938d0
      force = ( 0.242d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.254d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.254d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.836d0
      force = ( 0.683d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.758d0
      force = ( 0.621d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.806d0
      force = ( 0.658d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 108.296d0
      force = ( 0.776d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 109.865d0
      force = ( 0.620d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 110.000d0
      force = ( 0.623d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 117.875d0
      force = ( 0.583d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.181d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.108d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.438d0
      force = ( 0.532d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.855d0
      force = ( 0.653d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 122.000d0
      force = ( 0.519d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.128d0
      force = ( 0.658d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 112.000d0
      force = ( 0.665d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.409d0
      force = ( 1.136d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.583d0
      force = ( 1.328d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '17'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.009d0
      force = ( 1.381d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.942d0
      force = ( 1.124d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  60.711d0
      force = ( 0.205d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.170d0
      force = ( 1.093d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  68.138d0
      force = ( 0.179d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.368d0
      force = ( 1.422d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.144d0
      force = ( 0.925d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  61.507d0
      force = ( 0.176d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.411d0
      force = ( 0.916d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  60.603d0
      force = ( 0.184d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.859d0
      force = ( 1.610d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 116.086d0
      force = ( 1.062d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.988d0
      force = ( 1.067d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 117.971d0
      force = ( 0.925d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.473d0
      force = ( 1.085d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '13'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '13'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.606d0
      force = ( 0.908d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '13'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.404d0
      force = ( 0.918d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.106d0
      force = ( 1.029d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '17'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.054d0
      force = ( 1.078d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.430d0
      force = ( 0.812d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  92.930d0
      force = ( 1.198d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.070d0
      force = ( 0.787d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  60.000d0
      force = ( 0.171d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.000d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.000d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 5 a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  91.653d0
      force = ( 1.225d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.514d0
      force = ( 0.777d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.415d0
      force = ( 0.983d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.135d0
      force = ( 0.847d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  61.163d0
      force = ( 0.178d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.045d0
      force = ( 0.886d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  61.536d0
      force = ( 0.176d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.380d0
      force = ( 1.022d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.095d0
      force = ( 1.008d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '34'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.774d0
      force = ( 0.846d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  51.029d0
      force = ( 0.237d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.789d0
      force = ( 0.936d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '22'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-25-* MMFF94 DEF 
      bclocal(0) = 106.500d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:4:*-25-* MMFF94 DEF 
      bclocal(0) =  89.100d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) =  99.158d0
      force = ( 1.072d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  91.423d0
      force = ( 1.268d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) =  98.288d0
      force = ( 1.394d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.775d0
      force = ( 1.150d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.890d0
      force = ( 1.180d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.431d0
      force = ( 1.074d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 100.707d0
      force = ( 0.852d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 107.891d0
      force = ( 1.186d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.924d0
      force = ( 0.972d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  93.644d0
      force = ( 1.358d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) =  98.760d0
      force = ( 1.190d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.363d0
      force = ( 0.537d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '71'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 111.306d0
      force = ( 0.976d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.892d0
      force = ( 1.302d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.148d0
      force = ( 1.022d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  85.839d0
      force = ( 1.629d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.127d0
      force = ( 0.983d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.249d0
      force = ( 0.863d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.026d0
      force = ( 1.277d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.307d0
      force = ( 1.164d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) =  99.311d0
      force = ( 1.769d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.161d0
      force = ( 1.419d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.407d0
      force = ( 1.403d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.194d0
      force = ( 1.448d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  99.260d0
      force = ( 1.680d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.818d0
      force = ( 1.489d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 109.688d0
      force = ( 1.501d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.280d0
      force = ( 1.312d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  97.314d0
      force = ( 1.617d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.601d0
      force = ( 1.380d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 100.242d0
      force = ( 0.844d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '71'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.058d0
      force = ( 1.219d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.341d0
      force = ( 1.224d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.893d0
      force = ( 1.214d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.655d0
      force = ( 1.411d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.094d0
      force = ( 1.010d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.325d0
      force = ( 1.217d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.742d0
      force = ( 1.106d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.617d0
      force = ( 1.265d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.767d0
      force = ( 0.977d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.493d0
      force = ( 1.232d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.856d0
      force = ( 1.346d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.640d0
      force = ( 1.273d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.624d0
      force = ( 1.021d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.045d0
      force = ( 1.528d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  99.224d0
      force = ( 1.303d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.320d0
      force = ( 1.305d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.673d0
      force = ( 1.113d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  93.138d0
      force = ( 1.264d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.964d0
      force = ( 1.248d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.729d0
      force = ( 0.933d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  85.039d0
      force = ( 1.220d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.595d0
      force = ( 0.965d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.612d0
      force = ( 0.890d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 122.857d0
      force = ( 1.248d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.430d0
      force = ( 1.097d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  99.255d0
      force = ( 1.605d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.057d0
      force = ( 1.122d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 110.308d0
      force = ( 1.257d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.740d0
      force = ( 1.219d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.168d0
      force = ( 1.211d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 117.733d0
      force = ( 0.642d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '71'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.823d0
      force = ( 1.050d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.124d0
      force = ( 0.947d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.107d0
      force = ( 0.965d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.776d0
      force = ( 0.868d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  95.270d0
      force = ( 1.496d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.441d0
      force = ( 1.035d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.995d0
      force = ( 1.059d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '57'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.950d0
      force = ( 1.032d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 100.483d0
      force = ( 0.419d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '71'
      bnlocal(1,2) = '25'
      bnlocal(1,3) = '71'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-26-* MMFF94 DEF 
      bclocal(0) =  98.100d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:4:*-26-* MMFF94 DEF 
      bclocal(0) =  83.600d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.054d0
      force = ( 1.085d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  96.331d0
      force = ( 1.263d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.175d0
      force = ( 1.115d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) =  98.926d0
      force = ( 1.147d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 100.260d0
      force = ( 1.141d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.171d0
      force = ( 1.075d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  92.571d0
      force = ( 0.997d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  98.754d0
      force = ( 1.081d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) =  97.353d0
      force = ( 0.672d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '71'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  97.935d0
      force = ( 1.833d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 100.061d0
      force = ( 1.663d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  99.021d0
      force = ( 1.442d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.662d0
      force = ( 1.189d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.069d0
      force = ( 1.028d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  93.096d0
      force = ( 1.509d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  94.795d0
      force = ( 1.757d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  99.730d0
      force = ( 1.271d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  90.565d0
      force = ( 1.508d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.783d0
      force = ( 1.165d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) =  96.577d0
      force = ( 0.704d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '71'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  96.592d0
      force = ( 1.047d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  91.164d0
      force = ( 1.543d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  83.624d0
      force = ( 1.252d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) =  94.470d0
      force = ( 0.473d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '71'
      bnlocal(1,2) = '26'
      bnlocal(1,3) = '71'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-30-* MMFF94 DEF 
      bclocal(0) = 134.200d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:1:*-30-* MMFF94 DEF 
      bclocal(0) = 131.800d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:4:*-30-* MMFF94 DEF 
      bclocal(0) =  97.700d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:7:*-30-* MMFF94 DEF 
      bclocal(0) =  92.300d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:7 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.756d0
      force = ( 0.778d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 130.439d0
      force = ( 0.805d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 132.187d0
      force = ( 0.727d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 131.100d0
      force = ( 0.737d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 132.225d0
      force = ( 0.751d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 134.566d0
      force = ( 0.721d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 135.975d0
      force = ( 0.410d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = ' 5'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 137.596d0
      force = ( 0.845d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 130.677d0
      force = ( 0.714d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  89.957d0
      force = ( 1.280d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:7 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.418d0
      force = ( 0.857d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  93.102d0
      force = ( 1.260d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:7 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 136.444d0
      force = ( 0.690d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 131.835d0
      force = ( 0.390d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.251d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.007d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 132.652d0
      force = ( 0.364d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.267d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.054d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 139.045d0
      force = ( 0.876d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 130.718d0
      force = ( 0.876d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.359d0
      force = ( 1.239d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 132.228d0
      force = ( 0.782d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 100.902d0
      force = ( 1.141d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  95.513d0
      force = ( 1.117d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.413d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.705d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 4 a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  93.909d0
      force = ( 1.191d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:7 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 134.526d0
      force = ( 0.769d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 138.631d0
      force = ( 0.704d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  93.007d0
      force = ( 1.179d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  93.732d0
      force = ( 1.230d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:8 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '30'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 145.470d0
      force = ( 0.706d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '30'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.792d0
      force = ( 0.907d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '30'
      bnlocal(1,2) = '30'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-34-* MMFF94 DEF 
      bclocal(0) = 109.400d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:4:*-34-* MMFF94 DEF 
      bclocal(0) =  89.400d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 112.251d0
      force = ( 0.862d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.202d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.202d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.212d0
      force = ( 1.154d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.399d0
      force = ( 1.330d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.989d0
      force = ( 1.166d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.291d0
      force = ( 1.388d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.135d0
      force = ( 1.201d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.004d0
      force = ( 0.913d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = '26'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.206d0
      force = ( 0.576d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.160d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.009d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.045d0
      force = ( 1.141d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 112.000d0
      force = ( 0.694d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.753d0
      force = ( 0.796d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.649d0
      force = ( 0.793d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 108.000d0
      force = ( 0.828d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  89.411d0
      force = ( 1.448d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.526d0
      force = ( 0.665d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 110.000d0
      force = ( 0.694d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 107.787d0
      force = ( 0.578d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.087d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.087d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.668d0
      force = ( 0.717d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 108.000d0
      force = ( 0.840d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '34'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-37-* MMFF94 DEF 
      bclocal(0) = 118.800d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:1:*-37-* MMFF94 DEF 
      bclocal(0) = 115.900d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:3::*-37-* MMFF94 DEF
      bclocal(0) =  64.700d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:4:*-37-* MMFF94 DEF 
      bclocal(0) =  91.800d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 120.419d0
      force = ( 0.803d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.485d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.311d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.432d0
      force = ( 0.992d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.528d0
      force = ( 1.027d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.024d0
      force = ( 0.837d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.073d0
      force = ( 0.821d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.506d0
      force = ( 1.038d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '69'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 119.695d0
      force = ( 0.712d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.321d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.235d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 1 a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.220d0
      force = ( 1.029d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 114.475d0
      force = ( 0.798d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.179d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.217d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 1 a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  90.784d0
      force = ( 1.320d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:7 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.724d0
      force = ( 1.109d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.566d0
      force = ( 1.134d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.916d0
      force = ( 1.119d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '69'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.614d0
      force = ( 0.906d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.623d0
      force = ( 1.087d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 120.571d0
      force = ( 0.563d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.279d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.250d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 115.588d0
      force = ( 0.693d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.267d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.389d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.316d0
      force = ( 0.699d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 121.238d0
      force = ( 0.702d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.216d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.434d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 121.446d0
      force = ( 0.523d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.167d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.364d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.638d0
      force = ( 0.794d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.273d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.391d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '69'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.432d0
      force = ( 0.563d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 116.495d0
      force = ( 0.968d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.830d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.339d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.886d0
      force = ( 1.324d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.868d0
      force = ( 1.139d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.003d0
      force = ( 0.974d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.591d0
      force = ( 1.137d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.918d0
      force = ( 1.025d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.135d0
      force = ( 1.088d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.925d0
      force = ( 1.077d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.065d0
      force = ( 1.094d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.328d0
      force = ( 1.223d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.495d0
      force = ( 0.950d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.859d0
      force = ( 1.126d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.320d0
      force = ( 1.076d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.117d0
      force = ( 0.917d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '13'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.045d0
      force = ( 0.861d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '14'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 121.037d0
      force = ( 0.755d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.650d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.259d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.421d0
      force = ( 1.027d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.125d0
      force = ( 0.976d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.408d0
      force = ( 0.930d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '17'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.828d0
      force = ( 1.179d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '17'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.357d0
      force = ( 0.946d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '17'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 113.991d0
      force = ( 1.029d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.908d0
      force = ( 1.278d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.029d0
      force = ( 0.975d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.278d0
      force = ( 0.660d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '19'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 129.614d0
      force = ( 0.744d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  93.425d0
      force = ( 1.217d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.777d0
      force = ( 0.805d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  64.704d0
      force = ( 0.152d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.494d0
      force = ( 0.904d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.600d0
      force = ( 0.718d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.967d0
      force = ( 0.691d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '26'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.423d0
      force = ( 1.030d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '34'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.905d0
      force = ( 1.074d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '34'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 131.858d0
      force = ( 0.964d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '35'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.980d0
      force = ( 1.187d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '35'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 119.977d0
      force = ( 0.669d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.411d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.411d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.227d0
      force = ( 0.864d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  90.193d0
      force = ( 1.380d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 126.139d0
      force = ( 0.596d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.424d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.466d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.271d0
      force = ( 1.033d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.619d0
      force = ( 1.038d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.622d0
      force = ( 1.078d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 121.633d0
      force = ( 1.045d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.429d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.901d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.572d0
      force = ( 0.892d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 117.860d0
      force = ( 1.013d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.337d0
      force = ( 1.114d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.038d0
      force = ( 0.999d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '46'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.163d0
      force = ( 1.002d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.801d0
      force = ( 1.020d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '56'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.932d0
      force = ( 0.881d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.052d0
      force = ( 1.014d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.251d0
      force = ( 1.127d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.515d0
      force = ( 1.072d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '61'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.384d0
      force = ( 0.941d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.243d0
      force = ( 0.478d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.173d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.215d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.190d0
      force = ( 0.894d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 112.567d0
      force = ( 0.423d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.229d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.229d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.973d0
      force = ( 0.912d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.980d0
      force = ( 1.064d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 116.778d0
      force = ( 0.872d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.244d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.555d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '69'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.438d0
      force = ( 1.042d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '69'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.439d0
      force = ( 0.974d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.664d0
      force = ( 1.034d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.759d0
      force = ( 1.104d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 128.938d0
      force = ( 0.725d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.516d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.516d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.755d0
      force = ( 1.024d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.355d0
      force = ( 1.165d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.362d0
      force = ( 0.979d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.356d0
      force = ( 1.257d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.349d0
      force = ( 1.148d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.386d0
      force = ( 1.095d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.910d0
      force = ( 1.076d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.605d0
      force = ( 1.070d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.610d0
      force = ( 1.289d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.417d0
      force = ( 1.103d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.904d0
      force = ( 0.943d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.541d0
      force = ( 0.931d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.604d0
      force = ( 0.931d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.535d0
      force = ( 0.967d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '41'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.781d0
      force = ( 1.031d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.199d0
      force = ( 1.156d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.041d0
      force = ( 1.248d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '69'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.987d0
      force = ( 1.016d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '58'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.628d0
      force = ( 1.152d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '58'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.250d0
      force = ( 1.291d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '58'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.166d0
      force = ( 1.108d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '58'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.842d0
      force = ( 1.188d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '58'
      bnlocal(1,2) = '37'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-38-* MMFF94 DEF 
      bclocal(0) = 113.800d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '38'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 115.406d0
      force = ( 1.085d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.342d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.342d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '38'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 112.016d0
      force = ( 1.289d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.164d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-1.130d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '38'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.181d0
      force = ( 1.230d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '38'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.032d0
      force = ( 1.207d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '38'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.692d0
      force = ( 1.238d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '38'
      bnlocal(1,3) = '69'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.813d0
      force = ( 1.118d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '38'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.516d0
      force = ( 1.343d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '38'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-39-* MMFF94 DEF 
      bclocal(0) = 120.700d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:1:*-39-* MMFF94 DEF 
      bclocal(0) = 125.400d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 123.380d0
      force = ( 0.854d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.313d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.500d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.049d0
      force = ( 1.111d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 130.275d0
      force = ( 0.858d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 133.220d0
      force = ( 0.900d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.045d0
      force = ( 0.900d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.909d0
      force = ( 1.126d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.985d0
      force = ( 1.166d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.707d0
      force = ( 1.396d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.868d0
      force = ( 1.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.145d0
      force = ( 1.057d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.725d0
      force = ( 0.981d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.487d0
      force = ( 1.170d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.788d0
      force = ( 1.109d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.961d0
      force = ( 1.118d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 117.061d0
      force = ( 1.108d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 127.770d0
      force = ( 0.551d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.131d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.422d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '23'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 118.352d0
      force = ( 0.752d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.122d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.281d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '23'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 124.000d0
      force = ( 0.581d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '23'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 134.561d0
      force = ( 0.667d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.135d0
      force = ( 0.944d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.009d0
      force = ( 0.900d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.312d0
      force = ( 0.922d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.090d0
      force = ( 1.080d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.832d0
      force = ( 0.984d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.641d0
      force = ( 1.056d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.464d0
      force = ( 1.354d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 109.599d0
      force = ( 1.152d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.469d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.469d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.078d0
      force = ( 0.887d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.577d0
      force = ( 1.004d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.936d0
      force = ( 0.899d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 112.087d0
      force = ( 1.284d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.741d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.506d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.990d0
      force = ( 1.146d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.800d0
      force = ( 1.300d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.117d0
      force = ( 1.007d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '64'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 116.898d0
      force = ( 1.462d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.706d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.706d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '65'
      bnlocal(1,2) = '39'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-40-* MMFF94 DEF 
      bclocal(0) = 115.000d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:3::*-40-* MMFF94 DEF
      bclocal(0) =  57.800d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.703d0
      force = ( 1.064d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.873d0
      force = ( 0.998d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.319d0
      force = ( 1.007d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.742d0
      force = ( 1.421d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.198d0
      force = ( 1.203d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.320d0
      force = ( 1.232d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.665d0
      force = ( 1.436d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '11'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.320d0
      force = ( 1.202d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.970d0
      force = ( 1.047d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.483d0
      force = ( 0.912d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 112.374d0
      force = ( 0.689d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.238d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.091d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.604d0
      force = ( 1.024d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 107.349d0
      force = ( 0.835d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.153d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.590d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.622d0
      force = ( 1.254d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.011d0
      force = ( 1.183d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.226d0
      force = ( 1.223d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.982d0
      force = ( 1.025d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '46'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.473d0
      force = ( 1.084d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.483d0
      force = ( 1.064d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.651d0
      force = ( 0.997d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.660d0
      force = ( 0.981d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.626d0
      force = ( 1.316d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.196d0
      force = ( 1.118d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.260d0
      force = ( 1.142d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.087d0
      force = ( 0.732d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.053d0
      force = ( 0.767d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.342d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.156d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.022d0
      force = ( 1.049d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.106d0
      force = ( 1.192d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.253d0
      force = ( 1.060d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.447d0
      force = ( 1.008d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.240d0
      force = ( 0.883d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.557d0
      force = ( 1.259d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.822d0
      force = ( 1.106d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.261d0
      force = ( 1.269d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '10'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.718d0
      force = ( 1.146d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '12'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.871d0
      force = ( 1.105d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.139d0
      force = ( 1.130d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.420d0
      force = ( 1.072d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.724d0
      force = ( 0.820d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 114.808d0
      force = ( 0.700d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.228d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.104d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.655d0
      force = ( 1.056d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.511d0
      force = ( 1.147d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.602d0
      force = ( 1.132d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 110.000d0
      force = ( 0.889d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.915d0
      force = ( 0.764d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.920d0
      force = ( 1.216d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.085d0
      force = ( 1.351d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.549d0
      force = ( 0.774d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.751d0
      force = ( 1.236d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.725d0
      force = ( 0.799d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.686d0
      force = ( 1.316d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.687d0
      force = ( 1.546d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '11'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.497d0
      force = ( 1.154d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  57.777d0
      force = ( 0.204d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.220d0
      force = ( 1.066d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.006d0
      force = ( 1.126d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 120.000d0
      force = ( 0.485d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.977d0
      force = ( 0.868d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 118.000d0
      force = ( 0.506d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '26'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.336d0
      force = ( 0.812d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '26'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 109.160d0
      force = ( 0.560d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.094d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.094d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.230d0
      force = ( 0.656d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 110.288d0
      force = ( 0.662d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.186d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.423d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.951d0
      force = ( 0.789d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.731d0
      force = ( 0.782d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 120.000d0
      force = ( 0.674d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.714d0
      force = ( 0.738d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '54'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.188d0
      force = ( 0.670d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.057d0
      force = ( 0.659d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.829d0
      force = ( 0.618d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.018d0
      force = ( 1.004d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.579d0
      force = ( 1.376d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.777d0
      force = ( 1.394d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '54'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.867d0
      force = ( 1.060d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.332d0
      force = ( 1.283d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.678d0
      force = ( 1.410d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.345d0
      force = ( 1.189d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '46'
      bnlocal(1,2) = '40'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-41-* MMFF94 DEF 
      bclocal(0) = 118.300d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 114.689d0
      force = ( 1.209d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.503d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.943d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 114.936d0
      force = ( 1.024d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 115.461d0
      force = ( 1.309d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.594d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.969d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.810d0
      force = ( 1.210d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 113.960d0
      force = ( 0.912d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.276d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.852d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.899d0
      force = ( 1.319d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.795d0
      force = ( 1.089d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.240d0
      force = ( 1.039d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.965d0
      force = ( 1.090d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.748d0
      force = ( 1.079d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 130.600d0
      force = ( 1.181d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.652d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.652d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.871d0
      force = ( 1.136d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.694d0
      force = ( 1.401d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.919d0
      force = ( 1.035d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.972d0
      force = ( 0.982d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '55'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.425d0
      force = ( 1.052d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '62'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 130.128d0
      force = ( 0.912d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '72'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.175d0
      force = ( 1.094d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '72'
      bnlocal(1,2) = '41'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-43-* MMFF94 DEF 
      bclocal(0) = 113.300d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.353d0
      force = ( 1.109d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.321d0
      force = ( 1.052d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = ' 2'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.050d0
      force = ( 0.938d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.204d0
      force = ( 0.927d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = ' 4'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 115.011d0
      force = ( 1.116d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 115.637d0
      force = ( 0.853d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '25'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 113.739d0
      force = ( 0.646d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.511d0
      force = ( 1.083d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.034d0
      force = ( 1.140d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.188d0
      force = ( 1.025d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.268d0
      force = ( 1.227d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 121.488d0
      force = ( 1.011d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.913d0
      force = ( 1.053d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  93.575d0
      force = ( 1.327d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 117.464d0
      force = ( 0.626d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.000d0
      force = ( 0.616d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.373d0
      force = ( 1.253d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.311d0
      force = ( 1.673d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  54.827d0
      force = ( 0.279d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 110.000d0
      force = ( 0.868d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.833d0
      force = ( 1.519d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.652d0
      force = ( 1.603d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.036d0
      force = ( 1.511d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.320d0
      force = ( 0.794d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.008d0
      force = ( 1.558d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '15'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.458d0
      force = ( 1.409d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.904d0
      force = ( 1.367d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '17'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.463d0
      force = ( 1.144d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.768d0
      force = ( 0.961d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  92.867d0
      force = ( 1.451d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:4 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '20'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.379d0
      force = ( 1.171d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 116.881d0
      force = ( 0.628d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.347d0
      force = ( 1.324d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 112.132d0
      force = ( 1.185d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.036d0
      force = ( 1.379d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.279d0
      force = ( 1.108d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 115.000d0
      force = ( 0.626d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  57.032d0
      force = ( 0.209d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:3 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '22'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 118.274d0
      force = ( 0.468d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 112.596d0
      force = ( 0.477d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 110.000d0
      force = ( 0.810d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '34'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 113.350d0
      force = ( 0.669d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.293d0
      force = ( 0.658d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '28'
      bnlocal(1,2) = '43'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-44-* MMFF94 DEF 
      bclocal(0) =  91.600d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '44'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  88.495d0
      force = ( 1.962d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.591d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.591d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '44'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) =  94.137d0
      force = ( 2.261d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.857d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.978d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '44'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  86.270d0
      force = ( 1.738d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '44'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  86.194d0
      force = ( 1.748d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '44'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.147d0
      force = ( 1.530d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '65'
      bnlocal(1,2) = '44'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  93.534d0
      force = ( 1.629d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '65'
      bnlocal(1,2) = '44'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.401d0
      force = ( 0.903d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '78'
      bnlocal(1,2) = '44'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-45-* MMFF94 DEF 
      bclocal(0) = 116.700d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 118.182d0
      force = ( 1.260d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 118.082d0
      force = ( 1.294d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.589d0
      force = ( 1.343d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 111.682d0
      force = ( 1.787d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.695d0
      force = ( 1.515d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.850d0
      force = ( 1.339d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.194d0
      force = ( 1.578d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.893d0
      force = ( 1.245d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '20'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.503d0
      force = ( 1.293d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '22'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 128.036d0
      force = ( 1.467d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.857d0
      force = ( 1.298d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.633d0
      force = ( 1.715d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.432d0
      force = ( 1.497d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.711d0
      force = ( 1.545d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '43'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.765d0
      force = ( 1.335d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.908d0
      force = ( 1.330d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.962d0
      force = ( 1.394d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '45'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-46-* MMFF94 DEF 
      bclocal(0) = 111.000d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '46'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 110.492d0
      force = ( 1.440d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '46'
      bnlocal(1,3) = ' 7'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.709d0
      force = ( 1.489d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '46'
      bnlocal(1,3) = ' 7'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.817d0
      force = ( 1.724d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = '46'
      bnlocal(1,3) = ' 8'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.569d0
      force = ( 1.519d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = '46'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.405d0
      force = ( 1.650d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 7'
      bnlocal(1,2) = '46'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-48-* MMFF94 DEF 
      bclocal(0) = 118.400d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '48'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.928d0
      force = ( 1.065d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '48'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 113.969d0
      force = ( 0.736d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '48'
      bnlocal(1,3) = '28'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-49-* MMFF94 DEF 
      bclocal(0) = 111.400d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '49'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.433d0
      force = ( 0.522d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.072d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.072d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '50'
      bnlocal(1,2) = '49'
      bnlocal(1,3) = '50'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-51-* MMFF94 DEF 
      bclocal(0) = 111.400d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '51'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 111.360d0
      force = ( 0.913d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '51'
      bnlocal(1,3) = '52'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-53-* MMFF94 DEF 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.000d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '53'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.574d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '53'
      bnlocal(1,3) = '47'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.649d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '53'
      bnlocal(1,3) = '47'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-54-* MMFF94 DEF 
      bclocal(0) = 119.500d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '54'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:1:*-54-* MMFF94 DEF 
      bclocal(0) = 115.700d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '54'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.439d0
      force = ( 0.923d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '54'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 124.083d0
      force = ( 0.707d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.192d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.051d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '54'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 122.881d0
      force = ( 0.294d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.240d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.079d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '54'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.398d0
      force = ( 1.376d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '54'
      bnlocal(1,3) = ' 6'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.457d0
      force = ( 1.128d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '54'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 119.698d0
      force = ( 0.685d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.005d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.127d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '54'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.439d0
      force = ( 1.105d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '54'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 115.000d0
      force = ( 0.826d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '54'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.403d0
      force = ( 1.195d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '54'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 113.943d0
      force = ( 0.300d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.148d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.148d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '54'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-55-* MMFF94 DEF 
      bclocal(0) = 120.800d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.946d0
      force = ( 0.951d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 126.448d0
      force = ( 0.307d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.189d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.033d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.035d0
      force = ( 1.032d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 120.606d0
      force = ( 0.751d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.166d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.211d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.082d0
      force = ( 0.972d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.994d0
      force = ( 1.041d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 120.000d0
      force = ( 0.621d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.847d0
      force = ( 1.047d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.298d0
      force = ( 1.053d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 124.000d0
      force = ( 0.567d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.573d0
      force = ( 0.953d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.163d0
      force = ( 1.041d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 114.000d0
      force = ( 0.833d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.958d0
      force = ( 1.408d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 122.000d0
      force = ( 0.656d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.209d0
      force = ( 1.259d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.373d0
      force = ( 1.001d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 125.000d0
      force = ( 0.578d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.320d0
      force = ( 1.054d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 117.729d0
      force = ( 0.355d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.106d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.106d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.405d0
      force = ( 0.623d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 134.689d0
      force = ( 0.485d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 119.499d0
      force = ( 0.663d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.093d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.080d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 118.000d0
      force = ( 0.632d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.880d0
      force = ( 0.684d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.816d0
      force = ( 1.110d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.801d0
      force = ( 0.911d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '41'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.366d0
      force = ( 1.054d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '57'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.465d0
      force = ( 1.026d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '57'
      bnlocal(1,2) = '55'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-56-* MMFF94 DEF 
      bclocal(0) = 119.100d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 123.585d0
      force = ( 0.472d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.211d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.040d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 119.267d0
      force = ( 0.774d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.026d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.386d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.311d0
      force = ( 1.181d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.037d0
      force = ( 0.582d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.607d0
      force = ( 1.029d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.521d0
      force = ( 0.585d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.567d0
      force = ( 0.885d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.009d0
      force = ( 0.785d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.357d0
      force = ( 1.288d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 8'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.258d0
      force = ( 0.683d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.661d0
      force = ( 1.186d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 117.534d0
      force = ( 0.450d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.101d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.101d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '36'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 120.000d0
      force = ( 0.602d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 120.649d0
      force = ( 0.646d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.108d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.068d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.766d0
      force = ( 0.579d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 120.000d0
      force = ( 0.625d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.912d0
      force = ( 1.058d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.915d0
      force = ( 1.019d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '57'
      bnlocal(1,2) = '56'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-57-* MMFF94 DEF 
      bclocal(0) = 120.900d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '57'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:1:*-57-* MMFF94 DEF 
      bclocal(0) = 118.100d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '57'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.865d0
      force = ( 1.017d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '57'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.034d0
      force = ( 1.085d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '57'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 116.747d0
      force = ( 0.674d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.043d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.420d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '57'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.257d0
      force = ( 1.279d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '57'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.143d0
      force = ( 0.980d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '57'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.327d0
      force = ( 1.058d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '57'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.646d0
      force = ( 0.983d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '57'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.889d0
      force = ( 0.790d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = '57'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.379d0
      force = ( 0.967d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '57'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 126.476d0
      force = ( 0.855d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.125d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.125d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '55'
      bnlocal(1,2) = '57'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.800d0
      force = ( 1.016d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '55'
      bnlocal(1,2) = '57'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.166d0
      force = ( 1.039d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '55'
      bnlocal(1,2) = '57'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 120.010d0
      force = ( 1.342d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.431d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.431d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '56'
      bnlocal(1,2) = '57'
      bnlocal(1,3) = '56'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-58-* MMFF94 DEF 
      bclocal(0) = 119.000d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '58'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:1:*-58-* MMFF94 DEF 
      bclocal(0) = 119.900d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '58'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.236d0
      force = ( 1.003d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '58'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.070d0
      force = ( 0.961d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '58'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.506d0
      force = ( 0.983d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '58'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.370d0
      force = ( 1.371d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '58'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.665d0
      force = ( 1.005d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '58'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.713d0
      force = ( 0.650d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '58'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 118.000d0
      force = ( 0.650d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '58'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.051d0
      force = ( 0.620d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '58'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.710d0
      force = ( 0.996d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '58'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.260d0
      force = ( 1.036d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '58'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.989d0
      force = ( 1.087d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '58'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.942d0
      force = ( 1.061d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '58'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-59-* MMFF94 DEF 
      bclocal(0) = 105.600d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '59'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 106.313d0
      force = ( 1.273d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.497d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.497d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '59'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 107.755d0
      force = ( 1.750d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.723d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.874d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '59'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.179d0
      force = ( 1.713d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '59'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.341d0
      force = ( 1.599d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '59'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.683d0
      force = ( 1.754d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '65'
      bnlocal(1,2) = '59'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.142d0
      force = ( 1.644d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '65'
      bnlocal(1,2) = '59'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.624d0
      force = ( 1.864d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '65'
      bnlocal(1,2) = '59'
      bnlocal(1,3) = '82'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-61-* MMFF94 DEF 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.000d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '61'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.475d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '61'
      bnlocal(1,3) = '60'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.536d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '61'
      bnlocal(1,3) = '42'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style =  2
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 180.000d0
      bclocal(1) = 143.9325d0*(CON_KCALTOK)* 0.484d0
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '61'
      bnlocal(1,3) = '60'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-62-* MMFF94 DEF 
      bclocal(0) = 108.300d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '62'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 109.273d0
      force = ( 1.316d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '62'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 105.542d0
      force = ( 0.817d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '62'
      bnlocal(1,3) = '23'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.821d0
      force = ( 1.318d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '62'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.144d0
      force = ( 1.311d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '62'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.414d0
      force = ( 1.528d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '62'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.660d0
      force = ( 1.515d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '62'
      bnlocal(1,3) = '18'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.618d0
      force = ( 1.229d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '62'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.722d0
      force = ( 1.366d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '62'
      bnlocal(1,3) = '41'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.284d0
      force = ( 1.427d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '62'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.366d0
      force = ( 1.317d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '62'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-63-* MMFF94 DEF 
      bclocal(0) = 123.300d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:1:*-63-* MMFF94 DEF 
      bclocal(0) = 124.300d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.832d0
      force = ( 0.935d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.101d0
      force = ( 0.902d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '44'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.253d0
      force = ( 1.175d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '59'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 131.378d0
      force = ( 0.737d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.610d0
      force = ( 0.865d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.864d0
      force = ( 1.027d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.524d0
      force = ( 0.987d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '59'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 133.818d0
      force = ( 0.730d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 132.383d0
      force = ( 0.828d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.395d0
      force = ( 0.900d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.481d0
      force = ( 0.935d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '44'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.219d0
      force = ( 1.158d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '59'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 130.065d0
      force = ( 0.766d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.049d0
      force = ( 0.950d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.602d0
      force = ( 0.848d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '44'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.804d0
      force = ( 1.211d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '59'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.817d0
      force = ( 0.795d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 121.127d0
      force = ( 0.617d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.009d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.654d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 126.141d0
      force = ( 0.393d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.015d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.446d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '44'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 114.076d0
      force = ( 0.784d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.067d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.588d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '59'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 131.721d0
      force = ( 0.577d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.055d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.370d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 125.134d0
      force = ( 0.643d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.110d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.464d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 130.000d0
      force = ( 0.482d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 124.000d0
      force = ( 0.588d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.509d0
      force = ( 1.234d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.514d0
      force = ( 1.564d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '59'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 131.301d0
      force = ( 0.951d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.741d0
      force = ( 1.068d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.598d0
      force = ( 0.963d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '44'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 134.237d0
      force = ( 0.804d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 133.020d0
      force = ( 0.912d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.356d0
      force = ( 1.084d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.732d0
      force = ( 1.112d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '44'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.218d0
      force = ( 1.307d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '59'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.750d0
      force = ( 0.867d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.617d0
      force = ( 0.981d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.439d0
      force = ( 1.111d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.321d0
      force = ( 1.035d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '44'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.226d0
      force = ( 0.838d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.280d0
      force = ( 0.980d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.958d0
      force = ( 1.064d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.654d0
      force = ( 0.952d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '44'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 129.284d0
      force = ( 0.813d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.490d0
      force = ( 0.962d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.077d0
      force = ( 1.110d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '44'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 135.028d0
      force = ( 0.740d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 132.369d0
      force = ( 0.647d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '19'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 141.986d0
      force = ( 0.517d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '19'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 139.439d0
      force = ( 0.597d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.699d0
      force = ( 0.776d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '25'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.475d0
      force = ( 1.351d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '35'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '59'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 145.098d0
      force = ( 0.808d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '35'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 132.046d0
      force = ( 1.011d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.178d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.523d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.481d0
      force = ( 0.934d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 133.930d0
      force = ( 0.764d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '44'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.637d0
      force = ( 0.915d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '44'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.836d0
      force = ( 1.041d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '59'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.211d0
      force = ( 1.214d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '59'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 122.881d0
      force = ( 0.679d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.045d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.497d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 131.784d0
      force = ( 0.742d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 140.668d0
      force = ( 0.742d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.130d0
      force = ( 0.871d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.814d0
      force = ( 1.022d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.513d0
      force = ( 0.910d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 131.461d0
      force = ( 0.910d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.174d0
      force = ( 1.105d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '39'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.261d0
      force = ( 1.112d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '40'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.126d0
      force = ( 1.144d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '44'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.115d0
      force = ( 1.166d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.222d0
      force = ( 0.931d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '57'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.231d0
      force = ( 1.042d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '58'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.353d0
      force = ( 0.949d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 107.255d0
      force = ( 0.813d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.422d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.409d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.441d0
      force = ( 0.943d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 110.865d0
      force = ( 1.012d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.436d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.525d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.834d0
      force = ( 1.095d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.881d0
      force = ( 0.943d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '44'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.078d0
      force = ( 1.298d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '59'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 130.865d0
      force = ( 0.845d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 130.926d0
      force = ( 0.940d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.633d0
      force = ( 1.125d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '45'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.178d0
      force = ( 1.030d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '56'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.899d0
      force = ( 0.991d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '62'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.341d0
      force = ( 0.894d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.480d0
      force = ( 0.853d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.581d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.426d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 114.516d0
      force = ( 0.854d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.542d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.365d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 129.129d0
      force = ( 0.915d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.254d0
      force = ( 1.217d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.400d0
      force = ( 1.278d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.824d0
      force = ( 1.467d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '59'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.725d0
      force = ( 0.940d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.157d0
      force = ( 1.164d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 134.888d0
      force = ( 0.875d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '56'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.246d0
      force = ( 0.945d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '57'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.522d0
      force = ( 0.965d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '58'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 110.108d0
      force = ( 1.035d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.852d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.332d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '59'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 115.592d0
      force = ( 1.181d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.775d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.300d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '59'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.662d0
      force = ( 0.976d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '62'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 129.499d0
      force = ( 0.776d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.689d0
      force = ( 0.929d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 129.610d0
      force = ( 0.911d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '66'
      bnlocal(1,2) = '63'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-64-* MMFF94 DEF 
      bclocal(0) = 121.400d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:1:*-64-* MMFF94 DEF 
      bclocal(0) = 121.700d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.041d0
      force = ( 0.776d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.061d0
      force = ( 0.766d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.640d0
      force = ( 0.963d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.685d0
      force = ( 0.952d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.735d0
      force = ( 1.050d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.414d0
      force = ( 1.013d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '82'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.947d0
      force = ( 0.861d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.433d0
      force = ( 0.816d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.781d0
      force = ( 0.907d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.540d0
      force = ( 1.010d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.473d0
      force = ( 0.923d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '82'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.890d0
      force = ( 0.828d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.286d0
      force = ( 0.774d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.954d0
      force = ( 0.973d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.821d0
      force = ( 0.949d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.754d0
      force = ( 0.995d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.889d0
      force = ( 0.845d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.131d0
      force = ( 0.804d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.401d0
      force = ( 1.036d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.254d0
      force = ( 1.010d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 4'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 126.170d0
      force = ( 0.501d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.086d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.345d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 127.405d0
      force = ( 0.546d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.085d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.369d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 118.412d0
      force = ( 0.664d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.051d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.436d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 120.478d0
      force = ( 0.699d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.113d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.452d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.331d0
      force = ( 0.482d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 120.000d0
      force = ( 0.605d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 122.000d0
      force = ( 0.597d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '82'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.985d0
      force = ( 1.112d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.922d0
      force = ( 1.043d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.506d0
      force = ( 1.348d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.890d0
      force = ( 1.156d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 6'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.924d0
      force = ( 0.959d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.529d0
      force = ( 1.098d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.743d0
      force = ( 1.013d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.695d0
      force = ( 0.937d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.735d0
      force = ( 0.893d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.788d0
      force = ( 1.016d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.125d0
      force = ( 1.065d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '10'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.259d0
      force = ( 0.845d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.058d0
      force = ( 0.869d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.198d0
      force = ( 1.020d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.900d0
      force = ( 0.971d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '12'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.004d0
      force = ( 0.845d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '13'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.111d0
      force = ( 0.883d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '13'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.581d0
      force = ( 0.870d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.309d0
      force = ( 0.882d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.049d0
      force = ( 1.008d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.826d0
      force = ( 0.990d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '15'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.404d0
      force = ( 1.065d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.002d0
      force = ( 1.067d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 117.966d0
      force = ( 0.906d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.059d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.299d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 136.087d0
      force = ( 0.854d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.277d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.377d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.673d0
      force = ( 0.772d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 134.844d0
      force = ( 0.799d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.866d0
      force = ( 0.942d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 130.337d0
      force = ( 0.845d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 135.432d0
      force = ( 0.706d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.856d0
      force = ( 0.917d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.684d0
      force = ( 0.946d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '82'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.086d0
      force = ( 1.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '82'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.242d0
      force = ( 0.988d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 129.014d0
      force = ( 0.858d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.335d0
      force = ( 0.989d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.454d0
      force = ( 1.022d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.312d0
      force = ( 1.086d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.481d0
      force = ( 1.060d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.188d0
      force = ( 1.204d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.157d0
      force = ( 1.170d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.538d0
      force = ( 0.948d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.853d0
      force = ( 0.928d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 129.125d0
      force = ( 0.958d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.154d0
      force = ( 1.035d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.934d0
      force = ( 1.183d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '82'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.749d0
      force = ( 0.885d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '43'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.876d0
      force = ( 0.898d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '43'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.706d0
      force = ( 1.024d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '43'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.409d0
      force = ( 1.017d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '43'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.063d0
      force = ( 0.981d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.014d0
      force = ( 0.921d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.521d0
      force = ( 1.276d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.371d0
      force = ( 1.199d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.405d0
      force = ( 0.907d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '55'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.220d0
      force = ( 1.002d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '55'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.950d0
      force = ( 1.020d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '57'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.017d0
      force = ( 0.959d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '57'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.646d0
      force = ( 1.075d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '58'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 131.812d0
      force = ( 0.815d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '58'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.562d0
      force = ( 0.978d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '58'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.560d0
      force = ( 0.885d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '62'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.703d0
      force = ( 1.073d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '62'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.239d0
      force = ( 0.866d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.206d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.030d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.584d0
      force = ( 0.827d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.621d0
      force = ( 1.038d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.171d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.078d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.176d0
      force = ( 1.172d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.895d0
      force = ( 1.164d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.902d0
      force = ( 1.395d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '82'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.037d0
      force = ( 0.967d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '64'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 113.570d0
      force = ( 0.916d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.079d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.403d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '64'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.067d0
      force = ( 1.003d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '64'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 103.479d0
      force = ( 1.194d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '64'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 108.553d0
      force = ( 1.210d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '64'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '82'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 115.369d0
      force = ( 1.055d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.406d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.066d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '65'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.240d0
      force = ( 1.168d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '65'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 129.624d0
      force = ( 0.932d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '66'
      bnlocal(1,2) = '64'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-65-* MMFF94 DEF 
      bclocal(0) = 104.500d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '65'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 101.550d0
      force = ( 1.738d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.528d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.644d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '65'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 106.360d0
      force = ( 1.589d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.397d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.258d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '65'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 101.208d0
      force = ( 1.740d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '65'
      bnlocal(1,3) = '82'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 103.829d0
      force = ( 1.430d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.816d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.543d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '65'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.552d0
      force = ( 1.366d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '65'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.213d0
      force = ( 1.419d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '65'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 103.452d0
      force = ( 1.788d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 1.177d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.594d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '59'
      bnlocal(1,2) = '65'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.872d0
      force = ( 1.774d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '59'
      bnlocal(1,2) = '65'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-66-* MMFF94 DEF 
      bclocal(0) = 106.900d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '66'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 103.779d0
      force = ( 1.206d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.213d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.173d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '66'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 106.735d0
      force = ( 1.406d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.234d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.077d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '66'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.365d0
      force = ( 1.339d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '66'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 106.806d0
      force = ( 1.408d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '66'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 107.658d0
      force = ( 1.709d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.149d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.383d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '64'
      bnlocal(1,2) = '66'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 111.306d0
      force = ( 1.932d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.199d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.101d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '65'
      bnlocal(1,2) = '66'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-67-* MMFF94 DEF 
      bclocal(0) = 119.900d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:1:*-67-* MMFF94 DEF 
      bclocal(0) = 116.600d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.683d0
      force = ( 0.982d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.581d0
      force = ( 1.178d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = ' 9'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.589d0
      force = ( 1.233d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.574d0
      force = ( 1.257d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.320d0
      force = ( 1.118d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.438d0
      force = ( 1.231d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 128.000d0
      force = ( 0.567d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = '23'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.945d0
      force = ( 1.290d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.631d0
      force = ( 1.122d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.899d0
      force = ( 1.142d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = '30'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.531d0
      force = ( 1.325d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 115.979d0
      force = ( 1.186d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 120.000d0
      force = ( 0.805d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '23'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.854d0
      force = ( 1.370d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '30'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.019d0
      force = ( 1.240d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.327d0
      force = ( 1.504d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.017d0
      force = ( 1.310d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '67'
      bnlocal(1,3) = '67'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-68-* MMFF94 DEF 
      bclocal(0) = 108.800d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '68'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.238d0
      force = ( 1.159d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.217d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.217d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '68'
      bnlocal(1,3) = ' 1'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 107.200d0
      force = ( 0.772d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.285d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.050d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '68'
      bnlocal(1,3) = '23'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 110.757d0
      force = ( 0.958d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.047d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.503d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '68'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 104.892d0
      force = ( 0.650d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.145d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.145d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '23'
      bnlocal(1,2) = '68'
      bnlocal(1,3) = '23'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 112.977d0
      force = ( 0.659d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.182d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.504d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '23'
      bnlocal(1,2) = '68'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-69-* MMFF94 DEF 
      bclocal(0) = 120.300d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '69'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 121.777d0
      force = ( 1.123d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 1.018d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.418d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '69'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.217d0
      force = ( 1.486d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '69'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 116.447d0
      force = ( 1.223d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.169d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*(-0.169d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '69'
      bnlocal(1,3) = '37'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.930d0
      force = ( 1.122d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '69'
      bnlocal(1,3) = '38'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 103.978d0
      force = ( 0.658d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.210d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.210d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '31'
      bnlocal(1,2) = '70'
      bnlocal(1,3) = '31'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-73-* MMFF94 DEF 
      bclocal(0) = 106.600d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '73'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 100.180d0
      force = ( 1.590d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '73'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) =  96.166d0
      force = ( 1.481d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '73'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 115.012d0
      force = ( 1.665d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '73'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 115.134d0
      force = ( 1.326d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '73'
      bnlocal(1,3) = '72'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-74-* MMFF94 DEF 
      bclocal(0) = 113.000d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '74'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 113.010d0
      force = ( 1.357d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '74'
      bnlocal(1,3) = ' 7'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-75-* MMFF94 DEF 
      bclocal(0) =  94.900d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '75'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  96.779d0
      force = ( 1.138d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '75'
      bnlocal(1,3) = ' 3'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  91.970d0
      force = ( 1.044d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '75'
      bnlocal(1,3) = '19'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) =  95.899d0
      force = ( 0.729d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '75'
      bnlocal(1,3) = '71'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-76-* MMFF94 DEF 
      bclocal(0) = 107.600d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '76'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 109.889d0
      force = ( 1.434d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '76'
      bnlocal(1,2) = '76'
      bnlocal(1,3) = '76'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 103.519d0
      force = ( 1.493d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '76'
      bnlocal(1,2) = '76'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.421d0
      force = ( 1.235d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '78'
      bnlocal(1,2) = '76'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-77-* MMFF94 DEF 
      bclocal(0) = 109.500d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '77'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 109.472d0
      force = ( 1.652d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '77'
      bnlocal(1,3) = '32'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-78-* MMFF94 DEF 
      bclocal(0) = 121.900d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:1:*-78-* MMFF94 DEF 
      bclocal(0) = 126.100d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 130.960d0
      force = ( 0.744d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.477d0
      force = ( 0.938d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.468d0
      force = ( 0.827d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.748d0
      force = ( 0.922d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 123.407d0
      force = ( 0.584d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '76'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 128.000d0
      force = ( 0.546d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.279d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.250d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:#E94                
      bclocal(0) = 122.000d0
      force = ( 0.617d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '79'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 109.881d0
      force = ( 0.542d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.083d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.250d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 129.501d0
      force = ( 0.863d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.857d0
      force = ( 0.991d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 137.282d0
      force = ( 0.770d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '76'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.249d0
      force = ( 0.803d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.714d0
      force = ( 0.864d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 130.617d0
      force = ( 0.844d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.532d0
      force = ( 1.023d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '38'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 138.714d0
      force = ( 0.734d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 109.426d0
      force = ( 1.202d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '39'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 130.150d0
      force = ( 0.930d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '76'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 135.746d0
      force = ( 0.778d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 121.251d0
      force = ( 1.058d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '40'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 141.902d0
      force = ( 0.677d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 142.589d0
      force = ( 0.663d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 134.701d0
      force = ( 0.816d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.702d0
      force = ( 1.089d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.467d0
      force = ( 1.199d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '76'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.050d0
      force = ( 0.915d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.926d0
      force = ( 1.216d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '45'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.471d0
      force = ( 0.963d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '59'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.375d0
      force = ( 1.097d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '59'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.916d0
      force = ( 1.443d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '59'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 117.779d0
      force = ( 0.942d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 131.530d0
      force = ( 0.835d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '64'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.834d0
      force = ( 1.038d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '64'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 118.376d0
      force = ( 1.030d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '66'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:X94                 
      bclocal(0) = 113.179d0
      force = ( 1.245d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '76'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '76'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 111.900d0
      force = ( 1.159d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '76'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) =  99.459d0
      force = ( 1.336d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '78'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 105.130d0
      force = ( 1.302d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.398d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.314d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '78'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.792d0
      force = ( 1.217d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '79'
      bnlocal(1,2) = '78'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-79-* MMFF94 DEF 
      bclocal(0) = 103.400d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '79'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 102.043d0
      force = ( 1.569d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '78'
      bnlocal(1,2) = '79'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 104.857d0
      force = ( 1.625d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '79'
      bnlocal(1,2) = '79'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-80-* MMFF94 DEF 
      bclocal(0) = 121.900d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '80'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:1:*-80-* MMFF94 DEF 
      bclocal(0) = 128.200d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '80'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.147d0
      force = ( 0.864d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '80'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.181d0
      force = ( 0.886d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 3'
      bnlocal(1,2) = '80'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 125.682d0
      force = ( 0.651d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*(-0.101d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.691d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 5'
      bnlocal(1,2) = '80'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.869d0
      force = ( 1.032d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '18'
      bnlocal(1,2) = '80'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.057d0
      force = ( 0.909d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '41'
      bnlocal(1,2) = '80'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.755d0
      force = ( 0.918d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '80'
      bnlocal(1,3) = '55'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.411d0
      force = ( 1.184d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '44'
      bnlocal(1,2) = '80'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.263d0
      force = ( 1.254d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '55'
      bnlocal(1,2) = '80'
      bnlocal(1,3) = '59'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 127.612d0
      force = ( 0.991d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '55'
      bnlocal(1,2) = '80'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.038d0
      force = ( 1.003d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '56'
      bnlocal(1,2) = '80'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.030d0
      force = ( 1.439d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '59'
      bnlocal(1,2) = '80'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 108.609d0
      force = ( 1.205d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.732d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.732d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '81'
      bnlocal(1,2) = '80'
      bnlocal(1,3) = '81'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:0:*-81-* MMFF94 DEF 
      bclocal(0) = 119.500d0
      force = ( 0.000d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 0'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = ' 0'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.129d0
      force = ( 0.996d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '63'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.970d0
      force = ( 0.978d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.535d0
      force = ( 0.879d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 116.113d0
      force = ( 1.144d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '79'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.324d0
      force = ( 0.895d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 1'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 125.080d0
      force = ( 0.927d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.399d0
      force = ( 0.895d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 2'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 124.270d0
      force = ( 1.015d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 120.028d0
      force = ( 1.106d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = ' 9'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 130.295d0
      force = ( 0.522d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 128.738d0
      force = ( 0.583d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '66'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 124.658d0
      force = ( 0.578d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.021d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.368d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 124.787d0
      force = ( 0.575d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.018d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.422d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '36'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.408d0
      force = ( 0.929d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 119.722d0
      force = ( 0.983d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.158d0
      force = ( 1.184d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 110.477d0
      force = ( 1.281d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 126.208d0
      force = ( 0.884d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '78'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 123.333d0
      force = ( 0.940d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:1 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '37'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.945d0
      force = ( 1.115d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '63'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.099d0
      force = ( 1.075d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '64'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 113.176d0
      force = ( 1.143d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '64'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 122.250d0
      force = ( 1.067d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '66'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:C94                 
      bclocal(0) = 110.556d0
      force = ( 0.957d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.true.)
c     --- cross term origin:C94                 
      bclocal(3) = 2.51210d0*(CON_KCALTOK)*( 0.366d0)
      bclocal(4) = 2.51210d0*(CON_KCALTOK)*( 0.419d0)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb: 0 a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '78'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 107.936d0
      force = ( 1.379d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '79'
      bnlocal(1,2) = '81'
      bnlocal(1,3) = '80'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 114.660d0
      force = ( 1.666d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '82'
      bnlocal(1,3) = '59'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 131.706d0
      force = ( 1.075d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '82'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 129.293d0
      force = ( 1.238d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '32'
      bnlocal(1,2) = '82'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 105.660d0
      force = ( 1.563d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '59'
      bnlocal(1,2) = '82'
      bnlocal(1,3) = '64'
      call twh_set_bendnames(type,1,bnlocal)

      type = type + 1
      style = 15
      call twh_anglestyle(GLB_SET,type,style)
c     --- origin:E94                 
      bclocal(0) = 112.955d0
      force = ( 1.281d0)*0.043844d0*0.5d0*(CON_KCALTOK)*radtodegsq
      bclocal(1) = force
      bclocal(2) = force*cb
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      orderang = 'MMFF sb:** a:0 '
      call twh_angleorder(GLB_SET,type,orderang)
      call twh_bendff(GLB_SET,type,ffname)
      bnlocal(1,1) = '64'
      bnlocal(1,2) = '82'
      bnlocal(1,3) = '65'
      call twh_set_bendnames(type,1,bnlocal)



      nangletypes = type
      call twh_ffcheck(lfailure,2,type)
      if ( lfailure ) return

      return
      end

      subroutine twh_ffmmff94_dihedral(lfailure,ffname,ndihedtypes)
c     ******************************************************************
c     * sets up the MMFF94 dihedral parameters                         *
c     *                                                                *
c     * originally written 12-13-2006 by M.G. Martin                   *
c     * last modified 08-04-2011 by M.G. Martin                        *
c     ******************************************************************
      implicit none
c     --- variables passed to/from the subroutine
      character*(*) ffname
      logical lfailure
      integer ndihedtypes
c     --- local variables
      integer type
      character*15 ordertor
      character*(FFNAMELEN) tnlocal
      dimension tnlocal(MAXSAMET,MAXTNAME)
      double precision tclocal
      dimension tclocal(MINTORCOEFF:MAXTORCOEFF)
      double precision onepi

      call twh_constant_pi(GLB_GET,onepi)

c     --- initialize type count
      type = 0

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.300d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.200d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.500d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.103d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.681d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.332d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.144d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.547d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.126d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.295d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.438d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.584d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.066d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.156d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.143d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.639d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.630d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.264d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.688d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.757d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.477d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.054d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.420d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.092d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.101d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 8'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.158d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.323d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 8'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.593d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.662d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.120d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '11'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.678d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.417d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.624d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '12'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.714d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.698d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '15'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.647d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.550d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.590d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '34'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.321d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.411d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.144d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.443d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-1.140d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 3'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.256d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.058d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 3'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.679d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.029d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 3'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.284d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-1.386d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.314d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.654d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.072d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.279d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.744d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-1.235d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.337d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 8'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.427d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '10'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.516d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.291d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '11'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.678d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.602d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.398d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '12'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.142d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.644d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.367d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '15'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.295d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '25'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.692d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.530d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.278d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '34'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.389d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.278d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '39'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.141d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '41'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.324d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '56'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.136d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '68'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.408d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.397d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.961d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 6'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.313d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-1.035d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.631d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 6'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.055d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.834d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 8'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = ' 8'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.387d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.543d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.405d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '11'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '11'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.893d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '12'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '12'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.177d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.049d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '15'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 1'
      tnlocal(1,4) = '15'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.650d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.650d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.419d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.296d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.282d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.494d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.274d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.630d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.075d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.358d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.293d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.115d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.508d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.301d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.104d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.507d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.565d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.554d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.234d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 3'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.577d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.482d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.427d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 3'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.082d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.123d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 3'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.184d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.220d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.501d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.410d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.535d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.055d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.108d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.523d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.228d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.208d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.467d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.490d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 6'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.425d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.168d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.875d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 6'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.136d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.396d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 6'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.504d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.371d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.557d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 8'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.541d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.539d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-1.009d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 8'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.204d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.464d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 8'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.400d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.300d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.500d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.350d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.550d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.200d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.700d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.400d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.400d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 7'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.103d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.177d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.545d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.072d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.316d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.674d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.117d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.333d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.202d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.825d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.139d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.325d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 7'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.927d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.112d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.388d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = '10'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.663d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.167d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.426d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.758d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.112d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.563d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 7'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.073d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.085d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.531d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.115d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.446d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.822d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.501d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.008d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.624d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.330d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.659d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-1.407d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.308d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 7'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.412d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.693d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.087d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = '10'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.056d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.027d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.360d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = '43'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.543d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.350d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = '51'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.501d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = '53'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.513d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.344d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = '74'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.511d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.186d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = '75'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.447d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.652d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.318d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 6'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.395d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.730d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.139d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 6'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 7'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.338d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.772d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 2.145d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '10'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 7'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.548d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.795d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '10'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = '10'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 4'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.200d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.200d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.400d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.681d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.755d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.755d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.243d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.596d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.547d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.320d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.270d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.237d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '21'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.102d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.460d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.128d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '21'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.652d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-1.660d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.283d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 3'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '21'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.571d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.319d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.570d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.306d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.572d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.304d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.596d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.276d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.346d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '21'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.061d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '25'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.106d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.174d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '45'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.229d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.710d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.722d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 6'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.040d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 6'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.488d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-3.401d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.320d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 6'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '21'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.712d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.320d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.507d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '37'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '21'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.300d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.500d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 8'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.297d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 8'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.439d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.786d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.272d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 8'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.115d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.390d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.658d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 8'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.608d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.339d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.496d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 8'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.428d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.323d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.280d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 8'
      tnlocal(1,4) = '23'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.594d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.409d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.155d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 8'
      tnlocal(1,4) = '23'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.393d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.385d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.562d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 8'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.598d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.158d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.399d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 8'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.152d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.440d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.357d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 8'
      tnlocal(1,4) = '23'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.204d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.335d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.352d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.097d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = '53'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.300d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.027d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.694d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.948d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.159d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.552d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.198d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.552d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.380d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.326d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 3.100d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-2.529d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.494d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 3'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.079d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.280d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.402d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 3'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.779d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-2.099d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.363d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.021d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.162d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.832d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.552d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.616d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.274d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.400d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.336d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.047d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.170d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.398d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.438d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.263d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.501d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = '15'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.376d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.133d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.288d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = '71'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.143d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.231d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.447d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.555d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.323d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.456d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = '15'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.459d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.229d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.203d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.440d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = '71'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.350d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '17'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '17'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.536d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '17'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.212d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '17'
      tnlocal(1,4) = ' 7'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.100d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.112d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.099d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.585d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.388d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.412d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.121d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = '43'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.195d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = '48'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.088d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = '62'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.150d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '19'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.179d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '19'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.196d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '19'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.176d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '19'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.152d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '19'
      tnlocal(1,4) = '12'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.350d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.350d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.344d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.361d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = '20'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.236d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.236d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.300d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.251d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.207d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.232d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.288d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.218d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.152d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.235d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.495d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.130d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.214d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.466d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = '43'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.243d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = '72'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.450d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '26'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.376d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '26'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.439d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '26'
      tnlocal(1,4) = '12'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.472d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '26'
      tnlocal(1,4) = '71'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.250d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '34'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.198d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '34'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.187d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '34'
      tnlocal(1,4) = '36'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.247d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '34'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.259d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '34'
      tnlocal(1,4) = '36'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.200d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.449d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.420d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.391d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.150d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 6'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '39'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '39'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.080d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.056d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '39'
      tnlocal(1,4) = '63'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.113d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '39'
      tnlocal(1,4) = '63'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.250d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.297d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.097d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.203d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.329d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.263d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.106d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.632d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = '72'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.150d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.297d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.357d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.918d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '18'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.061d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '25'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.249d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.382d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.343d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.100d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.125d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.500d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '46'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.540d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '46'
      tnlocal(1,4) = ' 7'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '54'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '54'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '54'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.315d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '54'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.315d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '54'
      tnlocal(1,4) = '36'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '55'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '55'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.058d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.084d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '55'
      tnlocal(1,4) = '36'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.058d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.092d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '55'
      tnlocal(1,4) = '57'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.300d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '56'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.875d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.668d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.015d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '56'
      tnlocal(1,4) = '36'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.870d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.775d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.406d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '56'
      tnlocal(1,4) = '57'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.958d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.629d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.372d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '56'
      tnlocal(1,4) = '36'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.952d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.715d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.483d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '56'
      tnlocal(1,4) = '57'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '58'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.250d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '62'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.270d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '62'
      tnlocal(1,4) = '18'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '67'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '67'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.400d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '68'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.117d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.090d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.751d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '68'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.373d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.153d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.635d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '68'
      tnlocal(1,4) = '23'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.090d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.169d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.075d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '68'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.134d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.112d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.329d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '68'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.361d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.202d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.560d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '68'
      tnlocal(1,4) = '23'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.072d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.218d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.093d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '68'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.500d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '73'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.509d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '73'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.443d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '73'
      tnlocal(1,4) = '72'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '75'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '78'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '80'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 1'
      tnlocal(1,3) = '81'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.403d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.418d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.089d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.310d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.412d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.120d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.269d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.094d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.621d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.877d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.317d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.421d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.870d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 3'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.406d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.767d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = '40'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = '41'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = '45'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 2'
      tnlocal(1,4) = '62'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.500d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.136d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.798d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.630d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.497d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.405d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.357d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.211d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.925d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.131d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.401d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.028d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.318d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 7'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.084d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.214d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.610d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = '10'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.325d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.553d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.487d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.295d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.024d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.590d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.143d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.466d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.362d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.978d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 7'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.296d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.514d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.481d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 9'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.095d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.583d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.380d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = '10'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.213d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.728d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.042d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.208d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.622d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.223d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.359d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.539d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.194d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.046d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 7'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.290d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.519d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.470d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 9'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.395d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.227d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = '10'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 4'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.100d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.953d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.953d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-1.055d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.712d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.596d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.330d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.215d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.810d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.456d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '29'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.951d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.936d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.130d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.719d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.628d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.360d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.216d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.808d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.456d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '29'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '17'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '19'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '30'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '34'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '34'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.952d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.079d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.542d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.434d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.308d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.357d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '39'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.700d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.756d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.530d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.073d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.698d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.291d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.200d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.235d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.231d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.200d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.212d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.225d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '46'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '46'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '55'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '56'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 8.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '62'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.693d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.903d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.532d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '62'
      tnlocal(1,4) = '23'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.696d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.897d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.482d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '62'
      tnlocal(1,4) = '23'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '67'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 2'
      tnlocal(1,3) = '81'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.486d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.714d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.081d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.125d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.132d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.053d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.327d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 7'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.188d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.436d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.177d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.412d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 7'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.269d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.437d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 6'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.495d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.793d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.318d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 6'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 7'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.260d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.084d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.193d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 3'
      tnlocal(1,4) = ' 7'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 5.500d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 5.500d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.244d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 5.482d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.365d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.166d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 5.078d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.545d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '24'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.677d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 5.854d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.521d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.256d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.519d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.258d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '24'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.663d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.073d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.094d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 3'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '24'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.526d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 5.631d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.691d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.159d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.586d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.216d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-2.285d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.737d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.468d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '24'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.700d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.500d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.400d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.682d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.184d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.935d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.168d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.572d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.151d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 2'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.662d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.152d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.058d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '24'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.635d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 5.890d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.446d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.892d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.094d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '37'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = '24'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (16.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (16.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = '27'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.687d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (16.152d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.894d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (16.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = '27'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.758d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (18.216d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.188d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '40'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (16.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '40'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = '27'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.647d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.159d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.507d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.035d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 8.791d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.464d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.294d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 5.805d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.342d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.287d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.142d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.120d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.183d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.314d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.753d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.751d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 5.348d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.209d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.388d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 5.972d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.459d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.319d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.294d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.147d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.776d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.585d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.145d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.107d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 8.631d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.452d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.435d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.975d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.454d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.495d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.291d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '10'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '17'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '17'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.300d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.300d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.400d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.400d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.131d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = '20'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.085d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '20'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '20'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = '20'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.400d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.400d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '30'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '30'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.500d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.428d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.743d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 6'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.256d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.241d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.385d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.838d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '43'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 5.500d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '39'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.900d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.477d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.362d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.902d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.496d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.369d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.417d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 9'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.178d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.149d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.778d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '40'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.500d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.712d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.309d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.233d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '18'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.414d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.168d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.875d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.880d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 5.091d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.129d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '18'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.536d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 5.276d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.556d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 7'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.701d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.871d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.225d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '37'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '18'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.086d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 5.073d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.878d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '37'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.892d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '48'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (13.500d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '51'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (13.549d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '51'
      tnlocal(1,4) = '52'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 8.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '54'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.500d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '54'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '54'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 8.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '54'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 8.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '54'
      tnlocal(1,4) = '36'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '55'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '55'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '56'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '56'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.500d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '58'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '62'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '62'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '62'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.500d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.500d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '67'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (19.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '74'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (19.349d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '74'
      tnlocal(1,4) = ' 7'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (19.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '75'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (18.751d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '75'
      tnlocal(1,4) = '71'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.500d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '78'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.500d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 3'
      tnlocal(1,3) = '80'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-2.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-2.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = ' 6'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.900d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-1.100d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.500d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = ' 8'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.274d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = ' 8'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.261d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.330d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.542d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '21'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = ' 8'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.503d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-1.853d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.476d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '21'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = ' 8'
      tnlocal(1,4) = '23'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.200d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.500d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-1.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.875d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.180d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.733d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '21'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.529d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-1.163d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '21'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-4.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '17'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '17'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.100d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.103d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.520d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.471d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.267d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '33'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.623d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.204d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.438d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '33'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.616d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.425d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.191d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '33'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.150d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '19'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.165d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '19'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.620d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.329d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.303d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '21'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '19'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.683d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.220d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '21'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '19'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.400d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.217d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.217d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.079d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '20'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '20'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = '20'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.217d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.650d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.231d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.704d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.452d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.556d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.777d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.205d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.914d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.612d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-3.209d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-7.622d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.065d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '24'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-5.891d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-3.332d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.290d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '24'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.346d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '26'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '30'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '30'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.200d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.382d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.576d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 3'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.801d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '29'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '39'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.274d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.274d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.208d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = '32'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '54'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '55'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '58'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 6'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.375d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = ' 8'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.375d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = ' 8'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.424d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '17'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '17'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '17'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.225d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '19'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.350d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.300d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.297d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.120d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.472d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '20'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.097d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.200d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '20'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = '20'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.101d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.324d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.371d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '23'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.107d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.253d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.151d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '23'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = '20'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.297d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.316d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.316d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.474d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '26'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.474d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '26'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.250d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '34'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '39'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.375d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.375d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '46'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '55'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 8'
      tnlocal(1,3) = '56'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = ' 9'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '19'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '34'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '34'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '39'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '54'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '55'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '56'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '62'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '67'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '78'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = ' 9'
      tnlocal(1,3) = '81'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '10'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.743d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '17'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '26'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '26'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '34'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '39'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '10'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.400d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-8.300d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-8.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.663d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-8.408d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.433d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.088d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-8.245d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.411d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '15'
      tnlocal(1,4) = '71'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.160d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '18'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.255d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '19'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.255d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '19'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.336d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.336d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.336d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.358d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.358d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.537d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '26'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '30'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '30'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.300d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.177d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.505d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.333d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '71'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.424d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.424d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '15'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '17'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '17'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '17'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '17'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '17'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.795d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '17'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.112d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.112d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.112d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.112d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-1.200d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.300d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.173d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.965d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.610d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '32'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.760d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.227d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '39'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.228d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-1.741d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.371d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '43'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.500d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '39'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.687d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.680d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '32'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '39'
      tnlocal(1,4) = '63'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.513d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.357d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '37'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '39'
      tnlocal(1,4) = '63'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.350d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.141d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.141d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.914d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.482d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.179d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.392d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-2.724d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.312d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.508d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-1.816d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.175d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.823d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-1.220d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.770d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.588d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.499d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.410d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '32'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.653d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.254d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '32'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 3'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.528d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.342d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '32'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.812d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.513d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.266d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '32'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.139d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.703d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.088d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '37'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-2.014d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-1.646d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-2.068d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '37'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.519d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.328d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.437d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '37'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 3.011d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-1.405d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 2.038d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '43'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.400d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '48'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 1.767d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.606d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.408d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '48'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-1.463d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-2.548d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.310d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '32'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '48'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '55'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '58'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.500d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '62'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.403d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.273d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.440d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '62'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.291d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.385d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.582d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '32'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '62'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '18'
      tnlocal(1,3) = '80'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.179d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '19'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.179d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '19'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '19'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.225d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '19'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '19'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '19'
      tnlocal(1,3) = '75'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.200d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.236d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.067d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.081d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.347d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.063d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.064d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.140d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = '20'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.083d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 3'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 3'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = '20'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.424d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.080d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.127d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.450d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = ' 8'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.072d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.269d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.439d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = '12'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.057d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.307d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = '20'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 6'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = '20'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.091d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.192d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 8'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = '20'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.077d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.202d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.183d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '12'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = '20'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '20'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '20'
      tnlocal(1,4) = '20'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.236d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.236d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.251d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.251d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.376d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '26'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.376d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '26'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.376d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '26'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '30'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '30'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '30'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.500d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '30'
      tnlocal(1,4) = '30'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.198d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '34'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.198d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '34'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.297d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.297d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.297d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '20'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.236d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '22'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.236d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '22'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.236d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '22'
      tnlocal(1,3) = '22'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '22'
      tnlocal(1,3) = '30'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '22'
      tnlocal(1,3) = '30'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.198d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '22'
      tnlocal(1,3) = '34'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '22'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.297d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '22'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '22'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.297d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '22'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.297d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '22'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '22'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.267d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '25'
      tnlocal(1,3) = '25'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '25'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '25'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '25'
      tnlocal(1,3) = '39'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.316d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '25'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.316d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '25'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.250d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '25'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-2.686d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-1.512d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.591d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '25'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-3.730d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (-0.531d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '25'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 2.108d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.896d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.965d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '32'
      tnlocal(1,2) = '25'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 2.977d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.732d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.502d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '32'
      tnlocal(1,2) = '25'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '25'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '25'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.600d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '26'
      tnlocal(1,3) = '26'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.600d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '26'
      tnlocal(1,3) = '26'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.316d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '26'
      tnlocal(1,3) = '34'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.316d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '26'
      tnlocal(1,3) = '34'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '26'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.474d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '26'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '30'
      tnlocal(1,3) = '30'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '30'
      tnlocal(1,3) = '30'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '30'
      tnlocal(1,3) = '30'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '30'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '30'
      tnlocal(1,3) = '67'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '34'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.250d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '34'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:4'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 2'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 3'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 3'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = ' 6'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '15'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '18'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '40'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '43'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 6'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '15'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '18'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '37'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '37'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '37'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '40'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '37'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '37'
      tnlocal(1,4) = '43'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '38'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '39'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '39'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.336d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.370d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '37'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 1'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.715d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.628d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 3.355d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '37'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.372d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.284d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 2.034d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '37'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '18'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.694d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 1.508d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '37'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = '28'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '46'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '55'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '56'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '58'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '58'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '62'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '67'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '69'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '78'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '81'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '37'
      tnlocal(1,3) = '81'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '38'
      tnlocal(1,3) = '38'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '38'
      tnlocal(1,3) = '58'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '38'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '38'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '38'
      tnlocal(1,3) = '69'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '38'
      tnlocal(1,3) = '78'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '39'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '39'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '39'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '39'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '39'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '39'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '39'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = '64'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '18'
      tnlocal(1,2) = '39'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '18'
      tnlocal(1,2) = '39'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = '64'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '63'
      tnlocal(1,2) = '39'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '63'
      tnlocal(1,2) = '39'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = '64'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '39'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '39'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '39'
      tnlocal(1,3) = '65'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '39'
      tnlocal(1,3) = '78'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.375d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '40'
      tnlocal(1,3) = '40'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '40'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '40'
      tnlocal(1,3) = '46'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '40'
      tnlocal(1,3) = '54'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '40'
      tnlocal(1,3) = '54'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '40'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '40'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '40'
      tnlocal(1,3) = '78'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '41'
      tnlocal(1,3) = '41'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '41'
      tnlocal(1,3) = '55'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '41'
      tnlocal(1,3) = '62'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '41'
      tnlocal(1,3) = '80'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.375d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '43'
      tnlocal(1,3) = '43'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '43'
      tnlocal(1,3) = '45'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '43'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '44'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '44'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '44'
      tnlocal(1,3) = '65'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.846d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '44'
      tnlocal(1,3) = '78'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 2.846d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '44'
      tnlocal(1,3) = '80'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '45'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '45'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '45'
      tnlocal(1,3) = '78'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (10.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '55'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:2'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '55'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '55'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.423d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.064d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.090d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '55'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.428d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.044d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '55'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = '55'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = (-0.268d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 8.077d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.806d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '36'
      tnlocal(1,2) = '55'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.273d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 8.025d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.692d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '36'
      tnlocal(1,2) = '55'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = '55'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '55'
      tnlocal(1,3) = '62'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '55'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '55'
      tnlocal(1,3) = '80'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '56'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.886d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = (-0.161d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 1'
      tnlocal(1,2) = '56'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = '56'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.688d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.107d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '36'
      tnlocal(1,2) = '56'
      tnlocal(1,3) = '57'
      tnlocal(1,4) = '56'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '56'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '56'
      tnlocal(1,3) = '80'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '57'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '57'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '58'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '58'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '59'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '59'
      tnlocal(1,3) = '65'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '59'
      tnlocal(1,3) = '78'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '59'
      tnlocal(1,3) = '80'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '59'
      tnlocal(1,3) = '82'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '62'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '62'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '63'
      tnlocal(1,3) = '63'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '63'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = '63'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = '63'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = '64'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '39'
      tnlocal(1,2) = '63'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '39'
      tnlocal(1,2) = '63'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = '64'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '63'
      tnlocal(1,3) = '66'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '63'
      tnlocal(1,3) = '78'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '63'
      tnlocal(1,3) = '81'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '64'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 1.800d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:1'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '64'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = '64'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = ' 5'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 5'
      tnlocal(1,2) = '64'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = '63'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = '63'
      tnlocal(1,2) = '64'
      tnlocal(1,3) = '64'
      tnlocal(1,4) = '63'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '64'
      tnlocal(1,3) = '65'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '64'
      tnlocal(1,3) = '66'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '64'
      tnlocal(1,3) = '78'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '64'
      tnlocal(1,3) = '81'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '64'
      tnlocal(1,3) = '81'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '64'
      tnlocal(1,3) = '82'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '65'
      tnlocal(1,3) = '66'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '65'
      tnlocal(1,3) = '78'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '65'
      tnlocal(1,3) = '81'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '65'
      tnlocal(1,3) = '82'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '66'
      tnlocal(1,3) = '66'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '66'
      tnlocal(1,3) = '78'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '66'
      tnlocal(1,3) = '81'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '67'
      tnlocal(1,3) = '67'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = (12.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:5'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '67'
      tnlocal(1,3) = '67'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '76'
      tnlocal(1,3) = '76'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 3.600d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '76'
      tnlocal(1,3) = '78'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 7.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '78'
      tnlocal(1,3) = '78'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '78'
      tnlocal(1,3) = '79'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '78'
      tnlocal(1,3) = '81'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '79'
      tnlocal(1,3) = '79'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 6.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '79'
      tnlocal(1,3) = '81'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      type = type + 1
      call twh_torstyle(GLB_SET,type,2)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 0.75d0
      tclocal(1) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(2) = ( 4.000d0)*(0.5d0)*(CON_KCALTOK)
      tclocal(3) = ( 0.000d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_torcoeff(type,tclocal)
      ordertor = 'MMFF t:0'
      call twh_torsionorder(GLB_SET,type,ordertor)
      call twh_torff(GLB_SET,type,ffname)
      tnlocal(1,1) = ' 0'
      tnlocal(1,2) = '80'
      tnlocal(1,3) = '81'
      tnlocal(1,4) = ' 0'
      call twh_set_tornames(type,1,tnlocal)

      ndihedtypes = type
      call twh_ffcheck(lfailure,3,type)
      if ( lfailure ) return

      return
      end

      subroutine twh_ffmmff94_improper(lfailure,ffname,nimprotypes)
c     ******************************************************************
c     * sets up the MMFF94 improper torsion parameters                 *
c     *                                                                *
c     * originally written 12-19-2006 by M.G. Martin                   *
c     * last modified 08-04-2011 by M.G. Martin                        *
c     ******************************************************************
      implicit none
c     --- variables passed to/from the subroutine
      character*(FFNAMELEN) ffname
      logical lfailure
      integer nimprotypes
c     --- local variables
      integer type
      double precision iclocal
      dimension iclocal(MINIMPCOEFF:MAXIMPCOEFF)
      character*(FFNAMELEN) ninlocal 
      dimension ninlocal(MAXSAMEIMP,MAXIMPNAME)
      double precision onepi

      call twh_constant_pi(GLB_GET,onepi)

c     --- initialize type count
      type = 0

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.020d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-2-*-* C94 DEF     
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = ' 2'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.030d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = ' 2'
      ninlocal(1,3) = ' 1'
      ninlocal(1,4) = ' 2'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.027d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = ' 2'
      ninlocal(1,3) = ' 2'
      ninlocal(1,4) = ' 2'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.026d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = ' 2'
      ninlocal(1,3) = ' 2'
      ninlocal(1,4) = ' 3'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.013d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = ' 2'
      ninlocal(1,3) = ' 2'
      ninlocal(1,4) = ' 5'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.032d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = ' 2'
      ninlocal(1,3) = ' 2'
      ninlocal(1,4) = '37'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.013d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 2'
      ninlocal(1,2) = ' 2'
      ninlocal(1,3) = ' 2'
      ninlocal(1,4) = ' 5'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.012d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 2'
      ninlocal(1,2) = ' 2'
      ninlocal(1,3) = ' 3'
      ninlocal(1,4) = ' 5'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.006d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 2'
      ninlocal(1,2) = ' 2'
      ninlocal(1,3) = ' 5'
      ninlocal(1,4) = ' 5'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.027d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 2'
      ninlocal(1,2) = ' 2'
      ninlocal(1,3) = ' 5'
      ninlocal(1,4) = ' 6'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.017d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 2'
      ninlocal(1,2) = ' 2'
      ninlocal(1,3) = ' 5'
      ninlocal(1,4) = '37'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.012d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 2'
      ninlocal(1,2) = ' 2'
      ninlocal(1,3) = ' 5'
      ninlocal(1,4) = '40'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.008d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 2'
      ninlocal(1,2) = ' 2'
      ninlocal(1,3) = ' 5'
      ninlocal(1,4) = '41'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.130d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-3-*-* C94 DEF     
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.146d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 1'
      ninlocal(1,4) = ' 7'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.138d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 2'
      ninlocal(1,4) = ' 7'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.134d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 3'
      ninlocal(1,4) = ' 7'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.122d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 5'
      ninlocal(1,4) = ' 7'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.141d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 6'
      ninlocal(1,4) = ' 7'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.129d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 7'
      ninlocal(1,4) = '10'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.138d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 7'
      ninlocal(1,4) = '37'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.113d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 2'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 5'
      ninlocal(1,4) = ' 7'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.081d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 2'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 5'
      ninlocal(1,4) = ' 9'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.127d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 2'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 6'
      ninlocal(1,4) = ' 7'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.116d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 2'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 7'
      ninlocal(1,4) = '10'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.113d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 3'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 5'
      ninlocal(1,4) = ' 7'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.127d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 3'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 6'
      ninlocal(1,4) = ' 7'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.103d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 5'
      ninlocal(1,4) = ' 7'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.074d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 5'
      ninlocal(1,4) = ' 9'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.078d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 5'
      ninlocal(1,4) = '54'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.119d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 6'
      ninlocal(1,4) = ' 7'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.102d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 7'
      ninlocal(1,4) = '10'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.067d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 9'
      ninlocal(1,4) = '40'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.127d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 6'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = ' 7'
      ninlocal(1,4) = '37'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.113d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 7'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = '10'
      ninlocal(1,4) = '10'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.151d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 7'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = '20'
      ninlocal(1,4) = '20'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.057d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 9'
      ninlocal(1,2) = ' 3'
      ninlocal(1,3) = '40'
      ninlocal(1,4) = '40'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.000d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-8-*-* C94 DEF     
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = ' 8'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = (-0.020d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-10-*-* C94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '10'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = (-0.020d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- #C94 (C93=-0.034)   
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = '10'
      ninlocal(1,3) = ' 1'
      ninlocal(1,4) = ' 3'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = (-0.033d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = '10'
      ninlocal(1,3) = ' 3'
      ninlocal(1,4) = ' 6'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = (-0.020d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- #C94 (C93=-0.030)   
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = '10'
      ninlocal(1,3) = ' 3'
      ninlocal(1,4) = '28'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = (-0.030d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 3'
      ninlocal(1,2) = '10'
      ninlocal(1,3) = ' 3'
      ninlocal(1,4) = '28'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = (-0.019d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 3'
      ninlocal(1,2) = '10'
      ninlocal(1,3) = '28'
      ninlocal(1,4) = '28'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.000d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-17-*-* E94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '17'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.000d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-26-*-* CE4 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '26'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.010d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-30-*-* C94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '30'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.008d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '30'
      ninlocal(1,3) = '20'
      ninlocal(1,4) = '30'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.035d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-37-*-* C94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '37'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.040d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = '37'
      ninlocal(1,3) = '37'
      ninlocal(1,4) = '37'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.031d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 2'
      ninlocal(1,2) = '37'
      ninlocal(1,3) = '37'
      ninlocal(1,4) = '37'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.027d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 3'
      ninlocal(1,2) = '37'
      ninlocal(1,3) = '37'
      ninlocal(1,4) = '37'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.015d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '37'
      ninlocal(1,3) = '37'
      ninlocal(1,4) = '37'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.046d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '37'
      ninlocal(1,3) = '37'
      ninlocal(1,4) = '38'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.008d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '37'
      ninlocal(1,3) = '37'
      ninlocal(1,4) = '63'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.012d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '37'
      ninlocal(1,3) = '37'
      ninlocal(1,4) = '64'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.016d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '37'
      ninlocal(1,3) = '37'
      ninlocal(1,4) = '69'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.084d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '37'
      ninlocal(1,3) = '38'
      ninlocal(1,4) = '38'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.048d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 6'
      ninlocal(1,2) = '37'
      ninlocal(1,3) = '37'
      ninlocal(1,4) = '37'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.025d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = '15'
      ninlocal(1,2) = '37'
      ninlocal(1,3) = '37'
      ninlocal(1,4) = '37'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.046d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = '37'
      ninlocal(1,2) = '37'
      ninlocal(1,3) = '37'
      ninlocal(1,4) = '40'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.020d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-39-*-* C94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '39'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.012d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = '39'
      ninlocal(1,3) = '63'
      ninlocal(1,4) = '63'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = (-0.014d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = '23'
      ninlocal(1,2) = '39'
      ninlocal(1,3) = '63'
      ninlocal(1,4) = '63'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.021d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = '23'
      ninlocal(1,2) = '39'
      ninlocal(1,3) = '63'
      ninlocal(1,4) = '65'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.062d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = '23'
      ninlocal(1,2) = '39'
      ninlocal(1,3) = '65'
      ninlocal(1,4) = '65'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = (-0.005d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-40-*-* C94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '40'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = (-0.006d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = '40'
      ninlocal(1,3) = '28'
      ninlocal(1,4) = '37'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = (-0.007d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 2'
      ninlocal(1,2) = '40'
      ninlocal(1,3) = '28'
      ninlocal(1,4) = '28'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = (-0.007d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 3'
      ninlocal(1,2) = '40'
      ninlocal(1,3) = '28'
      ninlocal(1,4) = '28'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.004d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = '28'
      ninlocal(1,2) = '40'
      ninlocal(1,3) = '28'
      ninlocal(1,4) = '37'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.180d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-41-*-* C94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '41'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.178d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = '41'
      ninlocal(1,3) = '32'
      ninlocal(1,4) = '32'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.161d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 2'
      ninlocal(1,2) = '41'
      ninlocal(1,3) = '32'
      ninlocal(1,4) = '32'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.158d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '41'
      ninlocal(1,3) = '32'
      ninlocal(1,4) = '32'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.000d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-43-*-* E94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '43'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.150d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-45-*-* E94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '45'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.000d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-49-*-* E94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '49'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.000d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- #C94                
      ninlocal(1,1) = '50'
      ninlocal(1,2) = '49'
      ninlocal(1,3) = '50'
      ninlocal(1,4) = '50'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.020d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-54-*-* C94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '54'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.016d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = '54'
      ninlocal(1,3) = ' 3'
      ninlocal(1,4) = '36'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.018d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 3'
      ninlocal(1,2) = '54'
      ninlocal(1,3) = '36'
      ninlocal(1,4) = '36'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.020d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-55-*-* C94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '55'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.020d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- #C94                
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = '55'
      ninlocal(1,3) = '36'
      ninlocal(1,4) = '57'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.020d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- #C94                
      ninlocal(1,1) = '36'
      ninlocal(1,2) = '55'
      ninlocal(1,3) = '36'
      ninlocal(1,4) = '57'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.020d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-56-*-* C94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '56'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.020d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- #C94                
      ninlocal(1,1) = ' 1'
      ninlocal(1,2) = '56'
      ninlocal(1,3) = '36'
      ninlocal(1,4) = '57'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.020d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- #C94                
      ninlocal(1,1) = '36'
      ninlocal(1,2) = '56'
      ninlocal(1,3) = '36'
      ninlocal(1,4) = '57'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.080d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-57-*-* C94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '57'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.038d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '57'
      ninlocal(1,3) = '55'
      ninlocal(1,4) = '55'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.158d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = '56'
      ninlocal(1,2) = '57'
      ninlocal(1,3) = '56'
      ninlocal(1,4) = '56'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.025d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-58-*-* E94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '58'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.050d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-63-*-* C94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '63'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.019d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '63'
      ninlocal(1,3) = '39'
      ninlocal(1,4) = '64'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.068d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '63'
      ninlocal(1,3) = '39'
      ninlocal(1,4) = '66'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.014d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '63'
      ninlocal(1,3) = '44'
      ninlocal(1,4) = '64'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.055d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '63'
      ninlocal(1,3) = '44'
      ninlocal(1,4) = '66'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.033d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '63'
      ninlocal(1,3) = '59'
      ninlocal(1,4) = '64'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.085d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '63'
      ninlocal(1,3) = '59'
      ninlocal(1,4) = '66'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.010d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = '37'
      ninlocal(1,2) = '63'
      ninlocal(1,3) = '39'
      ninlocal(1,4) = '64'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.040d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-64-*-* C94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '64'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.006d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '64'
      ninlocal(1,3) = '63'
      ninlocal(1,4) = '64'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.043d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '64'
      ninlocal(1,3) = '63'
      ninlocal(1,4) = '66'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.052d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '64'
      ninlocal(1,3) = '64'
      ninlocal(1,4) = '65'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.094d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '64'
      ninlocal(1,3) = '65'
      ninlocal(1,4) = '66'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = (-0.011d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = '37'
      ninlocal(1,2) = '64'
      ninlocal(1,3) = '63'
      ninlocal(1,4) = '64'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.070d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-67-*-* E94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '67'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.070d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-69-*-* C94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '69'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.067d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = '32'
      ninlocal(1,2) = '69'
      ninlocal(1,3) = '37'
      ninlocal(1,4) = '37'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.000d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-73-*-* E94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '73'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.045d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-78-*-* C94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '78'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.046d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '78'
      ninlocal(1,3) = '78'
      ninlocal(1,4) = '81'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.080d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-80-*-* C94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '80'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.057d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = ' 5'
      ninlocal(1,2) = '80'
      ninlocal(1,3) = '81'
      ninlocal(1,4) = '81'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.025d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-81-*-* C94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '81'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.016d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- C94                 
      ninlocal(1,1) = '36'
      ninlocal(1,2) = '81'
      ninlocal(1,3) = '78'
      ninlocal(1,4) = '80'
      call twh_set_impnames(type,1,ninlocal)

      type = type + 1
      call twh_impform(GLB_SET,type,5)
      call twh_impstyle(GLB_SET,type,8)
      iclocal(1) = ( 0.000d0)*(0.043844d0)*(0.5d0)*(CON_KCALTOK)
      call twh_set_impcoeff(type,iclocal)
      call twh_impff(GLB_SET,type,ffname)
c     --- *-82-*-* E94 DEF    
      ninlocal(1,1) = ' 0'
      ninlocal(1,2) = '82'
      ninlocal(1,3) = ' 0'
      ninlocal(1,4) = ' 0'
      call twh_set_impnames(type,1,ninlocal)

      nimprotypes = type
      call twh_ffcheck(lfailure,4,type)
      if ( lfailure ) return
      
      return
      end

      subroutine twh_ffmmff94_bondincrement(lfailure,ffname,nbitypes)
c     ******************************************************************
c     * sets up the bond increments for MMFF94                         *
c     *                                                                *
c     * originally writen 12-20-2006 by M.G. Martin                    *
c     * last modified 08-04-2011 by M.G. Martin                        *
c     ******************************************************************
      implicit none
c     --- variables passed to/from the subroutine
      character*(FFNAMELEN) ffname
      logical lfailure
      integer nbitypes
c     --- local variables
      character*(FFNAMELEN) order
      character*(FFNAMELEN) qbinlocal 
      dimension qbinlocal(2)
      integer type
      double precision charge

      type = 0

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = ' 1'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1382d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = ' 2'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0610d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = ' 3'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = ' 4'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = ' 5'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2800d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = ' 6'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2700d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = ' 8'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2460d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = ' 9'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3001d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '10'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3400d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '11'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2900d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '12'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2300d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '13'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1900d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '14'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2300d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '15'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1935d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '17'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1052d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '18'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0805d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '19'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0950d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '22'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '25'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1669d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '26'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.5030d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '34'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4274d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '35'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1435d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2556d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '39'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3691d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1060d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '41'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3557d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '43'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2402d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '45'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3332d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '46'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3461d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '54'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4895d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '55'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3276d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '56'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1050d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '57'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4880d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '58'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2657d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '61'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '62'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1800d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1810d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0990d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '67'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2560d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '68'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.5500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '72'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0877d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '73'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2550d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '75'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1680d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1440d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '80'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.5140d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 1'
      qbinlocal(2) = '81'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = ' 2'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = ' 2'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0144d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = ' 3'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0650d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = ' 4'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0650d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = ' 4'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = ' 5'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0767d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = ' 6'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1710d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = ' 9'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1090d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '10'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1495d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '11'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1400d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '12'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1100d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '13'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0900d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '14'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1010d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '15'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0560d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '17'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0170d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '18'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2290d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '19'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1160d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0400d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '22'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1470d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '25'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0310d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '30'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3560d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '34'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '35'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0284d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0310d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '39'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '41'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1910d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '43'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2044d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '45'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2940d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '46'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3410d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '55'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3030d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '56'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '62'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0450d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0460d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0360d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '67'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '72'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3790d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 2'
      qbinlocal(2) = '81'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = ' 3'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1050d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = ' 4'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0600d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = ' 5'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = ' 6'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.5700d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = ' 7'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = ' 9'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2110d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = ' 9'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0600d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '10'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2220d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '11'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2090d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '12'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1410d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '15'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3800d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '16'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0960d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '17'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0230d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '18'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0530d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '22'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1070d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '25'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0710d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '30'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3610d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '35'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0862d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0090d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '39'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1470d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '41'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2363d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '43'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1650d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '45'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4300d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '48'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.9500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '51'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0134d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '53'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '54'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3290d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '54'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3810d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '55'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3430d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '56'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0100d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '57'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3930d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '58'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0300d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '62'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0850d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0860d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0040d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '67'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3190d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '74'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2474d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '75'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0730d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0490d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 3'
      qbinlocal(2) = '80'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1770d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 4'
      qbinlocal(2) = ' 5'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0430d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 4'
      qbinlocal(2) = ' 6'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4870d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 4'
      qbinlocal(2) = ' 7'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 4'
      qbinlocal(2) = ' 9'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1060d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 4'
      qbinlocal(2) = ' 9'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0440d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 4'
      qbinlocal(2) = '10'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0360d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 4'
      qbinlocal(2) = '15'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1810d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 4'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1050d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 4'
      qbinlocal(2) = '22'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0340d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 4'
      qbinlocal(2) = '30'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0730d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 4'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0640d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 4'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.5571d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 4'
      qbinlocal(2) = '42'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1260d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 4'
      qbinlocal(2) = '43'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0200d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 4'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0190d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 4'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 5'
      qbinlocal(2) = '19'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 5'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 5'
      qbinlocal(2) = '22'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 5'
      qbinlocal(2) = '30'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 5'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2203d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 5'
      qbinlocal(2) = '41'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 5'
      qbinlocal(2) = '57'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 5'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 5'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 5'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 5'
      qbinlocal(2) = '80'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = ' 6'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = ' 8'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0630d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = ' 9'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0355d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '10'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0070d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '15'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0520d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '17'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1837d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '18'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2974d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '19'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2579d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.4000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '21'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1480d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '22'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.5000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '24'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2712d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '25'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1010d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '26'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.4500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '29'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0770d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '30'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.5000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '33'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0825d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1390d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '39'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0210d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2950d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '41'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0830d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '43'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0090d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '45'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1810d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '54'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2330d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '55'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1380d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '57'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2450d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '58'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0630d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0620d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 6'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.5000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 7'
      qbinlocal(2) = '17'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1618d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 7'
      qbinlocal(2) = '46'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.5000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 7'
      qbinlocal(2) = '74'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = ' 8'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0530d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = ' 9'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0090d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '10'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0510d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '12'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0170d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '15'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0620d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '17'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3470d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '19'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2096d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1580d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '22'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3600d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '23'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2679d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '25'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1110d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '26'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2380d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '34'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1490d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '39'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0110d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0730d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '43'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0070d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '45'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1760d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '46'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2230d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '55'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1850d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 8'
      qbinlocal(2) = '56'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = ' 9'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0620d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '10'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0020d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '12'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0700d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '15'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1880d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '18'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.4000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '19'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2870d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3180d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '25'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.4000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '27'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1850d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '34'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '35'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1790d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2020d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '39'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0420d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3580d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '41'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0460d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '45'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3179d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '53'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1180d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '54'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1700d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '55'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1320d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '56'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2010d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '57'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1810d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '62'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1260d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1250d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2070d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '67'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1380d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2080d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = ' 9'
      qbinlocal(2) = '81'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '10'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0060d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '13'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0360d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '14'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0080d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '15'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0530d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '17'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2250d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1490d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '22'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2560d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '25'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1020d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '26'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3700d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '28'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2470d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '34'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2120d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '35'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1170d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1400d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '39'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0200d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2960d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '41'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0160d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '45'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0640d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0630d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '10'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2980d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '11'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2317d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '11'
      qbinlocal(2) = '22'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3290d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '11'
      qbinlocal(2) = '25'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1750d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '11'
      qbinlocal(2) = '26'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1900d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '11'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0530d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '11'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0680d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '12'
      qbinlocal(2) = '15'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1860d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '12'
      qbinlocal(2) = '18'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3701d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '12'
      qbinlocal(2) = '19'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2900d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '12'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2273d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '12'
      qbinlocal(2) = '22'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3160d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '12'
      qbinlocal(2) = '25'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2112d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '12'
      qbinlocal(2) = '26'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1770d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '12'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0400d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '12'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1990d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '12'
      qbinlocal(2) = '57'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1240d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '12'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1230d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '12'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2190d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '13'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1430d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '13'
      qbinlocal(2) = '22'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1110d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '13'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0570d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '13'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1890d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '14'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0810d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '14'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '15'
      qbinlocal(2) = '15'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1180d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '15'
      qbinlocal(2) = '18'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3300d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '15'
      qbinlocal(2) = '19'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2170d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '15'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1410d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '15'
      qbinlocal(2) = '22'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2480d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '15'
      qbinlocal(2) = '25'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0940d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '15'
      qbinlocal(2) = '26'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0700d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '15'
      qbinlocal(2) = '30'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1015d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '15'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0280d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '15'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0900d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '15'
      qbinlocal(2) = '43'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1310d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '15'
      qbinlocal(2) = '57'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0560d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '15'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0550d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '15'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1800d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '15'
      qbinlocal(2) = '71'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '16'
      qbinlocal(2) = '16'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '17'
      qbinlocal(2) = '17'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1720d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '17'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0960d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '17'
      qbinlocal(2) = '22'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0640d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '17'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1350d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '17'
      qbinlocal(2) = '43'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '18'
      qbinlocal(2) = '18'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0990d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '18'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0230d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '18'
      qbinlocal(2) = '22'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.6500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '18'
      qbinlocal(2) = '32'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0090d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '18'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0140d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '18'
      qbinlocal(2) = '39'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1380d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '18'
      qbinlocal(2) = '43'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.5895d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '18'
      qbinlocal(2) = '48'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3580d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '18'
      qbinlocal(2) = '55'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3700d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '18'
      qbinlocal(2) = '58'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2099d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '18'
      qbinlocal(2) = '62'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0620d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '18'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0630d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '18'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0260d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '18'
      qbinlocal(2) = '80'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '19'
      qbinlocal(2) = '19'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1130d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '19'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2210d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '19'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3580d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '19'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2740d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '19'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3490d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '19'
      qbinlocal(2) = '75'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '20'
      qbinlocal(2) = '20'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0760d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '20'
      qbinlocal(2) = '22'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0310d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '20'
      qbinlocal(2) = '25'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1230d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '20'
      qbinlocal(2) = '26'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1380d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '20'
      qbinlocal(2) = '30'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4720d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '20'
      qbinlocal(2) = '34'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1080d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '20'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2450d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '20'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0710d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '20'
      qbinlocal(2) = '41'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3070d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '20'
      qbinlocal(2) = '43'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2410d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '20'
      qbinlocal(2) = '45'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '22'
      qbinlocal(2) = '22'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0710d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '22'
      qbinlocal(2) = '30'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3960d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '22'
      qbinlocal(2) = '34'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0320d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '22'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1690d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '22'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1470d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '22'
      qbinlocal(2) = '41'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2310d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '22'
      qbinlocal(2) = '43'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1650d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '22'
      qbinlocal(2) = '45'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2700d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '23'
      qbinlocal(2) = '39'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '23'
      qbinlocal(2) = '62'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2920d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '23'
      qbinlocal(2) = '67'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3600d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '23'
      qbinlocal(2) = '68'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '25'
      qbinlocal(2) = '25'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.7000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '25'
      qbinlocal(2) = '32'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1390d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '25'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1160d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '25'
      qbinlocal(2) = '39'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2760d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '25'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3380d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '25'
      qbinlocal(2) = '43'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1170d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '25'
      qbinlocal(2) = '57'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1920d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '25'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0362d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '25'
      qbinlocal(2) = '71'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.6773d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '25'
      qbinlocal(2) = '72'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '26'
      qbinlocal(2) = '26'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3490d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '26'
      qbinlocal(2) = '34'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0150d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '26'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1220d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '26'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0960d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '26'
      qbinlocal(2) = '71'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '28'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4200d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '28'
      qbinlocal(2) = '43'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '28'
      qbinlocal(2) = '48'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '30'
      qbinlocal(2) = '30'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0980d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '30'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0670d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '30'
      qbinlocal(2) = '67'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4300d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '31'
      qbinlocal(2) = '70'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.6500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '32'
      qbinlocal(2) = '41'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.5200d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '32'
      qbinlocal(2) = '45'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.6330d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '32'
      qbinlocal(2) = '67'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.7500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '32'
      qbinlocal(2) = '68'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.7500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '32'
      qbinlocal(2) = '69'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '32'
      qbinlocal(2) = '73'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.4500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '32'
      qbinlocal(2) = '77'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.6330d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '32'
      qbinlocal(2) = '82'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.4500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '34'
      qbinlocal(2) = '36'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3640d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '34'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1650d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '34'
      qbinlocal(2) = '43'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3290d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '35'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2760d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '35'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '36'
      qbinlocal(2) = '54'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '36'
      qbinlocal(2) = '55'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '36'
      qbinlocal(2) = '56'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4570d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '36'
      qbinlocal(2) = '58'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'four      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '36'
      qbinlocal(2) = '58'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '36'
      qbinlocal(2) = '81'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '37'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3100d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '38'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0230d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '39'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0230d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '39'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1790d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '41'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1990d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '43'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1330d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '45'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3020d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '46'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3490d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '55'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3110d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '56'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0220d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '57'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3610d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '58'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3610d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '58'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'four      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '58'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1380d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '61'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0020d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '62'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0530d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0540d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0280d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '67'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0895d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '69'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0410d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3870d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '81'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3870d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '37'
      qbinlocal(2) = '81'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '38'
      qbinlocal(2) = '38'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2570d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '38'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2560d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '38'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3380d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '38'
      qbinlocal(2) = '69'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2690d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '38'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '39'
      qbinlocal(2) = '39'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1600d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '39'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1560d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '39'
      qbinlocal(2) = '45'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1516d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '39'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0760d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '39'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0770d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '39'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0770d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '39'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4180d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '39'
      qbinlocal(2) = '65'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0640d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '39'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '40'
      qbinlocal(2) = '40'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0040d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '40'
      qbinlocal(2) = '45'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1650d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '40'
      qbinlocal(2) = '46'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1600d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '40'
      qbinlocal(2) = '54'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0840d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '40'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0830d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '40'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0960d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '40'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '41'
      qbinlocal(2) = '41'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.5280d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '41'
      qbinlocal(2) = '55'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1770d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '41'
      qbinlocal(2) = '62'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.5000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '41'
      qbinlocal(2) = '72'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1960d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '41'
      qbinlocal(2) = '80'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.4920d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '42'
      qbinlocal(2) = '61'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '43'
      qbinlocal(2) = '43'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0660d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '43'
      qbinlocal(2) = '45'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1450d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '43'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0400d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '44'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2207d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '44'
      qbinlocal(2) = '65'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0690d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '44'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0930d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '44'
      qbinlocal(2) = '80'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0800d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '45'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0790d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '45'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0920d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '45'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3700d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '47'
      qbinlocal(2) = '53'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.5673d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '49'
      qbinlocal(2) = '50'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.5000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '51'
      qbinlocal(2) = '52'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3544d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '55'
      qbinlocal(2) = '57'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3510d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '55'
      qbinlocal(2) = '62'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2950d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '55'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3320d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '55'
      qbinlocal(2) = '80'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.4000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '56'
      qbinlocal(2) = '57'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2580d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '56'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2700d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '56'
      qbinlocal(2) = '80'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'four      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '57'
      qbinlocal(2) = '58'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0750d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '57'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0760d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '57'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3080d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '58'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3070d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '58'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1400d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '59'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.1209d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '59'
      qbinlocal(2) = '65'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1690d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '59'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.1930d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '59'
      qbinlocal(2) = '80'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2380d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '59'
      qbinlocal(2) = '82'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3700d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '60'
      qbinlocal(2) = '61'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0550d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '62'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0560d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '62'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '63'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '63'
      qbinlocal(2) = '63'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '63'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3381d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '63'
      qbinlocal(2) = '66'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '63'
      qbinlocal(2) = '72'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0120d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '63'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3340d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '63'
      qbinlocal(2) = '81'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '64'
      qbinlocal(2) = '64'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2888d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '64'
      qbinlocal(2) = '65'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.2272d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '64'
      qbinlocal(2) = '66'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0130d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '64'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3330d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '64'
      qbinlocal(2) = '81'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0820d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '64'
      qbinlocal(2) = '82'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '65'
      qbinlocal(2) = '66'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3070d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '65'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0390d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '65'
      qbinlocal(2) = '81'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.3760d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '65'
      qbinlocal(2) = '82'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '66'
      qbinlocal(2) = '66'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.2990d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '66'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0470d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '66'
      qbinlocal(2) = '81'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0958d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '71'
      qbinlocal(2) = '75'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.4500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '72'
      qbinlocal(2) = '73'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '76'
      qbinlocal(2) = '76'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.4000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '76'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '78'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge =  0.0000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'single    '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '78'
      qbinlocal(2) = '78'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3030d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '78'
      qbinlocal(2) = '79'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.3500d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '78'
      qbinlocal(2) = '81'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.0430d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '79'
      qbinlocal(2) = '81'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      charge = -0.4000d0
      call twh_qbivalue(GLB_SET,type,charge)
      order = 'wild      '
      call twh_biorder(GLB_SET,type,order)
      qbinlocal(1) = '80'
      qbinlocal(2) = '81'
      call twh_set_qbinames(type,qbinlocal)



      nbitypes = type
      call twh_ffcheck(lfailure,6,type)
      if ( lfailure ) return

      return
      end
