#include "preproc.h"
c     ******************************************************************
c     * MCCCS - Towhee: A Monte Carlo molecular simulation program     *
c     * Copyright (C) 2006-2011 Marcus G. Martin                       *
c     * see the file license.gpl for the full license information      *
c     *                                                                *
c     * This program is free software; you can redistribute it and/or  *
c     * modify it under the terms of the GNU General Public License    *
c     * as published by the Free Software Foundation; either version 2 *
c     * of the License, or (at your option) any later version.         *
c     *                                                                *
c     * This program is distributed in the hope that it will be useful,*
c     * but WITHOUT ANY WARRANTY; without even the implied warranty of *
c     * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  *
c     * GNU General Public License for more details.                   *
c     *                                                                *
c     * You should have received a copy of the GNU General Public      *
c     * License along with this program; if not, write to the Free     *
c     * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,*
c     * MA  02111-1307, USA.                                           *
c     ******************************************************************
      subroutine twh_ffqmff_viii(lfailure,ntypes,nbondtypes,nangletypes
     &     ,ndihedtypes,naatypes,nbitypes)
c     ******************************************************************
c     * sets up all of the force field parameters for the QMFF-VIII    *
c     * forcefield                                                     *
c     *                                                                *
c     * originally written 04-17-2006 by M.G. Martin                   *
c     * last modified 08-04-2011 by M.G. Martin                        *
c     ******************************************************************
      implicit none
c     --- variables passed to/from the subroutine
      logical lfailure
      integer ntypes,nbondtypes,nangletypes,ndihedtypes
     &        ,naatypes,nbitypes
c     --- local variables
      character*(FFNAMELEN) ffname

c     --- reference
c     C.S. Ewig; R. Berry; U. Dinur; J.-R. Hill; M.-J. Hwang; H. Li;
c     Ch. Liang; J. Maple; Z. Peng; T.P. Stockfisch; T.S. Thacher;
c     L. Yan; X. Ni; A.T. Hagler;
c     "Derivation of Class II Force Fields.  VIII. Derivation of a 
c     General Quantum Mechanical Force Field for Organic Compounds",
c     J. Comp. Chem. 22, 1782-1800 (2001)

      ffname = 'QMFF-VIII'
      call twh_classical_potential(GLB_SET,'9-6')
      call twh_classical_mixrule(GLB_SET,'Sixth Power')

      call twh_ffqmff_viii_nonbond(lfailure,ffname,ntypes)
      if ( lfailure ) return
      call twh_ffqmff_viii_bond(lfailure,ffname,nbondtypes)
      if ( lfailure ) return
      call twh_ffqmff_viii_angle(lfailure,ffname,nangletypes)
      if ( lfailure ) return
      call twh_ffqmff_viii_dihedral(lfailure,ffname,ndihedtypes)
      if ( lfailure ) return
      call twh_ffqmff_viii_angleangle(lfailure,ffname,naatypes)
      if ( lfailure ) return
      call twh_ffqmff_viii_bond_increment(lfailure,ffname,nbitypes)
      if ( lfailure ) return

      return
      end

      subroutine twh_ffqmff_viii_nonbond(lfailure,ffname,ntypes)
c     *****************************************************************
c     * sets up the nonbonded parameters for QMFF-VIII                *
c     *                                                               *
c     * originally written 04-17-2006 by M.G. Martin                  *
c     * last modified 08-04-2011 by M.G. Martin                       *
c     *****************************************************************
      implicit none
#define FUNCTION_GETNBTYPE
#include "functions.h"
c     --- variables passed to/from the subroutine
      character*(FFNAMELEN) ffname
      logical lfailure
      integer ntypes
c     --- local variables
      character*(FFNAMELEN) nbnlocal,bonlocal,anglocal,torlocal
      integer ii,type
      double precision nbclocal
      dimension nbclocal(MINNBCOEFF:MAXNBCOEFF)

c     --- initialize number of nonbonded types
      type = 0

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,79.9040d0)
      call twh_elementname(GLB_SET,type,'Br')
      nbnlocal = 'Br '
      bonlocal = 'Br '
      anglocal = 'Br '
      torlocal = 'Br '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     5.470000d0
      nbclocal(2) =     0.083013d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,40.08d0)
      call twh_elementname(GLB_SET,type,'Ca')
      nbnlocal = 'Ca '
      bonlocal = 'Ca '
      anglocal = 'Ca '
      torlocal = 'Ca '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.400000d0
      nbclocal(2) =     0.112000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,35.4530d0)
      call twh_elementname(GLB_SET,type,'Cl')
      nbnlocal = 'Cl '
      bonlocal = 'Cl '
      anglocal = 'Cl '
      torlocal = 'Cl '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     5.220000d0
      nbclocal(2) =     0.061802d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,132.90d0)
      call twh_elementname(GLB_SET,type,'Cs')
      nbnlocal = 'Cs '
      bonlocal = 'Cs '
      anglocal = 'Cs '
      torlocal = 'Cs '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.300000d0
      nbclocal(2) =     0.312195d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,63.5460d0)
      call twh_elementname(GLB_SET,type,'Cu')
      nbnlocal = 'Cu '
      bonlocal = 'Cu '
      anglocal = 'Cu '
      torlocal = 'Cu '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     2.650000d0
      nbclocal(2) =     0.045000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,18.9980d0)
      call twh_elementname(GLB_SET,type,' F')
      nbnlocal = 'F  '
      bonlocal = 'F  '
      anglocal = 'F  '
      torlocal = 'F  '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.600000d0
      nbclocal(2) =     0.016501d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,55.8470d0)
      call twh_elementname(GLB_SET,type,'Fe')
      nbnlocal = 'Fe '
      bonlocal = 'Fe '
      anglocal = 'Fe '
      torlocal = 'Fe '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     2.650000d0
      nbclocal(2) =     0.080000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,126.900d0)
      call twh_elementname(GLB_SET,type,' I')
      nbnlocal = 'I  '
      bonlocal = 'I  '
      anglocal = 'I  '
      torlocal = 'I  '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     5.770000d0
      nbclocal(2) =     0.122919d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,39.0980d0)
      call twh_elementname(GLB_SET,type,' K')
      nbnlocal = 'K  '
      bonlocal = 'K  '
      anglocal = 'K  '
      torlocal = 'K  '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.600000d0
      nbclocal(2) =     0.172962d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,6.9410d0)
      call twh_elementname(GLB_SET,type,'Li')
      nbnlocal = 'Li '
      bonlocal = 'Li '
      anglocal = 'Li '
      torlocal = 'Li '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     2.400000d0
      nbclocal(2) =     0.016483d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,24.3050d0)
      call twh_elementname(GLB_SET,type,'Mg')
      nbnlocal = 'Mg '
      bonlocal = 'Mg '
      anglocal = 'Mg '
      torlocal = 'Mg '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     2.600000d0
      nbclocal(2) =     0.046000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,22.990d0)
      call twh_elementname(GLB_SET,type,'Na')
      nbnlocal = 'Na '
      bonlocal = 'Na '
      anglocal = 'Na '
      torlocal = 'Na '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.030000d0
      nbclocal(2) =     0.051754d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,85.470d0)
      call twh_elementname(GLB_SET,type,'Rb')
      nbnlocal = 'Rb '
      bonlocal = 'Rb '
      anglocal = 'Rb '
      torlocal = 'Rb '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.900000d0
      nbclocal(2) =     0.225507d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,65.390d0)
      call twh_elementname(GLB_SET,type,'Zn')
      nbnlocal = 'Zn '
      bonlocal = 'Zn '
      anglocal = 'Zn '
      torlocal = 'Zn '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     2.700000d0
      nbclocal(2) =     0.047000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,79.9040d0)
      call twh_elementname(GLB_SET,type,'Br')
      nbnlocal = 'br '
      bonlocal = 'br '
      anglocal = 'br '
      torlocal = 'br '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.215000d0
      nbclocal(2) =     0.305000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,12.0110d0)
      call twh_elementname(GLB_SET,type,' C')
      nbnlocal = 'c  '
      bonlocal = 'c  '
      anglocal = 'c  '
      torlocal = 'c  '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.010000d0
      nbclocal(2) =     0.054000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,12.0110d0)
      call twh_elementname(GLB_SET,type,' C')
      nbnlocal = 'c" '
      bonlocal = 'c" '
      anglocal = 'c" '
      torlocal = 'c" '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.308000d0
      nbclocal(2) =     0.120000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,12.0110d0)
      call twh_elementname(GLB_SET,type,' C')
      nbnlocal = 'c` '
      bonlocal = 'c` '
      anglocal = 'c` '
      torlocal = 'c` '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.308000d0
      nbclocal(2) =     0.120000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,12.0110d0)
      call twh_elementname(GLB_SET,type,' C')
      nbnlocal = 'c+ '
      bonlocal = 'c+ '
      anglocal = 'c+ '
      torlocal = 'c+ '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.308000d0
      nbclocal(2) =     0.120000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,12.0110d0)
      call twh_elementname(GLB_SET,type,' C')
      nbnlocal = 'c- '
      bonlocal = 'c- '
      anglocal = 'c- '
      torlocal = 'c- '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.908000d0
      nbclocal(2) =     0.120000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,12.0110d0)
      call twh_elementname(GLB_SET,type,' C')
      nbnlocal = 'c= '
      bonlocal = 'c= '
      anglocal = 'c= '
      torlocal = 'c= '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.010000d0
      nbclocal(2) =     0.064000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,35.4530d0)
      call twh_elementname(GLB_SET,type,'Cl')
      nbnlocal = 'cl '
      bonlocal = 'cl '
      anglocal = 'cl '
      torlocal = 'cl '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.127710d0
      nbclocal(2) =     0.230210d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,12.0110d0)
      call twh_elementname(GLB_SET,type,' C')
      nbnlocal = 'co '
      bonlocal = 'co '
      anglocal = 'co '
      torlocal = 'co '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.010000d0
      nbclocal(2) =     0.054000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,12.0110d0)
      call twh_elementname(GLB_SET,type,' C')
      nbnlocal = 'cp '
      bonlocal = 'cp '
      anglocal = 'cp '
      torlocal = 'cp '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.010000d0
      nbclocal(2) =     0.064000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,12.0110d0)
      call twh_elementname(GLB_SET,type,' C')
      nbnlocal = 'cpb'
      bonlocal = 'cpb'
      anglocal = 'cpb'
      torlocal = 'cpb'
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.010000d0
      nbclocal(2) =     0.064000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,12.0110d0)
      call twh_elementname(GLB_SET,type,' C')
      nbnlocal = 'cr '
      bonlocal = 'cr '
      anglocal = 'cr '
      torlocal = 'cr '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.308000d0
      nbclocal(2) =     0.120000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,12.0110d0)
      call twh_elementname(GLB_SET,type,' C')
      nbnlocal = 'ct '
      bonlocal = 'ct '
      anglocal = 'ct '
      torlocal = 'ct '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.010000d0
      nbclocal(2) =     0.064000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,18.9980d0)
      call twh_elementname(GLB_SET,type,' F')
      nbnlocal = 'f  '
      bonlocal = 'f  '
      anglocal = 'f  '
      torlocal = 'f  '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.632060d0
      nbclocal(2) =     0.053310d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,1.00790d0)
      call twh_elementname(GLB_SET,type,' H')
      nbnlocal = 'h  '
      bonlocal = 'h  '
      anglocal = 'h  '
      torlocal = 'h  '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     2.995000d0
      nbclocal(2) =     0.020000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,1.00790d0)
      call twh_elementname(GLB_SET,type,' H')
      nbnlocal = 'h* '
c     --- with the file mixup all of the h* turned into plain h
c     --- given that h* and h are not bound to the same atoms it was easier
c     --- to set the rest of the intramolecular types to h
      bonlocal = 'h'
      anglocal = 'h'
      torlocal = 'h'
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     1.098000d0
      nbclocal(2) =     0.013000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,1.00790d0)
      call twh_elementname(GLB_SET,type,' H')
      nbnlocal = 'h+ '
      bonlocal = 'h+ '
      anglocal = 'h+ '
      torlocal = 'h+ '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     1.098000d0
      nbclocal(2) =     0.013000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,126.900d0)
      call twh_elementname(GLB_SET,type,' I')
      nbnlocal = 'i  '
      bonlocal = 'i  '
      anglocal = 'i  '
      torlocal = 'i  '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.800000d0
      nbclocal(2) =     0.400000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,14.0070d0)
      call twh_elementname(GLB_SET,type,' N')
      nbnlocal = 'n  '
      bonlocal = 'n  '
      anglocal = 'n  '
      torlocal = 'n  '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.865500d0
      nbclocal(2) =     0.065500d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,14.0070d0)
      call twh_elementname(GLB_SET,type,' N')
      nbnlocal = 'n` '
      bonlocal = 'n` '
      anglocal = 'n` '
      torlocal = 'n` '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.700000d0
      nbclocal(2) =     0.170000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,14.0070d0)
      call twh_elementname(GLB_SET,type,' N')
      nbnlocal = 'n+ '
      bonlocal = 'n+ '
      anglocal = 'n+ '
      torlocal = 'n+ '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.700000d0
      nbclocal(2) =     0.150000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,14.0070d0)
      call twh_elementname(GLB_SET,type,' N')
      nbnlocal = 'n= '
      bonlocal = 'n= '
      anglocal = 'n= '
      torlocal = 'n= '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.700000d0
      nbclocal(2) =     0.170000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,14.0070d0)
      call twh_elementname(GLB_SET,type,' N')
      nbnlocal = 'na '
      bonlocal = 'na '
      anglocal = 'na '
      torlocal = 'na '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.070000d0
      nbclocal(2) =     0.065000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,14.0070d0)
      call twh_elementname(GLB_SET,type,' N')
      nbnlocal = 'nh '
      bonlocal = 'nh '
      anglocal = 'nh '
      torlocal = 'nh '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.700000d0
      nbclocal(2) =     0.170000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,14.0070d0)
      call twh_elementname(GLB_SET,type,' N')
      nbnlocal = 'nn '
      bonlocal = 'nn '
      anglocal = 'nn '
      torlocal = 'nn '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.700000d0
      nbclocal(2) =     0.170000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,14.0070d0)
      call twh_elementname(GLB_SET,type,' N')
      nbnlocal = 'np '
      bonlocal = 'np '
      anglocal = 'np '
      torlocal = 'np '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.700000d0
      nbclocal(2) =     0.170000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,15.9990d0)
      call twh_elementname(GLB_SET,type,' O')
      nbnlocal = 'o  '
      bonlocal = 'o  '
      anglocal = 'o  '
      torlocal = 'o  '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.535000d0
      nbclocal(2) =     0.240000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,15.9990d0)
      call twh_elementname(GLB_SET,type,' O')
      nbnlocal = 'o` '
      bonlocal = 'o` '
      anglocal = 'o` '
      torlocal = 'o` '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.535000d0
      nbclocal(2) =     0.267000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,15.9990d0)
      call twh_elementname(GLB_SET,type,' O')
      nbnlocal = 'o? '
      bonlocal = 'o? '
      anglocal = 'o? '
      torlocal = 'o? '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.608000d0
      nbclocal(2) =     0.274000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,15.9990d0)
      call twh_elementname(GLB_SET,type,' O')
      nbnlocal = 'o- '
      bonlocal = 'o- '
      anglocal = 'o- '
      torlocal = 'o- '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.596000d0
      nbclocal(2) =     0.167000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,15.9990d0)
      call twh_elementname(GLB_SET,type,' O')
      nbnlocal = 'op '
      bonlocal = 'op '
      anglocal = 'op '
      torlocal = 'op '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     3.535000d0
      nbclocal(2) =     0.109000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,30.9740d0)
      call twh_elementname(GLB_SET,type,' P')
      nbnlocal = 'p  '
      bonlocal = 'p  '
      anglocal = 'p  '
      torlocal = 'p  '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.200000d0
      nbclocal(2) =     0.200000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,30.9740d0)
      call twh_elementname(GLB_SET,type,' P')
      nbnlocal = 'p- '
      bonlocal = 'p- '
      anglocal = 'p- '
      torlocal = 'p- '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.200000d0
      nbclocal(2) =     0.200000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,32.0660d0)
      call twh_elementname(GLB_SET,type,' S')
      nbnlocal = 's  '
      bonlocal = 's  '
      anglocal = 's  '
      torlocal = 's  '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.027000d0
      nbclocal(2) =     0.071000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,32.0660d0)
      call twh_elementname(GLB_SET,type,' S')
      nbnlocal = 's" '
      bonlocal = 's" '
      anglocal = 's" '
      torlocal = 's" '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.027000d0
      nbclocal(2) =     0.071000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      type = type + 1
      call twh_nbff(GLB_SET,type,ffname)
      call twh_qbasevalue(GLB_SET,type,0.0d0)
      call twh_mass(GLB_SET,type,32.0660d0)
      call twh_elementname(GLB_SET,type,' S')
      nbnlocal = 'sp '
      bonlocal = 'sp '
      anglocal = 'sp '
      torlocal = 'sp '
      call twh_set_names(type,nbnlocal,bonlocal,anglocal,torlocal)
      ii = twh_getnbtype(type,type)
      nbclocal(1) =     4.027000d0
      nbclocal(2) =     0.071000d0*CON_KCALTOK
      call twh_set_nbcoeff(ii,nbclocal)

      ntypes = type
      call twh_ffcheck(lfailure,0,type)
      if ( lfailure ) return

      return
      end

      subroutine twh_ffqmff_viii_bond(lfailure,ffname,nbondtypes)
c     ******************************************************************
c     * sets up the bond parameters for QMFF-VIII                      *
c     *                                                                *
c     * originally written 05-04-2006 by M.G. Martin                   *
c     * last modified 08-04-2011 by M.G. Martin                        *
c     ******************************************************************
      implicit none
c     --- variables passed to/from the subroutine
      character*(FFNAMELEN) ffname
      logical lfailure
      integer nbondtypes
c     --- local variables
      integer type,nsame
      double precision vclocal
      dimension vclocal(MINVIBCOEFF:MAXVIBCOEFF)
      character*(FFNAMELEN) vnlocal
      dimension vnlocal(MAXSAMEV,MAXVNAME)

c     --- initialize types
      type = 0

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.52604d0
      vclocal(1) =    309.89988d0*CON_KCALTOK
      vclocal(2) =   -518.88533d0*CON_KCALTOK
      vclocal(3) =    703.00732d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'c  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.51254d0
      vclocal(1) =    323.03196d0*CON_KCALTOK
      vclocal(2) =   -481.72576d0*CON_KCALTOK
      vclocal(3) =    490.00000d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'c" '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.50201d0
      vclocal(1) =    367.19212d0*CON_KCALTOK
      vclocal(2) =   -482.52231d0*CON_KCALTOK
      vclocal(3) =    574.37074d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'c` '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.54831d0
      vclocal(1) =    297.74510d0*CON_KCALTOK
      vclocal(2) =   -528.26600d0*CON_KCALTOK
      vclocal(3) =    538.01810d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'c- '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.50518d0
      vclocal(1) =    346.13442d0*CON_KCALTOK
      vclocal(2) =   -645.15327d0*CON_KCALTOK
      vclocal(3) =    376.65895d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'c= '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.78230d0
      vclocal(1) =    249.12229d0*CON_KCALTOK
      vclocal(2) =   -374.73117d0*CON_KCALTOK
      vclocal(3) =    336.41105d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'cl '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.51266d0
      vclocal(1) =    390.57695d0*CON_KCALTOK
      vclocal(2) =   -440.62528d0*CON_KCALTOK
      vclocal(3) =    287.40386d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'co '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.49996d0
      vclocal(1) =    409.37169d0*CON_KCALTOK
      vclocal(2) =   -639.53851d0*CON_KCALTOK
      vclocal(3) =    495.04714d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'cp '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.46955d0
      vclocal(1) =    404.41436d0*CON_KCALTOK
      vclocal(2) =   -759.38519d0*CON_KCALTOK
      vclocal(3) =   1024.14800d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'ct '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.32360d0
      vclocal(1) =    537.74228d0*CON_KCALTOK
      vclocal(2) =  -1082.29600d0*CON_KCALTOK
      vclocal(3) =   1257.77430d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'f  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.08466d0
      vclocal(1) =    415.84688d0*CON_KCALTOK
      vclocal(2) =   -843.77460d0*CON_KCALTOK
      vclocal(3) =   1029.97780d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'h  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.44075d0
      vclocal(1) =    434.48963d0*CON_KCALTOK
      vclocal(2) =   -742.19076d0*CON_KCALTOK
      vclocal(3) =    475.89448d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'n  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.52281d0
      vclocal(1) =    288.91285d0*CON_KCALTOK
      vclocal(2) =   -577.95491d0*CON_KCALTOK
      vclocal(3) =    667.41371d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'n+ '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.46604d0
      vclocal(1) =    362.15770d0*CON_KCALTOK
      vclocal(2) =   -708.26683d0*CON_KCALTOK
      vclocal(3) =    617.90766d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'n= '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.45168d0
      vclocal(1) =    378.28874d0*CON_KCALTOK
      vclocal(2) =   -723.51280d0*CON_KCALTOK
      vclocal(3) =   1032.55870d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'na '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.44156d0
      vclocal(1) =    403.90825d0*CON_KCALTOK
      vclocal(2) =   -676.41918d0*CON_KCALTOK
      vclocal(3) =    650.00000d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'nh '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.44156d0
      vclocal(1) =    403.90825d0*CON_KCALTOK
      vclocal(2) =   -676.41918d0*CON_KCALTOK
      vclocal(3) =    650.00000d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'nn '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.40193d0
      vclocal(1) =    414.05934d0*CON_KCALTOK
      vclocal(2) =   -863.69713d0*CON_KCALTOK
      vclocal(3) =   1357.82230d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 'o  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.82015d0
      vclocal(1) =    245.98034d0*CON_KCALTOK
      vclocal(2) =   -423.13191d0*CON_KCALTOK
      vclocal(3) =    274.28647d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 's  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.78759d0
      vclocal(1) =    268.69147d0*CON_KCALTOK
      vclocal(2) =   -396.42741d0*CON_KCALTOK
      vclocal(3) =    594.51548d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c  '
      vnlocal(nsame,2) = 's" '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.47020d0
      vclocal(1) =    427.71666d0*CON_KCALTOK
      vclocal(2) =   -432.62860d0*CON_KCALTOK
      vclocal(3) =    642.51686d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c" '
      vnlocal(nsame,2) = 'cp '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.09619d0
      vclocal(1) =    390.85012d0*CON_KCALTOK
      vclocal(2) =   -799.19280d0*CON_KCALTOK
      vclocal(3) =    897.79851d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c" '
      vnlocal(nsame,2) = 'h  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.39830d0
      vclocal(1) =    443.40625d0*CON_KCALTOK
      vclocal(2) =   -689.47285d0*CON_KCALTOK
      vclocal(3) =   1415.28450d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c" '
      vnlocal(nsame,2) = 'n  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.34248d0
      vclocal(1) =    472.73190d0*CON_KCALTOK
      vclocal(2) =  -1067.28040d0*CON_KCALTOK
      vclocal(3) =   1633.36250d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c" '
      vnlocal(nsame,2) = 'o  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.19294d0
      vclocal(1) =   1128.48590d0*CON_KCALTOK
      vclocal(2) =  -2573.68520d0*CON_KCALTOK
      vclocal(3) =   3155.52160d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c" '
      vnlocal(nsame,2) = 'o` '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.51027d0
      vclocal(1) =    333.21819d0*CON_KCALTOK
      vclocal(2) =   -522.26834d0*CON_KCALTOK
      vclocal(3) =    490.00000d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c` '
      vnlocal(nsame,2) = 'co '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.49598d0
      vclocal(1) =    391.48546d0*CON_KCALTOK
      vclocal(2) =   -464.58560d0*CON_KCALTOK
      vclocal(3) =    197.55506d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c` '
      vnlocal(nsame,2) = 'cp '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.09293d0
      vclocal(1) =    402.74935d0*CON_KCALTOK
      vclocal(2) =   -857.77151d0*CON_KCALTOK
      vclocal(3) =    983.77871d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c` '
      vnlocal(nsame,2) = 'h  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.40009d0
      vclocal(1) =    467.86594d0*CON_KCALTOK
      vclocal(2) =   -544.00237d0*CON_KCALTOK
      vclocal(3) =   2018.82070d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c` '
      vnlocal(nsame,2) = 'n  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.34248d0
      vclocal(1) =    472.73190d0*CON_KCALTOK
      vclocal(2) =  -1067.28040d0*CON_KCALTOK
      vclocal(3) =   1633.36250d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c` '
      vnlocal(nsame,2) = 'o  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.18988d0
      vclocal(1) =   1157.03830d0*CON_KCALTOK
      vclocal(2) =  -2686.07180d0*CON_KCALTOK
      vclocal(3) =   3575.89300d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c` '
      vnlocal(nsame,2) = 'o` '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.47000d0
      vclocal(1) =    453.56575d0*CON_KCALTOK
      vclocal(2) =   -372.57821d0*CON_KCALTOK
      vclocal(3) =    179.12791d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c+ '
      vnlocal(nsame,2) = 'cp '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.08638d0
      vclocal(1) =    422.32795d0*CON_KCALTOK
      vclocal(2) =   -865.16683d0*CON_KCALTOK
      vclocal(3) =   1123.50220d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c+ '
      vnlocal(nsame,2) = 'h  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.36312d0
      vclocal(1) =    604.42083d0*CON_KCALTOK
      vclocal(2) =   -769.28210d0*CON_KCALTOK
      vclocal(3) =   2284.83140d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c+ '
      vnlocal(nsame,2) = 'n  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.13306d0
      vclocal(1) =    293.97280d0*CON_KCALTOK
      vclocal(2) =   -700.95200d0*CON_KCALTOK
      vclocal(3) =   1041.15900d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c- '
      vnlocal(nsame,2) = 'h  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.23390d0
      vclocal(1) =    889.19130d0*CON_KCALTOK
      vclocal(2) =  -1929.84000d0*CON_KCALTOK
      vclocal(3) =   2323.29100d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c- '
      vnlocal(nsame,2) = 'o- '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.33115d0
      vclocal(1) =    746.94016d0*CON_KCALTOK
      vclocal(2) =  -1377.57810d0*CON_KCALTOK
      vclocal(3) =   1679.09810d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c= '
      vnlocal(nsame,2) = 'c= '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.46328d0
      vclocal(1) =    455.90884d0*CON_KCALTOK
      vclocal(2) =   -602.81771d0*CON_KCALTOK
      vclocal(3) =    -16.12737d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c= '
      vnlocal(nsame,2) = 'cp '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.07506d0
      vclocal(1) =    447.58678d0*CON_KCALTOK
      vclocal(2) =   -887.83660d0*CON_KCALTOK
      vclocal(3) =    956.51260d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c= '
      vnlocal(nsame,2) = 'h  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.38507d0
      vclocal(1) =    540.38223d0*CON_KCALTOK
      vclocal(2) =   -612.86278d0*CON_KCALTOK
      vclocal(3) =    407.28594d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c= '
      vnlocal(nsame,2) = 'n  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.25993d0
      vclocal(1) =    936.54686d0*CON_KCALTOK
      vclocal(2) =  -2079.23450d0*CON_KCALTOK
      vclocal(3) =   2693.74510d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'c= '
      vnlocal(nsame,2) = 'n= '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.72720d0
      vclocal(1) =    327.71180d0*CON_KCALTOK
      vclocal(2) =   -550.99661d0*CON_KCALTOK
      vclocal(3) =    511.59920d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cl '
      vnlocal(nsame,2) = 'cp '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.08177d0
      vclocal(1) =    422.51099d0*CON_KCALTOK
      vclocal(2) =   -883.95475d0*CON_KCALTOK
      vclocal(3) =   1102.38290d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'co '
      vnlocal(nsame,2) = 'h  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.44156d0
      vclocal(1) =    403.90825d0*CON_KCALTOK
      vclocal(2) =   -676.41918d0*CON_KCALTOK
      vclocal(3) =    650.00000d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'co '
      vnlocal(nsame,2) = 'nh '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.38729d0
      vclocal(1) =    466.01982d0*CON_KCALTOK
      vclocal(2) =   -824.08529d0*CON_KCALTOK
      vclocal(3) =    875.55044d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'co '
      vnlocal(nsame,2) = 'o  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.42754d0
      vclocal(1) =    438.79209d0*CON_KCALTOK
      vclocal(2) =   -703.30292d0*CON_KCALTOK
      vclocal(3) =   1656.11740d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 'cp '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.42754d0
      vclocal(1) =    438.79209d0*CON_KCALTOK
      vclocal(2) =   -703.30292d0*CON_KCALTOK
      vclocal(3) =   1656.11740d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 'cpb'
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.43466d0
      vclocal(1) =    452.50609d0*CON_KCALTOK
      vclocal(2) =   -791.87535d0*CON_KCALTOK
      vclocal(3) =    902.58397d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 'ct '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.31950d0
      vclocal(1) =    557.41119d0*CON_KCALTOK
      vclocal(2) =  -1135.46430d0*CON_KCALTOK
      vclocal(3) =   1382.53640d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 'f  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.07094d0
      vclocal(1) =    466.33336d0*CON_KCALTOK
      vclocal(2) =   -989.32036d0*CON_KCALTOK
      vclocal(3) =   1120.42850d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 'h  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.35829d0
      vclocal(1) =    601.91527d0*CON_KCALTOK
      vclocal(2) =   -715.49795d0*CON_KCALTOK
      vclocal(3) =   1287.15070d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 'n  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.41814d0
      vclocal(1) =    509.65228d0*CON_KCALTOK
      vclocal(2) =   -481.73771d0*CON_KCALTOK
      vclocal(3) =    273.75505d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 'n` '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.48437d0
      vclocal(1) =    380.88502d0*CON_KCALTOK
      vclocal(2) =   -693.43613d0*CON_KCALTOK
      vclocal(3) =    754.58459d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 'n+ '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.43000d0
      vclocal(1) =    404.18901d0*CON_KCALTOK
      vclocal(2) =   -693.58910d0*CON_KCALTOK
      vclocal(3) =   1210.36050d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 'n= '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.36165d0
      vclocal(1) =    562.35033d0*CON_KCALTOK
      vclocal(2) =   -759.20705d0*CON_KCALTOK
      vclocal(3) =    379.19691d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 'nh '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.35829d0
      vclocal(1) =    601.91527d0*CON_KCALTOK
      vclocal(2) =   -715.49795d0*CON_KCALTOK
      vclocal(3) =   1287.15070d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 'nn '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.33345d0
      vclocal(1) =    576.22904d0*CON_KCALTOK
      vclocal(2) =  -1152.12840d0*CON_KCALTOK
      vclocal(3) =   1685.64660d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 'np '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.35403d0
      vclocal(1) =    548.51948d0*CON_KCALTOK
      vclocal(2) =   -777.90554d0*CON_KCALTOK
      vclocal(3) =    750.00000d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 'o  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.35843d0
      vclocal(1) =    470.01045d0*CON_KCALTOK
      vclocal(2) =   -560.19556d0*CON_KCALTOK
      vclocal(3) =    807.60697d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 'op '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.77443d0
      vclocal(1) =    272.96562d0*CON_KCALTOK
      vclocal(2) =   -448.60990d0*CON_KCALTOK
      vclocal(3) =    594.46579d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 's  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.71456d0
      vclocal(1) =    349.94694d0*CON_KCALTOK
      vclocal(2) =   -438.37938d0*CON_KCALTOK
      vclocal(3) =    319.53738d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 's" '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.73492d0
      vclocal(1) =    287.33122d0*CON_KCALTOK
      vclocal(2) =   -383.33133d0*CON_KCALTOK
      vclocal(3) =    885.62025d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cp '
      vnlocal(nsame,2) = 'sp '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.47424d0
      vclocal(1) =    404.35389d0*CON_KCALTOK
      vclocal(2) =   -685.60397d0*CON_KCALTOK
      vclocal(3) =    255.93191d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cpb'
      vnlocal(nsame,2) = 'cpb'
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.36165d0
      vclocal(1) =    562.35033d0*CON_KCALTOK
      vclocal(2) =   -759.20705d0*CON_KCALTOK
      vclocal(3) =    379.19691d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cpb'
      vnlocal(nsame,2) = 'nh '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.38286d0
      vclocal(1) =    515.90524d0*CON_KCALTOK
      vclocal(2) =   -445.31769d0*CON_KCALTOK
      vclocal(3) =   2813.44920d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cpb'
      vnlocal(nsame,2) = 'nn '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.33345d0
      vclocal(1) =    576.22904d0*CON_KCALTOK
      vclocal(2) =  -1152.12840d0*CON_KCALTOK
      vclocal(3) =   1685.64660d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cpb'
      vnlocal(nsame,2) = 'np '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.35843d0
      vclocal(1) =    470.01045d0*CON_KCALTOK
      vclocal(2) =   -560.19556d0*CON_KCALTOK
      vclocal(3) =    807.60697d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cpb'
      vnlocal(nsame,2) = 'op '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.73492d0
      vclocal(1) =    287.33122d0*CON_KCALTOK
      vclocal(2) =   -383.33133d0*CON_KCALTOK
      vclocal(3) =    885.62025d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'cpb'
      vnlocal(nsame,2) = 'sp '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.18754d0
      vclocal(1) =   1447.35240d0*CON_KCALTOK
      vclocal(2) =  -2692.58780d0*CON_KCALTOK
      vclocal(3) =   3266.72060d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'ct '
      vnlocal(nsame,2) = 'ct '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.05622d0
      vclocal(1) =    516.65405d0*CON_KCALTOK
      vclocal(2) =   -992.21427d0*CON_KCALTOK
      vclocal(3) =   1241.06290d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'ct '
      vnlocal(nsame,2) = 'h  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.32596d0
      vclocal(1) =    352.53248d0*CON_KCALTOK
      vclocal(2) =   -640.03642d0*CON_KCALTOK
      vclocal(3) =    718.54703d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'h  '
      vnlocal(nsame,2) = 's  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.33041d0
      vclocal(1) =    341.86018d0*CON_KCALTOK
      vclocal(2) =   -654.82317d0*CON_KCALTOK
      vclocal(3) =    627.78457d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'h  '
      vnlocal(nsame,2) = 's" '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.00135d0
      vclocal(1) =    576.24337d0*CON_KCALTOK
      vclocal(2) =  -1309.47650d0*CON_KCALTOK
      vclocal(3) =   1818.25150d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'h  '
      vnlocal(nsame,2) = 'n  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.01652d0
      vclocal(1) =    502.45649d0*CON_KCALTOK
      vclocal(2) =  -1179.60880d0*CON_KCALTOK
      vclocal(3) =   1580.56380d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'h  '
      vnlocal(nsame,2) = 'n= '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.00139d0
      vclocal(1) =    569.19520d0*CON_KCALTOK
      vclocal(2) =  -1309.27050d0*CON_KCALTOK
      vclocal(3) =   1525.73850d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'h  '
      vnlocal(nsame,2) = 'na '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.00151d0
      vclocal(1) =    571.29631d0*CON_KCALTOK
      vclocal(2) =  -1300.78440d0*CON_KCALTOK
      vclocal(3) =   1908.34210d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'h  '
      vnlocal(nsame,2) = 'nh '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.00151d0
      vclocal(1) =    571.29631d0*CON_KCALTOK
      vclocal(2) =  -1300.78440d0*CON_KCALTOK
      vclocal(3) =   1908.34210d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'h  '
      vnlocal(nsame,2) = 'nn '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      0.94653d0
      vclocal(1) =    682.70024d0*CON_KCALTOK
      vclocal(2) =  -1644.75020d0*CON_KCALTOK
      vclocal(3) =   2570.21260d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'h  '
      vnlocal(nsame,2) = 'o  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      0.97000d0
      vclocal(1) =    563.28000d0*CON_KCALTOK
      vclocal(2) =  -1428.22000d0*CON_KCALTOK
      vclocal(3) =   1902.12000d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'h  '
      vnlocal(nsame,2) = 'o? '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.01192d0
      vclocal(1) =    538.24267d0*CON_KCALTOK
      vclocal(2) =  -1199.19730d0*CON_KCALTOK
      vclocal(3) =   1737.65800d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'h+ '
      vnlocal(nsame,2) = 'n+ '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.65255d0
      vclocal(1) =    424.44931d0*CON_KCALTOK
      vclocal(2) =   -673.86653d0*CON_KCALTOK
      vclocal(3) =    694.46086d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'n  '
      vnlocal(nsame,2) = 's" '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.19480d0
      vclocal(1) =    976.73770d0*CON_KCALTOK
      vclocal(2) =  -2500.08440d0*CON_KCALTOK
      vclocal(3) =   3672.43750d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'n` '
      vnlocal(nsame,2) = 'o` '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.21921d0
      vclocal(1) =   1077.85210d0*CON_KCALTOK
      vclocal(2) =  -2231.92700d0*CON_KCALTOK
      vclocal(3) =   2681.53480d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'n  '
      vnlocal(nsame,2) = 'n  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.43587d0
      vclocal(1) =    407.59874d0*CON_KCALTOK
      vclocal(2) =   -492.66272d0*CON_KCALTOK
      vclocal(3) =   1146.29960d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'na '
      vnlocal(nsame,2) = 'na '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.28726d0
      vclocal(1) =    636.16453d0*CON_KCALTOK
      vclocal(2) =  -1040.20600d0*CON_KCALTOK
      vclocal(3) =    965.30103d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'nh '
      vnlocal(nsame,2) = 'np '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.26965d0
      vclocal(1) =    739.27311d0*CON_KCALTOK
      vclocal(2) =  -1662.03650d0*CON_KCALTOK
      vclocal(3) =   3047.70240d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'np '
      vnlocal(nsame,2) = 'np '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.36115d0
      vclocal(1) =    473.78075d0*CON_KCALTOK
      vclocal(2) =   -636.05190d0*CON_KCALTOK
      vclocal(3) =    837.50000d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'np '
      vnlocal(nsame,2) = 'op '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.64860d0
      vclocal(1) =    364.49484d0*CON_KCALTOK
      vclocal(2) =   -545.62257d0*CON_KCALTOK
      vclocal(3) =    650.00000d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'np '
      vnlocal(nsame,2) = 'sp '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.63549d0
      vclocal(1) =    398.67842d0*CON_KCALTOK
      vclocal(2) =   -622.23348d0*CON_KCALTOK
      vclocal(3) =    826.84440d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'o  '
      vnlocal(nsame,2) = 'p  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.69973d0
      vclocal(1) =    341.95620d0*CON_KCALTOK
      vclocal(2) =   -557.68919d0*CON_KCALTOK
      vclocal(3) =    360.06776d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'o  '
      vnlocal(nsame,2) = 'p- '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.43574d0
      vclocal(1) =    814.84012d0*CON_KCALTOK
      vclocal(2) =  -1897.21980d0*CON_KCALTOK
      vclocal(3) =   2627.93700d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'o` '
      vnlocal(nsame,2) = 's" '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.47036d0
      vclocal(1) =    785.88168d0*CON_KCALTOK
      vclocal(2) =  -1629.62140d0*CON_KCALTOK
      vclocal(3) =   2056.56230d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'o- '
      vnlocal(nsame,2) = 'p  '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      1.49681d0
      vclocal(1) =    683.42789d0*CON_KCALTOK
      vclocal(2) =  -1401.99000d0*CON_KCALTOK
      vclocal(3) =   1933.74420d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 'o- '
      vnlocal(nsame,2) = 'p- '
      call twh_set_vibnames(type,nsame,vnlocal)

      type = type + 1
      call twh_vibff(GLB_SET,type,ffname)
      call twh_vibstyle(GLB_SET,type,6)
      vclocal(0) =      2.08189d0
      vclocal(1) =    212.21289d0*CON_KCALTOK
      vclocal(2) =   -102.04010d0*CON_KCALTOK
      vclocal(3) =    820.35444d0*CON_KCALTOK
      call twh_set_vibcoeffs(type,vclocal)
      nsame = 0
      nsame = nsame + 1
      vnlocal(nsame,1) = 's  '
      vnlocal(nsame,2) = 's  '
      call twh_set_vibnames(type,nsame,vnlocal)

      nbondtypes = type
      call twh_ffcheck(lfailure,1,type)
      if ( lfailure ) return

      return
      end

      subroutine twh_ffqmff_viii_angle(lfailure,ffname,nangletypes)
c     ******************************************************************
c     * sets the angle and bond-angle terms for QMFF-VIII              *
c     *                                                                *
c     * originally written 05-05-2006 by M.G. Martin                   *
c     * last modified 08-04-2011 by M.G. Martin                        *
c     ******************************************************************
      implicit none
c     --- variables passed to/from the subroutine
      character*(FFNAMELEN) ffname
      logical lfailure
      integer nangletypes
c     --- local variables
      integer type,nsame
      double precision bclocal
      dimension bclocal(MINBENDCOEFF:MAXBENDCOEFF)
      character*(FFNAMELEN) bnlocal
      dimension bnlocal(MAXSAMEB,MAXBNAME)

      type = 0

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.67330d0
      bclocal(1) =     49.33332d0*CON_KCALTOK
      bclocal(2) =     -9.29214d0*CON_KCALTOK
      bclocal(3) =    -11.93331d0*CON_KCALTOK
      bclocal(4) =      9.32093d0*CON_KCALTOK
      bclocal(5) =      9.32093d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      0.00000d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'c  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.31040d0
      bclocal(1) =     51.71700d0*CON_KCALTOK
      bclocal(2) =     -9.87345d0*CON_KCALTOK
      bclocal(3) =    -10.98580d0*CON_KCALTOK
      bclocal(4) =     20.58096d0*CON_KCALTOK
      bclocal(5) =     15.98888d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     18.80034d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'c" '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.60960d0
      bclocal(1) =     39.98473d0*CON_KCALTOK
      bclocal(2) =     -8.00025d0*CON_KCALTOK
      bclocal(3) =     -8.34042d0*CON_KCALTOK
      bclocal(4) =     26.10646d0*CON_KCALTOK
      bclocal(5) =     22.67961d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     26.09970d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'c` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    104.49140d0
      bclocal(1) =     37.35106d0*CON_KCALTOK
      bclocal(2) =     -5.24080d0*CON_KCALTOK
      bclocal(3) =     -7.77036d0*CON_KCALTOK
      bclocal(4) =     33.45471d0*CON_KCALTOK
      bclocal(5) =     27.76641d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     23.86221d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'c- '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.76270d0
      bclocal(1) =     57.05687d0*CON_KCALTOK
      bclocal(2) =    -13.28287d0*CON_KCALTOK
      bclocal(3) =    -12.37473d0*CON_KCALTOK
      bclocal(4) =     18.58349d0*CON_KCALTOK
      bclocal(5) =     22.06727d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      9.04967d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'c= '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.67330d0
      bclocal(1) =     49.33332d0*CON_KCALTOK
      bclocal(2) =     -9.29214d0*CON_KCALTOK
      bclocal(3) =    -11.93331d0*CON_KCALTOK
      bclocal(4) =     29.30599d0*CON_KCALTOK
      bclocal(5) =     35.70267d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     27.78116d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'co '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.87683d0
      bclocal(1) =     46.38439d0*CON_KCALTOK
      bclocal(2) =     -9.57236d0*CON_KCALTOK
      bclocal(3) =     -9.85164d0*CON_KCALTOK
      bclocal(4) =     20.68617d0*CON_KCALTOK
      bclocal(5) =     38.43762d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     16.82884d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.40156d0
      bclocal(1) =     71.68820d0*CON_KCALTOK
      bclocal(2) =    -15.99149d0*CON_KCALTOK
      bclocal(3) =    -15.33785d0*CON_KCALTOK
      bclocal(4) =     11.88440d0*CON_KCALTOK
      bclocal(5) =     41.86683d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     16.81443d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'ct '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.77000d0
      bclocal(1) =     51.75238d0*CON_KCALTOK
      bclocal(2) =    -13.23853d0*CON_KCALTOK
      bclocal(3) =      6.40332d0*CON_KCALTOK
      bclocal(4) =     24.13337d0*CON_KCALTOK
      bclocal(5) =     13.28037d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      3.93858d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    116.11920d0
      bclocal(1) =     60.52110d0*CON_KCALTOK
      bclocal(2) =    -16.13980d0*CON_KCALTOK
      bclocal(3) =    -13.27676d0*CON_KCALTOK
      bclocal(4) =     20.34787d0*CON_KCALTOK
      bclocal(5) =     59.50808d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     24.15187d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.06674d0
      bclocal(1) =     75.18900d0*CON_KCALTOK
      bclocal(2) =    -14.93687d0*CON_KCALTOK
      bclocal(3) =    -15.99325d0*CON_KCALTOK
      bclocal(4) =     16.16399d0*CON_KCALTOK
      bclocal(5) =     72.08106d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     20.94066d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'n+ '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.01469d0
      bclocal(1) =     67.14361d0*CON_KCALTOK
      bclocal(2) =    -16.57412d0*CON_KCALTOK
      bclocal(3) =      0.00000d0*CON_KCALTOK
      bclocal(4) =     40.12299d0*CON_KCALTOK
      bclocal(5) =     55.56789d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     47.57650d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'n= '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.91000d0
      bclocal(1) =     75.79865d0*CON_KCALTOK
      bclocal(2) =    -16.64995d0*CON_KCALTOK
      bclocal(3) =    -16.32768d0*CON_KCALTOK
      bclocal(4) =      7.07855d0*CON_KCALTOK
      bclocal(5) =     19.26768d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      5.37412d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'na '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    114.30180d0
      bclocal(1) =     50.78439d0*CON_KCALTOK
      bclocal(2) =    -12.55520d0*CON_KCALTOK
      bclocal(3) =    -11.10041d0*CON_KCALTOK
      bclocal(4) =     -7.94059d0*CON_KCALTOK
      bclocal(5) =      6.67121d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      5.13708d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    114.30180d0
      bclocal(1) =     50.78439d0*CON_KCALTOK
      bclocal(2) =    -12.55520d0*CON_KCALTOK
      bclocal(3) =    -11.10041d0*CON_KCALTOK
      bclocal(4) =     -7.94059d0*CON_KCALTOK
      bclocal(5) =      6.67121d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      5.13708d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'nn '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.88660d0
      bclocal(1) =     68.08755d0*CON_KCALTOK
      bclocal(2) =    -10.44224d0*CON_KCALTOK
      bclocal(3) =    -16.33432d0*CON_KCALTOK
      bclocal(4) =      3.12419d0*CON_KCALTOK
      bclocal(5) =     23.72473d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     13.29279d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.94106d0
      bclocal(1) =     56.36328d0*CON_KCALTOK
      bclocal(2) =    -12.32061d0*CON_KCALTOK
      bclocal(3) =    -12.06504d0*CON_KCALTOK
      bclocal(4) =      4.09206d0*CON_KCALTOK
      bclocal(5) =     28.12382d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     17.19728d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 's  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    103.01330d0
      bclocal(1) =     39.86377d0*CON_KCALTOK
      bclocal(2) =     -4.99710d0*CON_KCALTOK
      bclocal(3) =     -8.25058d0*CON_KCALTOK
      bclocal(4) =      4.07784d0*CON_KCALTOK
      bclocal(5) =     29.52654d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     31.59583d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 's" '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    107.85940d0
      bclocal(1) =     47.54466d0*CON_KCALTOK
      bclocal(2) =    -21.85686d0*CON_KCALTOK
      bclocal(3) =      0.00000d0*CON_KCALTOK
      bclocal(4) =     18.13813d0*CON_KCALTOK
      bclocal(5) =     17.01012d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      2.61887d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c" '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    100.56630d0
      bclocal(1) =     62.01977d0*CON_KCALTOK
      bclocal(2) =     -6.26696d0*CON_KCALTOK
      bclocal(3) =    -12.74348d0*CON_KCALTOK
      bclocal(4) =     -7.48412d0*CON_KCALTOK
      bclocal(5) =     25.20899d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -5.55845d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c" '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    106.17640d0
      bclocal(1) =     92.90171d0*CON_KCALTOK
      bclocal(2) =    -15.73263d0*CON_KCALTOK
      bclocal(3) =    -60.90512d0*CON_KCALTOK
      bclocal(4) =    -14.12609d0*CON_KCALTOK
      bclocal(5) =      7.62521d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      7.23383d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c" '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    108.52950d0
      bclocal(1) =     61.87459d0*CON_KCALTOK
      bclocal(2) =    -11.29179d0*CON_KCALTOK
      bclocal(3) =    -13.09342d0*CON_KCALTOK
      bclocal(4) =     23.00836d0*CON_KCALTOK
      bclocal(5) =     26.33007d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      7.85491d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c` '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'co '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    108.44949d0
      bclocal(1) =     56.68412d0*CON_KCALTOK
      bclocal(2) =    -10.17170d0*CON_KCALTOK
      bclocal(3) =    -11.84576d0*CON_KCALTOK
      bclocal(4) =     26.97550d0*CON_KCALTOK
      bclocal(5) =     20.17597d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     12.39727d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c` '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.48634d0
      bclocal(1) =     83.32298d0*CON_KCALTOK
      bclocal(2) =    -17.65044d0*CON_KCALTOK
      bclocal(3) =    -17.81593d0*CON_KCALTOK
      bclocal(4) =     27.39694d0*CON_KCALTOK
      bclocal(5) =     75.48290d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     29.24749d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c` '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    100.56630d0
      bclocal(1) =     62.01977d0*CON_KCALTOK
      bclocal(2) =     -6.26696d0*CON_KCALTOK
      bclocal(3) =    -12.74348d0*CON_KCALTOK
      bclocal(4) =     -7.48412d0*CON_KCALTOK
      bclocal(5) =     25.20899d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -5.55845d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c` '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'nn '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    106.17640d0
      bclocal(1) =     92.90171d0*CON_KCALTOK
      bclocal(2) =    -15.73263d0*CON_KCALTOK
      bclocal(3) =    -60.90512d0*CON_KCALTOK
      bclocal(4) =    -14.12609d0*CON_KCALTOK
      bclocal(5) =      7.62521d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      7.23383d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c` '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.67080d0
      bclocal(1) =     45.14111d0*CON_KCALTOK
      bclocal(2) =     -8.79486d0*CON_KCALTOK
      bclocal(3) =     -9.60640d0*CON_KCALTOK
      bclocal(4) =     17.19578d0*CON_KCALTOK
      bclocal(5) =     21.68856d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -1.69573d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c- '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =     99.14982d0
      bclocal(1) =     75.20999d0*CON_KCALTOK
      bclocal(2) =     -6.53977d0*CON_KCALTOK
      bclocal(3) =    -15.35690d0*CON_KCALTOK
      bclocal(4) =    -17.56974d0*CON_KCALTOK
      bclocal(5) =     36.95922d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      2.37539d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c- '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'n+ '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    113.00900d0
      bclocal(1) =     55.21236d0*CON_KCALTOK
      bclocal(2) =    -12.81934d0*CON_KCALTOK
      bclocal(3) =    -11.97077d0*CON_KCALTOK
      bclocal(4) =      9.56577d0*CON_KCALTOK
      bclocal(5) =      9.56577d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      3.92208d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c= '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'c= '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.05660d0
      bclocal(1) =     51.53359d0*CON_KCALTOK
      bclocal(2) =    -17.84811d0*CON_KCALTOK
      bclocal(3) =      6.52047d0*CON_KCALTOK
      bclocal(4) =     24.27522d0*CON_KCALTOK
      bclocal(5) =     16.59777d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     11.61888d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c= '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    108.05620d0
      bclocal(1) =    119.17813d0*CON_KCALTOK
      bclocal(2) =    -21.11503d0*CON_KCALTOK
      bclocal(3) =    -25.12576d0*CON_KCALTOK
      bclocal(4) =     75.57630d0*CON_KCALTOK
      bclocal(5) =     75.57630d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     70.51203d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cl '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'cl '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.84480d0
      bclocal(1) =     73.62862d0*CON_KCALTOK
      bclocal(2) =    -15.16498d0*CON_KCALTOK
      bclocal(3) =    -15.63140d0*CON_KCALTOK
      bclocal(4) =     48.13443d0*CON_KCALTOK
      bclocal(5) =     40.38681d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     45.31126d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cl '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.77000d0
      bclocal(1) =     51.75238d0*CON_KCALTOK
      bclocal(2) =    -13.23853d0*CON_KCALTOK
      bclocal(3) =      6.40332d0*CON_KCALTOK
      bclocal(4) =     38.36982d0*CON_KCALTOK
      bclocal(5) =     17.06951d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     11.21324d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'co '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    114.30180d0
      bclocal(1) =     50.78439d0*CON_KCALTOK
      bclocal(2) =    -12.55520d0*CON_KCALTOK
      bclocal(3) =    -11.10041d0*CON_KCALTOK
      bclocal(4) =     -7.94059d0*CON_KCALTOK
      bclocal(5) =      6.67121d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      5.13708d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'co '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.86480d0
      bclocal(1) =     69.17548d0*CON_KCALTOK
      bclocal(2) =     -5.55519d0*CON_KCALTOK
      bclocal(3) =    -16.60479d0*CON_KCALTOK
      bclocal(4) =     45.61526d0*CON_KCALTOK
      bclocal(5) =     67.35950d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     49.33756d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'co '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.87683d0
      bclocal(1) =     46.38439d0*CON_KCALTOK
      bclocal(2) =     -9.57236d0*CON_KCALTOK
      bclocal(3) =     -9.85164d0*CON_KCALTOK
      bclocal(4) =     38.43762d0*CON_KCALTOK
      bclocal(5) =     38.43762d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     16.82884d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    113.15560d0
      bclocal(1) =     64.46562d0*CON_KCALTOK
      bclocal(2) =    -14.96774d0*CON_KCALTOK
      bclocal(3) =    -13.88846d0*CON_KCALTOK
      bclocal(4) =     26.54816d0*CON_KCALTOK
      bclocal(5) =     20.20154d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      3.13208d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'ct '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.72160d0
      bclocal(1) =     97.17380d0*CON_KCALTOK
      bclocal(2) =    -21.10107d0*CON_KCALTOK
      bclocal(3) =    -20.87212d0*CON_KCALTOK
      bclocal(4) =     10.74452d0*CON_KCALTOK
      bclocal(5) =     75.82548d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     37.76506d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'f  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.64559d0
      bclocal(1) =     53.89350d0*CON_KCALTOK
      bclocal(2) =    -10.31145d0*CON_KCALTOK
      bclocal(3) =    -11.27739d0*CON_KCALTOK
      bclocal(4) =     39.14668d0*CON_KCALTOK
      bclocal(5) =     19.00083d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     13.31149d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.77570d0
      bclocal(1) =     62.95977d0*CON_KCALTOK
      bclocal(2) =    -12.15042d0*CON_KCALTOK
      bclocal(3) =    -13.20120d0*CON_KCALTOK
      bclocal(4) =     45.46574d0*CON_KCALTOK
      bclocal(5) =     16.09545d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     16.24327d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'ct '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    107.12750d0
      bclocal(1) =    138.64696d0*CON_KCALTOK
      bclocal(2) =    -23.12969d0*CON_KCALTOK
      bclocal(3) =    -29.01548d0*CON_KCALTOK
      bclocal(4) =    105.71680d0*CON_KCALTOK
      bclocal(5) =    105.71680d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    124.55792d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'f  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'f  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    107.66290d0
      bclocal(1) =     49.49007d0*CON_KCALTOK
      bclocal(2) =    -16.13105d0*CON_KCALTOK
      bclocal(3) =     -3.03603d0*CON_KCALTOK
      bclocal(4) =     21.05050d0*CON_KCALTOK
      bclocal(5) =     21.05050d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      6.19953d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.09619d0
      bclocal(1) =     72.10056d0*CON_KCALTOK
      bclocal(2) =    -13.57880d0*CON_KCALTOK
      bclocal(3) =    -15.27807d0*CON_KCALTOK
      bclocal(4) =     14.72555d0*CON_KCALTOK
      bclocal(5) =     56.84823d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     19.69139d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    105.12703d0
      bclocal(1) =     71.10819d0*CON_KCALTOK
      bclocal(2) =    -10.42399d0*CON_KCALTOK
      bclocal(3) =    -14.80588d0*CON_KCALTOK
      bclocal(4) =      7.68072d0*CON_KCALTOK
      bclocal(5) =     56.73837d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -2.27640d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'n+ '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    107.78549d0
      bclocal(1) =     64.15919d0*CON_KCALTOK
      bclocal(2) =     -6.68651d0*CON_KCALTOK
      bclocal(3) =     13.99578d0*CON_KCALTOK
      bclocal(4) =      9.58351d0*CON_KCALTOK
      bclocal(5) =     48.30987d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     14.53499d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'n= '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.62040d0
      bclocal(1) =     64.06204d0*CON_KCALTOK
      bclocal(2) =     -8.38928d0*CON_KCALTOK
      bclocal(3) =     -3.24633d0*CON_KCALTOK
      bclocal(4) =     15.64911d0*CON_KCALTOK
      bclocal(5) =     49.34091d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     14.44882d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'na '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    108.93717d0
      bclocal(1) =     68.33446d0*CON_KCALTOK
      bclocal(2) =      3.49686d0*CON_KCALTOK
      bclocal(3) =      0.00000d0*CON_KCALTOK
      bclocal(4) =     15.49523d0*CON_KCALTOK
      bclocal(5) =     50.56625d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     22.17308d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    108.93717d0
      bclocal(1) =     68.33446d0*CON_KCALTOK
      bclocal(2) =      3.49686d0*CON_KCALTOK
      bclocal(3) =      0.00000d0*CON_KCALTOK
      bclocal(4) =     15.49523d0*CON_KCALTOK
      bclocal(5) =     50.56625d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     22.17308d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'nn '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    108.72800d0
      bclocal(1) =     73.08940d0*CON_KCALTOK
      bclocal(2) =    -13.49419d0*CON_KCALTOK
      bclocal(3) =    -15.48137d0*CON_KCALTOK
      bclocal(4) =      5.37087d0*CON_KCALTOK
      bclocal(5) =     64.33374d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     26.97426d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    107.89356d0
      bclocal(1) =     57.11693d0*CON_KCALTOK
      bclocal(2) =    -10.01529d0*CON_KCALTOK
      bclocal(3) =    -12.02598d0*CON_KCALTOK
      bclocal(4) =      6.83209d0*CON_KCALTOK
      bclocal(5) =     50.70975d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      8.85947d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 's  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    106.40520d0
      bclocal(1) =     53.43819d0*CON_KCALTOK
      bclocal(2) =     -8.55233d0*CON_KCALTOK
      bclocal(3) =    -11.20101d0*CON_KCALTOK
      bclocal(4) =     16.04687d0*CON_KCALTOK
      bclocal(5) =     24.37547d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      3.20331d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 's" '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.80918d0
      bclocal(1) =     89.75792d0*CON_KCALTOK
      bclocal(2) =    -69.56963d0*CON_KCALTOK
      bclocal(3) =    125.20346d0*CON_KCALTOK
      bclocal(4) =     62.61968d0*CON_KCALTOK
      bclocal(5) =     62.61968d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     30.20615d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'n= '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'n= '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.43520d0
      bclocal(1) =    116.10309d0*CON_KCALTOK
      bclocal(2) =    -23.53009d0*CON_KCALTOK
      bclocal(3) =    -24.73995d0*CON_KCALTOK
      bclocal(4) =     72.77172d0*CON_KCALTOK
      bclocal(5) =     72.77172d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -8.27109d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'o  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.09671d0
      bclocal(1) =     67.33503d0*CON_KCALTOK
      bclocal(2) =    -14.85868d0*CON_KCALTOK
      bclocal(3) =    -14.44796d0*CON_KCALTOK
      bclocal(4) =     11.87796d0*CON_KCALTOK
      bclocal(5) =     11.87796d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -2.84883d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 's  '
      bnlocal(nsame,2) = 'c  '
      bnlocal(nsame,3) = 's  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.58840d0
      bclocal(1) =     40.73110d0*CON_KCALTOK
      bclocal(2) =     -8.34581d0*CON_KCALTOK
      bclocal(3) =     -8.70963d0*CON_KCALTOK
      bclocal(4) =     15.99942d0*CON_KCALTOK
      bclocal(5) =     15.99942d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c" '
      bnlocal(nsame,3) = 'c  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    105.78990d0
      bclocal(1) =     32.33862d0*CON_KCALTOK
      bclocal(2) =     -2.27647d0*CON_KCALTOK
      bclocal(3) =     17.12295d0*CON_KCALTOK
      bclocal(4) =     11.79116d0*CON_KCALTOK
      bclocal(5) =      5.67417d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -7.93001d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c" '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    116.92570d0
      bclocal(1) =     46.92773d0*CON_KCALTOK
      bclocal(2) =    -13.08867d0*CON_KCALTOK
      bclocal(3) =    -10.44437d0*CON_KCALTOK
      bclocal(4) =     36.76978d0*CON_KCALTOK
      bclocal(5) =      8.81202d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -9.38950d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c" '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    100.31820d0
      bclocal(1) =     45.72125d0*CON_KCALTOK
      bclocal(2) =     -4.50863d0*CON_KCALTOK
      bclocal(3) =     -9.38846d0*CON_KCALTOK
      bclocal(4) =      1.56223d0*CON_KCALTOK
      bclocal(5) =      5.46251d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     22.21727d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c" '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    119.77920d0
      bclocal(1) =     76.59017d0*CON_KCALTOK
      bclocal(2) =    -21.14898d0*CON_KCALTOK
      bclocal(3) =      0.00000d0*CON_KCALTOK
      bclocal(4) =     37.02965d0*CON_KCALTOK
      bclocal(5) =     54.25736d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     90.13962d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c" '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    113.11680d0
      bclocal(1) =     69.22291d0*CON_KCALTOK
      bclocal(2) =    -16.10624d0*CON_KCALTOK
      bclocal(3) =    -14.96994d0*CON_KCALTOK
      bclocal(4) =     63.75480d0*CON_KCALTOK
      bclocal(5) =     35.39382d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     54.64805d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'c" '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    115.78727d0
      bclocal(1) =     62.96260d0*CON_KCALTOK
      bclocal(2) =    -16.42769d0*CON_KCALTOK
      bclocal(3) =    -13.68753d0*CON_KCALTOK
      bclocal(4) =     40.66383d0*CON_KCALTOK
      bclocal(5) =     81.08685d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    116.76991d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'c" '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    107.35680d0
      bclocal(1) =     50.03298d0*CON_KCALTOK
      bclocal(2) =     -8.50673d0*CON_KCALTOK
      bclocal(3) =    -10.53045d0*CON_KCALTOK
      bclocal(4) =     10.78789d0*CON_KCALTOK
      bclocal(5) =     10.78789d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -1.35419d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c" '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.39350d0
      bclocal(1) =     62.10086d0*CON_KCALTOK
      bclocal(2) =    -22.00537d0*CON_KCALTOK
      bclocal(3) =      0.00000d0*CON_KCALTOK
      bclocal(4) =     32.35553d0*CON_KCALTOK
      bclocal(5) =     45.37663d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      4.05798d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c" '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =     92.02086d0
      bclocal(1) =     38.08841d0*CON_KCALTOK
      bclocal(2) =     -0.72631d0*CON_KCALTOK
      bclocal(3) =     -7.72222d0*CON_KCALTOK
      bclocal(4) =      8.97785d0*CON_KCALTOK
      bclocal(5) =     23.12459d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     38.50132d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c" '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    116.03810d0
      bclocal(1) =     70.16703d0*CON_KCALTOK
      bclocal(2) =    -25.78580d0*CON_KCALTOK
      bclocal(3) =      0.00000d0*CON_KCALTOK
      bclocal(4) =     14.50385d0*CON_KCALTOK
      bclocal(5) =     74.34365d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     84.56667d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c" '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    122.52920d0
      bclocal(1) =    123.91159d0*CON_KCALTOK
      bclocal(2) =    -43.72802d0*CON_KCALTOK
      bclocal(3) =    -28.88287d0*CON_KCALTOK
      bclocal(4) =     98.65779d0*CON_KCALTOK
      bclocal(5) =     98.65779d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     37.61300d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'n  '
      bnlocal(nsame,2) = 'c" '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    124.54200d0
      bclocal(1) =    115.20474d0*CON_KCALTOK
      bclocal(2) =    -41.04756d0*CON_KCALTOK
      bclocal(3) =    -13.31795d0*CON_KCALTOK
      bclocal(4) =     90.88749d0*CON_KCALTOK
      bclocal(5) =     75.94857d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    200.71797d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'n  '
      bnlocal(nsame,2) = 'c" '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    103.71952d0
      bclocal(1) =    111.50659d0*CON_KCALTOK
      bclocal(2) =    -14.70447d0*CON_KCALTOK
      bclocal(3) =    -23.02843d0*CON_KCALTOK
      bclocal(4) =     65.20315d0*CON_KCALTOK
      bclocal(5) =     65.20315d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    153.76230d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'o  '
      bnlocal(nsame,2) = 'c" '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    118.98550d0
      bclocal(1) =    116.09572d0*CON_KCALTOK
      bclocal(2) =    -26.17467d0*CON_KCALTOK
      bclocal(3) =     12.19682d0*CON_KCALTOK
      bclocal(4) =     66.39385d0*CON_KCALTOK
      bclocal(5) =     92.38333d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    244.39682d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'o  '
      bnlocal(nsame,2) = 'c" '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.58840d0
      bclocal(1) =     40.73110d0*CON_KCALTOK
      bclocal(2) =     -8.34581d0*CON_KCALTOK
      bclocal(3) =     -8.70963d0*CON_KCALTOK
      bclocal(4) =     15.99942d0*CON_KCALTOK
      bclocal(5) =     15.99942d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c` '
      bnlocal(nsame,3) = 'c  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    105.78990d0
      bclocal(1) =     32.33862d0*CON_KCALTOK
      bclocal(2) =     -2.27647d0*CON_KCALTOK
      bclocal(3) =     17.12295d0*CON_KCALTOK
      bclocal(4) =     11.79116d0*CON_KCALTOK
      bclocal(5) =      5.67417d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -7.93001d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c` '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    116.47839d0
      bclocal(1) =     47.18995d0*CON_KCALTOK
      bclocal(2) =    -13.13954d0*CON_KCALTOK
      bclocal(3) =    -10.46449d0*CON_KCALTOK
      bclocal(4) =     52.66651d0*CON_KCALTOK
      bclocal(5) =     58.78738d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     44.75129d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c` '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    100.31820d0
      bclocal(1) =     45.72125d0*CON_KCALTOK
      bclocal(2) =     -4.50863d0*CON_KCALTOK
      bclocal(3) =     -9.38846d0*CON_KCALTOK
      bclocal(4) =      1.56223d0*CON_KCALTOK
      bclocal(5) =      5.46251d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     22.21727d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c` '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    121.21019d0
      bclocal(1) =     54.10131d0*CON_KCALTOK
      bclocal(2) =    -18.06801d0*CON_KCALTOK
      bclocal(3) =    -12.43849d0*CON_KCALTOK
      bclocal(4) =     37.03361d0*CON_KCALTOK
      bclocal(5) =     80.14035d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     72.04957d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c` '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    123.14510d0
      bclocal(1) =     66.12279d0*CON_KCALTOK
      bclocal(2) =    -20.49084d0*CON_KCALTOK
      bclocal(3) =      0.16046d0*CON_KCALTOK
      bclocal(4) =     50.72197d0*CON_KCALTOK
      bclocal(5) =     53.81127d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     66.76599d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'co '
      bnlocal(nsame,2) = 'c` '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    106.92088d0
      bclocal(1) =     39.80700d0*CON_KCALTOK
      bclocal(2) =     -6.45198d0*CON_KCALTOK
      bclocal(3) =     -8.19264d0*CON_KCALTOK
      bclocal(4) =     59.45699d0*CON_KCALTOK
      bclocal(5) =     46.67251d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     32.96163d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'c` '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    117.58731d0
      bclocal(1) =     67.61201d0*CON_KCALTOK
      bclocal(2) =    -19.24031d0*CON_KCALTOK
      bclocal(3) =    -14.98500d0*CON_KCALTOK
      bclocal(4) =     54.73928d0*CON_KCALTOK
      bclocal(5) =     59.19983d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    110.73481d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'c` '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    107.35680d0
      bclocal(1) =     50.03298d0*CON_KCALTOK
      bclocal(2) =     -8.50673d0*CON_KCALTOK
      bclocal(3) =    -10.53045d0*CON_KCALTOK
      bclocal(4) =     10.78789d0*CON_KCALTOK
      bclocal(5) =     10.78789d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -1.35419d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c` '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.52304d0
      bclocal(1) =     55.23517d0*CON_KCALTOK
      bclocal(2) =    -10.62720d0*CON_KCALTOK
      bclocal(3) =    -11.69568d0*CON_KCALTOK
      bclocal(4) =     29.92901d0*CON_KCALTOK
      bclocal(5) =     49.37186d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     24.98940d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c` '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =     92.02086d0
      bclocal(1) =     38.08841d0*CON_KCALTOK
      bclocal(2) =     -0.72631d0*CON_KCALTOK
      bclocal(3) =     -7.72222d0*CON_KCALTOK
      bclocal(4) =      8.97785d0*CON_KCALTOK
      bclocal(5) =     23.12459d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     38.50132d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c` '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    118.53193d0
      bclocal(1) =     61.41831d0*CON_KCALTOK
      bclocal(2) =    -18.24893d0*CON_KCALTOK
      bclocal(3) =    -13.74232d0*CON_KCALTOK
      bclocal(4) =     20.62556d0*CON_KCALTOK
      bclocal(5) =     84.84015d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     68.06060d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c` '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    122.52920d0
      bclocal(1) =    123.91159d0*CON_KCALTOK
      bclocal(2) =    -43.72802d0*CON_KCALTOK
      bclocal(3) =    -28.88287d0*CON_KCALTOK
      bclocal(4) =     98.65779d0*CON_KCALTOK
      bclocal(5) =     98.65779d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     81.83563d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'n  '
      bnlocal(nsame,2) = 'c` '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.41090d0
      bclocal(1) =     84.66801d0*CON_KCALTOK
      bclocal(2) =    -17.04707d0*CON_KCALTOK
      bclocal(3) =    -17.94491d0*CON_KCALTOK
      bclocal(4) =     41.64449d0*CON_KCALTOK
      bclocal(5) =     77.53399d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    110.75546d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'n  '
      bnlocal(nsame,2) = 'c` '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    123.70551d0
      bclocal(1) =    105.99296d0*CON_KCALTOK
      bclocal(2) =    -38.82368d0*CON_KCALTOK
      bclocal(3) =    -24.74854d0*CON_KCALTOK
      bclocal(4) =     75.76610d0*CON_KCALTOK
      bclocal(5) =    104.21592d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    221.27407d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'n  '
      bnlocal(nsame,2) = 'c` '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    118.98550d0
      bclocal(1) =    116.09572d0*CON_KCALTOK
      bclocal(2) =    -26.17467d0*CON_KCALTOK
      bclocal(3) =     12.19682d0*CON_KCALTOK
      bclocal(4) =     66.39385d0*CON_KCALTOK
      bclocal(5) =     92.38333d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    244.39682d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'o  '
      bnlocal(nsame,2) = 'c` '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    114.84087d0
      bclocal(1) =     63.58001d0*CON_KCALTOK
      bclocal(2) =    -15.97706d0*CON_KCALTOK
      bclocal(3) =    -13.81924d0*CON_KCALTOK
      bclocal(4) =     71.44427d0*CON_KCALTOK
      bclocal(5) =     90.06067d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     79.37687d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'c+ '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    107.66290d0
      bclocal(1) =     57.27177d0*CON_KCALTOK
      bclocal(2) =     -9.87577d0*CON_KCALTOK
      bclocal(3) =    -12.01332d0*CON_KCALTOK
      bclocal(4) =     -1.34132d0*CON_KCALTOK
      bclocal(5) =     -1.34132d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    -14.91972d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c+ '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.30000d0
      bclocal(1) =     66.14710d0*CON_KCALTOK
      bclocal(2) =    -14.78910d0*CON_KCALTOK
      bclocal(3) =    -14.24921d0*CON_KCALTOK
      bclocal(4) =     18.36769d0*CON_KCALTOK
      bclocal(5) =     94.38949d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      0.40321d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c+ '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    121.80000d0
      bclocal(1) =    104.03111d0*CON_KCALTOK
      bclocal(2) =    -35.64114d0*CON_KCALTOK
      bclocal(3) =    -24.08121d0*CON_KCALTOK
      bclocal(4) =    107.61866d0*CON_KCALTOK
      bclocal(5) =    107.61866d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    139.51596d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'n  '
      bnlocal(nsame,2) = 'c+ '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    115.06200d0
      bclocal(1) =     70.35181d0*CON_KCALTOK
      bclocal(2) =    -18.02690d0*CON_KCALTOK
      bclocal(3) =    -15.45470d0*CON_KCALTOK
      bclocal(4) =     74.75925d0*CON_KCALTOK
      bclocal(5) =     89.93618d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     83.87667d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c- '
      bnlocal(nsame,3) = 'o- '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.75340d0
      bclocal(1) =     72.80066d0*CON_KCALTOK
      bclocal(2) =    -16.68950d0*CON_KCALTOK
      bclocal(3) =    -15.75980d0*CON_KCALTOK
      bclocal(4) =     53.02320d0*CON_KCALTOK
      bclocal(5) =     87.38008d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     80.57331d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c- '
      bnlocal(nsame,3) = 'o- '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    130.01000d0
      bclocal(1) =    132.48810d0*CON_KCALTOK
      bclocal(2) =    -62.30840d0*CON_KCALTOK
      bclocal(3) =    -33.46050d0*CON_KCALTOK
      bclocal(4) =    106.30440d0*CON_KCALTOK
      bclocal(5) =    106.30440d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    241.43720d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'o- '
      bnlocal(nsame,2) = 'c- '
      bnlocal(nsame,3) = 'o- '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    122.01260d0
      bclocal(1) =     36.54230d0*CON_KCALTOK
      bclocal(2) =    -12.63106d0*CON_KCALTOK
      bclocal(3) =     -8.47760d0*CON_KCALTOK
      bclocal(4) =      5.76483d0*CON_KCALTOK
      bclocal(5) =      5.76483d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c= '
      bnlocal(nsame,3) = 'c  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    126.75940d0
      bclocal(1) =     48.69441d0*CON_KCALTOK
      bclocal(2) =    -30.80725d0*CON_KCALTOK
      bclocal(3) =      1.11735d0*CON_KCALTOK
      bclocal(4) =     36.73083d0*CON_KCALTOK
      bclocal(5) =     28.16887d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     20.68750d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c= '
      bnlocal(nsame,3) = 'c= '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    117.27190d0
      bclocal(1) =     37.57104d0*CON_KCALTOK
      bclocal(2) =    -10.09062d0*CON_KCALTOK
      bclocal(3) =    -10.83409d0*CON_KCALTOK
      bclocal(4) =     20.51768d0*CON_KCALTOK
      bclocal(5) =     17.84789d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      3.99929d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'c= '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    124.87820d0
      bclocal(1) =     39.19627d0*CON_KCALTOK
      bclocal(2) =    -19.74888d0*CON_KCALTOK
      bclocal(3) =     -1.80168d0*CON_KCALTOK
      bclocal(4) =     27.16141d0*CON_KCALTOK
      bclocal(5) =     22.16183d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     11.74963d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c= '
      bnlocal(nsame,2) = 'c= '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.89210d0
      bclocal(1) =     72.03217d0*CON_KCALTOK
      bclocal(2) =    -14.20172d0*CON_KCALTOK
      bclocal(3) =    -15.33961d0*CON_KCALTOK
      bclocal(4) =     45.07942d0*CON_KCALTOK
      bclocal(5) =     77.38613d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     38.96059d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'c= '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    127.75631d0
      bclocal(1) =     41.26839d0*CON_KCALTOK
      bclocal(2) =    -17.71816d0*CON_KCALTOK
      bclocal(3) =    -10.08284d0*CON_KCALTOK
      bclocal(4) =     43.82912d0*CON_KCALTOK
      bclocal(5) =     72.45798d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     73.41891d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'c= '
      bnlocal(nsame,3) = 'n= '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    115.49080d0
      bclocal(1) =     36.99910d0*CON_KCALTOK
      bclocal(2) =    -15.58713d0*CON_KCALTOK
      bclocal(3) =     -7.76752d0*CON_KCALTOK
      bclocal(4) =     20.90641d0*CON_KCALTOK
      bclocal(5) =     20.90641d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      5.64020d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c= '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    114.99616d0
      bclocal(1) =     44.83517d0*CON_KCALTOK
      bclocal(2) =    -11.23997d0*CON_KCALTOK
      bclocal(3) =     -9.66152d0*CON_KCALTOK
      bclocal(4) =     21.55090d0*CON_KCALTOK
      bclocal(5) =     61.34895d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     13.39812d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c= '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    123.10141d0
      bclocal(1) =     55.81749d0*CON_KCALTOK
      bclocal(2) =    -19.85665d0*CON_KCALTOK
      bclocal(3) =    -12.88883d0*CON_KCALTOK
      bclocal(4) =     22.63093d0*CON_KCALTOK
      bclocal(5) =     69.41965d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     48.37883d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'c= '
      bnlocal(nsame,3) = 'n= '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    129.60436d0
      bclocal(1) =     71.47831d0*CON_KCALTOK
      bclocal(2) =    -32.88896d0*CON_KCALTOK
      bclocal(3) =    -17.84272d0*CON_KCALTOK
      bclocal(4) =    100.87101d0*CON_KCALTOK
      bclocal(5) =    100.87101d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     76.09080d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'n  '
      bnlocal(nsame,2) = 'c= '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    131.71020d0
      bclocal(1) =     84.60596d0*CON_KCALTOK
      bclocal(2) =    -42.08314d0*CON_KCALTOK
      bclocal(3) =    -21.67803d0*CON_KCALTOK
      bclocal(4) =    100.44568d0*CON_KCALTOK
      bclocal(5) =     70.94149d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    142.87417d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'n  '
      bnlocal(nsame,2) = 'c= '
      bnlocal(nsame,3) = 'n= '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.67330d0
      bclocal(1) =     49.33332d0*CON_KCALTOK
      bclocal(2) =     -9.29214d0*CON_KCALTOK
      bclocal(3) =    -11.93331d0*CON_KCALTOK
      bclocal(4) =     32.02799d0*CON_KCALTOK
      bclocal(5) =     32.02799d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'co '
      bnlocal(nsame,3) = 'c  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    108.52950d0
      bclocal(1) =     61.87459d0*CON_KCALTOK
      bclocal(2) =    -11.29179d0*CON_KCALTOK
      bclocal(3) =    -13.09342d0*CON_KCALTOK
      bclocal(4) =     26.33007d0*CON_KCALTOK
      bclocal(5) =     23.00836d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      7.85491d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'co '
      bnlocal(nsame,3) = 'c` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.77000d0
      bclocal(1) =     51.75238d0*CON_KCALTOK
      bclocal(2) =    -13.23853d0*CON_KCALTOK
      bclocal(3) =      6.40332d0*CON_KCALTOK
      bclocal(4) =     15.50126d0*CON_KCALTOK
      bclocal(5) =     14.39897d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      5.47586d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'co '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    114.30180d0
      bclocal(1) =     50.78439d0*CON_KCALTOK
      bclocal(2) =    -12.55520d0*CON_KCALTOK
      bclocal(3) =    -11.10041d0*CON_KCALTOK
      bclocal(4) =     23.36569d0*CON_KCALTOK
      bclocal(5) =     42.16626d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      5.13708d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'co '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.92500d0
      bclocal(1) =     75.94754d0*CON_KCALTOK
      bclocal(2) =    -14.62436d0*CON_KCALTOK
      bclocal(3) =    -14.53986d0*CON_KCALTOK
      bclocal(4) =     40.40422d0*CON_KCALTOK
      bclocal(5) =     74.96181d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     33.85512d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'co '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    108.52950d0
      bclocal(1) =     61.87459d0*CON_KCALTOK
      bclocal(2) =    -11.29179d0*CON_KCALTOK
      bclocal(3) =    -13.09342d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.false.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c` '
      bnlocal(nsame,2) = 'co '
      bnlocal(nsame,3) = 'co '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    107.73360d0
      bclocal(1) =     48.34510d0*CON_KCALTOK
      bclocal(2) =    -34.30013d0*CON_KCALTOK
      bclocal(3) =      0.00000d0*CON_KCALTOK
      bclocal(4) =     18.06260d0*CON_KCALTOK
      bclocal(5) =     13.29930d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      1.03109d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c` '
      bnlocal(nsame,2) = 'co '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.34410d0
      bclocal(1) =     64.54747d0*CON_KCALTOK
      bclocal(2) =    -11.55455d0*CON_KCALTOK
      bclocal(3) =    -12.20306d0*CON_KCALTOK
      bclocal(4) =     21.97302d0*CON_KCALTOK
      bclocal(5) =     21.97302d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      9.35062d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'co '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    108.93717d0
      bclocal(1) =     68.33446d0*CON_KCALTOK
      bclocal(2) =      3.49686d0*CON_KCALTOK
      bclocal(3) =      0.00000d0*CON_KCALTOK
      bclocal(4) =      6.42188d0*CON_KCALTOK
      bclocal(5) =     50.48082d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     14.94287d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'co '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.61980d0
      bclocal(1) =     80.77312d0*CON_KCALTOK
      bclocal(2) =    -15.67803d0*CON_KCALTOK
      bclocal(3) =    -17.16923d0*CON_KCALTOK
      bclocal(4) =     18.65907d0*CON_KCALTOK
      bclocal(5) =     86.03369d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     35.35021d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'co '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    106.66693d0
      bclocal(1) =     98.33405d0*CON_KCALTOK
      bclocal(2) =    -15.95194d0*CON_KCALTOK
      bclocal(3) =    -20.56419d0*CON_KCALTOK
      bclocal(4) =     64.60368d0*CON_KCALTOK
      bclocal(5) =     74.17382d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     72.12945d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'nh '
      bnlocal(nsame,2) = 'co '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.43520d0
      bclocal(1) =    116.10309d0*CON_KCALTOK
      bclocal(2) =    -23.53009d0*CON_KCALTOK
      bclocal(3) =    -24.73995d0*CON_KCALTOK
      bclocal(4) =     72.77172d0*CON_KCALTOK
      bclocal(5) =     72.77172d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     90.28287d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'o  '
      bnlocal(nsame,2) = 'co '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    124.57975d0
      bclocal(1) =     39.99201d0*CON_KCALTOK
      bclocal(2) =    -14.92217d0*CON_KCALTOK
      bclocal(3) =     -9.27180d0*CON_KCALTOK
      bclocal(4) =     52.36545d0*CON_KCALTOK
      bclocal(5) =     20.80388d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     45.99867d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    115.42148d0
      bclocal(1) =     58.48329d0*CON_KCALTOK
      bclocal(2) =    -15.11177d0*CON_KCALTOK
      bclocal(3) =    -12.77228d0*CON_KCALTOK
      bclocal(4) =     55.20781d0*CON_KCALTOK
      bclocal(5) =     86.03087d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     54.06974d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    122.12620d0
      bclocal(1) =     49.86593d0*CON_KCALTOK
      bclocal(2) =    -17.01477d0*CON_KCALTOK
      bclocal(3) =    -11.37943d0*CON_KCALTOK
      bclocal(4) =     35.27518d0*CON_KCALTOK
      bclocal(5) =     32.97449d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     61.70730d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.23183d0
      bclocal(1) =     39.99201d0*CON_KCALTOK
      bclocal(2) =     -8.70882d0*CON_KCALTOK
      bclocal(3) =     -8.41663d0*CON_KCALTOK
      bclocal(4) =     24.88986d0*CON_KCALTOK
      bclocal(5) =     10.20751d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     54.05087d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c" '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    107.10940d0
      bclocal(1) =     58.48329d0*CON_KCALTOK
      bclocal(2) =     -9.71013d0*CON_KCALTOK
      bclocal(3) =    -12.19394d0*CON_KCALTOK
      bclocal(4) =     46.20861d0*CON_KCALTOK
      bclocal(5) =     81.48778d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     64.72098d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c" '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    105.51431d0
      bclocal(1) =     49.86593d0*CON_KCALTOK
      bclocal(2) =     -7.38934d0*CON_KCALTOK
      bclocal(3) =    -10.23357d0*CON_KCALTOK
      bclocal(4) =     20.17474d0*CON_KCALTOK
      bclocal(5) =     45.37480d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     64.86317d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c" '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.49497d0
      bclocal(1) =     29.72316d0*CON_KCALTOK
      bclocal(2) =     -6.23474d0*CON_KCALTOK
      bclocal(3) =     -6.23212d0*CON_KCALTOK
      bclocal(4) =     30.39131d0*CON_KCALTOK
      bclocal(5) =      7.48318d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     49.58604d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c` '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    106.77907d0
      bclocal(1) =     45.96337d0*CON_KCALTOK
      bclocal(2) =     -8.52708d0*CON_KCALTOK
      bclocal(3) =     -9.55271d0*CON_KCALTOK
      bclocal(4) =     46.44611d0*CON_KCALTOK
      bclocal(5) =     67.23428d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     60.27381d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c` '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.72506d0
      bclocal(1) =     41.94061d0*CON_KCALTOK
      bclocal(2) =     -8.82847d0*CON_KCALTOK
      bclocal(3) =     -8.81242d0*CON_KCALTOK
      bclocal(4) =     11.47146d0*CON_KCALTOK
      bclocal(5) =     44.60193d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     61.70234d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c` '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    103.36266d0
      bclocal(1) =     34.45333d0*CON_KCALTOK
      bclocal(2) =     -4.27891d0*CON_KCALTOK
      bclocal(3) =     -7.03810d0*CON_KCALTOK
      bclocal(4) =     34.49674d0*CON_KCALTOK
      bclocal(5) =     57.38261d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     54.78958d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c` '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'op '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    114.32120d0
      bclocal(1) =     35.13950d0*CON_KCALTOK
      bclocal(2) =     -8.17160d0*CON_KCALTOK
      bclocal(3) =     -7.59365d0*CON_KCALTOK
      bclocal(4) =     30.58992d0*CON_KCALTOK
      bclocal(5) =     44.94319d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     27.04484d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c` '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'sp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.69689d0
      bclocal(1) =     49.88869d0*CON_KCALTOK
      bclocal(2) =     -9.56063d0*CON_KCALTOK
      bclocal(3) =    -10.42900d0*CON_KCALTOK
      bclocal(4) =     78.83844d0*CON_KCALTOK
      bclocal(5) =     16.72615d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     68.16876d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c+ '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    115.41695d0
      bclocal(1) =     80.79138d0*CON_KCALTOK
      bclocal(2) =    -20.83947d0*CON_KCALTOK
      bclocal(3) =    -17.61640d0*CON_KCALTOK
      bclocal(4) =     95.62953d0*CON_KCALTOK
      bclocal(5) =     56.95460d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     65.00869d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c+ '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.33509d0
      bclocal(1) =     55.34122d0*CON_KCALTOK
      bclocal(2) =    -11.72604d0*CON_KCALTOK
      bclocal(3) =    -11.80894d0*CON_KCALTOK
      bclocal(4) =     69.06555d0*CON_KCALTOK
      bclocal(5) =     39.54774d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     99.65456d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c+ '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    116.17933d0
      bclocal(1) =     39.99201d0*CON_KCALTOK
      bclocal(2) =    -10.51948d0*CON_KCALTOK
      bclocal(3) =     -8.63356d0*CON_KCALTOK
      bclocal(4) =     36.42032d0*CON_KCALTOK
      bclocal(5) =     11.05972d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     44.39016d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c= '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.34133d0
      bclocal(1) =     58.48329d0*CON_KCALTOK
      bclocal(2) =    -13.02373d0*CON_KCALTOK
      bclocal(3) =    -12.52507d0*CON_KCALTOK
      bclocal(4) =     51.63864d0*CON_KCALTOK
      bclocal(5) =     66.86768d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     51.23397d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c= '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    117.07714d0
      bclocal(1) =     49.86593d0*CON_KCALTOK
      bclocal(2) =    -13.75877d0*CON_KCALTOK
      bclocal(3) =    -10.91769d0*CON_KCALTOK
      bclocal(4) =     34.10896d0*CON_KCALTOK
      bclocal(5) =     39.85745d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     53.64743d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c= '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    123.17430d0
      bclocal(1) =     58.10852d0*CON_KCALTOK
      bclocal(2) =    -21.00034d0*CON_KCALTOK
      bclocal(3) =    -13.60124d0*CON_KCALTOK
      bclocal(4) =     97.61172d0*CON_KCALTOK
      bclocal(5) =     41.94887d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     48.50996d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cl '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    121.11600d0
      bclocal(1) =     91.06005d0*CON_KCALTOK
      bclocal(2) =    -30.33107d0*CON_KCALTOK
      bclocal(3) =    -20.94391d0*CON_KCALTOK
      bclocal(4) =    107.75738d0*CON_KCALTOK
      bclocal(5) =     45.57408d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     35.46438d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cl '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    122.32440d0
      bclocal(1) =     71.07145d0*CON_KCALTOK
      bclocal(2) =    -24.85106d0*CON_KCALTOK
      bclocal(3) =    -16.51842d0*CON_KCALTOK
      bclocal(4) =     84.06303d0*CON_KCALTOK
      bclocal(5) =     37.92332d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     72.77355d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cl '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    119.99517d0
      bclocal(1) =     41.97218d0*CON_KCALTOK
      bclocal(2) =    -13.04808d0*CON_KCALTOK
      bclocal(3) =     -9.34650d0*CON_KCALTOK
      bclocal(4) =     49.16767d0*CON_KCALTOK
      bclocal(5) =     49.16767d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    238.98827d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    119.99517d0
      bclocal(1) =     41.97218d0*CON_KCALTOK
      bclocal(2) =    -13.04808d0*CON_KCALTOK
      bclocal(3) =     -9.34650d0*CON_KCALTOK
      bclocal(4) =     49.16767d0*CON_KCALTOK
      bclocal(5) =     49.16767d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    238.98827d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'cpb'
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    121.21588d0
      bclocal(1) =     56.86507d0*CON_KCALTOK
      bclocal(2) =    -18.74154d0*CON_KCALTOK
      bclocal(3) =    -12.89982d0*CON_KCALTOK
      bclocal(4) =      9.66637d0*CON_KCALTOK
      bclocal(5) =     57.44866d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     40.05212d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'ct '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    122.78640d0
      bclocal(1) =     60.53543d0*CON_KCALTOK
      bclocal(2) =    -21.39227d0*CON_KCALTOK
      bclocal(3) =    -14.01900d0*CON_KCALTOK
      bclocal(4) =     41.94791d0*CON_KCALTOK
      bclocal(5) =     88.28035d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     31.51240d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'f  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    120.88133d0
      bclocal(1) =     52.70070d0*CON_KCALTOK
      bclocal(2) =    -17.16631d0*CON_KCALTOK
      bclocal(3) =    -11.94359d0*CON_KCALTOK
      bclocal(4) =     32.09886d0*CON_KCALTOK
      bclocal(5) =     25.73685d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     24.46602d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    128.01014d0
      bclocal(1) =     45.32104d0*CON_KCALTOK
      bclocal(2) =    -19.63407d0*CON_KCALTOK
      bclocal(3) =    -11.09851d0*CON_KCALTOK
      bclocal(4) =     13.03492d0*CON_KCALTOK
      bclocal(5) =    118.25526d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     60.26693d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    123.67190d0
      bclocal(1) =     38.77453d0*CON_KCALTOK
      bclocal(2) =    -14.03190d0*CON_KCALTOK
      bclocal(3) =     -8.95370d0*CON_KCALTOK
      bclocal(4) =     39.41190d0*CON_KCALTOK
      bclocal(5) =     68.66455d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    113.08194d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'n` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    114.61702d0
      bclocal(1) =     43.25782d0*CON_KCALTOK
      bclocal(2) =    -11.46107d0*CON_KCALTOK
      bclocal(3) =     -9.50154d0*CON_KCALTOK
      bclocal(4) =     -7.76150d0*CON_KCALTOK
      bclocal(5) =    134.90238d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     23.07656d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'n+ '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    121.98874d0
      bclocal(1) =     77.41959d0*CON_KCALTOK
      bclocal(2) =    -22.95522d0*CON_KCALTOK
      bclocal(3) =     27.62039d0*CON_KCALTOK
      bclocal(4) =     60.26678d0*CON_KCALTOK
      bclocal(5) =     36.50949d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     72.52149d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'n= '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    118.80586d0
      bclocal(1) =     57.07924d0*CON_KCALTOK
      bclocal(2) =    -17.25319d0*CON_KCALTOK
      bclocal(3) =    -12.86893d0*CON_KCALTOK
      bclocal(4) =     25.04342d0*CON_KCALTOK
      bclocal(5) =     88.25947d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    109.28100d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    128.01014d0
      bclocal(1) =     45.32104d0*CON_KCALTOK
      bclocal(2) =    -19.63407d0*CON_KCALTOK
      bclocal(3) =    -11.09851d0*CON_KCALTOK
      bclocal(4) =     13.03492d0*CON_KCALTOK
      bclocal(5) =    118.25526d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     60.26693d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'nn '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    122.85718d0
      bclocal(1) =     57.07924d0*CON_KCALTOK
      bclocal(2) =    -20.38001d0*CON_KCALTOK
      bclocal(3) =    -13.32686d0*CON_KCALTOK
      bclocal(4) =     38.17526d0*CON_KCALTOK
      bclocal(5) =     54.30593d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    178.03045d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    122.45771d0
      bclocal(1) =     80.60236d0*CON_KCALTOK
      bclocal(2) =    -28.15122d0*CON_KCALTOK
      bclocal(3) =    -18.63512d0*CON_KCALTOK
      bclocal(4) =     41.27612d0*CON_KCALTOK
      bclocal(5) =    134.90455d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     86.04314d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    122.66696d0
      bclocal(1) =     98.89011d0*CON_KCALTOK
      bclocal(2) =    -34.79762d0*CON_KCALTOK
      bclocal(3) =    -22.87575d0*CON_KCALTOK
      bclocal(4) =      0.00000d0*CON_KCALTOK
      bclocal(5) =     60.19713d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     96.94938d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'op '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    120.19096d0
      bclocal(1) =     62.24384d0*CON_KCALTOK
      bclocal(2) =    -19.89259d0*CON_KCALTOK
      bclocal(3) =    -14.15691d0*CON_KCALTOK
      bclocal(4) =      2.96637d0*CON_KCALTOK
      bclocal(5) =     51.60347d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     22.14038d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 's  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    115.88780d0
      bclocal(1) =     41.02206d0*CON_KCALTOK
      bclocal(2) =    -10.91345d0*CON_KCALTOK
      bclocal(3) =     -9.05776d0*CON_KCALTOK
      bclocal(4) =     21.75471d0*CON_KCALTOK
      bclocal(5) =     31.62592d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     34.58635d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 's" '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    123.84870d0
      bclocal(1) =     55.47367d0*CON_KCALTOK
      bclocal(2) =    -20.45101d0*CON_KCALTOK
      bclocal(3) =    -12.98155d0*CON_KCALTOK
      bclocal(4) =      0.00000d0*CON_KCALTOK
      bclocal(5) =      0.00000d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     76.18451d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'sp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    120.88133d0
      bclocal(1) =     52.70070d0*CON_KCALTOK
      bclocal(2) =    -17.16631d0*CON_KCALTOK
      bclocal(3) =    -11.94359d0*CON_KCALTOK
      bclocal(4) =     32.09886d0*CON_KCALTOK
      bclocal(5) =     25.73685d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     24.46602d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cpb'
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    118.80586d0
      bclocal(1) =     57.07924d0*CON_KCALTOK
      bclocal(2) =    -17.25319d0*CON_KCALTOK
      bclocal(3) =    -12.86893d0*CON_KCALTOK
      bclocal(4) =     25.04342d0*CON_KCALTOK
      bclocal(5) =     88.25947d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    109.28100d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cpb'
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    122.85718d0
      bclocal(1) =     57.07924d0*CON_KCALTOK
      bclocal(2) =    -20.38001d0*CON_KCALTOK
      bclocal(3) =    -13.32686d0*CON_KCALTOK
      bclocal(4) =     38.17526d0*CON_KCALTOK
      bclocal(5) =     54.30593d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    178.03045d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cpb'
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    122.66696d0
      bclocal(1) =     98.89011d0*CON_KCALTOK
      bclocal(2) =    -34.79762d0*CON_KCALTOK
      bclocal(3) =    -22.87575d0*CON_KCALTOK
      bclocal(4) =      0.00000d0*CON_KCALTOK
      bclocal(5) =     60.19713d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     96.94938d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cpb'
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'op '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    123.84870d0
      bclocal(1) =     55.47367d0*CON_KCALTOK
      bclocal(2) =    -20.45101d0*CON_KCALTOK
      bclocal(3) =    -12.98155d0*CON_KCALTOK
      bclocal(4) =      0.00000d0*CON_KCALTOK
      bclocal(5) =      0.00000d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     76.18451d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cpb'
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'sp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    119.62090d0
      bclocal(1) =    104.26173d0*CON_KCALTOK
      bclocal(2) =    -32.58361d0*CON_KCALTOK
      bclocal(3) =    -23.63496d0*CON_KCALTOK
      bclocal(4) =    126.61611d0*CON_KCALTOK
      bclocal(5) =     53.47326d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     42.51349d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'f  '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    121.93370d0
      bclocal(1) =     82.29067d0*CON_KCALTOK
      bclocal(2) =    -28.25378d0*CON_KCALTOK
      bclocal(3) =    -19.00996d0*CON_KCALTOK
      bclocal(4) =     96.16215d0*CON_KCALTOK
      bclocal(5) =     51.35550d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     60.31628d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'f  '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    115.46270d0
      bclocal(1) =     49.96057d0*CON_KCALTOK
      bclocal(2) =    -12.79114d0*CON_KCALTOK
      bclocal(3) =    -10.79341d0*CON_KCALTOK
      bclocal(4) =     18.29048d0*CON_KCALTOK
      bclocal(5) =     67.34958d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     26.64624d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    117.07243d0
      bclocal(1) =     48.02810d0*CON_KCALTOK
      bclocal(2) =    -13.47517d0*CON_KCALTOK
      bclocal(3) =    -10.69451d0*CON_KCALTOK
      bclocal(4) =      6.76665d0*CON_KCALTOK
      bclocal(5) =     55.91549d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     38.74725d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.64780d0
      bclocal(1) =     63.23359d0*CON_KCALTOK
      bclocal(2) =    -12.98216d0*CON_KCALTOK
      bclocal(3) =    -13.47909d0*CON_KCALTOK
      bclocal(4) =      9.55422d0*CON_KCALTOK
      bclocal(5) =     77.28483d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     27.70270d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'op '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.05956d0
      bclocal(1) =     51.67824d0*CON_KCALTOK
      bclocal(2) =    -10.71645d0*CON_KCALTOK
      bclocal(3) =    -10.93341d0*CON_KCALTOK
      bclocal(4) =      0.00000d0*CON_KCALTOK
      bclocal(5) =     47.07822d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      6.06955d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'sp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    124.03348d0
      bclocal(1) =     73.75180d0*CON_KCALTOK
      bclocal(2) =    -27.34156d0*CON_KCALTOK
      bclocal(3) =    -17.26120d0*CON_KCALTOK
      bclocal(4) =    122.28348d0*CON_KCALTOK
      bclocal(5) =     95.07399d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     83.80762d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'n  '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    123.05172d0
      bclocal(1) =     90.83905d0*CON_KCALTOK
      bclocal(2) =    -32.42871d0*CON_KCALTOK
      bclocal(3) =    -21.08092d0*CON_KCALTOK
      bclocal(4) =    129.77133d0*CON_KCALTOK
      bclocal(5) =    102.17486d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    151.43485d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'n  '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    120.57100d0
      bclocal(1) =     73.88939d0*CON_KCALTOK
      bclocal(2) =    -23.79319d0*CON_KCALTOK
      bclocal(3) =    -16.72233d0*CON_KCALTOK
      bclocal(4) =     90.32017d0*CON_KCALTOK
      bclocal(5) =     49.46482d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     93.05951d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'n` '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    123.53540d0
      bclocal(1) =     41.92943d0*CON_KCALTOK
      bclocal(2) =    -15.05848d0*CON_KCALTOK
      bclocal(3) =     -9.64787d0*CON_KCALTOK
      bclocal(4) =     64.43360d0*CON_KCALTOK
      bclocal(5) =     31.19670d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    143.12252d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'n` '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.78867d0
      bclocal(1) =     56.91964d0*CON_KCALTOK
      bclocal(2) =    -12.22435d0*CON_KCALTOK
      bclocal(3) =    -12.05449d0*CON_KCALTOK
      bclocal(4) =    134.34268d0*CON_KCALTOK
      bclocal(5) =     42.80268d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     37.02964d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'n+ '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.24367d0
      bclocal(1) =     71.36648d0*CON_KCALTOK
      bclocal(2) =    -14.82745d0*CON_KCALTOK
      bclocal(3) =    -15.21134d0*CON_KCALTOK
      bclocal(4) =    131.62124d0*CON_KCALTOK
      bclocal(5) =     76.00375d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    103.04661d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'n+ '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    122.85361d0
      bclocal(1) =     82.90811d0*CON_KCALTOK
      bclocal(2) =    -29.31444d0*CON_KCALTOK
      bclocal(3) =    -19.17134d0*CON_KCALTOK
      bclocal(4) =     85.34556d0*CON_KCALTOK
      bclocal(5) =     85.34556d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    112.00117d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'nh '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    124.03348d0
      bclocal(1) =     73.75180d0*CON_KCALTOK
      bclocal(2) =    -27.34156d0*CON_KCALTOK
      bclocal(3) =    -17.26120d0*CON_KCALTOK
      bclocal(4) =     95.07399d0*CON_KCALTOK
      bclocal(5) =    122.28348d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     83.80762d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'nh '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'nn '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    124.26692d0
      bclocal(1) =     99.05923d0*CON_KCALTOK
      bclocal(2) =    -37.41994d0*CON_KCALTOK
      bclocal(3) =    -23.46291d0*CON_KCALTOK
      bclocal(4) =     98.52694d0*CON_KCALTOK
      bclocal(5) =     33.25403d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    162.95555d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'nh '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    118.66552d0
      bclocal(1) =     72.69853d0*CON_KCALTOK
      bclocal(2) =    -21.66329d0*CON_KCALTOK
      bclocal(3) =    -16.23748d0*CON_KCALTOK
      bclocal(4) =     84.48599d0*CON_KCALTOK
      bclocal(5) =     98.89555d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     74.43317d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'nh '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    116.98038d0
      bclocal(1) =     40.32519d0*CON_KCALTOK
      bclocal(2) =    -11.18357d0*CON_KCALTOK
      bclocal(3) =     -8.90608d0*CON_KCALTOK
      bclocal(4) =     30.87712d0*CON_KCALTOK
      bclocal(5) =     24.22130d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     21.29005d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'nh '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 's" '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    123.05172d0
      bclocal(1) =     90.83905d0*CON_KCALTOK
      bclocal(2) =    -32.42871d0*CON_KCALTOK
      bclocal(3) =    -21.08092d0*CON_KCALTOK
      bclocal(4) =    129.77133d0*CON_KCALTOK
      bclocal(5) =    102.17486d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    151.43485d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'nn '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    126.98948d0
      bclocal(1) =     56.77877d0*CON_KCALTOK
      bclocal(2) =    -23.50703d0*CON_KCALTOK
      bclocal(3) =    -13.65443d0*CON_KCALTOK
      bclocal(4) =     38.47487d0*CON_KCALTOK
      bclocal(5) =     38.47487d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    203.36115d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'np '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    123.46406d0
      bclocal(1) =     71.29657d0*CON_KCALTOK
      bclocal(2) =    -26.00359d0*CON_KCALTOK
      bclocal(3) =    -16.69585d0*CON_KCALTOK
      bclocal(4) =     82.34702d0*CON_KCALTOK
      bclocal(5) =    106.20348d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    130.01858d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'np '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    119.52814d0
      bclocal(1) =    115.46416d0*CON_KCALTOK
      bclocal(2) =    -35.95305d0*CON_KCALTOK
      bclocal(3) =    -26.12392d0*CON_KCALTOK
      bclocal(4) =     77.32018d0*CON_KCALTOK
      bclocal(5) =     99.94416d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    203.19237d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'np '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'op '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.11529d0
      bclocal(1) =     30.90676d0*CON_KCALTOK
      bclocal(2) =     -6.32242d0*CON_KCALTOK
      bclocal(3) =     -6.43339d0*CON_KCALTOK
      bclocal(4) =     29.50136d0*CON_KCALTOK
      bclocal(5) =     22.42973d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     43.20987d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'np '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 's" '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    120.61224d0
      bclocal(1) =     97.07880d0*CON_KCALTOK
      bclocal(2) =    -31.66941d0*CON_KCALTOK
      bclocal(3) =    -22.22791d0*CON_KCALTOK
      bclocal(4) =     57.72760d0*CON_KCALTOK
      bclocal(5) =     49.92338d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    173.15840d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'np '
      bnlocal(nsame,2) = 'cp '
      bnlocal(nsame,3) = 'sp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    119.99517d0
      bclocal(1) =     41.97218d0*CON_KCALTOK
      bclocal(2) =    -13.04808d0*CON_KCALTOK
      bclocal(3) =     -9.34650d0*CON_KCALTOK
      bclocal(4) =     49.16767d0*CON_KCALTOK
      bclocal(5) =     49.16767d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cpb'
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    121.85401d0
      bclocal(1) =     35.81420d0*CON_KCALTOK
      bclocal(2) =    -12.02082d0*CON_KCALTOK
      bclocal(3) =     -8.10819d0*CON_KCALTOK
      bclocal(4) =     -4.57113d0*CON_KCALTOK
      bclocal(5) =     26.81834d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     32.88027d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cpb'
      bnlocal(nsame,3) = 'cpb'
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    118.80586d0
      bclocal(1) =     57.07924d0*CON_KCALTOK
      bclocal(2) =    -17.25319d0*CON_KCALTOK
      bclocal(3) =    -12.86893d0*CON_KCALTOK
      bclocal(4) =     25.04342d0*CON_KCALTOK
      bclocal(5) =     88.25947d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    109.28100d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cpb'
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    122.41250d0
      bclocal(1) =     70.70319d0*CON_KCALTOK
      bclocal(2) =    -24.58770d0*CON_KCALTOK
      bclocal(3) =    -16.29894d0*CON_KCALTOK
      bclocal(4) =     50.29612d0*CON_KCALTOK
      bclocal(5) =     74.23887d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    100.98563d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cpb'
      bnlocal(nsame,3) = 'nn '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    122.85718d0
      bclocal(1) =     57.07924d0*CON_KCALTOK
      bclocal(2) =    -20.38001d0*CON_KCALTOK
      bclocal(3) =    -13.32686d0*CON_KCALTOK
      bclocal(4) =     38.17526d0*CON_KCALTOK
      bclocal(5) =     54.30593d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    178.03045d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cpb'
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    122.66696d0
      bclocal(1) =     98.89011d0*CON_KCALTOK
      bclocal(2) =    -34.79762d0*CON_KCALTOK
      bclocal(3) =    -22.87575d0*CON_KCALTOK
      bclocal(4) =      0.00000d0*CON_KCALTOK
      bclocal(5) =     60.19713d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     96.94938d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cpb'
      bnlocal(nsame,3) = 'op '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    123.84870d0
      bclocal(1) =     55.47367d0*CON_KCALTOK
      bclocal(2) =    -20.45101d0*CON_KCALTOK
      bclocal(3) =    -12.98155d0*CON_KCALTOK
      bclocal(4) =      0.00000d0*CON_KCALTOK
      bclocal(5) =      0.00000d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     76.18451d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'cpb'
      bnlocal(nsame,3) = 'sp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    124.39400d0
      bclocal(1) =     47.06288d0*CON_KCALTOK
      bclocal(2) =    -16.16832d0*CON_KCALTOK
      bclocal(3) =    -10.89205d0*CON_KCALTOK
      bclocal(4) =     38.59890d0*CON_KCALTOK
      bclocal(5) =     42.41893d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     29.29632d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cpb'
      bnlocal(nsame,2) = 'cpb'
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    118.05200d0
      bclocal(1) =     66.69684d0*CON_KCALTOK
      bclocal(2) =    -19.34054d0*CON_KCALTOK
      bclocal(3) =    -14.81353d0*CON_KCALTOK
      bclocal(4) =     25.71485d0*CON_KCALTOK
      bclocal(5) =     42.35100d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     52.18873d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cpb'
      bnlocal(nsame,2) = 'cpb'
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    116.85300d0
      bclocal(1) =     33.51567d0*CON_KCALTOK
      bclocal(2) =     -9.17435d0*CON_KCALTOK
      bclocal(3) =     -7.34069d0*CON_KCALTOK
      bclocal(4) =     41.14840d0*CON_KCALTOK
      bclocal(5) =     59.94995d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     24.93666d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cpb'
      bnlocal(nsame,2) = 'cpb'
      bnlocal(nsame,3) = 'op '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    121.11800d0
      bclocal(1) =     70.29633d0*CON_KCALTOK
      bclocal(2) =    -23.33350d0*CON_KCALTOK
      bclocal(3) =    -16.11098d0*CON_KCALTOK
      bclocal(4) =     36.96314d0*CON_KCALTOK
      bclocal(5) =     62.17483d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     23.28215d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cpb'
      bnlocal(nsame,2) = 'cpb'
      bnlocal(nsame,3) = 'sp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    119.37070d0
      bclocal(1) =     87.70841d0*CON_KCALTOK
      bclocal(2) =    -26.97209d0*CON_KCALTOK
      bclocal(3) =    -19.73123d0*CON_KCALTOK
      bclocal(4) =    102.41154d0*CON_KCALTOK
      bclocal(5) =     80.44993d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     82.23746d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'nh '
      bnlocal(nsame,2) = 'cpb'
      bnlocal(nsame,3) = 'nn '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    124.26692d0
      bclocal(1) =     99.05923d0*CON_KCALTOK
      bclocal(2) =    -37.41994d0*CON_KCALTOK
      bclocal(3) =    -23.46291d0*CON_KCALTOK
      bclocal(4) =     98.52694d0*CON_KCALTOK
      bclocal(5) =     33.25403d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    162.95555d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'nh '
      bnlocal(nsame,2) = 'cpb'
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    126.98948d0
      bclocal(1) =     56.77877d0*CON_KCALTOK
      bclocal(2) =    -23.50703d0*CON_KCALTOK
      bclocal(3) =    -13.65443d0*CON_KCALTOK
      bclocal(4) =     38.47487d0*CON_KCALTOK
      bclocal(5) =     38.47487d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    203.36115d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'np '
      bnlocal(nsame,2) = 'cpb'
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    180.75000d0
      bclocal(1) =     29.89432d0*CON_KCALTOK
      bclocal(2) =      0.00000d0*CON_KCALTOK
      bclocal(3) =     -5.51315d0*CON_KCALTOK
      bclocal(4) =     -1.38155d0*CON_KCALTOK
      bclocal(5) =      3.62205d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -7.27954d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'ct '
      bnlocal(nsame,3) = 'ct '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    180.75000d0
      bclocal(1) =     31.19084d0*CON_KCALTOK
      bclocal(2) =      0.00000d0*CON_KCALTOK
      bclocal(3) =     -7.14688d0*CON_KCALTOK
      bclocal(4) =      3.99395d0*CON_KCALTOK
      bclocal(5) =     21.14555d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     11.17243d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'ct '
      bnlocal(nsame,3) = 'ct '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    180.75000d0
      bclocal(1) =     17.98723d0*CON_KCALTOK
      bclocal(2) =      0.00000d0*CON_KCALTOK
      bclocal(3) =      1.43144d0*CON_KCALTOK
      bclocal(4) =     13.32554d0*CON_KCALTOK
      bclocal(5) =      9.08895d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    -10.34855d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'ct '
      bnlocal(nsame,2) = 'ct '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.18516d0
      bclocal(1) =     56.41830d0*CON_KCALTOK
      bclocal(2) =    -11.84547d0*CON_KCALTOK
      bclocal(3) =    -12.01363d0*CON_KCALTOK
      bclocal(4) =     16.63972d0*CON_KCALTOK
      bclocal(5) =     16.63972d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 'c  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.03720d0
      bclocal(1) =     37.97123d0*CON_KCALTOK
      bclocal(2) =     -7.96931d0*CON_KCALTOK
      bclocal(3) =     -8.13929d0*CON_KCALTOK
      bclocal(4) =      5.47996d0*CON_KCALTOK
      bclocal(5) =     21.54096d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     17.56320d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 'c" '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.07212d0
      bclocal(1) =     38.23787d0*CON_KCALTOK
      bclocal(2) =     -7.15671d0*CON_KCALTOK
      bclocal(3) =     -8.06247d0*CON_KCALTOK
      bclocal(4) =     22.22791d0*CON_KCALTOK
      bclocal(5) =     27.82666d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     58.26556d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 'c` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    120.20000d0
      bclocal(1) =     44.29908d0*CON_KCALTOK
      bclocal(2) =    -14.12262d0*CON_KCALTOK
      bclocal(3) =    -10.04760d0*CON_KCALTOK
      bclocal(4) =    -17.51819d0*CON_KCALTOK
      bclocal(5) =    100.56026d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     86.04429d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 'c+ '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    107.91370d0
      bclocal(1) =     45.74921d0*CON_KCALTOK
      bclocal(2) =     -7.91644d0*CON_KCALTOK
      bclocal(3) =     -9.49508d0*CON_KCALTOK
      bclocal(4) =      3.73778d0*CON_KCALTOK
      bclocal(5) =     65.06255d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     23.15068d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 'c= '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.25613d0
      bclocal(1) =     55.82880d0*CON_KCALTOK
      bclocal(2) =    -12.29685d0*CON_KCALTOK
      bclocal(3) =    -11.87129d0*CON_KCALTOK
      bclocal(4) =      3.14026d0*CON_KCALTOK
      bclocal(5) =     69.24604d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      6.23370d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.69067d0
      bclocal(1) =     57.06097d0*CON_KCALTOK
      bclocal(2) =    -13.02628d0*CON_KCALTOK
      bclocal(3) =    -12.33465d0*CON_KCALTOK
      bclocal(4) =     21.40282d0*CON_KCALTOK
      bclocal(5) =     20.58735d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     11.77293d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.50000d0
      bclocal(1) =     41.12346d0*CON_KCALTOK
      bclocal(2) =     -8.38606d0*CON_KCALTOK
      bclocal(3) =     -8.78937d0*CON_KCALTOK
      bclocal(4) =    -40.80506d0*CON_KCALTOK
      bclocal(5) =     62.22637d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     31.00211d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 's" '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    121.95560d0
      bclocal(1) =     90.84582d0*CON_KCALTOK
      bclocal(2) =    -31.32934d0*CON_KCALTOK
      bclocal(3) =    -21.06484d0*CON_KCALTOK
      bclocal(4) =     29.06280d0*CON_KCALTOK
      bclocal(5) =     29.06280d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c" '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 'c" '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    116.96070d0
      bclocal(1) =     44.63853d0*CON_KCALTOK
      bclocal(2) =     -9.74250d0*CON_KCALTOK
      bclocal(3) =      0.00000d0*CON_KCALTOK
      bclocal(4) =     42.86129d0*CON_KCALTOK
      bclocal(5) =     15.71334d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -6.25015d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c" '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    123.50828d0
      bclocal(1) =     47.53502d0*CON_KCALTOK
      bclocal(2) =    -17.24370d0*CON_KCALTOK
      bclocal(3) =    -11.05687d0*CON_KCALTOK
      bclocal(4) =     39.87382d0*CON_KCALTOK
      bclocal(5) =     39.87382d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c` '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 'c` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    129.32818d0
      bclocal(1) =     38.93482d0*CON_KCALTOK
      bclocal(2) =    -17.42165d0*CON_KCALTOK
      bclocal(3) =     -9.51786d0*CON_KCALTOK
      bclocal(4) =     64.84500d0*CON_KCALTOK
      bclocal(5) =     63.33835d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     19.38828d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c` '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    115.33515d0
      bclocal(1) =     48.80314d0*CON_KCALTOK
      bclocal(2) =    -12.45650d0*CON_KCALTOK
      bclocal(3) =    -10.56395d0*CON_KCALTOK
      bclocal(4) =     38.36442d0*CON_KCALTOK
      bclocal(5) =     19.64696d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      5.65780d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c` '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    120.00066d0
      bclocal(1) =     57.78872d0*CON_KCALTOK
      bclocal(2) =    -18.14416d0*CON_KCALTOK
      bclocal(3) =    -12.99451d0*CON_KCALTOK
      bclocal(4) =     60.03525d0*CON_KCALTOK
      bclocal(5) =     19.28677d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     13.58408d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c+ '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    117.89442d0
      bclocal(1) =     53.37327d0*CON_KCALTOK
      bclocal(2) =    -15.42520d0*CON_KCALTOK
      bclocal(3) =    -11.88139d0*CON_KCALTOK
      bclocal(4) =     63.57245d0*CON_KCALTOK
      bclocal(5) =     22.20343d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -1.31223d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c= '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    121.55032d0
      bclocal(1) =     49.49435d0*CON_KCALTOK
      bclocal(2) =    -16.62352d0*CON_KCALTOK
      bclocal(3) =    -11.32070d0*CON_KCALTOK
      bclocal(4) =     65.59005d0*CON_KCALTOK
      bclocal(5) =     23.16032d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -9.83228d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    127.68774d0
      bclocal(1) =     41.17911d0*CON_KCALTOK
      bclocal(2) =    -15.69229d0*CON_KCALTOK
      bclocal(3) =     -9.74805d0*CON_KCALTOK
      bclocal(4) =      5.39118d0*CON_KCALTOK
      bclocal(5) =     26.75049d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      9.22965d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 's" '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.98437d0
      bclocal(1) =     49.97057d0*CON_KCALTOK
      bclocal(2) =     -9.71453d0*CON_KCALTOK
      bclocal(3) =    -10.44444d0*CON_KCALTOK
      bclocal(4) =     18.74961d0*CON_KCALTOK
      bclocal(5) =     18.74961d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    122.02220d0
      bclocal(1) =     53.97580d0*CON_KCALTOK
      bclocal(2) =    -18.66427d0*CON_KCALTOK
      bclocal(3) =    -12.52316d0*CON_KCALTOK
      bclocal(4) =     33.20599d0*CON_KCALTOK
      bclocal(5) =     78.26656d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     13.20383d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'n  '
      bnlocal(nsame,3) = 's" '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    117.37230d0
      bclocal(1) =     82.32983d0*CON_KCALTOK
      bclocal(2) =    -23.32676d0*CON_KCALTOK
      bclocal(3) =    -18.31037d0*CON_KCALTOK
      bclocal(4) =    107.13904d0*CON_KCALTOK
      bclocal(5) =     94.41246d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    129.83190d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'n` '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    125.25470d0
      bclocal(1) =    154.85213d0*CON_KCALTOK
      bclocal(2) =    -60.48647d0*CON_KCALTOK
      bclocal(3) =    -36.86338d0*CON_KCALTOK
      bclocal(4) =    110.34975d0*CON_KCALTOK
      bclocal(5) =    110.34975d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    315.12874d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'o` '
      bnlocal(nsame,2) = 'n` '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.73950d0
      bclocal(1) =     59.28437d0*CON_KCALTOK
      bclocal(2) =    -12.18775d0*CON_KCALTOK
      bclocal(3) =    -12.62639d0*CON_KCALTOK
      bclocal(4) =     36.26821d0*CON_KCALTOK
      bclocal(5) =     36.26821d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'n+ '
      bnlocal(nsame,3) = 'c  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.98145d0
      bclocal(1) =     64.02268d0*CON_KCALTOK
      bclocal(2) =    -14.10991d0*CON_KCALTOK
      bclocal(3) =    -13.79183d0*CON_KCALTOK
      bclocal(4) =     32.17345d0*CON_KCALTOK
      bclocal(5) =     62.61769d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     57.95278d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'n+ '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.24527d0
      bclocal(1) =     54.83865d0*CON_KCALTOK
      bclocal(2) =    -10.86038d0*CON_KCALTOK
      bclocal(3) =    -11.52591d0*CON_KCALTOK
      bclocal(4) =     28.61490d0*CON_KCALTOK
      bclocal(5) =      4.20121d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     12.16920d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'n+ '
      bnlocal(nsame,3) = 'h+ '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.59531d0
      bclocal(1) =     58.75534d0*CON_KCALTOK
      bclocal(2) =    -12.53478d0*CON_KCALTOK
      bclocal(3) =    -12.47129d0*CON_KCALTOK
      bclocal(4) =     40.56243d0*CON_KCALTOK
      bclocal(5) =      9.49524d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     40.90519d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'n+ '
      bnlocal(nsame,3) = 'h+ '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    105.91333d0
      bclocal(1) =     55.10303d0*CON_KCALTOK
      bclocal(2) =     -8.52108d0*CON_KCALTOK
      bclocal(3) =    -11.50502d0*CON_KCALTOK
      bclocal(4) =     12.80121d0*CON_KCALTOK
      bclocal(5) =     12.80121d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h+ '
      bnlocal(nsame,2) = 'n+ '
      bnlocal(nsame,3) = 'h+ '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.02316d0
      bclocal(1) =     66.26258d0*CON_KCALTOK
      bclocal(2) =    -13.84160d0*CON_KCALTOK
      bclocal(3) =    -14.14628d0*CON_KCALTOK
      bclocal(4) =     86.37713d0*CON_KCALTOK
      bclocal(5) =     86.37713d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'n= '
      bnlocal(nsame,3) = 'c  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    114.75387d0
      bclocal(1) =    122.00799d0*CON_KCALTOK
      bclocal(2) =    -41.54859d0*CON_KCALTOK
      bclocal(3) =     23.38316d0*CON_KCALTOK
      bclocal(4) =     83.15308d0*CON_KCALTOK
      bclocal(5) =     88.20607d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     83.00360d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'n= '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.58743d0
      bclocal(1) =     66.84106d0*CON_KCALTOK
      bclocal(2) =    -15.00519d0*CON_KCALTOK
      bclocal(3) =    -14.27343d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.false.)
      bclocal(6) =    -17.13356d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c= '
      bnlocal(nsame,2) = 'n= '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    114.75387d0
      bclocal(1) =    127.03118d0*CON_KCALTOK
      bclocal(2) =    -92.12250d0*CON_KCALTOK
      bclocal(3) =     51.07143d0*CON_KCALTOK
      bclocal(4) =     72.90447d0*CON_KCALTOK
      bclocal(5) =     74.43586d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    123.48999d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'n= '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.51922d0
      bclocal(1) =     98.57054d0*CON_KCALTOK
      bclocal(2) =    -24.19394d0*CON_KCALTOK
      bclocal(3) =      0.00000d0*CON_KCALTOK
      bclocal(4) =     23.20321d0*CON_KCALTOK
      bclocal(5) =     98.31307d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'n= '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.44360d0
      bclocal(1) =     58.96836d0*CON_KCALTOK
      bclocal(2) =    -13.30981d0*CON_KCALTOK
      bclocal(3) =    -12.74187d0*CON_KCALTOK
      bclocal(4) =     -8.39875d0*CON_KCALTOK
      bclocal(5) =     -8.39875d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'na '
      bnlocal(nsame,3) = 'c  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.95380d0
      bclocal(1) =     63.50213d0*CON_KCALTOK
      bclocal(2) =     -5.55832d0*CON_KCALTOK
      bclocal(3) =    -12.52156d0*CON_KCALTOK
      bclocal(4) =     36.98790d0*CON_KCALTOK
      bclocal(5) =     23.93017d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -7.46139d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'na '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    108.82838d0
      bclocal(1) =     90.13072d0*CON_KCALTOK
      bclocal(2) =    -16.71352d0*CON_KCALTOK
      bclocal(3) =    -19.07255d0*CON_KCALTOK
      bclocal(4) =     29.31304d0*CON_KCALTOK
      bclocal(5) =     64.03543d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      8.19238d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'na '
      bnlocal(nsame,3) = 'na '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    107.06710d0
      bclocal(1) =     56.49442d0*CON_KCALTOK
      bclocal(2) =     -9.43294d0*CON_KCALTOK
      bclocal(3) =    -11.87521d0*CON_KCALTOK
      bclocal(4) =     32.59561d0*CON_KCALTOK
      bclocal(5) =     32.59561d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'na '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    106.86791d0
      bclocal(1) =     78.43829d0*CON_KCALTOK
      bclocal(2) =    -12.86062d0*CON_KCALTOK
      bclocal(3) =    -16.38153d0*CON_KCALTOK
      bclocal(4) =     24.36200d0*CON_KCALTOK
      bclocal(5) =     55.76991d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -0.33140d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'na '
      bnlocal(nsame,3) = 'na '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.56350d0
      bclocal(1) =     47.15292d0*CON_KCALTOK
      bclocal(2) =    -10.17371d0*CON_KCALTOK
      bclocal(3) =    -10.13711d0*CON_KCALTOK
      bclocal(4) =     -2.97055d0*CON_KCALTOK
      bclocal(5) =     -2.97055d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'nh '
      bnlocal(nsame,3) = 'c  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.56350d0
      bclocal(1) =     47.15292d0*CON_KCALTOK
      bclocal(2) =    -10.17371d0*CON_KCALTOK
      bclocal(3) =    -10.13711d0*CON_KCALTOK
      bclocal(4) =     -2.97055d0*CON_KCALTOK
      bclocal(5) =     -2.97055d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'nh '
      bnlocal(nsame,3) = 'co '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    119.78144d0
      bclocal(1) =     35.00000d0*CON_KCALTOK
      bclocal(2) =    -11.03965d0*CON_KCALTOK
      bclocal(3) =     -7.96459d0*CON_KCALTOK
      bclocal(4) =     16.95686d0*CON_KCALTOK
      bclocal(5) =     40.98306d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     38.01291d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'nh '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    113.86830d0
      bclocal(1) =     54.67515d0*CON_KCALTOK
      bclocal(2) =    -23.90761d0*CON_KCALTOK
      bclocal(3) =      0.00000d0*CON_KCALTOK
      bclocal(4) =     17.22152d0*CON_KCALTOK
      bclocal(5) =      8.59992d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -5.03044d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'nh '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.56350d0
      bclocal(1) =     47.15292d0*CON_KCALTOK
      bclocal(2) =    -10.17371d0*CON_KCALTOK
      bclocal(3) =    -10.13711d0*CON_KCALTOK
      bclocal(4) =     -2.97055d0*CON_KCALTOK
      bclocal(5) =     -2.97055d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'co '
      bnlocal(nsame,2) = 'nh '
      bnlocal(nsame,3) = 'co '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    119.78144d0
      bclocal(1) =     35.00000d0*CON_KCALTOK
      bclocal(2) =    -11.03965d0*CON_KCALTOK
      bclocal(3) =     -7.96459d0*CON_KCALTOK
      bclocal(4) =     -8.38873d0*CON_KCALTOK
      bclocal(5) =     26.96578d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     52.05734d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'co '
      bnlocal(nsame,2) = 'nh '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    113.86830d0
      bclocal(1) =     54.67515d0*CON_KCALTOK
      bclocal(2) =    -23.90761d0*CON_KCALTOK
      bclocal(3) =      0.00000d0*CON_KCALTOK
      bclocal(4) =     17.22152d0*CON_KCALTOK
      bclocal(5) =      8.59992d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)

      bclocal(6) =     -5.03044d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'co '
      bnlocal(nsame,2) = 'nh '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    124.50000d0
      bclocal(1) =     57.31697d0*CON_KCALTOK
      bclocal(2) =    -21.74167d0*CON_KCALTOK
      bclocal(3) =    -13.54029d0*CON_KCALTOK
      bclocal(4) =     59.22050d0*CON_KCALTOK
      bclocal(5) =     59.22050d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'nh '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    124.50000d0
      bclocal(1) =     57.31697d0*CON_KCALTOK
      bclocal(2) =    -21.74167d0*CON_KCALTOK
      bclocal(3) =    -13.54029d0*CON_KCALTOK
      bclocal(4) =     59.22050d0*CON_KCALTOK
      bclocal(5) =     59.22050d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    112.00117d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'nh '
      bnlocal(nsame,3) = 'cpb'
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    119.32072d0
      bclocal(1) =     40.00000d0*CON_KCALTOK
      bclocal(2) =    -12.37484d0*CON_KCALTOK
      bclocal(3) =     -9.06816d0*CON_KCALTOK
      bclocal(4) =     26.39096d0*CON_KCALTOK
      bclocal(5) =      8.82867d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     10.64565d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'nh '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    119.04280d0
      bclocal(1) =     58.24963d0*CON_KCALTOK
      bclocal(2) =    -17.73371d0*CON_KCALTOK
      bclocal(3) =    -13.11943d0*CON_KCALTOK
      bclocal(4) =     30.42499d0*CON_KCALTOK
      bclocal(5) =    -17.39450d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     85.49058d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'nh '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    119.32072d0
      bclocal(1) =     40.00000d0*CON_KCALTOK
      bclocal(2) =    -12.37484d0*CON_KCALTOK
      bclocal(3) =     -9.06816d0*CON_KCALTOK
      bclocal(4) =     26.39096d0*CON_KCALTOK
      bclocal(5) =      8.82867d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     10.64565d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cpb'
      bnlocal(nsame,2) = 'nh '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    119.04280d0
      bclocal(1) =     58.24963d0*CON_KCALTOK
      bclocal(2) =    -17.73371d0*CON_KCALTOK
      bclocal(3) =    -13.11943d0*CON_KCALTOK
      bclocal(4) =     30.42499d0*CON_KCALTOK
      bclocal(5) =    -17.39450d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     85.49058d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cpb'
      bnlocal(nsame,2) = 'nh '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.94020d0
      bclocal(1) =     56.73197d0*CON_KCALTOK
      bclocal(2) =    -11.31860d0*CON_KCALTOK
      bclocal(3) =      0.00000d0*CON_KCALTOK
      bclocal(4) =     28.71378d0*CON_KCALTOK
      bclocal(5) =     28.71378d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'nh '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.47552d0
      bclocal(1) =     45.82977d0*CON_KCALTOK
      bclocal(2) =     -8.67174d0*CON_KCALTOK
      bclocal(3) =     -9.56689d0*CON_KCALTOK
      bclocal(4) =     -3.29691d0*CON_KCALTOK
      bclocal(5) =      9.34364d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     27.56423d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'nh '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    114.25770d0
      bclocal(1) =    133.31739d0*CON_KCALTOK
      bclocal(2) =    -32.81197d0*CON_KCALTOK
      bclocal(3) =    -29.06274d0*CON_KCALTOK
      bclocal(4) =     93.95172d0*CON_KCALTOK
      bclocal(5) =     93.95172d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    137.12466d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'np '
      bnlocal(nsame,2) = 'nh '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.03852d0
      bclocal(1) =     67.14337d0*CON_KCALTOK
      bclocal(2) =    -14.06278d0*CON_KCALTOK
      bclocal(3) =    -14.36183d0*CON_KCALTOK
      bclocal(4) =     29.63997d0*CON_KCALTOK
      bclocal(5) =     29.63997d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'nn '
      bnlocal(nsame,3) = 'c  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.25613d0
      bclocal(1) =     55.82880d0*CON_KCALTOK
      bclocal(2) =    -12.29685d0*CON_KCALTOK
      bclocal(3) =    -11.87129d0*CON_KCALTOK
      bclocal(4) =      3.14026d0*CON_KCALTOK
      bclocal(5) =     69.24604d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      6.23370d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'nn '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.81940d0
      bclocal(1) =     48.21012d0*CON_KCALTOK
      bclocal(2) =     -9.95797d0*CON_KCALTOK
      bclocal(3) =    -10.27676d0*CON_KCALTOK
      bclocal(4) =    -35.23269d0*CON_KCALTOK
      bclocal(5) =     84.77521d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     24.11417d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'nn '
      bnlocal(nsame,3) = 'cpb'
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.86332d0
      bclocal(1) =     66.85267d0*CON_KCALTOK
      bclocal(2) =    -12.99748d0*CON_KCALTOK
      bclocal(3) =    -14.05922d0*CON_KCALTOK
      bclocal(4) =     20.65399d0*CON_KCALTOK
      bclocal(5) =     47.19855d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -5.03044d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'nn '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    118.26466d0
      bclocal(1) =     53.40841d0*CON_KCALTOK
      bclocal(2) =    -15.67444d0*CON_KCALTOK
      bclocal(3) =    -11.91523d0*CON_KCALTOK
      bclocal(4) =     65.59005d0*CON_KCALTOK
      bclocal(5) =     23.16032d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -9.83228d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'nn '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    115.79203d0
      bclocal(1) =     58.15919d0*CON_KCALTOK
      bclocal(2) =    -15.37118d0*CON_KCALTOK
      bclocal(3) =    -12.80488d0*CON_KCALTOK
      bclocal(4) =     61.42596d0*CON_KCALTOK
      bclocal(5) =     22.56262d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    -18.59473d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cpb'
      bnlocal(nsame,2) = 'nn '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.56296d0
      bclocal(1) =     54.22024d0*CON_KCALTOK
      bclocal(2) =    -12.26188d0*CON_KCALTOK
      bclocal(3) =    -11.67656d0*CON_KCALTOK
      bclocal(4) =     29.46388d0*CON_KCALTOK
      bclocal(5) =     29.46388d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'nn '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    115.92865d0
      bclocal(1) =     58.81903d0*CON_KCALTOK
      bclocal(2) =    -15.46730d0*CON_KCALTOK
      bclocal(3) =    -12.81707d0*CON_KCALTOK
      bclocal(4) =     79.36598d0*CON_KCALTOK
      bclocal(5) =     79.36598d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'np '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    115.92865d0
      bclocal(1) =     58.81903d0*CON_KCALTOK
      bclocal(2) =    -15.46730d0*CON_KCALTOK
      bclocal(3) =    -12.81707d0*CON_KCALTOK
      bclocal(4) =     79.36598d0*CON_KCALTOK
      bclocal(5) =     79.36598d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    203.36115d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'np '
      bnlocal(nsame,3) = 'cpb'
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    120.22296d0
      bclocal(1) =     53.03215d0*CON_KCALTOK
      bclocal(2) =    -17.02756d0*CON_KCALTOK
      bclocal(3) =    -12.10513d0*CON_KCALTOK
      bclocal(4) =     63.76716d0*CON_KCALTOK
      bclocal(5) =    115.41114d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    128.59514d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'np '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    121.33706d0
      bclocal(1) =     74.23989d0*CON_KCALTOK
      bclocal(2) =    -24.88823d0*CON_KCALTOK
      bclocal(3) =    -17.06435d0*CON_KCALTOK
      bclocal(4) =     96.29391d0*CON_KCALTOK
      bclocal(5) =    137.43456d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    241.21361d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'np '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    116.64582d0
      bclocal(1) =     89.75882d0*CON_KCALTOK
      bclocal(2) =    -24.55396d0*CON_KCALTOK
      bclocal(3) =    -19.77295d0*CON_KCALTOK
      bclocal(4) =     81.69687d0*CON_KCALTOK
      bclocal(5) =    236.78932d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    146.94626d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'np '
      bnlocal(nsame,3) = 'op '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    123.49962d0
      bclocal(1) =     26.24184d0*CON_KCALTOK
      bclocal(2) =     -9.53581d0*CON_KCALTOK
      bclocal(3) =     -6.11606d0*CON_KCALTOK
      bclocal(4) =     68.14613d0*CON_KCALTOK
      bclocal(5) =    121.43785d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    155.76093d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'np '
      bnlocal(nsame,3) = 'sp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    120.22296d0
      bclocal(1) =     53.03215d0*CON_KCALTOK
      bclocal(2) =    -17.02756d0*CON_KCALTOK
      bclocal(3) =    -12.10513d0*CON_KCALTOK
      bclocal(4) =     63.76716d0*CON_KCALTOK
      bclocal(5) =    115.41114d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    128.59514d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cpb'
      bnlocal(nsame,2) = 'np '
      bnlocal(nsame,3) = 'nh '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    121.33706d0
      bclocal(1) =     74.23989d0*CON_KCALTOK
      bclocal(2) =    -24.88823d0*CON_KCALTOK
      bclocal(3) =    -17.06435d0*CON_KCALTOK
      bclocal(4) =     96.29391d0*CON_KCALTOK
      bclocal(5) =    137.43456d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    241.21361d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cpb'
      bnlocal(nsame,2) = 'np '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    116.64582d0
      bclocal(1) =     89.75882d0*CON_KCALTOK
      bclocal(2) =    -24.55396d0*CON_KCALTOK
      bclocal(3) =    -19.77295d0*CON_KCALTOK
      bclocal(4) =     81.69687d0*CON_KCALTOK
      bclocal(5) =    236.78932d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    146.94626d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cpb'
      bnlocal(nsame,2) = 'np '
      bnlocal(nsame,3) = 'op '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    108.35747d0
      bclocal(1) =    106.41610d0*CON_KCALTOK
      bclocal(2) =    -19.14918d0*CON_KCALTOK
      bclocal(3) =    -22.41256d0*CON_KCALTOK
      bclocal(4) =    183.85966d0*CON_KCALTOK
      bclocal(5) =    115.54851d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    273.06595d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'nh '
      bnlocal(nsame,2) = 'np '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    118.36598d0
      bclocal(1) =    136.08429d0*CON_KCALTOK
      bclocal(2) =    -40.39715d0*CON_KCALTOK
      bclocal(3) =    -30.59897d0*CON_KCALTOK
      bclocal(4) =    131.75263d0*CON_KCALTOK
      bclocal(5) =    131.75263d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    269.75374d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'np '
      bnlocal(nsame,2) = 'np '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    119.84696d0
      bclocal(1) =    158.59242d0*CON_KCALTOK
      bclocal(2) =    -50.01159d0*CON_KCALTOK
      bclocal(3) =    -35.97945d0*CON_KCALTOK
      bclocal(4) =    140.03032d0*CON_KCALTOK
      bclocal(5) =    264.31766d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    296.18444d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'np '
      bnlocal(nsame,2) = 'np '
      bnlocal(nsame,3) = 'op '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    121.48723d0
      bclocal(1) =    103.46695d0*CON_KCALTOK
      bclocal(2) =    -34.85393d0*CON_KCALTOK
      bclocal(3) =    -23.78325d0*CON_KCALTOK
      bclocal(4) =    108.27053d0*CON_KCALTOK
      bclocal(5) =    148.75006d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    259.85255d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'np '
      bnlocal(nsame,2) = 'np '
      bnlocal(nsame,3) = 'sp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    108.58000d0
      bclocal(1) =     44.62597d0*CON_KCALTOK
      bclocal(2) =    -12.49272d0*CON_KCALTOK
      bclocal(3) =     -7.83129d0*CON_KCALTOK
      bclocal(4) =     -3.26886d0*CON_KCALTOK
      bclocal(5) =     -3.26886d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'c  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    106.45950d0
      bclocal(1) =     38.97388d0*CON_KCALTOK
      bclocal(2) =     -6.25952d0*CON_KCALTOK
      bclocal(3) =     -8.17105d0*CON_KCALTOK
      bclocal(4) =    -19.38933d0*CON_KCALTOK
      bclocal(5) =     25.04251d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     30.39068d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'c" '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    106.45950d0
      bclocal(1) =     38.97388d0*CON_KCALTOK
      bclocal(2) =     -6.25952d0*CON_KCALTOK
      bclocal(3) =     -8.17105d0*CON_KCALTOK
      bclocal(4) =    -19.38933d0*CON_KCALTOK
      bclocal(5) =     25.04251d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     30.39068d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'c` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    115.02700d0
      bclocal(1) =     94.82152d0*CON_KCALTOK
      bclocal(2) =    -15.92693d0*CON_KCALTOK
      bclocal(3) =    -17.84957d0*CON_KCALTOK
      bclocal(4) =     59.21065d0*CON_KCALTOK
      bclocal(5) =     64.55751d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     36.80598d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'co '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.53735d0
      bclocal(1) =     54.88291d0*CON_KCALTOK
      bclocal(2) =    -11.03416d0*CON_KCALTOK
      bclocal(3) =    -11.54380d0*CON_KCALTOK
      bclocal(4) =      5.28260d0*CON_KCALTOK
      bclocal(5) =     77.73328d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     38.53648d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.09270d0
      bclocal(1) =     65.80032d0*CON_KCALTOK
      bclocal(2) =    -15.11739d0*CON_KCALTOK
      bclocal(3) =    -12.31977d0*CON_KCALTOK
      bclocal(4) =     33.23260d0*CON_KCALTOK
      bclocal(5) =     22.00895d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    -11.26498d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.82337d0
      bclocal(1) =     41.78864d0*CON_KCALTOK
      bclocal(2) =     -8.05630d0*CON_KCALTOK
      bclocal(3) =     -8.73196d0*CON_KCALTOK
      bclocal(4) =     61.36611d0*CON_KCALTOK
      bclocal(5) =     60.38909d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     57.00476d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'p  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.32337d0
      bclocal(1) =     56.92757d0*CON_KCALTOK
      bclocal(2) =    -11.93191d0*CON_KCALTOK
      bclocal(3) =    -12.02286d0*CON_KCALTOK
      bclocal(4) =    165.65171d0*CON_KCALTOK
      bclocal(5) =     72.39487d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    128.37830d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'p- '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    106.45950d0
      bclocal(1) =     38.97388d0*CON_KCALTOK
      bclocal(2) =     -6.25952d0*CON_KCALTOK
      bclocal(3) =     -8.17105d0*CON_KCALTOK
      bclocal(4) =     25.04251d0*CON_KCALTOK
      bclocal(5) =    -19.38933d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      0.00000d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c" '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'co '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.05370d0
      bclocal(1) =     66.82931d0*CON_KCALTOK
      bclocal(2) =    -14.78829d0*CON_KCALTOK
      bclocal(3) =    -14.40755d0*CON_KCALTOK
      bclocal(4) =     34.67539d0*CON_KCALTOK
      bclocal(5) =     26.18829d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -7.22538d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c" '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    112.05370d0
      bclocal(1) =     66.82931d0*CON_KCALTOK
      bclocal(2) =    -14.78829d0*CON_KCALTOK
      bclocal(3) =    -14.40755d0*CON_KCALTOK
      bclocal(4) =     34.67539d0*CON_KCALTOK
      bclocal(5) =     26.18829d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -7.22538d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c` '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.62280d0
      bclocal(1) =     77.98005d0*CON_KCALTOK
      bclocal(2) =    -14.23097d0*CON_KCALTOK
      bclocal(3) =    -15.58216d0*CON_KCALTOK
      bclocal(4) =     53.28180d0*CON_KCALTOK
      bclocal(5) =     53.28180d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'co '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'co '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.00950d0
      bclocal(1) =     65.37896d0*CON_KCALTOK
      bclocal(2) =    -13.03236d0*CON_KCALTOK
      bclocal(3) =    -13.99397d0*CON_KCALTOK
      bclocal(4) =     48.55883d0*CON_KCALTOK
      bclocal(5) =     27.30584d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      2.68725d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'co '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.43885d0
      bclocal(1) =     75.47173d0*CON_KCALTOK
      bclocal(2) =    -16.06859d0*CON_KCALTOK
      bclocal(3) =    -16.11154d0*CON_KCALTOK
      bclocal(4) =     75.58950d0*CON_KCALTOK
      bclocal(5) =     38.38701d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     11.84690d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    106.20000d0
      bclocal(1) =     43.46824d0*CON_KCALTOK
      bclocal(2) =     -6.78991d0*CON_KCALTOK
      bclocal(3) =     -9.00540d0*CON_KCALTOK
      bclocal(4) =    138.49674d0*CON_KCALTOK
      bclocal(5) =    -57.40836d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    188.08361d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'p- '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    116.52000d0
      bclocal(1) =     65.51762d0*CON_KCALTOK
      bclocal(2) =    -17.80372d0*CON_KCALTOK
      bclocal(3) =    -14.42418d0*CON_KCALTOK
      bclocal(4) =     57.58541d0*CON_KCALTOK
      bclocal(5) =     38.81948d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     14.41883d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'p- '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    103.70000d0
      bclocal(1) =     49.84000d0*CON_KCALTOK
      bclocal(2) =    -11.60000d0*CON_KCALTOK
      bclocal(3) =     -8.00000d0*CON_KCALTOK
      bclocal(4) =     22.35000d0*CON_KCALTOK
      bclocal(5) =     22.35000d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 'o  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.49063d0
      bclocal(1) =     92.01300d0*CON_KCALTOK
      bclocal(2) =    -19.80425d0*CON_KCALTOK
      bclocal(3) =    -19.77356d0*CON_KCALTOK
      bclocal(4) =     38.97311d0*CON_KCALTOK
      bclocal(5) =     38.97311d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'op '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.49063d0
      bclocal(1) =     92.01300d0*CON_KCALTOK
      bclocal(2) =    -19.80425d0*CON_KCALTOK
      bclocal(3) =    -19.77356d0*CON_KCALTOK
      bclocal(4) =     38.97311d0*CON_KCALTOK
      bclocal(5) =     38.97311d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    114.51123d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'op '
      bnlocal(nsame,3) = 'cpb'
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    121.44342d0
      bclocal(1) =     90.78883d0*CON_KCALTOK
      bclocal(2) =    -30.46234d0*CON_KCALTOK
      bclocal(3) =    -20.78259d0*CON_KCALTOK
      bclocal(4) =     32.97279d0*CON_KCALTOK
      bclocal(5) =    142.99565d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    127.95408d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'op '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    116.90458d0
      bclocal(1) =    226.22745d0*CON_KCALTOK
      bclocal(2) =    -62.99811d0*CON_KCALTOK
      bclocal(3) =    -50.29453d0*CON_KCALTOK
      bclocal(4) =    231.93491d0*CON_KCALTOK
      bclocal(5) =    231.93491d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'np '
      bnlocal(nsame,2) = 'op '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =     98.51026d0
      bclocal(1) =    134.72759d0*CON_KCALTOK
      bclocal(2) =    -10.85237d0*CON_KCALTOK
      bclocal(3) =    -27.39906d0*CON_KCALTOK
      bclocal(4) =    100.56616d0*CON_KCALTOK
      bclocal(5) =    100.56616d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'o  '
      bnlocal(nsame,2) = 'p  '
      bnlocal(nsame,3) = 'o  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    109.59368d0
      bclocal(1) =    135.93403d0*CON_KCALTOK
      bclocal(2) =    -26.18870d0*CON_KCALTOK
      bclocal(3) =    -28.71785d0*CON_KCALTOK
      bclocal(4) =     93.51291d0*CON_KCALTOK
      bclocal(5) =     52.79181d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     60.10570d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'o  '
      bnlocal(nsame,2) = 'p  '
      bnlocal(nsame,3) = 'o- '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    124.49976d0
      bclocal(1) =    155.76218d0*CON_KCALTOK
      bclocal(2) =    -59.12154d0*CON_KCALTOK
      bclocal(3) =    -36.81998d0*CON_KCALTOK
      bclocal(4) =     52.74532d0*CON_KCALTOK
      bclocal(5) =     52.74532d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'o- '
      bnlocal(nsame,2) = 'p  '
      bnlocal(nsame,3) = 'o- '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    110.78690d0
      bclocal(1) =    125.82436d0*CON_KCALTOK
      bclocal(2) =    -25.88716d0*CON_KCALTOK
      bclocal(3) =    -26.75769d0*CON_KCALTOK
      bclocal(4) =     95.10955d0*CON_KCALTOK
      bclocal(5) =     42.39055d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     82.33028d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'o  '
      bnlocal(nsame,2) = 'p- '
      bnlocal(nsame,3) = 'o- '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    123.49976d0
      bclocal(1) =    121.12976d0*CON_KCALTOK
      bclocal(2) =    -44.37844d0*CON_KCALTOK
      bclocal(3) =    -28.46323d0*CON_KCALTOK
      bclocal(4) =     58.37197d0*CON_KCALTOK
      bclocal(5) =     58.37197d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'o- '
      bnlocal(nsame,2) = 'p- '
      bnlocal(nsame,3) = 'o- '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =     99.47449d0
      bclocal(1) =     75.63337d0*CON_KCALTOK
      bclocal(2) =     -6.77694d0*CON_KCALTOK
      bclocal(3) =    -15.36849d0*CON_KCALTOK
      bclocal(4) =      0.21855d0*CON_KCALTOK
      bclocal(5) =      0.21855d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 's  '
      bnlocal(nsame,3) = 'c  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =     99.66314d0
      bclocal(1) =     80.49931d0*CON_KCALTOK
      bclocal(2) =     -7.37598d0*CON_KCALTOK
      bclocal(3) =    -16.40045d0*CON_KCALTOK
      bclocal(4) =      0.70210d0*CON_KCALTOK
      bclocal(5) =      5.15559d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -0.68142d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 's  '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =     96.70906d0
      bclocal(1) =     69.73130d0*CON_KCALTOK
      bclocal(2) =     -4.39043d0*CON_KCALTOK
      bclocal(3) =    -14.06046d0*CON_KCALTOK
      bclocal(4) =     18.20522d0*CON_KCALTOK
      bclocal(5) =      1.94371d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      0.23920d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 's  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    100.47191d0
      bclocal(1) =     71.62970d0*CON_KCALTOK
      bclocal(2) =     -7.10854d0*CON_KCALTOK
      bclocal(3) =    -14.58507d0*CON_KCALTOK
      bclocal(4) =     -3.80804d0*CON_KCALTOK
      bclocal(5) =     -3.49644d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -2.51635d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 's  '
      bnlocal(nsame,3) = 's  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =     94.37109d0
      bclocal(1) =     68.62377d0*CON_KCALTOK
      bclocal(2) =      0.00000d0*CON_KCALTOK
      bclocal(3) =      0.00000d0*CON_KCALTOK
      bclocal(4) =      9.08464d0*CON_KCALTOK
      bclocal(5) =      9.08464d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 's  '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =     97.29227d0
      bclocal(1) =     67.84173d0*CON_KCALTOK
      bclocal(2) =     -4.63847d0*CON_KCALTOK
      bclocal(3) =    -13.66676d0*CON_KCALTOK
      bclocal(4) =     -8.87261d0*CON_KCALTOK
      bclocal(5) =     30.12135d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      1.94704d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 's  '
      bnlocal(nsame,3) = 's  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    102.56840d0
      bclocal(1) =    105.36460d0*CON_KCALTOK
      bclocal(2) =    -12.73808d0*CON_KCALTOK
      bclocal(3) =    -21.77600d0*CON_KCALTOK
      bclocal(4) =     17.33395d0*CON_KCALTOK
      bclocal(5) =     17.33395d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 's" '
      bnlocal(nsame,3) = 'c  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    101.71060d0
      bclocal(1) =    107.21449d0*CON_KCALTOK
      bclocal(2) =    -12.02141d0*CON_KCALTOK
      bclocal(3) =    -22.05616d0*CON_KCALTOK
      bclocal(4) =      0.08508d0*CON_KCALTOK
      bclocal(5) =     -1.79763d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      5.09263d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 's" '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    100.95050d0
      bclocal(1) =    105.67667d0*CON_KCALTOK
      bclocal(2) =    -11.07814d0*CON_KCALTOK
      bclocal(3) =    -21.73636d0*CON_KCALTOK
      bclocal(4) =     15.39777d0*CON_KCALTOK
      bclocal(5) =     -2.28849d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -5.05339d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 's" '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    102.58400d0
      bclocal(1) =    117.10244d0*CON_KCALTOK
      bclocal(2) =    -14.17541d0*CON_KCALTOK
      bclocal(3) =    -24.20309d0*CON_KCALTOK
      bclocal(4) =    -14.99310d0*CON_KCALTOK
      bclocal(5) =     34.05793d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    -12.01948d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 's" '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    108.37770d0
      bclocal(1) =    118.96301d0*CON_KCALTOK
      bclocal(2) =    -21.51673d0*CON_KCALTOK
      bclocal(3) =    -25.15585d0*CON_KCALTOK
      bclocal(4) =     15.55039d0*CON_KCALTOK
      bclocal(5) =     64.22546d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     -5.27196d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'c  '
      bnlocal(nsame,2) = 's" '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =     99.98533d0
      bclocal(1) =    121.11564d0*CON_KCALTOK
      bclocal(2) =    -11.53728d0*CON_KCALTOK
      bclocal(3) =    -24.82531d0*CON_KCALTOK
      bclocal(4) =      4.79979d0*CON_KCALTOK
      bclocal(5) =     26.70204d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      8.99969d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 's" '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    105.30130d0
      bclocal(1) =    109.33344d0*CON_KCALTOK
      bclocal(2) =    -16.19830d0*CON_KCALTOK
      bclocal(3) =    -22.74547d0*CON_KCALTOK
      bclocal(4) =     -2.91983d0*CON_KCALTOK
      bclocal(5) =     70.28276d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     22.76736d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 's" '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =     98.76162d0
      bclocal(1) =    117.39175d0*CON_KCALTOK
      bclocal(2) =     -9.79341d0*CON_KCALTOK
      bclocal(3) =    -24.01615d0*CON_KCALTOK
      bclocal(4) =      9.42870d0*CON_KCALTOK
      bclocal(5) =      9.42870d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 's" '
      bnlocal(nsame,3) = 'h  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =     99.23194d0
      bclocal(1) =    133.51370d0*CON_KCALTOK
      bclocal(2) =    -11.74867d0*CON_KCALTOK
      bclocal(3) =    -27.34324d0*CON_KCALTOK
      bclocal(4) =     -1.63190d0*CON_KCALTOK
      bclocal(5) =     64.79539d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =    -17.83545d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 's" '
      bnlocal(nsame,3) = 'n  '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    106.60670d0
      bclocal(1) =    126.28202d0*CON_KCALTOK
      bclocal(2) =    -20.47600d0*CON_KCALTOK
      bclocal(3) =    -26.49207d0*CON_KCALTOK
      bclocal(4) =     -2.69416d0*CON_KCALTOK
      bclocal(5) =     70.65801d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     12.89275d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'h  '
      bnlocal(nsame,2) = 's" '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    111.02490d0
      bclocal(1) =    153.00188d0*CON_KCALTOK
      bclocal(2) =    -32.09071d0*CON_KCALTOK
      bclocal(3) =    -32.79437d0*CON_KCALTOK
      bclocal(4) =     82.12622d0*CON_KCALTOK
      bclocal(5) =     71.60504d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     27.95716d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'n  '
      bnlocal(nsame,2) = 's" '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =    120.24320d0
      bclocal(1) =    104.35496d0*CON_KCALTOK
      bclocal(2) =    -33.55477d0*CON_KCALTOK
      bclocal(3) =    -23.83859d0*CON_KCALTOK
      bclocal(4) =     62.91764d0*CON_KCALTOK
      bclocal(5) =     62.91764d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'o` '
      bnlocal(nsame,2) = 's" '
      bnlocal(nsame,3) = 'o` '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =     91.93673d0
      bclocal(1) =    105.41543d0*CON_KCALTOK
      bclocal(2) =     -1.91883d0*CON_KCALTOK
      bclocal(3) =    -21.28731d0*CON_KCALTOK
      bclocal(4) =      0.00000d0*CON_KCALTOK
      bclocal(5) =      0.00000d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'sp '
      bnlocal(nsame,3) = 'cp '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =     91.93673d0
      bclocal(1) =    105.41543d0*CON_KCALTOK
      bclocal(2) =     -1.91883d0*CON_KCALTOK
      bclocal(3) =    -21.28731d0*CON_KCALTOK
      bclocal(4) =      0.00000d0*CON_KCALTOK
      bclocal(5) =      0.00000d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =     10.60348d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'sp '
      bnlocal(nsame,3) = 'cpb'
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =     92.77422d0
      bclocal(1) =     86.16114d0*CON_KCALTOK
      bclocal(2) =     -2.25231d0*CON_KCALTOK
      bclocal(3) =    -17.44382d0*CON_KCALTOK
      bclocal(4) =      4.95997d0*CON_KCALTOK
      bclocal(5) =     86.82968d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      bclocal(6) =      0.00000d0*CON_KCALTOK
      call twh_lbonbon(GLB_SET,type,.true.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'cp '
      bnlocal(nsame,2) = 'sp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      type = type + 1
      call twh_bendff(GLB_SET,type,ffname)
      call twh_anglestyle(GLB_SET,type,4)
      bclocal(0) =     92.13413d0
      bclocal(1) =    214.89015d0*CON_KCALTOK
      bclocal(2) =     -4.30978d0*CON_KCALTOK
      bclocal(3) =    -43.38986d0*CON_KCALTOK
      bclocal(4) =    145.75877d0*CON_KCALTOK
      bclocal(5) =    145.75877d0*CON_KCALTOK
      call twh_lbonang(GLB_SET,type,.true.)
      call twh_lbonbon(GLB_SET,type,.false.)
      call twh_set_bencoeff(type,bclocal)
      nsame = 0
      nsame = nsame + 1
      bnlocal(nsame,1) = 'np '
      bnlocal(nsame,2) = 'sp '
      bnlocal(nsame,3) = 'np '
      call twh_set_bendnames(type,nsame,bnlocal)

      nangletypes = type
      call twh_ffcheck(lfailure,2,type)
      if ( lfailure ) return

      return
      end

      subroutine twh_ffqmff_viii_dihedral(lfailure,ffname,ndihedtypes)
c     ******************************************************************
c     * sets the parameters for the Class-2 dihedral interactions      *
c     *                                                                *
c     * originally written 05-10-2006 by M.G. Martin                   *
c     * last modified 08-04-2011 by M.G. Martin                        *
c     ******************************************************************
      implicit none
c     --- variables passed to/from the subroutine
      character*(FFNAMELEN) ffname
      logical lfailure
      integer ndihedtypes
c     --- local variables
      integer type,nsame
      character*(FFNAMELEN) tnlocal
      dimension tnlocal(MAXSAMET,MAXTNAME)
      double precision tclocal
      dimension tclocal(MINTORCOEFF:MAXTORCOEFF)

      type = 0

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.14221d0*CON_KCALTOK
      tclocal(1) =      0.05980d0*CON_KCALTOK
      tclocal(2) =     -0.25933d0*CON_KCALTOK
      tclocal(3) =     -0.08505d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =    -20.68280d0*CON_KCALTOK
      tclocal(7) =     -8.35782d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =     -0.08505d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.45191d0*CON_KCALTOK
      tclocal(13) =     -0.36499d0*CON_KCALTOK
      tclocal(14) =      0.16149d0*CON_KCALTOK
      tclocal(15) =      0.45191d0*CON_KCALTOK
      tclocal(16) =     -0.36499d0*CON_KCALTOK
      tclocal(17) =      0.16149d0*CON_KCALTOK
      tclocal(18) =    -25.63391d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =     -2.26708d0*CON_KCALTOK
      tclocal(4) =     -1.73828d0*CON_KCALTOK
      tclocal(5) =      0.14250d0*CON_KCALTOK
      tclocal(6) =    -13.30602d0*CON_KCALTOK
      tclocal(7) =     -4.70800d0*CON_KCALTOK
      tclocal(8) =     -1.86366d0*CON_KCALTOK
      tclocal( 9) =     -1.77241d0*CON_KCALTOK
      tclocal(10) =     -1.18399d0*CON_KCALTOK
      tclocal(11) =     -0.28093d0*CON_KCALTOK
      tclocal(12) =     -0.86701d0*CON_KCALTOK
      tclocal(13) =     -1.11427d0*CON_KCALTOK
      tclocal(14) =      0.98307d0*CON_KCALTOK
      tclocal(15) =     -3.00518d0*CON_KCALTOK
      tclocal(16) =     -1.48170d0*CON_KCALTOK
      tclocal(17) =      1.99316d0*CON_KCALTOK
      tclocal(18) =    -23.63997d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'c" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'c" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =      0.00900d0*CON_KCALTOK
      tclocal(4) =     -0.00033d0*CON_KCALTOK
      tclocal(5) =      0.00525d0*CON_KCALTOK
      tclocal(6) =     -2.31093d0*CON_KCALTOK
      tclocal(7) =      0.32848d0*CON_KCALTOK
      tclocal(8) =     -1.00157d0*CON_KCALTOK
      tclocal( 9) =      0.00800d0*CON_KCALTOK
      tclocal(10) =      0.00864d0*CON_KCALTOK
      tclocal(11) =     -0.00124d0*CON_KCALTOK
      tclocal(12) =     -0.37784d0*CON_KCALTOK
      tclocal(13) =      0.46418d0*CON_KCALTOK
      tclocal(14) =     -0.33094d0*CON_KCALTOK
      tclocal(15) =      0.07459d0*CON_KCALTOK
      tclocal(16) =     -0.09765d0*CON_KCALTOK
      tclocal(17) =     -0.06875d0*CON_KCALTOK
      tclocal(18) =     -0.55093d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.47142d0*CON_KCALTOK
      tclocal(1) =      0.56103d0*CON_KCALTOK
      tclocal(2) =      0.50527d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =    -23.08140d0*CON_KCALTOK
      tclocal(7) =     -8.59687d0*CON_KCALTOK
      tclocal(8) =     -3.67556d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     24.05871d0*CON_KCALTOK
      tclocal(13) =      0.18358d0*CON_KCALTOK
      tclocal(14) =      4.60531d0*CON_KCALTOK
      tclocal(15) =     -1.12062d0*CON_KCALTOK
      tclocal(16) =      3.50782d0*CON_KCALTOK
      tclocal(17) =     -2.20064d0*CON_KCALTOK
      tclocal(18) =    -40.26650d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'c- ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'c- '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10262d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =     -0.02307d0*CON_KCALTOK
      tclocal(3) =     -0.70097d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.89246d0*CON_KCALTOK
      tclocal(6) =     -2.60561d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =     -6.38019d0*CON_KCALTOK
      tclocal( 9) =      1.20414d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.05885d0*CON_KCALTOK
      tclocal(12) =      2.79379d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      1.34406d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =     -1.50000d0*CON_KCALTOK
      tclocal(18) =    -32.45658d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'c= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'c= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.14221d0*CON_KCALTOK
      tclocal(1) =      0.05980d0*CON_KCALTOK
      tclocal(2) =     -0.25933d0*CON_KCALTOK
      tclocal(3) =      1.45769d0*CON_KCALTOK
      tclocal(4) =      2.24118d0*CON_KCALTOK
      tclocal(5) =     -0.74824d0*CON_KCALTOK
      tclocal(6) =    -18.64965d0*CON_KCALTOK
      tclocal(7) =     -4.69784d0*CON_KCALTOK
      tclocal(8) =     -2.35862d0*CON_KCALTOK
      tclocal( 9) =     -0.27648d0*CON_KCALTOK
      tclocal(10) =      0.24497d0*CON_KCALTOK
      tclocal(11) =     -0.50226d0*CON_KCALTOK
      tclocal(12) =     -1.45886d0*CON_KCALTOK
      tclocal(13) =      2.57989d0*CON_KCALTOK
      tclocal(14) =      0.06492d0*CON_KCALTOK
      tclocal(15) =     -0.91417d0*CON_KCALTOK
      tclocal(16) =     -1.20340d0*CON_KCALTOK
      tclocal(17) =     -0.28224d0*CON_KCALTOK
      tclocal(18) =    -30.85178d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.14221d0*CON_KCALTOK
      tclocal(1) =      0.05980d0*CON_KCALTOK
      tclocal(2) =     -0.25933d0*CON_KCALTOK
      tclocal(3) =     -0.08505d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =    -20.68280d0*CON_KCALTOK
      tclocal(7) =     -8.35782d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =     -0.08505d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.45191d0*CON_KCALTOK
      tclocal(13) =     -0.36499d0*CON_KCALTOK
      tclocal(14) =      0.16149d0*CON_KCALTOK
      tclocal(15) =      0.45191d0*CON_KCALTOK
      tclocal(16) =     -0.36499d0*CON_KCALTOK
      tclocal(17) =      0.16149d0*CON_KCALTOK
      tclocal(18) =    -25.63391d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.03674d0*CON_KCALTOK
      tclocal(2) =     -0.20715d0*CON_KCALTOK
      tclocal(3) =      0.28905d0*CON_KCALTOK
      tclocal(4) =      0.28161d0*CON_KCALTOK
      tclocal(5) =     -0.10760d0*CON_KCALTOK
      tclocal(6) =    -17.30148d0*CON_KCALTOK
      tclocal(7) =     -4.25365d0*CON_KCALTOK
      tclocal(8) =     -0.36493d0*CON_KCALTOK
      tclocal( 9) =      0.09464d0*CON_KCALTOK
      tclocal(10) =      0.06866d0*CON_KCALTOK
      tclocal(11) =      0.25800d0*CON_KCALTOK
      tclocal(12) =     -0.28533d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =     -0.13206d0*CON_KCALTOK
      tclocal(15) =      0.36202d0*CON_KCALTOK
      tclocal(16) =      0.52506d0*CON_KCALTOK
      tclocal(17) =     -0.23118d0*CON_KCALTOK
      tclocal(18) =    -18.79537d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =      0.10756d0*CON_KCALTOK
      tclocal(4) =      0.01525d0*CON_KCALTOK
      tclocal(5) =      0.07505d0*CON_KCALTOK
      tclocal(6) =     -6.13386d0*CON_KCALTOK
      tclocal(7) =     -4.78596d0*CON_KCALTOK
      tclocal(8) =     -1.91941d0*CON_KCALTOK
      tclocal( 9) =     -0.11549d0*CON_KCALTOK
      tclocal(10) =     -0.05877d0*CON_KCALTOK
      tclocal(11) =      0.03694d0*CON_KCALTOK
      tclocal(12) =     -0.79726d0*CON_KCALTOK
      tclocal(13) =     -2.46111d0*CON_KCALTOK
      tclocal(14) =      0.36007d0*CON_KCALTOK
      tclocal(15) =      0.29544d0*CON_KCALTOK
      tclocal(16) =      0.23213d0*CON_KCALTOK
      tclocal(17) =     -1.15153d0*CON_KCALTOK
      tclocal(18) =     -1.54067d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.65141d0*CON_KCALTOK
      tclocal(1) =     -0.54647d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      1.10182d0*CON_KCALTOK
      tclocal(4) =     -0.28171d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =    -13.76484d0*CON_KCALTOK
      tclocal(7) =     -7.19831d0*CON_KCALTOK
      tclocal(8) =     -2.36838d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.46607d0*CON_KCALTOK
      tclocal(11) =      0.35820d0*CON_KCALTOK
      tclocal(12) =     -3.15063d0*CON_KCALTOK
      tclocal(13) =      2.58893d0*CON_KCALTOK
      tclocal(14) =      0.21725d0*CON_KCALTOK
      tclocal(15) =     -0.52557d0*CON_KCALTOK
      tclocal(16) =      2.20339d0*CON_KCALTOK
      tclocal(17) =      1.95558d0*CON_KCALTOK
      tclocal(18) =    -45.10989d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'n+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'n+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.14452d0*CON_KCALTOK
      tclocal(1) =     -0.18139d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.83376d0*CON_KCALTOK
      tclocal(5) =      1.15303d0*CON_KCALTOK
      tclocal(6) =    -48.58173d0*CON_KCALTOK
      tclocal(7) =     -1.45795d0*CON_KCALTOK
      tclocal(8) =      0.87988d0*CON_KCALTOK
      tclocal( 9) =      1.25715d0*CON_KCALTOK
      tclocal(10) =      1.97221d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.93347d0*CON_KCALTOK
      tclocal(13) =      4.81590d0*CON_KCALTOK
      tclocal(14) =      0.93607d0*CON_KCALTOK
      tclocal(15) =      6.04774d0*CON_KCALTOK
      tclocal(16) =      4.62548d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -49.62211d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'n= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'n= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.20516d0*CON_KCALTOK
      tclocal(1) =      0.20536d0*CON_KCALTOK
      tclocal(2) =     -0.60541d0*CON_KCALTOK
      tclocal(3) =      0.11994d0*CON_KCALTOK
      tclocal(4) =      0.68556d0*CON_KCALTOK
      tclocal(5) =     -0.56234d0*CON_KCALTOK
      tclocal(6) =    -17.92726d0*CON_KCALTOK
      tclocal(7) =     -8.49472d0*CON_KCALTOK
      tclocal(8) =     -1.24992d0*CON_KCALTOK
      tclocal( 9) =      0.06730d0*CON_KCALTOK
      tclocal(10) =     -0.00502d0*CON_KCALTOK
      tclocal(11) =     -0.22158d0*CON_KCALTOK
      tclocal(12) =     -2.23543d0*CON_KCALTOK
      tclocal(13) =     -1.56396d0*CON_KCALTOK
      tclocal(14) =      0.25694d0*CON_KCALTOK
      tclocal(15) =      2.34008d0*CON_KCALTOK
      tclocal(16) =      1.09762d0*CON_KCALTOK
      tclocal(17) =     -3.21074d0*CON_KCALTOK
      tclocal(18) =    -31.85504d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'na ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'na '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =      0.10756d0*CON_KCALTOK
      tclocal(4) =      0.01525d0*CON_KCALTOK
      tclocal(5) =      0.07505d0*CON_KCALTOK
      tclocal(6) =     -6.13386d0*CON_KCALTOK
      tclocal(7) =     -4.78596d0*CON_KCALTOK
      tclocal(8) =     -1.91941d0*CON_KCALTOK
      tclocal( 9) =     -0.11549d0*CON_KCALTOK
      tclocal(10) =     -0.05877d0*CON_KCALTOK
      tclocal(11) =      0.03694d0*CON_KCALTOK
      tclocal(12) =     -0.79726d0*CON_KCALTOK
      tclocal(13) =     -2.46111d0*CON_KCALTOK
      tclocal(14) =      0.36007d0*CON_KCALTOK
      tclocal(15) =      0.29544d0*CON_KCALTOK
      tclocal(16) =      0.23213d0*CON_KCALTOK
      tclocal(17) =     -1.15153d0*CON_KCALTOK
      tclocal(18) =     -1.54067d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =      0.10756d0*CON_KCALTOK
      tclocal(4) =      0.01525d0*CON_KCALTOK
      tclocal(5) =      0.07505d0*CON_KCALTOK
      tclocal(6) =     -6.13386d0*CON_KCALTOK
      tclocal(7) =     -4.78596d0*CON_KCALTOK
      tclocal(8) =     -1.91941d0*CON_KCALTOK
      tclocal( 9) =     -0.11549d0*CON_KCALTOK
      tclocal(10) =     -0.05877d0*CON_KCALTOK
      tclocal(11) =      0.03694d0*CON_KCALTOK
      tclocal(12) =     -0.79726d0*CON_KCALTOK
      tclocal(13) =     -2.46111d0*CON_KCALTOK
      tclocal(14) =      0.36007d0*CON_KCALTOK
      tclocal(15) =      0.29544d0*CON_KCALTOK
      tclocal(16) =      0.23213d0*CON_KCALTOK
      tclocal(17) =     -1.15153d0*CON_KCALTOK
      tclocal(18) =     -1.54067d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'nn ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'nn '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.82986d0*CON_KCALTOK
      tclocal(1) =      0.30935d0*CON_KCALTOK
      tclocal(2) =     -0.29593d0*CON_KCALTOK
      tclocal(3) =     -0.37091d0*CON_KCALTOK
      tclocal(4) =      0.51289d0*CON_KCALTOK
      tclocal(5) =     -0.83417d0*CON_KCALTOK
      tclocal(6) =    -25.44672d0*CON_KCALTOK
      tclocal(7) =     -8.92599d0*CON_KCALTOK
      tclocal(8) =     -0.79859d0*CON_KCALTOK
      tclocal( 9) =      1.34159d0*CON_KCALTOK
      tclocal(10) =      0.97775d0*CON_KCALTOK
      tclocal(11) =     -1.06252d0*CON_KCALTOK
      tclocal(12) =      0.65386d0*CON_KCALTOK
      tclocal(13) =     -0.35365d0*CON_KCALTOK
      tclocal(14) =     -0.46685d0*CON_KCALTOK
      tclocal(15) =      1.12466d0*CON_KCALTOK
      tclocal(16) =     -0.87972d0*CON_KCALTOK
      tclocal(17) =     -1.43380d0*CON_KCALTOK
      tclocal(18) =    -33.76980d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.50374d0*CON_KCALTOK
      tclocal(1) =     -0.39173d0*CON_KCALTOK
      tclocal(2) =     -0.04594d0*CON_KCALTOK
      tclocal(3) =     -0.77191d0*CON_KCALTOK
      tclocal(4) =      0.22230d0*CON_KCALTOK
      tclocal(5) =     -0.30865d0*CON_KCALTOK
      tclocal(6) =    -22.52261d0*CON_KCALTOK
      tclocal(7) =     -7.52987d0*CON_KCALTOK
      tclocal(8) =     -0.20482d0*CON_KCALTOK
      tclocal( 9) =      0.54597d0*CON_KCALTOK
      tclocal(10) =      0.18508d0*CON_KCALTOK
      tclocal(11) =     -0.13306d0*CON_KCALTOK
      tclocal(12) =     -1.96614d0*CON_KCALTOK
      tclocal(13) =      0.15048d0*CON_KCALTOK
      tclocal(14) =     -0.30300d0*CON_KCALTOK
      tclocal(15) =     -1.05002d0*CON_KCALTOK
      tclocal(16) =      0.15815d0*CON_KCALTOK
      tclocal(17) =      1.31939d0*CON_KCALTOK
      tclocal(18) =    -21.72558d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'s  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 's  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10689d0*CON_KCALTOK
      tclocal(1) =     -0.62045d0*CON_KCALTOK
      tclocal(2) =      0.48083d0*CON_KCALTOK
      tclocal(3) =     -0.55649d0*CON_KCALTOK
      tclocal(4) =     -1.05946d0*CON_KCALTOK
      tclocal(5) =      0.81814d0*CON_KCALTOK
      tclocal(6) =     -2.82209d0*CON_KCALTOK
      tclocal(7) =     -6.16468d0*CON_KCALTOK
      tclocal(8) =      1.57270d0*CON_KCALTOK
      tclocal( 9) =     -0.17154d0*CON_KCALTOK
      tclocal(10) =      0.40965d0*CON_KCALTOK
      tclocal(11) =     -0.08873d0*CON_KCALTOK
      tclocal(12) =      6.31645d0*CON_KCALTOK
      tclocal(13) =      4.60534d0*CON_KCALTOK
      tclocal(14) =     -2.44199d0*CON_KCALTOK
      tclocal(15) =     -1.50047d0*CON_KCALTOK
      tclocal(16) =     -0.83152d0*CON_KCALTOK
      tclocal(17) =      4.17067d0*CON_KCALTOK
      tclocal(18) =    -28.97368d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'s" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 's" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.43625d0*CON_KCALTOK
      tclocal(1) =     -0.29665d0*CON_KCALTOK
      tclocal(2) =     -0.15519d0*CON_KCALTOK
      tclocal(3) =      1.00204d0*CON_KCALTOK
      tclocal(4) =      0.92875d0*CON_KCALTOK
      tclocal(5) =      0.36357d0*CON_KCALTOK
      tclocal(6) =     -2.26787d0*CON_KCALTOK
      tclocal(7) =     -2.21638d0*CON_KCALTOK
      tclocal(8) =      2.20937d0*CON_KCALTOK
      tclocal( 9) =      1.00204d0*CON_KCALTOK
      tclocal(10) =      0.92875d0*CON_KCALTOK
      tclocal(11) =      0.36357d0*CON_KCALTOK
      tclocal(12) =     -1.47924d0*CON_KCALTOK
      tclocal(13) =      1.62783d0*CON_KCALTOK
      tclocal(14) =      2.46344d0*CON_KCALTOK
      tclocal(15) =     -1.47924d0*CON_KCALTOK
      tclocal(16) =      1.62783d0*CON_KCALTOK
      tclocal(17) =      2.46344d0*CON_KCALTOK
      tclocal(18) =     -4.97477d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'c" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'c" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.85389d0*CON_KCALTOK
      tclocal(1) =     -0.10358d0*CON_KCALTOK
      tclocal(2) =     -0.39016d0*CON_KCALTOK
      tclocal(3) =     -4.14461d0*CON_KCALTOK
      tclocal(4) =     -0.85874d0*CON_KCALTOK
      tclocal(5) =     -1.65037d0*CON_KCALTOK
      tclocal(6) =    -20.26343d0*CON_KCALTOK
      tclocal(7) =     -6.52401d0*CON_KCALTOK
      tclocal(8) =     -0.24647d0*CON_KCALTOK
      tclocal( 9) =     -0.53728d0*CON_KCALTOK
      tclocal(10) =     -0.59794d0*CON_KCALTOK
      tclocal(11) =     -2.30555d0*CON_KCALTOK
      tclocal(12) =      3.84066d0*CON_KCALTOK
      tclocal(13) =     -0.96955d0*CON_KCALTOK
      tclocal(14) =     -3.31285d0*CON_KCALTOK
      tclocal(15) =     -0.19074d0*CON_KCALTOK
      tclocal(16) =     -3.94694d0*CON_KCALTOK
      tclocal(17) =      1.55142d0*CON_KCALTOK
      tclocal(18) =     -1.47259d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'c- ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'c- '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.33688d0*CON_KCALTOK
      tclocal(1) =     -0.75140d0*CON_KCALTOK
      tclocal(2) =     -0.25063d0*CON_KCALTOK
      tclocal(3) =     -0.73166d0*CON_KCALTOK
      tclocal(4) =      0.23059d0*CON_KCALTOK
      tclocal(5) =     -0.57668d0*CON_KCALTOK
      tclocal(6) =      0.87883d0*CON_KCALTOK
      tclocal(7) =     -2.06579d0*CON_KCALTOK
      tclocal(8) =     -1.40656d0*CON_KCALTOK
      tclocal( 9) =     -0.10726d0*CON_KCALTOK
      tclocal(10) =      0.03156d0*CON_KCALTOK
      tclocal(11) =      0.15310d0*CON_KCALTOK
      tclocal(12) =     -0.95989d0*CON_KCALTOK
      tclocal(13) =     -0.59327d0*CON_KCALTOK
      tclocal(14) =     -0.95556d0*CON_KCALTOK
      tclocal(15) =      2.30543d0*CON_KCALTOK
      tclocal(16) =      0.19896d0*CON_KCALTOK
      tclocal(17) =      0.70869d0*CON_KCALTOK
      tclocal(18) =    -28.30247d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'c= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'c= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =     -1.77241d0*CON_KCALTOK
      tclocal(4) =     -1.18399d0*CON_KCALTOK
      tclocal(5) =     -0.28093d0*CON_KCALTOK
      tclocal(6) =    -13.30602d0*CON_KCALTOK
      tclocal(7) =     -4.70800d0*CON_KCALTOK
      tclocal(8) =     -1.86366d0*CON_KCALTOK
      tclocal( 9) =     -2.26708d0*CON_KCALTOK
      tclocal(10) =     -1.73828d0*CON_KCALTOK
      tclocal(11) =      0.14250d0*CON_KCALTOK
      tclocal(12) =     -3.00518d0*CON_KCALTOK
      tclocal(13) =     -1.48170d0*CON_KCALTOK
      tclocal(14) =      1.99316d0*CON_KCALTOK
      tclocal(15) =     -0.86701d0*CON_KCALTOK
      tclocal(16) =     -1.11427d0*CON_KCALTOK
      tclocal(17) =      0.98307d0*CON_KCALTOK
      tclocal(18) =    -23.63997d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.18242d0*CON_KCALTOK
      tclocal(1) =     -0.23609d0*CON_KCALTOK
      tclocal(2) =     -0.18597d0*CON_KCALTOK
      tclocal(3) =      0.25774d0*CON_KCALTOK
      tclocal(4) =      0.55583d0*CON_KCALTOK
      tclocal(5) =     -0.09505d0*CON_KCALTOK
      tclocal(6) =    -11.64868d0*CON_KCALTOK
      tclocal(7) =     -3.27271d0*CON_KCALTOK
      tclocal(8) =      0.19359d0*CON_KCALTOK
      tclocal( 9) =      0.10112d0*CON_KCALTOK
      tclocal(10) =      0.59797d0*CON_KCALTOK
      tclocal(11) =     -0.28462d0*CON_KCALTOK
      tclocal(12) =     -1.96862d0*CON_KCALTOK
      tclocal(13) =     -0.72694d0*CON_KCALTOK
      tclocal(14) =     -0.24971d0*CON_KCALTOK
      tclocal(15) =      0.05723d0*CON_KCALTOK
      tclocal(16) =      0.83283d0*CON_KCALTOK
      tclocal(17) =     -0.26481d0*CON_KCALTOK
      tclocal(18) =    -16.64590d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =     -0.19084d0*CON_KCALTOK
      tclocal(4) =     -0.16147d0*CON_KCALTOK
      tclocal(5) =     -0.16771d0*CON_KCALTOK
      tclocal(6) =     -1.77039d0*CON_KCALTOK
      tclocal(7) =     -5.89937d0*CON_KCALTOK
      tclocal(8) =     -4.92366d0*CON_KCALTOK
      tclocal( 9) =     -0.24426d0*CON_KCALTOK
      tclocal(10) =     -0.21441d0*CON_KCALTOK
      tclocal(11) =     -0.21433d0*CON_KCALTOK
      tclocal(12) =     -1.95858d0*CON_KCALTOK
      tclocal(13) =     -3.44855d0*CON_KCALTOK
      tclocal(14) =     -0.38258d0*CON_KCALTOK
      tclocal(15) =     -2.29159d0*CON_KCALTOK
      tclocal(16) =     -1.20225d0*CON_KCALTOK
      tclocal(17) =     -2.32980d0*CON_KCALTOK
      tclocal(18) =      0.50864d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.33554d0*CON_KCALTOK
      tclocal(1) =     -1.39688d0*CON_KCALTOK
      tclocal(2) =      0.15128d0*CON_KCALTOK
      tclocal(3) =      4.40383d0*CON_KCALTOK
      tclocal(4) =     -1.92019d0*CON_KCALTOK
      tclocal(5) =     -1.91046d0*CON_KCALTOK
      tclocal(6) =     -8.64342d0*CON_KCALTOK
      tclocal(7) =     -7.05219d0*CON_KCALTOK
      tclocal(8) =      0.02910d0*CON_KCALTOK
      tclocal( 9) =      1.87651d0*CON_KCALTOK
      tclocal(10) =     -0.36980d0*CON_KCALTOK
      tclocal(11) =     -0.07592d0*CON_KCALTOK
      tclocal(12) =     -6.13391d0*CON_KCALTOK
      tclocal(13) =      1.96670d0*CON_KCALTOK
      tclocal(14) =      2.88381d0*CON_KCALTOK
      tclocal(15) =      0.00178d0*CON_KCALTOK
      tclocal(16) =      7.62085d0*CON_KCALTOK
      tclocal(17) =      2.64794d0*CON_KCALTOK
      tclocal(18) =    -53.84060d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'n+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'n+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.04055d0*CON_KCALTOK
      tclocal(1) =      0.22838d0*CON_KCALTOK
      tclocal(2) =      0.19347d0*CON_KCALTOK
      tclocal(3) =     -1.66784d0*CON_KCALTOK
      tclocal(4) =     -0.26610d0*CON_KCALTOK
      tclocal(5) =     -0.00863d0*CON_KCALTOK
      tclocal(6) =     -7.00939d0*CON_KCALTOK
      tclocal(7) =     -5.83754d0*CON_KCALTOK
      tclocal(8) =     -2.74939d0*CON_KCALTOK
      tclocal( 9) =     -1.38381d0*CON_KCALTOK
      tclocal(10) =     -0.38665d0*CON_KCALTOK
      tclocal(11) =     -1.58973d0*CON_KCALTOK
      tclocal(12) =      0.33791d0*CON_KCALTOK
      tclocal(13) =      2.59651d0*CON_KCALTOK
      tclocal(14) =      0.41736d0*CON_KCALTOK
      tclocal(15) =      5.06774d0*CON_KCALTOK
      tclocal(16) =      2.19897d0*CON_KCALTOK
      tclocal(17) =     -1.30475d0*CON_KCALTOK
      tclocal(18) =    -40.38741d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'na ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'na '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.09980d0*CON_KCALTOK
      tclocal(1) =     -0.15344d0*CON_KCALTOK
      tclocal(2) =     -0.68710d0*CON_KCALTOK
      tclocal(3) =     -2.96774d0*CON_KCALTOK
      tclocal(4) =     -6.88835d0*CON_KCALTOK
      tclocal(5) =     -2.81680d0*CON_KCALTOK
      tclocal(6) =    -15.14783d0*CON_KCALTOK
      tclocal(7) =    -10.18856d0*CON_KCALTOK
      tclocal(8) =     -6.52142d0*CON_KCALTOK
      tclocal( 9) =     -5.63105d0*CON_KCALTOK
      tclocal(10) =     -2.83712d0*CON_KCALTOK
      tclocal(11) =     -3.78105d0*CON_KCALTOK
      tclocal(12) =     -1.76841d0*CON_KCALTOK
      tclocal(13) =     -0.29270d0*CON_KCALTOK
      tclocal(14) =     -1.73659d0*CON_KCALTOK
      tclocal(15) =      0.22716d0*CON_KCALTOK
      tclocal(16) =      3.34176d0*CON_KCALTOK
      tclocal(17) =      0.26099d0*CON_KCALTOK
      tclocal(18) =    -37.72513d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =      0.00781d0*CON_KCALTOK
      tclocal(4) =     -0.00068d0*CON_KCALTOK
      tclocal(5) =     -0.00646d0*CON_KCALTOK
      tclocal(6) =      2.13898d0*CON_KCALTOK
      tclocal(7) =      0.84807d0*CON_KCALTOK
      tclocal(8) =     -0.26999d0*CON_KCALTOK
      tclocal( 9) =      0.00781d0*CON_KCALTOK
      tclocal(10) =     -0.00068d0*CON_KCALTOK
      tclocal(11) =     -0.00646d0*CON_KCALTOK
      tclocal(12) =      5.00204d0*CON_KCALTOK
      tclocal(13) =      1.81257d0*CON_KCALTOK
      tclocal(14) =     -0.06487d0*CON_KCALTOK
      tclocal(15) =      5.00204d0*CON_KCALTOK
      tclocal(16) =      1.81257d0*CON_KCALTOK
      tclocal(17) =     -0.06487d0*CON_KCALTOK
      tclocal(18) =      0.20264d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.85389d0*CON_KCALTOK
      tclocal(1) =     -0.10358d0*CON_KCALTOK
      tclocal(2) =     -0.39016d0*CON_KCALTOK
      tclocal(3) =     -4.14461d0*CON_KCALTOK
      tclocal(4) =     -0.85874d0*CON_KCALTOK
      tclocal(5) =     -1.65037d0*CON_KCALTOK
      tclocal(6) =    -20.26343d0*CON_KCALTOK
      tclocal(7) =     -6.52401d0*CON_KCALTOK
      tclocal(8) =     -0.24647d0*CON_KCALTOK
      tclocal( 9) =     -0.53728d0*CON_KCALTOK
      tclocal(10) =     -0.59794d0*CON_KCALTOK
      tclocal(11) =     -2.30555d0*CON_KCALTOK
      tclocal(12) =      3.84066d0*CON_KCALTOK
      tclocal(13) =     -0.96955d0*CON_KCALTOK
      tclocal(14) =     -3.31285d0*CON_KCALTOK
      tclocal(15) =     -0.19074d0*CON_KCALTOK
      tclocal(16) =     -3.94694d0*CON_KCALTOK
      tclocal(17) =      1.55142d0*CON_KCALTOK
      tclocal(18) =     -1.47259d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'c- ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'c- '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.33688d0*CON_KCALTOK
      tclocal(1) =     -0.75140d0*CON_KCALTOK
      tclocal(2) =     -0.25063d0*CON_KCALTOK
      tclocal(3) =     -0.73166d0*CON_KCALTOK
      tclocal(4) =      0.23059d0*CON_KCALTOK
      tclocal(5) =     -0.57668d0*CON_KCALTOK
      tclocal(6) =      0.87883d0*CON_KCALTOK
      tclocal(7) =     -2.06579d0*CON_KCALTOK
      tclocal(8) =     -1.40656d0*CON_KCALTOK
      tclocal( 9) =     -0.10726d0*CON_KCALTOK
      tclocal(10) =      0.03156d0*CON_KCALTOK
      tclocal(11) =      0.15310d0*CON_KCALTOK
      tclocal(12) =     -0.95989d0*CON_KCALTOK
      tclocal(13) =     -0.59327d0*CON_KCALTOK
      tclocal(14) =     -0.95556d0*CON_KCALTOK
      tclocal(15) =      2.30543d0*CON_KCALTOK
      tclocal(16) =      0.19896d0*CON_KCALTOK
      tclocal(17) =      0.70869d0*CON_KCALTOK
      tclocal(18) =    -28.30247d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'c= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'c= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =      0.00800d0*CON_KCALTOK
      tclocal(4) =      0.00864d0*CON_KCALTOK
      tclocal(5) =     -0.00124d0*CON_KCALTOK
      tclocal(6) =     -2.31093d0*CON_KCALTOK
      tclocal(7) =      0.32848d0*CON_KCALTOK
      tclocal(8) =     -1.00157d0*CON_KCALTOK
      tclocal( 9) =      0.00900d0*CON_KCALTOK
      tclocal(10) =     -0.00033d0*CON_KCALTOK
      tclocal(11) =      0.00525d0*CON_KCALTOK
      tclocal(12) =      0.07459d0*CON_KCALTOK
      tclocal(13) =     -0.09765d0*CON_KCALTOK
      tclocal(14) =     -0.06875d0*CON_KCALTOK
      tclocal(15) =     -0.37784d0*CON_KCALTOK
      tclocal(16) =      0.46418d0*CON_KCALTOK
      tclocal(17) =     -0.33094d0*CON_KCALTOK
      tclocal(18) =     -0.55093d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =     -1.77241d0*CON_KCALTOK
      tclocal(4) =     -1.18399d0*CON_KCALTOK
      tclocal(5) =     -0.28093d0*CON_KCALTOK
      tclocal(6) =    -13.30602d0*CON_KCALTOK
      tclocal(7) =     -4.70800d0*CON_KCALTOK
      tclocal(8) =     -1.86366d0*CON_KCALTOK
      tclocal( 9) =     -2.26708d0*CON_KCALTOK
      tclocal(10) =     -1.73828d0*CON_KCALTOK
      tclocal(11) =      0.14250d0*CON_KCALTOK
      tclocal(12) =     -3.00518d0*CON_KCALTOK
      tclocal(13) =     -1.48170d0*CON_KCALTOK
      tclocal(14) =      1.99316d0*CON_KCALTOK
      tclocal(15) =     -0.86701d0*CON_KCALTOK
      tclocal(16) =     -1.11427d0*CON_KCALTOK
      tclocal(17) =      0.98307d0*CON_KCALTOK
      tclocal(18) =    -23.63997d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.24410d0*CON_KCALTOK
      tclocal(1) =     -0.20872d0*CON_KCALTOK
      tclocal(2) =     -0.29522d0*CON_KCALTOK
      tclocal(3) =      0.51929d0*CON_KCALTOK
      tclocal(4) =      0.69022d0*CON_KCALTOK
      tclocal(5) =     -0.18892d0*CON_KCALTOK
      tclocal(6) =     -4.02013d0*CON_KCALTOK
      tclocal(7) =     -6.71091d0*CON_KCALTOK
      tclocal(8) =     -0.88071d0*CON_KCALTOK
      tclocal( 9) =     -0.38431d0*CON_KCALTOK
      tclocal(10) =     -0.37199d0*CON_KCALTOK
      tclocal(11) =     -0.66065d0*CON_KCALTOK
      tclocal(12) =     -0.33003d0*CON_KCALTOK
      tclocal(13) =     -1.30076d0*CON_KCALTOK
      tclocal(14) =     -0.31839d0*CON_KCALTOK
      tclocal(15) =     -2.95448d0*CON_KCALTOK
      tclocal(16) =     -0.17162d0*CON_KCALTOK
      tclocal(17) =     -1.76145d0*CON_KCALTOK
      tclocal(18) =    -21.85631d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =     -0.19084d0*CON_KCALTOK
      tclocal(4) =     -0.16147d0*CON_KCALTOK
      tclocal(5) =     -0.16771d0*CON_KCALTOK
      tclocal(6) =     -1.77039d0*CON_KCALTOK
      tclocal(7) =     -5.89937d0*CON_KCALTOK
      tclocal(8) =     -4.92366d0*CON_KCALTOK
      tclocal( 9) =     -0.24426d0*CON_KCALTOK
      tclocal(10) =     -0.21441d0*CON_KCALTOK
      tclocal(11) =     -0.21433d0*CON_KCALTOK
      tclocal(12) =     -1.95858d0*CON_KCALTOK
      tclocal(13) =     -3.44855d0*CON_KCALTOK
      tclocal(14) =     -0.38258d0*CON_KCALTOK
      tclocal(15) =     -2.29159d0*CON_KCALTOK
      tclocal(16) =     -1.20225d0*CON_KCALTOK
      tclocal(17) =     -2.32980d0*CON_KCALTOK
      tclocal(18) =      0.50864d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.33554d0*CON_KCALTOK
      tclocal(1) =     -1.39688d0*CON_KCALTOK
      tclocal(2) =      0.15128d0*CON_KCALTOK
      tclocal(3) =      4.40383d0*CON_KCALTOK
      tclocal(4) =     -1.92019d0*CON_KCALTOK
      tclocal(5) =     -1.91046d0*CON_KCALTOK
      tclocal(6) =     -8.64342d0*CON_KCALTOK
      tclocal(7) =     -7.05219d0*CON_KCALTOK
      tclocal(8) =      0.02910d0*CON_KCALTOK
      tclocal( 9) =      1.87651d0*CON_KCALTOK
      tclocal(10) =     -0.36980d0*CON_KCALTOK
      tclocal(11) =     -0.07592d0*CON_KCALTOK
      tclocal(12) =     -6.13391d0*CON_KCALTOK
      tclocal(13) =      1.96670d0*CON_KCALTOK
      tclocal(14) =      2.88381d0*CON_KCALTOK
      tclocal(15) =      0.00178d0*CON_KCALTOK
      tclocal(16) =      7.62085d0*CON_KCALTOK
      tclocal(17) =      2.64794d0*CON_KCALTOK
      tclocal(18) =    -53.84060d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'n+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'n+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.04055d0*CON_KCALTOK
      tclocal(1) =      0.22838d0*CON_KCALTOK
      tclocal(2) =      0.19347d0*CON_KCALTOK
      tclocal(3) =     -1.66784d0*CON_KCALTOK
      tclocal(4) =     -0.26610d0*CON_KCALTOK
      tclocal(5) =     -0.00863d0*CON_KCALTOK
      tclocal(6) =     -7.00939d0*CON_KCALTOK
      tclocal(7) =     -5.83754d0*CON_KCALTOK
      tclocal(8) =     -2.74939d0*CON_KCALTOK
      tclocal( 9) =     -1.38381d0*CON_KCALTOK
      tclocal(10) =     -0.38665d0*CON_KCALTOK
      tclocal(11) =     -1.58973d0*CON_KCALTOK
      tclocal(12) =      0.33791d0*CON_KCALTOK
      tclocal(13) =      2.59651d0*CON_KCALTOK
      tclocal(14) =      0.41736d0*CON_KCALTOK
      tclocal(15) =      5.06774d0*CON_KCALTOK
      tclocal(16) =      2.19897d0*CON_KCALTOK
      tclocal(17) =     -1.30475d0*CON_KCALTOK
      tclocal(18) =    -40.38741d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'na ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'na '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =     -0.19084d0*CON_KCALTOK
      tclocal(4) =     -0.16147d0*CON_KCALTOK
      tclocal(5) =     -0.16771d0*CON_KCALTOK
      tclocal(6) =     -1.77039d0*CON_KCALTOK
      tclocal(7) =     -5.89937d0*CON_KCALTOK
      tclocal(8) =     -4.92366d0*CON_KCALTOK
      tclocal( 9) =     -0.24426d0*CON_KCALTOK
      tclocal(10) =     -0.21441d0*CON_KCALTOK
      tclocal(11) =     -0.21433d0*CON_KCALTOK
      tclocal(12) =     -1.95858d0*CON_KCALTOK
      tclocal(13) =     -3.44855d0*CON_KCALTOK
      tclocal(14) =     -0.38258d0*CON_KCALTOK
      tclocal(15) =     -2.29159d0*CON_KCALTOK
      tclocal(16) =     -1.20225d0*CON_KCALTOK
      tclocal(17) =     -2.32980d0*CON_KCALTOK
      tclocal(18) =      0.50864d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'nn ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'nn '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.09980d0*CON_KCALTOK
      tclocal(1) =     -0.15344d0*CON_KCALTOK
      tclocal(2) =     -0.68710d0*CON_KCALTOK
      tclocal(3) =     -2.96774d0*CON_KCALTOK
      tclocal(4) =     -6.88835d0*CON_KCALTOK
      tclocal(5) =     -2.81680d0*CON_KCALTOK
      tclocal(6) =    -15.14783d0*CON_KCALTOK
      tclocal(7) =    -10.18856d0*CON_KCALTOK
      tclocal(8) =     -6.52142d0*CON_KCALTOK
      tclocal( 9) =     -5.63105d0*CON_KCALTOK
      tclocal(10) =     -2.83712d0*CON_KCALTOK
      tclocal(11) =     -3.78105d0*CON_KCALTOK
      tclocal(12) =     -1.76841d0*CON_KCALTOK
      tclocal(13) =     -0.29270d0*CON_KCALTOK
      tclocal(14) =     -1.73659d0*CON_KCALTOK
      tclocal(15) =      0.22716d0*CON_KCALTOK
      tclocal(16) =      3.34176d0*CON_KCALTOK
      tclocal(17) =      0.26099d0*CON_KCALTOK
      tclocal(18) =    -37.72513d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.48448d0*CON_KCALTOK
      tclocal(1) =      0.69177d0*CON_KCALTOK
      tclocal(2) =      0.27658d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =    -19.77140d0*CON_KCALTOK
      tclocal(7) =     -1.28161d0*CON_KCALTOK
      tclocal(8) =      0.30700d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      2.40214d0*CON_KCALTOK
      tclocal(13) =     -0.66337d0*CON_KCALTOK
      tclocal(14) =      0.52324d0*CON_KCALTOK
      tclocal(15) =      5.69825d0*CON_KCALTOK
      tclocal(16) =      3.22244d0*CON_KCALTOK
      tclocal(17) =      0.53184d0*CON_KCALTOK
      tclocal(18) =    -18.14780d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c- ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c- '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.08238d0*CON_KCALTOK
      tclocal(1) =      0.84834d0*CON_KCALTOK
      tclocal(2) =     -0.16205d0*CON_KCALTOK
      tclocal(3) =     -1.10299d0*CON_KCALTOK
      tclocal(4) =      0.13914d0*CON_KCALTOK
      tclocal(5) =     -0.70435d0*CON_KCALTOK
      tclocal(6) =    -16.74291d0*CON_KCALTOK
      tclocal(7) =     -4.44635d0*CON_KCALTOK
      tclocal(8) =     -1.36954d0*CON_KCALTOK
      tclocal( 9) =      3.79253d0*CON_KCALTOK
      tclocal(10) =     -2.62657d0*CON_KCALTOK
      tclocal(11) =     -1.26740d0*CON_KCALTOK
      tclocal(12) =      9.57216d0*CON_KCALTOK
      tclocal(13) =      3.01366d0*CON_KCALTOK
      tclocal(14) =      1.46339d0*CON_KCALTOK
      tclocal(15) =     13.63743d0*CON_KCALTOK
      tclocal(16) =      1.19126d0*CON_KCALTOK
      tclocal(17) =     -1.93559d0*CON_KCALTOK
      tclocal(18) =    -53.00966d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c- ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'na ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c- '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'na '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      2.14673d0*CON_KCALTOK
      tclocal(1) =      2.31356d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      2.51059d0*CON_KCALTOK
      tclocal(4) =     -1.89639d0*CON_KCALTOK
      tclocal(5) =      3.73254d0*CON_KCALTOK
      tclocal(6) =    -16.32655d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      3.15677d0*CON_KCALTOK
      tclocal( 9) =      7.73731d0*CON_KCALTOK
      tclocal(10) =     -1.67854d0*CON_KCALTOK
      tclocal(11) =      0.77211d0*CON_KCALTOK
      tclocal(12) =      9.53986d0*CON_KCALTOK
      tclocal(13) =      5.99605d0*CON_KCALTOK
      tclocal(14) =      1.88237d0*CON_KCALTOK
      tclocal(15) =     18.15258d0*CON_KCALTOK
      tclocal(16) =      4.38072d0*CON_KCALTOK
      tclocal(17) =      2.43825d0*CON_KCALTOK
      tclocal(18) =    -52.15896d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c- ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c- '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =     -0.36742d0*CON_KCALTOK
      tclocal(3) =      1.18211d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.05182d0*CON_KCALTOK
      tclocal(6) =     -0.41236d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.05612d0*CON_KCALTOK
      tclocal( 9) =      1.18211d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.05182d0*CON_KCALTOK
      tclocal(12) =     -0.47129d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.50000d0*CON_KCALTOK
      tclocal(15) =     -0.47129d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.50000d0*CON_KCALTOK
      tclocal(18) =    -11.22768d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c= ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'c= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c= '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'c= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =     -0.13560d0*CON_KCALTOK
      tclocal(3) =      1.14604d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =     -0.10045d0*CON_KCALTOK
      tclocal(6) =     -5.82706d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.68549d0*CON_KCALTOK
      tclocal( 9) =      0.22724d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =     -0.10125d0*CON_KCALTOK
      tclocal(12) =      0.70735d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =     -1.01330d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -21.36254d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c= ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c= '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.96515d0*CON_KCALTOK
      tclocal(1) =     -0.49926d0*CON_KCALTOK
      tclocal(2) =     -0.28003d0*CON_KCALTOK
      tclocal(3) =      3.33767d0*CON_KCALTOK
      tclocal(4) =      0.65518d0*CON_KCALTOK
      tclocal(5) =      0.31064d0*CON_KCALTOK
      tclocal(6) =     -5.63014d0*CON_KCALTOK
      tclocal(7) =     -4.24646d0*CON_KCALTOK
      tclocal(8) =     -2.81523d0*CON_KCALTOK
      tclocal( 9) =      0.17262d0*CON_KCALTOK
      tclocal(10) =     -0.80073d0*CON_KCALTOK
      tclocal(11) =     -0.49566d0*CON_KCALTOK
      tclocal(12) =     -1.77663d0*CON_KCALTOK
      tclocal(13) =      2.02692d0*CON_KCALTOK
      tclocal(14) =      0.69392d0*CON_KCALTOK
      tclocal(15) =     -2.47577d0*CON_KCALTOK
      tclocal(16) =      1.81703d0*CON_KCALTOK
      tclocal(17) =      0.97331d0*CON_KCALTOK
      tclocal(18) =    -40.98616d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c= ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'n+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c= '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'n+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.56222d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =     -0.25982d0*CON_KCALTOK
      tclocal(3) =      0.04618d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      2.12746d0*CON_KCALTOK
      tclocal(6) =     -8.77305d0*CON_KCALTOK
      tclocal(7) =     -2.08915d0*CON_KCALTOK
      tclocal(8) =     -4.77299d0*CON_KCALTOK
      tclocal( 9) =      0.01070d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =     -0.13039d0*CON_KCALTOK
      tclocal(12) =      1.29774d0*CON_KCALTOK
      tclocal(13) =      0.80000d0*CON_KCALTOK
      tclocal(14) =     -1.53035d0*CON_KCALTOK
      tclocal(15) =      1.91501d0*CON_KCALTOK
      tclocal(16) =      1.96731d0*CON_KCALTOK
      tclocal(17) =     -0.37751d0*CON_KCALTOK
      tclocal(18) =    -20.72972d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c= ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c= '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.22990d0*CON_KCALTOK
      tclocal(1) =     -0.21175d0*CON_KCALTOK
      tclocal(2) =     -0.20715d0*CON_KCALTOK
      tclocal(3) =      0.80279d0*CON_KCALTOK
      tclocal(4) =      0.44620d0*CON_KCALTOK
      tclocal(5) =      0.70885d0*CON_KCALTOK
      tclocal(6) =    -18.80036d0*CON_KCALTOK
      tclocal(7) =     -5.75316d0*CON_KCALTOK
      tclocal(8) =     -2.48989d0*CON_KCALTOK
      tclocal( 9) =     -0.87545d0*CON_KCALTOK
      tclocal(10) =     -0.62731d0*CON_KCALTOK
      tclocal(11) =     -0.48080d0*CON_KCALTOK
      tclocal(12) =      0.04098d0*CON_KCALTOK
      tclocal(13) =     -1.64077d0*CON_KCALTOK
      tclocal(14) =     -1.53937d0*CON_KCALTOK
      tclocal(15) =     -2.51227d0*CON_KCALTOK
      tclocal(16) =      0.49202d0*CON_KCALTOK
      tclocal(17) =     -2.14991d0*CON_KCALTOK
      tclocal(18) =    -19.14161d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'co ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'co '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =      0.10756d0*CON_KCALTOK
      tclocal(4) =      0.01525d0*CON_KCALTOK
      tclocal(5) =      0.07505d0*CON_KCALTOK
      tclocal(6) =     -6.13386d0*CON_KCALTOK
      tclocal(7) =     -4.78596d0*CON_KCALTOK
      tclocal(8) =     -1.91941d0*CON_KCALTOK
      tclocal( 9) =     -0.11549d0*CON_KCALTOK
      tclocal(10) =     -0.05877d0*CON_KCALTOK
      tclocal(11) =      0.03694d0*CON_KCALTOK
      tclocal(12) =     -0.79726d0*CON_KCALTOK
      tclocal(13) =     -2.46111d0*CON_KCALTOK
      tclocal(14) =      0.36007d0*CON_KCALTOK
      tclocal(15) =      0.29544d0*CON_KCALTOK
      tclocal(16) =      0.23213d0*CON_KCALTOK
      tclocal(17) =     -1.15153d0*CON_KCALTOK
      tclocal(18) =     -1.54067d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'co ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'co '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.76787d0*CON_KCALTOK
      tclocal(1) =     -0.14113d0*CON_KCALTOK
      tclocal(2) =     -0.31559d0*CON_KCALTOK
      tclocal(3) =     -0.59568d0*CON_KCALTOK
      tclocal(4) =      0.53885d0*CON_KCALTOK
      tclocal(5) =     -0.77741d0*CON_KCALTOK
      tclocal(6) =    -32.61224d0*CON_KCALTOK
      tclocal(7) =     -7.03215d0*CON_KCALTOK
      tclocal(8) =     -1.94634d0*CON_KCALTOK
      tclocal( 9) =      3.04584d0*CON_KCALTOK
      tclocal(10) =     -0.12482d0*CON_KCALTOK
      tclocal(11) =     -1.28274d0*CON_KCALTOK
      tclocal(12) =      3.64856d0*CON_KCALTOK
      tclocal(13) =     -0.46970d0*CON_KCALTOK
      tclocal(14) =     -0.74154d0*CON_KCALTOK
      tclocal(15) =      2.41316d0*CON_KCALTOK
      tclocal(16) =      3.81803d0*CON_KCALTOK
      tclocal(17) =     -1.58422d0*CON_KCALTOK
      tclocal(18) =    -39.79369d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'co ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'co '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.14221d0*CON_KCALTOK
      tclocal(1) =      0.05980d0*CON_KCALTOK
      tclocal(2) =     -0.25933d0*CON_KCALTOK
      tclocal(3) =     -0.08505d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =    -20.68280d0*CON_KCALTOK
      tclocal(7) =     -8.35782d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =     -0.08505d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.45191d0*CON_KCALTOK
      tclocal(13) =     -0.36499d0*CON_KCALTOK
      tclocal(14) =      0.16149d0*CON_KCALTOK
      tclocal(15) =      0.45191d0*CON_KCALTOK
      tclocal(16) =     -0.36499d0*CON_KCALTOK
      tclocal(17) =      0.16149d0*CON_KCALTOK
      tclocal(18) =    -25.63391d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00527d0*CON_KCALTOK
      tclocal(1) =      0.28689d0*CON_KCALTOK
      tclocal(2) =     -0.32485d0*CON_KCALTOK
      tclocal(3) =      0.81288d0*CON_KCALTOK
      tclocal(4) =      0.87072d0*CON_KCALTOK
      tclocal(5) =      0.20260d0*CON_KCALTOK
      tclocal(6) =    -15.71712d0*CON_KCALTOK
      tclocal(7) =     -0.14479d0*CON_KCALTOK
      tclocal(8) =      0.40498d0*CON_KCALTOK
      tclocal( 9) =      0.46283d0*CON_KCALTOK
      tclocal(10) =      0.70286d0*CON_KCALTOK
      tclocal(11) =      0.00730d0*CON_KCALTOK
      tclocal(12) =     -0.25873d0*CON_KCALTOK
      tclocal(13) =      0.48962d0*CON_KCALTOK
      tclocal(14) =      0.08121d0*CON_KCALTOK
      tclocal(15) =      0.79974d0*CON_KCALTOK
      tclocal(16) =      0.91406d0*CON_KCALTOK
      tclocal(17) =     -0.54581d0*CON_KCALTOK
      tclocal(18) =    -24.29965d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =      0.10756d0*CON_KCALTOK
      tclocal(4) =      0.01525d0*CON_KCALTOK
      tclocal(5) =      0.07505d0*CON_KCALTOK
      tclocal(6) =     -6.13386d0*CON_KCALTOK
      tclocal(7) =     -4.78596d0*CON_KCALTOK
      tclocal(8) =     -1.91941d0*CON_KCALTOK
      tclocal( 9) =     -0.11549d0*CON_KCALTOK
      tclocal(10) =     -0.05877d0*CON_KCALTOK
      tclocal(11) =      0.03694d0*CON_KCALTOK
      tclocal(12) =     -0.79726d0*CON_KCALTOK
      tclocal(13) =     -2.46111d0*CON_KCALTOK
      tclocal(14) =      0.36007d0*CON_KCALTOK
      tclocal(15) =      0.29544d0*CON_KCALTOK
      tclocal(16) =      0.23213d0*CON_KCALTOK
      tclocal(17) =     -1.15153d0*CON_KCALTOK
      tclocal(18) =     -1.54067d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.65141d0*CON_KCALTOK
      tclocal(1) =     -0.54647d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      1.10182d0*CON_KCALTOK
      tclocal(4) =     -0.28171d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =    -13.76484d0*CON_KCALTOK
      tclocal(7) =     -7.19831d0*CON_KCALTOK
      tclocal(8) =     -2.36838d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.46607d0*CON_KCALTOK
      tclocal(11) =      0.35820d0*CON_KCALTOK
      tclocal(12) =     -3.15063d0*CON_KCALTOK
      tclocal(13) =      2.58893d0*CON_KCALTOK
      tclocal(14) =      0.21725d0*CON_KCALTOK
      tclocal(15) =     -0.52557d0*CON_KCALTOK
      tclocal(16) =      2.20339d0*CON_KCALTOK
      tclocal(17) =      1.95558d0*CON_KCALTOK
      tclocal(18) =    -45.10989d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'n+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'n+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.20516d0*CON_KCALTOK
      tclocal(1) =      0.20536d0*CON_KCALTOK
      tclocal(2) =     -0.60541d0*CON_KCALTOK
      tclocal(3) =      0.11994d0*CON_KCALTOK
      tclocal(4) =      0.68556d0*CON_KCALTOK
      tclocal(5) =     -0.56234d0*CON_KCALTOK
      tclocal(6) =    -17.92726d0*CON_KCALTOK
      tclocal(7) =     -8.49472d0*CON_KCALTOK
      tclocal(8) =     -1.24992d0*CON_KCALTOK
      tclocal( 9) =      0.06730d0*CON_KCALTOK
      tclocal(10) =     -0.00502d0*CON_KCALTOK
      tclocal(11) =     -0.22158d0*CON_KCALTOK
      tclocal(12) =     -2.23543d0*CON_KCALTOK
      tclocal(13) =     -1.56396d0*CON_KCALTOK
      tclocal(14) =      0.25694d0*CON_KCALTOK
      tclocal(15) =      2.34008d0*CON_KCALTOK
      tclocal(16) =      1.09762d0*CON_KCALTOK
      tclocal(17) =     -3.21074d0*CON_KCALTOK
      tclocal(18) =    -31.85504d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'na ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'na '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.82986d0*CON_KCALTOK
      tclocal(1) =      0.30935d0*CON_KCALTOK
      tclocal(2) =     -0.29593d0*CON_KCALTOK
      tclocal(3) =     -0.37091d0*CON_KCALTOK
      tclocal(4) =      0.51289d0*CON_KCALTOK
      tclocal(5) =     -0.83417d0*CON_KCALTOK
      tclocal(6) =    -25.44672d0*CON_KCALTOK
      tclocal(7) =     -8.92599d0*CON_KCALTOK
      tclocal(8) =     -0.79859d0*CON_KCALTOK
      tclocal( 9) =      1.34159d0*CON_KCALTOK
      tclocal(10) =      0.97775d0*CON_KCALTOK
      tclocal(11) =     -1.06252d0*CON_KCALTOK
      tclocal(12) =      0.65386d0*CON_KCALTOK
      tclocal(13) =     -0.35365d0*CON_KCALTOK
      tclocal(14) =     -0.46685d0*CON_KCALTOK
      tclocal(15) =      1.12466d0*CON_KCALTOK
      tclocal(16) =     -0.87972d0*CON_KCALTOK
      tclocal(17) =     -1.43380d0*CON_KCALTOK
      tclocal(18) =    -33.76980d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.12657d0*CON_KCALTOK
      tclocal(1) =     -0.04098d0*CON_KCALTOK
      tclocal(2) =     -0.18331d0*CON_KCALTOK
      tclocal(3) =      0.00672d0*CON_KCALTOK
      tclocal(4) =      0.01477d0*CON_KCALTOK
      tclocal(5) =      0.02871d0*CON_KCALTOK
      tclocal(6) =     -1.71422d0*CON_KCALTOK
      tclocal(7) =     -0.82763d0*CON_KCALTOK
      tclocal(8) =      0.16406d0*CON_KCALTOK
      tclocal( 9) =      0.02717d0*CON_KCALTOK
      tclocal(10) =      0.00144d0*CON_KCALTOK
      tclocal(11) =      0.00429d0*CON_KCALTOK
      tclocal(12) =      0.00101d0*CON_KCALTOK
      tclocal(13) =      0.03862d0*CON_KCALTOK
      tclocal(14) =     -0.09556d0*CON_KCALTOK
      tclocal(15) =     -0.07132d0*CON_KCALTOK
      tclocal(16) =      0.15945d0*CON_KCALTOK
      tclocal(17) =     -0.07593d0*CON_KCALTOK
      tclocal(18) =    -18.21247d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'ct ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'ct '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.28279d0*CON_KCALTOK
      tclocal(1) =      0.07184d0*CON_KCALTOK
      tclocal(2) =     -0.16075d0*CON_KCALTOK
      tclocal(3) =      0.24772d0*CON_KCALTOK
      tclocal(4) =      0.36276d0*CON_KCALTOK
      tclocal(5) =      0.09044d0*CON_KCALTOK
      tclocal(6) =    -16.58242d0*CON_KCALTOK
      tclocal(7) =     -0.61875d0*CON_KCALTOK
      tclocal(8) =     -0.56555d0*CON_KCALTOK
      tclocal( 9) =      0.24772d0*CON_KCALTOK
      tclocal(10) =      0.36276d0*CON_KCALTOK
      tclocal(11) =      0.09044d0*CON_KCALTOK
      tclocal(12) =     -0.94012d0*CON_KCALTOK
      tclocal(13) =      0.64758d0*CON_KCALTOK
      tclocal(14) =     -0.28665d0*CON_KCALTOK
      tclocal(15) =     -0.94012d0*CON_KCALTOK
      tclocal(16) =      0.64758d0*CON_KCALTOK
      tclocal(17) =     -0.28665d0*CON_KCALTOK
      tclocal(18) =    -14.60867d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.31486d0*CON_KCALTOK
      tclocal(1) =      0.41431d0*CON_KCALTOK
      tclocal(2) =     -0.13187d0*CON_KCALTOK
      tclocal(3) =      0.71196d0*CON_KCALTOK
      tclocal(4) =      1.55902d0*CON_KCALTOK
      tclocal(5) =     -0.29482d0*CON_KCALTOK
      tclocal(6) =     -2.71249d0*CON_KCALTOK
      tclocal(7) =     -3.19730d0*CON_KCALTOK
      tclocal(8) =     -0.75702d0*CON_KCALTOK
      tclocal( 9) =     -7.67427d0*CON_KCALTOK
      tclocal(10) =      5.85449d0*CON_KCALTOK
      tclocal(11) =     -1.34743d0*CON_KCALTOK
      tclocal(12) =     -2.70865d0*CON_KCALTOK
      tclocal(13) =      2.76802d0*CON_KCALTOK
      tclocal(14) =     -1.61829d0*CON_KCALTOK
      tclocal(15) =     -1.34561d0*CON_KCALTOK
      tclocal(16) =      2.19391d0*CON_KCALTOK
      tclocal(17) =     -2.41233d0*CON_KCALTOK
      tclocal(18) =    -27.58957d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =     -0.39931d0*CON_KCALTOK
      tclocal(2) =     -0.24955d0*CON_KCALTOK
      tclocal(3) =     -1.59861d0*CON_KCALTOK
      tclocal(4) =      0.51598d0*CON_KCALTOK
      tclocal(5) =     -1.10159d0*CON_KCALTOK
      tclocal(6) =    -11.63281d0*CON_KCALTOK
      tclocal(7) =     -2.14897d0*CON_KCALTOK
      tclocal(8) =     -0.97720d0*CON_KCALTOK
      tclocal( 9) =      1.07022d0*CON_KCALTOK
      tclocal(10) =      1.47617d0*CON_KCALTOK
      tclocal(11) =     -0.58437d0*CON_KCALTOK
      tclocal(12) =     -2.20500d0*CON_KCALTOK
      tclocal(13) =      1.73826d0*CON_KCALTOK
      tclocal(14) =     -1.61293d0*CON_KCALTOK
      tclocal(15) =     -1.74351d0*CON_KCALTOK
      tclocal(16) =      2.38796d0*CON_KCALTOK
      tclocal(17) =     -2.13294d0*CON_KCALTOK
      tclocal(18) =    -30.04941d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'n+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'n+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.29556d0*CON_KCALTOK
      tclocal(1) =     -0.15488d0*CON_KCALTOK
      tclocal(2) =     -0.15794d0*CON_KCALTOK
      tclocal(3) =      0.93140d0*CON_KCALTOK
      tclocal(4) =      0.27626d0*CON_KCALTOK
      tclocal(5) =     -0.18336d0*CON_KCALTOK
      tclocal(6) =    -50.31937d0*CON_KCALTOK
      tclocal(7) =     -1.80430d0*CON_KCALTOK
      tclocal(8) =     -0.14777d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.84644d0*CON_KCALTOK
      tclocal(11) =      0.34842d0*CON_KCALTOK
      tclocal(12) =     -0.88976d0*CON_KCALTOK
      tclocal(13) =      1.37163d0*CON_KCALTOK
      tclocal(14) =     -0.93165d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =     -0.37080d0*CON_KCALTOK
      tclocal(18) =    -23.90731d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'n= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'n= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.28228d0*CON_KCALTOK
      tclocal(1) =      0.47272d0*CON_KCALTOK
      tclocal(2) =     -0.35797d0*CON_KCALTOK
      tclocal(3) =      0.22796d0*CON_KCALTOK
      tclocal(4) =      0.82044d0*CON_KCALTOK
      tclocal(5) =      0.13026d0*CON_KCALTOK
      tclocal(6) =    -12.20448d0*CON_KCALTOK
      tclocal(7) =     -0.88914d0*CON_KCALTOK
      tclocal(8) =     -0.06339d0*CON_KCALTOK
      tclocal( 9) =      0.11882d0*CON_KCALTOK
      tclocal(10) =      0.24298d0*CON_KCALTOK
      tclocal(11) =      0.74806d0*CON_KCALTOK
      tclocal(12) =      0.59426d0*CON_KCALTOK
      tclocal(13) =      1.89857d0*CON_KCALTOK
      tclocal(14) =     -1.18078d0*CON_KCALTOK
      tclocal(15) =     -1.28776d0*CON_KCALTOK
      tclocal(16) =      0.32786d0*CON_KCALTOK
      tclocal(17) =      0.96724d0*CON_KCALTOK
      tclocal(18) =    -18.32228d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'na ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'na '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.02536d0*CON_KCALTOK
      tclocal(1) =      0.03110d0*CON_KCALTOK
      tclocal(2) =     -0.20695d0*CON_KCALTOK
      tclocal(3) =     -0.08709d0*CON_KCALTOK
      tclocal(4) =     -0.54539d0*CON_KCALTOK
      tclocal(5) =     -0.27189d0*CON_KCALTOK
      tclocal(6) =     -5.94603d0*CON_KCALTOK
      tclocal(7) =     -0.86095d0*CON_KCALTOK
      tclocal(8) =     -0.06818d0*CON_KCALTOK
      tclocal( 9) =      0.43792d0*CON_KCALTOK
      tclocal(10) =      0.36427d0*CON_KCALTOK
      tclocal(11) =      0.67258d0*CON_KCALTOK
      tclocal(12) =     -1.80714d0*CON_KCALTOK
      tclocal(13) =      2.45410d0*CON_KCALTOK
      tclocal(14) =     -1.75086d0*CON_KCALTOK
      tclocal(15) =     -2.87247d0*CON_KCALTOK
      tclocal(16) =      0.33702d0*CON_KCALTOK
      tclocal(17) =     -0.56922d0*CON_KCALTOK
      tclocal(18) =    -18.54698d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.02536d0*CON_KCALTOK
      tclocal(1) =      0.03110d0*CON_KCALTOK
      tclocal(2) =     -0.20695d0*CON_KCALTOK
      tclocal(3) =     -0.08709d0*CON_KCALTOK
      tclocal(4) =     -0.54539d0*CON_KCALTOK
      tclocal(5) =     -0.27189d0*CON_KCALTOK
      tclocal(6) =     -5.94603d0*CON_KCALTOK
      tclocal(7) =     -0.86095d0*CON_KCALTOK
      tclocal(8) =     -0.06818d0*CON_KCALTOK
      tclocal( 9) =      0.43792d0*CON_KCALTOK
      tclocal(10) =      0.36427d0*CON_KCALTOK
      tclocal(11) =      0.67258d0*CON_KCALTOK
      tclocal(12) =     -1.80714d0*CON_KCALTOK
      tclocal(13) =      2.45410d0*CON_KCALTOK
      tclocal(14) =     -1.75086d0*CON_KCALTOK
      tclocal(15) =     -2.87247d0*CON_KCALTOK
      tclocal(16) =      0.33702d0*CON_KCALTOK
      tclocal(17) =     -0.56922d0*CON_KCALTOK
      tclocal(18) =    -18.54698d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'nn ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'nn '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.16688d0*CON_KCALTOK
      tclocal(1) =      0.29418d0*CON_KCALTOK
      tclocal(2) =     -0.10522d0*CON_KCALTOK
      tclocal(3) =      1.12568d0*CON_KCALTOK
      tclocal(4) =      1.11058d0*CON_KCALTOK
      tclocal(5) =      0.05066d0*CON_KCALTOK
      tclocal(6) =    -19.53201d0*CON_KCALTOK
      tclocal(7) =     -1.42980d0*CON_KCALTOK
      tclocal(8) =     -0.31974d0*CON_KCALTOK
      tclocal( 9) =      0.68635d0*CON_KCALTOK
      tclocal(10) =      0.77547d0*CON_KCALTOK
      tclocal(11) =      0.99817d0*CON_KCALTOK
      tclocal(12) =      2.75208d0*CON_KCALTOK
      tclocal(13) =      2.89767d0*CON_KCALTOK
      tclocal(14) =     -1.17702d0*CON_KCALTOK
      tclocal(15) =     -0.22001d0*CON_KCALTOK
      tclocal(16) =      0.57189d0*CON_KCALTOK
      tclocal(17) =      0.84571d0*CON_KCALTOK
      tclocal(18) =    -23.48908d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.13896d0*CON_KCALTOK
      tclocal(1) =     -0.08200d0*CON_KCALTOK
      tclocal(2) =     -0.20910d0*CON_KCALTOK
      tclocal(3) =      0.24953d0*CON_KCALTOK
      tclocal(4) =      0.52755d0*CON_KCALTOK
      tclocal(5) =     -0.19297d0*CON_KCALTOK
      tclocal(6) =    -19.76317d0*CON_KCALTOK
      tclocal(7) =     -2.96490d0*CON_KCALTOK
      tclocal(8) =      0.11504d0*CON_KCALTOK
      tclocal( 9) =      0.30186d0*CON_KCALTOK
      tclocal(10) =      0.85858d0*CON_KCALTOK
      tclocal(11) =      0.19439d0*CON_KCALTOK
      tclocal(12) =     -1.57309d0*CON_KCALTOK
      tclocal(13) =      1.20417d0*CON_KCALTOK
      tclocal(14) =     -0.52718d0*CON_KCALTOK
      tclocal(15) =     -0.96008d0*CON_KCALTOK
      tclocal(16) =      0.32203d0*CON_KCALTOK
      tclocal(17) =     -0.67656d0*CON_KCALTOK
      tclocal(18) =    -11.93661d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'s  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 's  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.60185d0*CON_KCALTOK
      tclocal(1) =     -0.40142d0*CON_KCALTOK
      tclocal(2) =     -0.06892d0*CON_KCALTOK
      tclocal(3) =     -1.88159d0*CON_KCALTOK
      tclocal(4) =     -1.75668d0*CON_KCALTOK
      tclocal(5) =     -1.24637d0*CON_KCALTOK
      tclocal(6) =     -3.61473d0*CON_KCALTOK
      tclocal(7) =     -3.61320d0*CON_KCALTOK
      tclocal(8) =     -1.25162d0*CON_KCALTOK
      tclocal( 9) =      0.43425d0*CON_KCALTOK
      tclocal(10) =      0.69710d0*CON_KCALTOK
      tclocal(11) =     -0.28374d0*CON_KCALTOK
      tclocal(12) =     -1.93607d0*CON_KCALTOK
      tclocal(13) =     -1.40587d0*CON_KCALTOK
      tclocal(14) =     -2.13424d0*CON_KCALTOK
      tclocal(15) =     -0.90633d0*CON_KCALTOK
      tclocal(16) =     -0.82306d0*CON_KCALTOK
      tclocal(17) =     -0.76338d0*CON_KCALTOK
      tclocal(18) =    -21.61441d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'s" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 's" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.13593d0*CON_KCALTOK
      tclocal(1) =     -0.71665d0*CON_KCALTOK
      tclocal(2) =     -1.24980d0*CON_KCALTOK
      tclocal(3) =      4.01213d0*CON_KCALTOK
      tclocal(4) =      2.18608d0*CON_KCALTOK
      tclocal(5) =     -2.19316d0*CON_KCALTOK
      tclocal(6) =      6.70193d0*CON_KCALTOK
      tclocal(7) =     -4.67279d0*CON_KCALTOK
      tclocal(8) =     -6.13412d0*CON_KCALTOK
      tclocal( 9) =      4.01213d0*CON_KCALTOK
      tclocal(10) =      2.18608d0*CON_KCALTOK
      tclocal(11) =     -2.19316d0*CON_KCALTOK
      tclocal(12) =     -5.59475d0*CON_KCALTOK
      tclocal(13) =      2.00916d0*CON_KCALTOK
      tclocal(14) =     -1.33381d0*CON_KCALTOK
      tclocal(15) =     -5.59475d0*CON_KCALTOK
      tclocal(16) =      2.00916d0*CON_KCALTOK
      tclocal(17) =     -1.33381d0*CON_KCALTOK
      tclocal(18) =    -30.07128d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.31522d0*CON_KCALTOK
      tclocal(1) =      0.74890d0*CON_KCALTOK
      tclocal(2) =     -0.18022d0*CON_KCALTOK
      tclocal(3) =     -0.29233d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =     -0.37374d0*CON_KCALTOK
      tclocal(6) =     -1.42006d0*CON_KCALTOK
      tclocal(7) =     -2.95738d0*CON_KCALTOK
      tclocal(8) =     -3.86708d0*CON_KCALTOK
      tclocal( 9) =      0.78661d0*CON_KCALTOK
      tclocal(10) =      1.50073d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      1.83584d0*CON_KCALTOK
      tclocal(13) =      3.03218d0*CON_KCALTOK
      tclocal(14) =     -0.94856d0*CON_KCALTOK
      tclocal(15) =     -6.87173d0*CON_KCALTOK
      tclocal(16) =      2.61066d0*CON_KCALTOK
      tclocal(17) =     -2.76518d0*CON_KCALTOK
      tclocal(18) =    -10.77798d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'na ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'na '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.20352d0*CON_KCALTOK
      tclocal(1) =      0.76794d0*CON_KCALTOK
      tclocal(2) =     -0.79444d0*CON_KCALTOK
      tclocal(3) =      2.98084d0*CON_KCALTOK
      tclocal(4) =      2.33048d0*CON_KCALTOK
      tclocal(5) =     -0.50653d0*CON_KCALTOK
      tclocal(6) =     -6.41796d0*CON_KCALTOK
      tclocal(7) =     -3.60256d0*CON_KCALTOK
      tclocal(8) =     -4.43058d0*CON_KCALTOK
      tclocal( 9) =      4.65783d0*CON_KCALTOK
      tclocal(10) =      5.33153d0*CON_KCALTOK
      tclocal(11) =     -0.93935d0*CON_KCALTOK
      tclocal(12) =      6.89310d0*CON_KCALTOK
      tclocal(13) =      8.72196d0*CON_KCALTOK
      tclocal(14) =      1.13085d0*CON_KCALTOK
      tclocal(15) =     -0.10423d0*CON_KCALTOK
      tclocal(16) =      5.93013d0*CON_KCALTOK
      tclocal(17) =     -1.58367d0*CON_KCALTOK
      tclocal(18) =    -51.82136d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.06021d0*CON_KCALTOK
      tclocal(1) =     -0.55440d0*CON_KCALTOK
      tclocal(2) =     -0.77858d0*CON_KCALTOK
      tclocal(3) =      8.50244d0*CON_KCALTOK
      tclocal(4) =      2.25137d0*CON_KCALTOK
      tclocal(5) =      1.62280d0*CON_KCALTOK
      tclocal(6) =     -5.28763d0*CON_KCALTOK
      tclocal(7) =     -0.98554d0*CON_KCALTOK
      tclocal(8) =      7.12412d0*CON_KCALTOK
      tclocal( 9) =      1.49612d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =     -5.46000d0*CON_KCALTOK
      tclocal(12) =      1.12927d0*CON_KCALTOK
      tclocal(13) =      5.93687d0*CON_KCALTOK
      tclocal(14) =      2.75557d0*CON_KCALTOK
      tclocal(15) =     13.31708d0*CON_KCALTOK
      tclocal(16) =      6.26918d0*CON_KCALTOK
      tclocal(17) =      3.70106d0*CON_KCALTOK
      tclocal(18) =    -47.92818d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n+ ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n+ '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.50812d0*CON_KCALTOK
      tclocal(1) =     -1.16632d0*CON_KCALTOK
      tclocal(2) =     -0.42929d0*CON_KCALTOK
      tclocal(3) =     -1.56049d0*CON_KCALTOK
      tclocal(4) =      0.46293d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =    -52.76954d0*CON_KCALTOK
      tclocal(7) =     -6.41013d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =     -1.56049d0*CON_KCALTOK
      tclocal(10) =      0.46293d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -0.86142d0*CON_KCALTOK
      tclocal(14) =      0.47287d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.86142d0*CON_KCALTOK
      tclocal(17) =      0.47287d0*CON_KCALTOK
      tclocal(18) =    -23.12141d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n= ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'n= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n= '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'n= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.44244d0*CON_KCALTOK
      tclocal(1) =      0.41242d0*CON_KCALTOK
      tclocal(2) =     -0.12811d0*CON_KCALTOK
      tclocal(3) =      0.03326d0*CON_KCALTOK
      tclocal(4) =      0.06580d0*CON_KCALTOK
      tclocal(5) =     -0.05733d0*CON_KCALTOK
      tclocal(6) =     -3.89500d0*CON_KCALTOK
      tclocal(7) =      1.17945d0*CON_KCALTOK
      tclocal(8) =     -3.49561d0*CON_KCALTOK
      tclocal( 9) =      0.03326d0*CON_KCALTOK
      tclocal(10) =      0.06580d0*CON_KCALTOK
      tclocal(11) =     -0.05733d0*CON_KCALTOK
      tclocal(12) =      1.58984d0*CON_KCALTOK
      tclocal(13) =      0.52649d0*CON_KCALTOK
      tclocal(14) =     -3.22089d0*CON_KCALTOK
      tclocal(15) =      1.58984d0*CON_KCALTOK
      tclocal(16) =      0.52649d0*CON_KCALTOK
      tclocal(17) =     -3.22089d0*CON_KCALTOK
      tclocal(18) =    -13.05895d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'na ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'na ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'na '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'na '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.57878d0*CON_KCALTOK
      tclocal(1) =      0.29469d0*CON_KCALTOK
      tclocal(2) =     -0.01006d0*CON_KCALTOK
      tclocal(3) =      3.07217d0*CON_KCALTOK
      tclocal(4) =      0.84284d0*CON_KCALTOK
      tclocal(5) =      0.06724d0*CON_KCALTOK
      tclocal(6) =      0.05538d0*CON_KCALTOK
      tclocal(7) =     -2.16993d0*CON_KCALTOK
      tclocal(8) =     -2.37837d0*CON_KCALTOK
      tclocal( 9) =      1.29923d0*CON_KCALTOK
      tclocal(10) =      0.50508d0*CON_KCALTOK
      tclocal(11) =      0.30171d0*CON_KCALTOK
      tclocal(12) =     -0.69705d0*CON_KCALTOK
      tclocal(13) =      0.60531d0*CON_KCALTOK
      tclocal(14) =     -0.63781d0*CON_KCALTOK
      tclocal(15) =      0.37626d0*CON_KCALTOK
      tclocal(16) =      1.31079d0*CON_KCALTOK
      tclocal(17) =      0.32935d0*CON_KCALTOK
      tclocal(18) =     -0.30973d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'na ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'na '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.36835d0*CON_KCALTOK
      tclocal(1) =      0.32508d0*CON_KCALTOK
      tclocal(2) =      0.03926d0*CON_KCALTOK
      tclocal(3) =      0.82994d0*CON_KCALTOK
      tclocal(4) =      0.70949d0*CON_KCALTOK
      tclocal(5) =      0.25675d0*CON_KCALTOK
      tclocal(6) =     -0.20008d0*CON_KCALTOK
      tclocal(7) =     -1.08789d0*CON_KCALTOK
      tclocal(8) =     -0.98747d0*CON_KCALTOK
      tclocal( 9) =      2.56339d0*CON_KCALTOK
      tclocal(10) =      0.84285d0*CON_KCALTOK
      tclocal(11) =      0.20213d0*CON_KCALTOK
      tclocal(12) =     -0.68543d0*CON_KCALTOK
      tclocal(13) =      0.87198d0*CON_KCALTOK
      tclocal(14) =     -0.95800d0*CON_KCALTOK
      tclocal(15) =     -1.07669d0*CON_KCALTOK
      tclocal(16) =      1.72577d0*CON_KCALTOK
      tclocal(17) =      0.87527d0*CON_KCALTOK
      tclocal(18) =    -12.47870d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'na ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'nn ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'na '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'nn '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.25034d0*CON_KCALTOK
      tclocal(1) =      0.89416d0*CON_KCALTOK
      tclocal(2) =     -0.25536d0*CON_KCALTOK
      tclocal(3) =      3.16217d0*CON_KCALTOK
      tclocal(4) =      1.89381d0*CON_KCALTOK
      tclocal(5) =     -0.27736d0*CON_KCALTOK
      tclocal(6) =    -16.44345d0*CON_KCALTOK
      tclocal(7) =     -4.96942d0*CON_KCALTOK
      tclocal(8) =     -2.14598d0*CON_KCALTOK
      tclocal( 9) =      1.42424d0*CON_KCALTOK
      tclocal(10) =      1.39478d0*CON_KCALTOK
      tclocal(11) =     -0.73912d0*CON_KCALTOK
      tclocal(12) =      1.79906d0*CON_KCALTOK
      tclocal(13) =      6.17605d0*CON_KCALTOK
      tclocal(14) =     -1.65706d0*CON_KCALTOK
      tclocal(15) =      5.82925d0*CON_KCALTOK
      tclocal(16) =      3.75551d0*CON_KCALTOK
      tclocal(17) =      0.47934d0*CON_KCALTOK
      tclocal(18) =    -45.58651d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'na ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'na '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.13634d0*CON_KCALTOK
      tclocal(1) =      0.46331d0*CON_KCALTOK
      tclocal(2) =      0.35420d0*CON_KCALTOK
      tclocal(3) =      2.87503d0*CON_KCALTOK
      tclocal(4) =      2.01431d0*CON_KCALTOK
      tclocal(5) =     -1.18191d0*CON_KCALTOK
      tclocal(6) =     -4.20728d0*CON_KCALTOK
      tclocal(7) =     -2.74576d0*CON_KCALTOK
      tclocal(8) =      0.80108d0*CON_KCALTOK
      tclocal( 9) =      4.33320d0*CON_KCALTOK
      tclocal(10) =      2.91569d0*CON_KCALTOK
      tclocal(11) =      0.03588d0*CON_KCALTOK
      tclocal(12) =      0.62219d0*CON_KCALTOK
      tclocal(13) =      8.40277d0*CON_KCALTOK
      tclocal(14) =     -0.60153d0*CON_KCALTOK
      tclocal(15) =      3.04742d0*CON_KCALTOK
      tclocal(16) =      4.42904d0*CON_KCALTOK
      tclocal(17) =     -0.73392d0*CON_KCALTOK
      tclocal(18) =    -31.11571d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.09115d0*CON_KCALTOK
      tclocal(1) =     -0.11698d0*CON_KCALTOK
      tclocal(2) =     -0.67797d0*CON_KCALTOK
      tclocal(3) =      1.61519d0*CON_KCALTOK
      tclocal(4) =      0.56392d0*CON_KCALTOK
      tclocal(5) =     -0.82381d0*CON_KCALTOK
      tclocal(6) =     -5.25576d0*CON_KCALTOK
      tclocal(7) =     -3.68117d0*CON_KCALTOK
      tclocal(8) =     -3.10835d0*CON_KCALTOK
      tclocal( 9) =      1.20106d0*CON_KCALTOK
      tclocal(10) =      1.15493d0*CON_KCALTOK
      tclocal(11) =      0.31543d0*CON_KCALTOK
      tclocal(12) =      1.32784d0*CON_KCALTOK
      tclocal(13) =      1.67370d0*CON_KCALTOK
      tclocal(14) =      0.87824d0*CON_KCALTOK
      tclocal(15) =      0.36716d0*CON_KCALTOK
      tclocal(16) =      0.95114d0*CON_KCALTOK
      tclocal(17) =     -1.04243d0*CON_KCALTOK
      tclocal(18) =    -22.75322d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nn ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nn '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.21158d0*CON_KCALTOK
      tclocal(1) =     -0.12608d0*CON_KCALTOK
      tclocal(2) =     -0.81937d0*CON_KCALTOK
      tclocal(3) =      1.18198d0*CON_KCALTOK
      tclocal(4) =      0.87828d0*CON_KCALTOK
      tclocal(5) =     -0.53590d0*CON_KCALTOK
      tclocal(6) =    -20.06801d0*CON_KCALTOK
      tclocal(7) =     -4.20430d0*CON_KCALTOK
      tclocal(8) =     -0.97258d0*CON_KCALTOK
      tclocal( 9) =      1.18198d0*CON_KCALTOK
      tclocal(10) =      0.87828d0*CON_KCALTOK
      tclocal(11) =     -0.53590d0*CON_KCALTOK
      tclocal(12) =      0.64080d0*CON_KCALTOK
      tclocal(13) =      1.13219d0*CON_KCALTOK
      tclocal(14) =     -0.77596d0*CON_KCALTOK
      tclocal(15) =      0.64080d0*CON_KCALTOK
      tclocal(16) =      1.13219d0*CON_KCALTOK
      tclocal(17) =     -0.77596d0*CON_KCALTOK
      tclocal(18) =    -16.33532d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.22085d0*CON_KCALTOK
      tclocal(1) =     -0.26456d0*CON_KCALTOK
      tclocal(2) =     -0.97779d0*CON_KCALTOK
      tclocal(3) =     -2.45003d0*CON_KCALTOK
      tclocal(4) =      0.58383d0*CON_KCALTOK
      tclocal(5) =     -1.07250d0*CON_KCALTOK
      tclocal(6) =     -4.48386d0*CON_KCALTOK
      tclocal(7) =     -9.08692d0*CON_KCALTOK
      tclocal(8) =     -2.54796d0*CON_KCALTOK
      tclocal( 9) =     -1.29239d0*CON_KCALTOK
      tclocal(10) =     -2.79426d0*CON_KCALTOK
      tclocal(11) =     -0.28263d0*CON_KCALTOK
      tclocal(12) =      7.11732d0*CON_KCALTOK
      tclocal(13) =      2.65520d0*CON_KCALTOK
      tclocal(14) =     -0.31716d0*CON_KCALTOK
      tclocal(15) =      3.68307d0*CON_KCALTOK
      tclocal(16) =      1.77371d0*CON_KCALTOK
      tclocal(17) =     -2.06814d0*CON_KCALTOK
      tclocal(18) =    -43.73046d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'s" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 's" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.64969d0*CON_KCALTOK
      tclocal(1) =     -0.21869d0*CON_KCALTOK
      tclocal(2) =     -0.31631d0*CON_KCALTOK
      tclocal(3) =      0.76276d0*CON_KCALTOK
      tclocal(4) =     -0.65343d0*CON_KCALTOK
      tclocal(5) =      0.54373d0*CON_KCALTOK
      tclocal(6) =    -22.75464d0*CON_KCALTOK
      tclocal(7) =     -8.13809d0*CON_KCALTOK
      tclocal(8) =      0.12353d0*CON_KCALTOK
      tclocal( 9) =      0.76276d0*CON_KCALTOK
      tclocal(10) =     -0.65343d0*CON_KCALTOK
      tclocal(11) =      0.54373d0*CON_KCALTOK
      tclocal(12) =      0.42131d0*CON_KCALTOK
      tclocal(13) =     -0.74856d0*CON_KCALTOK
      tclocal(14) =      2.10715d0*CON_KCALTOK
      tclocal(15) =      0.42131d0*CON_KCALTOK
      tclocal(16) =     -0.74856d0*CON_KCALTOK
      tclocal(17) =      2.10715d0*CON_KCALTOK
      tclocal(18) =    -17.11560d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'s  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c  ')
      call twh_torstring(GLB_SET,type,4,'s  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 's  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c  '
      tnlocal(nsame,4) = 's  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.33769d0*CON_KCALTOK
      tclocal(1) =      1.18333d0*CON_KCALTOK
      tclocal(2) =     -0.12615d0*CON_KCALTOK
      tclocal(3) =     -0.35516d0*CON_KCALTOK
      tclocal(4) =      0.59500d0*CON_KCALTOK
      tclocal(5) =      0.13903d0*CON_KCALTOK
      tclocal(6) =    -26.94069d0*CON_KCALTOK
      tclocal(7) =     -8.96291d0*CON_KCALTOK
      tclocal(8) =     -3.19164d0*CON_KCALTOK
      tclocal( 9) =      0.00636d0*CON_KCALTOK
      tclocal(10) =     -0.24989d0*CON_KCALTOK
      tclocal(11) =      0.16993d0*CON_KCALTOK
      tclocal(12) =     13.80357d0*CON_KCALTOK
      tclocal(13) =      0.54366d0*CON_KCALTOK
      tclocal(14) =     -1.15489d0*CON_KCALTOK
      tclocal(15) =      3.84267d0*CON_KCALTOK
      tclocal(16) =      1.87401d0*CON_KCALTOK
      tclocal(17) =      0.75816d0*CON_KCALTOK
      tclocal(18) =    -12.01266d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.35521d0*CON_KCALTOK
      tclocal(1) =      0.77938d0*CON_KCALTOK
      tclocal(2) =     -0.32748d0*CON_KCALTOK
      tclocal(3) =      1.08592d0*CON_KCALTOK
      tclocal(4) =      1.44875d0*CON_KCALTOK
      tclocal(5) =      0.20938d0*CON_KCALTOK
      tclocal(6) =    -25.51389d0*CON_KCALTOK
      tclocal(7) =     -8.38171d0*CON_KCALTOK
      tclocal(8) =     -3.27379d0*CON_KCALTOK
      tclocal( 9) =     -0.36529d0*CON_KCALTOK
      tclocal(10) =     -0.59516d0*CON_KCALTOK
      tclocal(11) =      0.37045d0*CON_KCALTOK
      tclocal(12) =     12.33582d0*CON_KCALTOK
      tclocal(13) =     -1.31680d0*CON_KCALTOK
      tclocal(14) =     -0.03929d0*CON_KCALTOK
      tclocal(15) =      1.28573d0*CON_KCALTOK
      tclocal(16) =      0.29693d0*CON_KCALTOK
      tclocal(17) =     -0.44921d0*CON_KCALTOK
      tclocal(18) =    -11.50300d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.05261d0*CON_KCALTOK
      tclocal(1) =      0.05562d0*CON_KCALTOK
      tclocal(2) =     -0.07553d0*CON_KCALTOK
      tclocal(3) =     -0.38125d0*CON_KCALTOK
      tclocal(4) =     -0.01101d0*CON_KCALTOK
      tclocal(5) =     -0.16589d0*CON_KCALTOK
      tclocal(6) =     -2.90747d0*CON_KCALTOK
      tclocal(7) =     -2.23647d0*CON_KCALTOK
      tclocal(8) =      2.76745d0*CON_KCALTOK
      tclocal( 9) =     -0.39871d0*CON_KCALTOK
      tclocal(10) =     -0.44312d0*CON_KCALTOK
      tclocal(11) =     -0.25615d0*CON_KCALTOK
      tclocal(12) =      3.15972d0*CON_KCALTOK
      tclocal(13) =     -0.04851d0*CON_KCALTOK
      tclocal(14) =     -2.00230d0*CON_KCALTOK
      tclocal(15) =      3.07550d0*CON_KCALTOK
      tclocal(16) =      0.72929d0*CON_KCALTOK
      tclocal(17) =     -0.11118d0*CON_KCALTOK
      tclocal(18) =     -7.90060d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      2.13265d0*CON_KCALTOK
      tclocal(1) =      2.39567d0*CON_KCALTOK
      tclocal(2) =     -0.02272d0*CON_KCALTOK
      tclocal(3) =      1.00695d0*CON_KCALTOK
      tclocal(4) =      0.27208d0*CON_KCALTOK
      tclocal(5) =      0.33761d0*CON_KCALTOK
      tclocal(6) =    -20.14880d0*CON_KCALTOK
      tclocal(7) =     -6.73257d0*CON_KCALTOK
      tclocal(8) =     -3.73414d0*CON_KCALTOK
      tclocal( 9) =      2.80542d0*CON_KCALTOK
      tclocal(10) =     -1.26671d0*CON_KCALTOK
      tclocal(11) =      0.13983d0*CON_KCALTOK
      tclocal(12) =     12.43866d0*CON_KCALTOK
      tclocal(13) =      3.17717d0*CON_KCALTOK
      tclocal(14) =     -1.38207d0*CON_KCALTOK
      tclocal(15) =      6.10624d0*CON_KCALTOK
      tclocal(16) =      2.29666d0*CON_KCALTOK
      tclocal(17) =      0.12098d0*CON_KCALTOK
      tclocal(18) =     -1.02548d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.81269d0*CON_KCALTOK
      tclocal(1) =      1.40565d0*CON_KCALTOK
      tclocal(2) =     -0.05170d0*CON_KCALTOK
      tclocal(3) =      1.76605d0*CON_KCALTOK
      tclocal(4) =      1.00778d0*CON_KCALTOK
      tclocal(5) =      0.31215d0*CON_KCALTOK
      tclocal(6) =    -27.98704d0*CON_KCALTOK
      tclocal(7) =     13.53711d0*CON_KCALTOK
      tclocal(8) =     -2.81299d0*CON_KCALTOK
      tclocal( 9) =      0.56778d0*CON_KCALTOK
      tclocal(10) =     -0.41648d0*CON_KCALTOK
      tclocal(11) =      0.43151d0*CON_KCALTOK
      tclocal(12) =     13.41572d0*CON_KCALTOK
      tclocal(13) =     -0.49019d0*CON_KCALTOK
      tclocal(14) =     -0.09117d0*CON_KCALTOK
      tclocal(15) =      0.78843d0*CON_KCALTOK
      tclocal(16) =      1.68287d0*CON_KCALTOK
      tclocal(17) =     -0.16145d0*CON_KCALTOK
      tclocal(18) =    -28.72089d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.55446d0*CON_KCALTOK
      tclocal(1) =      0.70360d0*CON_KCALTOK
      tclocal(2) =     -0.14006d0*CON_KCALTOK
      tclocal(3) =      1.46920d0*CON_KCALTOK
      tclocal(4) =      0.84896d0*CON_KCALTOK
      tclocal(5) =      0.51804d0*CON_KCALTOK
      tclocal(6) =    -19.68213d0*CON_KCALTOK
      tclocal(7) =     -7.13739d0*CON_KCALTOK
      tclocal(8) =      1.07928d0*CON_KCALTOK
      tclocal( 9) =      0.12425d0*CON_KCALTOK
      tclocal(10) =     -0.49883d0*CON_KCALTOK
      tclocal(11) =     -0.04294d0*CON_KCALTOK
      tclocal(12) =     16.06319d0*CON_KCALTOK
      tclocal(13) =      0.13174d0*CON_KCALTOK
      tclocal(14) =      0.18620d0*CON_KCALTOK
      tclocal(15) =      2.16561d0*CON_KCALTOK
      tclocal(16) =      0.76224d0*CON_KCALTOK
      tclocal(17) =      0.21037d0*CON_KCALTOK
      tclocal(18) =    -14.96333d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.61216d0*CON_KCALTOK
      tclocal(1) =      0.42178d0*CON_KCALTOK
      tclocal(2) =     -0.20840d0*CON_KCALTOK
      tclocal(3) =      0.86212d0*CON_KCALTOK
      tclocal(4) =      0.74781d0*CON_KCALTOK
      tclocal(5) =      0.45828d0*CON_KCALTOK
      tclocal(6) =    -18.04661d0*CON_KCALTOK
      tclocal(7) =     -5.77606d0*CON_KCALTOK
      tclocal(8) =      0.95151d0*CON_KCALTOK
      tclocal( 9) =      0.30152d0*CON_KCALTOK
      tclocal(10) =     -0.10416d0*CON_KCALTOK
      tclocal(11) =      0.21973d0*CON_KCALTOK
      tclocal(12) =     14.72695d0*CON_KCALTOK
      tclocal(13) =     -1.17309d0*CON_KCALTOK
      tclocal(14) =     -0.00774d0*CON_KCALTOK
      tclocal(15) =     -0.60818d0*CON_KCALTOK
      tclocal(16) =      0.57606d0*CON_KCALTOK
      tclocal(17) =     -0.11495d0*CON_KCALTOK
      tclocal(18) =    -10.84373d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.24192d0*CON_KCALTOK
      tclocal(1) =     -0.01285d0*CON_KCALTOK
      tclocal(2) =     -0.09810d0*CON_KCALTOK
      tclocal(3) =     -0.03889d0*CON_KCALTOK
      tclocal(4) =      1.13571d0*CON_KCALTOK
      tclocal(5) =      0.00510d0*CON_KCALTOK
      tclocal(6) =      0.33275d0*CON_KCALTOK
      tclocal(7) =     -0.60136d0*CON_KCALTOK
      tclocal(8) =      1.15987d0*CON_KCALTOK
      tclocal( 9) =      0.51482d0*CON_KCALTOK
      tclocal(10) =     -0.38905d0*CON_KCALTOK
      tclocal(11) =      0.84546d0*CON_KCALTOK
      tclocal(12) =     10.28254d0*CON_KCALTOK
      tclocal(13) =      0.01086d0*CON_KCALTOK
      tclocal(14) =      1.00143d0*CON_KCALTOK
      tclocal(15) =      2.90036d0*CON_KCALTOK
      tclocal(16) =      0.73453d0*CON_KCALTOK
      tclocal(17) =      1.21822d0*CON_KCALTOK
      tclocal(18) =    -17.74155d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.73936d0*CON_KCALTOK
      tclocal(1) =      1.72176d0*CON_KCALTOK
      tclocal(2) =     -0.05091d0*CON_KCALTOK
      tclocal(3) =      0.90697d0*CON_KCALTOK
      tclocal(4) =      1.55099d0*CON_KCALTOK
      tclocal(5) =      0.38004d0*CON_KCALTOK
      tclocal(6) =    -16.00997d0*CON_KCALTOK
      tclocal(7) =     -3.01844d0*CON_KCALTOK
      tclocal(8) =      1.38765d0*CON_KCALTOK
      tclocal( 9) =      0.48368d0*CON_KCALTOK
      tclocal(10) =     -0.13260d0*CON_KCALTOK
      tclocal(11) =      0.82545d0*CON_KCALTOK
      tclocal(12) =     15.46031d0*CON_KCALTOK
      tclocal(13) =      0.93080d0*CON_KCALTOK
      tclocal(14) =     -0.00825d0*CON_KCALTOK
      tclocal(15) =     -0.02808d0*CON_KCALTOK
      tclocal(16) =      1.67752d0*CON_KCALTOK
      tclocal(17) =      0.14097d0*CON_KCALTOK
      tclocal(18) =    -16.24810d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.15983d0*CON_KCALTOK
      tclocal(1) =      1.05354d0*CON_KCALTOK
      tclocal(2) =     -0.02268d0*CON_KCALTOK
      tclocal(3) =      3.37623d0*CON_KCALTOK
      tclocal(4) =      0.61704d0*CON_KCALTOK
      tclocal(5) =      0.16732d0*CON_KCALTOK
      tclocal(6) =    -20.92620d0*CON_KCALTOK
      tclocal(7) =     11.61176d0*CON_KCALTOK
      tclocal(8) =      1.27784d0*CON_KCALTOK
      tclocal( 9) =      0.06233d0*CON_KCALTOK
      tclocal(10) =     -0.32266d0*CON_KCALTOK
      tclocal(11) =      0.17008d0*CON_KCALTOK
      tclocal(12) =     16.82885d0*CON_KCALTOK
      tclocal(13) =      0.38820d0*CON_KCALTOK
      tclocal(14) =      0.09298d0*CON_KCALTOK
      tclocal(15) =     -2.40309d0*CON_KCALTOK
      tclocal(16) =      1.11880d0*CON_KCALTOK
      tclocal(17) =     -0.34095d0*CON_KCALTOK
      tclocal(18) =    -26.96785d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.63913d0*CON_KCALTOK
      tclocal(1) =      3.80558d0*CON_KCALTOK
      tclocal(2) =     -1.96358d0*CON_KCALTOK
      tclocal(3) =     -6.32136d0*CON_KCALTOK
      tclocal(4) =      1.72467d0*CON_KCALTOK
      tclocal(5) =     -0.90461d0*CON_KCALTOK
      tclocal(6) =     -4.90876d0*CON_KCALTOK
      tclocal(7) =      2.55152d0*CON_KCALTOK
      tclocal(8) =     -4.27426d0*CON_KCALTOK
      tclocal( 9) =      0.58814d0*CON_KCALTOK
      tclocal(10) =     -2.30905d0*CON_KCALTOK
      tclocal(11) =      0.98386d0*CON_KCALTOK
      tclocal(12) =     -4.85075d0*CON_KCALTOK
      tclocal(13) =      2.41353d0*CON_KCALTOK
      tclocal(14) =     -0.30393d0*CON_KCALTOK
      tclocal(15) =      5.32581d0*CON_KCALTOK
      tclocal(16) =      4.23743d0*CON_KCALTOK
      tclocal(17) =     -1.79814d0*CON_KCALTOK
      tclocal(18) =    -14.53396d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.12746d0*CON_KCALTOK
      tclocal(1) =      0.17992d0*CON_KCALTOK
      tclocal(2) =     -0.12626d0*CON_KCALTOK
      tclocal(3) =     -0.25194d0*CON_KCALTOK
      tclocal(4) =      0.10413d0*CON_KCALTOK
      tclocal(5) =     -0.86494d0*CON_KCALTOK
      tclocal(6) =     -6.32704d0*CON_KCALTOK
      tclocal(7) =     -3.23648d0*CON_KCALTOK
      tclocal(8) =     -7.96797d0*CON_KCALTOK
      tclocal( 9) =     -0.01838d0*CON_KCALTOK
      tclocal(10) =      0.02812d0*CON_KCALTOK
      tclocal(11) =     -1.38690d0*CON_KCALTOK
      tclocal(12) =     -1.18792d0*CON_KCALTOK
      tclocal(13) =     -1.19411d0*CON_KCALTOK
      tclocal(14) =      3.56390d0*CON_KCALTOK
      tclocal(15) =     -0.06463d0*CON_KCALTOK
      tclocal(16) =      0.92422d0*CON_KCALTOK
      tclocal(17) =     -9.22307d0*CON_KCALTOK
      tclocal(18) =     -2.59240d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      3.00865d0*CON_KCALTOK
      tclocal(1) =      6.91912d0*CON_KCALTOK
      tclocal(2) =     -2.13775d0*CON_KCALTOK
      tclocal(3) =      1.45795d0*CON_KCALTOK
      tclocal(4) =     -4.22769d0*CON_KCALTOK
      tclocal(5) =      0.43182d0*CON_KCALTOK
      tclocal(6) =      0.56451d0*CON_KCALTOK
      tclocal(7) =      7.79921d0*CON_KCALTOK
      tclocal(8) =     -4.85915d0*CON_KCALTOK
      tclocal( 9) =      1.78794d0*CON_KCALTOK
      tclocal(10) =      0.63281d0*CON_KCALTOK
      tclocal(11) =     -0.27767d0*CON_KCALTOK
      tclocal(12) =      6.33059d0*CON_KCALTOK
      tclocal(13) =      6.84811d0*CON_KCALTOK
      tclocal(14) =     -1.34837d0*CON_KCALTOK
      tclocal(15) =      8.73869d0*CON_KCALTOK
      tclocal(16) =      6.03935d0*CON_KCALTOK
      tclocal(17) =     -3.53047d0*CON_KCALTOK
      tclocal(18) =    -25.09044d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.12836d0*CON_KCALTOK
      tclocal(1) =      0.17426d0*CON_KCALTOK
      tclocal(2) =      0.12922d0*CON_KCALTOK
      tclocal(3) =     -0.11049d0*CON_KCALTOK
      tclocal(4) =      0.37890d0*CON_KCALTOK
      tclocal(5) =      0.23445d0*CON_KCALTOK
      tclocal(6) =     -6.52689d0*CON_KCALTOK
      tclocal(7) =      7.09888d0*CON_KCALTOK
      tclocal(8) =      0.61943d0*CON_KCALTOK
      tclocal( 9) =      1.45601d0*CON_KCALTOK
      tclocal(10) =     -0.47934d0*CON_KCALTOK
      tclocal(11) =      0.94474d0*CON_KCALTOK
      tclocal(12) =     -2.63590d0*CON_KCALTOK
      tclocal(13) =     -4.07851d0*CON_KCALTOK
      tclocal(14) =      3.40964d0*CON_KCALTOK
      tclocal(15) =      1.42549d0*CON_KCALTOK
      tclocal(16) =      5.47705d0*CON_KCALTOK
      tclocal(17) =      7.98473d0*CON_KCALTOK
      tclocal(18) =     -9.46938d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      2.73371d0*CON_KCALTOK
      tclocal(1) =      3.85664d0*CON_KCALTOK
      tclocal(2) =      0.93907d0*CON_KCALTOK
      tclocal(3) =     -3.78725d0*CON_KCALTOK
      tclocal(4) =      0.09032d0*CON_KCALTOK
      tclocal(5) =     -0.72195d0*CON_KCALTOK
      tclocal(6) =     -2.62573d0*CON_KCALTOK
      tclocal(7) =     -0.34946d0*CON_KCALTOK
      tclocal(8) =     -6.62710d0*CON_KCALTOK
      tclocal( 9) =      1.55495d0*CON_KCALTOK
      tclocal(10) =      0.36136d0*CON_KCALTOK
      tclocal(11) =      1.69875d0*CON_KCALTOK
      tclocal(12) =      4.36192d0*CON_KCALTOK
      tclocal(13) =      1.05164d0*CON_KCALTOK
      tclocal(14) =     -0.32731d0*CON_KCALTOK
      tclocal(15) =      7.03528d0*CON_KCALTOK
      tclocal(16) =      4.55114d0*CON_KCALTOK
      tclocal(17) =      2.58773d0*CON_KCALTOK
      tclocal(18) =    -11.97806d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.04534d0*CON_KCALTOK
      tclocal(1) =      1.63396d0*CON_KCALTOK
      tclocal(2) =      0.08800d0*CON_KCALTOK
      tclocal(3) =      0.21383d0*CON_KCALTOK
      tclocal(4) =      0.44073d0*CON_KCALTOK
      tclocal(5) =     -0.56733d0*CON_KCALTOK
      tclocal(6) =     -0.67971d0*CON_KCALTOK
      tclocal(7) =      3.34357d0*CON_KCALTOK
      tclocal(8) =      4.45612d0*CON_KCALTOK
      tclocal( 9) =      0.72204d0*CON_KCALTOK
      tclocal(10) =     -1.38093d0*CON_KCALTOK
      tclocal(11) =     -0.35575d0*CON_KCALTOK
      tclocal(12) =     -0.21612d0*CON_KCALTOK
      tclocal(13) =      0.96304d0*CON_KCALTOK
      tclocal(14) =     -0.35580d0*CON_KCALTOK
      tclocal(15) =      0.57311d0*CON_KCALTOK
      tclocal(16) =     -0.40679d0*CON_KCALTOK
      tclocal(17) =     -0.30013d0*CON_KCALTOK
      tclocal(18) =     -2.34077d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      3.81342d0*CON_KCALTOK
      tclocal(1) =      5.94375d0*CON_KCALTOK
      tclocal(2) =      0.63170d0*CON_KCALTOK
      tclocal(3) =      3.43652d0*CON_KCALTOK
      tclocal(4) =      5.94793d0*CON_KCALTOK
      tclocal(5) =     -4.24991d0*CON_KCALTOK
      tclocal(6) =      3.22716d0*CON_KCALTOK
      tclocal(7) =      1.14950d0*CON_KCALTOK
      tclocal(8) =     -4.68802d0*CON_KCALTOK
      tclocal( 9) =      7.08579d0*CON_KCALTOK
      tclocal(10) =      4.56319d0*CON_KCALTOK
      tclocal(11) =     -5.15371d0*CON_KCALTOK
      tclocal(12) =      7.34843d0*CON_KCALTOK
      tclocal(13) =      5.10437d0*CON_KCALTOK
      tclocal(14) =      0.87439d0*CON_KCALTOK
      tclocal(15) =     14.10817d0*CON_KCALTOK
      tclocal(16) =      6.30586d0*CON_KCALTOK
      tclocal(17) =     -0.62094d0*CON_KCALTOK
      tclocal(18) =    -24.14843d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.74928d0*CON_KCALTOK
      tclocal(1) =      0.91829d0*CON_KCALTOK
      tclocal(2) =      1.23298d0*CON_KCALTOK
      tclocal(3) =      1.01878d0*CON_KCALTOK
      tclocal(4) =      0.45294d0*CON_KCALTOK
      tclocal(5) =     -0.69073d0*CON_KCALTOK
      tclocal(6) =     -6.21908d0*CON_KCALTOK
      tclocal(7) =      8.64646d0*CON_KCALTOK
      tclocal(8) =     -7.84346d0*CON_KCALTOK
      tclocal( 9) =      3.50242d0*CON_KCALTOK
      tclocal(10) =     -3.27998d0*CON_KCALTOK
      tclocal(11) =      2.03402d0*CON_KCALTOK
      tclocal(12) =      1.31820d0*CON_KCALTOK
      tclocal(13) =     -1.36388d0*CON_KCALTOK
      tclocal(14) =      1.55122d0*CON_KCALTOK
      tclocal(15) =      1.12532d0*CON_KCALTOK
      tclocal(16) =     -0.23229d0*CON_KCALTOK
      tclocal(17) =     -0.38227d0*CON_KCALTOK
      tclocal(18) =    -27.45814d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c" ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c" '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.33769d0*CON_KCALTOK
      tclocal(1) =      1.18333d0*CON_KCALTOK
      tclocal(2) =     -0.12615d0*CON_KCALTOK
      tclocal(3) =     -0.35516d0*CON_KCALTOK
      tclocal(4) =      0.59500d0*CON_KCALTOK
      tclocal(5) =      0.13903d0*CON_KCALTOK
      tclocal(6) =    -26.94069d0*CON_KCALTOK
      tclocal(7) =     -8.96291d0*CON_KCALTOK
      tclocal(8) =     -3.19164d0*CON_KCALTOK
      tclocal( 9) =      0.00636d0*CON_KCALTOK
      tclocal(10) =     -0.24989d0*CON_KCALTOK
      tclocal(11) =      0.16993d0*CON_KCALTOK
      tclocal(12) =     13.80357d0*CON_KCALTOK
      tclocal(13) =      0.54366d0*CON_KCALTOK
      tclocal(14) =     -1.15489d0*CON_KCALTOK
      tclocal(15) =      3.84267d0*CON_KCALTOK
      tclocal(16) =      1.87401d0*CON_KCALTOK
      tclocal(17) =      0.75816d0*CON_KCALTOK
      tclocal(18) =    -12.01266d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.35521d0*CON_KCALTOK
      tclocal(1) =      0.77938d0*CON_KCALTOK
      tclocal(2) =     -0.32748d0*CON_KCALTOK
      tclocal(3) =      1.08592d0*CON_KCALTOK
      tclocal(4) =      1.44875d0*CON_KCALTOK
      tclocal(5) =      0.20938d0*CON_KCALTOK
      tclocal(6) =    -25.51389d0*CON_KCALTOK
      tclocal(7) =     -8.38171d0*CON_KCALTOK
      tclocal(8) =     -3.27379d0*CON_KCALTOK
      tclocal( 9) =     -0.36529d0*CON_KCALTOK
      tclocal(10) =     -0.59516d0*CON_KCALTOK
      tclocal(11) =      0.37045d0*CON_KCALTOK
      tclocal(12) =     12.33582d0*CON_KCALTOK
      tclocal(13) =     -1.31680d0*CON_KCALTOK
      tclocal(14) =     -0.03929d0*CON_KCALTOK
      tclocal(15) =      1.28573d0*CON_KCALTOK
      tclocal(16) =      0.29693d0*CON_KCALTOK
      tclocal(17) =     -0.44921d0*CON_KCALTOK
      tclocal(18) =    -11.50300d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.29338d0*CON_KCALTOK
      tclocal(1) =      0.76749d0*CON_KCALTOK
      tclocal(2) =      0.00469d0*CON_KCALTOK
      tclocal(3) =     -0.40118d0*CON_KCALTOK
      tclocal(4) =      0.29778d0*CON_KCALTOK
      tclocal(5) =      0.04581d0*CON_KCALTOK
      tclocal(6) =     -1.13275d0*CON_KCALTOK
      tclocal(7) =      1.24065d0*CON_KCALTOK
      tclocal(8) =     -0.13507d0*CON_KCALTOK
      tclocal( 9) =     -0.79589d0*CON_KCALTOK
      tclocal(10) =      0.24219d0*CON_KCALTOK
      tclocal(11) =     -0.05267d0*CON_KCALTOK
      tclocal(12) =     14.47648d0*CON_KCALTOK
      tclocal(13) =      1.15867d0*CON_KCALTOK
      tclocal(14) =     -0.25131d0*CON_KCALTOK
      tclocal(15) =     -0.60453d0*CON_KCALTOK
      tclocal(16) =      2.47987d0*CON_KCALTOK
      tclocal(17) =      0.06113d0*CON_KCALTOK
      tclocal(18) =    -28.36487d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      2.13265d0*CON_KCALTOK
      tclocal(1) =      2.39567d0*CON_KCALTOK
      tclocal(2) =     -0.02272d0*CON_KCALTOK
      tclocal(3) =      1.00695d0*CON_KCALTOK
      tclocal(4) =      0.27208d0*CON_KCALTOK
      tclocal(5) =      0.33761d0*CON_KCALTOK
      tclocal(6) =    -20.14880d0*CON_KCALTOK
      tclocal(7) =     -6.73257d0*CON_KCALTOK
      tclocal(8) =     -3.73414d0*CON_KCALTOK
      tclocal( 9) =      2.80542d0*CON_KCALTOK
      tclocal(10) =     -1.26671d0*CON_KCALTOK
      tclocal(11) =      0.13983d0*CON_KCALTOK
      tclocal(12) =     12.43866d0*CON_KCALTOK
      tclocal(13) =      3.17717d0*CON_KCALTOK
      tclocal(14) =     -1.38207d0*CON_KCALTOK
      tclocal(15) =      6.10624d0*CON_KCALTOK
      tclocal(16) =      2.29666d0*CON_KCALTOK
      tclocal(17) =      0.12098d0*CON_KCALTOK
      tclocal(18) =     -1.02548d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.61391d0*CON_KCALTOK
      tclocal(1) =      0.71607d0*CON_KCALTOK
      tclocal(2) =      0.15942d0*CON_KCALTOK
      tclocal(3) =      1.15211d0*CON_KCALTOK
      tclocal(4) =      1.30397d0*CON_KCALTOK
      tclocal(5) =      0.19463d0*CON_KCALTOK
      tclocal(6) =      0.58411d0*CON_KCALTOK
      tclocal(7) =      1.69411d0*CON_KCALTOK
      tclocal(8) =      0.81700d0*CON_KCALTOK
      tclocal( 9) =      0.88401d0*CON_KCALTOK
      tclocal(10) =      0.01129d0*CON_KCALTOK
      tclocal(11) =      0.18591d0*CON_KCALTOK
      tclocal(12) =     19.47589d0*CON_KCALTOK
      tclocal(13) =      1.93012d0*CON_KCALTOK
      tclocal(14) =      0.84611d0*CON_KCALTOK
      tclocal(15) =      1.86951d0*CON_KCALTOK
      tclocal(16) =      2.52036d0*CON_KCALTOK
      tclocal(17) =      1.18221d0*CON_KCALTOK
      tclocal(18) =    -41.68310d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.06313d0*CON_KCALTOK
      tclocal(1) =      0.04173d0*CON_KCALTOK
      tclocal(2) =      0.08025d0*CON_KCALTOK
      tclocal(3) =      0.38463d0*CON_KCALTOK
      tclocal(4) =      0.07290d0*CON_KCALTOK
      tclocal(5) =      0.15160d0*CON_KCALTOK
      tclocal(6) =      0.49102d0*CON_KCALTOK
      tclocal(7) =     -0.15883d0*CON_KCALTOK
      tclocal(8) =      0.17660d0*CON_KCALTOK
      tclocal( 9) =     -0.40720d0*CON_KCALTOK
      tclocal(10) =      0.11830d0*CON_KCALTOK
      tclocal(11) =     -0.22051d0*CON_KCALTOK
      tclocal(12) =      0.12831d0*CON_KCALTOK
      tclocal(13) =     -1.98595d0*CON_KCALTOK
      tclocal(14) =     -0.79020d0*CON_KCALTOK
      tclocal(15) =      0.97822d0*CON_KCALTOK
      tclocal(16) =      0.86445d0*CON_KCALTOK
      tclocal(17) =      0.97466d0*CON_KCALTOK
      tclocal(18) =    -11.62172d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'co ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'co '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.55446d0*CON_KCALTOK
      tclocal(1) =      0.70360d0*CON_KCALTOK
      tclocal(2) =     -0.14006d0*CON_KCALTOK
      tclocal(3) =      1.46920d0*CON_KCALTOK
      tclocal(4) =      0.84896d0*CON_KCALTOK
      tclocal(5) =      0.51804d0*CON_KCALTOK
      tclocal(6) =    -19.68213d0*CON_KCALTOK
      tclocal(7) =     -7.13739d0*CON_KCALTOK
      tclocal(8) =      1.07928d0*CON_KCALTOK
      tclocal( 9) =      0.12425d0*CON_KCALTOK
      tclocal(10) =     -0.49883d0*CON_KCALTOK
      tclocal(11) =     -0.04294d0*CON_KCALTOK
      tclocal(12) =     16.06319d0*CON_KCALTOK
      tclocal(13) =      0.13174d0*CON_KCALTOK
      tclocal(14) =      0.18620d0*CON_KCALTOK
      tclocal(15) =      2.16561d0*CON_KCALTOK
      tclocal(16) =      0.76224d0*CON_KCALTOK
      tclocal(17) =      0.21037d0*CON_KCALTOK
      tclocal(18) =    -14.96333d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.61216d0*CON_KCALTOK
      tclocal(1) =      0.42178d0*CON_KCALTOK
      tclocal(2) =     -0.20840d0*CON_KCALTOK
      tclocal(3) =      0.86212d0*CON_KCALTOK
      tclocal(4) =      0.74781d0*CON_KCALTOK
      tclocal(5) =      0.45828d0*CON_KCALTOK
      tclocal(6) =    -18.04661d0*CON_KCALTOK
      tclocal(7) =     -5.77606d0*CON_KCALTOK
      tclocal(8) =      0.95151d0*CON_KCALTOK
      tclocal( 9) =      0.30152d0*CON_KCALTOK
      tclocal(10) =     -0.10416d0*CON_KCALTOK
      tclocal(11) =      0.21973d0*CON_KCALTOK
      tclocal(12) =     14.72695d0*CON_KCALTOK
      tclocal(13) =     -1.17309d0*CON_KCALTOK
      tclocal(14) =     -0.00774d0*CON_KCALTOK
      tclocal(15) =     -0.60818d0*CON_KCALTOK
      tclocal(16) =      0.57606d0*CON_KCALTOK
      tclocal(17) =     -0.11495d0*CON_KCALTOK
      tclocal(18) =    -10.84373d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.84384d0*CON_KCALTOK
      tclocal(1) =      0.75433d0*CON_KCALTOK
      tclocal(2) =     -0.18643d0*CON_KCALTOK
      tclocal(3) =      3.76903d0*CON_KCALTOK
      tclocal(4) =      0.79275d0*CON_KCALTOK
      tclocal(5) =     -0.10464d0*CON_KCALTOK
      tclocal(6) =     -0.83830d0*CON_KCALTOK
      tclocal(7) =      1.52105d0*CON_KCALTOK
      tclocal(8) =      0.49068d0*CON_KCALTOK
      tclocal( 9) =      0.36660d0*CON_KCALTOK
      tclocal(10) =     -0.07795d0*CON_KCALTOK
      tclocal(11) =      0.67095d0*CON_KCALTOK
      tclocal(12) =     10.26239d0*CON_KCALTOK
      tclocal(13) =      0.58360d0*CON_KCALTOK
      tclocal(14) =      0.10934d0*CON_KCALTOK
      tclocal(15) =     -1.71192d0*CON_KCALTOK
      tclocal(16) =      1.42394d0*CON_KCALTOK
      tclocal(17) =      0.20894d0*CON_KCALTOK
      tclocal(18) =    -21.63856d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.73936d0*CON_KCALTOK
      tclocal(1) =      1.72176d0*CON_KCALTOK
      tclocal(2) =     -0.05091d0*CON_KCALTOK
      tclocal(3) =      0.90697d0*CON_KCALTOK
      tclocal(4) =      1.55099d0*CON_KCALTOK
      tclocal(5) =      0.38004d0*CON_KCALTOK
      tclocal(6) =    -16.00997d0*CON_KCALTOK
      tclocal(7) =     -3.01844d0*CON_KCALTOK
      tclocal(8) =      1.38765d0*CON_KCALTOK
      tclocal( 9) =      0.48368d0*CON_KCALTOK
      tclocal(10) =     -0.13260d0*CON_KCALTOK
      tclocal(11) =      0.82545d0*CON_KCALTOK
      tclocal(12) =     15.46031d0*CON_KCALTOK
      tclocal(13) =      0.93080d0*CON_KCALTOK
      tclocal(14) =     -0.00825d0*CON_KCALTOK
      tclocal(15) =     -0.02808d0*CON_KCALTOK
      tclocal(16) =      1.67752d0*CON_KCALTOK
      tclocal(17) =      0.14097d0*CON_KCALTOK
      tclocal(18) =    -16.24810d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.08342d0*CON_KCALTOK
      tclocal(1) =      0.59960d0*CON_KCALTOK
      tclocal(2) =     -0.00439d0*CON_KCALTOK
      tclocal(3) =      5.62180d0*CON_KCALTOK
      tclocal(4) =      0.85136d0*CON_KCALTOK
      tclocal(5) =     -0.11627d0*CON_KCALTOK
      tclocal(6) =     -0.22519d0*CON_KCALTOK
      tclocal(7) =      1.87715d0*CON_KCALTOK
      tclocal(8) =      0.06226d0*CON_KCALTOK
      tclocal( 9) =      1.47826d0*CON_KCALTOK
      tclocal(10) =     -0.01743d0*CON_KCALTOK
      tclocal(11) =      0.06670d0*CON_KCALTOK
      tclocal(12) =     15.36986d0*CON_KCALTOK
      tclocal(13) =      0.30028d0*CON_KCALTOK
      tclocal(14) =      0.22246d0*CON_KCALTOK
      tclocal(15) =     -2.41777d0*CON_KCALTOK
      tclocal(16) =      0.96787d0*CON_KCALTOK
      tclocal(17) =     -0.56720d0*CON_KCALTOK
      tclocal(18) =    -27.61263d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.00627d0*CON_KCALTOK
      tclocal(1) =      1.30823d0*CON_KCALTOK
      tclocal(2) =     -0.10639d0*CON_KCALTOK
      tclocal(3) =      4.11910d0*CON_KCALTOK
      tclocal(4) =     -0.18750d0*CON_KCALTOK
      tclocal(5) =     -0.52442d0*CON_KCALTOK
      tclocal(6) =      2.18907d0*CON_KCALTOK
      tclocal(7) =      2.50717d0*CON_KCALTOK
      tclocal(8) =     -0.50233d0*CON_KCALTOK
      tclocal( 9) =      1.91640d0*CON_KCALTOK
      tclocal(10) =     -0.35016d0*CON_KCALTOK
      tclocal(11) =     -0.53969d0*CON_KCALTOK
      tclocal(12) =      9.77889d0*CON_KCALTOK
      tclocal(13) =      0.72675d0*CON_KCALTOK
      tclocal(14) =     -0.43656d0*CON_KCALTOK
      tclocal(15) =      2.05132d0*CON_KCALTOK
      tclocal(16) =      3.08261d0*CON_KCALTOK
      tclocal(17) =      0.01755d0*CON_KCALTOK
      tclocal(18) =    -26.92172d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.74647d0*CON_KCALTOK
      tclocal(1) =      1.26422d0*CON_KCALTOK
      tclocal(2) =     -0.16475d0*CON_KCALTOK
      tclocal(3) =      9.03488d0*CON_KCALTOK
      tclocal(4) =      1.18835d0*CON_KCALTOK
      tclocal(5) =     -0.07870d0*CON_KCALTOK
      tclocal(6) =     -1.30742d0*CON_KCALTOK
      tclocal(7) =      2.84241d0*CON_KCALTOK
      tclocal(8) =     -0.01517d0*CON_KCALTOK
      tclocal( 9) =      2.01759d0*CON_KCALTOK
      tclocal(10) =      0.48649d0*CON_KCALTOK
      tclocal(11) =     -0.01445d0*CON_KCALTOK
      tclocal(12) =     14.39259d0*CON_KCALTOK
      tclocal(13) =      0.35493d0*CON_KCALTOK
      tclocal(14) =      0.28551d0*CON_KCALTOK
      tclocal(15) =     -4.76235d0*CON_KCALTOK
      tclocal(16) =      2.45669d0*CON_KCALTOK
      tclocal(17) =      0.17298d0*CON_KCALTOK
      tclocal(18) =    -33.13403d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.12836d0*CON_KCALTOK
      tclocal(1) =      0.17426d0*CON_KCALTOK
      tclocal(2) =      0.12922d0*CON_KCALTOK
      tclocal(3) =     -0.11049d0*CON_KCALTOK
      tclocal(4) =      0.37890d0*CON_KCALTOK
      tclocal(5) =      0.23445d0*CON_KCALTOK
      tclocal(6) =     -6.52689d0*CON_KCALTOK
      tclocal(7) =      7.09888d0*CON_KCALTOK
      tclocal(8) =      0.61943d0*CON_KCALTOK
      tclocal( 9) =      1.45601d0*CON_KCALTOK
      tclocal(10) =     -0.47934d0*CON_KCALTOK
      tclocal(11) =      0.94474d0*CON_KCALTOK
      tclocal(12) =     -2.63590d0*CON_KCALTOK
      tclocal(13) =     -4.07851d0*CON_KCALTOK
      tclocal(14) =      3.40964d0*CON_KCALTOK
      tclocal(15) =      1.42549d0*CON_KCALTOK
      tclocal(16) =      5.47705d0*CON_KCALTOK
      tclocal(17) =      7.98473d0*CON_KCALTOK
      tclocal(18) =     -9.46938d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nn ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nn '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.04534d0*CON_KCALTOK
      tclocal(1) =      1.63396d0*CON_KCALTOK
      tclocal(2) =      0.08800d0*CON_KCALTOK
      tclocal(3) =      0.21383d0*CON_KCALTOK
      tclocal(4) =      0.44073d0*CON_KCALTOK
      tclocal(5) =     -0.56733d0*CON_KCALTOK
      tclocal(6) =     -0.67971d0*CON_KCALTOK
      tclocal(7) =      3.34357d0*CON_KCALTOK
      tclocal(8) =      4.45612d0*CON_KCALTOK
      tclocal( 9) =      0.72204d0*CON_KCALTOK
      tclocal(10) =     -1.38093d0*CON_KCALTOK
      tclocal(11) =     -0.35575d0*CON_KCALTOK
      tclocal(12) =     -0.21612d0*CON_KCALTOK
      tclocal(13) =      0.96304d0*CON_KCALTOK
      tclocal(14) =     -0.35580d0*CON_KCALTOK
      tclocal(15) =      0.57311d0*CON_KCALTOK
      tclocal(16) =     -0.40679d0*CON_KCALTOK
      tclocal(17) =     -0.30013d0*CON_KCALTOK
      tclocal(18) =     -2.34077d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      2.48190d0*CON_KCALTOK
      tclocal(1) =      2.25798d0*CON_KCALTOK
      tclocal(2) =      1.29601d0*CON_KCALTOK
      tclocal(3) =      4.95868d0*CON_KCALTOK
      tclocal(4) =      9.28925d0*CON_KCALTOK
      tclocal(5) =      3.45596d0*CON_KCALTOK
      tclocal(6) =     -2.69204d0*CON_KCALTOK
      tclocal(7) =     -1.31267d0*CON_KCALTOK
      tclocal(8) =     -1.39124d0*CON_KCALTOK
      tclocal( 9) =      1.34411d0*CON_KCALTOK
      tclocal(10) =      8.11661d0*CON_KCALTOK
      tclocal(11) =      1.41463d0*CON_KCALTOK
      tclocal(12) =      2.93173d0*CON_KCALTOK
      tclocal(13) =      1.56430d0*CON_KCALTOK
      tclocal(14) =      1.83287d0*CON_KCALTOK
      tclocal(15) =      3.81630d0*CON_KCALTOK
      tclocal(16) =     -1.71111d0*CON_KCALTOK
      tclocal(17) =      2.79735d0*CON_KCALTOK
      tclocal(18) =     -0.54690d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.74928d0*CON_KCALTOK
      tclocal(1) =      0.91829d0*CON_KCALTOK
      tclocal(2) =      1.23298d0*CON_KCALTOK
      tclocal(3) =      1.01878d0*CON_KCALTOK
      tclocal(4) =      0.45294d0*CON_KCALTOK
      tclocal(5) =     -0.69073d0*CON_KCALTOK
      tclocal(6) =     -6.21908d0*CON_KCALTOK
      tclocal(7) =      8.64646d0*CON_KCALTOK
      tclocal(8) =     -7.84346d0*CON_KCALTOK
      tclocal( 9) =      3.50242d0*CON_KCALTOK
      tclocal(10) =     -3.27998d0*CON_KCALTOK
      tclocal(11) =      2.03402d0*CON_KCALTOK
      tclocal(12) =      1.31820d0*CON_KCALTOK
      tclocal(13) =     -1.36388d0*CON_KCALTOK
      tclocal(14) =      1.55122d0*CON_KCALTOK
      tclocal(15) =      1.12532d0*CON_KCALTOK
      tclocal(16) =     -0.23229d0*CON_KCALTOK
      tclocal(17) =     -0.38227d0*CON_KCALTOK
      tclocal(18) =    -27.45814d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c` ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c` '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      2.01294d0*CON_KCALTOK
      tclocal(1) =      2.15238d0*CON_KCALTOK
      tclocal(2) =     -0.22471d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.98520d0*CON_KCALTOK
      tclocal(7) =      1.36056d0*CON_KCALTOK
      tclocal(8) =     -2.67791d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     19.16296d0*CON_KCALTOK
      tclocal(13) =      1.92337d0*CON_KCALTOK
      tclocal(14) =     -0.57113d0*CON_KCALTOK
      tclocal(15) =      4.25109d0*CON_KCALTOK
      tclocal(16) =      3.27435d0*CON_KCALTOK
      tclocal(17) =     -0.80770d0*CON_KCALTOK
      tclocal(18) =    -33.42060d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c- ')
      call twh_torstring(GLB_SET,type,4,'o- ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c- '
      tnlocal(nsame,4) = 'o- '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.02312d0*CON_KCALTOK
      tclocal(1) =      1.21953d0*CON_KCALTOK
      tclocal(2) =     -0.24291d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =     -8.26215d0*CON_KCALTOK
      tclocal(7) =      1.12441d0*CON_KCALTOK
      tclocal(8) =     -0.59276d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     17.49494d0*CON_KCALTOK
      tclocal(13) =      0.34608d0*CON_KCALTOK
      tclocal(14) =     -0.06174d0*CON_KCALTOK
      tclocal(15) =     -6.20654d0*CON_KCALTOK
      tclocal(16) =      1.63097d0*CON_KCALTOK
      tclocal(17) =     -0.21465d0*CON_KCALTOK
      tclocal(18) =    -22.62470d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c- ')
      call twh_torstring(GLB_SET,type,4,'o- ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c- '
      tnlocal(nsame,4) = 'o- '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.89385d0*CON_KCALTOK
      tclocal(1) =      4.01078d0*CON_KCALTOK
      tclocal(2) =     -0.59886d0*CON_KCALTOK
      tclocal(3) =      9.69332d0*CON_KCALTOK
      tclocal(4) =     -0.54456d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      1.22665d0*CON_KCALTOK
      tclocal(7) =      3.24292d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =    -13.89742d0*CON_KCALTOK
      tclocal(10) =     12.70822d0*CON_KCALTOK
      tclocal(11) =      1.31071d0*CON_KCALTOK
      tclocal(12) =     14.56721d0*CON_KCALTOK
      tclocal(13) =     -1.31200d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      8.18822d0*CON_KCALTOK
      tclocal(16) =     18.79505d0*CON_KCALTOK
      tclocal(17) =     -2.88278d0*CON_KCALTOK
      tclocal(18) =    -29.72540d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n+ ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c- ')
      call twh_torstring(GLB_SET,type,4,'o- ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n+ '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c- '
      tnlocal(nsame,4) = 'o- '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.10132d0*CON_KCALTOK
      tclocal(1) =      0.07196d0*CON_KCALTOK
      tclocal(2) =     -0.85712d0*CON_KCALTOK
      tclocal(3) =      0.59332d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.56417d0*CON_KCALTOK
      tclocal(6) =      0.62003d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =     -3.75425d0*CON_KCALTOK
      tclocal( 9) =      0.70682d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.20000d0*CON_KCALTOK
      tclocal(12) =      2.41031d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =     -2.25271d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -2.45025d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c= ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c= '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.28292d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =      0.12090d0*CON_KCALTOK
      tclocal(3) =      0.13472d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =     -0.98985d0*CON_KCALTOK
      tclocal(6) =     -2.49350d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =     -0.12068d0*CON_KCALTOK
      tclocal( 9) =     -0.75421d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =     -0.97606d0*CON_KCALTOK
      tclocal(12) =     -0.28017d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.45000d0*CON_KCALTOK
      tclocal(15) =     -0.19134d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -23.68683d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c= ')
      call twh_torstring(GLB_SET,type,4,'c= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c= '
      tnlocal(nsame,4) = 'c= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.28292d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =     -0.38149d0*CON_KCALTOK
      tclocal(3) =      0.21531d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.68671d0*CON_KCALTOK
      tclocal(6) =      1.49000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =     -1.28158d0*CON_KCALTOK
      tclocal( 9) =      1.36390d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =     -0.06762d0*CON_KCALTOK
      tclocal(12) =      0.61755d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =     -1.06648d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -19.65026d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c= ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c= '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00292d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =      0.02015d0*CON_KCALTOK
      tclocal(3) =     -1.17534d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =     -0.09654d0*CON_KCALTOK
      tclocal(6) =     -1.93964d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =     -1.15572d0*CON_KCALTOK
      tclocal( 9) =      0.08021d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =     -0.83002d0*CON_KCALTOK
      tclocal(12) =      1.57076d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =     -1.00000d0*CON_KCALTOK
      tclocal(15) =     -0.28421d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      1.50000d0*CON_KCALTOK
      tclocal(18) =    -23.41510d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c= ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c= ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c= '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c= '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00292d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =     -0.33851d0*CON_KCALTOK
      tclocal(3) =      0.88931d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.05117d0*CON_KCALTOK
      tclocal(6) =      3.42484d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =     -0.24229d0*CON_KCALTOK
      tclocal( 9) =      0.24886d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.26025d0*CON_KCALTOK
      tclocal(12) =      4.67799d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.19238d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -9.71525d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c= ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c= ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c= '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c= '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.13292d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =     -0.19558d0*CON_KCALTOK
      tclocal(3) =      0.41631d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.39900d0*CON_KCALTOK
      tclocal(6) =      2.14262d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =     -0.31896d0*CON_KCALTOK
      tclocal( 9) =      1.09851d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.68044d0*CON_KCALTOK
      tclocal(12) =      1.13404d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.87502d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -14.95964d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c= ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c= '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.13292d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =      0.21555d0*CON_KCALTOK
      tclocal(3) =      2.30080d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =     -0.20993d0*CON_KCALTOK
      tclocal(6) =     -1.82871d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.76337d0*CON_KCALTOK
      tclocal( 9) =      0.99606d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.09436d0*CON_KCALTOK
      tclocal(12) =      1.85837d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =     -2.19456d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -15.91003d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c= ')
      call twh_torstring(GLB_SET,type,4,'c= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c= '
      tnlocal(nsame,4) = 'c= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.13292d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =     -0.15683d0*CON_KCALTOK
      tclocal(3) =      0.25721d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.10635d0*CON_KCALTOK
      tclocal(6) =      2.17787d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =     -0.43050d0*CON_KCALTOK
      tclocal( 9) =      0.69001d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.04933d0*CON_KCALTOK
      tclocal(12) =      2.21639d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =     -0.07873d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -12.73400d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'c= ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'c= '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.76701d0*CON_KCALTOK
      tclocal(1) =      0.19954d0*CON_KCALTOK
      tclocal(2) =     -0.25933d0*CON_KCALTOK
      tclocal(3) =     -0.27606d0*CON_KCALTOK
      tclocal(4) =     -0.42614d0*CON_KCALTOK
      tclocal(5) =     -0.63943d0*CON_KCALTOK
      tclocal(6) =    -21.98882d0*CON_KCALTOK
      tclocal(7) =     -8.20211d0*CON_KCALTOK
      tclocal(8) =     -0.38896d0*CON_KCALTOK
      tclocal( 9) =     -0.62017d0*CON_KCALTOK
      tclocal(10) =     -0.14651d0*CON_KCALTOK
      tclocal(11) =     -0.09606d0*CON_KCALTOK
      tclocal(12) =      0.14123d0*CON_KCALTOK
      tclocal(13) =     -1.78780d0*CON_KCALTOK
      tclocal(14) =     -2.24306d0*CON_KCALTOK
      tclocal(15) =      0.63847d0*CON_KCALTOK
      tclocal(16) =     -1.87012d0*CON_KCALTOK
      tclocal(17) =     -2.97092d0*CON_KCALTOK
      tclocal(18) =    -24.29166d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =      0.00900d0*CON_KCALTOK
      tclocal(4) =     -0.00033d0*CON_KCALTOK
      tclocal(5) =      0.00525d0*CON_KCALTOK
      tclocal(6) =     -2.31093d0*CON_KCALTOK
      tclocal(7) =      0.32848d0*CON_KCALTOK
      tclocal(8) =     -1.00157d0*CON_KCALTOK
      tclocal( 9) =      0.00800d0*CON_KCALTOK
      tclocal(10) =      0.00864d0*CON_KCALTOK
      tclocal(11) =     -0.00124d0*CON_KCALTOK
      tclocal(12) =     -0.37784d0*CON_KCALTOK
      tclocal(13) =      0.46418d0*CON_KCALTOK
      tclocal(14) =     -0.33094d0*CON_KCALTOK
      tclocal(15) =      0.07459d0*CON_KCALTOK
      tclocal(16) =     -0.09765d0*CON_KCALTOK
      tclocal(17) =     -0.06875d0*CON_KCALTOK
      tclocal(18) =     -0.55093d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.89744d0*CON_KCALTOK
      tclocal(1) =      0.70222d0*CON_KCALTOK
      tclocal(2) =     -0.20715d0*CON_KCALTOK
      tclocal(3) =      0.35413d0*CON_KCALTOK
      tclocal(4) =      0.88117d0*CON_KCALTOK
      tclocal(5) =      0.66219d0*CON_KCALTOK
      tclocal(6) =    -18.65910d0*CON_KCALTOK
      tclocal(7) =     -4.43667d0*CON_KCALTOK
      tclocal(8) =     -0.40776d0*CON_KCALTOK
      tclocal( 9) =     -0.07952d0*CON_KCALTOK
      tclocal(10) =     -0.16094d0*CON_KCALTOK
      tclocal(11) =      0.72560d0*CON_KCALTOK
      tclocal(12) =     -1.25536d0*CON_KCALTOK
      tclocal(13) =      0.63566d0*CON_KCALTOK
      tclocal(14) =      1.29646d0*CON_KCALTOK
      tclocal(15) =      0.49463d0*CON_KCALTOK
      tclocal(16) =      0.58089d0*CON_KCALTOK
      tclocal(17) =     -2.66449d0*CON_KCALTOK
      tclocal(18) =    -19.62147d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =      0.10756d0*CON_KCALTOK
      tclocal(4) =      0.01525d0*CON_KCALTOK
      tclocal(5) =      0.07505d0*CON_KCALTOK
      tclocal(6) =     -6.13386d0*CON_KCALTOK
      tclocal(7) =     -4.78596d0*CON_KCALTOK
      tclocal(8) =     -1.91941d0*CON_KCALTOK
      tclocal( 9) =     -0.11549d0*CON_KCALTOK
      tclocal(10) =     -0.05877d0*CON_KCALTOK
      tclocal(11) =      0.03694d0*CON_KCALTOK
      tclocal(12) =     -0.79726d0*CON_KCALTOK
      tclocal(13) =     -2.46111d0*CON_KCALTOK
      tclocal(14) =      0.36007d0*CON_KCALTOK
      tclocal(15) =      0.29544d0*CON_KCALTOK
      tclocal(16) =      0.23213d0*CON_KCALTOK
      tclocal(17) =     -1.15153d0*CON_KCALTOK
      tclocal(18) =     -1.54067d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.01914d0*CON_KCALTOK
      tclocal(1) =      0.62659d0*CON_KCALTOK
      tclocal(2) =     -0.31559d0*CON_KCALTOK
      tclocal(3) =      0.01775d0*CON_KCALTOK
      tclocal(4) =      0.66848d0*CON_KCALTOK
      tclocal(5) =     -0.27516d0*CON_KCALTOK
      tclocal(6) =    -31.13116d0*CON_KCALTOK
      tclocal(7) =     -5.18855d0*CON_KCALTOK
      tclocal(8) =     -3.78456d0*CON_KCALTOK
      tclocal( 9) =      3.83339d0*CON_KCALTOK
      tclocal(10) =      1.05993d0*CON_KCALTOK
      tclocal(11) =     -1.21634d0*CON_KCALTOK
      tclocal(12) =      3.31961d0*CON_KCALTOK
      tclocal(13) =      0.86147d0*CON_KCALTOK
      tclocal(14) =     -0.64223d0*CON_KCALTOK
      tclocal(15) =      0.19227d0*CON_KCALTOK
      tclocal(16) =      1.68609d0*CON_KCALTOK
      tclocal(17) =     -3.53725d0*CON_KCALTOK
      tclocal(18) =    -32.61270d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =      0.00800d0*CON_KCALTOK
      tclocal(4) =      0.00864d0*CON_KCALTOK
      tclocal(5) =     -0.00124d0*CON_KCALTOK
      tclocal(6) =     -2.31093d0*CON_KCALTOK
      tclocal(7) =      0.32848d0*CON_KCALTOK
      tclocal(8) =     -1.00157d0*CON_KCALTOK
      tclocal( 9) =      0.00900d0*CON_KCALTOK
      tclocal(10) =     -0.00033d0*CON_KCALTOK
      tclocal(11) =      0.00525d0*CON_KCALTOK
      tclocal(12) =      0.07459d0*CON_KCALTOK
      tclocal(13) =     -0.09765d0*CON_KCALTOK
      tclocal(14) =     -0.06875d0*CON_KCALTOK
      tclocal(15) =     -0.37784d0*CON_KCALTOK
      tclocal(16) =      0.46418d0*CON_KCALTOK
      tclocal(17) =     -0.33094d0*CON_KCALTOK
      tclocal(18) =     -0.55093d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =      0.00781d0*CON_KCALTOK
      tclocal(4) =     -0.00068d0*CON_KCALTOK
      tclocal(5) =     -0.00646d0*CON_KCALTOK
      tclocal(6) =      2.13898d0*CON_KCALTOK
      tclocal(7) =      0.84807d0*CON_KCALTOK
      tclocal(8) =     -0.26999d0*CON_KCALTOK
      tclocal( 9) =      0.00781d0*CON_KCALTOK
      tclocal(10) =     -0.00068d0*CON_KCALTOK
      tclocal(11) =     -0.00646d0*CON_KCALTOK
      tclocal(12) =      5.00204d0*CON_KCALTOK
      tclocal(13) =      1.81257d0*CON_KCALTOK
      tclocal(14) =     -0.06487d0*CON_KCALTOK
      tclocal(15) =      5.00204d0*CON_KCALTOK
      tclocal(16) =      1.81257d0*CON_KCALTOK
      tclocal(17) =     -0.06487d0*CON_KCALTOK
      tclocal(18) =      0.20264d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.02536d0*CON_KCALTOK
      tclocal(1) =      0.03110d0*CON_KCALTOK
      tclocal(2) =     -0.20695d0*CON_KCALTOK
      tclocal(3) =     -0.02959d0*CON_KCALTOK
      tclocal(4) =      0.52584d0*CON_KCALTOK
      tclocal(5) =     -0.64150d0*CON_KCALTOK
      tclocal(6) =     -5.07814d0*CON_KCALTOK
      tclocal(7) =      1.80555d0*CON_KCALTOK
      tclocal(8) =     -1.10296d0*CON_KCALTOK
      tclocal( 9) =     -0.01399d0*CON_KCALTOK
      tclocal(10) =     -0.04560d0*CON_KCALTOK
      tclocal(11) =     -0.10941d0*CON_KCALTOK
      tclocal(12) =     -1.08202d0*CON_KCALTOK
      tclocal(13) =     -1.36928d0*CON_KCALTOK
      tclocal(14) =     -0.91613d0*CON_KCALTOK
      tclocal(15) =      0.02341d0*CON_KCALTOK
      tclocal(16) =      2.05953d0*CON_KCALTOK
      tclocal(17) =     -2.04236d0*CON_KCALTOK
      tclocal(18) =     -7.77160d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.85000d0*CON_KCALTOK
      tclocal(1) =     -0.14642d0*CON_KCALTOK
      tclocal(2) =     -0.20715d0*CON_KCALTOK
      tclocal(3) =      1.50557d0*CON_KCALTOK
      tclocal(4) =     -0.19936d0*CON_KCALTOK
      tclocal(5) =      0.07448d0*CON_KCALTOK
      tclocal(6) =    -20.22350d0*CON_KCALTOK
      tclocal(7) =     -5.90517d0*CON_KCALTOK
      tclocal(8) =      0.66587d0*CON_KCALTOK
      tclocal( 9) =     -0.03837d0*CON_KCALTOK
      tclocal(10) =      0.83625d0*CON_KCALTOK
      tclocal(11) =      1.29802d0*CON_KCALTOK
      tclocal(12) =      1.42575d0*CON_KCALTOK
      tclocal(13) =     -0.36073d0*CON_KCALTOK
      tclocal(14) =      1.40556d0*CON_KCALTOK
      tclocal(15) =     -1.83757d0*CON_KCALTOK
      tclocal(16) =      1.44836d0*CON_KCALTOK
      tclocal(17) =     -0.05612d0*CON_KCALTOK
      tclocal(18) =    -19.48951d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.02536d0*CON_KCALTOK
      tclocal(1) =      0.03110d0*CON_KCALTOK
      tclocal(2) =     -0.20695d0*CON_KCALTOK
      tclocal(3) =     -0.01399d0*CON_KCALTOK
      tclocal(4) =     -0.04560d0*CON_KCALTOK
      tclocal(5) =     -0.10941d0*CON_KCALTOK
      tclocal(6) =     -5.07814d0*CON_KCALTOK
      tclocal(7) =      1.80555d0*CON_KCALTOK
      tclocal(8) =     -1.10296d0*CON_KCALTOK
      tclocal( 9) =     -0.02959d0*CON_KCALTOK
      tclocal(10) =      0.52584d0*CON_KCALTOK
      tclocal(11) =     -0.64150d0*CON_KCALTOK
      tclocal(12) =      0.02341d0*CON_KCALTOK
      tclocal(13) =      2.05953d0*CON_KCALTOK
      tclocal(14) =     -2.04236d0*CON_KCALTOK
      tclocal(15) =     -1.08202d0*CON_KCALTOK
      tclocal(16) =     -1.36928d0*CON_KCALTOK
      tclocal(17) =     -0.91613d0*CON_KCALTOK
      tclocal(18) =     -7.77160d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.76883d0*CON_KCALTOK
      tclocal(1) =      0.59188d0*CON_KCALTOK
      tclocal(2) =     -0.16075d0*CON_KCALTOK
      tclocal(3) =      0.51889d0*CON_KCALTOK
      tclocal(4) =      0.58598d0*CON_KCALTOK
      tclocal(5) =     -0.17846d0*CON_KCALTOK
      tclocal(6) =    -16.42538d0*CON_KCALTOK
      tclocal(7) =     -2.29894d0*CON_KCALTOK
      tclocal(8) =     -1.30787d0*CON_KCALTOK
      tclocal( 9) =      0.37044d0*CON_KCALTOK
      tclocal(10) =      1.08552d0*CON_KCALTOK
      tclocal(11) =     -0.29738d0*CON_KCALTOK
      tclocal(12) =     -1.48781d0*CON_KCALTOK
      tclocal(13) =     -0.32268d0*CON_KCALTOK
      tclocal(14) =     -0.62547d0*CON_KCALTOK
      tclocal(15) =     -0.30430d0*CON_KCALTOK
      tclocal(16) =      0.24684d0*CON_KCALTOK
      tclocal(17) =      1.32643d0*CON_KCALTOK
      tclocal(18) =    -14.25124d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.02536d0*CON_KCALTOK
      tclocal(1) =      0.03110d0*CON_KCALTOK
      tclocal(2) =     -0.20695d0*CON_KCALTOK
      tclocal(3) =     -0.08709d0*CON_KCALTOK
      tclocal(4) =     -0.54539d0*CON_KCALTOK
      tclocal(5) =     -0.27189d0*CON_KCALTOK
      tclocal(6) =     -5.94603d0*CON_KCALTOK
      tclocal(7) =     -0.86095d0*CON_KCALTOK
      tclocal(8) =     -0.06818d0*CON_KCALTOK
      tclocal( 9) =      0.43792d0*CON_KCALTOK
      tclocal(10) =      0.36427d0*CON_KCALTOK
      tclocal(11) =      0.67258d0*CON_KCALTOK
      tclocal(12) =     -1.80714d0*CON_KCALTOK
      tclocal(13) =      2.45410d0*CON_KCALTOK
      tclocal(14) =     -1.75086d0*CON_KCALTOK
      tclocal(15) =     -2.87247d0*CON_KCALTOK
      tclocal(16) =      0.33702d0*CON_KCALTOK
      tclocal(17) =     -0.56922d0*CON_KCALTOK
      tclocal(18) =    -18.54698d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.11947d0*CON_KCALTOK
      tclocal(1) =      0.44685d0*CON_KCALTOK
      tclocal(2) =     -0.07278d0*CON_KCALTOK
      tclocal(3) =      1.61547d0*CON_KCALTOK
      tclocal(4) =      2.08556d0*CON_KCALTOK
      tclocal(5) =     -0.19638d0*CON_KCALTOK
      tclocal(6) =    -26.84663d0*CON_KCALTOK
      tclocal(7) =     -1.36819d0*CON_KCALTOK
      tclocal(8) =      1.00577d0*CON_KCALTOK
      tclocal( 9) =      3.55042d0*CON_KCALTOK
      tclocal(10) =      0.78138d0*CON_KCALTOK
      tclocal(11) =      1.01929d0*CON_KCALTOK
      tclocal(12) =      4.05257d0*CON_KCALTOK
      tclocal(13) =      2.53231d0*CON_KCALTOK
      tclocal(14) =     -1.23400d0*CON_KCALTOK
      tclocal(15) =     -0.55700d0*CON_KCALTOK
      tclocal(16) =      1.59474d0*CON_KCALTOK
      tclocal(17) =      2.96094d0*CON_KCALTOK
      tclocal(18) =    -25.54719d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10799d0*CON_KCALTOK
      tclocal(1) =      0.08017d0*CON_KCALTOK
      tclocal(2) =     -0.28679d0*CON_KCALTOK
      tclocal(3) =     -0.11549d0*CON_KCALTOK
      tclocal(4) =     -0.05877d0*CON_KCALTOK
      tclocal(5) =      0.03694d0*CON_KCALTOK
      tclocal(6) =     -6.13386d0*CON_KCALTOK
      tclocal(7) =     -4.78596d0*CON_KCALTOK
      tclocal(8) =     -1.91941d0*CON_KCALTOK
      tclocal( 9) =      0.10756d0*CON_KCALTOK
      tclocal(10) =      0.01525d0*CON_KCALTOK
      tclocal(11) =      0.07505d0*CON_KCALTOK
      tclocal(12) =      0.29544d0*CON_KCALTOK
      tclocal(13) =      0.23213d0*CON_KCALTOK
      tclocal(14) =     -1.15153d0*CON_KCALTOK
      tclocal(15) =     -0.79726d0*CON_KCALTOK
      tclocal(16) =     -2.46111d0*CON_KCALTOK
      tclocal(17) =      0.36007d0*CON_KCALTOK
      tclocal(18) =     -1.54067d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.02536d0*CON_KCALTOK
      tclocal(1) =      0.03110d0*CON_KCALTOK
      tclocal(2) =     -0.20695d0*CON_KCALTOK
      tclocal(3) =      0.43792d0*CON_KCALTOK
      tclocal(4) =      0.36427d0*CON_KCALTOK
      tclocal(5) =      0.67258d0*CON_KCALTOK
      tclocal(6) =     -5.94603d0*CON_KCALTOK
      tclocal(7) =     -0.86095d0*CON_KCALTOK
      tclocal(8) =     -0.06818d0*CON_KCALTOK
      tclocal( 9) =     -0.08709d0*CON_KCALTOK
      tclocal(10) =     -0.54539d0*CON_KCALTOK
      tclocal(11) =     -0.27189d0*CON_KCALTOK
      tclocal(12) =     -2.87247d0*CON_KCALTOK
      tclocal(13) =      0.33702d0*CON_KCALTOK
      tclocal(14) =     -0.56922d0*CON_KCALTOK
      tclocal(15) =     -1.80714d0*CON_KCALTOK
      tclocal(16) =      2.45410d0*CON_KCALTOK
      tclocal(17) =     -1.75086d0*CON_KCALTOK
      tclocal(18) =    -18.54698d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.73479d0*CON_KCALTOK
      tclocal(1) =      0.26654d0*CON_KCALTOK
      tclocal(2) =     -0.31559d0*CON_KCALTOK
      tclocal(3) =      4.98752d0*CON_KCALTOK
      tclocal(4) =      1.36857d0*CON_KCALTOK
      tclocal(5) =      0.39865d0*CON_KCALTOK
      tclocal(6) =    -30.22398d0*CON_KCALTOK
      tclocal(7) =     -4.88715d0*CON_KCALTOK
      tclocal(8) =     -5.96029d0*CON_KCALTOK
      tclocal( 9) =      1.08701d0*CON_KCALTOK
      tclocal(10) =      0.58050d0*CON_KCALTOK
      tclocal(11) =     -2.93596d0*CON_KCALTOK
      tclocal(12) =      5.02981d0*CON_KCALTOK
      tclocal(13) =     -1.08204d0*CON_KCALTOK
      tclocal(14) =     -1.63883d0*CON_KCALTOK
      tclocal(15) =      2.56843d0*CON_KCALTOK
      tclocal(16) =     -1.00327d0*CON_KCALTOK
      tclocal(17) =     -1.89522d0*CON_KCALTOK
      tclocal(18) =    -36.08685d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.50144d0*CON_KCALTOK
      tclocal(1) =      0.01840d0*CON_KCALTOK
      tclocal(2) =     -0.07278d0*CON_KCALTOK
      tclocal(3) =      2.18462d0*CON_KCALTOK
      tclocal(4) =      2.33211d0*CON_KCALTOK
      tclocal(5) =      0.69879d0*CON_KCALTOK
      tclocal(6) =    -29.24123d0*CON_KCALTOK
      tclocal(7) =     -2.63406d0*CON_KCALTOK
      tclocal(8) =      2.84046d0*CON_KCALTOK
      tclocal( 9) =      1.40185d0*CON_KCALTOK
      tclocal(10) =      0.40296d0*CON_KCALTOK
      tclocal(11) =      0.06300d0*CON_KCALTOK
      tclocal(12) =     -4.00310d0*CON_KCALTOK
      tclocal(13) =      4.54451d0*CON_KCALTOK
      tclocal(14) =     -0.23869d0*CON_KCALTOK
      tclocal(15) =      1.96190d0*CON_KCALTOK
      tclocal(16) =      1.32745d0*CON_KCALTOK
      tclocal(17) =     -3.11114d0*CON_KCALTOK
      tclocal(18) =    -24.26350d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.57168d0*CON_KCALTOK
      tclocal(1) =      0.08646d0*CON_KCALTOK
      tclocal(2) =     -0.44577d0*CON_KCALTOK
      tclocal(3) =      4.09649d0*CON_KCALTOK
      tclocal(4) =      2.73388d0*CON_KCALTOK
      tclocal(5) =      1.85591d0*CON_KCALTOK
      tclocal(6) =    -14.16091d0*CON_KCALTOK
      tclocal(7) =      2.47373d0*CON_KCALTOK
      tclocal(8) =     -9.79029d0*CON_KCALTOK
      tclocal( 9) =      1.95663d0*CON_KCALTOK
      tclocal(10) =      1.87865d0*CON_KCALTOK
      tclocal(11) =     -2.32720d0*CON_KCALTOK
      tclocal(12) =     -0.89989d0*CON_KCALTOK
      tclocal(13) =      4.92658d0*CON_KCALTOK
      tclocal(14) =      2.29946d0*CON_KCALTOK
      tclocal(15) =      3.75327d0*CON_KCALTOK
      tclocal(16) =      4.62063d0*CON_KCALTOK
      tclocal(17) =      2.31824d0*CON_KCALTOK
      tclocal(18) =    -28.99191d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.82409d0*CON_KCALTOK
      tclocal(1) =      0.82652d0*CON_KCALTOK
      tclocal(2) =     -0.37524d0*CON_KCALTOK
      tclocal(3) =      7.63953d0*CON_KCALTOK
      tclocal(4) =      3.02874d0*CON_KCALTOK
      tclocal(5) =     -0.24682d0*CON_KCALTOK
      tclocal(6) =    -37.36401d0*CON_KCALTOK
      tclocal(7) =     -0.47698d0*CON_KCALTOK
      tclocal(8) =     -3.10824d0*CON_KCALTOK
      tclocal( 9) =      6.94855d0*CON_KCALTOK
      tclocal(10) =      3.21805d0*CON_KCALTOK
      tclocal(11) =     -1.24339d0*CON_KCALTOK
      tclocal(12) =      8.35519d0*CON_KCALTOK
      tclocal(13) =      3.61241d0*CON_KCALTOK
      tclocal(14) =     -0.43196d0*CON_KCALTOK
      tclocal(15) =      5.71384d0*CON_KCALTOK
      tclocal(16) =      1.62767d0*CON_KCALTOK
      tclocal(17) =     -2.77203d0*CON_KCALTOK
      tclocal(18) =    -52.53303d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.96268d0*CON_KCALTOK
      tclocal(1) =      0.17311d0*CON_KCALTOK
      tclocal(2) =     -0.08032d0*CON_KCALTOK
      tclocal(3) =     -1.56918d0*CON_KCALTOK
      tclocal(4) =      0.82684d0*CON_KCALTOK
      tclocal(5) =     -0.69237d0*CON_KCALTOK
      tclocal(6) =     -3.19757d0*CON_KCALTOK
      tclocal(7) =      1.79823d0*CON_KCALTOK
      tclocal(8) =     -1.55326d0*CON_KCALTOK
      tclocal( 9) =      0.22277d0*CON_KCALTOK
      tclocal(10) =      1.27933d0*CON_KCALTOK
      tclocal(11) =      0.13953d0*CON_KCALTOK
      tclocal(12) =      1.86742d0*CON_KCALTOK
      tclocal(13) =      1.59656d0*CON_KCALTOK
      tclocal(14) =      0.29805d0*CON_KCALTOK
      tclocal(15) =      0.09670d0*CON_KCALTOK
      tclocal(16) =      1.51865d0*CON_KCALTOK
      tclocal(17) =     -0.46220d0*CON_KCALTOK
      tclocal(18) =    -19.63795d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.06778d0*CON_KCALTOK
      tclocal(1) =      0.78830d0*CON_KCALTOK
      tclocal(2) =     -0.06554d0*CON_KCALTOK
      tclocal(3) =     -0.60257d0*CON_KCALTOK
      tclocal(4) =      1.14119d0*CON_KCALTOK
      tclocal(5) =     -0.07295d0*CON_KCALTOK
      tclocal(6) =      3.03545d0*CON_KCALTOK
      tclocal(7) =      2.21127d0*CON_KCALTOK
      tclocal(8) =     -1.59244d0*CON_KCALTOK
      tclocal( 9) =     -0.58848d0*CON_KCALTOK
      tclocal(10) =      1.14373d0*CON_KCALTOK
      tclocal(11) =      1.52012d0*CON_KCALTOK
      tclocal(12) =      3.08684d0*CON_KCALTOK
      tclocal(13) =      2.03062d0*CON_KCALTOK
      tclocal(14) =     -0.24872d0*CON_KCALTOK
      tclocal(15) =      2.53181d0*CON_KCALTOK
      tclocal(16) =      2.39007d0*CON_KCALTOK
      tclocal(17) =      0.38527d0*CON_KCALTOK
      tclocal(18) =    -34.91741d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.74205d0*CON_KCALTOK
      tclocal(1) =      1.01337d0*CON_KCALTOK
      tclocal(2) =     -0.41256d0*CON_KCALTOK
      tclocal(3) =     -0.92622d0*CON_KCALTOK
      tclocal(4) =      1.63993d0*CON_KCALTOK
      tclocal(5) =     -0.36553d0*CON_KCALTOK
      tclocal(6) =      0.69851d0*CON_KCALTOK
      tclocal(7) =      0.15474d0*CON_KCALTOK
      tclocal(8) =     -0.50501d0*CON_KCALTOK
      tclocal( 9) =      1.30735d0*CON_KCALTOK
      tclocal(10) =      0.09737d0*CON_KCALTOK
      tclocal(11) =      0.11736d0*CON_KCALTOK
      tclocal(12) =      2.97857d0*CON_KCALTOK
      tclocal(13) =      0.95937d0*CON_KCALTOK
      tclocal(14) =      0.01795d0*CON_KCALTOK
      tclocal(15) =      2.35339d0*CON_KCALTOK
      tclocal(16) =     -0.57165d0*CON_KCALTOK
      tclocal(17) =      1.15914d0*CON_KCALTOK
      tclocal(18) =    -16.07868d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.36773d0*CON_KCALTOK
      tclocal(2) =      0.24130d0*CON_KCALTOK
      tclocal(3) =     -1.61210d0*CON_KCALTOK
      tclocal(4) =      1.55682d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     -6.34108d0*CON_KCALTOK
      tclocal(8) =      4.49167d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.78881d0*CON_KCALTOK
      tclocal(12) =     -2.52314d0*CON_KCALTOK
      tclocal(13) =      4.09998d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      1.07438d0*CON_KCALTOK
      tclocal(18) =    -12.50982d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cl ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cl '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.88017d0*CON_KCALTOK
      tclocal(2) =     -0.33909d0*CON_KCALTOK
      tclocal(3) =     -1.62436d0*CON_KCALTOK
      tclocal(4) =      2.10297d0*CON_KCALTOK
      tclocal(5) =     -0.61388d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     -1.31716d0*CON_KCALTOK
      tclocal(8) =      2.08519d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =     -0.40206d0*CON_KCALTOK
      tclocal(12) =     -1.29726d0*CON_KCALTOK
      tclocal(13) =      3.73613d0*CON_KCALTOK
      tclocal(14) =     -2.67408d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -14.88008d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cl ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cl '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.98503d0*CON_KCALTOK
      tclocal(1) =      0.66800d0*CON_KCALTOK
      tclocal(2) =     -0.25250d0*CON_KCALTOK
      tclocal(3) =      2.97183d0*CON_KCALTOK
      tclocal(4) =      2.04032d0*CON_KCALTOK
      tclocal(5) =     -0.63895d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     -0.03183d0*CON_KCALTOK
      tclocal(8) =      3.07216d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.55648d0*CON_KCALTOK
      tclocal(12) =     -2.54594d0*CON_KCALTOK
      tclocal(13) =      2.76527d0*CON_KCALTOK
      tclocal(14) =     -2.04484d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.40153d0*CON_KCALTOK
      tclocal(18) =     -5.57878d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cl ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cl '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.96268d0*CON_KCALTOK
      tclocal(1) =      0.17311d0*CON_KCALTOK
      tclocal(2) =     -0.08032d0*CON_KCALTOK
      tclocal(3) =      0.22277d0*CON_KCALTOK
      tclocal(4) =      1.27933d0*CON_KCALTOK
      tclocal(5) =      0.13953d0*CON_KCALTOK
      tclocal(6) =     -3.19757d0*CON_KCALTOK
      tclocal(7) =      1.79823d0*CON_KCALTOK
      tclocal(8) =     -1.55326d0*CON_KCALTOK
      tclocal( 9) =     -1.56918d0*CON_KCALTOK
      tclocal(10) =      0.82684d0*CON_KCALTOK
      tclocal(11) =     -0.69237d0*CON_KCALTOK
      tclocal(12) =      0.09670d0*CON_KCALTOK
      tclocal(13) =      1.51865d0*CON_KCALTOK
      tclocal(14) =     -0.46220d0*CON_KCALTOK
      tclocal(15) =      1.86742d0*CON_KCALTOK
      tclocal(16) =      1.59656d0*CON_KCALTOK
      tclocal(17) =      0.29805d0*CON_KCALTOK
      tclocal(18) =    -19.63795d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.27375d0*CON_KCALTOK
      tclocal(1) =      0.63540d0*CON_KCALTOK
      tclocal(2) =      0.06691d0*CON_KCALTOK
      tclocal(3) =      0.12950d0*CON_KCALTOK
      tclocal(4) =      0.82203d0*CON_KCALTOK
      tclocal(5) =     -0.29890d0*CON_KCALTOK
      tclocal(6) =      0.03406d0*CON_KCALTOK
      tclocal(7) =      1.31187d0*CON_KCALTOK
      tclocal(8) =     -0.08487d0*CON_KCALTOK
      tclocal( 9) =     -0.17054d0*CON_KCALTOK
      tclocal(10) =      0.00913d0*CON_KCALTOK
      tclocal(11) =      0.43909d0*CON_KCALTOK
      tclocal(12) =     -0.00015d0*CON_KCALTOK
      tclocal(13) =      0.81273d0*CON_KCALTOK
      tclocal(14) =     -0.12349d0*CON_KCALTOK
      tclocal(15) =     -5.35432d0*CON_KCALTOK
      tclocal(16) =     -2.68453d0*CON_KCALTOK
      tclocal(17) =      0.16533d0*CON_KCALTOK
      tclocal(18) =     -5.41174d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'ct ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'ct '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.15663d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =     -1.44067d0*CON_KCALTOK
      tclocal(4) =      0.54088d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      2.45514d0*CON_KCALTOK
      tclocal(8) =      2.17797d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.50631d0*CON_KCALTOK
      tclocal(12) =     -1.09228d0*CON_KCALTOK
      tclocal(13) =      3.26607d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      1.47235d0*CON_KCALTOK
      tclocal(17) =      0.41792d0*CON_KCALTOK
      tclocal(18) =    -17.09150d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'f  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'f  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.38320d0*CON_KCALTOK
      tclocal(2) =     -0.19713d0*CON_KCALTOK
      tclocal(3) =     -1.91313d0*CON_KCALTOK
      tclocal(4) =      1.02105d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      2.39366d0*CON_KCALTOK
      tclocal(8) =      1.66547d0*CON_KCALTOK
      tclocal( 9) =      0.88686d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.24677d0*CON_KCALTOK
      tclocal(12) =     -1.47393d0*CON_KCALTOK
      tclocal(13) =      3.54117d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.22410d0*CON_KCALTOK
      tclocal(18) =    -25.45543d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'f  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'f  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.64206d0*CON_KCALTOK
      tclocal(1) =      1.32493d0*CON_KCALTOK
      tclocal(2) =     -0.35797d0*CON_KCALTOK
      tclocal(3) =      3.88631d0*CON_KCALTOK
      tclocal(4) =      0.86722d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      2.94201d0*CON_KCALTOK
      tclocal(8) =      1.23719d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.42259d0*CON_KCALTOK
      tclocal(12) =     -3.49964d0*CON_KCALTOK
      tclocal(13) =      1.31684d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =     -0.73152d0*CON_KCALTOK
      tclocal(18) =     -7.09888d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'f  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'f  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.01789d0*CON_KCALTOK
      tclocal(1) =      0.20534d0*CON_KCALTOK
      tclocal(2) =      0.05659d0*CON_KCALTOK
      tclocal(3) =     -0.48570d0*CON_KCALTOK
      tclocal(4) =      0.60685d0*CON_KCALTOK
      tclocal(5) =      0.00734d0*CON_KCALTOK
      tclocal(6) =     -2.34161d0*CON_KCALTOK
      tclocal(7) =      2.04669d0*CON_KCALTOK
      tclocal(8) =      1.13711d0*CON_KCALTOK
      tclocal( 9) =     -0.83104d0*CON_KCALTOK
      tclocal(10) =      0.24668d0*CON_KCALTOK
      tclocal(11) =      0.45917d0*CON_KCALTOK
      tclocal(12) =      1.05000d0*CON_KCALTOK
      tclocal(13) =      0.73080d0*CON_KCALTOK
      tclocal(14) =      0.43132d0*CON_KCALTOK
      tclocal(15) =     -2.77447d0*CON_KCALTOK
      tclocal(16) =      0.84694d0*CON_KCALTOK
      tclocal(17) =      0.27014d0*CON_KCALTOK
      tclocal(18) =    -14.87158d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.15525d0*CON_KCALTOK
      tclocal(1) =      0.54927d0*CON_KCALTOK
      tclocal(2) =     -0.22248d0*CON_KCALTOK
      tclocal(3) =     -1.51633d0*CON_KCALTOK
      tclocal(4) =      1.41549d0*CON_KCALTOK
      tclocal(5) =      0.04455d0*CON_KCALTOK
      tclocal(6) =      3.69767d0*CON_KCALTOK
      tclocal(7) =      2.96100d0*CON_KCALTOK
      tclocal(8) =     -0.70021d0*CON_KCALTOK
      tclocal( 9) =     -0.37023d0*CON_KCALTOK
      tclocal(10) =      1.64275d0*CON_KCALTOK
      tclocal(11) =     -0.96007d0*CON_KCALTOK
      tclocal(12) =      1.28773d0*CON_KCALTOK
      tclocal(13) =      1.80673d0*CON_KCALTOK
      tclocal(14) =     -1.09179d0*CON_KCALTOK
      tclocal(15) =     -1.29663d0*CON_KCALTOK
      tclocal(16) =      2.12769d0*CON_KCALTOK
      tclocal(17) =     -0.47170d0*CON_KCALTOK
      tclocal(18) =    -25.68663d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.66850d0*CON_KCALTOK
      tclocal(2) =     -0.02742d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.81549d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      1.97924d0*CON_KCALTOK
      tclocal(8) =      1.51255d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      1.25093d0*CON_KCALTOK
      tclocal(12) =      0.59972d0*CON_KCALTOK
      tclocal(13) =      2.37923d0*CON_KCALTOK
      tclocal(14) =      1.13568d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.63027d0*CON_KCALTOK
      tclocal(18) =    -14.06046d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'ct ')
      call twh_torstring(GLB_SET,type,4,'ct ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'ct '
      tnlocal(nsame,4) = 'ct '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'ct ')
      call twh_torstring(GLB_SET,type,4,'ct ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'ct '
      tnlocal(nsame,4) = 'ct '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      2.38751d0*CON_KCALTOK
      tclocal(1) =      0.34405d0*CON_KCALTOK
      tclocal(2) =     -0.55079d0*CON_KCALTOK
      tclocal(3) =      0.70464d0*CON_KCALTOK
      tclocal(4) =      0.48907d0*CON_KCALTOK
      tclocal(5) =     -0.27134d0*CON_KCALTOK
      tclocal(6) =      0.15293d0*CON_KCALTOK
      tclocal(7) =     -0.94710d0*CON_KCALTOK
      tclocal(8) =      1.36415d0*CON_KCALTOK
      tclocal( 9) =      0.02219d0*CON_KCALTOK
      tclocal(10) =      1.04948d0*CON_KCALTOK
      tclocal(11) =      0.42087d0*CON_KCALTOK
      tclocal(12) =     -1.55845d0*CON_KCALTOK
      tclocal(13) =      1.47427d0*CON_KCALTOK
      tclocal(14) =     -0.80249d0*CON_KCALTOK
      tclocal(15) =     -0.12207d0*CON_KCALTOK
      tclocal(16) =      0.51456d0*CON_KCALTOK
      tclocal(17) =     -1.32188d0*CON_KCALTOK
      tclocal(18) =    -12.71025d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.02042d0*CON_KCALTOK
      tclocal(1) =     -0.01892d0*CON_KCALTOK
      tclocal(2) =      0.01303d0*CON_KCALTOK
      tclocal(3) =     -0.29467d0*CON_KCALTOK
      tclocal(4) =      0.00502d0*CON_KCALTOK
      tclocal(5) =      0.08117d0*CON_KCALTOK
      tclocal(6) =     -5.72485d0*CON_KCALTOK
      tclocal(7) =     -0.57994d0*CON_KCALTOK
      tclocal(8) =     -0.98519d0*CON_KCALTOK
      tclocal( 9) =     -0.18037d0*CON_KCALTOK
      tclocal(10) =     -1.35787d0*CON_KCALTOK
      tclocal(11) =      1.12764d0*CON_KCALTOK
      tclocal(12) =     -1.09491d0*CON_KCALTOK
      tclocal(13) =      0.08172d0*CON_KCALTOK
      tclocal(14) =      1.76478d0*CON_KCALTOK
      tclocal(15) =     -2.20720d0*CON_KCALTOK
      tclocal(16) =      1.63712d0*CON_KCALTOK
      tclocal(17) =      1.03874d0*CON_KCALTOK
      tclocal(18) =    -10.77022d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.60915d0*CON_KCALTOK
      tclocal(1) =     -0.54436d0*CON_KCALTOK
      tclocal(2) =     -0.01526d0*CON_KCALTOK
      tclocal(3) =     -1.54183d0*CON_KCALTOK
      tclocal(4) =      0.87570d0*CON_KCALTOK
      tclocal(5) =     -0.07147d0*CON_KCALTOK
      tclocal(6) =     -2.12577d0*CON_KCALTOK
      tclocal(7) =     -0.59963d0*CON_KCALTOK
      tclocal(8) =      0.55229d0*CON_KCALTOK
      tclocal( 9) =      0.91335d0*CON_KCALTOK
      tclocal(10) =      2.50996d0*CON_KCALTOK
      tclocal(11) =      0.19551d0*CON_KCALTOK
      tclocal(12) =     -1.23944d0*CON_KCALTOK
      tclocal(13) =      2.15682d0*CON_KCALTOK
      tclocal(14) =     -0.16664d0*CON_KCALTOK
      tclocal(15) =     -4.38764d0*CON_KCALTOK
      tclocal(16) =     -0.43524d0*CON_KCALTOK
      tclocal(17) =      0.00720d0*CON_KCALTOK
      tclocal(18) =    -22.28837d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.38601d0*CON_KCALTOK
      tclocal(1) =     -1.45517d0*CON_KCALTOK
      tclocal(2) =      0.10799d0*CON_KCALTOK
      tclocal(3) =      2.03705d0*CON_KCALTOK
      tclocal(4) =      1.08914d0*CON_KCALTOK
      tclocal(5) =      0.32258d0*CON_KCALTOK
      tclocal(6) =     12.42340d0*CON_KCALTOK
      tclocal(7) =      5.77202d0*CON_KCALTOK
      tclocal(8) =     -7.06655d0*CON_KCALTOK
      tclocal( 9) =     -3.02927d0*CON_KCALTOK
      tclocal(10) =      0.78021d0*CON_KCALTOK
      tclocal(11) =      0.53705d0*CON_KCALTOK
      tclocal(12) =     -5.25409d0*CON_KCALTOK
      tclocal(13) =     -0.55734d0*CON_KCALTOK
      tclocal(14) =      0.03964d0*CON_KCALTOK
      tclocal(15) =     -4.41987d0*CON_KCALTOK
      tclocal(16) =     -0.47336d0*CON_KCALTOK
      tclocal(17) =     -0.61793d0*CON_KCALTOK
      tclocal(18) =     -7.34779d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.02553d0*CON_KCALTOK
      tclocal(1) =     -0.05174d0*CON_KCALTOK
      tclocal(2) =      0.01086d0*CON_KCALTOK
      tclocal(3) =     -0.74280d0*CON_KCALTOK
      tclocal(4) =      0.85935d0*CON_KCALTOK
      tclocal(5) =     -0.12143d0*CON_KCALTOK
      tclocal(6) =     -0.83350d0*CON_KCALTOK
      tclocal(7) =      3.35391d0*CON_KCALTOK
      tclocal(8) =      0.15176d0*CON_KCALTOK
      tclocal( 9) =     -1.66371d0*CON_KCALTOK
      tclocal(10) =      0.57859d0*CON_KCALTOK
      tclocal(11) =     -0.53843d0*CON_KCALTOK
      tclocal(12) =      2.77840d0*CON_KCALTOK
      tclocal(13) =      1.64707d0*CON_KCALTOK
      tclocal(14) =      0.56526d0*CON_KCALTOK
      tclocal(15) =      0.03339d0*CON_KCALTOK
      tclocal(16) =      1.48740d0*CON_KCALTOK
      tclocal(17) =     -0.33076d0*CON_KCALTOK
      tclocal(18) =    -19.61376d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.84192d0*CON_KCALTOK
      tclocal(1) =     -0.28518d0*CON_KCALTOK
      tclocal(2) =      0.01043d0*CON_KCALTOK
      tclocal(3) =      0.27299d0*CON_KCALTOK
      tclocal(4) =      0.21638d0*CON_KCALTOK
      tclocal(5) =     -0.86059d0*CON_KCALTOK
      tclocal(6) =      0.54650d0*CON_KCALTOK
      tclocal(7) =      0.74030d0*CON_KCALTOK
      tclocal(8) =      0.47267d0*CON_KCALTOK
      tclocal( 9) =      0.89740d0*CON_KCALTOK
      tclocal(10) =     -0.35643d0*CON_KCALTOK
      tclocal(11) =     -0.68452d0*CON_KCALTOK
      tclocal(12) =      1.54304d0*CON_KCALTOK
      tclocal(13) =      3.55973d0*CON_KCALTOK
      tclocal(14) =     -1.10820d0*CON_KCALTOK
      tclocal(15) =     -2.53261d0*CON_KCALTOK
      tclocal(16) =      0.55578d0*CON_KCALTOK
      tclocal(17) =      0.76116d0*CON_KCALTOK
      tclocal(18) =    -17.95722d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.47706d0*CON_KCALTOK
      tclocal(1) =     -0.21339d0*CON_KCALTOK
      tclocal(2) =     -0.01223d0*CON_KCALTOK
      tclocal(3) =     -0.26414d0*CON_KCALTOK
      tclocal(4) =      0.71340d0*CON_KCALTOK
      tclocal(5) =     -0.07122d0*CON_KCALTOK
      tclocal(6) =      1.28609d0*CON_KCALTOK
      tclocal(7) =      0.52487d0*CON_KCALTOK
      tclocal(8) =      0.56906d0*CON_KCALTOK
      tclocal( 9) =     -1.83948d0*CON_KCALTOK
      tclocal(10) =      1.03574d0*CON_KCALTOK
      tclocal(11) =      0.12106d0*CON_KCALTOK
      tclocal(12) =     -6.41331d0*CON_KCALTOK
      tclocal(13) =      1.90511d0*CON_KCALTOK
      tclocal(14) =     -0.97942d0*CON_KCALTOK
      tclocal(15) =     -2.58014d0*CON_KCALTOK
      tclocal(16) =     -0.91178d0*CON_KCALTOK
      tclocal(17) =     -0.02696d0*CON_KCALTOK
      tclocal(18) =    -10.10311d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.70037d0*CON_KCALTOK
      tclocal(1) =      0.27936d0*CON_KCALTOK
      tclocal(2) =      0.32406d0*CON_KCALTOK
      tclocal(3) =      0.56704d0*CON_KCALTOK
      tclocal(4) =      0.10556d0*CON_KCALTOK
      tclocal(5) =     -0.04434d0*CON_KCALTOK
      tclocal(6) =      3.31317d0*CON_KCALTOK
      tclocal(7) =      1.31272d0*CON_KCALTOK
      tclocal(8) =     -1.51195d0*CON_KCALTOK
      tclocal( 9) =     -0.01739d0*CON_KCALTOK
      tclocal(10) =      2.35608d0*CON_KCALTOK
      tclocal(11) =      0.24214d0*CON_KCALTOK
      tclocal(12) =     -5.56442d0*CON_KCALTOK
      tclocal(13) =      0.86975d0*CON_KCALTOK
      tclocal(14) =      0.03834d0*CON_KCALTOK
      tclocal(15) =     -0.97663d0*CON_KCALTOK
      tclocal(16) =      3.82869d0*CON_KCALTOK
      tclocal(17) =      0.45302d0*CON_KCALTOK
      tclocal(18) =     -7.02860d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'s" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 's" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.00518d0*CON_KCALTOK
      tclocal(1) =      0.00704d0*CON_KCALTOK
      tclocal(2) =      0.00562d0*CON_KCALTOK
      tclocal(3) =      0.09671d0*CON_KCALTOK
      tclocal(4) =     -0.11471d0*CON_KCALTOK
      tclocal(5) =     -0.13422d0*CON_KCALTOK
      tclocal(6) =      1.82760d0*CON_KCALTOK
      tclocal(7) =     -5.16391d0*CON_KCALTOK
      tclocal(8) =     -4.47851d0*CON_KCALTOK
      tclocal( 9) =      0.02634d0*CON_KCALTOK
      tclocal(10) =     -0.13126d0*CON_KCALTOK
      tclocal(11) =     -0.11660d0*CON_KCALTOK
      tclocal(12) =      5.07083d0*CON_KCALTOK
      tclocal(13) =     -5.18633d0*CON_KCALTOK
      tclocal(14) =     -3.20178d0*CON_KCALTOK
      tclocal(15) =     -0.65035d0*CON_KCALTOK
      tclocal(16) =      3.64057d0*CON_KCALTOK
      tclocal(17) =      3.72352d0*CON_KCALTOK
      tclocal(18) =      0.22992d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.09831d0*CON_KCALTOK
      tclocal(1) =      0.10889d0*CON_KCALTOK
      tclocal(2) =     -0.08828d0*CON_KCALTOK
      tclocal(3) =     -0.25911d0*CON_KCALTOK
      tclocal(4) =      0.31067d0*CON_KCALTOK
      tclocal(5) =     -0.40570d0*CON_KCALTOK
      tclocal(6) =     -6.87529d0*CON_KCALTOK
      tclocal(7) =     -7.39848d0*CON_KCALTOK
      tclocal(8) =     -7.86871d0*CON_KCALTOK
      tclocal( 9) =     -0.64471d0*CON_KCALTOK
      tclocal(10) =      0.61723d0*CON_KCALTOK
      tclocal(11) =     -0.63083d0*CON_KCALTOK
      tclocal(12) =      6.92908d0*CON_KCALTOK
      tclocal(13) =      2.66770d0*CON_KCALTOK
      tclocal(14) =      1.88258d0*CON_KCALTOK
      tclocal(15) =     -3.49053d0*CON_KCALTOK
      tclocal(16) =      1.26969d0*CON_KCALTOK
      tclocal(17) =     -1.60373d0*CON_KCALTOK
      tclocal(18) =    -13.36549d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.09831d0*CON_KCALTOK
      tclocal(1) =      0.10889d0*CON_KCALTOK
      tclocal(2) =     -0.08828d0*CON_KCALTOK
      tclocal(3) =     -0.25911d0*CON_KCALTOK
      tclocal(4) =      0.31067d0*CON_KCALTOK
      tclocal(5) =     -0.40570d0*CON_KCALTOK
      tclocal(6) =     -6.87529d0*CON_KCALTOK
      tclocal(7) =     -7.39848d0*CON_KCALTOK
      tclocal(8) =     -7.86871d0*CON_KCALTOK
      tclocal( 9) =     -0.64471d0*CON_KCALTOK
      tclocal(10) =      0.61723d0*CON_KCALTOK
      tclocal(11) =     -0.63083d0*CON_KCALTOK
      tclocal(12) =      6.92908d0*CON_KCALTOK
      tclocal(13) =      2.66770d0*CON_KCALTOK
      tclocal(14) =      1.88258d0*CON_KCALTOK
      tclocal(15) =     -3.49053d0*CON_KCALTOK
      tclocal(16) =      1.26969d0*CON_KCALTOK
      tclocal(17) =     -1.60373d0*CON_KCALTOK
      tclocal(18) =    -13.36549d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.07804d0*CON_KCALTOK
      tclocal(1) =      0.10796d0*CON_KCALTOK
      tclocal(2) =      0.10395d0*CON_KCALTOK
      tclocal(3) =      0.15822d0*CON_KCALTOK
      tclocal(4) =      0.48679d0*CON_KCALTOK
      tclocal(5) =      0.13365d0*CON_KCALTOK
      tclocal(6) =     -2.10384d0*CON_KCALTOK
      tclocal(7) =      5.85877d0*CON_KCALTOK
      tclocal(8) =     -0.76467d0*CON_KCALTOK
      tclocal( 9) =      1.30534d0*CON_KCALTOK
      tclocal(10) =      0.42981d0*CON_KCALTOK
      tclocal(11) =      0.08485d0*CON_KCALTOK
      tclocal(12) =     -5.32388d0*CON_KCALTOK
      tclocal(13) =      2.94489d0*CON_KCALTOK
      tclocal(14) =      0.54780d0*CON_KCALTOK
      tclocal(15) =      0.15871d0*CON_KCALTOK
      tclocal(16) =     -0.81199d0*CON_KCALTOK
      tclocal(17) =     -3.75274d0*CON_KCALTOK
      tclocal(18) =     -1.43702d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.78734d0*CON_KCALTOK
      tclocal(1) =     -1.30085d0*CON_KCALTOK
      tclocal(2) =     -0.04755d0*CON_KCALTOK
      tclocal(3) =      0.40536d0*CON_KCALTOK
      tclocal(4) =      0.74322d0*CON_KCALTOK
      tclocal(5) =      0.00636d0*CON_KCALTOK
      tclocal(6) =     -1.84051d0*CON_KCALTOK
      tclocal(7) =     -0.67906d0*CON_KCALTOK
      tclocal(8) =     -5.36366d0*CON_KCALTOK
      tclocal( 9) =     -0.22947d0*CON_KCALTOK
      tclocal(10) =      3.14065d0*CON_KCALTOK
      tclocal(11) =      0.00909d0*CON_KCALTOK
      tclocal(12) =     -7.47951d0*CON_KCALTOK
      tclocal(13) =     -2.55551d0*CON_KCALTOK
      tclocal(14) =      0.16148d0*CON_KCALTOK
      tclocal(15) =     -2.65709d0*CON_KCALTOK
      tclocal(16) =      1.13069d0*CON_KCALTOK
      tclocal(17) =      0.22589d0*CON_KCALTOK
      tclocal(18) =    -12.82776d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.35104d0*CON_KCALTOK
      tclocal(1) =     -0.96057d0*CON_KCALTOK
      tclocal(2) =      0.00284d0*CON_KCALTOK
      tclocal(3) =     -1.82000d0*CON_KCALTOK
      tclocal(4) =      0.18069d0*CON_KCALTOK
      tclocal(5) =     -0.61483d0*CON_KCALTOK
      tclocal(6) =     -3.32554d0*CON_KCALTOK
      tclocal(7) =     -1.58274d0*CON_KCALTOK
      tclocal(8) =     -4.00988d0*CON_KCALTOK
      tclocal( 9) =      5.00579d0*CON_KCALTOK
      tclocal(10) =      3.79409d0*CON_KCALTOK
      tclocal(11) =     -0.53431d0*CON_KCALTOK
      tclocal(12) =    -11.57718d0*CON_KCALTOK
      tclocal(13) =      1.33948d0*CON_KCALTOK
      tclocal(14) =     -0.25098d0*CON_KCALTOK
      tclocal(15) =     -6.42718d0*CON_KCALTOK
      tclocal(16) =     -0.91086d0*CON_KCALTOK
      tclocal(17) =      0.78663d0*CON_KCALTOK
      tclocal(18) =      0.61421d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.27200d0*CON_KCALTOK
      tclocal(1) =     -0.88787d0*CON_KCALTOK
      tclocal(2) =     -0.14944d0*CON_KCALTOK
      tclocal(3) =      2.28330d0*CON_KCALTOK
      tclocal(4) =      0.42723d0*CON_KCALTOK
      tclocal(5) =      0.25031d0*CON_KCALTOK
      tclocal(6) =      3.15490d0*CON_KCALTOK
      tclocal(7) =     -0.91490d0*CON_KCALTOK
      tclocal(8) =     -2.41144d0*CON_KCALTOK
      tclocal( 9) =     -0.11711d0*CON_KCALTOK
      tclocal(10) =      1.56347d0*CON_KCALTOK
      tclocal(11) =      0.18383d0*CON_KCALTOK
      tclocal(12) =     -7.55557d0*CON_KCALTOK
      tclocal(13) =     -0.96769d0*CON_KCALTOK
      tclocal(14) =     -0.16290d0*CON_KCALTOK
      tclocal(15) =     -2.81735d0*CON_KCALTOK
      tclocal(16) =     -1.08745d0*CON_KCALTOK
      tclocal(17) =     -0.51651d0*CON_KCALTOK
      tclocal(18) =      3.24108d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.59508d0*CON_KCALTOK
      tclocal(1) =      0.48958d0*CON_KCALTOK
      tclocal(2) =     -0.40239d0*CON_KCALTOK
      tclocal(3) =     -2.59927d0*CON_KCALTOK
      tclocal(4) =      1.30384d0*CON_KCALTOK
      tclocal(5) =      0.03449d0*CON_KCALTOK
      tclocal(6) =      2.56391d0*CON_KCALTOK
      tclocal(7) =      2.66991d0*CON_KCALTOK
      tclocal(8) =      1.31064d0*CON_KCALTOK
      tclocal( 9) =     -0.38051d0*CON_KCALTOK
      tclocal(10) =      1.59367d0*CON_KCALTOK
      tclocal(11) =     -0.70138d0*CON_KCALTOK
      tclocal(12) =     -2.15298d0*CON_KCALTOK
      tclocal(13) =      1.75692d0*CON_KCALTOK
      tclocal(14) =     -0.44480d0*CON_KCALTOK
      tclocal(15) =     -1.22722d0*CON_KCALTOK
      tclocal(16) =      0.93819d0*CON_KCALTOK
      tclocal(17) =     -0.86823d0*CON_KCALTOK
      tclocal(18) =    -18.82325d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.03130d0*CON_KCALTOK
      tclocal(1) =     -0.03712d0*CON_KCALTOK
      tclocal(2) =      0.10195d0*CON_KCALTOK
      tclocal(3) =     -0.53146d0*CON_KCALTOK
      tclocal(4) =      1.18800d0*CON_KCALTOK
      tclocal(5) =      0.19354d0*CON_KCALTOK
      tclocal(6) =     -0.99984d0*CON_KCALTOK
      tclocal(7) =     -3.28199d0*CON_KCALTOK
      tclocal(8) =      1.67813d0*CON_KCALTOK
      tclocal( 9) =      0.33211d0*CON_KCALTOK
      tclocal(10) =      1.70030d0*CON_KCALTOK
      tclocal(11) =     -0.08399d0*CON_KCALTOK
      tclocal(12) =     -2.19665d0*CON_KCALTOK
      tclocal(13) =      3.01170d0*CON_KCALTOK
      tclocal(14) =      0.77739d0*CON_KCALTOK
      tclocal(15) =      0.05384d0*CON_KCALTOK
      tclocal(16) =     -0.49542d0*CON_KCALTOK
      tclocal(17) =     -0.11239d0*CON_KCALTOK
      tclocal(18) =    -11.78773d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.54742d0*CON_KCALTOK
      tclocal(1) =     -0.33979d0*CON_KCALTOK
      tclocal(2) =     -0.05737d0*CON_KCALTOK
      tclocal(3) =     -1.67517d0*CON_KCALTOK
      tclocal(4) =      1.18171d0*CON_KCALTOK
      tclocal(5) =     -0.34161d0*CON_KCALTOK
      tclocal(6) =      3.47574d0*CON_KCALTOK
      tclocal(7) =      1.18659d0*CON_KCALTOK
      tclocal(8) =      1.84183d0*CON_KCALTOK
      tclocal( 9) =      1.86544d0*CON_KCALTOK
      tclocal(10) =      2.11915d0*CON_KCALTOK
      tclocal(11) =     -0.82293d0*CON_KCALTOK
      tclocal(12) =     -2.34354d0*CON_KCALTOK
      tclocal(13) =      2.20539d0*CON_KCALTOK
      tclocal(14) =      0.20986d0*CON_KCALTOK
      tclocal(15) =     -2.93587d0*CON_KCALTOK
      tclocal(16) =     -1.30477d0*CON_KCALTOK
      tclocal(17) =     -0.61775d0*CON_KCALTOK
      tclocal(18) =    -12.94769d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.25244d0*CON_KCALTOK
      tclocal(1) =     -1.04489d0*CON_KCALTOK
      tclocal(2) =     -0.00660d0*CON_KCALTOK
      tclocal(3) =     -1.92281d0*CON_KCALTOK
      tclocal(4) =      0.66059d0*CON_KCALTOK
      tclocal(5) =     -0.40170d0*CON_KCALTOK
      tclocal(6) =     15.34178d0*CON_KCALTOK
      tclocal(7) =      0.07047d0*CON_KCALTOK
      tclocal(8) =      4.29002d0*CON_KCALTOK
      tclocal( 9) =     -0.28373d0*CON_KCALTOK
      tclocal(10) =      1.51652d0*CON_KCALTOK
      tclocal(11) =     -1.36986d0*CON_KCALTOK
      tclocal(12) =     -8.26919d0*CON_KCALTOK
      tclocal(13) =      0.71364d0*CON_KCALTOK
      tclocal(14) =     -0.40510d0*CON_KCALTOK
      tclocal(15) =     -1.79469d0*CON_KCALTOK
      tclocal(16) =     -0.38676d0*CON_KCALTOK
      tclocal(17) =     -0.39302d0*CON_KCALTOK
      tclocal(18) =    -13.23230d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.34500d0*CON_KCALTOK
      tclocal(1) =      0.03818d0*CON_KCALTOK
      tclocal(2) =      0.08762d0*CON_KCALTOK
      tclocal(3) =     -0.18921d0*CON_KCALTOK
      tclocal(4) =      1.38472d0*CON_KCALTOK
      tclocal(5) =     -0.33311d0*CON_KCALTOK
      tclocal(6) =      2.32290d0*CON_KCALTOK
      tclocal(7) =     -0.57838d0*CON_KCALTOK
      tclocal(8) =      0.76018d0*CON_KCALTOK
      tclocal( 9) =      0.08777d0*CON_KCALTOK
      tclocal(10) =      2.23118d0*CON_KCALTOK
      tclocal(11) =      0.07200d0*CON_KCALTOK
      tclocal(12) =      0.77057d0*CON_KCALTOK
      tclocal(13) =      2.49844d0*CON_KCALTOK
      tclocal(14) =      0.37763d0*CON_KCALTOK
      tclocal(15) =      1.24927d0*CON_KCALTOK
      tclocal(16) =     -0.14479d0*CON_KCALTOK
      tclocal(17) =      0.47251d0*CON_KCALTOK
      tclocal(18) =    -17.17163d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.07669d0*CON_KCALTOK
      tclocal(1) =      0.54063d0*CON_KCALTOK
      tclocal(2) =     -0.10474d0*CON_KCALTOK
      tclocal(3) =     -0.10006d0*CON_KCALTOK
      tclocal(4) =      2.32434d0*CON_KCALTOK
      tclocal(5) =      0.01826d0*CON_KCALTOK
      tclocal(6) =      3.98037d0*CON_KCALTOK
      tclocal(7) =      5.09466d0*CON_KCALTOK
      tclocal(8) =      0.68809d0*CON_KCALTOK
      tclocal( 9) =      1.31113d0*CON_KCALTOK
      tclocal(10) =      1.33989d0*CON_KCALTOK
      tclocal(11) =      0.28485d0*CON_KCALTOK
      tclocal(12) =     -1.08356d0*CON_KCALTOK
      tclocal(13) =      3.05451d0*CON_KCALTOK
      tclocal(14) =      1.11958d0*CON_KCALTOK
      tclocal(15) =     -1.19957d0*CON_KCALTOK
      tclocal(16) =      1.98283d0*CON_KCALTOK
      tclocal(17) =      0.22742d0*CON_KCALTOK
      tclocal(18) =    -19.46011d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.13163d0*CON_KCALTOK
      tclocal(1) =     -0.10516d0*CON_KCALTOK
      tclocal(2) =     -0.12552d0*CON_KCALTOK
      tclocal(3) =     -3.42627d0*CON_KCALTOK
      tclocal(4) =      1.03878d0*CON_KCALTOK
      tclocal(5) =     -0.08808d0*CON_KCALTOK
      tclocal(6) =      3.23310d0*CON_KCALTOK
      tclocal(7) =      1.25928d0*CON_KCALTOK
      tclocal(8) =      0.48936d0*CON_KCALTOK
      tclocal( 9) =      0.08394d0*CON_KCALTOK
      tclocal(10) =      0.34170d0*CON_KCALTOK
      tclocal(11) =      0.21776d0*CON_KCALTOK
      tclocal(12) =     -5.25824d0*CON_KCALTOK
      tclocal(13) =      1.72864d0*CON_KCALTOK
      tclocal(14) =     -0.41467d0*CON_KCALTOK
      tclocal(15) =     -0.62835d0*CON_KCALTOK
      tclocal(16) =     -0.37606d0*CON_KCALTOK
      tclocal(17) =     -0.18608d0*CON_KCALTOK
      tclocal(18) =    -11.56082d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00423d0*CON_KCALTOK
      tclocal(1) =     -0.06722d0*CON_KCALTOK
      tclocal(2) =     -0.15552d0*CON_KCALTOK
      tclocal(3) =     -0.19173d0*CON_KCALTOK
      tclocal(4) =     -0.22032d0*CON_KCALTOK
      tclocal(5) =     -1.09787d0*CON_KCALTOK
      tclocal(6) =      0.06828d0*CON_KCALTOK
      tclocal(7) =      0.08677d0*CON_KCALTOK
      tclocal(8) =      0.76134d0*CON_KCALTOK
      tclocal( 9) =      0.12338d0*CON_KCALTOK
      tclocal(10) =      0.32465d0*CON_KCALTOK
      tclocal(11) =      0.54027d0*CON_KCALTOK
      tclocal(12) =     -5.15316d0*CON_KCALTOK
      tclocal(13) =      1.46165d0*CON_KCALTOK
      tclocal(14) =     -0.07047d0*CON_KCALTOK
      tclocal(15) =     -0.19752d0*CON_KCALTOK
      tclocal(16) =      0.39239d0*CON_KCALTOK
      tclocal(17) =     -0.37997d0*CON_KCALTOK
      tclocal(18) =     -7.37809d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'s" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 's" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =     -0.35125d0*CON_KCALTOK
      tclocal(2) =      0.11612d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.60576d0*CON_KCALTOK
      tclocal(5) =     -1.73754d0*CON_KCALTOK
      tclocal(6) =     -5.60121d0*CON_KCALTOK
      tclocal(7) =     -2.76341d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.26980d0*CON_KCALTOK
      tclocal(11) =     -0.90805d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      1.18371d0*CON_KCALTOK
      tclocal(14) =      1.55392d0*CON_KCALTOK
      tclocal(15) =      1.27328d0*CON_KCALTOK
      tclocal(16) =      0.58501d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -28.86951d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n+ ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n+ '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.80113d0*CON_KCALTOK
      tclocal(1) =     -0.58298d0*CON_KCALTOK
      tclocal(2) =     -0.23696d0*CON_KCALTOK
      tclocal(3) =      0.84627d0*CON_KCALTOK
      tclocal(4) =      0.57731d0*CON_KCALTOK
      tclocal(5) =      1.20827d0*CON_KCALTOK
      tclocal(6) =     -0.56127d0*CON_KCALTOK
      tclocal(7) =     -1.63256d0*CON_KCALTOK
      tclocal(8) =      1.46780d0*CON_KCALTOK
      tclocal( 9) =      0.58945d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.89087d0*CON_KCALTOK
      tclocal(12) =     -5.29776d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =     -0.50095d0*CON_KCALTOK
      tclocal(15) =     -0.38238d0*CON_KCALTOK
      tclocal(16) =     -1.07851d0*CON_KCALTOK
      tclocal(17) =      0.99589d0*CON_KCALTOK
      tclocal(18) =    -19.08592d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n+ ')
      call twh_torstring(GLB_SET,type,4,'h+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n+ '
      tnlocal(nsame,4) = 'h+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.47228d0*CON_KCALTOK
      tclocal(1) =      1.18503d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      3.07962d0*CON_KCALTOK
      tclocal(4) =      2.23175d0*CON_KCALTOK
      tclocal(5) =     -1.00848d0*CON_KCALTOK
      tclocal(6) =     -2.52855d0*CON_KCALTOK
      tclocal(7) =     -1.78934d0*CON_KCALTOK
      tclocal(8) =     -3.34240d0*CON_KCALTOK
      tclocal( 9) =     -7.24427d0*CON_KCALTOK
      tclocal(10) =     -4.01282d0*CON_KCALTOK
      tclocal(11) =     -2.28551d0*CON_KCALTOK
      tclocal(12) =    -10.22477d0*CON_KCALTOK
      tclocal(13) =     -6.95131d0*CON_KCALTOK
      tclocal(14) =     -1.73595d0*CON_KCALTOK
      tclocal(15) =      4.92033d0*CON_KCALTOK
      tclocal(16) =      2.36047d0*CON_KCALTOK
      tclocal(17) =      0.61815d0*CON_KCALTOK
      tclocal(18) =     28.18115d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c- ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n+ ')
      call twh_torstring(GLB_SET,type,4,'h+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c- '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n+ '
      tnlocal(nsame,4) = 'h+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.12369d0*CON_KCALTOK
      tclocal(1) =     -0.16421d0*CON_KCALTOK
      tclocal(2) =     -0.15043d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.23801d0*CON_KCALTOK
      tclocal(5) =     -1.23318d0*CON_KCALTOK
      tclocal(6) =      1.91203d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =     -3.09941d0*CON_KCALTOK
      tclocal( 9) =      0.98064d0*CON_KCALTOK
      tclocal(10) =      1.00982d0*CON_KCALTOK
      tclocal(11) =     -0.21596d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.24842d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      1.12465d0*CON_KCALTOK
      tclocal(17) =      0.38600d0*CON_KCALTOK
      tclocal(18) =    -20.60064d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n+ ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n+ '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.34117d0*CON_KCALTOK
      tclocal(1) =     -0.05467d0*CON_KCALTOK
      tclocal(2) =     -0.04903d0*CON_KCALTOK
      tclocal(3) =      1.07962d0*CON_KCALTOK
      tclocal(4) =     -0.50825d0*CON_KCALTOK
      tclocal(5) =     -0.73870d0*CON_KCALTOK
      tclocal(6) =      0.08111d0*CON_KCALTOK
      tclocal(7) =      0.08499d0*CON_KCALTOK
      tclocal(8) =     -2.48646d0*CON_KCALTOK
      tclocal( 9) =      0.16300d0*CON_KCALTOK
      tclocal(10) =      0.45069d0*CON_KCALTOK
      tclocal(11) =     -1.62523d0*CON_KCALTOK
      tclocal(12) =      1.19090d0*CON_KCALTOK
      tclocal(13) =      0.83238d0*CON_KCALTOK
      tclocal(14) =     -0.20075d0*CON_KCALTOK
      tclocal(15) =      0.56075d0*CON_KCALTOK
      tclocal(16) =      0.57348d0*CON_KCALTOK
      tclocal(17) =      0.12351d0*CON_KCALTOK
      tclocal(18) =    -17.89225d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n+ ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n+ '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.69764d0*CON_KCALTOK
      tclocal(1) =     -0.23116d0*CON_KCALTOK
      tclocal(2) =     -0.15293d0*CON_KCALTOK
      tclocal(3) =      0.15724d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =     -0.92879d0*CON_KCALTOK
      tclocal(6) =      4.13164d0*CON_KCALTOK
      tclocal(7) =      0.19588d0*CON_KCALTOK
      tclocal(8) =     -0.63105d0*CON_KCALTOK
      tclocal( 9) =      0.39277d0*CON_KCALTOK
      tclocal(10) =      0.37514d0*CON_KCALTOK
      tclocal(11) =     -1.03861d0*CON_KCALTOK
      tclocal(12) =     -2.32411d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =     -0.45879d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.21264d0*CON_KCALTOK
      tclocal(17) =     -0.91531d0*CON_KCALTOK
      tclocal(18) =    -15.54836d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n+ ')
      call twh_torstring(GLB_SET,type,4,'h+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n+ '
      tnlocal(nsame,4) = 'h+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.01963d0*CON_KCALTOK
      tclocal(1) =      0.47755d0*CON_KCALTOK
      tclocal(2) =      0.06309d0*CON_KCALTOK
      tclocal(3) =     -3.24716d0*CON_KCALTOK
      tclocal(4) =     -0.90585d0*CON_KCALTOK
      tclocal(5) =      0.30712d0*CON_KCALTOK
      tclocal(6) =     -1.94128d0*CON_KCALTOK
      tclocal(7) =      0.46084d0*CON_KCALTOK
      tclocal(8) =      1.36355d0*CON_KCALTOK
      tclocal( 9) =     -1.61154d0*CON_KCALTOK
      tclocal(10) =      0.22919d0*CON_KCALTOK
      tclocal(11) =      0.75970d0*CON_KCALTOK
      tclocal(12) =      2.83500d0*CON_KCALTOK
      tclocal(13) =     -2.56238d0*CON_KCALTOK
      tclocal(14) =      1.30915d0*CON_KCALTOK
      tclocal(15) =      0.78719d0*CON_KCALTOK
      tclocal(16) =     -0.03972d0*CON_KCALTOK
      tclocal(17) =     -0.31016d0*CON_KCALTOK
      tclocal(18) =    -18.86608d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'c= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'c= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.72366d0*CON_KCALTOK
      tclocal(1) =      0.92758d0*CON_KCALTOK
      tclocal(2) =      0.21899d0*CON_KCALTOK
      tclocal(3) =     -1.53842d0*CON_KCALTOK
      tclocal(4) =      0.88934d0*CON_KCALTOK
      tclocal(5) =     -0.36027d0*CON_KCALTOK
      tclocal(6) =     -7.03712d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.47324d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      1.12965d0*CON_KCALTOK
      tclocal(11) =     -0.30198d0*CON_KCALTOK
      tclocal(12) =     -6.83214d0*CON_KCALTOK
      tclocal(13) =     -1.30506d0*CON_KCALTOK
      tclocal(14) =      0.53369d0*CON_KCALTOK
      tclocal(15) =     -3.28827d0*CON_KCALTOK
      tclocal(16) =     -0.56132d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -32.19290d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'n= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'n= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.02177d0*CON_KCALTOK
      tclocal(1) =      0.51135d0*CON_KCALTOK
      tclocal(2) =      0.49550d0*CON_KCALTOK
      tclocal(3) =     -3.18677d0*CON_KCALTOK
      tclocal(4) =      1.19838d0*CON_KCALTOK
      tclocal(5) =      0.17806d0*CON_KCALTOK
      tclocal(6) =      1.21935d0*CON_KCALTOK
      tclocal(7) =      3.32711d0*CON_KCALTOK
      tclocal(8) =      0.84780d0*CON_KCALTOK
      tclocal( 9) =      0.31515d0*CON_KCALTOK
      tclocal(10) =      1.01342d0*CON_KCALTOK
      tclocal(11) =      0.12729d0*CON_KCALTOK
      tclocal(12) =     -0.36738d0*CON_KCALTOK
      tclocal(13) =      0.67741d0*CON_KCALTOK
      tclocal(14) =      1.51602d0*CON_KCALTOK
      tclocal(15) =     -0.22959d0*CON_KCALTOK
      tclocal(16) =      0.32283d0*CON_KCALTOK
      tclocal(17) =      0.37174d0*CON_KCALTOK
      tclocal(18) =    -13.37682d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'c= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'c= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.81111d0*CON_KCALTOK
      tclocal(1) =      1.07901d0*CON_KCALTOK
      tclocal(2) =      0.31496d0*CON_KCALTOK
      tclocal(3) =     -0.73667d0*CON_KCALTOK
      tclocal(4) =      0.79584d0*CON_KCALTOK
      tclocal(5) =     -0.26038d0*CON_KCALTOK
      tclocal(6) =     -3.39769d0*CON_KCALTOK
      tclocal(7) =      2.54570d0*CON_KCALTOK
      tclocal(8) =      1.61567d0*CON_KCALTOK
      tclocal( 9) =      0.84889d0*CON_KCALTOK
      tclocal(10) =     -0.38273d0*CON_KCALTOK
      tclocal(11) =      0.33268d0*CON_KCALTOK
      tclocal(12) =     -2.16698d0*CON_KCALTOK
      tclocal(13) =     -0.95273d0*CON_KCALTOK
      tclocal(14) =      1.45835d0*CON_KCALTOK
      tclocal(15) =     -3.75296d0*CON_KCALTOK
      tclocal(16) =      0.39247d0*CON_KCALTOK
      tclocal(17) =      0.34572d0*CON_KCALTOK
      tclocal(18) =    -22.30013d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'n= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'n= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.00018d0*CON_KCALTOK
      tclocal(1) =      1.57365d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =     -0.89734d0*CON_KCALTOK
      tclocal(4) =      0.96360d0*CON_KCALTOK
      tclocal(5) =      0.35894d0*CON_KCALTOK
      tclocal(6) =     -5.13861d0*CON_KCALTOK
      tclocal(7) =      8.29270d0*CON_KCALTOK
      tclocal(8) =      4.92788d0*CON_KCALTOK
      tclocal( 9) =     -1.54905d0*CON_KCALTOK
      tclocal(10) =      0.79219d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     -5.69560d0*CON_KCALTOK
      tclocal(13) =      4.49803d0*CON_KCALTOK
      tclocal(14) =      3.38655d0*CON_KCALTOK
      tclocal(15) =     -4.24626d0*CON_KCALTOK
      tclocal(16) =      3.60948d0*CON_KCALTOK
      tclocal(17) =      3.89594d0*CON_KCALTOK
      tclocal(18) =    -37.17538d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'n= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'n= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.16354d0*CON_KCALTOK
      tclocal(1) =      0.48463d0*CON_KCALTOK
      tclocal(2) =      0.01749d0*CON_KCALTOK
      tclocal(3) =      0.11591d0*CON_KCALTOK
      tclocal(4) =     -0.00532d0*CON_KCALTOK
      tclocal(5) =     -0.30891d0*CON_KCALTOK
      tclocal(6) =     -9.30649d0*CON_KCALTOK
      tclocal(7) =     -8.99081d0*CON_KCALTOK
      tclocal(8) =     -3.56277d0*CON_KCALTOK
      tclocal( 9) =     -0.01494d0*CON_KCALTOK
      tclocal(10) =     -0.05760d0*CON_KCALTOK
      tclocal(11) =     -0.12551d0*CON_KCALTOK
      tclocal(12) =     -3.24221d0*CON_KCALTOK
      tclocal(13) =      1.76664d0*CON_KCALTOK
      tclocal(14) =      1.72042d0*CON_KCALTOK
      tclocal(15) =      1.39898d0*CON_KCALTOK
      tclocal(16) =      1.58661d0*CON_KCALTOK
      tclocal(17) =     -0.82218d0*CON_KCALTOK
      tclocal(18) =    -28.35098d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'na ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'na '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.33790d0*CON_KCALTOK
      tclocal(1) =     -0.73773d0*CON_KCALTOK
      tclocal(2) =     -0.21451d0*CON_KCALTOK
      tclocal(3) =     -1.10068d0*CON_KCALTOK
      tclocal(4) =      1.08789d0*CON_KCALTOK
      tclocal(5) =     -0.64444d0*CON_KCALTOK
      tclocal(6) =     -2.58231d0*CON_KCALTOK
      tclocal(7) =      0.63704d0*CON_KCALTOK
      tclocal(8) =     -0.41013d0*CON_KCALTOK
      tclocal( 9) =      0.06628d0*CON_KCALTOK
      tclocal(10) =      0.07263d0*CON_KCALTOK
      tclocal(11) =      0.47809d0*CON_KCALTOK
      tclocal(12) =     -3.88724d0*CON_KCALTOK
      tclocal(13) =      5.18111d0*CON_KCALTOK
      tclocal(14) =     -0.04018d0*CON_KCALTOK
      tclocal(15) =      0.33403d0*CON_KCALTOK
      tclocal(16) =     -0.93861d0*CON_KCALTOK
      tclocal(17) =     -0.11159d0*CON_KCALTOK
      tclocal(18) =     -8.77896d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'na ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'na '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.32843d0*CON_KCALTOK
      tclocal(1) =      0.39945d0*CON_KCALTOK
      tclocal(2) =     -0.88356d0*CON_KCALTOK
      tclocal(3) =     -1.88565d0*CON_KCALTOK
      tclocal(4) =      0.07684d0*CON_KCALTOK
      tclocal(5) =     -1.21776d0*CON_KCALTOK
      tclocal(6) =     -3.22287d0*CON_KCALTOK
      tclocal(7) =      0.56285d0*CON_KCALTOK
      tclocal(8) =     -2.99769d0*CON_KCALTOK
      tclocal( 9) =      3.81352d0*CON_KCALTOK
      tclocal(10) =      3.76989d0*CON_KCALTOK
      tclocal(11) =     -2.23615d0*CON_KCALTOK
      tclocal(12) =     -4.97111d0*CON_KCALTOK
      tclocal(13) =      3.15848d0*CON_KCALTOK
      tclocal(14) =     -4.14430d0*CON_KCALTOK
      tclocal(15) =      3.08288d0*CON_KCALTOK
      tclocal(16) =      2.16835d0*CON_KCALTOK
      tclocal(17) =     -1.23436d0*CON_KCALTOK
      tclocal(18) =    -16.23960d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'na ')
      call twh_torstring(GLB_SET,type,4,'na ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'na '
      tnlocal(nsame,4) = 'na '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.22137d0*CON_KCALTOK
      tclocal(1) =     -0.15604d0*CON_KCALTOK
      tclocal(2) =     -0.28609d0*CON_KCALTOK
      tclocal(3) =     -0.68515d0*CON_KCALTOK
      tclocal(4) =      0.83022d0*CON_KCALTOK
      tclocal(5) =      0.40761d0*CON_KCALTOK
      tclocal(6) =     -7.50332d0*CON_KCALTOK
      tclocal(7) =     -7.92112d0*CON_KCALTOK
      tclocal(8) =     -1.35257d0*CON_KCALTOK
      tclocal( 9) =      0.07303d0*CON_KCALTOK
      tclocal(10) =      0.10156d0*CON_KCALTOK
      tclocal(11) =     -0.10258d0*CON_KCALTOK
      tclocal(12) =     -3.06058d0*CON_KCALTOK
      tclocal(13) =      1.08760d0*CON_KCALTOK
      tclocal(14) =     -0.97655d0*CON_KCALTOK
      tclocal(15) =     -1.57929d0*CON_KCALTOK
      tclocal(16) =      0.17036d0*CON_KCALTOK
      tclocal(17) =     -0.66617d0*CON_KCALTOK
      tclocal(18) =    -14.60080d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'na ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'na '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.60319d0*CON_KCALTOK
      tclocal(1) =     -0.56243d0*CON_KCALTOK
      tclocal(2) =     -0.19674d0*CON_KCALTOK
      tclocal(3) =     -1.35874d0*CON_KCALTOK
      tclocal(4) =      1.07743d0*CON_KCALTOK
      tclocal(5) =     -0.11551d0*CON_KCALTOK
      tclocal(6) =     -4.02458d0*CON_KCALTOK
      tclocal(7) =      1.97628d0*CON_KCALTOK
      tclocal(8) =     -0.69848d0*CON_KCALTOK
      tclocal( 9) =      0.09887d0*CON_KCALTOK
      tclocal(10) =      0.35592d0*CON_KCALTOK
      tclocal(11) =      0.24469d0*CON_KCALTOK
      tclocal(12) =     -4.60259d0*CON_KCALTOK
      tclocal(13) =      2.33290d0*CON_KCALTOK
      tclocal(14) =      0.37358d0*CON_KCALTOK
      tclocal(15) =     -0.49926d0*CON_KCALTOK
      tclocal(16) =     -0.51651d0*CON_KCALTOK
      tclocal(17) =     -0.71409d0*CON_KCALTOK
      tclocal(18) =    -12.12305d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'na ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'na '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.43776d0*CON_KCALTOK
      tclocal(1) =      0.28733d0*CON_KCALTOK
      tclocal(2) =     -0.32755d0*CON_KCALTOK
      tclocal(3) =     -0.91432d0*CON_KCALTOK
      tclocal(4) =      2.64532d0*CON_KCALTOK
      tclocal(5) =      0.57405d0*CON_KCALTOK
      tclocal(6) =     -3.74098d0*CON_KCALTOK
      tclocal(7) =      2.65078d0*CON_KCALTOK
      tclocal(8) =     -1.33121d0*CON_KCALTOK
      tclocal( 9) =      1.59231d0*CON_KCALTOK
      tclocal(10) =      2.51677d0*CON_KCALTOK
      tclocal(11) =     -0.06677d0*CON_KCALTOK
      tclocal(12) =     -1.44342d0*CON_KCALTOK
      tclocal(13) =      3.47014d0*CON_KCALTOK
      tclocal(14) =     -2.36528d0*CON_KCALTOK
      tclocal(15) =     -1.34535d0*CON_KCALTOK
      tclocal(16) =      2.30732d0*CON_KCALTOK
      tclocal(17) =     -0.07537d0*CON_KCALTOK
      tclocal(18) =    -24.84136d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'na ')
      call twh_torstring(GLB_SET,type,4,'na ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'na '
      tnlocal(nsame,4) = 'na '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.00239d0*CON_KCALTOK
      tclocal(1) =     -0.01023d0*CON_KCALTOK
      tclocal(2) =      0.00121d0*CON_KCALTOK
      tclocal(3) =     -0.18137d0*CON_KCALTOK
      tclocal(4) =     -0.16374d0*CON_KCALTOK
      tclocal(5) =     -0.11171d0*CON_KCALTOK
      tclocal(6) =     -6.82181d0*CON_KCALTOK
      tclocal(7) =     -1.52968d0*CON_KCALTOK
      tclocal(8) =     -4.32519d0*CON_KCALTOK
      tclocal( 9) =     -0.11876d0*CON_KCALTOK
      tclocal(10) =     -0.01053d0*CON_KCALTOK
      tclocal(11) =     -0.13512d0*CON_KCALTOK
      tclocal(12) =     -0.41651d0*CON_KCALTOK
      tclocal(13) =      2.57106d0*CON_KCALTOK
      tclocal(14) =      0.39667d0*CON_KCALTOK
      tclocal(15) =      1.87362d0*CON_KCALTOK
      tclocal(16) =      2.99021d0*CON_KCALTOK
      tclocal(17) =     -0.22314d0*CON_KCALTOK
      tclocal(18) =     -2.54332d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.00239d0*CON_KCALTOK
      tclocal(1) =     -0.01023d0*CON_KCALTOK
      tclocal(2) =      0.00121d0*CON_KCALTOK
      tclocal(3) =     -0.18137d0*CON_KCALTOK
      tclocal(4) =     -0.16374d0*CON_KCALTOK
      tclocal(5) =     -0.11171d0*CON_KCALTOK
      tclocal(6) =     -6.82181d0*CON_KCALTOK
      tclocal(7) =     -1.52968d0*CON_KCALTOK
      tclocal(8) =     -4.32519d0*CON_KCALTOK
      tclocal( 9) =     -0.11876d0*CON_KCALTOK
      tclocal(10) =     -0.01053d0*CON_KCALTOK
      tclocal(11) =     -0.13512d0*CON_KCALTOK
      tclocal(12) =     -0.41651d0*CON_KCALTOK
      tclocal(13) =      2.57106d0*CON_KCALTOK
      tclocal(14) =      0.39667d0*CON_KCALTOK
      tclocal(15) =      1.87362d0*CON_KCALTOK
      tclocal(16) =      2.99021d0*CON_KCALTOK
      tclocal(17) =     -0.22314d0*CON_KCALTOK
      tclocal(18) =     -2.54332d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.39998d0*CON_KCALTOK
      tclocal(1) =      0.23788d0*CON_KCALTOK
      tclocal(2) =     -0.13637d0*CON_KCALTOK
      tclocal(3) =     -1.70945d0*CON_KCALTOK
      tclocal(4) =      2.28852d0*CON_KCALTOK
      tclocal(5) =      0.18644d0*CON_KCALTOK
      tclocal(6) =     -1.01155d0*CON_KCALTOK
      tclocal(7) =      3.63376d0*CON_KCALTOK
      tclocal(8) =      0.20027d0*CON_KCALTOK
      tclocal( 9) =     -1.88618d0*CON_KCALTOK
      tclocal(10) =      0.00580d0*CON_KCALTOK
      tclocal(11) =     -0.77032d0*CON_KCALTOK
      tclocal(12) =     -3.59501d0*CON_KCALTOK
      tclocal(13) =      3.17815d0*CON_KCALTOK
      tclocal(14) =      0.20686d0*CON_KCALTOK
      tclocal(15) =      0.63497d0*CON_KCALTOK
      tclocal(16) =      0.20824d0*CON_KCALTOK
      tclocal(17) =      0.55132d0*CON_KCALTOK
      tclocal(18) =    -20.83121d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.06899d0*CON_KCALTOK
      tclocal(1) =     -0.01107d0*CON_KCALTOK
      tclocal(2) =     -0.00202d0*CON_KCALTOK
      tclocal(3) =      0.19127d0*CON_KCALTOK
      tclocal(4) =      0.00214d0*CON_KCALTOK
      tclocal(5) =      0.19187d0*CON_KCALTOK
      tclocal(6) =     -5.13124d0*CON_KCALTOK
      tclocal(7) =     -4.90812d0*CON_KCALTOK
      tclocal(8) =      0.05103d0*CON_KCALTOK
      tclocal( 9) =     -0.14380d0*CON_KCALTOK
      tclocal(10) =     -0.10543d0*CON_KCALTOK
      tclocal(11) =     -0.59990d0*CON_KCALTOK
      tclocal(12) =     -4.19806d0*CON_KCALTOK
      tclocal(13) =      3.92527d0*CON_KCALTOK
      tclocal(14) =     -0.05429d0*CON_KCALTOK
      tclocal(15) =     -1.48890d0*CON_KCALTOK
      tclocal(16) =      0.52336d0*CON_KCALTOK
      tclocal(17) =     -0.35473d0*CON_KCALTOK
      tclocal(18) =     -6.71545d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.00239d0*CON_KCALTOK
      tclocal(1) =     -0.01023d0*CON_KCALTOK
      tclocal(2) =      0.00121d0*CON_KCALTOK
      tclocal(3) =     -0.18137d0*CON_KCALTOK
      tclocal(4) =     -0.16374d0*CON_KCALTOK
      tclocal(5) =     -0.11171d0*CON_KCALTOK
      tclocal(6) =     -6.82181d0*CON_KCALTOK
      tclocal(7) =     -1.52968d0*CON_KCALTOK
      tclocal(8) =     -4.32519d0*CON_KCALTOK
      tclocal( 9) =     -0.11876d0*CON_KCALTOK
      tclocal(10) =     -0.01053d0*CON_KCALTOK
      tclocal(11) =     -0.13512d0*CON_KCALTOK
      tclocal(12) =     -0.41651d0*CON_KCALTOK
      tclocal(13) =      2.57106d0*CON_KCALTOK
      tclocal(14) =      0.39667d0*CON_KCALTOK
      tclocal(15) =      1.87362d0*CON_KCALTOK
      tclocal(16) =      2.99021d0*CON_KCALTOK
      tclocal(17) =     -0.22314d0*CON_KCALTOK
      tclocal(18) =     -2.54332d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'co ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'co '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.00239d0*CON_KCALTOK
      tclocal(1) =     -0.01023d0*CON_KCALTOK
      tclocal(2) =      0.00121d0*CON_KCALTOK
      tclocal(3) =     -0.18137d0*CON_KCALTOK
      tclocal(4) =     -0.16374d0*CON_KCALTOK
      tclocal(5) =     -0.11171d0*CON_KCALTOK
      tclocal(6) =     -6.82181d0*CON_KCALTOK
      tclocal(7) =     -1.52968d0*CON_KCALTOK
      tclocal(8) =     -4.32519d0*CON_KCALTOK
      tclocal( 9) =     -0.11876d0*CON_KCALTOK
      tclocal(10) =     -0.01053d0*CON_KCALTOK
      tclocal(11) =     -0.13512d0*CON_KCALTOK
      tclocal(12) =     -0.41651d0*CON_KCALTOK
      tclocal(13) =      2.57106d0*CON_KCALTOK
      tclocal(14) =      0.39667d0*CON_KCALTOK
      tclocal(15) =      1.87362d0*CON_KCALTOK
      tclocal(16) =      2.99021d0*CON_KCALTOK
      tclocal(17) =     -0.22314d0*CON_KCALTOK
      tclocal(18) =     -2.54332d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'co ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'co '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.05795d0*CON_KCALTOK
      tclocal(1) =      0.05054d0*CON_KCALTOK
      tclocal(2) =     -0.23551d0*CON_KCALTOK
      tclocal(3) =     -1.95355d0*CON_KCALTOK
      tclocal(4) =      1.15916d0*CON_KCALTOK
      tclocal(5) =      0.97852d0*CON_KCALTOK
      tclocal(6) =     -3.32323d0*CON_KCALTOK
      tclocal(7) =      0.47166d0*CON_KCALTOK
      tclocal(8) =      1.39969d0*CON_KCALTOK
      tclocal( 9) =      0.18136d0*CON_KCALTOK
      tclocal(10) =      0.47024d0*CON_KCALTOK
      tclocal(11) =     -0.10791d0*CON_KCALTOK
      tclocal(12) =     -2.54063d0*CON_KCALTOK
      tclocal(13) =      1.88372d0*CON_KCALTOK
      tclocal(14) =      0.73313d0*CON_KCALTOK
      tclocal(15) =      0.51342d0*CON_KCALTOK
      tclocal(16) =     -0.57696d0*CON_KCALTOK
      tclocal(17) =     -0.28270d0*CON_KCALTOK
      tclocal(18) =    -17.73433d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.05795d0*CON_KCALTOK
      tclocal(1) =      0.05054d0*CON_KCALTOK
      tclocal(2) =     -0.23551d0*CON_KCALTOK
      tclocal(3) =     -1.95355d0*CON_KCALTOK
      tclocal(4) =      1.15916d0*CON_KCALTOK
      tclocal(5) =      0.97852d0*CON_KCALTOK
      tclocal(6) =     -3.32323d0*CON_KCALTOK
      tclocal(7) =      0.47166d0*CON_KCALTOK
      tclocal(8) =      1.39969d0*CON_KCALTOK
      tclocal( 9) =      0.18136d0*CON_KCALTOK
      tclocal(10) =      0.47024d0*CON_KCALTOK
      tclocal(11) =     -0.10791d0*CON_KCALTOK
      tclocal(12) =     -2.54063d0*CON_KCALTOK
      tclocal(13) =      1.88372d0*CON_KCALTOK
      tclocal(14) =      0.73313d0*CON_KCALTOK
      tclocal(15) =      0.51342d0*CON_KCALTOK
      tclocal(16) =     -0.57696d0*CON_KCALTOK
      tclocal(17) =     -0.28270d0*CON_KCALTOK
      tclocal(18) =    -17.73433d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.62970d0*CON_KCALTOK
      tclocal(1) =      1.09706d0*CON_KCALTOK
      tclocal(2) =     -0.11408d0*CON_KCALTOK
      tclocal(3) =     -2.75632d0*CON_KCALTOK
      tclocal(4) =      0.95194d0*CON_KCALTOK
      tclocal(5) =     -0.14979d0*CON_KCALTOK
      tclocal(6) =      7.15285d0*CON_KCALTOK
      tclocal(7) =      2.51276d0*CON_KCALTOK
      tclocal(8) =      0.88303d0*CON_KCALTOK
      tclocal( 9) =     -0.15513d0*CON_KCALTOK
      tclocal(10) =      1.91785d0*CON_KCALTOK
      tclocal(11) =      1.96830d0*CON_KCALTOK
      tclocal(12) =     -5.54574d0*CON_KCALTOK
      tclocal(13) =      2.10506d0*CON_KCALTOK
      tclocal(14) =     -1.96168d0*CON_KCALTOK
      tclocal(15) =      2.32108d0*CON_KCALTOK
      tclocal(16) =     -0.19690d0*CON_KCALTOK
      tclocal(17) =     -0.49309d0*CON_KCALTOK
      tclocal(18) =    -17.11531d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.02111d0*CON_KCALTOK
      tclocal(1) =     -0.11300d0*CON_KCALTOK
      tclocal(2) =     -0.02108d0*CON_KCALTOK
      tclocal(3) =     -1.30182d0*CON_KCALTOK
      tclocal(4) =      0.40951d0*CON_KCALTOK
      tclocal(5) =      0.12770d0*CON_KCALTOK
      tclocal(6) =     -1.70318d0*CON_KCALTOK
      tclocal(7) =      4.06641d0*CON_KCALTOK
      tclocal(8) =      1.17145d0*CON_KCALTOK
      tclocal( 9) =     -0.70931d0*CON_KCALTOK
      tclocal(10) =      0.23822d0*CON_KCALTOK
      tclocal(11) =      0.45004d0*CON_KCALTOK
      tclocal(12) =     -5.36544d0*CON_KCALTOK
      tclocal(13) =      2.01095d0*CON_KCALTOK
      tclocal(14) =      0.34685d0*CON_KCALTOK
      tclocal(15) =     -0.99180d0*CON_KCALTOK
      tclocal(16) =      0.52340d0*CON_KCALTOK
      tclocal(17) =      0.11419d0*CON_KCALTOK
      tclocal(18) =     -9.65067d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.00239d0*CON_KCALTOK
      tclocal(1) =     -0.01023d0*CON_KCALTOK
      tclocal(2) =      0.00121d0*CON_KCALTOK
      tclocal(3) =     -0.18137d0*CON_KCALTOK
      tclocal(4) =     -0.16374d0*CON_KCALTOK
      tclocal(5) =     -0.11171d0*CON_KCALTOK
      tclocal(6) =     -6.82181d0*CON_KCALTOK
      tclocal(7) =     -1.52968d0*CON_KCALTOK
      tclocal(8) =     -4.32519d0*CON_KCALTOK
      tclocal( 9) =     -0.11876d0*CON_KCALTOK
      tclocal(10) =     -0.01053d0*CON_KCALTOK
      tclocal(11) =     -0.13512d0*CON_KCALTOK
      tclocal(12) =     -0.41651d0*CON_KCALTOK
      tclocal(13) =      2.57106d0*CON_KCALTOK
      tclocal(14) =      0.39667d0*CON_KCALTOK
      tclocal(15) =      1.87362d0*CON_KCALTOK
      tclocal(16) =      2.99021d0*CON_KCALTOK
      tclocal(17) =     -0.22314d0*CON_KCALTOK
      tclocal(18) =     -2.54332d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.84192d0*CON_KCALTOK
      tclocal(1) =     -0.28518d0*CON_KCALTOK
      tclocal(2) =      0.01043d0*CON_KCALTOK
      tclocal(3) =      0.27299d0*CON_KCALTOK
      tclocal(4) =      0.21638d0*CON_KCALTOK
      tclocal(5) =     -0.86059d0*CON_KCALTOK
      tclocal(6) =      0.54650d0*CON_KCALTOK
      tclocal(7) =      0.74030d0*CON_KCALTOK
      tclocal(8) =      0.47267d0*CON_KCALTOK
      tclocal( 9) =      0.89740d0*CON_KCALTOK
      tclocal(10) =     -0.35643d0*CON_KCALTOK
      tclocal(11) =     -0.68452d0*CON_KCALTOK
      tclocal(12) =      1.54304d0*CON_KCALTOK
      tclocal(13) =      3.55973d0*CON_KCALTOK
      tclocal(14) =     -1.10820d0*CON_KCALTOK
      tclocal(15) =     -2.53261d0*CON_KCALTOK
      tclocal(16) =      0.55578d0*CON_KCALTOK
      tclocal(17) =      0.76116d0*CON_KCALTOK
      tclocal(18) =    -17.95722d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.35031d0*CON_KCALTOK
      tclocal(1) =     -0.43667d0*CON_KCALTOK
      tclocal(2) =     -0.15752d0*CON_KCALTOK
      tclocal(3) =      1.19215d0*CON_KCALTOK
      tclocal(4) =     -0.49630d0*CON_KCALTOK
      tclocal(5) =      0.67323d0*CON_KCALTOK
      tclocal(6) =      3.52944d0*CON_KCALTOK
      tclocal(7) =      3.45353d0*CON_KCALTOK
      tclocal(8) =     -0.34484d0*CON_KCALTOK
      tclocal( 9) =     -0.15988d0*CON_KCALTOK
      tclocal(10) =     -0.32007d0*CON_KCALTOK
      tclocal(11) =      0.30573d0*CON_KCALTOK
      tclocal(12) =     -0.78781d0*CON_KCALTOK
      tclocal(13) =      1.11400d0*CON_KCALTOK
      tclocal(14) =     -0.22165d0*CON_KCALTOK
      tclocal(15) =     -1.29241d0*CON_KCALTOK
      tclocal(16) =      0.78718d0*CON_KCALTOK
      tclocal(17) =     -1.29313d0*CON_KCALTOK
      tclocal(18) =    -14.50733d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.06899d0*CON_KCALTOK
      tclocal(1) =     -0.01107d0*CON_KCALTOK
      tclocal(2) =     -0.00202d0*CON_KCALTOK
      tclocal(3) =      0.19127d0*CON_KCALTOK
      tclocal(4) =      0.00214d0*CON_KCALTOK
      tclocal(5) =      0.19187d0*CON_KCALTOK
      tclocal(6) =     -5.13124d0*CON_KCALTOK
      tclocal(7) =     -4.90812d0*CON_KCALTOK
      tclocal(8) =      0.05103d0*CON_KCALTOK
      tclocal( 9) =     -0.14380d0*CON_KCALTOK
      tclocal(10) =     -0.10543d0*CON_KCALTOK
      tclocal(11) =     -0.59990d0*CON_KCALTOK
      tclocal(12) =     -4.19806d0*CON_KCALTOK
      tclocal(13) =      3.92527d0*CON_KCALTOK
      tclocal(14) =     -0.05429d0*CON_KCALTOK
      tclocal(15) =     -1.48890d0*CON_KCALTOK
      tclocal(16) =      0.52336d0*CON_KCALTOK
      tclocal(17) =     -0.35473d0*CON_KCALTOK
      tclocal(18) =     -6.71545d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.00518d0*CON_KCALTOK
      tclocal(1) =      0.00704d0*CON_KCALTOK
      tclocal(2) =      0.00562d0*CON_KCALTOK
      tclocal(3) =      0.09671d0*CON_KCALTOK
      tclocal(4) =     -0.11471d0*CON_KCALTOK
      tclocal(5) =     -0.13422d0*CON_KCALTOK
      tclocal(6) =      1.82760d0*CON_KCALTOK
      tclocal(7) =     -5.16391d0*CON_KCALTOK
      tclocal(8) =     -4.47851d0*CON_KCALTOK
      tclocal( 9) =      0.02634d0*CON_KCALTOK
      tclocal(10) =     -0.13126d0*CON_KCALTOK
      tclocal(11) =     -0.11660d0*CON_KCALTOK
      tclocal(12) =      5.07083d0*CON_KCALTOK
      tclocal(13) =     -5.18633d0*CON_KCALTOK
      tclocal(14) =     -3.20178d0*CON_KCALTOK
      tclocal(15) =     -0.65035d0*CON_KCALTOK
      tclocal(16) =      3.64057d0*CON_KCALTOK
      tclocal(17) =      3.72352d0*CON_KCALTOK
      tclocal(18) =      0.22992d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.07804d0*CON_KCALTOK
      tclocal(1) =      0.10796d0*CON_KCALTOK
      tclocal(2) =      0.10395d0*CON_KCALTOK
      tclocal(3) =      0.15822d0*CON_KCALTOK
      tclocal(4) =      0.48679d0*CON_KCALTOK
      tclocal(5) =      0.13365d0*CON_KCALTOK
      tclocal(6) =     -2.10384d0*CON_KCALTOK
      tclocal(7) =      5.85877d0*CON_KCALTOK
      tclocal(8) =     -0.76467d0*CON_KCALTOK
      tclocal( 9) =      1.30534d0*CON_KCALTOK
      tclocal(10) =      0.42981d0*CON_KCALTOK
      tclocal(11) =      0.08485d0*CON_KCALTOK
      tclocal(12) =     -5.32388d0*CON_KCALTOK
      tclocal(13) =      2.94489d0*CON_KCALTOK
      tclocal(14) =      0.54780d0*CON_KCALTOK
      tclocal(15) =      0.15871d0*CON_KCALTOK
      tclocal(16) =     -0.81199d0*CON_KCALTOK
      tclocal(17) =     -3.75274d0*CON_KCALTOK
      tclocal(18) =     -1.43702d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.05795d0*CON_KCALTOK
      tclocal(1) =      0.05054d0*CON_KCALTOK
      tclocal(2) =     -0.23551d0*CON_KCALTOK
      tclocal(3) =     -1.95355d0*CON_KCALTOK
      tclocal(4) =      1.15916d0*CON_KCALTOK
      tclocal(5) =      0.97852d0*CON_KCALTOK
      tclocal(6) =     -3.32323d0*CON_KCALTOK
      tclocal(7) =      0.47166d0*CON_KCALTOK
      tclocal(8) =      1.39969d0*CON_KCALTOK
      tclocal( 9) =      0.18136d0*CON_KCALTOK
      tclocal(10) =      0.47024d0*CON_KCALTOK
      tclocal(11) =     -0.10791d0*CON_KCALTOK
      tclocal(12) =     -2.54063d0*CON_KCALTOK
      tclocal(13) =      1.88372d0*CON_KCALTOK
      tclocal(14) =      0.73313d0*CON_KCALTOK
      tclocal(15) =      0.51342d0*CON_KCALTOK
      tclocal(16) =     -0.57696d0*CON_KCALTOK
      tclocal(17) =     -0.28270d0*CON_KCALTOK
      tclocal(18) =    -17.73433d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.07669d0*CON_KCALTOK
      tclocal(1) =      0.54063d0*CON_KCALTOK
      tclocal(2) =     -0.10474d0*CON_KCALTOK
      tclocal(3) =     -0.10006d0*CON_KCALTOK
      tclocal(4) =      2.32434d0*CON_KCALTOK
      tclocal(5) =      0.01826d0*CON_KCALTOK
      tclocal(6) =      3.98037d0*CON_KCALTOK
      tclocal(7) =      5.09466d0*CON_KCALTOK
      tclocal(8) =      0.68809d0*CON_KCALTOK
      tclocal( 9) =      1.31113d0*CON_KCALTOK
      tclocal(10) =      1.33989d0*CON_KCALTOK
      tclocal(11) =      0.28485d0*CON_KCALTOK
      tclocal(12) =     -1.08356d0*CON_KCALTOK
      tclocal(13) =      3.05451d0*CON_KCALTOK
      tclocal(14) =      1.11958d0*CON_KCALTOK
      tclocal(15) =     -1.19957d0*CON_KCALTOK
      tclocal(16) =      1.98283d0*CON_KCALTOK
      tclocal(17) =      0.22742d0*CON_KCALTOK
      tclocal(18) =    -19.46011d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.28345d0*CON_KCALTOK
      tclocal(1) =     -0.30601d0*CON_KCALTOK
      tclocal(2) =     -0.06991d0*CON_KCALTOK
      tclocal(3) =     -1.57253d0*CON_KCALTOK
      tclocal(4) =      0.89247d0*CON_KCALTOK
      tclocal(5) =     -0.97031d0*CON_KCALTOK
      tclocal(6) =      2.05294d0*CON_KCALTOK
      tclocal(7) =      1.79481d0*CON_KCALTOK
      tclocal(8) =      0.95885d0*CON_KCALTOK
      tclocal( 9) =      0.06316d0*CON_KCALTOK
      tclocal(10) =      1.36518d0*CON_KCALTOK
      tclocal(11) =     -0.40411d0*CON_KCALTOK
      tclocal(12) =     -2.58293d0*CON_KCALTOK
      tclocal(13) =      1.41495d0*CON_KCALTOK
      tclocal(14) =     -1.10757d0*CON_KCALTOK
      tclocal(15) =     -0.45014d0*CON_KCALTOK
      tclocal(16) =      0.93815d0*CON_KCALTOK
      tclocal(17) =     -0.04974d0*CON_KCALTOK
      tclocal(18) =    -15.86719d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.02111d0*CON_KCALTOK
      tclocal(1) =     -0.11300d0*CON_KCALTOK
      tclocal(2) =     -0.02108d0*CON_KCALTOK
      tclocal(3) =     -1.30182d0*CON_KCALTOK
      tclocal(4) =      0.40951d0*CON_KCALTOK
      tclocal(5) =      0.12770d0*CON_KCALTOK
      tclocal(6) =     -1.70318d0*CON_KCALTOK
      tclocal(7) =      4.06641d0*CON_KCALTOK
      tclocal(8) =      1.17145d0*CON_KCALTOK
      tclocal( 9) =     -0.70931d0*CON_KCALTOK
      tclocal(10) =      0.23822d0*CON_KCALTOK
      tclocal(11) =      0.45004d0*CON_KCALTOK
      tclocal(12) =     -5.36544d0*CON_KCALTOK
      tclocal(13) =      2.01095d0*CON_KCALTOK
      tclocal(14) =      0.34685d0*CON_KCALTOK
      tclocal(15) =     -0.99180d0*CON_KCALTOK
      tclocal(16) =      0.52340d0*CON_KCALTOK
      tclocal(17) =      0.11419d0*CON_KCALTOK
      tclocal(18) =     -9.65067d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.60503d0*CON_KCALTOK
      tclocal(1) =     -0.35211d0*CON_KCALTOK
      tclocal(2) =     -0.40116d0*CON_KCALTOK
      tclocal(3) =     -0.28562d0*CON_KCALTOK
      tclocal(4) =      1.22290d0*CON_KCALTOK
      tclocal(5) =     -0.90644d0*CON_KCALTOK
      tclocal(6) =     -6.89393d0*CON_KCALTOK
      tclocal(7) =     -3.14036d0*CON_KCALTOK
      tclocal(8) =     -0.36923d0*CON_KCALTOK
      tclocal( 9) =      0.55126d0*CON_KCALTOK
      tclocal(10) =      1.46914d0*CON_KCALTOK
      tclocal(11) =      0.64835d0*CON_KCALTOK
      tclocal(12) =     -3.19370d0*CON_KCALTOK
      tclocal(13) =      1.72991d0*CON_KCALTOK
      tclocal(14) =     -1.04128d0*CON_KCALTOK
      tclocal(15) =      0.65996d0*CON_KCALTOK
      tclocal(16) =      1.09880d0*CON_KCALTOK
      tclocal(17) =      0.08170d0*CON_KCALTOK
      tclocal(18) =    -22.09994d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.15144d0*CON_KCALTOK
      tclocal(1) =     -0.37789d0*CON_KCALTOK
      tclocal(2) =      0.13183d0*CON_KCALTOK
      tclocal(3) =     -1.41440d0*CON_KCALTOK
      tclocal(4) =     -0.19938d0*CON_KCALTOK
      tclocal(5) =     -0.11214d0*CON_KCALTOK
      tclocal(6) =     11.56003d0*CON_KCALTOK
      tclocal(7) =      3.07222d0*CON_KCALTOK
      tclocal(8) =      2.59682d0*CON_KCALTOK
      tclocal( 9) =      0.29772d0*CON_KCALTOK
      tclocal(10) =      0.94567d0*CON_KCALTOK
      tclocal(11) =     -0.08462d0*CON_KCALTOK
      tclocal(12) =     -0.53718d0*CON_KCALTOK
      tclocal(13) =      1.68510d0*CON_KCALTOK
      tclocal(14) =     -0.78658d0*CON_KCALTOK
      tclocal(15) =     -0.10354d0*CON_KCALTOK
      tclocal(16) =     -1.06501d0*CON_KCALTOK
      tclocal(17) =      0.84062d0*CON_KCALTOK
      tclocal(18) =    -18.26542d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.15144d0*CON_KCALTOK
      tclocal(1) =     -0.37789d0*CON_KCALTOK
      tclocal(2) =      0.13183d0*CON_KCALTOK
      tclocal(3) =     -1.41440d0*CON_KCALTOK
      tclocal(4) =     -0.19938d0*CON_KCALTOK
      tclocal(5) =     -0.11214d0*CON_KCALTOK
      tclocal(6) =     11.56003d0*CON_KCALTOK
      tclocal(7) =      3.07222d0*CON_KCALTOK
      tclocal(8) =      2.59682d0*CON_KCALTOK
      tclocal( 9) =      0.29772d0*CON_KCALTOK
      tclocal(10) =      0.94567d0*CON_KCALTOK
      tclocal(11) =     -0.08462d0*CON_KCALTOK
      tclocal(12) =     -0.53718d0*CON_KCALTOK
      tclocal(13) =      1.68510d0*CON_KCALTOK
      tclocal(14) =     -0.78658d0*CON_KCALTOK
      tclocal(15) =     -0.10354d0*CON_KCALTOK
      tclocal(16) =     -1.06501d0*CON_KCALTOK
      tclocal(17) =      0.84062d0*CON_KCALTOK
      tclocal(18) =    -18.26542d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.52832d0*CON_KCALTOK
      tclocal(1) =     -0.02616d0*CON_KCALTOK
      tclocal(2) =     -0.42449d0*CON_KCALTOK
      tclocal(3) =     -1.31363d0*CON_KCALTOK
      tclocal(4) =      0.62628d0*CON_KCALTOK
      tclocal(5) =     -1.21845d0*CON_KCALTOK
      tclocal(6) =     -3.96910d0*CON_KCALTOK
      tclocal(7) =      0.68873d0*CON_KCALTOK
      tclocal(8) =      0.00168d0*CON_KCALTOK
      tclocal( 9) =      3.07259d0*CON_KCALTOK
      tclocal(10) =      3.52930d0*CON_KCALTOK
      tclocal(11) =      2.57370d0*CON_KCALTOK
      tclocal(12) =     -5.60924d0*CON_KCALTOK
      tclocal(13) =      3.42885d0*CON_KCALTOK
      tclocal(14) =      1.62327d0*CON_KCALTOK
      tclocal(15) =      3.58343d0*CON_KCALTOK
      tclocal(16) =      3.12873d0*CON_KCALTOK
      tclocal(17) =      5.27261d0*CON_KCALTOK
      tclocal(18) =    -19.28023d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.36054d0*CON_KCALTOK
      tclocal(1) =     -0.33042d0*CON_KCALTOK
      tclocal(2) =     -0.29403d0*CON_KCALTOK
      tclocal(3) =     -0.66652d0*CON_KCALTOK
      tclocal(4) =      0.34751d0*CON_KCALTOK
      tclocal(5) =      0.00857d0*CON_KCALTOK
      tclocal(6) =     20.54226d0*CON_KCALTOK
      tclocal(7) =      1.61828d0*CON_KCALTOK
      tclocal(8) =     -4.82969d0*CON_KCALTOK
      tclocal( 9) =     -2.73790d0*CON_KCALTOK
      tclocal(10) =     -1.10279d0*CON_KCALTOK
      tclocal(11) =      0.65248d0*CON_KCALTOK
      tclocal(12) =     -2.15718d0*CON_KCALTOK
      tclocal(13) =      2.16301d0*CON_KCALTOK
      tclocal(14) =      0.60275d0*CON_KCALTOK
      tclocal(15) =      1.59148d0*CON_KCALTOK
      tclocal(16) =      0.84185d0*CON_KCALTOK
      tclocal(17) =     -0.80372d0*CON_KCALTOK
      tclocal(18) =    -17.34369d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.78277d0*CON_KCALTOK
      tclocal(1) =     -0.55559d0*CON_KCALTOK
      tclocal(2) =     -0.19416d0*CON_KCALTOK
      tclocal(3) =     -0.67444d0*CON_KCALTOK
      tclocal(4) =      1.04702d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      1.45021d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.87036d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.62134d0*CON_KCALTOK
      tclocal(11) =      1.04937d0*CON_KCALTOK
      tclocal(12) =     -4.17481d0*CON_KCALTOK
      tclocal(13) =      2.93317d0*CON_KCALTOK
      tclocal(14) =      0.56832d0*CON_KCALTOK
      tclocal(15) =      1.01470d0*CON_KCALTOK
      tclocal(16) =     -0.41597d0*CON_KCALTOK
      tclocal(17) =      0.45204d0*CON_KCALTOK
      tclocal(18) =    -14.07415d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.19180d0*CON_KCALTOK
      tclocal(1) =     -0.08210d0*CON_KCALTOK
      tclocal(2) =     -0.11780d0*CON_KCALTOK
      tclocal(3) =     -1.31781d0*CON_KCALTOK
      tclocal(4) =      0.20641d0*CON_KCALTOK
      tclocal(5) =     -0.40601d0*CON_KCALTOK
      tclocal(6) =     -0.10186d0*CON_KCALTOK
      tclocal(7) =     -0.29186d0*CON_KCALTOK
      tclocal(8) =     -0.12748d0*CON_KCALTOK
      tclocal( 9) =      0.24202d0*CON_KCALTOK
      tclocal(10) =     -0.31172d0*CON_KCALTOK
      tclocal(11) =     -0.06336d0*CON_KCALTOK
      tclocal(12) =     -0.67740d0*CON_KCALTOK
      tclocal(13) =      0.38355d0*CON_KCALTOK
      tclocal(14) =      0.69781d0*CON_KCALTOK
      tclocal(15) =     -0.22191d0*CON_KCALTOK
      tclocal(16) =      1.27811d0*CON_KCALTOK
      tclocal(17) =     -1.20468d0*CON_KCALTOK
      tclocal(18) =     -6.57951d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'p  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'p  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.38747d0*CON_KCALTOK
      tclocal(1) =     -1.25707d0*CON_KCALTOK
      tclocal(2) =     -0.37931d0*CON_KCALTOK
      tclocal(3) =     -0.75307d0*CON_KCALTOK
      tclocal(4) =      0.31533d0*CON_KCALTOK
      tclocal(5) =     -1.38942d0*CON_KCALTOK
      tclocal(6) =    -30.46603d0*CON_KCALTOK
      tclocal(7) =      0.41377d0*CON_KCALTOK
      tclocal(8) =      1.26945d0*CON_KCALTOK
      tclocal( 9) =     -0.45778d0*CON_KCALTOK
      tclocal(10) =     -0.47894d0*CON_KCALTOK
      tclocal(11) =     -0.37403d0*CON_KCALTOK
      tclocal(12) =     -1.51031d0*CON_KCALTOK
      tclocal(13) =      0.49690d0*CON_KCALTOK
      tclocal(14) =      0.14905d0*CON_KCALTOK
      tclocal(15) =     -1.53641d0*CON_KCALTOK
      tclocal(16) =     -1.19446d0*CON_KCALTOK
      tclocal(17) =      0.23373d0*CON_KCALTOK
      tclocal(18) =    -12.98433d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'p- ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'p- '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.31224d0*CON_KCALTOK
      tclocal(1) =     -0.54277d0*CON_KCALTOK
      tclocal(2) =     -0.85905d0*CON_KCALTOK
      tclocal(3) =      9.77373d0*CON_KCALTOK
      tclocal(4) =     -0.89449d0*CON_KCALTOK
      tclocal(5) =      0.08699d0*CON_KCALTOK
      tclocal(6) =      0.45104d0*CON_KCALTOK
      tclocal(7) =     -3.53832d0*CON_KCALTOK
      tclocal(8) =     -4.75781d0*CON_KCALTOK
      tclocal( 9) =     -0.57186d0*CON_KCALTOK
      tclocal(10) =     -0.74443d0*CON_KCALTOK
      tclocal(11) =      0.47813d0*CON_KCALTOK
      tclocal(12) =     -4.81932d0*CON_KCALTOK
      tclocal(13) =     -0.81628d0*CON_KCALTOK
      tclocal(14) =      0.60844d0*CON_KCALTOK
      tclocal(15) =      6.78685d0*CON_KCALTOK
      tclocal(16) =      0.17642d0*CON_KCALTOK
      tclocal(17) =     -0.29394d0*CON_KCALTOK
      tclocal(18) =    -31.83376d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.15506d0*CON_KCALTOK
      tclocal(1) =      0.23038d0*CON_KCALTOK
      tclocal(2) =      0.28169d0*CON_KCALTOK
      tclocal(3) =     -0.23849d0*CON_KCALTOK
      tclocal(4) =     -0.41695d0*CON_KCALTOK
      tclocal(5) =      0.65664d0*CON_KCALTOK
      tclocal(6) =      0.29993d0*CON_KCALTOK
      tclocal(7) =      4.61016d0*CON_KCALTOK
      tclocal(8) =      2.37459d0*CON_KCALTOK
      tclocal( 9) =      0.01307d0*CON_KCALTOK
      tclocal(10) =      1.59725d0*CON_KCALTOK
      tclocal(11) =      0.79472d0*CON_KCALTOK
      tclocal(12) =     -0.45103d0*CON_KCALTOK
      tclocal(13) =      0.15156d0*CON_KCALTOK
      tclocal(14) =      0.05985d0*CON_KCALTOK
      tclocal(15) =      0.81220d0*CON_KCALTOK
      tclocal(16) =     -0.46526d0*CON_KCALTOK
      tclocal(17) =     -0.49498d0*CON_KCALTOK
      tclocal(18) =     -4.92086d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.51543d0*CON_KCALTOK
      tclocal(1) =     -0.98722d0*CON_KCALTOK
      tclocal(2) =     -0.34813d0*CON_KCALTOK
      tclocal(3) =      1.59744d0*CON_KCALTOK
      tclocal(4) =     -0.19149d0*CON_KCALTOK
      tclocal(5) =     -1.04508d0*CON_KCALTOK
      tclocal(6) =      4.63379d0*CON_KCALTOK
      tclocal(7) =      4.63365d0*CON_KCALTOK
      tclocal(8) =     -4.94119d0*CON_KCALTOK
      tclocal( 9) =      0.36495d0*CON_KCALTOK
      tclocal(10) =      0.59319d0*CON_KCALTOK
      tclocal(11) =      0.89120d0*CON_KCALTOK
      tclocal(12) =     -9.28660d0*CON_KCALTOK
      tclocal(13) =     -0.63416d0*CON_KCALTOK
      tclocal(14) =     -2.10149d0*CON_KCALTOK
      tclocal(15) =     -1.36386d0*CON_KCALTOK
      tclocal(16) =     -2.76639d0*CON_KCALTOK
      tclocal(17) =      0.30844d0*CON_KCALTOK
      tclocal(18) =     -7.08880d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.29980d0*CON_KCALTOK
      tclocal(1) =     -0.52110d0*CON_KCALTOK
      tclocal(2) =     -0.82470d0*CON_KCALTOK
      tclocal(3) =      8.11220d0*CON_KCALTOK
      tclocal(4) =     -0.74240d0*CON_KCALTOK
      tclocal(5) =      0.07220d0*CON_KCALTOK
      tclocal(6) =      0.37440d0*CON_KCALTOK
      tclocal(7) =     -2.93680d0*CON_KCALTOK
      tclocal(8) =     -3.94900d0*CON_KCALTOK
      tclocal( 9) =     -0.47460d0*CON_KCALTOK
      tclocal(10) =     -0.61790d0*CON_KCALTOK
      tclocal(11) =      0.39680d0*CON_KCALTOK
      tclocal(12) =     -4.00000d0*CON_KCALTOK
      tclocal(13) =     -0.67750d0*CON_KCALTOK
      tclocal(14) =      0.50500d0*CON_KCALTOK
      tclocal(15) =      5.63310d0*CON_KCALTOK
      tclocal(16) =      0.14640d0*CON_KCALTOK
      tclocal(17) =     -0.24400d0*CON_KCALTOK
      tclocal(18) =    -26.42200d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.15506d0*CON_KCALTOK
      tclocal(1) =      0.23038d0*CON_KCALTOK
      tclocal(2) =      0.28169d0*CON_KCALTOK
      tclocal(3) =     -0.23849d0*CON_KCALTOK
      tclocal(4) =     -0.41695d0*CON_KCALTOK
      tclocal(5) =      0.65664d0*CON_KCALTOK
      tclocal(6) =      0.29993d0*CON_KCALTOK
      tclocal(7) =      4.61016d0*CON_KCALTOK
      tclocal(8) =      2.37459d0*CON_KCALTOK
      tclocal( 9) =      0.01307d0*CON_KCALTOK
      tclocal(10) =      1.59725d0*CON_KCALTOK
      tclocal(11) =      0.79472d0*CON_KCALTOK
      tclocal(12) =     -0.45103d0*CON_KCALTOK
      tclocal(13) =      0.15156d0*CON_KCALTOK
      tclocal(14) =      0.05985d0*CON_KCALTOK
      tclocal(15) =      0.81220d0*CON_KCALTOK
      tclocal(16) =     -0.46526d0*CON_KCALTOK
      tclocal(17) =     -0.49498d0*CON_KCALTOK
      tclocal(18) =     -4.92086d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.51543d0*CON_KCALTOK
      tclocal(1) =     -0.98722d0*CON_KCALTOK
      tclocal(2) =     -0.34813d0*CON_KCALTOK
      tclocal(3) =      1.59744d0*CON_KCALTOK
      tclocal(4) =     -0.19149d0*CON_KCALTOK
      tclocal(5) =     -1.04508d0*CON_KCALTOK
      tclocal(6) =      4.63379d0*CON_KCALTOK
      tclocal(7) =      4.63365d0*CON_KCALTOK
      tclocal(8) =     -4.94119d0*CON_KCALTOK
      tclocal( 9) =      0.36495d0*CON_KCALTOK
      tclocal(10) =      0.59319d0*CON_KCALTOK
      tclocal(11) =      0.89120d0*CON_KCALTOK
      tclocal(12) =     -9.28660d0*CON_KCALTOK
      tclocal(13) =     -0.63416d0*CON_KCALTOK
      tclocal(14) =     -2.10149d0*CON_KCALTOK
      tclocal(15) =     -1.36386d0*CON_KCALTOK
      tclocal(16) =     -2.76639d0*CON_KCALTOK
      tclocal(17) =      0.30844d0*CON_KCALTOK
      tclocal(18) =     -7.08880d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.30657d0*CON_KCALTOK
      tclocal(1) =     -0.91521d0*CON_KCALTOK
      tclocal(2) =     -0.42449d0*CON_KCALTOK
      tclocal(3) =     -1.76572d0*CON_KCALTOK
      tclocal(4) =      1.36138d0*CON_KCALTOK
      tclocal(5) =     -1.13456d0*CON_KCALTOK
      tclocal(6) =     -6.19928d0*CON_KCALTOK
      tclocal(7) =      0.26564d0*CON_KCALTOK
      tclocal(8) =     -0.71725d0*CON_KCALTOK
      tclocal( 9) =      0.92720d0*CON_KCALTOK
      tclocal(10) =      3.60827d0*CON_KCALTOK
      tclocal(11) =      2.65386d0*CON_KCALTOK
      tclocal(12) =     -6.47725d0*CON_KCALTOK
      tclocal(13) =     -0.04097d0*CON_KCALTOK
      tclocal(14) =      1.02927d0*CON_KCALTOK
      tclocal(15) =     -1.57435d0*CON_KCALTOK
      tclocal(16) =      2.24512d0*CON_KCALTOK
      tclocal(17) =      5.45613d0*CON_KCALTOK
      tclocal(18) =    -20.67806d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'co ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'co '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.08323d0*CON_KCALTOK
      tclocal(1) =     -1.18558d0*CON_KCALTOK
      tclocal(2) =     -0.18197d0*CON_KCALTOK
      tclocal(3) =      0.58855d0*CON_KCALTOK
      tclocal(4) =      0.71214d0*CON_KCALTOK
      tclocal(5) =      0.28725d0*CON_KCALTOK
      tclocal(6) =      7.64467d0*CON_KCALTOK
      tclocal(7) =      3.90031d0*CON_KCALTOK
      tclocal(8) =    -10.07884d0*CON_KCALTOK
      tclocal( 9) =      2.66089d0*CON_KCALTOK
      tclocal(10) =      0.38804d0*CON_KCALTOK
      tclocal(11) =      1.00407d0*CON_KCALTOK
      tclocal(12) =     -3.05535d0*CON_KCALTOK
      tclocal(13) =      0.31038d0*CON_KCALTOK
      tclocal(14) =     -8.51716d0*CON_KCALTOK
      tclocal(15) =     -3.20024d0*CON_KCALTOK
      tclocal(16) =     -0.48349d0*CON_KCALTOK
      tclocal(17) =     -1.51313d0*CON_KCALTOK
      tclocal(18) =    -14.69531d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'co ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'co '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.61648d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =     -0.32971d0*CON_KCALTOK
      tclocal(3) =     -0.70397d0*CON_KCALTOK
      tclocal(4) =      1.55108d0*CON_KCALTOK
      tclocal(5) =      1.12186d0*CON_KCALTOK
      tclocal(6) =     -7.90782d0*CON_KCALTOK
      tclocal(7) =     -5.41233d0*CON_KCALTOK
      tclocal(8) =     -1.63970d0*CON_KCALTOK
      tclocal( 9) =     -0.18839d0*CON_KCALTOK
      tclocal(10) =      0.18190d0*CON_KCALTOK
      tclocal(11) =     -1.32652d0*CON_KCALTOK
      tclocal(12) =     -2.12026d0*CON_KCALTOK
      tclocal(13) =      1.90620d0*CON_KCALTOK
      tclocal(14) =      0.59817d0*CON_KCALTOK
      tclocal(15) =     -0.90434d0*CON_KCALTOK
      tclocal(16) =      0.50463d0*CON_KCALTOK
      tclocal(17) =     -0.77357d0*CON_KCALTOK
      tclocal(18) =    -19.12068d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.10617d0*CON_KCALTOK
      tclocal(1) =      0.13427d0*CON_KCALTOK
      tclocal(2) =      0.08377d0*CON_KCALTOK
      tclocal(3) =      1.11496d0*CON_KCALTOK
      tclocal(4) =      1.06865d0*CON_KCALTOK
      tclocal(5) =     -0.69942d0*CON_KCALTOK
      tclocal(6) =      8.97056d0*CON_KCALTOK
      tclocal(7) =      4.94847d0*CON_KCALTOK
      tclocal(8) =     -1.17653d0*CON_KCALTOK
      tclocal( 9) =      0.26539d0*CON_KCALTOK
      tclocal(10) =      2.67424d0*CON_KCALTOK
      tclocal(11) =     -0.52012d0*CON_KCALTOK
      tclocal(12) =     -0.58025d0*CON_KCALTOK
      tclocal(13) =      3.26291d0*CON_KCALTOK
      tclocal(14) =     -0.04659d0*CON_KCALTOK
      tclocal(15) =     -0.36539d0*CON_KCALTOK
      tclocal(16) =     -1.01152d0*CON_KCALTOK
      tclocal(17) =      0.11290d0*CON_KCALTOK
      tclocal(18) =    -15.29073d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.10617d0*CON_KCALTOK
      tclocal(1) =      0.13427d0*CON_KCALTOK
      tclocal(2) =      0.08377d0*CON_KCALTOK
      tclocal(3) =      1.11496d0*CON_KCALTOK
      tclocal(4) =      1.06865d0*CON_KCALTOK
      tclocal(5) =     -0.69942d0*CON_KCALTOK
      tclocal(6) =      8.97056d0*CON_KCALTOK
      tclocal(7) =      4.94847d0*CON_KCALTOK
      tclocal(8) =     -1.17653d0*CON_KCALTOK
      tclocal( 9) =      0.26539d0*CON_KCALTOK
      tclocal(10) =      2.67424d0*CON_KCALTOK
      tclocal(11) =     -0.52012d0*CON_KCALTOK
      tclocal(12) =     -0.58025d0*CON_KCALTOK
      tclocal(13) =      3.26291d0*CON_KCALTOK
      tclocal(14) =     -0.04659d0*CON_KCALTOK
      tclocal(15) =     -0.36539d0*CON_KCALTOK
      tclocal(16) =     -1.01152d0*CON_KCALTOK
      tclocal(17) =      0.11290d0*CON_KCALTOK
      tclocal(18) =    -15.29073d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.88005d0*CON_KCALTOK
      tclocal(1) =     -0.14980d0*CON_KCALTOK
      tclocal(2) =     -0.23137d0*CON_KCALTOK
      tclocal(3) =     -1.51922d0*CON_KCALTOK
      tclocal(4) =      1.12088d0*CON_KCALTOK
      tclocal(5) =      0.19700d0*CON_KCALTOK
      tclocal(6) =     -5.93747d0*CON_KCALTOK
      tclocal(7) =     -1.73455d0*CON_KCALTOK
      tclocal(8) =     -2.54440d0*CON_KCALTOK
      tclocal( 9) =     -0.46015d0*CON_KCALTOK
      tclocal(10) =      0.76862d0*CON_KCALTOK
      tclocal(11) =     -3.14089d0*CON_KCALTOK
      tclocal(12) =     -3.66946d0*CON_KCALTOK
      tclocal(13) =      1.07103d0*CON_KCALTOK
      tclocal(14) =     -0.71209d0*CON_KCALTOK
      tclocal(15) =     -3.00544d0*CON_KCALTOK
      tclocal(16) =     -2.05447d0*CON_KCALTOK
      tclocal(17) =     -4.46606d0*CON_KCALTOK
      tclocal(18) =    -10.25561d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.48076d0*CON_KCALTOK
      tclocal(1) =      0.32599d0*CON_KCALTOK
      tclocal(2) =     -0.21833d0*CON_KCALTOK
      tclocal(3) =      0.10819d0*CON_KCALTOK
      tclocal(4) =      1.11670d0*CON_KCALTOK
      tclocal(5) =     -0.54362d0*CON_KCALTOK
      tclocal(6) =     16.22719d0*CON_KCALTOK
      tclocal(7) =      2.38091d0*CON_KCALTOK
      tclocal(8) =      0.85690d0*CON_KCALTOK
      tclocal( 9) =      1.55721d0*CON_KCALTOK
      tclocal(10) =      1.87720d0*CON_KCALTOK
      tclocal(11) =     -1.43449d0*CON_KCALTOK
      tclocal(12) =     -1.37223d0*CON_KCALTOK
      tclocal(13) =      1.97894d0*CON_KCALTOK
      tclocal(14) =     -0.56013d0*CON_KCALTOK
      tclocal(15) =      0.46553d0*CON_KCALTOK
      tclocal(16) =      1.49353d0*CON_KCALTOK
      tclocal(17) =     -0.23819d0*CON_KCALTOK
      tclocal(18) =    -20.02920d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.21665d0*CON_KCALTOK
      tclocal(1) =     -0.50436d0*CON_KCALTOK
      tclocal(2) =     -0.24667d0*CON_KCALTOK
      tclocal(3) =     -2.04114d0*CON_KCALTOK
      tclocal(4) =      1.52845d0*CON_KCALTOK
      tclocal(5) =      0.26318d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      1.07456d0*CON_KCALTOK
      tclocal(8) =     -0.68482d0*CON_KCALTOK
      tclocal( 9) =      0.28986d0*CON_KCALTOK
      tclocal(10) =      0.79104d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     -3.96041d0*CON_KCALTOK
      tclocal(13) =      1.90656d0*CON_KCALTOK
      tclocal(14) =      0.08566d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.32676d0*CON_KCALTOK
      tclocal(17) =     -0.69120d0*CON_KCALTOK
      tclocal(18) =    -12.22015d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.11690d0*CON_KCALTOK
      tclocal(1) =      0.19270d0*CON_KCALTOK
      tclocal(2) =     -0.15840d0*CON_KCALTOK
      tclocal(3) =      0.31999d0*CON_KCALTOK
      tclocal(4) =     -0.08188d0*CON_KCALTOK
      tclocal(5) =      0.15278d0*CON_KCALTOK
      tclocal(6) =     -0.03153d0*CON_KCALTOK
      tclocal(7) =      0.25556d0*CON_KCALTOK
      tclocal(8) =     -0.65593d0*CON_KCALTOK
      tclocal( 9) =      1.80263d0*CON_KCALTOK
      tclocal(10) =      1.08006d0*CON_KCALTOK
      tclocal(11) =     -0.64510d0*CON_KCALTOK
      tclocal(12) =     -2.34150d0*CON_KCALTOK
      tclocal(13) =      0.77400d0*CON_KCALTOK
      tclocal(14) =      0.04617d0*CON_KCALTOK
      tclocal(15) =      1.10975d0*CON_KCALTOK
      tclocal(16) =     -0.04995d0*CON_KCALTOK
      tclocal(17) =     -0.24725d0*CON_KCALTOK
      tclocal(18) =     -8.39359d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'p  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'p  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.91910d0*CON_KCALTOK
      tclocal(1) =     -0.17329d0*CON_KCALTOK
      tclocal(2) =      0.02132d0*CON_KCALTOK
      tclocal(3) =      6.25295d0*CON_KCALTOK
      tclocal(4) =      0.27114d0*CON_KCALTOK
      tclocal(5) =      2.10545d0*CON_KCALTOK
      tclocal(6) =    -17.52647d0*CON_KCALTOK
      tclocal(7) =      0.78115d0*CON_KCALTOK
      tclocal(8) =     -2.61157d0*CON_KCALTOK
      tclocal( 9) =      1.95104d0*CON_KCALTOK
      tclocal(10) =      1.36452d0*CON_KCALTOK
      tclocal(11) =     -1.74070d0*CON_KCALTOK
      tclocal(12) =     -3.71187d0*CON_KCALTOK
      tclocal(13) =      1.37998d0*CON_KCALTOK
      tclocal(14) =      1.40220d0*CON_KCALTOK
      tclocal(15) =      1.48924d0*CON_KCALTOK
      tclocal(16) =     -0.87000d0*CON_KCALTOK
      tclocal(17) =     -0.59752d0*CON_KCALTOK
      tclocal(18) =     -4.08609d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'p- ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'p- '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      2.42163d0*CON_KCALTOK
      tclocal(1) =      0.85347d0*CON_KCALTOK
      tclocal(2) =     -0.45939d0*CON_KCALTOK
      tclocal(3) =      1.10526d0*CON_KCALTOK
      tclocal(4) =      0.48055d0*CON_KCALTOK
      tclocal(5) =     -0.09946d0*CON_KCALTOK
      tclocal(6) =      3.59293d0*CON_KCALTOK
      tclocal(7) =     -1.72277d0*CON_KCALTOK
      tclocal(8) =     -1.43319d0*CON_KCALTOK
      tclocal( 9) =      1.10526d0*CON_KCALTOK
      tclocal(10) =      0.48055d0*CON_KCALTOK
      tclocal(11) =     -0.09946d0*CON_KCALTOK
      tclocal(12) =      1.29753d0*CON_KCALTOK
      tclocal(13) =      3.08020d0*CON_KCALTOK
      tclocal(14) =     -0.89525d0*CON_KCALTOK
      tclocal(15) =      4.90388d0*CON_KCALTOK
      tclocal(16) =      2.07030d0*CON_KCALTOK
      tclocal(17) =     -0.98435d0*CON_KCALTOK
      tclocal(18) =    -14.24908d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.77376d0*CON_KCALTOK
      tclocal(1) =      0.02646d0*CON_KCALTOK
      tclocal(2) =     -0.26321d0*CON_KCALTOK
      tclocal(3) =     -4.38910d0*CON_KCALTOK
      tclocal(4) =      3.36266d0*CON_KCALTOK
      tclocal(5) =      2.06685d0*CON_KCALTOK
      tclocal(6) =      3.44569d0*CON_KCALTOK
      tclocal(7) =     -1.07003d0*CON_KCALTOK
      tclocal(8) =      0.68253d0*CON_KCALTOK
      tclocal( 9) =     -0.37726d0*CON_KCALTOK
      tclocal(10) =      1.13543d0*CON_KCALTOK
      tclocal(11) =      0.91186d0*CON_KCALTOK
      tclocal(12) =     -6.61810d0*CON_KCALTOK
      tclocal(13) =      5.25793d0*CON_KCALTOK
      tclocal(14) =      2.41267d0*CON_KCALTOK
      tclocal(15) =      2.30478d0*CON_KCALTOK
      tclocal(16) =      0.10063d0*CON_KCALTOK
      tclocal(17) =     -0.12577d0*CON_KCALTOK
      tclocal(18) =    -13.69961d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.30194d0*CON_KCALTOK
      tclocal(1) =     -0.72960d0*CON_KCALTOK
      tclocal(2) =     -0.40256d0*CON_KCALTOK
      tclocal(3) =      0.46097d0*CON_KCALTOK
      tclocal(4) =      0.39288d0*CON_KCALTOK
      tclocal(5) =     -0.39517d0*CON_KCALTOK
      tclocal(6) =     -3.96451d0*CON_KCALTOK
      tclocal(7) =     -2.50930d0*CON_KCALTOK
      tclocal(8) =     -1.10378d0*CON_KCALTOK
      tclocal( 9) =     -0.00437d0*CON_KCALTOK
      tclocal(10) =      0.59281d0*CON_KCALTOK
      tclocal(11) =     -0.04174d0*CON_KCALTOK
      tclocal(12) =      0.23050d0*CON_KCALTOK
      tclocal(13) =      3.14284d0*CON_KCALTOK
      tclocal(14) =     -0.38696d0*CON_KCALTOK
      tclocal(15) =      1.74067d0*CON_KCALTOK
      tclocal(16) =      1.17027d0*CON_KCALTOK
      tclocal(17) =     -0.60093d0*CON_KCALTOK
      tclocal(18) =    -32.89749d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.02385d0*CON_KCALTOK
      tclocal(1) =     -0.13168d0*CON_KCALTOK
      tclocal(2) =     -0.10385d0*CON_KCALTOK
      tclocal(3) =      0.59860d0*CON_KCALTOK
      tclocal(4) =     -0.53951d0*CON_KCALTOK
      tclocal(5) =     -0.10701d0*CON_KCALTOK
      tclocal(6) =      0.42216d0*CON_KCALTOK
      tclocal(7) =     -1.12343d0*CON_KCALTOK
      tclocal(8) =     -0.72705d0*CON_KCALTOK
      tclocal( 9) =      0.30197d0*CON_KCALTOK
      tclocal(10) =      0.34958d0*CON_KCALTOK
      tclocal(11) =      0.08303d0*CON_KCALTOK
      tclocal(12) =     -0.96183d0*CON_KCALTOK
      tclocal(13) =      2.60571d0*CON_KCALTOK
      tclocal(14) =      0.70389d0*CON_KCALTOK
      tclocal(15) =      0.74747d0*CON_KCALTOK
      tclocal(16) =      0.64859d0*CON_KCALTOK
      tclocal(17) =      0.44829d0*CON_KCALTOK
      tclocal(18) =    -10.39007d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s  ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's  '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.75350d0*CON_KCALTOK
      tclocal(1) =     -0.80851d0*CON_KCALTOK
      tclocal(2) =     -0.24657d0*CON_KCALTOK
      tclocal(3) =      0.92904d0*CON_KCALTOK
      tclocal(4) =      0.48431d0*CON_KCALTOK
      tclocal(5) =      0.55489d0*CON_KCALTOK
      tclocal(6) =     -1.09803d0*CON_KCALTOK
      tclocal(7) =     -1.53028d0*CON_KCALTOK
      tclocal(8) =     -0.68572d0*CON_KCALTOK
      tclocal( 9) =     -0.35974d0*CON_KCALTOK
      tclocal(10) =      0.39087d0*CON_KCALTOK
      tclocal(11) =      0.30587d0*CON_KCALTOK
      tclocal(12) =     -1.40095d0*CON_KCALTOK
      tclocal(13) =      2.28170d0*CON_KCALTOK
      tclocal(14) =     -0.23531d0*CON_KCALTOK
      tclocal(15) =      2.49310d0*CON_KCALTOK
      tclocal(16) =      1.07264d0*CON_KCALTOK
      tclocal(17) =     -0.34649d0*CON_KCALTOK
      tclocal(18) =    -27.20957d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.49987d0*CON_KCALTOK
      tclocal(1) =     -0.89944d0*CON_KCALTOK
      tclocal(2) =     -0.43926d0*CON_KCALTOK
      tclocal(3) =     -0.10443d0*CON_KCALTOK
      tclocal(4) =     -0.39361d0*CON_KCALTOK
      tclocal(5) =     -0.39303d0*CON_KCALTOK
      tclocal(6) =     -2.51377d0*CON_KCALTOK
      tclocal(7) =     -5.98057d0*CON_KCALTOK
      tclocal(8) =     -2.81187d0*CON_KCALTOK
      tclocal( 9) =     -0.62771d0*CON_KCALTOK
      tclocal(10) =      0.04317d0*CON_KCALTOK
      tclocal(11) =      0.40031d0*CON_KCALTOK
      tclocal(12) =     -2.46748d0*CON_KCALTOK
      tclocal(13) =      2.47086d0*CON_KCALTOK
      tclocal(14) =     -0.32351d0*CON_KCALTOK
      tclocal(15) =      2.28606d0*CON_KCALTOK
      tclocal(16) =      0.10737d0*CON_KCALTOK
      tclocal(17) =     -1.48748d0*CON_KCALTOK
      tclocal(18) =    -30.26087d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s  ')
      call twh_torstring(GLB_SET,type,4,'s  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's  '
      tnlocal(nsame,4) = 's  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.01820d0*CON_KCALTOK
      tclocal(1) =     -0.51078d0*CON_KCALTOK
      tclocal(2) =     -0.25307d0*CON_KCALTOK
      tclocal(3) =     -0.37231d0*CON_KCALTOK
      tclocal(4) =     -0.04700d0*CON_KCALTOK
      tclocal(5) =     -0.47539d0*CON_KCALTOK
      tclocal(6) =     -5.38995d0*CON_KCALTOK
      tclocal(7) =     -1.50478d0*CON_KCALTOK
      tclocal(8) =     -0.79067d0*CON_KCALTOK
      tclocal( 9) =      0.22907d0*CON_KCALTOK
      tclocal(10) =      0.59167d0*CON_KCALTOK
      tclocal(11) =     -0.59855d0*CON_KCALTOK
      tclocal(12) =     -1.16797d0*CON_KCALTOK
      tclocal(13) =      2.44921d0*CON_KCALTOK
      tclocal(14) =      0.39460d0*CON_KCALTOK
      tclocal(15) =     -0.40708d0*CON_KCALTOK
      tclocal(16) =      0.88025d0*CON_KCALTOK
      tclocal(17) =     -0.43123d0*CON_KCALTOK
      tclocal(18) =    -31.83810d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.04107d0*CON_KCALTOK
      tclocal(1) =     -0.09994d0*CON_KCALTOK
      tclocal(2) =     -0.33972d0*CON_KCALTOK
      tclocal(3) =      0.45720d0*CON_KCALTOK
      tclocal(4) =     -0.01736d0*CON_KCALTOK
      tclocal(5) =     -0.91321d0*CON_KCALTOK
      tclocal(6) =     -0.91696d0*CON_KCALTOK
      tclocal(7) =      0.47749d0*CON_KCALTOK
      tclocal(8) =     -0.59516d0*CON_KCALTOK
      tclocal( 9) =      0.43162d0*CON_KCALTOK
      tclocal(10) =      1.01530d0*CON_KCALTOK
      tclocal(11) =      0.38178d0*CON_KCALTOK
      tclocal(12) =     -1.37984d0*CON_KCALTOK
      tclocal(13) =      2.72164d0*CON_KCALTOK
      tclocal(14) =     -0.15516d0*CON_KCALTOK
      tclocal(15) =     -0.38765d0*CON_KCALTOK
      tclocal(16) =      1.50683d0*CON_KCALTOK
      tclocal(17) =     -0.56592d0*CON_KCALTOK
      tclocal(18) =    -26.44844d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s  ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's  '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.80396d0*CON_KCALTOK
      tclocal(1) =     -0.65011d0*CON_KCALTOK
      tclocal(2) =     -0.15594d0*CON_KCALTOK
      tclocal(3) =      0.24623d0*CON_KCALTOK
      tclocal(4) =      0.32417d0*CON_KCALTOK
      tclocal(5) =     -1.65455d0*CON_KCALTOK
      tclocal(6) =     -2.87045d0*CON_KCALTOK
      tclocal(7) =     -0.36039d0*CON_KCALTOK
      tclocal(8) =     -0.24201d0*CON_KCALTOK
      tclocal( 9) =      0.17646d0*CON_KCALTOK
      tclocal(10) =      0.42515d0*CON_KCALTOK
      tclocal(11) =      0.21333d0*CON_KCALTOK
      tclocal(12) =     -1.43319d0*CON_KCALTOK
      tclocal(13) =      2.06611d0*CON_KCALTOK
      tclocal(14) =     -0.53618d0*CON_KCALTOK
      tclocal(15) =      0.81540d0*CON_KCALTOK
      tclocal(16) =      0.47083d0*CON_KCALTOK
      tclocal(17) =     -0.25131d0*CON_KCALTOK
      tclocal(18) =    -24.03928d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.19879d0*CON_KCALTOK
      tclocal(1) =     -0.49319d0*CON_KCALTOK
      tclocal(2) =     -0.32440d0*CON_KCALTOK
      tclocal(3) =     -0.09280d0*CON_KCALTOK
      tclocal(4) =     -0.65375d0*CON_KCALTOK
      tclocal(5) =     -0.17406d0*CON_KCALTOK
      tclocal(6) =     -2.71052d0*CON_KCALTOK
      tclocal(7) =     -4.69941d0*CON_KCALTOK
      tclocal(8) =     -0.43616d0*CON_KCALTOK
      tclocal( 9) =      0.38063d0*CON_KCALTOK
      tclocal(10) =      0.43541d0*CON_KCALTOK
      tclocal(11) =     -0.52671d0*CON_KCALTOK
      tclocal(12) =     -2.75343d0*CON_KCALTOK
      tclocal(13) =      1.96234d0*CON_KCALTOK
      tclocal(14) =     -0.61358d0*CON_KCALTOK
      tclocal(15) =      1.86519d0*CON_KCALTOK
      tclocal(16) =      0.39552d0*CON_KCALTOK
      tclocal(17) =     -0.74469d0*CON_KCALTOK
      tclocal(18) =    -23.63017d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s  ')
      call twh_torstring(GLB_SET,type,4,'s  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's  '
      tnlocal(nsame,4) = 's  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.02650d0*CON_KCALTOK
      tclocal(1) =     -1.11091d0*CON_KCALTOK
      tclocal(2) =     -0.64956d0*CON_KCALTOK
      tclocal(3) =      0.20349d0*CON_KCALTOK
      tclocal(4) =      1.31618d0*CON_KCALTOK
      tclocal(5) =      0.64818d0*CON_KCALTOK
      tclocal(6) =     -5.30858d0*CON_KCALTOK
      tclocal(7) =     -4.18905d0*CON_KCALTOK
      tclocal(8) =     -0.28649d0*CON_KCALTOK
      tclocal( 9) =      0.20349d0*CON_KCALTOK
      tclocal(10) =      1.31618d0*CON_KCALTOK
      tclocal(11) =      0.64818d0*CON_KCALTOK
      tclocal(12) =     -2.95971d0*CON_KCALTOK
      tclocal(13) =      2.40933d0*CON_KCALTOK
      tclocal(14) =      1.66541d0*CON_KCALTOK
      tclocal(15) =     -3.27439d0*CON_KCALTOK
      tclocal(16) =      1.93406d0*CON_KCALTOK
      tclocal(17) =      0.56947d0*CON_KCALTOK
      tclocal(18) =    -22.38958d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'s  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 's  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.10939d0*CON_KCALTOK
      tclocal(1) =     -1.24025d0*CON_KCALTOK
      tclocal(2) =     -0.20092d0*CON_KCALTOK
      tclocal(3) =      0.18260d0*CON_KCALTOK
      tclocal(4) =      0.47646d0*CON_KCALTOK
      tclocal(5) =     -0.44199d0*CON_KCALTOK
      tclocal(6) =     -0.98995d0*CON_KCALTOK
      tclocal(7) =     -3.21296d0*CON_KCALTOK
      tclocal(8) =     -0.99860d0*CON_KCALTOK
      tclocal( 9) =      0.32210d0*CON_KCALTOK
      tclocal(10) =     -0.03755d0*CON_KCALTOK
      tclocal(11) =      0.27906d0*CON_KCALTOK
      tclocal(12) =     -3.05184d0*CON_KCALTOK
      tclocal(13) =      0.87716d0*CON_KCALTOK
      tclocal(14) =      2.17079d0*CON_KCALTOK
      tclocal(15) =     -0.80784d0*CON_KCALTOK
      tclocal(16) =      0.40470d0*CON_KCALTOK
      tclocal(17) =     -0.24040d0*CON_KCALTOK
      tclocal(18) =    -21.63867d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'s  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 's  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.34822d0*CON_KCALTOK
      tclocal(1) =     -0.31905d0*CON_KCALTOK
      tclocal(2) =     -0.07068d0*CON_KCALTOK
      tclocal(3) =      0.00801d0*CON_KCALTOK
      tclocal(4) =      0.12120d0*CON_KCALTOK
      tclocal(5) =     -0.25884d0*CON_KCALTOK
      tclocal(6) =     -1.22734d0*CON_KCALTOK
      tclocal(7) =      0.01617d0*CON_KCALTOK
      tclocal(8) =      0.39524d0*CON_KCALTOK
      tclocal( 9) =     -0.01509d0*CON_KCALTOK
      tclocal(10) =      0.18516d0*CON_KCALTOK
      tclocal(11) =     -0.05644d0*CON_KCALTOK
      tclocal(12) =      5.17909d0*CON_KCALTOK
      tclocal(13) =     -1.12220d0*CON_KCALTOK
      tclocal(14) =      0.68508d0*CON_KCALTOK
      tclocal(15) =      4.27482d0*CON_KCALTOK
      tclocal(16) =      1.88904d0*CON_KCALTOK
      tclocal(17) =     -2.51485d0*CON_KCALTOK
      tclocal(18) =    -20.17661d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.05427d0*CON_KCALTOK
      tclocal(1) =      0.03626d0*CON_KCALTOK
      tclocal(2) =      0.06994d0*CON_KCALTOK
      tclocal(3) =     -0.50849d0*CON_KCALTOK
      tclocal(4) =      0.06287d0*CON_KCALTOK
      tclocal(5) =     -0.39759d0*CON_KCALTOK
      tclocal(6) =     -0.59170d0*CON_KCALTOK
      tclocal(7) =      0.22310d0*CON_KCALTOK
      tclocal(8) =      0.29389d0*CON_KCALTOK
      tclocal( 9) =     -0.09722d0*CON_KCALTOK
      tclocal(10) =      0.60171d0*CON_KCALTOK
      tclocal(11) =     -0.98790d0*CON_KCALTOK
      tclocal(12) =      4.14544d0*CON_KCALTOK
      tclocal(13) =     -0.39432d0*CON_KCALTOK
      tclocal(14) =      0.18415d0*CON_KCALTOK
      tclocal(15) =      0.76251d0*CON_KCALTOK
      tclocal(16) =      0.48518d0*CON_KCALTOK
      tclocal(17) =      1.63980d0*CON_KCALTOK
      tclocal(18) =     -4.58236d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.08074d0*CON_KCALTOK
      tclocal(1) =      0.00745d0*CON_KCALTOK
      tclocal(2) =      0.08792d0*CON_KCALTOK
      tclocal(3) =      0.55096d0*CON_KCALTOK
      tclocal(4) =      0.02684d0*CON_KCALTOK
      tclocal(5) =      0.28906d0*CON_KCALTOK
      tclocal(6) =     -0.68152d0*CON_KCALTOK
      tclocal(7) =      0.10297d0*CON_KCALTOK
      tclocal(8) =     -1.66024d0*CON_KCALTOK
      tclocal( 9) =     -0.17229d0*CON_KCALTOK
      tclocal(10) =      0.10036d0*CON_KCALTOK
      tclocal(11) =      0.37168d0*CON_KCALTOK
      tclocal(12) =      4.79794d0*CON_KCALTOK
      tclocal(13) =     -0.73057d0*CON_KCALTOK
      tclocal(14) =      1.01040d0*CON_KCALTOK
      tclocal(15) =      3.37028d0*CON_KCALTOK
      tclocal(16) =      1.35371d0*CON_KCALTOK
      tclocal(17) =      0.02045d0*CON_KCALTOK
      tclocal(18) =    -21.81917d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.30425d0*CON_KCALTOK
      tclocal(1) =      0.14913d0*CON_KCALTOK
      tclocal(2) =     -0.20360d0*CON_KCALTOK
      tclocal(3) =     -0.22288d0*CON_KCALTOK
      tclocal(4) =      0.20119d0*CON_KCALTOK
      tclocal(5) =     -0.30480d0*CON_KCALTOK
      tclocal(6) =     -0.17419d0*CON_KCALTOK
      tclocal(7) =      0.30603d0*CON_KCALTOK
      tclocal(8) =     -0.17493d0*CON_KCALTOK
      tclocal( 9) =     -0.21697d0*CON_KCALTOK
      tclocal(10) =      0.39476d0*CON_KCALTOK
      tclocal(11) =     -0.05649d0*CON_KCALTOK
      tclocal(12) =      5.96997d0*CON_KCALTOK
      tclocal(13) =     -2.64900d0*CON_KCALTOK
      tclocal(14) =      1.12708d0*CON_KCALTOK
      tclocal(15) =      0.86705d0*CON_KCALTOK
      tclocal(16) =      0.12068d0*CON_KCALTOK
      tclocal(17) =      0.30195d0*CON_KCALTOK
      tclocal(18) =     -0.87799d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.89653d0*CON_KCALTOK
      tclocal(1) =      1.00846d0*CON_KCALTOK
      tclocal(2) =     -0.17039d0*CON_KCALTOK
      tclocal(3) =     -0.41228d0*CON_KCALTOK
      tclocal(4) =      0.82047d0*CON_KCALTOK
      tclocal(5) =     -0.18090d0*CON_KCALTOK
      tclocal(6) =     -0.43332d0*CON_KCALTOK
      tclocal(7) =      1.44504d0*CON_KCALTOK
      tclocal(8) =     -1.55111d0*CON_KCALTOK
      tclocal( 9) =      0.72425d0*CON_KCALTOK
      tclocal(10) =      0.49114d0*CON_KCALTOK
      tclocal(11) =     -0.98615d0*CON_KCALTOK
      tclocal(12) =     11.09428d0*CON_KCALTOK
      tclocal(13) =      0.14749d0*CON_KCALTOK
      tclocal(14) =      0.02439d0*CON_KCALTOK
      tclocal(15) =      5.94859d0*CON_KCALTOK
      tclocal(16) =      3.89207d0*CON_KCALTOK
      tclocal(17) =     -0.85129d0*CON_KCALTOK
      tclocal(18) =    -35.21984d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.01348d0*CON_KCALTOK
      tclocal(1) =     -0.08823d0*CON_KCALTOK
      tclocal(2) =      0.01489d0*CON_KCALTOK
      tclocal(3) =      0.35636d0*CON_KCALTOK
      tclocal(4) =      0.38172d0*CON_KCALTOK
      tclocal(5) =      0.16362d0*CON_KCALTOK
      tclocal(6) =      0.91600d0*CON_KCALTOK
      tclocal(7) =     -0.99486d0*CON_KCALTOK
      tclocal(8) =     -0.68628d0*CON_KCALTOK
      tclocal( 9) =      0.01434d0*CON_KCALTOK
      tclocal(10) =      0.22657d0*CON_KCALTOK
      tclocal(11) =      0.05073d0*CON_KCALTOK
      tclocal(12) =      6.13432d0*CON_KCALTOK
      tclocal(13) =      0.40692d0*CON_KCALTOK
      tclocal(14) =      0.37000d0*CON_KCALTOK
      tclocal(15) =     -0.99498d0*CON_KCALTOK
      tclocal(16) =      1.41122d0*CON_KCALTOK
      tclocal(17) =      0.65074d0*CON_KCALTOK
      tclocal(18) =    -25.81745d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.15010d0*CON_KCALTOK
      tclocal(1) =      0.14860d0*CON_KCALTOK
      tclocal(2) =     -0.16419d0*CON_KCALTOK
      tclocal(3) =      0.35464d0*CON_KCALTOK
      tclocal(4) =      0.80320d0*CON_KCALTOK
      tclocal(5) =     -0.22360d0*CON_KCALTOK
      tclocal(6) =      0.67812d0*CON_KCALTOK
      tclocal(7) =     -0.04139d0*CON_KCALTOK
      tclocal(8) =     -1.55985d0*CON_KCALTOK
      tclocal( 9) =      0.42993d0*CON_KCALTOK
      tclocal(10) =      0.33730d0*CON_KCALTOK
      tclocal(11) =      1.24364d0*CON_KCALTOK
      tclocal(12) =      5.96314d0*CON_KCALTOK
      tclocal(13) =      0.91443d0*CON_KCALTOK
      tclocal(14) =     -0.20993d0*CON_KCALTOK
      tclocal(15) =     -1.05440d0*CON_KCALTOK
      tclocal(16) =      1.02933d0*CON_KCALTOK
      tclocal(17) =     -1.78405d0*CON_KCALTOK
      tclocal(18) =    -20.10822d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.06220d0*CON_KCALTOK
      tclocal(1) =     -0.07290d0*CON_KCALTOK
      tclocal(2) =      0.01722d0*CON_KCALTOK
      tclocal(3) =     -0.06310d0*CON_KCALTOK
      tclocal(4) =      0.43837d0*CON_KCALTOK
      tclocal(5) =     -0.20970d0*CON_KCALTOK
      tclocal(6) =      0.56491d0*CON_KCALTOK
      tclocal(7) =     -0.13988d0*CON_KCALTOK
      tclocal(8) =      0.02188d0*CON_KCALTOK
      tclocal( 9) =      0.18311d0*CON_KCALTOK
      tclocal(10) =     -0.07137d0*CON_KCALTOK
      tclocal(11) =      0.14801d0*CON_KCALTOK
      tclocal(12) =      6.16653d0*CON_KCALTOK
      tclocal(13) =      0.39128d0*CON_KCALTOK
      tclocal(14) =     -0.17767d0*CON_KCALTOK
      tclocal(15) =     -1.84604d0*CON_KCALTOK
      tclocal(16) =      1.30072d0*CON_KCALTOK
      tclocal(17) =     -0.60048d0*CON_KCALTOK
      tclocal(18) =    -18.30929d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.33448d0*CON_KCALTOK
      tclocal(1) =      0.00074d0*CON_KCALTOK
      tclocal(2) =     -0.12718d0*CON_KCALTOK
      tclocal(3) =      0.67594d0*CON_KCALTOK
      tclocal(4) =      0.80179d0*CON_KCALTOK
      tclocal(5) =     -0.12479d0*CON_KCALTOK
      tclocal(6) =      0.26627d0*CON_KCALTOK
      tclocal(7) =     -0.21126d0*CON_KCALTOK
      tclocal(8) =     -0.45790d0*CON_KCALTOK
      tclocal( 9) =      0.36183d0*CON_KCALTOK
      tclocal(10) =      0.16642d0*CON_KCALTOK
      tclocal(11) =      0.59244d0*CON_KCALTOK
      tclocal(12) =      6.21037d0*CON_KCALTOK
      tclocal(13) =     -0.02112d0*CON_KCALTOK
      tclocal(14) =      0.45385d0*CON_KCALTOK
      tclocal(15) =     -0.13382d0*CON_KCALTOK
      tclocal(16) =      1.72763d0*CON_KCALTOK
      tclocal(17) =      0.15080d0*CON_KCALTOK
      tclocal(18) =    -11.85210d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.77196d0*CON_KCALTOK
      tclocal(1) =      0.63607d0*CON_KCALTOK
      tclocal(2) =     -0.17454d0*CON_KCALTOK
      tclocal(3) =      1.90344d0*CON_KCALTOK
      tclocal(4) =      0.91386d0*CON_KCALTOK
      tclocal(5) =      0.27322d0*CON_KCALTOK
      tclocal(6) =      0.61833d0*CON_KCALTOK
      tclocal(7) =      0.78540d0*CON_KCALTOK
      tclocal(8) =      0.08675d0*CON_KCALTOK
      tclocal( 9) =     -0.77980d0*CON_KCALTOK
      tclocal(10) =     -0.20551d0*CON_KCALTOK
      tclocal(11) =      0.73750d0*CON_KCALTOK
      tclocal(12) =     10.35153d0*CON_KCALTOK
      tclocal(13) =      0.79035d0*CON_KCALTOK
      tclocal(14) =     -0.11513d0*CON_KCALTOK
      tclocal(15) =     -3.63486d0*CON_KCALTOK
      tclocal(16) =      2.62161d0*CON_KCALTOK
      tclocal(17) =      0.25089d0*CON_KCALTOK
      tclocal(18) =    -27.38137d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.87916d0*CON_KCALTOK
      tclocal(1) =      1.74413d0*CON_KCALTOK
      tclocal(2) =     -0.06174d0*CON_KCALTOK
      tclocal(3) =     -1.51938d0*CON_KCALTOK
      tclocal(4) =     -0.67054d0*CON_KCALTOK
      tclocal(5) =     -0.17550d0*CON_KCALTOK
      tclocal(6) =      0.06668d0*CON_KCALTOK
      tclocal(7) =      2.77462d0*CON_KCALTOK
      tclocal(8) =     -0.03908d0*CON_KCALTOK
      tclocal( 9) =      0.48694d0*CON_KCALTOK
      tclocal(10) =     -1.14841d0*CON_KCALTOK
      tclocal(11) =      0.28408d0*CON_KCALTOK
      tclocal(12) =      5.51947d0*CON_KCALTOK
      tclocal(13) =      2.49383d0*CON_KCALTOK
      tclocal(14) =     -0.63584d0*CON_KCALTOK
      tclocal(15) =      9.33383d0*CON_KCALTOK
      tclocal(16) =      0.85861d0*CON_KCALTOK
      tclocal(17) =      0.22444d0*CON_KCALTOK
      tclocal(18) =    -23.42518d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.96991d0*CON_KCALTOK
      tclocal(1) =      2.01529d0*CON_KCALTOK
      tclocal(2) =     -0.36612d0*CON_KCALTOK
      tclocal(3) =     -1.64859d0*CON_KCALTOK
      tclocal(4) =     -0.13987d0*CON_KCALTOK
      tclocal(5) =     -0.46895d0*CON_KCALTOK
      tclocal(6) =     -0.52120d0*CON_KCALTOK
      tclocal(7) =      4.97922d0*CON_KCALTOK
      tclocal(8) =     -0.91668d0*CON_KCALTOK
      tclocal( 9) =     -0.72509d0*CON_KCALTOK
      tclocal(10) =     -0.47531d0*CON_KCALTOK
      tclocal(11) =     -0.61454d0*CON_KCALTOK
      tclocal(12) =      3.38498d0*CON_KCALTOK
      tclocal(13) =      2.91298d0*CON_KCALTOK
      tclocal(14) =     -2.81753d0*CON_KCALTOK
      tclocal(15) =      5.85635d0*CON_KCALTOK
      tclocal(16) =     -0.58678d0*CON_KCALTOK
      tclocal(17) =     -0.64235d0*CON_KCALTOK
      tclocal(18) =    -21.25132d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.26201d0*CON_KCALTOK
      tclocal(1) =      0.85187d0*CON_KCALTOK
      tclocal(2) =      0.16037d0*CON_KCALTOK
      tclocal(3) =      3.87988d0*CON_KCALTOK
      tclocal(4) =      0.41593d0*CON_KCALTOK
      tclocal(5) =      0.97208d0*CON_KCALTOK
      tclocal(6) =      1.16360d0*CON_KCALTOK
      tclocal(7) =      2.17119d0*CON_KCALTOK
      tclocal(8) =      0.84623d0*CON_KCALTOK
      tclocal( 9) =      1.89512d0*CON_KCALTOK
      tclocal(10) =     -0.52456d0*CON_KCALTOK
      tclocal(11) =      0.13585d0*CON_KCALTOK
      tclocal(12) =      7.91141d0*CON_KCALTOK
      tclocal(13) =      3.48970d0*CON_KCALTOK
      tclocal(14) =      0.72187d0*CON_KCALTOK
      tclocal(15) =     10.53772d0*CON_KCALTOK
      tclocal(16) =      0.37787d0*CON_KCALTOK
      tclocal(17) =      0.81285d0*CON_KCALTOK
      tclocal(18) =    -27.05356d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.56463d0*CON_KCALTOK
      tclocal(1) =      1.72998d0*CON_KCALTOK
      tclocal(2) =     -0.34136d0*CON_KCALTOK
      tclocal(3) =     -0.49253d0*CON_KCALTOK
      tclocal(4) =     -1.62775d0*CON_KCALTOK
      tclocal(5) =      0.16825d0*CON_KCALTOK
      tclocal(6) =     -0.12868d0*CON_KCALTOK
      tclocal(7) =      1.72954d0*CON_KCALTOK
      tclocal(8) =     -0.42024d0*CON_KCALTOK
      tclocal( 9) =     -0.76778d0*CON_KCALTOK
      tclocal(10) =     -1.39245d0*CON_KCALTOK
      tclocal(11) =      0.06059d0*CON_KCALTOK
      tclocal(12) =      3.87594d0*CON_KCALTOK
      tclocal(13) =      0.62702d0*CON_KCALTOK
      tclocal(14) =     -0.64672d0*CON_KCALTOK
      tclocal(15) =     14.36879d0*CON_KCALTOK
      tclocal(16) =      0.38708d0*CON_KCALTOK
      tclocal(17) =     -0.35524d0*CON_KCALTOK
      tclocal(18) =    -21.61833d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.00206d0*CON_KCALTOK
      tclocal(1) =      1.64629d0*CON_KCALTOK
      tclocal(2) =     -0.45392d0*CON_KCALTOK
      tclocal(3) =     -0.02826d0*CON_KCALTOK
      tclocal(4) =     -1.54169d0*CON_KCALTOK
      tclocal(5) =      0.63260d0*CON_KCALTOK
      tclocal(6) =      0.51639d0*CON_KCALTOK
      tclocal(7) =      4.68332d0*CON_KCALTOK
      tclocal(8) =      0.23593d0*CON_KCALTOK
      tclocal( 9) =      1.88169d0*CON_KCALTOK
      tclocal(10) =     -0.89955d0*CON_KCALTOK
      tclocal(11) =      0.16890d0*CON_KCALTOK
      tclocal(12) =      6.87051d0*CON_KCALTOK
      tclocal(13) =      1.42455d0*CON_KCALTOK
      tclocal(14) =     -0.54153d0*CON_KCALTOK
      tclocal(15) =     11.67507d0*CON_KCALTOK
      tclocal(16) =     -1.06911d0*CON_KCALTOK
      tclocal(17) =     -0.85421d0*CON_KCALTOK
      tclocal(18) =    -24.25878d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.65430d0*CON_KCALTOK
      tclocal(1) =      0.77505d0*CON_KCALTOK
      tclocal(2) =     -0.66843d0*CON_KCALTOK
      tclocal(3) =      1.67278d0*CON_KCALTOK
      tclocal(4) =      0.04900d0*CON_KCALTOK
      tclocal(5) =     -0.15565d0*CON_KCALTOK
      tclocal(6) =     -1.05760d0*CON_KCALTOK
      tclocal(7) =      1.03466d0*CON_KCALTOK
      tclocal(8) =     -0.20430d0*CON_KCALTOK
      tclocal( 9) =      1.07650d0*CON_KCALTOK
      tclocal(10) =     -0.94465d0*CON_KCALTOK
      tclocal(11) =     -0.11533d0*CON_KCALTOK
      tclocal(12) =      0.40333d0*CON_KCALTOK
      tclocal(13) =      1.30824d0*CON_KCALTOK
      tclocal(14) =     -0.93288d0*CON_KCALTOK
      tclocal(15) =     12.37492d0*CON_KCALTOK
      tclocal(16) =     -0.97905d0*CON_KCALTOK
      tclocal(17) =     -0.48310d0*CON_KCALTOK
      tclocal(18) =    -15.03646d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.75325d0*CON_KCALTOK
      tclocal(1) =      2.73925d0*CON_KCALTOK
      tclocal(2) =      0.09065d0*CON_KCALTOK
      tclocal(3) =      0.13524d0*CON_KCALTOK
      tclocal(4) =     -0.67120d0*CON_KCALTOK
      tclocal(5) =      0.41782d0*CON_KCALTOK
      tclocal(6) =    -13.40756d0*CON_KCALTOK
      tclocal(7) =      4.94097d0*CON_KCALTOK
      tclocal(8) =     -4.11084d0*CON_KCALTOK
      tclocal( 9) =      0.33317d0*CON_KCALTOK
      tclocal(10) =     -0.16533d0*CON_KCALTOK
      tclocal(11) =     -0.20640d0*CON_KCALTOK
      tclocal(12) =      6.10617d0*CON_KCALTOK
      tclocal(13) =      4.24671d0*CON_KCALTOK
      tclocal(14) =      4.76855d0*CON_KCALTOK
      tclocal(15) =      8.57398d0*CON_KCALTOK
      tclocal(16) =      2.58776d0*CON_KCALTOK
      tclocal(17) =      0.58724d0*CON_KCALTOK
      tclocal(18) =     -9.46881d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.91513d0*CON_KCALTOK
      tclocal(1) =      2.38519d0*CON_KCALTOK
      tclocal(2) =     -0.23804d0*CON_KCALTOK
      tclocal(3) =     -1.01141d0*CON_KCALTOK
      tclocal(4) =      0.81434d0*CON_KCALTOK
      tclocal(5) =      0.61044d0*CON_KCALTOK
      tclocal(6) =     -0.76789d0*CON_KCALTOK
      tclocal(7) =      6.86316d0*CON_KCALTOK
      tclocal(8) =     -1.54157d0*CON_KCALTOK
      tclocal( 9) =      0.92937d0*CON_KCALTOK
      tclocal(10) =      0.24286d0*CON_KCALTOK
      tclocal(11) =      0.20874d0*CON_KCALTOK
      tclocal(12) =     -3.20779d0*CON_KCALTOK
      tclocal(13) =      1.87084d0*CON_KCALTOK
      tclocal(14) =      1.40957d0*CON_KCALTOK
      tclocal(15) =      4.95017d0*CON_KCALTOK
      tclocal(16) =      0.53964d0*CON_KCALTOK
      tclocal(17) =      0.14294d0*CON_KCALTOK
      tclocal(18) =     -1.91790d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.33446d0*CON_KCALTOK
      tclocal(1) =      2.58385d0*CON_KCALTOK
      tclocal(2) =     -0.40063d0*CON_KCALTOK
      tclocal(3) =     -0.85881d0*CON_KCALTOK
      tclocal(4) =     -0.00651d0*CON_KCALTOK
      tclocal(5) =      0.21539d0*CON_KCALTOK
      tclocal(6) =    -16.99316d0*CON_KCALTOK
      tclocal(7) =      4.64500d0*CON_KCALTOK
      tclocal(8) =     -4.36512d0*CON_KCALTOK
      tclocal( 9) =     -0.77266d0*CON_KCALTOK
      tclocal(10) =      0.83017d0*CON_KCALTOK
      tclocal(11) =     -0.09357d0*CON_KCALTOK
      tclocal(12) =      0.19691d0*CON_KCALTOK
      tclocal(13) =      4.53929d0*CON_KCALTOK
      tclocal(14) =      0.31043d0*CON_KCALTOK
      tclocal(15) =      8.96046d0*CON_KCALTOK
      tclocal(16) =     -0.51138d0*CON_KCALTOK
      tclocal(17) =     -0.31138d0*CON_KCALTOK
      tclocal(18) =     -8.10579d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.21185d0*CON_KCALTOK
      tclocal(1) =      1.24574d0*CON_KCALTOK
      tclocal(2) =      0.04730d0*CON_KCALTOK
      tclocal(3) =      0.16566d0*CON_KCALTOK
      tclocal(4) =      0.21372d0*CON_KCALTOK
      tclocal(5) =      0.22294d0*CON_KCALTOK
      tclocal(6) =     -0.69933d0*CON_KCALTOK
      tclocal(7) =     -0.08566d0*CON_KCALTOK
      tclocal(8) =     -1.19745d0*CON_KCALTOK
      tclocal( 9) =     -0.30555d0*CON_KCALTOK
      tclocal(10) =     -0.60877d0*CON_KCALTOK
      tclocal(11) =     -0.32960d0*CON_KCALTOK
      tclocal(12) =     -3.73747d0*CON_KCALTOK
      tclocal(13) =      3.60254d0*CON_KCALTOK
      tclocal(14) =      1.45715d0*CON_KCALTOK
      tclocal(15) =      2.35553d0*CON_KCALTOK
      tclocal(16) =     -1.83240d0*CON_KCALTOK
      tclocal(17) =      0.85878d0*CON_KCALTOK
      tclocal(18) =     -1.08911d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.00779d0*CON_KCALTOK
      tclocal(1) =      2.61863d0*CON_KCALTOK
      tclocal(2) =     -0.09002d0*CON_KCALTOK
      tclocal(3) =     -0.37232d0*CON_KCALTOK
      tclocal(4) =      0.69344d0*CON_KCALTOK
      tclocal(5) =      0.51662d0*CON_KCALTOK
      tclocal(6) =     -1.39937d0*CON_KCALTOK
      tclocal(7) =      7.54167d0*CON_KCALTOK
      tclocal(8) =     -3.34587d0*CON_KCALTOK
      tclocal( 9) =      0.14263d0*CON_KCALTOK
      tclocal(10) =      0.80829d0*CON_KCALTOK
      tclocal(11) =      0.16983d0*CON_KCALTOK
      tclocal(12) =     -3.16300d0*CON_KCALTOK
      tclocal(13) =      2.96277d0*CON_KCALTOK
      tclocal(14) =     -0.21108d0*CON_KCALTOK
      tclocal(15) =      5.69392d0*CON_KCALTOK
      tclocal(16) =      1.46305d0*CON_KCALTOK
      tclocal(17) =     -0.04605d0*CON_KCALTOK
      tclocal(18) =     -7.75558d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.81758d0*CON_KCALTOK
      tclocal(1) =      0.51587d0*CON_KCALTOK
      tclocal(2) =     -1.23315d0*CON_KCALTOK
      tclocal(3) =      0.11151d0*CON_KCALTOK
      tclocal(4) =     -0.76722d0*CON_KCALTOK
      tclocal(5) =     -0.00606d0*CON_KCALTOK
      tclocal(6) =     -1.80333d0*CON_KCALTOK
      tclocal(7) =     -6.42941d0*CON_KCALTOK
      tclocal(8) =      3.20135d0*CON_KCALTOK
      tclocal( 9) =     -0.70728d0*CON_KCALTOK
      tclocal(10) =     -1.03289d0*CON_KCALTOK
      tclocal(11) =      0.19115d0*CON_KCALTOK
      tclocal(12) =     -3.87655d0*CON_KCALTOK
      tclocal(13) =      2.28400d0*CON_KCALTOK
      tclocal(14) =      0.10582d0*CON_KCALTOK
      tclocal(15) =      1.17755d0*CON_KCALTOK
      tclocal(16) =     -1.18274d0*CON_KCALTOK
      tclocal(17) =      0.89941d0*CON_KCALTOK
      tclocal(18) =     -2.19700d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.82969d0*CON_KCALTOK
      tclocal(1) =      3.72337d0*CON_KCALTOK
      tclocal(2) =     -0.04952d0*CON_KCALTOK
      tclocal(3) =      0.17770d0*CON_KCALTOK
      tclocal(4) =     -3.09080d0*CON_KCALTOK
      tclocal(5) =      0.80881d0*CON_KCALTOK
      tclocal(6) =    -12.79725d0*CON_KCALTOK
      tclocal(7) =     20.73606d0*CON_KCALTOK
      tclocal(8) =     -2.56758d0*CON_KCALTOK
      tclocal( 9) =      0.23153d0*CON_KCALTOK
      tclocal(10) =      1.05111d0*CON_KCALTOK
      tclocal(11) =     -0.14587d0*CON_KCALTOK
      tclocal(12) =      6.44432d0*CON_KCALTOK
      tclocal(13) =      5.84303d0*CON_KCALTOK
      tclocal(14) =      2.48243d0*CON_KCALTOK
      tclocal(15) =     10.78659d0*CON_KCALTOK
      tclocal(16) =      3.11669d0*CON_KCALTOK
      tclocal(17) =     -0.31975d0*CON_KCALTOK
      tclocal(18) =    -22.54298d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.45173d0*CON_KCALTOK
      tclocal(1) =      1.39033d0*CON_KCALTOK
      tclocal(2) =     -0.83411d0*CON_KCALTOK
      tclocal(3) =     -1.01729d0*CON_KCALTOK
      tclocal(4) =      1.20369d0*CON_KCALTOK
      tclocal(5) =     -0.99628d0*CON_KCALTOK
      tclocal(6) =     -0.16199d0*CON_KCALTOK
      tclocal(7) =     -1.73765d0*CON_KCALTOK
      tclocal(8) =      0.98318d0*CON_KCALTOK
      tclocal( 9) =      0.25014d0*CON_KCALTOK
      tclocal(10) =     -0.69893d0*CON_KCALTOK
      tclocal(11) =      0.38638d0*CON_KCALTOK
      tclocal(12) =     -2.28215d0*CON_KCALTOK
      tclocal(13) =      3.47788d0*CON_KCALTOK
      tclocal(14) =     -0.41319d0*CON_KCALTOK
      tclocal(15) =     -0.44023d0*CON_KCALTOK
      tclocal(16) =     -0.07942d0*CON_KCALTOK
      tclocal(17) =     -0.46203d0*CON_KCALTOK
      tclocal(18) =     -4.86312d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.88202d0*CON_KCALTOK
      tclocal(1) =      3.04293d0*CON_KCALTOK
      tclocal(2) =     -0.37336d0*CON_KCALTOK
      tclocal(3) =     -1.10203d0*CON_KCALTOK
      tclocal(4) =     -3.83059d0*CON_KCALTOK
      tclocal(5) =      1.80678d0*CON_KCALTOK
      tclocal(6) =     -1.31648d0*CON_KCALTOK
      tclocal(7) =      8.90536d0*CON_KCALTOK
      tclocal(8) =     -0.70311d0*CON_KCALTOK
      tclocal( 9) =      0.17588d0*CON_KCALTOK
      tclocal(10) =      0.28060d0*CON_KCALTOK
      tclocal(11) =      0.11833d0*CON_KCALTOK
      tclocal(12) =     -3.80264d0*CON_KCALTOK
      tclocal(13) =      0.52258d0*CON_KCALTOK
      tclocal(14) =      0.79338d0*CON_KCALTOK
      tclocal(15) =      6.11478d0*CON_KCALTOK
      tclocal(16) =      1.80259d0*CON_KCALTOK
      tclocal(17) =      0.35885d0*CON_KCALTOK
      tclocal(18) =    -10.60657d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.97609d0*CON_KCALTOK
      tclocal(1) =      2.55962d0*CON_KCALTOK
      tclocal(2) =      0.03777d0*CON_KCALTOK
      tclocal(3) =      0.22423d0*CON_KCALTOK
      tclocal(4) =      1.53333d0*CON_KCALTOK
      tclocal(5) =      0.99991d0*CON_KCALTOK
      tclocal(6) =      1.56337d0*CON_KCALTOK
      tclocal(7) =      4.12968d0*CON_KCALTOK
      tclocal(8) =     -5.72111d0*CON_KCALTOK
      tclocal( 9) =      0.00047d0*CON_KCALTOK
      tclocal(10) =     -1.27613d0*CON_KCALTOK
      tclocal(11) =      0.56178d0*CON_KCALTOK
      tclocal(12) =      1.12800d0*CON_KCALTOK
      tclocal(13) =     -2.92665d0*CON_KCALTOK
      tclocal(14) =      1.99942d0*CON_KCALTOK
      tclocal(15) =      1.02688d0*CON_KCALTOK
      tclocal(16) =     -0.95381d0*CON_KCALTOK
      tclocal(17) =      0.27970d0*CON_KCALTOK
      tclocal(18) =    -14.19418d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.38673d0*CON_KCALTOK
      tclocal(1) =      1.18595d0*CON_KCALTOK
      tclocal(2) =     -0.24157d0*CON_KCALTOK
      tclocal(3) =     -2.41548d0*CON_KCALTOK
      tclocal(4) =      0.19170d0*CON_KCALTOK
      tclocal(5) =      0.44559d0*CON_KCALTOK
      tclocal(6) =      2.49447d0*CON_KCALTOK
      tclocal(7) =      0.63746d0*CON_KCALTOK
      tclocal(8) =     -2.75500d0*CON_KCALTOK
      tclocal( 9) =     -0.47116d0*CON_KCALTOK
      tclocal(10) =     -3.56415d0*CON_KCALTOK
      tclocal(11) =      1.57692d0*CON_KCALTOK
      tclocal(12) =     -1.77027d0*CON_KCALTOK
      tclocal(13) =     -1.50365d0*CON_KCALTOK
      tclocal(14) =     -0.16020d0*CON_KCALTOK
      tclocal(15) =      2.56325d0*CON_KCALTOK
      tclocal(16) =      0.28870d0*CON_KCALTOK
      tclocal(17) =     -2.30199d0*CON_KCALTOK
      tclocal(18) =    -12.28636d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -4.18481d0*CON_KCALTOK
      tclocal(1) =      1.28289d0*CON_KCALTOK
      tclocal(2) =     -0.79940d0*CON_KCALTOK
      tclocal(3) =     -2.16188d0*CON_KCALTOK
      tclocal(4) =      1.47188d0*CON_KCALTOK
      tclocal(5) =     -0.79042d0*CON_KCALTOK
      tclocal(6) =     -3.25036d0*CON_KCALTOK
      tclocal(7) =      4.38907d0*CON_KCALTOK
      tclocal(8) =     -2.93841d0*CON_KCALTOK
      tclocal( 9) =      2.54574d0*CON_KCALTOK
      tclocal(10) =     -0.38734d0*CON_KCALTOK
      tclocal(11) =      0.74079d0*CON_KCALTOK
      tclocal(12) =     -4.74571d0*CON_KCALTOK
      tclocal(13) =      4.17561d0*CON_KCALTOK
      tclocal(14) =     -3.77563d0*CON_KCALTOK
      tclocal(15) =     -4.25464d0*CON_KCALTOK
      tclocal(16) =     -0.38808d0*CON_KCALTOK
      tclocal(17) =      0.09563d0*CON_KCALTOK
      tclocal(18) =     -9.11344d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.64539d0*CON_KCALTOK
      tclocal(1) =      2.57676d0*CON_KCALTOK
      tclocal(2) =     -0.19418d0*CON_KCALTOK
      tclocal(3) =     -3.66184d0*CON_KCALTOK
      tclocal(4) =      2.12543d0*CON_KCALTOK
      tclocal(5) =     -1.02284d0*CON_KCALTOK
      tclocal(6) =      3.34264d0*CON_KCALTOK
      tclocal(7) =      5.66960d0*CON_KCALTOK
      tclocal(8) =     -3.92782d0*CON_KCALTOK
      tclocal( 9) =      2.69416d0*CON_KCALTOK
      tclocal(10) =     -4.49630d0*CON_KCALTOK
      tclocal(11) =      0.32015d0*CON_KCALTOK
      tclocal(12) =     -7.07050d0*CON_KCALTOK
      tclocal(13) =     12.31017d0*CON_KCALTOK
      tclocal(14) =     -1.02771d0*CON_KCALTOK
      tclocal(15) =     -7.44810d0*CON_KCALTOK
      tclocal(16) =     -1.52969d0*CON_KCALTOK
      tclocal(17) =      1.70913d0*CON_KCALTOK
      tclocal(18) =    -13.39251d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.16195d0*CON_KCALTOK
      tclocal(1) =      0.24783d0*CON_KCALTOK
      tclocal(2) =     -0.49702d0*CON_KCALTOK
      tclocal(3) =     -1.95047d0*CON_KCALTOK
      tclocal(4) =      2.04096d0*CON_KCALTOK
      tclocal(5) =      0.96738d0*CON_KCALTOK
      tclocal(6) =      0.19613d0*CON_KCALTOK
      tclocal(7) =      4.07727d0*CON_KCALTOK
      tclocal(8) =     -2.40950d0*CON_KCALTOK
      tclocal( 9) =      1.76432d0*CON_KCALTOK
      tclocal(10) =     -1.77816d0*CON_KCALTOK
      tclocal(11) =      1.65467d0*CON_KCALTOK
      tclocal(12) =      4.06444d0*CON_KCALTOK
      tclocal(13) =     -3.70978d0*CON_KCALTOK
      tclocal(14) =      0.56975d0*CON_KCALTOK
      tclocal(15) =     -0.29441d0*CON_KCALTOK
      tclocal(16) =      0.54598d0*CON_KCALTOK
      tclocal(17) =      0.95192d0*CON_KCALTOK
      tclocal(18) =    -11.14654d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.16195d0*CON_KCALTOK
      tclocal(1) =      0.24783d0*CON_KCALTOK
      tclocal(2) =     -0.49702d0*CON_KCALTOK
      tclocal(3) =     -1.95047d0*CON_KCALTOK
      tclocal(4) =      2.04096d0*CON_KCALTOK
      tclocal(5) =      0.96738d0*CON_KCALTOK
      tclocal(6) =      0.19613d0*CON_KCALTOK
      tclocal(7) =      4.07727d0*CON_KCALTOK
      tclocal(8) =     -2.40950d0*CON_KCALTOK
      tclocal( 9) =      1.76432d0*CON_KCALTOK
      tclocal(10) =     -1.77816d0*CON_KCALTOK
      tclocal(11) =      1.65467d0*CON_KCALTOK
      tclocal(12) =      4.06444d0*CON_KCALTOK
      tclocal(13) =     -3.70978d0*CON_KCALTOK
      tclocal(14) =      0.56975d0*CON_KCALTOK
      tclocal(15) =     -0.29441d0*CON_KCALTOK
      tclocal(16) =      0.54598d0*CON_KCALTOK
      tclocal(17) =      0.95192d0*CON_KCALTOK
      tclocal(18) =    -11.14654d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.19072d0*CON_KCALTOK
      tclocal(1) =      0.97522d0*CON_KCALTOK
      tclocal(2) =     -0.57112d0*CON_KCALTOK
      tclocal(3) =     -2.19056d0*CON_KCALTOK
      tclocal(4) =      2.00696d0*CON_KCALTOK
      tclocal(5) =     -0.43088d0*CON_KCALTOK
      tclocal(6) =     -2.15561d0*CON_KCALTOK
      tclocal(7) =      2.99600d0*CON_KCALTOK
      tclocal(8) =     -0.72024d0*CON_KCALTOK
      tclocal( 9) =     -0.51522d0*CON_KCALTOK
      tclocal(10) =     -0.06929d0*CON_KCALTOK
      tclocal(11) =      0.44051d0*CON_KCALTOK
      tclocal(12) =     -0.96092d0*CON_KCALTOK
      tclocal(13) =     -1.39367d0*CON_KCALTOK
      tclocal(14) =     -1.23335d0*CON_KCALTOK
      tclocal(15) =      0.10647d0*CON_KCALTOK
      tclocal(16) =      0.94929d0*CON_KCALTOK
      tclocal(17) =     -1.52881d0*CON_KCALTOK
      tclocal(18) =     -7.25446d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      2.15868d0*CON_KCALTOK
      tclocal(1) =      4.73521d0*CON_KCALTOK
      tclocal(2) =      0.46205d0*CON_KCALTOK
      tclocal(3) =     -2.95959d0*CON_KCALTOK
      tclocal(4) =      1.80651d0*CON_KCALTOK
      tclocal(5) =     -0.72714d0*CON_KCALTOK
      tclocal(6) =      4.68928d0*CON_KCALTOK
      tclocal(7) =      9.18727d0*CON_KCALTOK
      tclocal(8) =     -1.19612d0*CON_KCALTOK
      tclocal( 9) =      3.45442d0*CON_KCALTOK
      tclocal(10) =     -0.10366d0*CON_KCALTOK
      tclocal(11) =      0.73012d0*CON_KCALTOK
      tclocal(12) =      7.76622d0*CON_KCALTOK
      tclocal(13) =      3.63524d0*CON_KCALTOK
      tclocal(14) =     -0.11314d0*CON_KCALTOK
      tclocal(15) =      4.43736d0*CON_KCALTOK
      tclocal(16) =      8.61166d0*CON_KCALTOK
      tclocal(17) =      1.36638d0*CON_KCALTOK
      tclocal(18) =     -5.18040d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.51920d0*CON_KCALTOK
      tclocal(1) =      1.30214d0*CON_KCALTOK
      tclocal(2) =      0.00726d0*CON_KCALTOK
      tclocal(3) =     -6.23545d0*CON_KCALTOK
      tclocal(4) =     -0.54456d0*CON_KCALTOK
      tclocal(5) =     -0.08618d0*CON_KCALTOK
      tclocal(6) =      3.91531d0*CON_KCALTOK
      tclocal(7) =     -0.73689d0*CON_KCALTOK
      tclocal(8) =     -3.90755d0*CON_KCALTOK
      tclocal( 9) =     -1.18096d0*CON_KCALTOK
      tclocal(10) =     -1.65733d0*CON_KCALTOK
      tclocal(11) =      0.02454d0*CON_KCALTOK
      tclocal(12) =     -4.40020d0*CON_KCALTOK
      tclocal(13) =     -2.08070d0*CON_KCALTOK
      tclocal(14) =      1.18816d0*CON_KCALTOK
      tclocal(15) =     -2.55885d0*CON_KCALTOK
      tclocal(16) =      0.04412d0*CON_KCALTOK
      tclocal(17) =     -0.05689d0*CON_KCALTOK
      tclocal(18) =     -7.04415d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.03551d0*CON_KCALTOK
      tclocal(1) =      3.79577d0*CON_KCALTOK
      tclocal(2) =      0.30769d0*CON_KCALTOK
      tclocal(3) =     -4.93273d0*CON_KCALTOK
      tclocal(4) =     11.75608d0*CON_KCALTOK
      tclocal(5) =      1.95625d0*CON_KCALTOK
      tclocal(6) =      0.52926d0*CON_KCALTOK
      tclocal(7) =      8.49898d0*CON_KCALTOK
      tclocal(8) =      0.33050d0*CON_KCALTOK
      tclocal( 9) =      0.10252d0*CON_KCALTOK
      tclocal(10) =     -2.82659d0*CON_KCALTOK
      tclocal(11) =     -0.86352d0*CON_KCALTOK
      tclocal(12) =      6.94554d0*CON_KCALTOK
      tclocal(13) =      3.16989d0*CON_KCALTOK
      tclocal(14) =      2.21537d0*CON_KCALTOK
      tclocal(15) =      2.74104d0*CON_KCALTOK
      tclocal(16) =      1.16965d0*CON_KCALTOK
      tclocal(17) =     -0.03801d0*CON_KCALTOK
      tclocal(18) =    -38.29857d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.03551d0*CON_KCALTOK
      tclocal(1) =      3.79577d0*CON_KCALTOK
      tclocal(2) =      0.30769d0*CON_KCALTOK
      tclocal(3) =     -4.93273d0*CON_KCALTOK
      tclocal(4) =     11.75608d0*CON_KCALTOK
      tclocal(5) =      1.95625d0*CON_KCALTOK
      tclocal(6) =      0.52926d0*CON_KCALTOK
      tclocal(7) =      8.49898d0*CON_KCALTOK
      tclocal(8) =      0.33050d0*CON_KCALTOK
      tclocal( 9) =      0.10252d0*CON_KCALTOK
      tclocal(10) =     -2.82659d0*CON_KCALTOK
      tclocal(11) =     -0.86352d0*CON_KCALTOK
      tclocal(12) =      6.94554d0*CON_KCALTOK
      tclocal(13) =      3.16989d0*CON_KCALTOK
      tclocal(14) =      2.21537d0*CON_KCALTOK
      tclocal(15) =      2.74104d0*CON_KCALTOK
      tclocal(16) =      1.16965d0*CON_KCALTOK
      tclocal(17) =     -0.03801d0*CON_KCALTOK
      tclocal(18) =    -38.29857d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.67816d0*CON_KCALTOK
      tclocal(1) =      2.84726d0*CON_KCALTOK
      tclocal(2) =     -0.01803d0*CON_KCALTOK
      tclocal(3) =     -9.30859d0*CON_KCALTOK
      tclocal(4) =      3.94229d0*CON_KCALTOK
      tclocal(5) =      1.91241d0*CON_KCALTOK
      tclocal(6) =      6.13275d0*CON_KCALTOK
      tclocal(7) =      3.93606d0*CON_KCALTOK
      tclocal(8) =     -9.21662d0*CON_KCALTOK
      tclocal( 9) =      2.05940d0*CON_KCALTOK
      tclocal(10) =     -3.19862d0*CON_KCALTOK
      tclocal(11) =     -3.03472d0*CON_KCALTOK
      tclocal(12) =     -4.95510d0*CON_KCALTOK
      tclocal(13) =     -2.48274d0*CON_KCALTOK
      tclocal(14) =     -2.05321d0*CON_KCALTOK
      tclocal(15) =     -0.52968d0*CON_KCALTOK
      tclocal(16) =      0.23761d0*CON_KCALTOK
      tclocal(17) =     -1.23331d0*CON_KCALTOK
      tclocal(18) =    -18.82354d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c" ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c" '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.06313d0*CON_KCALTOK
      tclocal(1) =      0.04173d0*CON_KCALTOK
      tclocal(2) =      0.08025d0*CON_KCALTOK
      tclocal(3) =     -0.40720d0*CON_KCALTOK
      tclocal(4) =      0.11830d0*CON_KCALTOK
      tclocal(5) =     -0.22051d0*CON_KCALTOK
      tclocal(6) =      0.49102d0*CON_KCALTOK
      tclocal(7) =     -0.15883d0*CON_KCALTOK
      tclocal(8) =      0.17660d0*CON_KCALTOK
      tclocal( 9) =      0.38463d0*CON_KCALTOK
      tclocal(10) =      0.07290d0*CON_KCALTOK
      tclocal(11) =      0.15160d0*CON_KCALTOK
      tclocal(12) =      0.97822d0*CON_KCALTOK
      tclocal(13) =      0.86445d0*CON_KCALTOK
      tclocal(14) =      0.97466d0*CON_KCALTOK
      tclocal(15) =      0.12831d0*CON_KCALTOK
      tclocal(16) =     -1.98595d0*CON_KCALTOK
      tclocal(17) =     -0.79020d0*CON_KCALTOK
      tclocal(18) =    -11.62172d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.25765d0*CON_KCALTOK
      tclocal(1) =      0.00165d0*CON_KCALTOK
      tclocal(2) =      0.05304d0*CON_KCALTOK
      tclocal(3) =     -0.33297d0*CON_KCALTOK
      tclocal(4) =      0.05132d0*CON_KCALTOK
      tclocal(5) =      0.55842d0*CON_KCALTOK
      tclocal(6) =      0.34185d0*CON_KCALTOK
      tclocal(7) =      1.32444d0*CON_KCALTOK
      tclocal(8) =      1.39046d0*CON_KCALTOK
      tclocal( 9) =      1.75989d0*CON_KCALTOK
      tclocal(10) =      0.41030d0*CON_KCALTOK
      tclocal(11) =      0.56751d0*CON_KCALTOK
      tclocal(12) =     -2.16615d0*CON_KCALTOK
      tclocal(13) =      1.05918d0*CON_KCALTOK
      tclocal(14) =     -0.30187d0*CON_KCALTOK
      tclocal(15) =     13.23175d0*CON_KCALTOK
      tclocal(16) =     -0.70250d0*CON_KCALTOK
      tclocal(17) =      0.51916d0*CON_KCALTOK
      tclocal(18) =    -22.24579d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'co ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'co '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      2.42191d0*CON_KCALTOK
      tclocal(1) =      1.99364d0*CON_KCALTOK
      tclocal(2) =      0.07462d0*CON_KCALTOK
      tclocal(3) =     -2.35154d0*CON_KCALTOK
      tclocal(4) =     -2.55106d0*CON_KCALTOK
      tclocal(5) =      0.00538d0*CON_KCALTOK
      tclocal(6) =      0.39658d0*CON_KCALTOK
      tclocal(7) =      2.12380d0*CON_KCALTOK
      tclocal(8) =      0.37185d0*CON_KCALTOK
      tclocal( 9) =      0.96795d0*CON_KCALTOK
      tclocal(10) =     -1.00184d0*CON_KCALTOK
      tclocal(11) =      0.81685d0*CON_KCALTOK
      tclocal(12) =      9.68519d0*CON_KCALTOK
      tclocal(13) =      1.47486d0*CON_KCALTOK
      tclocal(14) =     -0.32559d0*CON_KCALTOK
      tclocal(15) =     13.85169d0*CON_KCALTOK
      tclocal(16) =      1.27104d0*CON_KCALTOK
      tclocal(17) =      1.40167d0*CON_KCALTOK
      tclocal(18) =    -27.86866d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.32659d0*CON_KCALTOK
      tclocal(1) =      2.50407d0*CON_KCALTOK
      tclocal(2) =     -0.65048d0*CON_KCALTOK
      tclocal(3) =     -1.29062d0*CON_KCALTOK
      tclocal(4) =      1.69169d0*CON_KCALTOK
      tclocal(5) =     -0.53021d0*CON_KCALTOK
      tclocal(6) =      0.25101d0*CON_KCALTOK
      tclocal(7) =      5.01893d0*CON_KCALTOK
      tclocal(8) =     -1.07733d0*CON_KCALTOK
      tclocal( 9) =     -0.40988d0*CON_KCALTOK
      tclocal(10) =      0.48514d0*CON_KCALTOK
      tclocal(11) =     -1.01266d0*CON_KCALTOK
      tclocal(12) =      4.80072d0*CON_KCALTOK
      tclocal(13) =      2.51694d0*CON_KCALTOK
      tclocal(14) =     -1.64858d0*CON_KCALTOK
      tclocal(15) =      6.62383d0*CON_KCALTOK
      tclocal(16) =      1.78736d0*CON_KCALTOK
      tclocal(17) =     -1.74783d0*CON_KCALTOK
      tclocal(18) =    -20.41519d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.91914d0*CON_KCALTOK
      tclocal(1) =      2.38049d0*CON_KCALTOK
      tclocal(2) =      0.37605d0*CON_KCALTOK
      tclocal(3) =      3.50918d0*CON_KCALTOK
      tclocal(4) =      2.31174d0*CON_KCALTOK
      tclocal(5) =      1.23864d0*CON_KCALTOK
      tclocal(6) =      1.81641d0*CON_KCALTOK
      tclocal(7) =      2.76480d0*CON_KCALTOK
      tclocal(8) =      0.78807d0*CON_KCALTOK
      tclocal( 9) =      1.72023d0*CON_KCALTOK
      tclocal(10) =      0.52085d0*CON_KCALTOK
      tclocal(11) =      0.45527d0*CON_KCALTOK
      tclocal(12) =      8.06185d0*CON_KCALTOK
      tclocal(13) =      5.95126d0*CON_KCALTOK
      tclocal(14) =      1.00003d0*CON_KCALTOK
      tclocal(15) =     14.72115d0*CON_KCALTOK
      tclocal(16) =      2.50606d0*CON_KCALTOK
      tclocal(17) =      1.01732d0*CON_KCALTOK
      tclocal(18) =    -40.36525d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.25952d0*CON_KCALTOK
      tclocal(1) =      1.66413d0*CON_KCALTOK
      tclocal(2) =     -0.00981d0*CON_KCALTOK
      tclocal(3) =      0.24573d0*CON_KCALTOK
      tclocal(4) =      2.87250d0*CON_KCALTOK
      tclocal(5) =     -0.25866d0*CON_KCALTOK
      tclocal(6) =     -0.03214d0*CON_KCALTOK
      tclocal(7) =      3.67474d0*CON_KCALTOK
      tclocal(8) =      0.18681d0*CON_KCALTOK
      tclocal( 9) =     -0.76405d0*CON_KCALTOK
      tclocal(10) =     -0.46931d0*CON_KCALTOK
      tclocal(11) =      0.08224d0*CON_KCALTOK
      tclocal(12) =      5.32206d0*CON_KCALTOK
      tclocal(13) =      5.41647d0*CON_KCALTOK
      tclocal(14) =      0.04294d0*CON_KCALTOK
      tclocal(15) =      5.41304d0*CON_KCALTOK
      tclocal(16) =      0.93752d0*CON_KCALTOK
      tclocal(17) =      0.16054d0*CON_KCALTOK
      tclocal(18) =    -30.73435d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.17631d0*CON_KCALTOK
      tclocal(1) =      0.48285d0*CON_KCALTOK
      tclocal(2) =     -0.43886d0*CON_KCALTOK
      tclocal(3) =     -2.05400d0*CON_KCALTOK
      tclocal(4) =     -0.06846d0*CON_KCALTOK
      tclocal(5) =     -0.90440d0*CON_KCALTOK
      tclocal(6) =     -0.59428d0*CON_KCALTOK
      tclocal(7) =      0.95893d0*CON_KCALTOK
      tclocal(8) =     -0.02639d0*CON_KCALTOK
      tclocal( 9) =     -1.40812d0*CON_KCALTOK
      tclocal(10) =     -0.66525d0*CON_KCALTOK
      tclocal(11) =      0.12345d0*CON_KCALTOK
      tclocal(12) =     -0.40004d0*CON_KCALTOK
      tclocal(13) =     -0.75271d0*CON_KCALTOK
      tclocal(14) =     -0.92911d0*CON_KCALTOK
      tclocal(15) =      1.98627d0*CON_KCALTOK
      tclocal(16) =     -0.01879d0*CON_KCALTOK
      tclocal(17) =     -0.48799d0*CON_KCALTOK
      tclocal(18) =    -14.30229d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.23992d0*CON_KCALTOK
      tclocal(1) =      1.66070d0*CON_KCALTOK
      tclocal(2) =     -0.16959d0*CON_KCALTOK
      tclocal(3) =     -1.69961d0*CON_KCALTOK
      tclocal(4) =     -2.62185d0*CON_KCALTOK
      tclocal(5) =     -0.01657d0*CON_KCALTOK
      tclocal(6) =     -1.20083d0*CON_KCALTOK
      tclocal(7) =      0.45268d0*CON_KCALTOK
      tclocal(8) =     -0.54162d0*CON_KCALTOK
      tclocal( 9) =     -1.99136d0*CON_KCALTOK
      tclocal(10) =     -1.07394d0*CON_KCALTOK
      tclocal(11) =     -0.45182d0*CON_KCALTOK
      tclocal(12) =      6.99704d0*CON_KCALTOK
      tclocal(13) =     -0.21422d0*CON_KCALTOK
      tclocal(14) =     -0.56240d0*CON_KCALTOK
      tclocal(15) =      8.64490d0*CON_KCALTOK
      tclocal(16) =      0.70270d0*CON_KCALTOK
      tclocal(17) =     -0.56400d0*CON_KCALTOK
      tclocal(18) =    -26.27923d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.06504d0*CON_KCALTOK
      tclocal(1) =      1.63862d0*CON_KCALTOK
      tclocal(2) =     -0.51091d0*CON_KCALTOK
      tclocal(3) =     -1.69656d0*CON_KCALTOK
      tclocal(4) =     -0.18683d0*CON_KCALTOK
      tclocal(5) =     -0.14179d0*CON_KCALTOK
      tclocal(6) =     -0.15187d0*CON_KCALTOK
      tclocal(7) =      4.50335d0*CON_KCALTOK
      tclocal(8) =     -0.06325d0*CON_KCALTOK
      tclocal( 9) =      2.31942d0*CON_KCALTOK
      tclocal(10) =     -0.78332d0*CON_KCALTOK
      tclocal(11) =      0.01249d0*CON_KCALTOK
      tclocal(12) =      7.55549d0*CON_KCALTOK
      tclocal(13) =      1.34949d0*CON_KCALTOK
      tclocal(14) =     -0.23544d0*CON_KCALTOK
      tclocal(15) =      5.12280d0*CON_KCALTOK
      tclocal(16) =     -0.83554d0*CON_KCALTOK
      tclocal(17) =     -2.53701d0*CON_KCALTOK
      tclocal(18) =    -40.43994d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.59636d0*CON_KCALTOK
      tclocal(1) =      1.34003d0*CON_KCALTOK
      tclocal(2) =     -0.33807d0*CON_KCALTOK
      tclocal(3) =      3.50111d0*CON_KCALTOK
      tclocal(4) =      1.38521d0*CON_KCALTOK
      tclocal(5) =      0.54232d0*CON_KCALTOK
      tclocal(6) =     -1.06010d0*CON_KCALTOK
      tclocal(7) =      2.41091d0*CON_KCALTOK
      tclocal(8) =     -0.30275d0*CON_KCALTOK
      tclocal( 9) =      1.77418d0*CON_KCALTOK
      tclocal(10) =     -0.21269d0*CON_KCALTOK
      tclocal(11) =      0.49213d0*CON_KCALTOK
      tclocal(12) =      0.71224d0*CON_KCALTOK
      tclocal(13) =      3.05007d0*CON_KCALTOK
      tclocal(14) =     -0.72240d0*CON_KCALTOK
      tclocal(15) =      5.86135d0*CON_KCALTOK
      tclocal(16) =      0.88956d0*CON_KCALTOK
      tclocal(17) =     -0.07388d0*CON_KCALTOK
      tclocal(18) =    -18.84826d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.76586d0*CON_KCALTOK
      tclocal(1) =      1.70272d0*CON_KCALTOK
      tclocal(2) =     -0.30560d0*CON_KCALTOK
      tclocal(3) =      1.89718d0*CON_KCALTOK
      tclocal(4) =      2.83377d0*CON_KCALTOK
      tclocal(5) =      1.57622d0*CON_KCALTOK
      tclocal(6) =     -0.23485d0*CON_KCALTOK
      tclocal(7) =      3.82965d0*CON_KCALTOK
      tclocal(8) =     -0.44241d0*CON_KCALTOK
      tclocal( 9) =      2.21982d0*CON_KCALTOK
      tclocal(10) =     -1.21781d0*CON_KCALTOK
      tclocal(11) =     -0.05427d0*CON_KCALTOK
      tclocal(12) =      2.48936d0*CON_KCALTOK
      tclocal(13) =      3.49115d0*CON_KCALTOK
      tclocal(14) =      0.22431d0*CON_KCALTOK
      tclocal(15) =      1.11179d0*CON_KCALTOK
      tclocal(16) =     -0.05531d0*CON_KCALTOK
      tclocal(17) =     -0.05331d0*CON_KCALTOK
      tclocal(18) =    -21.60441d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.06783d0*CON_KCALTOK
      tclocal(1) =      1.21589d0*CON_KCALTOK
      tclocal(2) =      0.30575d0*CON_KCALTOK
      tclocal(3) =     -1.30856d0*CON_KCALTOK
      tclocal(4) =     -0.28950d0*CON_KCALTOK
      tclocal(5) =      0.35360d0*CON_KCALTOK
      tclocal(6) =      0.60470d0*CON_KCALTOK
      tclocal(7) =      0.99711d0*CON_KCALTOK
      tclocal(8) =     -0.32171d0*CON_KCALTOK
      tclocal( 9) =      1.76853d0*CON_KCALTOK
      tclocal(10) =     -0.59613d0*CON_KCALTOK
      tclocal(11) =      0.03985d0*CON_KCALTOK
      tclocal(12) =      0.68755d0*CON_KCALTOK
      tclocal(13) =      0.01179d0*CON_KCALTOK
      tclocal(14) =      0.31631d0*CON_KCALTOK
      tclocal(15) =      1.59642d0*CON_KCALTOK
      tclocal(16) =     -0.35985d0*CON_KCALTOK
      tclocal(17) =      0.52653d0*CON_KCALTOK
      tclocal(18) =    -22.42716d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.12062d0*CON_KCALTOK
      tclocal(1) =      2.52582d0*CON_KCALTOK
      tclocal(2) =     -0.47441d0*CON_KCALTOK
      tclocal(3) =     -0.55903d0*CON_KCALTOK
      tclocal(4) =     -0.49959d0*CON_KCALTOK
      tclocal(5) =      1.04116d0*CON_KCALTOK
      tclocal(6) =     -2.62179d0*CON_KCALTOK
      tclocal(7) =      9.84849d0*CON_KCALTOK
      tclocal(8) =     -3.95183d0*CON_KCALTOK
      tclocal( 9) =      0.24412d0*CON_KCALTOK
      tclocal(10) =      0.64519d0*CON_KCALTOK
      tclocal(11) =      0.30560d0*CON_KCALTOK
      tclocal(12) =      0.98532d0*CON_KCALTOK
      tclocal(13) =      5.99110d0*CON_KCALTOK
      tclocal(14) =      0.16747d0*CON_KCALTOK
      tclocal(15) =     13.10389d0*CON_KCALTOK
      tclocal(16) =      1.97924d0*CON_KCALTOK
      tclocal(17) =     -1.13180d0*CON_KCALTOK
      tclocal(18) =    -19.00875d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.67649d0*CON_KCALTOK
      tclocal(1) =     -0.04643d0*CON_KCALTOK
      tclocal(2) =     -0.99486d0*CON_KCALTOK
      tclocal(3) =      0.21518d0*CON_KCALTOK
      tclocal(4) =     -0.17196d0*CON_KCALTOK
      tclocal(5) =     -0.03161d0*CON_KCALTOK
      tclocal(6) =      0.64574d0*CON_KCALTOK
      tclocal(7) =      4.42277d0*CON_KCALTOK
      tclocal(8) =     -1.34126d0*CON_KCALTOK
      tclocal( 9) =      0.47732d0*CON_KCALTOK
      tclocal(10) =     -0.25231d0*CON_KCALTOK
      tclocal(11) =      0.66060d0*CON_KCALTOK
      tclocal(12) =      1.59012d0*CON_KCALTOK
      tclocal(13) =      1.02157d0*CON_KCALTOK
      tclocal(14) =      1.74173d0*CON_KCALTOK
      tclocal(15) =     -0.68858d0*CON_KCALTOK
      tclocal(16) =      0.25778d0*CON_KCALTOK
      tclocal(17) =      0.20015d0*CON_KCALTOK
      tclocal(18) =     -0.04361d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.43471d0*CON_KCALTOK
      tclocal(1) =      1.02690d0*CON_KCALTOK
      tclocal(2) =     -0.71455d0*CON_KCALTOK
      tclocal(3) =      1.52142d0*CON_KCALTOK
      tclocal(4) =     -1.65763d0*CON_KCALTOK
      tclocal(5) =      0.11155d0*CON_KCALTOK
      tclocal(6) =     -2.53817d0*CON_KCALTOK
      tclocal(7) =      3.87393d0*CON_KCALTOK
      tclocal(8) =     -0.05936d0*CON_KCALTOK
      tclocal( 9) =      1.66253d0*CON_KCALTOK
      tclocal(10) =     -2.07145d0*CON_KCALTOK
      tclocal(11) =     -1.67920d0*CON_KCALTOK
      tclocal(12) =     -1.90592d0*CON_KCALTOK
      tclocal(13) =     -0.66494d0*CON_KCALTOK
      tclocal(14) =     -0.35152d0*CON_KCALTOK
      tclocal(15) =      3.85535d0*CON_KCALTOK
      tclocal(16) =      5.00629d0*CON_KCALTOK
      tclocal(17) =     -0.98184d0*CON_KCALTOK
      tclocal(18) =     -6.99647d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.36976d0*CON_KCALTOK
      tclocal(1) =      2.16566d0*CON_KCALTOK
      tclocal(2) =     -0.37507d0*CON_KCALTOK
      tclocal(3) =     -1.42638d0*CON_KCALTOK
      tclocal(4) =      0.45499d0*CON_KCALTOK
      tclocal(5) =     -0.00468d0*CON_KCALTOK
      tclocal(6) =     -3.40490d0*CON_KCALTOK
      tclocal(7) =     10.16126d0*CON_KCALTOK
      tclocal(8) =     -2.64913d0*CON_KCALTOK
      tclocal( 9) =      1.82399d0*CON_KCALTOK
      tclocal(10) =     -0.21706d0*CON_KCALTOK
      tclocal(11) =      1.12930d0*CON_KCALTOK
      tclocal(12) =     -5.22622d0*CON_KCALTOK
      tclocal(13) =      4.98124d0*CON_KCALTOK
      tclocal(14) =     -2.69878d0*CON_KCALTOK
      tclocal(15) =      5.16517d0*CON_KCALTOK
      tclocal(16) =      0.04921d0*CON_KCALTOK
      tclocal(17) =      0.57405d0*CON_KCALTOK
      tclocal(18) =     -7.93382d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.37478d0*CON_KCALTOK
      tclocal(1) =      3.31887d0*CON_KCALTOK
      tclocal(2) =     -0.18578d0*CON_KCALTOK
      tclocal(3) =     -3.17267d0*CON_KCALTOK
      tclocal(4) =     -2.61212d0*CON_KCALTOK
      tclocal(5) =      2.11416d0*CON_KCALTOK
      tclocal(6) =    -11.45739d0*CON_KCALTOK
      tclocal(7) =      5.17743d0*CON_KCALTOK
      tclocal(8) =     -2.52483d0*CON_KCALTOK
      tclocal( 9) =     -1.18814d0*CON_KCALTOK
      tclocal(10) =     -1.76269d0*CON_KCALTOK
      tclocal(11) =     -0.60736d0*CON_KCALTOK
      tclocal(12) =     -1.95954d0*CON_KCALTOK
      tclocal(13) =      2.47915d0*CON_KCALTOK
      tclocal(14) =      0.19959d0*CON_KCALTOK
      tclocal(15) =     12.34609d0*CON_KCALTOK
      tclocal(16) =      1.47755d0*CON_KCALTOK
      tclocal(17) =     -0.85427d0*CON_KCALTOK
      tclocal(18) =    -13.24596d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.44687d0*CON_KCALTOK
      tclocal(1) =      0.95227d0*CON_KCALTOK
      tclocal(2) =     -0.82624d0*CON_KCALTOK
      tclocal(3) =     -0.64611d0*CON_KCALTOK
      tclocal(4) =     -0.51814d0*CON_KCALTOK
      tclocal(5) =      0.25352d0*CON_KCALTOK
      tclocal(6) =      0.80113d0*CON_KCALTOK
      tclocal(7) =      4.41814d0*CON_KCALTOK
      tclocal(8) =      0.71952d0*CON_KCALTOK
      tclocal( 9) =      0.12257d0*CON_KCALTOK
      tclocal(10) =     -3.15651d0*CON_KCALTOK
      tclocal(11) =     -1.95679d0*CON_KCALTOK
      tclocal(12) =     -1.75280d0*CON_KCALTOK
      tclocal(13) =     -0.53866d0*CON_KCALTOK
      tclocal(14) =     -0.26065d0*CON_KCALTOK
      tclocal(15) =      5.92686d0*CON_KCALTOK
      tclocal(16) =      2.03801d0*CON_KCALTOK
      tclocal(17) =      0.24041d0*CON_KCALTOK
      tclocal(18) =      0.09242d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.14042d0*CON_KCALTOK
      tclocal(1) =      2.77327d0*CON_KCALTOK
      tclocal(2) =     -0.87732d0*CON_KCALTOK
      tclocal(3) =     -3.14038d0*CON_KCALTOK
      tclocal(4) =     -1.16456d0*CON_KCALTOK
      tclocal(5) =      0.21715d0*CON_KCALTOK
      tclocal(6) =      3.05490d0*CON_KCALTOK
      tclocal(7) =      6.60559d0*CON_KCALTOK
      tclocal(8) =      0.08027d0*CON_KCALTOK
      tclocal( 9) =     -0.45830d0*CON_KCALTOK
      tclocal(10) =     -0.02506d0*CON_KCALTOK
      tclocal(11) =      1.62230d0*CON_KCALTOK
      tclocal(12) =     -5.64672d0*CON_KCALTOK
      tclocal(13) =      3.26104d0*CON_KCALTOK
      tclocal(14) =     -0.37035d0*CON_KCALTOK
      tclocal(15) =      6.34475d0*CON_KCALTOK
      tclocal(16) =      1.75131d0*CON_KCALTOK
      tclocal(17) =      1.04570d0*CON_KCALTOK
      tclocal(18) =     -7.78549d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.69729d0*CON_KCALTOK
      tclocal(1) =      3.09416d0*CON_KCALTOK
      tclocal(2) =     -0.76320d0*CON_KCALTOK
      tclocal(3) =     -2.31479d0*CON_KCALTOK
      tclocal(4) =     -0.32926d0*CON_KCALTOK
      tclocal(5) =     -0.14202d0*CON_KCALTOK
      tclocal(6) =      2.07925d0*CON_KCALTOK
      tclocal(7) =     10.08375d0*CON_KCALTOK
      tclocal(8) =     -0.73689d0*CON_KCALTOK
      tclocal( 9) =      0.53095d0*CON_KCALTOK
      tclocal(10) =     -0.45217d0*CON_KCALTOK
      tclocal(11) =      0.25141d0*CON_KCALTOK
      tclocal(12) =      1.46215d0*CON_KCALTOK
      tclocal(13) =      3.82514d0*CON_KCALTOK
      tclocal(14) =      0.67451d0*CON_KCALTOK
      tclocal(15) =     12.02032d0*CON_KCALTOK
      tclocal(16) =      1.00173d0*CON_KCALTOK
      tclocal(17) =     -1.09108d0*CON_KCALTOK
      tclocal(18) =    -14.00587d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.21835d0*CON_KCALTOK
      tclocal(1) =      1.56683d0*CON_KCALTOK
      tclocal(2) =      0.04976d0*CON_KCALTOK
      tclocal(3) =     -0.78052d0*CON_KCALTOK
      tclocal(4) =      0.04974d0*CON_KCALTOK
      tclocal(5) =      0.39207d0*CON_KCALTOK
      tclocal(6) =     -2.10490d0*CON_KCALTOK
      tclocal(7) =      2.97776d0*CON_KCALTOK
      tclocal(8) =     -1.55632d0*CON_KCALTOK
      tclocal( 9) =     -2.02954d0*CON_KCALTOK
      tclocal(10) =     -3.20496d0*CON_KCALTOK
      tclocal(11) =      1.49934d0*CON_KCALTOK
      tclocal(12) =     -3.37910d0*CON_KCALTOK
      tclocal(13) =      3.29226d0*CON_KCALTOK
      tclocal(14) =     -0.07731d0*CON_KCALTOK
      tclocal(15) =      3.39566d0*CON_KCALTOK
      tclocal(16) =      1.00242d0*CON_KCALTOK
      tclocal(17) =     -0.93370d0*CON_KCALTOK
      tclocal(18) =     -4.88000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.39260d0*CON_KCALTOK
      tclocal(1) =      2.81644d0*CON_KCALTOK
      tclocal(2) =     -0.29036d0*CON_KCALTOK
      tclocal(3) =      3.87087d0*CON_KCALTOK
      tclocal(4) =     -0.26996d0*CON_KCALTOK
      tclocal(5) =      0.45603d0*CON_KCALTOK
      tclocal(6) =     -4.32024d0*CON_KCALTOK
      tclocal(7) =      2.11566d0*CON_KCALTOK
      tclocal(8) =     -1.46836d0*CON_KCALTOK
      tclocal( 9) =     -1.05340d0*CON_KCALTOK
      tclocal(10) =     -4.68305d0*CON_KCALTOK
      tclocal(11) =      1.80466d0*CON_KCALTOK
      tclocal(12) =     -2.33800d0*CON_KCALTOK
      tclocal(13) =      3.72920d0*CON_KCALTOK
      tclocal(14) =     -0.63481d0*CON_KCALTOK
      tclocal(15) =      9.49570d0*CON_KCALTOK
      tclocal(16) =     -2.75876d0*CON_KCALTOK
      tclocal(17) =      0.54688d0*CON_KCALTOK
      tclocal(18) =     -6.24937d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.58920d0*CON_KCALTOK
      tclocal(1) =      2.54233d0*CON_KCALTOK
      tclocal(2) =     -0.27650d0*CON_KCALTOK
      tclocal(3) =     -2.35553d0*CON_KCALTOK
      tclocal(4) =      1.01263d0*CON_KCALTOK
      tclocal(5) =     -0.48318d0*CON_KCALTOK
      tclocal(6) =      2.45630d0*CON_KCALTOK
      tclocal(7) =      7.25443d0*CON_KCALTOK
      tclocal(8) =      0.01511d0*CON_KCALTOK
      tclocal( 9) =      1.12035d0*CON_KCALTOK
      tclocal(10) =     -0.09968d0*CON_KCALTOK
      tclocal(11) =      0.83983d0*CON_KCALTOK
      tclocal(12) =     -1.82923d0*CON_KCALTOK
      tclocal(13) =      2.48280d0*CON_KCALTOK
      tclocal(14) =     -1.04741d0*CON_KCALTOK
      tclocal(15) =      9.34899d0*CON_KCALTOK
      tclocal(16) =      0.91073d0*CON_KCALTOK
      tclocal(17) =     -0.03718d0*CON_KCALTOK
      tclocal(18) =     -4.12013d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.97508d0*CON_KCALTOK
      tclocal(1) =      1.90829d0*CON_KCALTOK
      tclocal(2) =     -0.17287d0*CON_KCALTOK
      tclocal(3) =     -1.39313d0*CON_KCALTOK
      tclocal(4) =      2.43439d0*CON_KCALTOK
      tclocal(5) =      1.39309d0*CON_KCALTOK
      tclocal(6) =     -5.90697d0*CON_KCALTOK
      tclocal(7) =      8.22754d0*CON_KCALTOK
      tclocal(8) =     -1.09517d0*CON_KCALTOK
      tclocal( 9) =      2.44991d0*CON_KCALTOK
      tclocal(10) =      0.15057d0*CON_KCALTOK
      tclocal(11) =     -1.84670d0*CON_KCALTOK
      tclocal(12) =     -0.72564d0*CON_KCALTOK
      tclocal(13) =      6.54427d0*CON_KCALTOK
      tclocal(14) =     -1.20839d0*CON_KCALTOK
      tclocal(15) =      9.99878d0*CON_KCALTOK
      tclocal(16) =      4.40348d0*CON_KCALTOK
      tclocal(17) =     -0.66396d0*CON_KCALTOK
      tclocal(18) =    -11.34202d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.01379d0*CON_KCALTOK
      tclocal(1) =      1.32615d0*CON_KCALTOK
      tclocal(2) =      1.02404d0*CON_KCALTOK
      tclocal(3) =      2.80165d0*CON_KCALTOK
      tclocal(4) =      3.48809d0*CON_KCALTOK
      tclocal(5) =      2.23209d0*CON_KCALTOK
      tclocal(6) =      9.46622d0*CON_KCALTOK
      tclocal(7) =     -1.25691d0*CON_KCALTOK
      tclocal(8) =    -10.78463d0*CON_KCALTOK
      tclocal( 9) =      2.80165d0*CON_KCALTOK
      tclocal(10) =      3.48809d0*CON_KCALTOK
      tclocal(11) =      2.23209d0*CON_KCALTOK
      tclocal(12) =     -0.65281d0*CON_KCALTOK
      tclocal(13) =      1.43408d0*CON_KCALTOK
      tclocal(14) =      0.73987d0*CON_KCALTOK
      tclocal(15) =     -3.48269d0*CON_KCALTOK
      tclocal(16) =     -0.56251d0*CON_KCALTOK
      tclocal(17) =     -3.41583d0*CON_KCALTOK
      tclocal(18) =      0.73281d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.97688d0*CON_KCALTOK
      tclocal(1) =     -0.30743d0*CON_KCALTOK
      tclocal(2) =     -0.45003d0*CON_KCALTOK
      tclocal(3) =      2.02826d0*CON_KCALTOK
      tclocal(4) =      4.06048d0*CON_KCALTOK
      tclocal(5) =      2.05646d0*CON_KCALTOK
      tclocal(6) =      5.10712d0*CON_KCALTOK
      tclocal(7) =      5.82484d0*CON_KCALTOK
      tclocal(8) =     -5.31026d0*CON_KCALTOK
      tclocal( 9) =      0.79614d0*CON_KCALTOK
      tclocal(10) =     -1.47544d0*CON_KCALTOK
      tclocal(11) =      0.21060d0*CON_KCALTOK
      tclocal(12) =     -5.57264d0*CON_KCALTOK
      tclocal(13) =      6.57884d0*CON_KCALTOK
      tclocal(14) =      3.37809d0*CON_KCALTOK
      tclocal(15) =      3.30807d0*CON_KCALTOK
      tclocal(16) =      4.48383d0*CON_KCALTOK
      tclocal(17) =      2.17526d0*CON_KCALTOK
      tclocal(18) =      3.14860d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.81758d0*CON_KCALTOK
      tclocal(1) =      0.51587d0*CON_KCALTOK
      tclocal(2) =     -1.23315d0*CON_KCALTOK
      tclocal(3) =      0.11151d0*CON_KCALTOK
      tclocal(4) =     -0.76722d0*CON_KCALTOK
      tclocal(5) =     -0.00606d0*CON_KCALTOK
      tclocal(6) =     -1.80333d0*CON_KCALTOK
      tclocal(7) =     -6.42941d0*CON_KCALTOK
      tclocal(8) =      3.20135d0*CON_KCALTOK
      tclocal( 9) =     -0.70728d0*CON_KCALTOK
      tclocal(10) =     -1.03289d0*CON_KCALTOK
      tclocal(11) =      0.19115d0*CON_KCALTOK
      tclocal(12) =     -3.87655d0*CON_KCALTOK
      tclocal(13) =      2.28400d0*CON_KCALTOK
      tclocal(14) =      0.10582d0*CON_KCALTOK
      tclocal(15) =      1.17755d0*CON_KCALTOK
      tclocal(16) =     -1.18274d0*CON_KCALTOK
      tclocal(17) =      0.89941d0*CON_KCALTOK
      tclocal(18) =     -2.19700d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.55898d0*CON_KCALTOK
      tclocal(1) =      4.41931d0*CON_KCALTOK
      tclocal(2) =      0.10886d0*CON_KCALTOK
      tclocal(3) =     -1.34510d0*CON_KCALTOK
      tclocal(4) =     -0.61486d0*CON_KCALTOK
      tclocal(5) =      1.00750d0*CON_KCALTOK
      tclocal(6) =     -9.49868d0*CON_KCALTOK
      tclocal(7) =      4.46672d0*CON_KCALTOK
      tclocal(8) =     -0.57138d0*CON_KCALTOK
      tclocal( 9) =     -0.22270d0*CON_KCALTOK
      tclocal(10) =      0.56380d0*CON_KCALTOK
      tclocal(11) =      0.26616d0*CON_KCALTOK
      tclocal(12) =      4.30703d0*CON_KCALTOK
      tclocal(13) =      9.03233d0*CON_KCALTOK
      tclocal(14) =      1.79308d0*CON_KCALTOK
      tclocal(15) =     12.93409d0*CON_KCALTOK
      tclocal(16) =      4.44045d0*CON_KCALTOK
      tclocal(17) =      0.62417d0*CON_KCALTOK
      tclocal(18) =    -14.38237d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.11104d0*CON_KCALTOK
      tclocal(1) =      1.41784d0*CON_KCALTOK
      tclocal(2) =     -0.68355d0*CON_KCALTOK
      tclocal(3) =     -0.97168d0*CON_KCALTOK
      tclocal(4) =     -0.10057d0*CON_KCALTOK
      tclocal(5) =     -0.39983d0*CON_KCALTOK
      tclocal(6) =      6.75990d0*CON_KCALTOK
      tclocal(7) =      6.30044d0*CON_KCALTOK
      tclocal(8) =      1.04669d0*CON_KCALTOK
      tclocal( 9) =     -0.14775d0*CON_KCALTOK
      tclocal(10) =     -0.20980d0*CON_KCALTOK
      tclocal(11) =     -0.08669d0*CON_KCALTOK
      tclocal(12) =      0.47182d0*CON_KCALTOK
      tclocal(13) =      0.83920d0*CON_KCALTOK
      tclocal(14) =      0.69564d0*CON_KCALTOK
      tclocal(15) =      0.12833d0*CON_KCALTOK
      tclocal(16) =     -0.50884d0*CON_KCALTOK
      tclocal(17) =     -0.18023d0*CON_KCALTOK
      tclocal(18) =     -0.08467d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.47166d0*CON_KCALTOK
      tclocal(1) =      2.12419d0*CON_KCALTOK
      tclocal(2) =     -0.75436d0*CON_KCALTOK
      tclocal(3) =     -2.59757d0*CON_KCALTOK
      tclocal(4) =     -1.23597d0*CON_KCALTOK
      tclocal(5) =      1.14613d0*CON_KCALTOK
      tclocal(6) =      1.92002d0*CON_KCALTOK
      tclocal(7) =      1.56961d0*CON_KCALTOK
      tclocal(8) =      0.58264d0*CON_KCALTOK
      tclocal( 9) =     -0.94861d0*CON_KCALTOK
      tclocal(10) =      0.65255d0*CON_KCALTOK
      tclocal(11) =      0.10872d0*CON_KCALTOK
      tclocal(12) =     -4.49237d0*CON_KCALTOK
      tclocal(13) =      1.17716d0*CON_KCALTOK
      tclocal(14) =     -0.08552d0*CON_KCALTOK
      tclocal(15) =      4.94196d0*CON_KCALTOK
      tclocal(16) =      0.21818d0*CON_KCALTOK
      tclocal(17) =      0.74793d0*CON_KCALTOK
      tclocal(18) =     -9.71598d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.01733d0*CON_KCALTOK
      tclocal(1) =      4.35901d0*CON_KCALTOK
      tclocal(2) =      0.14574d0*CON_KCALTOK
      tclocal(3) =     -4.72087d0*CON_KCALTOK
      tclocal(4) =     -5.19058d0*CON_KCALTOK
      tclocal(5) =     -1.67124d0*CON_KCALTOK
      tclocal(6) =      3.24374d0*CON_KCALTOK
      tclocal(7) =     13.46227d0*CON_KCALTOK
      tclocal(8) =      1.82334d0*CON_KCALTOK
      tclocal( 9) =     -0.98943d0*CON_KCALTOK
      tclocal(10) =     -1.32873d0*CON_KCALTOK
      tclocal(11) =      1.81206d0*CON_KCALTOK
      tclocal(12) =      5.46364d0*CON_KCALTOK
      tclocal(13) =      5.34675d0*CON_KCALTOK
      tclocal(14) =      0.12670d0*CON_KCALTOK
      tclocal(15) =     12.51609d0*CON_KCALTOK
      tclocal(16) =      3.20375d0*CON_KCALTOK
      tclocal(17) =      1.21168d0*CON_KCALTOK
      tclocal(18) =    -34.77281d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.02326d0*CON_KCALTOK
      tclocal(1) =      2.31515d0*CON_KCALTOK
      tclocal(2) =     -0.17011d0*CON_KCALTOK
      tclocal(3) =     -5.08823d0*CON_KCALTOK
      tclocal(4) =      0.19569d0*CON_KCALTOK
      tclocal(5) =     -0.96148d0*CON_KCALTOK
      tclocal(6) =      2.67695d0*CON_KCALTOK
      tclocal(7) =      3.96297d0*CON_KCALTOK
      tclocal(8) =      0.15866d0*CON_KCALTOK
      tclocal( 9) =      1.77011d0*CON_KCALTOK
      tclocal(10) =     -4.33520d0*CON_KCALTOK
      tclocal(11) =      0.56328d0*CON_KCALTOK
      tclocal(12) =    -13.31779d0*CON_KCALTOK
      tclocal(13) =      2.95929d0*CON_KCALTOK
      tclocal(14) =      0.06571d0*CON_KCALTOK
      tclocal(15) =      2.31096d0*CON_KCALTOK
      tclocal(16) =      1.46516d0*CON_KCALTOK
      tclocal(17) =     -0.17536d0*CON_KCALTOK
      tclocal(18) =    -26.11618d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.08745d0*CON_KCALTOK
      tclocal(1) =      3.18341d0*CON_KCALTOK
      tclocal(2) =     -0.59559d0*CON_KCALTOK
      tclocal(3) =     -1.06532d0*CON_KCALTOK
      tclocal(4) =     -3.83930d0*CON_KCALTOK
      tclocal(5) =      2.23662d0*CON_KCALTOK
      tclocal(6) =     -0.01307d0*CON_KCALTOK
      tclocal(7) =      5.37057d0*CON_KCALTOK
      tclocal(8) =     -0.58286d0*CON_KCALTOK
      tclocal( 9) =      1.13415d0*CON_KCALTOK
      tclocal(10) =     -6.62608d0*CON_KCALTOK
      tclocal(11) =      0.56147d0*CON_KCALTOK
      tclocal(12) =     -3.71424d0*CON_KCALTOK
      tclocal(13) =      3.11012d0*CON_KCALTOK
      tclocal(14) =      0.23990d0*CON_KCALTOK
      tclocal(15) =      6.82146d0*CON_KCALTOK
      tclocal(16) =     -0.18318d0*CON_KCALTOK
      tclocal(17) =      0.12226d0*CON_KCALTOK
      tclocal(18) =    -23.58564d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.53946d0*CON_KCALTOK
      tclocal(1) =      3.49950d0*CON_KCALTOK
      tclocal(2) =     -0.10406d0*CON_KCALTOK
      tclocal(3) =     -4.46844d0*CON_KCALTOK
      tclocal(4) =     -6.34899d0*CON_KCALTOK
      tclocal(5) =      0.48998d0*CON_KCALTOK
      tclocal(6) =      0.33678d0*CON_KCALTOK
      tclocal(7) =      8.58777d0*CON_KCALTOK
      tclocal(8) =      1.71456d0*CON_KCALTOK
      tclocal( 9) =      0.53551d0*CON_KCALTOK
      tclocal(10) =      0.45422d0*CON_KCALTOK
      tclocal(11) =      0.49953d0*CON_KCALTOK
      tclocal(12) =    -10.02582d0*CON_KCALTOK
      tclocal(13) =     -3.67941d0*CON_KCALTOK
      tclocal(14) =     -1.03455d0*CON_KCALTOK
      tclocal(15) =      6.29146d0*CON_KCALTOK
      tclocal(16) =      2.16621d0*CON_KCALTOK
      tclocal(17) =     -0.25705d0*CON_KCALTOK
      tclocal(18) =    -18.63320d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.97609d0*CON_KCALTOK
      tclocal(1) =      2.55962d0*CON_KCALTOK
      tclocal(2) =      0.03777d0*CON_KCALTOK
      tclocal(3) =      0.22423d0*CON_KCALTOK
      tclocal(4) =      1.53333d0*CON_KCALTOK
      tclocal(5) =      0.99991d0*CON_KCALTOK
      tclocal(6) =      1.56337d0*CON_KCALTOK
      tclocal(7) =      4.12968d0*CON_KCALTOK
      tclocal(8) =     -5.72111d0*CON_KCALTOK
      tclocal( 9) =      0.00047d0*CON_KCALTOK
      tclocal(10) =     -1.27613d0*CON_KCALTOK
      tclocal(11) =      0.56178d0*CON_KCALTOK
      tclocal(12) =      1.12800d0*CON_KCALTOK
      tclocal(13) =     -2.92665d0*CON_KCALTOK
      tclocal(14) =      1.99942d0*CON_KCALTOK
      tclocal(15) =      1.02688d0*CON_KCALTOK
      tclocal(16) =     -0.95381d0*CON_KCALTOK
      tclocal(17) =      0.27970d0*CON_KCALTOK
      tclocal(18) =    -14.19418d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.38673d0*CON_KCALTOK
      tclocal(1) =      1.18595d0*CON_KCALTOK
      tclocal(2) =     -0.24157d0*CON_KCALTOK
      tclocal(3) =     -2.41548d0*CON_KCALTOK
      tclocal(4) =      0.19170d0*CON_KCALTOK
      tclocal(5) =      0.44559d0*CON_KCALTOK
      tclocal(6) =      2.49447d0*CON_KCALTOK
      tclocal(7) =      0.63746d0*CON_KCALTOK
      tclocal(8) =     -2.75500d0*CON_KCALTOK
      tclocal( 9) =     -0.47116d0*CON_KCALTOK
      tclocal(10) =     -3.56415d0*CON_KCALTOK
      tclocal(11) =      1.57692d0*CON_KCALTOK
      tclocal(12) =     -1.77027d0*CON_KCALTOK
      tclocal(13) =     -1.50365d0*CON_KCALTOK
      tclocal(14) =     -0.16020d0*CON_KCALTOK
      tclocal(15) =      2.56325d0*CON_KCALTOK
      tclocal(16) =      0.28870d0*CON_KCALTOK
      tclocal(17) =     -2.30199d0*CON_KCALTOK
      tclocal(18) =    -12.28636d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.16195d0*CON_KCALTOK
      tclocal(1) =      0.24783d0*CON_KCALTOK
      tclocal(2) =     -0.49702d0*CON_KCALTOK
      tclocal(3) =     -1.95047d0*CON_KCALTOK
      tclocal(4) =      2.04096d0*CON_KCALTOK
      tclocal(5) =      0.96738d0*CON_KCALTOK
      tclocal(6) =      0.19613d0*CON_KCALTOK
      tclocal(7) =      4.07727d0*CON_KCALTOK
      tclocal(8) =     -2.40950d0*CON_KCALTOK
      tclocal( 9) =      1.76432d0*CON_KCALTOK
      tclocal(10) =     -1.77816d0*CON_KCALTOK
      tclocal(11) =      1.65467d0*CON_KCALTOK
      tclocal(12) =      4.06444d0*CON_KCALTOK
      tclocal(13) =     -3.70978d0*CON_KCALTOK
      tclocal(14) =      0.56975d0*CON_KCALTOK
      tclocal(15) =     -0.29441d0*CON_KCALTOK
      tclocal(16) =      0.54598d0*CON_KCALTOK
      tclocal(17) =      0.95192d0*CON_KCALTOK
      tclocal(18) =    -11.14654d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.19072d0*CON_KCALTOK
      tclocal(1) =      0.97522d0*CON_KCALTOK
      tclocal(2) =     -0.57112d0*CON_KCALTOK
      tclocal(3) =     -2.19056d0*CON_KCALTOK
      tclocal(4) =      2.00696d0*CON_KCALTOK
      tclocal(5) =     -0.43088d0*CON_KCALTOK
      tclocal(6) =     -2.15561d0*CON_KCALTOK
      tclocal(7) =      2.99600d0*CON_KCALTOK
      tclocal(8) =     -0.72024d0*CON_KCALTOK
      tclocal( 9) =     -0.51522d0*CON_KCALTOK
      tclocal(10) =     -0.06929d0*CON_KCALTOK
      tclocal(11) =      0.44051d0*CON_KCALTOK
      tclocal(12) =     -0.96092d0*CON_KCALTOK
      tclocal(13) =     -1.39367d0*CON_KCALTOK
      tclocal(14) =     -1.23335d0*CON_KCALTOK
      tclocal(15) =      0.10647d0*CON_KCALTOK
      tclocal(16) =      0.94929d0*CON_KCALTOK
      tclocal(17) =     -1.52881d0*CON_KCALTOK
      tclocal(18) =     -7.25446d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.33806d0*CON_KCALTOK
      tclocal(1) =      2.46247d0*CON_KCALTOK
      tclocal(2) =     -0.28354d0*CON_KCALTOK
      tclocal(3) =     -1.17469d0*CON_KCALTOK
      tclocal(4) =      2.34697d0*CON_KCALTOK
      tclocal(5) =     -0.99629d0*CON_KCALTOK
      tclocal(6) =     -1.42872d0*CON_KCALTOK
      tclocal(7) =      8.48326d0*CON_KCALTOK
      tclocal(8) =     -3.57696d0*CON_KCALTOK
      tclocal( 9) =      1.76265d0*CON_KCALTOK
      tclocal(10) =      0.33926d0*CON_KCALTOK
      tclocal(11) =      1.21267d0*CON_KCALTOK
      tclocal(12) =      4.15310d0*CON_KCALTOK
      tclocal(13) =      1.70655d0*CON_KCALTOK
      tclocal(14) =     -1.03480d0*CON_KCALTOK
      tclocal(15) =     -0.93482d0*CON_KCALTOK
      tclocal(16) =      4.24328d0*CON_KCALTOK
      tclocal(17) =      0.54435d0*CON_KCALTOK
      tclocal(18) =     -9.51708d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.72370d0*CON_KCALTOK
      tclocal(1) =      0.87749d0*CON_KCALTOK
      tclocal(2) =     -0.73748d0*CON_KCALTOK
      tclocal(3) =     -1.74284d0*CON_KCALTOK
      tclocal(4) =     -2.04020d0*CON_KCALTOK
      tclocal(5) =      0.00638d0*CON_KCALTOK
      tclocal(6) =      3.61005d0*CON_KCALTOK
      tclocal(7) =      0.11070d0*CON_KCALTOK
      tclocal(8) =     -3.83167d0*CON_KCALTOK
      tclocal( 9) =      0.00806d0*CON_KCALTOK
      tclocal(10) =     -1.52615d0*CON_KCALTOK
      tclocal(11) =      0.52539d0*CON_KCALTOK
      tclocal(12) =     -5.81102d0*CON_KCALTOK
      tclocal(13) =     -4.50076d0*CON_KCALTOK
      tclocal(14) =      0.44929d0*CON_KCALTOK
      tclocal(15) =     -3.02199d0*CON_KCALTOK
      tclocal(16) =     -0.77558d0*CON_KCALTOK
      tclocal(17) =     -0.79336d0*CON_KCALTOK
      tclocal(18) =    -12.10438d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.03551d0*CON_KCALTOK
      tclocal(1) =      3.79577d0*CON_KCALTOK
      tclocal(2) =      0.30769d0*CON_KCALTOK
      tclocal(3) =     -4.93273d0*CON_KCALTOK
      tclocal(4) =     11.75608d0*CON_KCALTOK
      tclocal(5) =      1.95625d0*CON_KCALTOK
      tclocal(6) =      0.52926d0*CON_KCALTOK
      tclocal(7) =      8.49898d0*CON_KCALTOK
      tclocal(8) =      0.33050d0*CON_KCALTOK
      tclocal( 9) =      0.10252d0*CON_KCALTOK
      tclocal(10) =     -2.82659d0*CON_KCALTOK
      tclocal(11) =     -0.86352d0*CON_KCALTOK
      tclocal(12) =      6.94554d0*CON_KCALTOK
      tclocal(13) =      3.16989d0*CON_KCALTOK
      tclocal(14) =      2.21537d0*CON_KCALTOK
      tclocal(15) =      2.74104d0*CON_KCALTOK
      tclocal(16) =      1.16965d0*CON_KCALTOK
      tclocal(17) =     -0.03801d0*CON_KCALTOK
      tclocal(18) =    -38.29857d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.67816d0*CON_KCALTOK
      tclocal(1) =      2.84726d0*CON_KCALTOK
      tclocal(2) =     -0.01803d0*CON_KCALTOK
      tclocal(3) =     -9.30859d0*CON_KCALTOK
      tclocal(4) =      3.94229d0*CON_KCALTOK
      tclocal(5) =      1.91241d0*CON_KCALTOK
      tclocal(6) =      6.13275d0*CON_KCALTOK
      tclocal(7) =      3.93606d0*CON_KCALTOK
      tclocal(8) =     -9.21662d0*CON_KCALTOK
      tclocal( 9) =      2.05940d0*CON_KCALTOK
      tclocal(10) =     -3.19862d0*CON_KCALTOK
      tclocal(11) =     -3.03472d0*CON_KCALTOK
      tclocal(12) =     -4.95510d0*CON_KCALTOK
      tclocal(13) =     -2.48274d0*CON_KCALTOK
      tclocal(14) =     -2.05321d0*CON_KCALTOK
      tclocal(15) =     -0.52968d0*CON_KCALTOK
      tclocal(16) =      0.23761d0*CON_KCALTOK
      tclocal(17) =     -1.23331d0*CON_KCALTOK
      tclocal(18) =    -18.82354d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o` ')
      call twh_torstring(GLB_SET,type,2,'c` ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o` '
      tnlocal(nsame,2) = 'c` '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.00936d0*CON_KCALTOK
      tclocal(1) =      1.70532d0*CON_KCALTOK
      tclocal(2) =     -0.38747d0*CON_KCALTOK
      tclocal(3) =     -2.82176d0*CON_KCALTOK
      tclocal(4) =     -1.93785d0*CON_KCALTOK
      tclocal(5) =     -0.17983d0*CON_KCALTOK
      tclocal(6) =     -3.35812d0*CON_KCALTOK
      tclocal(7) =      3.64415d0*CON_KCALTOK
      tclocal(8) =     -1.67515d0*CON_KCALTOK
      tclocal( 9) =     -1.90366d0*CON_KCALTOK
      tclocal(10) =     -0.54110d0*CON_KCALTOK
      tclocal(11) =     -0.19951d0*CON_KCALTOK
      tclocal(12) =      6.88745d0*CON_KCALTOK
      tclocal(13) =      2.71094d0*CON_KCALTOK
      tclocal(14) =     -1.02527d0*CON_KCALTOK
      tclocal(15) =      9.77987d0*CON_KCALTOK
      tclocal(16) =     -0.25275d0*CON_KCALTOK
      tclocal(17) =     -0.01916d0*CON_KCALTOK
      tclocal(18) =    -21.32546d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c+ ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c+ '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.44354d0*CON_KCALTOK
      tclocal(1) =      2.21240d0*CON_KCALTOK
      tclocal(2) =     -0.20310d0*CON_KCALTOK
      tclocal(3) =     -1.54661d0*CON_KCALTOK
      tclocal(4) =     -2.40168d0*CON_KCALTOK
      tclocal(5) =      0.59305d0*CON_KCALTOK
      tclocal(6) =     -1.01915d0*CON_KCALTOK
      tclocal(7) =      8.74516d0*CON_KCALTOK
      tclocal(8) =     -1.25453d0*CON_KCALTOK
      tclocal( 9) =      3.09778d0*CON_KCALTOK
      tclocal(10) =     -0.67368d0*CON_KCALTOK
      tclocal(11) =      0.03802d0*CON_KCALTOK
      tclocal(12) =      4.09996d0*CON_KCALTOK
      tclocal(13) =      1.61909d0*CON_KCALTOK
      tclocal(14) =     -0.54861d0*CON_KCALTOK
      tclocal(15) =     12.24875d0*CON_KCALTOK
      tclocal(16) =     -0.53625d0*CON_KCALTOK
      tclocal(17) =      0.12384d0*CON_KCALTOK
      tclocal(18) =    -21.54217d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c+ ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c+ '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.81678d0*CON_KCALTOK
      tclocal(1) =      2.25954d0*CON_KCALTOK
      tclocal(2) =     -0.16319d0*CON_KCALTOK
      tclocal(3) =      5.85950d0*CON_KCALTOK
      tclocal(4) =      0.02587d0*CON_KCALTOK
      tclocal(5) =      0.93429d0*CON_KCALTOK
      tclocal(6) =      3.88941d0*CON_KCALTOK
      tclocal(7) =      2.51236d0*CON_KCALTOK
      tclocal(8) =      0.68458d0*CON_KCALTOK
      tclocal( 9) =      3.21242d0*CON_KCALTOK
      tclocal(10) =      0.04811d0*CON_KCALTOK
      tclocal(11) =      0.32553d0*CON_KCALTOK
      tclocal(12) =      8.35954d0*CON_KCALTOK
      tclocal(13) =      4.20463d0*CON_KCALTOK
      tclocal(14) =      0.18603d0*CON_KCALTOK
      tclocal(15) =     12.48501d0*CON_KCALTOK
      tclocal(16) =      1.37381d0*CON_KCALTOK
      tclocal(17) =     -0.04182d0*CON_KCALTOK
      tclocal(18) =    -30.64042d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c+ ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c+ '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00824d0*CON_KCALTOK
      tclocal(1) =      2.62620d0*CON_KCALTOK
      tclocal(2) =     -0.56834d0*CON_KCALTOK
      tclocal(3) =      0.02284d0*CON_KCALTOK
      tclocal(4) =     -2.80236d0*CON_KCALTOK
      tclocal(5) =     -0.54818d0*CON_KCALTOK
      tclocal(6) =     -2.55408d0*CON_KCALTOK
      tclocal(7) =     23.24663d0*CON_KCALTOK
      tclocal(8) =     -3.85276d0*CON_KCALTOK
      tclocal( 9) =      4.35964d0*CON_KCALTOK
      tclocal(10) =     -0.16637d0*CON_KCALTOK
      tclocal(11) =      0.45073d0*CON_KCALTOK
      tclocal(12) =     -9.70535d0*CON_KCALTOK
      tclocal(13) =      2.44396d0*CON_KCALTOK
      tclocal(14) =      0.86749d0*CON_KCALTOK
      tclocal(15) =      4.95434d0*CON_KCALTOK
      tclocal(16) =      6.32959d0*CON_KCALTOK
      tclocal(17) =     -0.92209d0*CON_KCALTOK
      tclocal(18) =     -5.10847d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'c+ ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'c+ '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =    -12.28892d0*CON_KCALTOK
      tclocal(1) =      5.17270d0*CON_KCALTOK
      tclocal(2) =     -0.23497d0*CON_KCALTOK
      tclocal(3) =     -1.51561d0*CON_KCALTOK
      tclocal(4) =      2.61602d0*CON_KCALTOK
      tclocal(5) =      1.16028d0*CON_KCALTOK
      tclocal(6) =     30.30022d0*CON_KCALTOK
      tclocal(7) =     25.76416d0*CON_KCALTOK
      tclocal(8) =     -3.18613d0*CON_KCALTOK
      tclocal( 9) =     -5.50879d0*CON_KCALTOK
      tclocal(10) =     -7.88259d0*CON_KCALTOK
      tclocal(11) =     -0.00938d0*CON_KCALTOK
      tclocal(12) =      2.83720d0*CON_KCALTOK
      tclocal(13) =      3.79951d0*CON_KCALTOK
      tclocal(14) =     -0.28533d0*CON_KCALTOK
      tclocal(15) =     11.59289d0*CON_KCALTOK
      tclocal(16) =      2.71902d0*CON_KCALTOK
      tclocal(17) =     -4.78154d0*CON_KCALTOK
      tclocal(18) =     -2.35464d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c+ ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c+ '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -9.80043d0*CON_KCALTOK
      tclocal(1) =      3.88707d0*CON_KCALTOK
      tclocal(2) =      0.33307d0*CON_KCALTOK
      tclocal(3) =     -1.62176d0*CON_KCALTOK
      tclocal(4) =      3.62549d0*CON_KCALTOK
      tclocal(5) =      0.35593d0*CON_KCALTOK
      tclocal(6) =     32.28978d0*CON_KCALTOK
      tclocal(7) =     26.09925d0*CON_KCALTOK
      tclocal(8) =      1.68798d0*CON_KCALTOK
      tclocal( 9) =      2.93510d0*CON_KCALTOK
      tclocal(10) =      1.52587d0*CON_KCALTOK
      tclocal(11) =      1.39755d0*CON_KCALTOK
      tclocal(12) =      0.88276d0*CON_KCALTOK
      tclocal(13) =      7.22135d0*CON_KCALTOK
      tclocal(14) =      0.88212d0*CON_KCALTOK
      tclocal(15) =      7.79909d0*CON_KCALTOK
      tclocal(16) =      7.20407d0*CON_KCALTOK
      tclocal(17) =     -3.50507d0*CON_KCALTOK
      tclocal(18) =     -4.66933d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c+ ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c+ '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -8.09353d0*CON_KCALTOK
      tclocal(1) =      2.90331d0*CON_KCALTOK
      tclocal(2) =      0.29459d0*CON_KCALTOK
      tclocal(3) =     -5.78335d0*CON_KCALTOK
      tclocal(4) =     -5.07301d0*CON_KCALTOK
      tclocal(5) =      1.39122d0*CON_KCALTOK
      tclocal(6) =     11.44428d0*CON_KCALTOK
      tclocal(7) =     14.17007d0*CON_KCALTOK
      tclocal(8) =     -4.83160d0*CON_KCALTOK
      tclocal( 9) =     -3.04808d0*CON_KCALTOK
      tclocal(10) =     -3.54978d0*CON_KCALTOK
      tclocal(11) =      0.92278d0*CON_KCALTOK
      tclocal(12) =     -6.67330d0*CON_KCALTOK
      tclocal(13) =      5.53099d0*CON_KCALTOK
      tclocal(14) =      1.78429d0*CON_KCALTOK
      tclocal(15) =     10.07692d0*CON_KCALTOK
      tclocal(16) =      3.43222d0*CON_KCALTOK
      tclocal(17) =     -1.08390d0*CON_KCALTOK
      tclocal(18) =    -15.03406d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c+ ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c+ '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -6.30251d0*CON_KCALTOK
      tclocal(1) =      2.76186d0*CON_KCALTOK
      tclocal(2) =     -0.08261d0*CON_KCALTOK
      tclocal(3) =     -8.01180d0*CON_KCALTOK
      tclocal(4) =     -4.83644d0*CON_KCALTOK
      tclocal(5) =      0.44751d0*CON_KCALTOK
      tclocal(6) =     16.81131d0*CON_KCALTOK
      tclocal(7) =      8.41311d0*CON_KCALTOK
      tclocal(8) =     -2.24903d0*CON_KCALTOK
      tclocal( 9) =      3.00943d0*CON_KCALTOK
      tclocal(10) =     -0.61377d0*CON_KCALTOK
      tclocal(11) =     -0.06435d0*CON_KCALTOK
      tclocal(12) =    -10.56170d0*CON_KCALTOK
      tclocal(13) =      0.39104d0*CON_KCALTOK
      tclocal(14) =     -0.52309d0*CON_KCALTOK
      tclocal(15) =      1.61596d0*CON_KCALTOK
      tclocal(16) =      0.63774d0*CON_KCALTOK
      tclocal(17) =     -0.33406d0*CON_KCALTOK
      tclocal(18) =     -5.63999d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c+ ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c+ '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10000d0*CON_KCALTOK
      tclocal(1) =      6.04593d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =     -0.68792d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =     -0.63258d0*CON_KCALTOK
      tclocal(6) =     -0.22082d0*CON_KCALTOK
      tclocal(7) =      6.48463d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =     -0.68792d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =     -0.63258d0*CON_KCALTOK
      tclocal(12) =     -5.11283d0*CON_KCALTOK
      tclocal(13) =      3.00114d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =     -5.11283d0*CON_KCALTOK
      tclocal(16) =      3.00114d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -6.41922d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'c= ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'c= '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.05784d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =     -0.25593d0*CON_KCALTOK
      tclocal(4) =      0.78723d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      1.30470d0*CON_KCALTOK
      tclocal(7) =      7.05455d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.62863d0*CON_KCALTOK
      tclocal(10) =     -0.18731d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     -6.28865d0*CON_KCALTOK
      tclocal(13) =      1.71294d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =     -1.76464d0*CON_KCALTOK
      tclocal(16) =      4.31538d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -8.94327d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'c= ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'c= '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      5.69470d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.82896d0*CON_KCALTOK
      tclocal(4) =      0.60009d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.99515d0*CON_KCALTOK
      tclocal(7) =      7.43152d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.82896d0*CON_KCALTOK
      tclocal(10) =      0.60009d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     -2.19894d0*CON_KCALTOK
      tclocal(13) =      3.78372d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =     -2.19894d0*CON_KCALTOK
      tclocal(16) =      3.78372d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -8.14626d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'c= ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'c= '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.21439d0*CON_KCALTOK
      tclocal(1) =      1.61338d0*CON_KCALTOK
      tclocal(2) =     -0.13798d0*CON_KCALTOK
      tclocal(3) =     -1.07670d0*CON_KCALTOK
      tclocal(4) =     -0.59898d0*CON_KCALTOK
      tclocal(5) =     -0.02270d0*CON_KCALTOK
      tclocal(6) =      1.08406d0*CON_KCALTOK
      tclocal(7) =      2.05187d0*CON_KCALTOK
      tclocal(8) =     -0.20651d0*CON_KCALTOK
      tclocal( 9) =     -1.20820d0*CON_KCALTOK
      tclocal(10) =     -0.92894d0*CON_KCALTOK
      tclocal(11) =      0.16331d0*CON_KCALTOK
      tclocal(12) =      8.82004d0*CON_KCALTOK
      tclocal(13) =      3.16536d0*CON_KCALTOK
      tclocal(14) =     -0.29839d0*CON_KCALTOK
      tclocal(15) =     10.85080d0*CON_KCALTOK
      tclocal(16) =      1.07019d0*CON_KCALTOK
      tclocal(17) =      0.22285d0*CON_KCALTOK
      tclocal(18) =    -25.58006d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.21002d0*CON_KCALTOK
      tclocal(1) =      2.25378d0*CON_KCALTOK
      tclocal(2) =     -0.17072d0*CON_KCALTOK
      tclocal(3) =     -0.76264d0*CON_KCALTOK
      tclocal(4) =      1.18901d0*CON_KCALTOK
      tclocal(5) =     -0.12822d0*CON_KCALTOK
      tclocal(6) =      1.02629d0*CON_KCALTOK
      tclocal(7) =      4.25450d0*CON_KCALTOK
      tclocal(8) =     -0.26522d0*CON_KCALTOK
      tclocal( 9) =      0.71396d0*CON_KCALTOK
      tclocal(10) =      0.25461d0*CON_KCALTOK
      tclocal(11) =     -0.41087d0*CON_KCALTOK
      tclocal(12) =      8.93453d0*CON_KCALTOK
      tclocal(13) =      2.66308d0*CON_KCALTOK
      tclocal(14) =      0.21598d0*CON_KCALTOK
      tclocal(15) =     10.68895d0*CON_KCALTOK
      tclocal(16) =      1.16896d0*CON_KCALTOK
      tclocal(17) =      0.10121d0*CON_KCALTOK
      tclocal(18) =    -35.94460d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.79198d0*CON_KCALTOK
      tclocal(1) =      2.03927d0*CON_KCALTOK
      tclocal(2) =      0.20505d0*CON_KCALTOK
      tclocal(3) =      3.97147d0*CON_KCALTOK
      tclocal(4) =      2.43950d0*CON_KCALTOK
      tclocal(5) =      0.90416d0*CON_KCALTOK
      tclocal(6) =      2.89399d0*CON_KCALTOK
      tclocal(7) =      2.92317d0*CON_KCALTOK
      tclocal(8) =      0.70471d0*CON_KCALTOK
      tclocal( 9) =      1.56017d0*CON_KCALTOK
      tclocal(10) =      0.15211d0*CON_KCALTOK
      tclocal(11) =      0.18856d0*CON_KCALTOK
      tclocal(12) =      8.26106d0*CON_KCALTOK
      tclocal(13) =      5.27280d0*CON_KCALTOK
      tclocal(14) =      1.23979d0*CON_KCALTOK
      tclocal(15) =     11.63603d0*CON_KCALTOK
      tclocal(16) =      2.29363d0*CON_KCALTOK
      tclocal(17) =      0.79864d0*CON_KCALTOK
      tclocal(18) =    -43.90800d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.97980d0*CON_KCALTOK
      tclocal(1) =      0.53212d0*CON_KCALTOK
      tclocal(2) =     -0.14718d0*CON_KCALTOK
      tclocal(3) =     -1.75625d0*CON_KCALTOK
      tclocal(4) =     -1.34660d0*CON_KCALTOK
      tclocal(5) =      0.08566d0*CON_KCALTOK
      tclocal(6) =     -1.10355d0*CON_KCALTOK
      tclocal(7) =      0.24859d0*CON_KCALTOK
      tclocal(8) =     -0.80991d0*CON_KCALTOK
      tclocal( 9) =     -3.79235d0*CON_KCALTOK
      tclocal(10) =     -0.52828d0*CON_KCALTOK
      tclocal(11) =     -0.63717d0*CON_KCALTOK
      tclocal(12) =      0.57613d0*CON_KCALTOK
      tclocal(13) =     -0.08398d0*CON_KCALTOK
      tclocal(14) =     -0.75242d0*CON_KCALTOK
      tclocal(15) =      2.07491d0*CON_KCALTOK
      tclocal(16) =     -0.47615d0*CON_KCALTOK
      tclocal(17) =     -0.46946d0*CON_KCALTOK
      tclocal(18) =    -11.88265d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n= ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n= '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.01077d0*CON_KCALTOK
      tclocal(1) =      0.98568d0*CON_KCALTOK
      tclocal(2) =     -0.01796d0*CON_KCALTOK
      tclocal(3) =     -1.96837d0*CON_KCALTOK
      tclocal(4) =     -0.21438d0*CON_KCALTOK
      tclocal(5) =      0.56690d0*CON_KCALTOK
      tclocal(6) =     -2.06986d0*CON_KCALTOK
      tclocal(7) =      3.60951d0*CON_KCALTOK
      tclocal(8) =      0.63719d0*CON_KCALTOK
      tclocal( 9) =      1.80319d0*CON_KCALTOK
      tclocal(10) =     -0.11515d0*CON_KCALTOK
      tclocal(11) =      0.33902d0*CON_KCALTOK
      tclocal(12) =      1.96317d0*CON_KCALTOK
      tclocal(13) =      0.24017d0*CON_KCALTOK
      tclocal(14) =     -0.59759d0*CON_KCALTOK
      tclocal(15) =      0.18981d0*CON_KCALTOK
      tclocal(16) =     -0.05343d0*CON_KCALTOK
      tclocal(17) =     -0.32525d0*CON_KCALTOK
      tclocal(18) =    -14.46820d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n= ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n= '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.89787d0*CON_KCALTOK
      tclocal(1) =      0.34847d0*CON_KCALTOK
      tclocal(2) =     -0.17392d0*CON_KCALTOK
      tclocal(3) =      3.73043d0*CON_KCALTOK
      tclocal(4) =      0.00825d0*CON_KCALTOK
      tclocal(5) =     -0.01687d0*CON_KCALTOK
      tclocal(6) =     -1.13079d0*CON_KCALTOK
      tclocal(7) =      2.01710d0*CON_KCALTOK
      tclocal(8) =     -0.86291d0*CON_KCALTOK
      tclocal( 9) =     -1.82011d0*CON_KCALTOK
      tclocal(10) =      0.25922d0*CON_KCALTOK
      tclocal(11) =     -0.25610d0*CON_KCALTOK
      tclocal(12) =      0.32215d0*CON_KCALTOK
      tclocal(13) =      1.59121d0*CON_KCALTOK
      tclocal(14) =     -0.77428d0*CON_KCALTOK
      tclocal(15) =     -1.28234d0*CON_KCALTOK
      tclocal(16) =      0.48030d0*CON_KCALTOK
      tclocal(17) =     -0.64439d0*CON_KCALTOK
      tclocal(18) =    -11.02650d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n= ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n= '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -8.47771d0*CON_KCALTOK
      tclocal(1) =      0.44253d0*CON_KCALTOK
      tclocal(2) =      0.05557d0*CON_KCALTOK
      tclocal(3) =     -2.75821d0*CON_KCALTOK
      tclocal(4) =     -0.14443d0*CON_KCALTOK
      tclocal(5) =      0.04143d0*CON_KCALTOK
      tclocal(6) =     -1.70059d0*CON_KCALTOK
      tclocal(7) =      7.66589d0*CON_KCALTOK
      tclocal(8) =     -0.44210d0*CON_KCALTOK
      tclocal( 9) =      3.51636d0*CON_KCALTOK
      tclocal(10) =      1.03891d0*CON_KCALTOK
      tclocal(11) =      0.61149d0*CON_KCALTOK
      tclocal(12) =     -7.59337d0*CON_KCALTOK
      tclocal(13) =      1.47643d0*CON_KCALTOK
      tclocal(14) =      0.47963d0*CON_KCALTOK
      tclocal(15) =      2.82291d0*CON_KCALTOK
      tclocal(16) =     -0.23005d0*CON_KCALTOK
      tclocal(17) =      0.05355d0*CON_KCALTOK
      tclocal(18) =    -11.41657d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.40226d0*CON_KCALTOK
      tclocal(1) =      2.54983d0*CON_KCALTOK
      tclocal(2) =     -0.36614d0*CON_KCALTOK
      tclocal(3) =     -3.00317d0*CON_KCALTOK
      tclocal(4) =      0.53106d0*CON_KCALTOK
      tclocal(5) =      0.39141d0*CON_KCALTOK
      tclocal(6) =     -2.08033d0*CON_KCALTOK
      tclocal(7) =      8.15867d0*CON_KCALTOK
      tclocal(8) =     -0.28012d0*CON_KCALTOK
      tclocal( 9) =      1.05685d0*CON_KCALTOK
      tclocal(10) =      0.54607d0*CON_KCALTOK
      tclocal(11) =      0.12332d0*CON_KCALTOK
      tclocal(12) =     -0.44697d0*CON_KCALTOK
      tclocal(13) =      3.51153d0*CON_KCALTOK
      tclocal(14) =      0.18890d0*CON_KCALTOK
      tclocal(15) =      3.83315d0*CON_KCALTOK
      tclocal(16) =      0.59333d0*CON_KCALTOK
      tclocal(17) =     -0.53550d0*CON_KCALTOK
      tclocal(18) =    -14.29444d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -4.41192d0*CON_KCALTOK
      tclocal(1) =      2.22485d0*CON_KCALTOK
      tclocal(2) =     -0.24433d0*CON_KCALTOK
      tclocal(3) =     -2.81780d0*CON_KCALTOK
      tclocal(4) =      0.82569d0*CON_KCALTOK
      tclocal(5) =     -0.09987d0*CON_KCALTOK
      tclocal(6) =      0.84547d0*CON_KCALTOK
      tclocal(7) =      8.68198d0*CON_KCALTOK
      tclocal(8) =     -2.17757d0*CON_KCALTOK
      tclocal( 9) =      3.46024d0*CON_KCALTOK
      tclocal(10) =     -0.01280d0*CON_KCALTOK
      tclocal(11) =      0.60048d0*CON_KCALTOK
      tclocal(12) =     -5.53150d0*CON_KCALTOK
      tclocal(13) =      2.86482d0*CON_KCALTOK
      tclocal(14) =     -0.38049d0*CON_KCALTOK
      tclocal(15) =      5.72083d0*CON_KCALTOK
      tclocal(16) =      1.65283d0*CON_KCALTOK
      tclocal(17) =     -0.89326d0*CON_KCALTOK
      tclocal(18) =     -9.01529d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.58032d0*CON_KCALTOK
      tclocal(1) =      1.01242d0*CON_KCALTOK
      tclocal(2) =     -0.22005d0*CON_KCALTOK
      tclocal(3) =     -7.82667d0*CON_KCALTOK
      tclocal(4) =     -0.45270d0*CON_KCALTOK
      tclocal(5) =      0.56834d0*CON_KCALTOK
      tclocal(6) =     -2.33785d0*CON_KCALTOK
      tclocal(7) =      4.55238d0*CON_KCALTOK
      tclocal(8) =     -0.84820d0*CON_KCALTOK
      tclocal( 9) =      0.28936d0*CON_KCALTOK
      tclocal(10) =     -0.65354d0*CON_KCALTOK
      tclocal(11) =      2.04920d0*CON_KCALTOK
      tclocal(12) =    -10.36781d0*CON_KCALTOK
      tclocal(13) =      1.68668d0*CON_KCALTOK
      tclocal(14) =     -0.87337d0*CON_KCALTOK
      tclocal(15) =     -1.68774d0*CON_KCALTOK
      tclocal(16) =     -1.00930d0*CON_KCALTOK
      tclocal(17) =     -0.17021d0*CON_KCALTOK
      tclocal(18) =     -9.74403d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10853d0*CON_KCALTOK
      tclocal(1) =      1.92105d0*CON_KCALTOK
      tclocal(2) =     -0.42317d0*CON_KCALTOK
      tclocal(3) =     -5.91165d0*CON_KCALTOK
      tclocal(4) =     -2.51211d0*CON_KCALTOK
      tclocal(5) =      1.83806d0*CON_KCALTOK
      tclocal(6) =      1.99900d0*CON_KCALTOK
      tclocal(7) =      8.33552d0*CON_KCALTOK
      tclocal(8) =      1.96232d0*CON_KCALTOK
      tclocal( 9) =     -0.01241d0*CON_KCALTOK
      tclocal(10) =      1.35250d0*CON_KCALTOK
      tclocal(11) =      0.51766d0*CON_KCALTOK
      tclocal(12) =      2.88770d0*CON_KCALTOK
      tclocal(13) =      4.99649d0*CON_KCALTOK
      tclocal(14) =      4.37804d0*CON_KCALTOK
      tclocal(15) =     -1.01466d0*CON_KCALTOK
      tclocal(16) =      1.16042d0*CON_KCALTOK
      tclocal(17) =      0.62592d0*CON_KCALTOK
      tclocal(18) =    -22.91122d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n= ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n= '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -4.93740d0*CON_KCALTOK
      tclocal(1) =      1.68152d0*CON_KCALTOK
      tclocal(2) =      0.00859d0*CON_KCALTOK
      tclocal(3) =     -5.76285d0*CON_KCALTOK
      tclocal(4) =     -3.54590d0*CON_KCALTOK
      tclocal(5) =      0.25265d0*CON_KCALTOK
      tclocal(6) =     -0.26939d0*CON_KCALTOK
      tclocal(7) =      4.80301d0*CON_KCALTOK
      tclocal(8) =      0.73766d0*CON_KCALTOK
      tclocal( 9) =      3.36479d0*CON_KCALTOK
      tclocal(10) =      0.31910d0*CON_KCALTOK
      tclocal(11) =     -0.47241d0*CON_KCALTOK
      tclocal(12) =     -9.00108d0*CON_KCALTOK
      tclocal(13) =     -0.17846d0*CON_KCALTOK
      tclocal(14) =      0.58954d0*CON_KCALTOK
      tclocal(15) =      0.73007d0*CON_KCALTOK
      tclocal(16) =      1.25547d0*CON_KCALTOK
      tclocal(17) =      0.00130d0*CON_KCALTOK
      tclocal(18) =     -7.94471d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n= ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n= '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.38748d0*CON_KCALTOK
      tclocal(1) =      6.57015d0*CON_KCALTOK
      tclocal(2) =     -1.05406d0*CON_KCALTOK
      tclocal(3) =     -3.87097d0*CON_KCALTOK
      tclocal(4) =     -5.03402d0*CON_KCALTOK
      tclocal(5) =      0.23370d0*CON_KCALTOK
      tclocal(6) =      0.81559d0*CON_KCALTOK
      tclocal(7) =     12.10252d0*CON_KCALTOK
      tclocal(8) =     -1.13911d0*CON_KCALTOK
      tclocal( 9) =      1.64346d0*CON_KCALTOK
      tclocal(10) =      5.40165d0*CON_KCALTOK
      tclocal(11) =      0.22791d0*CON_KCALTOK
      tclocal(12) =     -3.87738d0*CON_KCALTOK
      tclocal(13) =      3.17298d0*CON_KCALTOK
      tclocal(14) =     -2.60400d0*CON_KCALTOK
      tclocal(15) =      0.49393d0*CON_KCALTOK
      tclocal(16) =      1.77397d0*CON_KCALTOK
      tclocal(17) =     -0.41862d0*CON_KCALTOK
      tclocal(18) =    -14.39783d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.67578d0*CON_KCALTOK
      tclocal(1) =      8.39441d0*CON_KCALTOK
      tclocal(2) =     -1.22703d0*CON_KCALTOK
      tclocal(3) =     -6.08434d0*CON_KCALTOK
      tclocal(4) =     -1.78247d0*CON_KCALTOK
      tclocal(5) =      1.61136d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     12.64635d0*CON_KCALTOK
      tclocal(8) =      1.04937d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      4.88192d0*CON_KCALTOK
      tclocal(11) =     -0.98750d0*CON_KCALTOK
      tclocal(12) =      0.35372d0*CON_KCALTOK
      tclocal(13) =      6.88012d0*CON_KCALTOK
      tclocal(14) =     -1.39856d0*CON_KCALTOK
      tclocal(15) =      2.95601d0*CON_KCALTOK
      tclocal(16) =      1.16811d0*CON_KCALTOK
      tclocal(17) =     -1.02501d0*CON_KCALTOK
      tclocal(18) =     -8.27703d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.40393d0*CON_KCALTOK
      tclocal(1) =      7.33486d0*CON_KCALTOK
      tclocal(2) =     -0.99587d0*CON_KCALTOK
      tclocal(3) =     -4.13825d0*CON_KCALTOK
      tclocal(4) =     -1.16706d0*CON_KCALTOK
      tclocal(5) =      0.68150d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     12.00590d0*CON_KCALTOK
      tclocal(8) =      1.42291d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      5.96542d0*CON_KCALTOK
      tclocal(11) =      0.42874d0*CON_KCALTOK
      tclocal(12) =     -6.05070d0*CON_KCALTOK
      tclocal(13) =      2.72751d0*CON_KCALTOK
      tclocal(14) =      0.40815d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      3.17692d0*CON_KCALTOK
      tclocal(17) =      1.37183d0*CON_KCALTOK
      tclocal(18) =     -6.87187d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.62698d0*CON_KCALTOK
      tclocal(1) =      7.87414d0*CON_KCALTOK
      tclocal(2) =      0.35306d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -7.66482d0*CON_KCALTOK
      tclocal(5) =     -3.05325d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      2.77177d0*CON_KCALTOK
      tclocal(8) =     -0.34542d0*CON_KCALTOK
      tclocal( 9) =     -0.84856d0*CON_KCALTOK
      tclocal(10) =      8.22026d0*CON_KCALTOK
      tclocal(11) =     -0.83422d0*CON_KCALTOK
      tclocal(12) =      1.40410d0*CON_KCALTOK
      tclocal(13) =     11.09672d0*CON_KCALTOK
      tclocal(14) =     -0.73502d0*CON_KCALTOK
      tclocal(15) =     -0.94546d0*CON_KCALTOK
      tclocal(16) =      6.24907d0*CON_KCALTOK
      tclocal(17) =      3.73888d0*CON_KCALTOK
      tclocal(18) =    -14.90716d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.55057d0*CON_KCALTOK
      tclocal(1) =      6.82738d0*CON_KCALTOK
      tclocal(2) =     -0.61629d0*CON_KCALTOK
      tclocal(3) =    -10.16013d0*CON_KCALTOK
      tclocal(4) =    -14.46213d0*CON_KCALTOK
      tclocal(5) =      2.39865d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     -3.86650d0*CON_KCALTOK
      tclocal(8) =     -0.62364d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      5.48014d0*CON_KCALTOK
      tclocal(11) =     -1.28118d0*CON_KCALTOK
      tclocal(12) =     -8.03966d0*CON_KCALTOK
      tclocal(13) =     -0.23589d0*CON_KCALTOK
      tclocal(14) =     -1.72794d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.01785d0*CON_KCALTOK
      tclocal(17) =     -1.60315d0*CON_KCALTOK
      tclocal(18) =     -6.93047d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'c= ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'c= '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.00239d0*CON_KCALTOK
      tclocal(1) =     -0.01023d0*CON_KCALTOK
      tclocal(2) =      0.00121d0*CON_KCALTOK
      tclocal(3) =     -0.18137d0*CON_KCALTOK
      tclocal(4) =     -0.16374d0*CON_KCALTOK
      tclocal(5) =     -0.11171d0*CON_KCALTOK
      tclocal(6) =     -6.82181d0*CON_KCALTOK
      tclocal(7) =     -1.52968d0*CON_KCALTOK
      tclocal(8) =     -4.32519d0*CON_KCALTOK
      tclocal( 9) =     -0.11876d0*CON_KCALTOK
      tclocal(10) =     -0.01053d0*CON_KCALTOK
      tclocal(11) =     -0.13512d0*CON_KCALTOK
      tclocal(12) =     -0.41651d0*CON_KCALTOK
      tclocal(13) =      2.57106d0*CON_KCALTOK
      tclocal(14) =      0.39667d0*CON_KCALTOK
      tclocal(15) =      1.87362d0*CON_KCALTOK
      tclocal(16) =      2.99021d0*CON_KCALTOK
      tclocal(17) =     -0.22314d0*CON_KCALTOK
      tclocal(18) =     -2.54332d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.00239d0*CON_KCALTOK
      tclocal(1) =     -0.01023d0*CON_KCALTOK
      tclocal(2) =      0.00121d0*CON_KCALTOK
      tclocal(3) =     -0.18137d0*CON_KCALTOK
      tclocal(4) =     -0.16374d0*CON_KCALTOK
      tclocal(5) =     -0.11171d0*CON_KCALTOK
      tclocal(6) =     -6.82181d0*CON_KCALTOK
      tclocal(7) =     -1.52968d0*CON_KCALTOK
      tclocal(8) =     -4.32519d0*CON_KCALTOK
      tclocal( 9) =     -0.11876d0*CON_KCALTOK
      tclocal(10) =     -0.01053d0*CON_KCALTOK
      tclocal(11) =     -0.13512d0*CON_KCALTOK
      tclocal(12) =     -0.41651d0*CON_KCALTOK
      tclocal(13) =      2.57106d0*CON_KCALTOK
      tclocal(14) =      0.39667d0*CON_KCALTOK
      tclocal(15) =      1.87362d0*CON_KCALTOK
      tclocal(16) =      2.99021d0*CON_KCALTOK
      tclocal(17) =     -0.22314d0*CON_KCALTOK
      tclocal(18) =     -2.54332d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.39998d0*CON_KCALTOK
      tclocal(1) =      0.23788d0*CON_KCALTOK
      tclocal(2) =     -0.13637d0*CON_KCALTOK
      tclocal(3) =     -1.70945d0*CON_KCALTOK
      tclocal(4) =      2.28852d0*CON_KCALTOK
      tclocal(5) =      0.18644d0*CON_KCALTOK
      tclocal(6) =     -1.01155d0*CON_KCALTOK
      tclocal(7) =      3.63376d0*CON_KCALTOK
      tclocal(8) =      0.20027d0*CON_KCALTOK
      tclocal( 9) =     -1.88618d0*CON_KCALTOK
      tclocal(10) =      0.00580d0*CON_KCALTOK
      tclocal(11) =     -0.77032d0*CON_KCALTOK
      tclocal(12) =     -3.59501d0*CON_KCALTOK
      tclocal(13) =      3.17815d0*CON_KCALTOK
      tclocal(14) =      0.20686d0*CON_KCALTOK
      tclocal(15) =      0.63497d0*CON_KCALTOK
      tclocal(16) =      0.20824d0*CON_KCALTOK
      tclocal(17) =      0.55132d0*CON_KCALTOK
      tclocal(18) =    -20.83121d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.06899d0*CON_KCALTOK
      tclocal(1) =     -0.01107d0*CON_KCALTOK
      tclocal(2) =     -0.00202d0*CON_KCALTOK
      tclocal(3) =      0.19127d0*CON_KCALTOK
      tclocal(4) =      0.00214d0*CON_KCALTOK
      tclocal(5) =      0.19187d0*CON_KCALTOK
      tclocal(6) =     -5.13124d0*CON_KCALTOK
      tclocal(7) =     -4.90812d0*CON_KCALTOK
      tclocal(8) =      0.05103d0*CON_KCALTOK
      tclocal( 9) =     -0.14380d0*CON_KCALTOK
      tclocal(10) =     -0.10543d0*CON_KCALTOK
      tclocal(11) =     -0.59990d0*CON_KCALTOK
      tclocal(12) =     -4.19806d0*CON_KCALTOK
      tclocal(13) =      3.92527d0*CON_KCALTOK
      tclocal(14) =     -0.05429d0*CON_KCALTOK
      tclocal(15) =     -1.48890d0*CON_KCALTOK
      tclocal(16) =      0.52336d0*CON_KCALTOK
      tclocal(17) =     -0.35473d0*CON_KCALTOK
      tclocal(18) =     -6.71545d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.05795d0*CON_KCALTOK
      tclocal(1) =      0.05054d0*CON_KCALTOK
      tclocal(2) =     -0.23551d0*CON_KCALTOK
      tclocal(3) =     -1.95355d0*CON_KCALTOK
      tclocal(4) =      1.15916d0*CON_KCALTOK
      tclocal(5) =      0.97852d0*CON_KCALTOK
      tclocal(6) =     -3.32323d0*CON_KCALTOK
      tclocal(7) =      0.47166d0*CON_KCALTOK
      tclocal(8) =      1.39969d0*CON_KCALTOK
      tclocal( 9) =      0.18136d0*CON_KCALTOK
      tclocal(10) =      0.47024d0*CON_KCALTOK
      tclocal(11) =     -0.10791d0*CON_KCALTOK
      tclocal(12) =     -2.54063d0*CON_KCALTOK
      tclocal(13) =      1.88372d0*CON_KCALTOK
      tclocal(14) =      0.73313d0*CON_KCALTOK
      tclocal(15) =      0.51342d0*CON_KCALTOK
      tclocal(16) =     -0.57696d0*CON_KCALTOK
      tclocal(17) =     -0.28270d0*CON_KCALTOK
      tclocal(18) =    -17.73433d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.05795d0*CON_KCALTOK
      tclocal(1) =      0.05054d0*CON_KCALTOK
      tclocal(2) =     -0.23551d0*CON_KCALTOK
      tclocal(3) =     -1.95355d0*CON_KCALTOK
      tclocal(4) =      1.15916d0*CON_KCALTOK
      tclocal(5) =      0.97852d0*CON_KCALTOK
      tclocal(6) =     -3.32323d0*CON_KCALTOK
      tclocal(7) =      0.47166d0*CON_KCALTOK
      tclocal(8) =      1.39969d0*CON_KCALTOK
      tclocal( 9) =      0.18136d0*CON_KCALTOK
      tclocal(10) =      0.47024d0*CON_KCALTOK
      tclocal(11) =     -0.10791d0*CON_KCALTOK
      tclocal(12) =     -2.54063d0*CON_KCALTOK
      tclocal(13) =      1.88372d0*CON_KCALTOK
      tclocal(14) =      0.73313d0*CON_KCALTOK
      tclocal(15) =      0.51342d0*CON_KCALTOK
      tclocal(16) =     -0.57696d0*CON_KCALTOK
      tclocal(17) =     -0.28270d0*CON_KCALTOK
      tclocal(18) =    -17.73433d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.12175d0*CON_KCALTOK
      tclocal(1) =      0.11597d0*CON_KCALTOK
      tclocal(2) =      0.12603d0*CON_KCALTOK
      tclocal(3) =     -4.33865d0*CON_KCALTOK
      tclocal(4) =      0.66001d0*CON_KCALTOK
      tclocal(5) =      0.47555d0*CON_KCALTOK
      tclocal(6) =     -5.51172d0*CON_KCALTOK
      tclocal(7) =      4.88644d0*CON_KCALTOK
      tclocal(8) =      1.09217d0*CON_KCALTOK
      tclocal( 9) =     -1.07830d0*CON_KCALTOK
      tclocal(10) =      0.68847d0*CON_KCALTOK
      tclocal(11) =      0.60227d0*CON_KCALTOK
      tclocal(12) =     -1.62965d0*CON_KCALTOK
      tclocal(13) =      2.02439d0*CON_KCALTOK
      tclocal(14) =      0.53672d0*CON_KCALTOK
      tclocal(15) =      0.93247d0*CON_KCALTOK
      tclocal(16) =     -0.28430d0*CON_KCALTOK
      tclocal(17) =     -0.44237d0*CON_KCALTOK
      tclocal(18) =    -16.87473d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.02111d0*CON_KCALTOK
      tclocal(1) =     -0.11300d0*CON_KCALTOK
      tclocal(2) =     -0.02108d0*CON_KCALTOK
      tclocal(3) =     -1.30182d0*CON_KCALTOK
      tclocal(4) =      0.40951d0*CON_KCALTOK
      tclocal(5) =      0.12770d0*CON_KCALTOK
      tclocal(6) =     -1.70318d0*CON_KCALTOK
      tclocal(7) =      4.06641d0*CON_KCALTOK
      tclocal(8) =      1.17145d0*CON_KCALTOK
      tclocal( 9) =     -0.70931d0*CON_KCALTOK
      tclocal(10) =      0.23822d0*CON_KCALTOK
      tclocal(11) =      0.45004d0*CON_KCALTOK
      tclocal(12) =     -5.36544d0*CON_KCALTOK
      tclocal(13) =      2.01095d0*CON_KCALTOK
      tclocal(14) =      0.34685d0*CON_KCALTOK
      tclocal(15) =     -0.99180d0*CON_KCALTOK
      tclocal(16) =      0.52340d0*CON_KCALTOK
      tclocal(17) =      0.11419d0*CON_KCALTOK
      tclocal(18) =     -9.65067d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.64590d0*CON_KCALTOK
      tclocal(1) =      1.16973d0*CON_KCALTOK
      tclocal(2) =     -0.07104d0*CON_KCALTOK
      tclocal(3) =     -5.33313d0*CON_KCALTOK
      tclocal(4) =      3.19836d0*CON_KCALTOK
      tclocal(5) =      0.52756d0*CON_KCALTOK
      tclocal(6) =      0.09529d0*CON_KCALTOK
      tclocal(7) =      3.19412d0*CON_KCALTOK
      tclocal(8) =     -0.85667d0*CON_KCALTOK
      tclocal( 9) =      0.19344d0*CON_KCALTOK
      tclocal(10) =      1.34506d0*CON_KCALTOK
      tclocal(11) =      0.49017d0*CON_KCALTOK
      tclocal(12) =      0.65126d0*CON_KCALTOK
      tclocal(13) =      6.22583d0*CON_KCALTOK
      tclocal(14) =     -0.80922d0*CON_KCALTOK
      tclocal(15) =      7.57803d0*CON_KCALTOK
      tclocal(16) =      4.53241d0*CON_KCALTOK
      tclocal(17) =      0.48174d0*CON_KCALTOK
      tclocal(18) =    -35.68070d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.48113d0*CON_KCALTOK
      tclocal(1) =      0.94117d0*CON_KCALTOK
      tclocal(2) =     -0.42449d0*CON_KCALTOK
      tclocal(3) =     -0.90507d0*CON_KCALTOK
      tclocal(4) =      1.86513d0*CON_KCALTOK
      tclocal(5) =     -1.12255d0*CON_KCALTOK
      tclocal(6) =     -0.24922d0*CON_KCALTOK
      tclocal(7) =      6.97820d0*CON_KCALTOK
      tclocal(8) =     -4.74526d0*CON_KCALTOK
      tclocal( 9) =      1.07534d0*CON_KCALTOK
      tclocal(10) =      2.42068d0*CON_KCALTOK
      tclocal(11) =      0.86989d0*CON_KCALTOK
      tclocal(12) =     -7.34997d0*CON_KCALTOK
      tclocal(13) =      0.30806d0*CON_KCALTOK
      tclocal(14) =     -1.00238d0*CON_KCALTOK
      tclocal(15) =     -0.81168d0*CON_KCALTOK
      tclocal(16) =      3.87195d0*CON_KCALTOK
      tclocal(17) =     -1.97235d0*CON_KCALTOK
      tclocal(18) =    -19.61000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.71561d0*CON_KCALTOK
      tclocal(1) =      1.11169d0*CON_KCALTOK
      tclocal(2) =     -0.41058d0*CON_KCALTOK
      tclocal(3) =     -1.92859d0*CON_KCALTOK
      tclocal(4) =      1.38253d0*CON_KCALTOK
      tclocal(5) =     -1.49448d0*CON_KCALTOK
      tclocal(6) =     -4.03698d0*CON_KCALTOK
      tclocal(7) =      0.52235d0*CON_KCALTOK
      tclocal(8) =     -3.70155d0*CON_KCALTOK
      tclocal( 9) =      0.97535d0*CON_KCALTOK
      tclocal(10) =      1.16026d0*CON_KCALTOK
      tclocal(11) =      0.10091d0*CON_KCALTOK
      tclocal(12) =     -2.81080d0*CON_KCALTOK
      tclocal(13) =      3.93810d0*CON_KCALTOK
      tclocal(14) =     -0.54632d0*CON_KCALTOK
      tclocal(15) =     -0.00429d0*CON_KCALTOK
      tclocal(16) =      0.82862d0*CON_KCALTOK
      tclocal(17) =     -2.31982d0*CON_KCALTOK
      tclocal(18) =    -11.98408d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.27474d0*CON_KCALTOK
      tclocal(1) =      0.95879d0*CON_KCALTOK
      tclocal(2) =     -0.18197d0*CON_KCALTOK
      tclocal(3) =     -1.43620d0*CON_KCALTOK
      tclocal(4) =      1.55103d0*CON_KCALTOK
      tclocal(5) =      0.09102d0*CON_KCALTOK
      tclocal(6) =      1.47935d0*CON_KCALTOK
      tclocal(7) =      1.45855d0*CON_KCALTOK
      tclocal(8) =      3.70166d0*CON_KCALTOK
      tclocal( 9) =      1.41101d0*CON_KCALTOK
      tclocal(10) =      1.37967d0*CON_KCALTOK
      tclocal(11) =      2.29667d0*CON_KCALTOK
      tclocal(12) =     -8.68415d0*CON_KCALTOK
      tclocal(13) =      2.91383d0*CON_KCALTOK
      tclocal(14) =     -1.98980d0*CON_KCALTOK
      tclocal(15) =      0.67921d0*CON_KCALTOK
      tclocal(16) =     -1.64729d0*CON_KCALTOK
      tclocal(17) =      0.98945d0*CON_KCALTOK
      tclocal(18) =    -17.00401d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.07367d0*CON_KCALTOK
      tclocal(1) =      0.57994d0*CON_KCALTOK
      tclocal(2) =     -0.29499d0*CON_KCALTOK
      tclocal(3) =     -1.67940d0*CON_KCALTOK
      tclocal(4) =      1.83924d0*CON_KCALTOK
      tclocal(5) =      0.76945d0*CON_KCALTOK
      tclocal(6) =     -6.44436d0*CON_KCALTOK
      tclocal(7) =      1.69206d0*CON_KCALTOK
      tclocal(8) =     -3.77954d0*CON_KCALTOK
      tclocal( 9) =     -0.60139d0*CON_KCALTOK
      tclocal(10) =      1.09058d0*CON_KCALTOK
      tclocal(11) =     -0.89063d0*CON_KCALTOK
      tclocal(12) =     -4.09668d0*CON_KCALTOK
      tclocal(13) =      2.94221d0*CON_KCALTOK
      tclocal(14) =     -1.62622d0*CON_KCALTOK
      tclocal(15) =     -5.19697d0*CON_KCALTOK
      tclocal(16) =      0.04721d0*CON_KCALTOK
      tclocal(17) =     -2.02807d0*CON_KCALTOK
      tclocal(18) =    -10.02400d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.10617d0*CON_KCALTOK
      tclocal(1) =      0.13427d0*CON_KCALTOK
      tclocal(2) =      0.08377d0*CON_KCALTOK
      tclocal(3) =      1.11496d0*CON_KCALTOK
      tclocal(4) =      1.06865d0*CON_KCALTOK
      tclocal(5) =     -0.69942d0*CON_KCALTOK
      tclocal(6) =      8.97056d0*CON_KCALTOK
      tclocal(7) =      4.94847d0*CON_KCALTOK
      tclocal(8) =     -1.17653d0*CON_KCALTOK
      tclocal( 9) =      0.26539d0*CON_KCALTOK
      tclocal(10) =      2.67424d0*CON_KCALTOK
      tclocal(11) =     -0.52012d0*CON_KCALTOK
      tclocal(12) =     -0.58025d0*CON_KCALTOK
      tclocal(13) =      3.26291d0*CON_KCALTOK
      tclocal(14) =     -0.04659d0*CON_KCALTOK
      tclocal(15) =     -0.36539d0*CON_KCALTOK
      tclocal(16) =     -1.01152d0*CON_KCALTOK
      tclocal(17) =      0.11290d0*CON_KCALTOK
      tclocal(18) =    -15.29073d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.97384d0*CON_KCALTOK
      tclocal(1) =      1.09801d0*CON_KCALTOK
      tclocal(2) =     -0.08347d0*CON_KCALTOK
      tclocal(3) =     -2.09894d0*CON_KCALTOK
      tclocal(4) =      1.29925d0*CON_KCALTOK
      tclocal(5) =      0.26373d0*CON_KCALTOK
      tclocal(6) =     -4.84910d0*CON_KCALTOK
      tclocal(7) =      2.32048d0*CON_KCALTOK
      tclocal(8) =     -0.66928d0*CON_KCALTOK
      tclocal( 9) =     -1.14376d0*CON_KCALTOK
      tclocal(10) =      0.39196d0*CON_KCALTOK
      tclocal(11) =     -0.94497d0*CON_KCALTOK
      tclocal(12) =     -0.37546d0*CON_KCALTOK
      tclocal(13) =      2.53628d0*CON_KCALTOK
      tclocal(14) =     -1.07744d0*CON_KCALTOK
      tclocal(15) =     -5.38858d0*CON_KCALTOK
      tclocal(16) =      1.24010d0*CON_KCALTOK
      tclocal(17) =     -0.26176d0*CON_KCALTOK
      tclocal(18) =    -11.51811d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.80324d0*CON_KCALTOK
      tclocal(1) =      1.11412d0*CON_KCALTOK
      tclocal(2) =     -0.15299d0*CON_KCALTOK
      tclocal(3) =     -2.37351d0*CON_KCALTOK
      tclocal(4) =      1.17791d0*CON_KCALTOK
      tclocal(5) =      0.28529d0*CON_KCALTOK
      tclocal(6) =     -1.34939d0*CON_KCALTOK
      tclocal(7) =      3.18242d0*CON_KCALTOK
      tclocal(8) =     -2.09923d0*CON_KCALTOK
      tclocal( 9) =      0.49223d0*CON_KCALTOK
      tclocal(10) =      1.00109d0*CON_KCALTOK
      tclocal(11) =      0.13732d0*CON_KCALTOK
      tclocal(12) =     -4.94310d0*CON_KCALTOK
      tclocal(13) =      2.35631d0*CON_KCALTOK
      tclocal(14) =     -0.14172d0*CON_KCALTOK
      tclocal(15) =      0.04528d0*CON_KCALTOK
      tclocal(16) =      0.26076d0*CON_KCALTOK
      tclocal(17) =     -1.53223d0*CON_KCALTOK
      tclocal(18) =    -13.43683d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.85478d0*CON_KCALTOK
      tclocal(1) =      1.13530d0*CON_KCALTOK
      tclocal(2) =     -0.20647d0*CON_KCALTOK
      tclocal(3) =     -3.11497d0*CON_KCALTOK
      tclocal(4) =      3.43115d0*CON_KCALTOK
      tclocal(5) =     -0.99350d0*CON_KCALTOK
      tclocal(6) =     -0.60253d0*CON_KCALTOK
      tclocal(7) =      4.68167d0*CON_KCALTOK
      tclocal(8) =     -0.42129d0*CON_KCALTOK
      tclocal( 9) =     -3.24942d0*CON_KCALTOK
      tclocal(10) =      5.71054d0*CON_KCALTOK
      tclocal(11) =     -1.05292d0*CON_KCALTOK
      tclocal(12) =      1.13451d0*CON_KCALTOK
      tclocal(13) =      8.15928d0*CON_KCALTOK
      tclocal(14) =     -1.13008d0*CON_KCALTOK
      tclocal(15) =      0.44924d0*CON_KCALTOK
      tclocal(16) =      5.17031d0*CON_KCALTOK
      tclocal(17) =     -1.23209d0*CON_KCALTOK
      tclocal(18) =    -27.23502d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.69614d0*CON_KCALTOK
      tclocal(1) =      0.27448d0*CON_KCALTOK
      tclocal(2) =      0.40301d0*CON_KCALTOK
      tclocal(3) =     -3.16452d0*CON_KCALTOK
      tclocal(4) =      3.58762d0*CON_KCALTOK
      tclocal(5) =      1.45706d0*CON_KCALTOK
      tclocal(6) =      1.12380d0*CON_KCALTOK
      tclocal(7) =     -1.07561d0*CON_KCALTOK
      tclocal(8) =      5.00554d0*CON_KCALTOK
      tclocal( 9) =      1.20207d0*CON_KCALTOK
      tclocal(10) =     -0.26222d0*CON_KCALTOK
      tclocal(11) =      0.63994d0*CON_KCALTOK
      tclocal(12) =     -8.53521d0*CON_KCALTOK
      tclocal(13) =      7.81714d0*CON_KCALTOK
      tclocal(14) =      9.22153d0*CON_KCALTOK
      tclocal(15) =     -0.50182d0*CON_KCALTOK
      tclocal(16) =      0.48102d0*CON_KCALTOK
      tclocal(17) =     -0.49422d0*CON_KCALTOK
      tclocal(18) =    -18.80590d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      2.42163d0*CON_KCALTOK
      tclocal(1) =      0.85347d0*CON_KCALTOK
      tclocal(2) =     -0.45939d0*CON_KCALTOK
      tclocal(3) =     -1.65479d0*CON_KCALTOK
      tclocal(4) =      3.03460d0*CON_KCALTOK
      tclocal(5) =     -1.11499d0*CON_KCALTOK
      tclocal(6) =      3.59293d0*CON_KCALTOK
      tclocal(7) =     -1.72277d0*CON_KCALTOK
      tclocal(8) =     -1.43319d0*CON_KCALTOK
      tclocal( 9) =      1.10526d0*CON_KCALTOK
      tclocal(10) =      0.48055d0*CON_KCALTOK
      tclocal(11) =     -0.09946d0*CON_KCALTOK
      tclocal(12) =      1.29753d0*CON_KCALTOK
      tclocal(13) =      3.08020d0*CON_KCALTOK
      tclocal(14) =     -0.89525d0*CON_KCALTOK
      tclocal(15) =      4.90388d0*CON_KCALTOK
      tclocal(16) =      2.07030d0*CON_KCALTOK
      tclocal(17) =     -0.98435d0*CON_KCALTOK
      tclocal(18) =    -14.24908d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.97612d0*CON_KCALTOK
      tclocal(1) =     -1.87608d0*CON_KCALTOK
      tclocal(2) =     -0.68100d0*CON_KCALTOK
      tclocal(3) =     -3.31172d0*CON_KCALTOK
      tclocal(4) =      2.44460d0*CON_KCALTOK
      tclocal(5) =     -0.24679d0*CON_KCALTOK
      tclocal(6) =      7.48383d0*CON_KCALTOK
      tclocal(7) =      0.56025d0*CON_KCALTOK
      tclocal(8) =      0.76830d0*CON_KCALTOK
      tclocal( 9) =     -2.67943d0*CON_KCALTOK
      tclocal(10) =      2.19247d0*CON_KCALTOK
      tclocal(11) =     -0.04152d0*CON_KCALTOK
      tclocal(12) =     -5.58321d0*CON_KCALTOK
      tclocal(13) =      2.65803d0*CON_KCALTOK
      tclocal(14) =     -0.45838d0*CON_KCALTOK
      tclocal(15) =      2.04411d0*CON_KCALTOK
      tclocal(16) =     -3.64589d0*CON_KCALTOK
      tclocal(17) =     -1.31690d0*CON_KCALTOK
      tclocal(18) =    -14.01629d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      2.32048d0*CON_KCALTOK
      tclocal(1) =      1.12161d0*CON_KCALTOK
      tclocal(2) =     -0.43615d0*CON_KCALTOK
      tclocal(3) =     -0.97763d0*CON_KCALTOK
      tclocal(4) =     -0.70391d0*CON_KCALTOK
      tclocal(5) =      0.80803d0*CON_KCALTOK
      tclocal(6) =      4.27090d0*CON_KCALTOK
      tclocal(7) =      0.38116d0*CON_KCALTOK
      tclocal(8) =     -2.43469d0*CON_KCALTOK
      tclocal( 9) =     -0.97763d0*CON_KCALTOK
      tclocal(10) =     -0.70391d0*CON_KCALTOK
      tclocal(11) =      0.80803d0*CON_KCALTOK
      tclocal(12) =     12.36587d0*CON_KCALTOK
      tclocal(13) =      6.19038d0*CON_KCALTOK
      tclocal(14) =     -0.23840d0*CON_KCALTOK
      tclocal(15) =      3.51252d0*CON_KCALTOK
      tclocal(16) =      4.09308d0*CON_KCALTOK
      tclocal(17) =     -1.20099d0*CON_KCALTOK
      tclocal(18) =    -46.20268d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.77376d0*CON_KCALTOK
      tclocal(1) =      0.02646d0*CON_KCALTOK
      tclocal(2) =     -0.26321d0*CON_KCALTOK
      tclocal(3) =     -4.38910d0*CON_KCALTOK
      tclocal(4) =      3.36266d0*CON_KCALTOK
      tclocal(5) =      2.06685d0*CON_KCALTOK
      tclocal(6) =      3.44569d0*CON_KCALTOK
      tclocal(7) =     -1.07003d0*CON_KCALTOK
      tclocal(8) =      0.68253d0*CON_KCALTOK
      tclocal( 9) =     -0.37726d0*CON_KCALTOK
      tclocal(10) =      1.13543d0*CON_KCALTOK
      tclocal(11) =      0.91186d0*CON_KCALTOK
      tclocal(12) =     -6.61810d0*CON_KCALTOK
      tclocal(13) =      5.25793d0*CON_KCALTOK
      tclocal(14) =      2.41267d0*CON_KCALTOK
      tclocal(15) =      2.30478d0*CON_KCALTOK
      tclocal(16) =      0.10063d0*CON_KCALTOK
      tclocal(17) =     -0.12577d0*CON_KCALTOK
      tclocal(18) =    -13.69961d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'co ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'co '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.12100d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     40.04594d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      3.64818d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      3.64818d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     14.02144d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.47648d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -1.99379d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     21.64678d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.08469d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      3.26633d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      1.44955d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'c" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'c" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.22258d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.35896d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     21.70867d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      3.73393d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      5.65586d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      3.74663d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      2.62573d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.59986d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.46242d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     16.79016d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.90517d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      1.61304d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.82743d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      7.34848d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'c+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'c+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.85890d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.97489d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     21.64226d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      1.19442d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      8.60336d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      4.28355d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -5.16565d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'c= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'c= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.98674d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     32.78031d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      6.24603d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     11.80544d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -8.10993d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cl ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cl '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.71443d0*CON_KCALTOK
      tclocal(1) =      6.83285d0*CON_KCALTOK
      tclocal(2) =      0.76449d0*CON_KCALTOK
      tclocal(3) =      2.54188d0*CON_KCALTOK
      tclocal(4) =      7.23685d0*CON_KCALTOK
      tclocal(5) =      0.87202d0*CON_KCALTOK
      tclocal(6) =    -24.16416d0*CON_KCALTOK
      tclocal(7) =     -2.63892d0*CON_KCALTOK
      tclocal(8) =    -11.64094d0*CON_KCALTOK
      tclocal( 9) =      7.19686d0*CON_KCALTOK
      tclocal(10) =     -4.64183d0*CON_KCALTOK
      tclocal(11) =     -0.02401d0*CON_KCALTOK
      tclocal(12) =     16.32022d0*CON_KCALTOK
      tclocal(13) =      5.63057d0*CON_KCALTOK
      tclocal(14) =      0.50302d0*CON_KCALTOK
      tclocal(15) =     -7.88123d0*CON_KCALTOK
      tclocal(16) =     11.86218d0*CON_KCALTOK
      tclocal(17) =      8.88582d0*CON_KCALTOK
      tclocal(18) =    -15.89423d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.09129d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     39.66542d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      9.76253d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     13.74885d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -11.26851d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'f  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'f  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.26088d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.01130d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     16.59824d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -1.20005d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      5.73742d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      3.29617d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -5.65222d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.66603d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      2.44380d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     23.40850d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -8.31630d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     19.22244d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     12.91863d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -35.73311d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.11980d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -5.84549d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     23.28540d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.82750d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -3.52415d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     10.89481d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -7.02133d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'n` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'n` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.57871d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -2.57495d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     28.81091d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      2.44607d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      4.95169d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      1.61547d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -4.03365d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'n+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'n+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.01853d0*CON_KCALTOK
      tclocal(1) =      4.13276d0*CON_KCALTOK
      tclocal(2) =     -1.85354d0*CON_KCALTOK
      tclocal(3) =      3.24394d0*CON_KCALTOK
      tclocal(4) =      1.94384d0*CON_KCALTOK
      tclocal(5) =      1.42460d0*CON_KCALTOK
      tclocal(6) =    -24.12834d0*CON_KCALTOK
      tclocal(7) =     24.32297d0*CON_KCALTOK
      tclocal(8) =     20.64361d0*CON_KCALTOK
      tclocal( 9) =     11.52584d0*CON_KCALTOK
      tclocal(10) =    -14.39445d0*CON_KCALTOK
      tclocal(11) =     -6.22186d0*CON_KCALTOK
      tclocal(12) =     15.19732d0*CON_KCALTOK
      tclocal(13) =     11.19000d0*CON_KCALTOK
      tclocal(14) =      0.73355d0*CON_KCALTOK
      tclocal(15) =    -12.79047d0*CON_KCALTOK
      tclocal(16) =     -2.24627d0*CON_KCALTOK
      tclocal(17) =      3.29244d0*CON_KCALTOK
      tclocal(18) =    -22.79744d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.66603d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      2.44380d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     23.40850d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -8.31630d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     19.22244d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     12.91863d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -35.73311d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nn ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nn '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.05556d0*CON_KCALTOK
      tclocal(1) =      4.89365d0*CON_KCALTOK
      tclocal(2) =     -0.50300d0*CON_KCALTOK
      tclocal(3) =      2.99525d0*CON_KCALTOK
      tclocal(4) =      1.51424d0*CON_KCALTOK
      tclocal(5) =      0.67055d0*CON_KCALTOK
      tclocal(6) =     -7.57439d0*CON_KCALTOK
      tclocal(7) =      9.63822d0*CON_KCALTOK
      tclocal(8) =      2.08364d0*CON_KCALTOK
      tclocal( 9) =     14.45695d0*CON_KCALTOK
      tclocal(10) =     -4.71558d0*CON_KCALTOK
      tclocal(11) =      4.22966d0*CON_KCALTOK
      tclocal(12) =     15.35360d0*CON_KCALTOK
      tclocal(13) =      4.83429d0*CON_KCALTOK
      tclocal(14) =     -0.93715d0*CON_KCALTOK
      tclocal(15) =    -13.50863d0*CON_KCALTOK
      tclocal(16) =      2.71617d0*CON_KCALTOK
      tclocal(17) =      3.79840d0*CON_KCALTOK
      tclocal(18) =    -22.15911d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.35712d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     16.92508d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      3.33308d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      5.01955d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     10.27343d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -7.72652d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.79906d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -4.74565d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     24.26616d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -12.04778d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =    -11.98171d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      7.01831d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -22.07069d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.44604d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -1.97876d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     20.52582d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      1.92798d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      3.50557d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      4.17532d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      1.48732d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.50650d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      7.67097d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     22.94643d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -11.50325d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -1.47395d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.77036d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -29.77504d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      5.24828d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -9.11795d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     15.73331d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -13.65501d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -1.45708d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      1.47074d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -30.01625d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.16415d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      1.17399d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     33.05640d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -7.09913d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -1.91578d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.82616d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -16.10665d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.05047d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      1.40259d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     17.21811d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.11189d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      3.89812d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      6.49234d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.52605d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.76838d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      7.92797d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     26.52704d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -5.42900d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.82178d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -4.71134d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -33.32224d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.61750d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -5.62975d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     17.20585d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -14.59059d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -1.49251d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -1.87675d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -27.58619d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.27692d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      6.42177d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     12.34282d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     18.44089d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     16.98377d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      3.90872d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -0.32739d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.14449d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.04281d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     32.16291d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -11.09826d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =    -11.92427d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      6.69955d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -18.60045d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c+ ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c+ '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.74042d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.73442d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     15.90886d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.45284d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      2.87632d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      6.15456d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -0.98486d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c+ ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c+ '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.93071d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      3.44826d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     19.32936d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      6.74390d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -1.90725d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.87306d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -31.52723d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c+ ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c+ '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.80087d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -3.91930d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      9.62813d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -4.51029d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -0.84239d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -1.10773d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -29.42891d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c+ ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c+ '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      5.50121d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -2.35415d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     29.50000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -11.08272d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -8.36605d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      9.53755d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -20.28316d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c= ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c= '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.37139d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      1.19212d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     17.20912d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      1.61283d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      3.81536d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      6.11072d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -5.35119d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c= ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c= '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.12878d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      4.34162d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     30.04236d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -12.38965d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      2.59665d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      2.63532d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -29.47656d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c= ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c= '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.91009d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -1.71051d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     13.53790d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -17.04064d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -1.83322d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      2.67086d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -24.71538d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c= ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c= '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.75678d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      2.83837d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     13.58150d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      2.83837d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      1.49380d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      1.49380d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -0.01102d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cl ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cl ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cl '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cl '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.88945d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      3.70845d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     19.60001d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.55747d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      8.48009d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      5.45795d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -11.96647d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cl ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cl '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.23519d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.18277d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     13.22631d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.57612d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      4.38768d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      4.87222d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -5.48091d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cl ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cl '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.16138d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     45.71344d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.91523d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =    -10.22517d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      1.20407d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cl ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cl '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.64763d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -2.03947d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     21.57576d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -2.65384d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      6.14994d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     16.81830d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -7.71624d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cl ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cl '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.83870d0*CON_KCALTOK
      tclocal(1) =      3.21349d0*CON_KCALTOK
      tclocal(2) =      0.09614d0*CON_KCALTOK
      tclocal(3) =     -4.61034d0*CON_KCALTOK
      tclocal(4) =     -5.18709d0*CON_KCALTOK
      tclocal(5) =     -1.74720d0*CON_KCALTOK
      tclocal(6) =     17.68280d0*CON_KCALTOK
      tclocal(7) =     30.87035d0*CON_KCALTOK
      tclocal(8) =     -5.85524d0*CON_KCALTOK
      tclocal( 9) =     -4.61034d0*CON_KCALTOK
      tclocal(10) =     -5.18709d0*CON_KCALTOK
      tclocal(11) =     -1.74720d0*CON_KCALTOK
      tclocal(12) =      6.36591d0*CON_KCALTOK
      tclocal(13) =     -1.79964d0*CON_KCALTOK
      tclocal(14) =      0.89910d0*CON_KCALTOK
      tclocal(15) =      6.36591d0*CON_KCALTOK
      tclocal(16) =     -1.79964d0*CON_KCALTOK
      tclocal(17) =      0.89910d0*CON_KCALTOK
      tclocal(18) =    -26.32333d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.83870d0*CON_KCALTOK
      tclocal(1) =      3.21349d0*CON_KCALTOK
      tclocal(2) =      0.09614d0*CON_KCALTOK
      tclocal(3) =     -4.61034d0*CON_KCALTOK
      tclocal(4) =     -5.18709d0*CON_KCALTOK
      tclocal(5) =     -1.74720d0*CON_KCALTOK
      tclocal(6) =     17.68280d0*CON_KCALTOK
      tclocal(7) =     30.87035d0*CON_KCALTOK
      tclocal(8) =     -5.85524d0*CON_KCALTOK
      tclocal( 9) =     -4.61034d0*CON_KCALTOK
      tclocal(10) =     -5.18709d0*CON_KCALTOK
      tclocal(11) =     -1.74720d0*CON_KCALTOK
      tclocal(12) =      6.36591d0*CON_KCALTOK
      tclocal(13) =     -1.79964d0*CON_KCALTOK
      tclocal(14) =      0.89910d0*CON_KCALTOK
      tclocal(15) =      6.36591d0*CON_KCALTOK
      tclocal(16) =     -1.79964d0*CON_KCALTOK
      tclocal(17) =      0.89910d0*CON_KCALTOK
      tclocal(18) =    -26.32333d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.58879d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -10.02273d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     20.38719d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -0.68720d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      6.31148d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.38471d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -13.94841d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'ct ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'ct '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.98183d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     22.31184d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      8.74818d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      8.74793d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -22.22110d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'f  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'f  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.59820d0*CON_KCALTOK
      tclocal(1) =      4.47983d0*CON_KCALTOK
      tclocal(2) =      0.17441d0*CON_KCALTOK
      tclocal(3) =      8.19541d0*CON_KCALTOK
      tclocal(4) =     -9.56593d0*CON_KCALTOK
      tclocal(5) =     -1.23934d0*CON_KCALTOK
      tclocal(6) =    -17.66026d0*CON_KCALTOK
      tclocal(7) =      8.29314d0*CON_KCALTOK
      tclocal(8) =     -2.50976d0*CON_KCALTOK
      tclocal( 9) =      1.68462d0*CON_KCALTOK
      tclocal(10) =     -1.01038d0*CON_KCALTOK
      tclocal(11) =     -1.05285d0*CON_KCALTOK
      tclocal(12) =      3.38375d0*CON_KCALTOK
      tclocal(13) =      8.39084d0*CON_KCALTOK
      tclocal(14) =      1.67226d0*CON_KCALTOK
      tclocal(15) =      5.04130d0*CON_KCALTOK
      tclocal(16) =      3.56413d0*CON_KCALTOK
      tclocal(17) =     -2.55767d0*CON_KCALTOK
      tclocal(18) =    -16.91904d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.06655d0*CON_KCALTOK
      tclocal(1) =      2.89651d0*CON_KCALTOK
      tclocal(2) =     -0.29366d0*CON_KCALTOK
      tclocal(3) =     11.87938d0*CON_KCALTOK
      tclocal(4) =     -6.47429d0*CON_KCALTOK
      tclocal(5) =     -3.21301d0*CON_KCALTOK
      tclocal(6) =     -5.35425d0*CON_KCALTOK
      tclocal(7) =      7.13891d0*CON_KCALTOK
      tclocal(8) =    -14.15417d0*CON_KCALTOK
      tclocal( 9) =     -1.96222d0*CON_KCALTOK
      tclocal(10) =     -0.98391d0*CON_KCALTOK
      tclocal(11) =      7.96453d0*CON_KCALTOK
      tclocal(12) =     -7.03551d0*CON_KCALTOK
      tclocal(13) =      4.60024d0*CON_KCALTOK
      tclocal(14) =     -4.15066d0*CON_KCALTOK
      tclocal(15) =      5.87329d0*CON_KCALTOK
      tclocal(16) =     -3.13641d0*CON_KCALTOK
      tclocal(17) =     -1.85273d0*CON_KCALTOK
      tclocal(18) =    -44.17072d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      5.28766d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     18.53807d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      5.35578d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     14.05763d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -22.88496d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'n` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'n` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.06655d0*CON_KCALTOK
      tclocal(1) =      2.89651d0*CON_KCALTOK
      tclocal(2) =     -0.29366d0*CON_KCALTOK
      tclocal(3) =     11.87938d0*CON_KCALTOK
      tclocal(4) =     -6.47429d0*CON_KCALTOK
      tclocal(5) =     -3.21301d0*CON_KCALTOK
      tclocal(6) =     -5.35425d0*CON_KCALTOK
      tclocal(7) =      7.13891d0*CON_KCALTOK
      tclocal(8) =    -14.15417d0*CON_KCALTOK
      tclocal( 9) =     -1.96222d0*CON_KCALTOK
      tclocal(10) =     -0.98391d0*CON_KCALTOK
      tclocal(11) =      7.96453d0*CON_KCALTOK
      tclocal(12) =     -7.03551d0*CON_KCALTOK
      tclocal(13) =      4.60024d0*CON_KCALTOK
      tclocal(14) =     -4.15066d0*CON_KCALTOK
      tclocal(15) =      5.87329d0*CON_KCALTOK
      tclocal(16) =     -3.13641d0*CON_KCALTOK
      tclocal(17) =     -1.85273d0*CON_KCALTOK
      tclocal(18) =    -44.17072d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'n+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'n+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.65009d0*CON_KCALTOK
      tclocal(2) =     -0.65425d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =    -21.46634d0*CON_KCALTOK
      tclocal(7) =      2.58208d0*CON_KCALTOK
      tclocal(8) =      1.99579d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'n= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'n= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.28800d0*CON_KCALTOK
      tclocal(1) =      3.06521d0*CON_KCALTOK
      tclocal(2) =     -0.38761d0*CON_KCALTOK
      tclocal(3) =     -2.17891d0*CON_KCALTOK
      tclocal(4) =     -8.73689d0*CON_KCALTOK
      tclocal(5) =      2.46424d0*CON_KCALTOK
      tclocal(6) =     10.19942d0*CON_KCALTOK
      tclocal(7) =     22.25165d0*CON_KCALTOK
      tclocal(8) =      3.80958d0*CON_KCALTOK
      tclocal( 9) =      3.99043d0*CON_KCALTOK
      tclocal(10) =      1.93974d0*CON_KCALTOK
      tclocal(11) =     -3.55199d0*CON_KCALTOK
      tclocal(12) =     -6.30373d0*CON_KCALTOK
      tclocal(13) =      5.44219d0*CON_KCALTOK
      tclocal(14) =      5.64250d0*CON_KCALTOK
      tclocal(15) =     10.34825d0*CON_KCALTOK
      tclocal(16) =     -1.78746d0*CON_KCALTOK
      tclocal(17) =      0.54376d0*CON_KCALTOK
      tclocal(18) =    -29.80582d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.06655d0*CON_KCALTOK
      tclocal(1) =      2.89651d0*CON_KCALTOK
      tclocal(2) =     -0.29366d0*CON_KCALTOK
      tclocal(3) =     11.87938d0*CON_KCALTOK
      tclocal(4) =     -6.47429d0*CON_KCALTOK
      tclocal(5) =     -3.21301d0*CON_KCALTOK
      tclocal(6) =     -5.35425d0*CON_KCALTOK
      tclocal(7) =      7.13891d0*CON_KCALTOK
      tclocal(8) =    -14.15417d0*CON_KCALTOK
      tclocal( 9) =     -1.96222d0*CON_KCALTOK
      tclocal(10) =     -0.98391d0*CON_KCALTOK
      tclocal(11) =      7.96453d0*CON_KCALTOK
      tclocal(12) =     -7.03551d0*CON_KCALTOK
      tclocal(13) =      4.60024d0*CON_KCALTOK
      tclocal(14) =     -4.15066d0*CON_KCALTOK
      tclocal(15) =      5.87329d0*CON_KCALTOK
      tclocal(16) =     -3.13641d0*CON_KCALTOK
      tclocal(17) =     -1.85273d0*CON_KCALTOK
      tclocal(18) =    -44.17072d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nn ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nn '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.15954d0*CON_KCALTOK
      tclocal(1) =      2.99746d0*CON_KCALTOK
      tclocal(2) =     -0.10416d0*CON_KCALTOK
      tclocal(3) =    -12.79235d0*CON_KCALTOK
      tclocal(4) =      3.64836d0*CON_KCALTOK
      tclocal(5) =     -2.30551d0*CON_KCALTOK
      tclocal(6) =     10.83592d0*CON_KCALTOK
      tclocal(7) =     17.08668d0*CON_KCALTOK
      tclocal(8) =     -3.01175d0*CON_KCALTOK
      tclocal( 9) =    -14.92780d0*CON_KCALTOK
      tclocal(10) =      1.54121d0*CON_KCALTOK
      tclocal(11) =     -3.34802d0*CON_KCALTOK
      tclocal(12) =     -5.16412d0*CON_KCALTOK
      tclocal(13) =      4.54387d0*CON_KCALTOK
      tclocal(14) =      3.47810d0*CON_KCALTOK
      tclocal(15) =      6.13531d0*CON_KCALTOK
      tclocal(16) =     -0.07078d0*CON_KCALTOK
      tclocal(17) =      0.03579d0*CON_KCALTOK
      tclocal(18) =    -29.03613d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.08185d0*CON_KCALTOK
      tclocal(1) =      3.17741d0*CON_KCALTOK
      tclocal(2) =     -0.58746d0*CON_KCALTOK
      tclocal(3) =      3.21036d0*CON_KCALTOK
      tclocal(4) =    -12.85046d0*CON_KCALTOK
      tclocal(5) =     -3.29261d0*CON_KCALTOK
      tclocal(6) =    -15.96612d0*CON_KCALTOK
      tclocal(7) =      7.26381d0*CON_KCALTOK
      tclocal(8) =     -5.17278d0*CON_KCALTOK
      tclocal( 9) =      2.71155d0*CON_KCALTOK
      tclocal(10) =      5.59249d0*CON_KCALTOK
      tclocal(11) =      2.14076d0*CON_KCALTOK
      tclocal(12) =     -3.38532d0*CON_KCALTOK
      tclocal(13) =     13.56384d0*CON_KCALTOK
      tclocal(14) =      1.20080d0*CON_KCALTOK
      tclocal(15) =     15.12146d0*CON_KCALTOK
      tclocal(16) =      9.83184d0*CON_KCALTOK
      tclocal(17) =      4.61827d0*CON_KCALTOK
      tclocal(18) =    -27.98516d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.00229d0*CON_KCALTOK
      tclocal(2) =     -0.63828d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -12.58806d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =     18.04693d0*CON_KCALTOK
      tclocal(7) =     25.50264d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =     11.87420d0*CON_KCALTOK
      tclocal(12) =    -21.87145d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      7.79593d0*CON_KCALTOK
      tclocal(15) =      5.49847d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -29.05384d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.04369d0*CON_KCALTOK
      tclocal(1) =      2.99180d0*CON_KCALTOK
      tclocal(2) =     -0.33678d0*CON_KCALTOK
      tclocal(3) =      1.35982d0*CON_KCALTOK
      tclocal(4) =    -15.50456d0*CON_KCALTOK
      tclocal(5) =     -3.20112d0*CON_KCALTOK
      tclocal(6) =    -25.97662d0*CON_KCALTOK
      tclocal(7) =      0.82600d0*CON_KCALTOK
      tclocal(8) =     -9.26919d0*CON_KCALTOK
      tclocal( 9) =      2.07879d0*CON_KCALTOK
      tclocal(10) =      2.30154d0*CON_KCALTOK
      tclocal(11) =      0.74965d0*CON_KCALTOK
      tclocal(12) =     -2.97730d0*CON_KCALTOK
      tclocal(13) =      4.01884d0*CON_KCALTOK
      tclocal(14) =     -1.51878d0*CON_KCALTOK
      tclocal(15) =      4.94360d0*CON_KCALTOK
      tclocal(16) =     -2.15408d0*CON_KCALTOK
      tclocal(17) =      1.43335d0*CON_KCALTOK
      tclocal(18) =     -8.95890d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'s  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 's  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.76990d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -7.92868d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     38.06891d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -2.73432d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     12.79120d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      2.33127d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -12.00360d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'s" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 's" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.33359d0*CON_KCALTOK
      tclocal(2) =     -0.98419d0*CON_KCALTOK
      tclocal(3) =     -9.53128d0*CON_KCALTOK
      tclocal(4) =    -14.65977d0*CON_KCALTOK
      tclocal(5) =      4.65249d0*CON_KCALTOK
      tclocal(6) =     12.56752d0*CON_KCALTOK
      tclocal(7) =     16.86552d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -0.03179d0*CON_KCALTOK
      tclocal(11) =     -0.35753d0*CON_KCALTOK
      tclocal(12) =    -19.02316d0*CON_KCALTOK
      tclocal(13) =     -4.08948d0*CON_KCALTOK
      tclocal(14) =      5.83842d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.71153d0*CON_KCALTOK
      tclocal(17) =      1.44527d0*CON_KCALTOK
      tclocal(18) =    -20.11977d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.59820d0*CON_KCALTOK
      tclocal(1) =      4.47983d0*CON_KCALTOK
      tclocal(2) =      0.17441d0*CON_KCALTOK
      tclocal(3) =      8.19541d0*CON_KCALTOK
      tclocal(4) =     -9.56593d0*CON_KCALTOK
      tclocal(5) =     -1.23934d0*CON_KCALTOK
      tclocal(6) =    -17.66026d0*CON_KCALTOK
      tclocal(7) =      8.29314d0*CON_KCALTOK
      tclocal(8) =     -2.50976d0*CON_KCALTOK
      tclocal( 9) =      1.68462d0*CON_KCALTOK
      tclocal(10) =     -1.01038d0*CON_KCALTOK
      tclocal(11) =     -1.05285d0*CON_KCALTOK
      tclocal(12) =      3.38375d0*CON_KCALTOK
      tclocal(13) =      8.39084d0*CON_KCALTOK
      tclocal(14) =      1.67226d0*CON_KCALTOK
      tclocal(15) =      5.04130d0*CON_KCALTOK
      tclocal(16) =      3.56413d0*CON_KCALTOK
      tclocal(17) =     -2.55767d0*CON_KCALTOK
      tclocal(18) =    -16.91904d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.78333d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     11.35442d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      6.23157d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     15.27969d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      4.44629d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     11.09367d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =   -113.05267d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.15954d0*CON_KCALTOK
      tclocal(1) =      2.99746d0*CON_KCALTOK
      tclocal(2) =     -0.10416d0*CON_KCALTOK
      tclocal(3) =    -12.79235d0*CON_KCALTOK
      tclocal(4) =      3.64836d0*CON_KCALTOK
      tclocal(5) =     -2.30551d0*CON_KCALTOK
      tclocal(6) =     10.83592d0*CON_KCALTOK
      tclocal(7) =     17.08668d0*CON_KCALTOK
      tclocal(8) =     -3.01175d0*CON_KCALTOK
      tclocal( 9) =    -14.92780d0*CON_KCALTOK
      tclocal(10) =      1.54121d0*CON_KCALTOK
      tclocal(11) =     -3.34802d0*CON_KCALTOK
      tclocal(12) =     -5.16412d0*CON_KCALTOK
      tclocal(13) =      4.54387d0*CON_KCALTOK
      tclocal(14) =      3.47810d0*CON_KCALTOK
      tclocal(15) =      6.13531d0*CON_KCALTOK
      tclocal(16) =     -0.07078d0*CON_KCALTOK
      tclocal(17) =      0.03579d0*CON_KCALTOK
      tclocal(18) =    -29.03613d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.00229d0*CON_KCALTOK
      tclocal(2) =     -0.63828d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -12.58806d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =     18.04693d0*CON_KCALTOK
      tclocal(7) =     25.50264d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =     11.87420d0*CON_KCALTOK
      tclocal(12) =    -21.87145d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      7.79593d0*CON_KCALTOK
      tclocal(15) =      5.49847d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -29.05384d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.48165d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.23181d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     17.22997d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      2.07050d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.11148d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      4.94099d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -7.85294d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'ct ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'ct '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.24012d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      1.00535d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     24.03533d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     18.71070d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      4.18555d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     15.14918d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -11.05307d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'ct ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'ct '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.18571d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -1.21267d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     11.23446d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      6.90652d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -1.93662d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -2.62747d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -0.20022d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'ct ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'s  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'ct '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 's  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.78986d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     12.82847d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      6.90441d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      6.90441d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -7.99921d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'f  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'f  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'f  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'f  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.65095d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     16.04664d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      4.86753d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      4.87221d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -8.88186d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'f  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'f  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.46738d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     49.23165d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      3.08711d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -2.08698d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -1.79869d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'f  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'f  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.15500d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -4.71367d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     25.57275d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -4.86618d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      7.60398d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     24.64593d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -11.75114d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'f  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'f  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.85507d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.92235d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     10.86806d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.92235d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      3.64876d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      3.64876d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -5.88275d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.05437d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -1.85708d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      1.76359d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -7.74534d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.75518d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -1.78817d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -6.81789d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.42334d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.86669d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     17.19448d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -3.57485d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      5.30906d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     13.14604d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -9.64584d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'n` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'n` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.96901d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.53509d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     25.72527d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      3.04924d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      5.46220d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      2.88215d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -5.94043d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'n+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'n+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      7.14891d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'n= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'n= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.46099d0*CON_KCALTOK
      tclocal(1) =      0.94018d0*CON_KCALTOK
      tclocal(2) =     -0.92764d0*CON_KCALTOK
      tclocal(3) =     -0.78507d0*CON_KCALTOK
      tclocal(4) =     -1.05797d0*CON_KCALTOK
      tclocal(5) =     -0.50997d0*CON_KCALTOK
      tclocal(6) =    -10.07646d0*CON_KCALTOK
      tclocal(7) =     24.25183d0*CON_KCALTOK
      tclocal(8) =     -0.61103d0*CON_KCALTOK
      tclocal( 9) =      1.83788d0*CON_KCALTOK
      tclocal(10) =    -18.76686d0*CON_KCALTOK
      tclocal(11) =     -2.93882d0*CON_KCALTOK
      tclocal(12) =     -0.11580d0*CON_KCALTOK
      tclocal(13) =      6.62951d0*CON_KCALTOK
      tclocal(14) =      0.93884d0*CON_KCALTOK
      tclocal(15) =     -5.75334d0*CON_KCALTOK
      tclocal(16) =     -5.65195d0*CON_KCALTOK
      tclocal(17) =     -0.03581d0*CON_KCALTOK
      tclocal(18) =    -16.79566d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.05437d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -1.85708d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      1.76359d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -7.74534d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.75518d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -1.78817d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -6.81789d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nn ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nn '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.52460d0*CON_KCALTOK
      tclocal(1) =      2.95674d0*CON_KCALTOK
      tclocal(2) =     -0.36716d0*CON_KCALTOK
      tclocal(3) =      2.25674d0*CON_KCALTOK
      tclocal(4) =     -1.29228d0*CON_KCALTOK
      tclocal(5) =     -0.20594d0*CON_KCALTOK
      tclocal(6) =    -11.63048d0*CON_KCALTOK
      tclocal(7) =      7.88590d0*CON_KCALTOK
      tclocal(8) =     -2.11438d0*CON_KCALTOK
      tclocal( 9) =     10.60293d0*CON_KCALTOK
      tclocal(10) =    -11.68286d0*CON_KCALTOK
      tclocal(11) =     -0.93904d0*CON_KCALTOK
      tclocal(12) =      1.05452d0*CON_KCALTOK
      tclocal(13) =      4.45810d0*CON_KCALTOK
      tclocal(14) =     -0.67922d0*CON_KCALTOK
      tclocal(15) =     -5.34550d0*CON_KCALTOK
      tclocal(16) =      2.60943d0*CON_KCALTOK
      tclocal(17) =      0.68200d0*CON_KCALTOK
      tclocal(18) =    -19.74410d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.35712d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     16.92508d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      3.33308d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      5.01955d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     10.27343d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -7.72652d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.14160d0*CON_KCALTOK
      tclocal(2) =     -1.15703d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -1.23502d0*CON_KCALTOK
      tclocal(5) =     -1.58700d0*CON_KCALTOK
      tclocal(6) =    -27.78486d0*CON_KCALTOK
      tclocal(7) =     30.13150d0*CON_KCALTOK
      tclocal(8) =     10.55468d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -3.07925d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      4.69599d0*CON_KCALTOK
      tclocal(14) =      0.97123d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -8.65942d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.95901d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.16750d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      2.12312d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -0.03620d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      2.18911d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.32891d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.80115d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'s  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 's  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.13843d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      2.71975d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     18.36016d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      2.29704d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      4.66259d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      3.34851d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      2.16595d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'s" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 's" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -7.55028d0*CON_KCALTOK
      tclocal(1) =      2.25002d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.77190d0*CON_KCALTOK
      tclocal(5) =     -1.77130d0*CON_KCALTOK
      tclocal(6) =    -23.91165d0*CON_KCALTOK
      tclocal(7) =     29.49367d0*CON_KCALTOK
      tclocal(8) =      8.60849d0*CON_KCALTOK
      tclocal( 9) =    -10.17280d0*CON_KCALTOK
      tclocal(10) =     -5.43550d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.84626d0*CON_KCALTOK
      tclocal(13) =      3.38825d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -3.77167d0*CON_KCALTOK
      tclocal(17) =     -2.81261d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10686d0*CON_KCALTOK
      tclocal(1) =      0.51178d0*CON_KCALTOK
      tclocal(2) =      1.08899d0*CON_KCALTOK
      tclocal(3) =      6.22623d0*CON_KCALTOK
      tclocal(4) =    -19.95534d0*CON_KCALTOK
      tclocal(5) =      5.75189d0*CON_KCALTOK
      tclocal(6) =     -7.85165d0*CON_KCALTOK
      tclocal(7) =     26.20418d0*CON_KCALTOK
      tclocal(8) =     -1.31925d0*CON_KCALTOK
      tclocal( 9) =      1.46711d0*CON_KCALTOK
      tclocal(10) =    -22.60483d0*CON_KCALTOK
      tclocal(11) =      2.47594d0*CON_KCALTOK
      tclocal(12) =     10.36675d0*CON_KCALTOK
      tclocal(13) =     10.72067d0*CON_KCALTOK
      tclocal(14) =      3.36674d0*CON_KCALTOK
      tclocal(15) =    -11.15231d0*CON_KCALTOK
      tclocal(16) =      9.87006d0*CON_KCALTOK
      tclocal(17) =      2.87655d0*CON_KCALTOK
      tclocal(18) =    -13.53522d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.04612d0*CON_KCALTOK
      tclocal(1) =      1.86320d0*CON_KCALTOK
      tclocal(2) =      0.60655d0*CON_KCALTOK
      tclocal(3) =     14.21823d0*CON_KCALTOK
      tclocal(4) =     -2.13996d0*CON_KCALTOK
      tclocal(5) =     -3.01427d0*CON_KCALTOK
      tclocal(6) =     -7.01852d0*CON_KCALTOK
      tclocal(7) =     16.19703d0*CON_KCALTOK
      tclocal(8) =      0.76747d0*CON_KCALTOK
      tclocal( 9) =     11.59332d0*CON_KCALTOK
      tclocal(10) =     -2.81831d0*CON_KCALTOK
      tclocal(11) =      5.61147d0*CON_KCALTOK
      tclocal(12) =      3.11880d0*CON_KCALTOK
      tclocal(13) =     -1.40790d0*CON_KCALTOK
      tclocal(14) =      2.15859d0*CON_KCALTOK
      tclocal(15) =     -8.14388d0*CON_KCALTOK
      tclocal(16) =     13.37553d0*CON_KCALTOK
      tclocal(17) =      0.04253d0*CON_KCALTOK
      tclocal(18) =    -16.63320d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.23170d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -4.44840d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     46.81879d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      9.59116d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     16.14962d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -7.81056d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -31.40547d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.83704d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      1.74598d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     15.12388d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      1.93337d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      7.19480d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     13.51440d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -18.96212d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10686d0*CON_KCALTOK
      tclocal(1) =      0.51178d0*CON_KCALTOK
      tclocal(2) =      1.08899d0*CON_KCALTOK
      tclocal(3) =      6.22623d0*CON_KCALTOK
      tclocal(4) =    -19.95534d0*CON_KCALTOK
      tclocal(5) =      5.75189d0*CON_KCALTOK
      tclocal(6) =     -7.85165d0*CON_KCALTOK
      tclocal(7) =     26.20418d0*CON_KCALTOK
      tclocal(8) =     -1.31925d0*CON_KCALTOK
      tclocal( 9) =      1.46711d0*CON_KCALTOK
      tclocal(10) =    -22.60483d0*CON_KCALTOK
      tclocal(11) =      2.47594d0*CON_KCALTOK
      tclocal(12) =     10.36675d0*CON_KCALTOK
      tclocal(13) =     10.72067d0*CON_KCALTOK
      tclocal(14) =      3.36674d0*CON_KCALTOK
      tclocal(15) =    -11.15231d0*CON_KCALTOK
      tclocal(16) =      9.87006d0*CON_KCALTOK
      tclocal(17) =      2.87655d0*CON_KCALTOK
      tclocal(18) =    -13.53522d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n+ ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n+ '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.04612d0*CON_KCALTOK
      tclocal(1) =      1.86320d0*CON_KCALTOK
      tclocal(2) =      0.60655d0*CON_KCALTOK
      tclocal(3) =     14.21823d0*CON_KCALTOK
      tclocal(4) =     -2.13996d0*CON_KCALTOK
      tclocal(5) =     -3.01427d0*CON_KCALTOK
      tclocal(6) =     -7.01852d0*CON_KCALTOK
      tclocal(7) =     16.19703d0*CON_KCALTOK
      tclocal(8) =      0.76747d0*CON_KCALTOK
      tclocal( 9) =     11.59332d0*CON_KCALTOK
      tclocal(10) =     -2.81831d0*CON_KCALTOK
      tclocal(11) =      5.61147d0*CON_KCALTOK
      tclocal(12) =      3.11880d0*CON_KCALTOK
      tclocal(13) =     -1.40790d0*CON_KCALTOK
      tclocal(14) =      2.15859d0*CON_KCALTOK
      tclocal(15) =     -8.14388d0*CON_KCALTOK
      tclocal(16) =     13.37553d0*CON_KCALTOK
      tclocal(17) =      0.04253d0*CON_KCALTOK
      tclocal(18) =    -16.63320d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n+ ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n+ '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.03422d0*CON_KCALTOK
      tclocal(1) =      1.99873d0*CON_KCALTOK
      tclocal(2) =     -0.13475d0*CON_KCALTOK
      tclocal(3) =     -3.00920d0*CON_KCALTOK
      tclocal(4) =    -17.04817d0*CON_KCALTOK
      tclocal(5) =      3.47762d0*CON_KCALTOK
      tclocal(6) =      2.61157d0*CON_KCALTOK
      tclocal(7) =     27.08211d0*CON_KCALTOK
      tclocal(8) =      6.15740d0*CON_KCALTOK
      tclocal( 9) =     -3.00920d0*CON_KCALTOK
      tclocal(10) =    -17.04817d0*CON_KCALTOK
      tclocal(11) =      3.47762d0*CON_KCALTOK
      tclocal(12) =     -1.78851d0*CON_KCALTOK
      tclocal(13) =     -1.64703d0*CON_KCALTOK
      tclocal(14) =      3.06433d0*CON_KCALTOK
      tclocal(15) =     -1.78851d0*CON_KCALTOK
      tclocal(16) =     -1.64703d0*CON_KCALTOK
      tclocal(17) =      3.06433d0*CON_KCALTOK
      tclocal(18) =    -38.48092d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10686d0*CON_KCALTOK
      tclocal(1) =      0.51178d0*CON_KCALTOK
      tclocal(2) =      1.08899d0*CON_KCALTOK
      tclocal(3) =      1.46711d0*CON_KCALTOK
      tclocal(4) =    -22.60483d0*CON_KCALTOK
      tclocal(5) =      2.47594d0*CON_KCALTOK
      tclocal(6) =     -7.85165d0*CON_KCALTOK
      tclocal(7) =     26.20418d0*CON_KCALTOK
      tclocal(8) =     -1.31925d0*CON_KCALTOK
      tclocal( 9) =      6.22623d0*CON_KCALTOK
      tclocal(10) =    -19.95534d0*CON_KCALTOK
      tclocal(11) =      5.75189d0*CON_KCALTOK
      tclocal(12) =    -11.15231d0*CON_KCALTOK
      tclocal(13) =      9.87006d0*CON_KCALTOK
      tclocal(14) =      2.87655d0*CON_KCALTOK
      tclocal(15) =     10.36675d0*CON_KCALTOK
      tclocal(16) =     10.72067d0*CON_KCALTOK
      tclocal(17) =      3.36674d0*CON_KCALTOK
      tclocal(18) =    -13.53522d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'nn ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'nn '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.20209d0*CON_KCALTOK
      tclocal(1) =      3.31469d0*CON_KCALTOK
      tclocal(2) =     -0.46180d0*CON_KCALTOK
      tclocal(3) =      0.08153d0*CON_KCALTOK
      tclocal(4) =     -1.83871d0*CON_KCALTOK
      tclocal(5) =      3.27669d0*CON_KCALTOK
      tclocal(6) =      7.05726d0*CON_KCALTOK
      tclocal(7) =     15.42651d0*CON_KCALTOK
      tclocal(8) =      6.74181d0*CON_KCALTOK
      tclocal( 9) =     -6.60999d0*CON_KCALTOK
      tclocal(10) =     -7.67111d0*CON_KCALTOK
      tclocal(11) =      4.34114d0*CON_KCALTOK
      tclocal(12) =     13.73371d0*CON_KCALTOK
      tclocal(13) =      8.23703d0*CON_KCALTOK
      tclocal(14) =      2.66717d0*CON_KCALTOK
      tclocal(15) =      6.76517d0*CON_KCALTOK
      tclocal(16) =     16.21790d0*CON_KCALTOK
      tclocal(17) =      3.14656d0*CON_KCALTOK
      tclocal(18) =    -25.36786d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.00000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =     -5.07177d0*CON_KCALTOK
      tclocal(4) =    -33.46185d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     33.43591d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -28.47525d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =    -23.70311d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =     14.17048d0*CON_KCALTOK
      tclocal(16) =     -9.89117d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -15.16234d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.27869d0*CON_KCALTOK
      tclocal(1) =      5.14564d0*CON_KCALTOK
      tclocal(2) =     -3.25274d0*CON_KCALTOK
      tclocal(3) =     -2.39182d0*CON_KCALTOK
      tclocal(4) =    -13.99266d0*CON_KCALTOK
      tclocal(5) =     -4.73207d0*CON_KCALTOK
      tclocal(6) =     -1.19672d0*CON_KCALTOK
      tclocal(7) =     47.66778d0*CON_KCALTOK
      tclocal(8) =     -1.07630d0*CON_KCALTOK
      tclocal( 9) =      9.08213d0*CON_KCALTOK
      tclocal(10) =    -22.86596d0*CON_KCALTOK
      tclocal(11) =      0.10440d0*CON_KCALTOK
      tclocal(12) =     -4.87103d0*CON_KCALTOK
      tclocal(13) =     -9.47320d0*CON_KCALTOK
      tclocal(14) =     -1.37966d0*CON_KCALTOK
      tclocal(15) =      3.35376d0*CON_KCALTOK
      tclocal(16) =     -0.32583d0*CON_KCALTOK
      tclocal(17) =     -6.30604d0*CON_KCALTOK
      tclocal(18) =    -47.04582d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.36213d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -5.43440d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     28.84457d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      5.08369d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      8.18984d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      7.17068d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -25.60020d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'s" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 's" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.04612d0*CON_KCALTOK
      tclocal(1) =      1.86320d0*CON_KCALTOK
      tclocal(2) =      0.60655d0*CON_KCALTOK
      tclocal(3) =     14.21823d0*CON_KCALTOK
      tclocal(4) =     -2.13996d0*CON_KCALTOK
      tclocal(5) =     -3.01427d0*CON_KCALTOK
      tclocal(6) =     -7.01852d0*CON_KCALTOK
      tclocal(7) =     16.19703d0*CON_KCALTOK
      tclocal(8) =      0.76747d0*CON_KCALTOK
      tclocal( 9) =     11.59332d0*CON_KCALTOK
      tclocal(10) =     -2.81831d0*CON_KCALTOK
      tclocal(11) =      5.61147d0*CON_KCALTOK
      tclocal(12) =      3.11880d0*CON_KCALTOK
      tclocal(13) =     -1.40790d0*CON_KCALTOK
      tclocal(14) =      2.15859d0*CON_KCALTOK
      tclocal(15) =     -8.14388d0*CON_KCALTOK
      tclocal(16) =     13.37553d0*CON_KCALTOK
      tclocal(17) =      0.04253d0*CON_KCALTOK
      tclocal(18) =    -16.63320d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nn ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nn '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.78389d0*CON_KCALTOK
      tclocal(1) =      5.46955d0*CON_KCALTOK
      tclocal(2) =      0.24975d0*CON_KCALTOK
      tclocal(3) =     -8.85612d0*CON_KCALTOK
      tclocal(4) =      6.17195d0*CON_KCALTOK
      tclocal(5) =     -0.18593d0*CON_KCALTOK
      tclocal(6) =      5.85495d0*CON_KCALTOK
      tclocal(7) =      6.91593d0*CON_KCALTOK
      tclocal(8) =      1.64940d0*CON_KCALTOK
      tclocal( 9) =     -8.85612d0*CON_KCALTOK
      tclocal(10) =      6.17195d0*CON_KCALTOK
      tclocal(11) =     -0.18593d0*CON_KCALTOK
      tclocal(12) =      5.05310d0*CON_KCALTOK
      tclocal(13) =      5.49347d0*CON_KCALTOK
      tclocal(14) =      2.28324d0*CON_KCALTOK
      tclocal(15) =      5.05310d0*CON_KCALTOK
      tclocal(16) =      5.49347d0*CON_KCALTOK
      tclocal(17) =      2.28324d0*CON_KCALTOK
      tclocal(18) =    -30.39045d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.00000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =     21.20968d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =    -10.89319d0*CON_KCALTOK
      tclocal(7) =      9.43779d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -5.58580d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     -9.09497d0*CON_KCALTOK
      tclocal(13) =     17.79625d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =     19.37933d0*CON_KCALTOK
      tclocal(16) =     -3.10936d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -27.45527d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.50000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =    -18.35542d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =     13.93419d0*CON_KCALTOK
      tclocal(7) =     21.19651d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      3.12602d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      7.60430d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     14.50671d0*CON_KCALTOK
      tclocal(17) =      7.09755d0*CON_KCALTOK
      tclocal(18) =    -36.49242d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.15117d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -10.76599d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     10.56684d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -0.52645d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      8.40318d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.82541d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -10.30663d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'s" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 's" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.34920d0*CON_KCALTOK
      tclocal(2) =      0.00907d0*CON_KCALTOK
      tclocal(3) =    -16.44449d0*CON_KCALTOK
      tclocal(4) =     -2.58394d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =     10.28010d0*CON_KCALTOK
      tclocal(7) =     15.86459d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      4.01612d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      7.56006d0*CON_KCALTOK
      tclocal(13) =     11.49839d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      8.58793d0*CON_KCALTOK
      tclocal(16) =     18.42219d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -21.10233d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.35712d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      3.33308d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     16.92508d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      3.33308d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     10.27343d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     10.27343d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -7.72652d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.00000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -28.48237d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     33.43344d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -3.07925d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     14.18368d0*CON_KCALTOK
      tclocal(13) =     -9.88168d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -15.16488d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.09169d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -9.78428d0*CON_KCALTOK
      tclocal(5) =      9.92313d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     41.67192d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -9.78428d0*CON_KCALTOK
      tclocal(11) =      9.92313d0*CON_KCALTOK
      tclocal(12) =    -12.96220d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =    -12.96220d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -39.69501d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'op ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'op '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.16164d0*CON_KCALTOK
      tclocal(1) =      2.85307d0*CON_KCALTOK
      tclocal(2) =     -2.29837d0*CON_KCALTOK
      tclocal(3) =     10.71090d0*CON_KCALTOK
      tclocal(4) =     -6.20546d0*CON_KCALTOK
      tclocal(5) =     -2.30009d0*CON_KCALTOK
      tclocal(6) =     -3.87689d0*CON_KCALTOK
      tclocal(7) =     40.05207d0*CON_KCALTOK
      tclocal(8) =      2.54154d0*CON_KCALTOK
      tclocal( 9) =     -2.80428d0*CON_KCALTOK
      tclocal(10) =     -3.90878d0*CON_KCALTOK
      tclocal(11) =     -0.47178d0*CON_KCALTOK
      tclocal(12) =      3.93937d0*CON_KCALTOK
      tclocal(13) =     -5.96374d0*CON_KCALTOK
      tclocal(14) =     -7.96313d0*CON_KCALTOK
      tclocal(15) =    -14.24563d0*CON_KCALTOK
      tclocal(16) =      0.02886d0*CON_KCALTOK
      tclocal(17) =     -1.37428d0*CON_KCALTOK
      tclocal(18) =    -16.77679d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'op ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'op '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.62901d0*CON_KCALTOK
      tclocal(2) =     -0.55265d0*CON_KCALTOK
      tclocal(3) =      6.31827d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =     -3.69641d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     27.18863d0*CON_KCALTOK
      tclocal(8) =     -5.23711d0*CON_KCALTOK
      tclocal( 9) =      6.31827d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =     -3.69641d0*CON_KCALTOK
      tclocal(12) =     -4.90035d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =     -3.29182d0*CON_KCALTOK
      tclocal(15) =     -4.90035d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =     -3.29182d0*CON_KCALTOK
      tclocal(18) =    -35.16866d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'sp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cp ')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'sp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cp '
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.83870d0*CON_KCALTOK
      tclocal(1) =      3.21349d0*CON_KCALTOK
      tclocal(2) =      0.09614d0*CON_KCALTOK
      tclocal(3) =     -4.61034d0*CON_KCALTOK
      tclocal(4) =     -5.18709d0*CON_KCALTOK
      tclocal(5) =     -1.74720d0*CON_KCALTOK
      tclocal(6) =     17.68280d0*CON_KCALTOK
      tclocal(7) =     30.87035d0*CON_KCALTOK
      tclocal(8) =     -5.85524d0*CON_KCALTOK
      tclocal( 9) =     -4.61034d0*CON_KCALTOK
      tclocal(10) =     -5.18709d0*CON_KCALTOK
      tclocal(11) =     -1.74720d0*CON_KCALTOK
      tclocal(12) =      6.36591d0*CON_KCALTOK
      tclocal(13) =     -1.79964d0*CON_KCALTOK
      tclocal(14) =      0.89910d0*CON_KCALTOK
      tclocal(15) =      6.36591d0*CON_KCALTOK
      tclocal(16) =     -1.79964d0*CON_KCALTOK
      tclocal(17) =      0.89910d0*CON_KCALTOK
      tclocal(18) =    -26.32333d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.71443d0*CON_KCALTOK
      tclocal(1) =      6.83285d0*CON_KCALTOK
      tclocal(2) =      0.76449d0*CON_KCALTOK
      tclocal(3) =      7.19686d0*CON_KCALTOK
      tclocal(4) =     -4.64183d0*CON_KCALTOK
      tclocal(5) =     -0.02401d0*CON_KCALTOK
      tclocal(6) =    -24.16416d0*CON_KCALTOK
      tclocal(7) =     -2.63892d0*CON_KCALTOK
      tclocal(8) =    -11.64094d0*CON_KCALTOK
      tclocal( 9) =      2.54188d0*CON_KCALTOK
      tclocal(10) =      7.23685d0*CON_KCALTOK
      tclocal(11) =      0.87202d0*CON_KCALTOK
      tclocal(12) =     -7.88123d0*CON_KCALTOK
      tclocal(13) =     11.86218d0*CON_KCALTOK
      tclocal(14) =      8.88582d0*CON_KCALTOK
      tclocal(15) =     16.32022d0*CON_KCALTOK
      tclocal(16) =      5.63057d0*CON_KCALTOK
      tclocal(17) =      0.50302d0*CON_KCALTOK
      tclocal(18) =    -15.89423d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.28800d0*CON_KCALTOK
      tclocal(1) =      3.06521d0*CON_KCALTOK
      tclocal(2) =     -0.38761d0*CON_KCALTOK
      tclocal(3) =     -2.17891d0*CON_KCALTOK
      tclocal(4) =     -8.73689d0*CON_KCALTOK
      tclocal(5) =      2.46424d0*CON_KCALTOK
      tclocal(6) =     10.19942d0*CON_KCALTOK
      tclocal(7) =     22.25165d0*CON_KCALTOK
      tclocal(8) =      3.80958d0*CON_KCALTOK
      tclocal( 9) =      3.99043d0*CON_KCALTOK
      tclocal(10) =      1.93974d0*CON_KCALTOK
      tclocal(11) =     -3.55199d0*CON_KCALTOK
      tclocal(12) =     -6.30373d0*CON_KCALTOK
      tclocal(13) =      5.44219d0*CON_KCALTOK
      tclocal(14) =      5.64250d0*CON_KCALTOK
      tclocal(15) =     10.34825d0*CON_KCALTOK
      tclocal(16) =     -1.78746d0*CON_KCALTOK
      tclocal(17) =      0.54376d0*CON_KCALTOK
      tclocal(18) =    -29.80582d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.51250d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      5.41108d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      4.20393d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -0.49561d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     10.46323d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -5.88263d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -36.53484d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'nn ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'nn '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.15954d0*CON_KCALTOK
      tclocal(1) =      2.99746d0*CON_KCALTOK
      tclocal(2) =     -0.10416d0*CON_KCALTOK
      tclocal(3) =    -12.79235d0*CON_KCALTOK
      tclocal(4) =      3.64836d0*CON_KCALTOK
      tclocal(5) =     -2.30551d0*CON_KCALTOK
      tclocal(6) =     10.83592d0*CON_KCALTOK
      tclocal(7) =     17.08668d0*CON_KCALTOK
      tclocal(8) =     -3.01175d0*CON_KCALTOK
      tclocal( 9) =    -14.92780d0*CON_KCALTOK
      tclocal(10) =      1.54121d0*CON_KCALTOK
      tclocal(11) =     -3.34802d0*CON_KCALTOK
      tclocal(12) =     -5.16412d0*CON_KCALTOK
      tclocal(13) =      4.54387d0*CON_KCALTOK
      tclocal(14) =      3.47810d0*CON_KCALTOK
      tclocal(15) =      6.13531d0*CON_KCALTOK
      tclocal(16) =     -0.07078d0*CON_KCALTOK
      tclocal(17) =      0.03579d0*CON_KCALTOK
      tclocal(18) =    -29.03613d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.00229d0*CON_KCALTOK
      tclocal(2) =     -0.63828d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -12.58806d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =     18.04693d0*CON_KCALTOK
      tclocal(7) =     25.50264d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =     11.87420d0*CON_KCALTOK
      tclocal(12) =    -21.87145d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      7.79593d0*CON_KCALTOK
      tclocal(15) =      5.49847d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -29.05384d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.33359d0*CON_KCALTOK
      tclocal(2) =     -0.98419d0*CON_KCALTOK
      tclocal(3) =     -9.53128d0*CON_KCALTOK
      tclocal(4) =    -14.65977d0*CON_KCALTOK
      tclocal(5) =      4.65249d0*CON_KCALTOK
      tclocal(6) =     12.56752d0*CON_KCALTOK
      tclocal(7) =     16.86552d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -0.03179d0*CON_KCALTOK
      tclocal(11) =     -0.35753d0*CON_KCALTOK
      tclocal(12) =    -19.02316d0*CON_KCALTOK
      tclocal(13) =     -4.08948d0*CON_KCALTOK
      tclocal(14) =      5.83842d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.71153d0*CON_KCALTOK
      tclocal(17) =      1.44527d0*CON_KCALTOK
      tclocal(18) =    -20.11977d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.59820d0*CON_KCALTOK
      tclocal(1) =      4.47983d0*CON_KCALTOK
      tclocal(2) =      0.17441d0*CON_KCALTOK
      tclocal(3) =      1.68462d0*CON_KCALTOK
      tclocal(4) =     -1.01038d0*CON_KCALTOK
      tclocal(5) =     -1.05285d0*CON_KCALTOK
      tclocal(6) =    -17.66026d0*CON_KCALTOK
      tclocal(7) =      8.29314d0*CON_KCALTOK
      tclocal(8) =     -2.50976d0*CON_KCALTOK
      tclocal( 9) =      8.19541d0*CON_KCALTOK
      tclocal(10) =     -9.56593d0*CON_KCALTOK
      tclocal(11) =     -1.23934d0*CON_KCALTOK
      tclocal(12) =      5.04130d0*CON_KCALTOK
      tclocal(13) =      3.56413d0*CON_KCALTOK
      tclocal(14) =     -2.55767d0*CON_KCALTOK
      tclocal(15) =      3.38375d0*CON_KCALTOK
      tclocal(16) =      8.39084d0*CON_KCALTOK
      tclocal(17) =      1.67226d0*CON_KCALTOK
      tclocal(18) =    -16.91904d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.26088d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -1.20005d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     16.59824d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -0.01130d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      3.29617d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      5.73742d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -5.65222d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.46099d0*CON_KCALTOK
      tclocal(1) =      0.94018d0*CON_KCALTOK
      tclocal(2) =     -0.92764d0*CON_KCALTOK
      tclocal(3) =     -0.78507d0*CON_KCALTOK
      tclocal(4) =     -1.05797d0*CON_KCALTOK
      tclocal(5) =     -0.50997d0*CON_KCALTOK
      tclocal(6) =    -10.07646d0*CON_KCALTOK
      tclocal(7) =     24.25183d0*CON_KCALTOK
      tclocal(8) =     -0.61103d0*CON_KCALTOK
      tclocal( 9) =      1.83788d0*CON_KCALTOK
      tclocal(10) =    -18.76686d0*CON_KCALTOK
      tclocal(11) =     -2.93882d0*CON_KCALTOK
      tclocal(12) =     -0.11580d0*CON_KCALTOK
      tclocal(13) =      6.62951d0*CON_KCALTOK
      tclocal(14) =      0.93884d0*CON_KCALTOK
      tclocal(15) =     -5.75334d0*CON_KCALTOK
      tclocal(16) =     -5.65195d0*CON_KCALTOK
      tclocal(17) =     -0.03581d0*CON_KCALTOK
      tclocal(18) =    -16.79566d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.36662d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      1.05656d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      2.96940d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -0.63325d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      4.18839d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -4.80060d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -6.92066d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'nn ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'nn '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.52460d0*CON_KCALTOK
      tclocal(1) =      2.95674d0*CON_KCALTOK
      tclocal(2) =     -0.36716d0*CON_KCALTOK
      tclocal(3) =      2.25674d0*CON_KCALTOK
      tclocal(4) =     -1.29228d0*CON_KCALTOK
      tclocal(5) =     -0.20594d0*CON_KCALTOK
      tclocal(6) =    -11.63048d0*CON_KCALTOK
      tclocal(7) =      7.88590d0*CON_KCALTOK
      tclocal(8) =     -2.11438d0*CON_KCALTOK
      tclocal( 9) =     10.60293d0*CON_KCALTOK
      tclocal(10) =    -11.68286d0*CON_KCALTOK
      tclocal(11) =     -0.93904d0*CON_KCALTOK
      tclocal(12) =      1.05452d0*CON_KCALTOK
      tclocal(13) =      4.45810d0*CON_KCALTOK
      tclocal(14) =     -0.67922d0*CON_KCALTOK
      tclocal(15) =     -5.34550d0*CON_KCALTOK
      tclocal(16) =      2.60943d0*CON_KCALTOK
      tclocal(17) =      0.68200d0*CON_KCALTOK
      tclocal(18) =    -19.74410d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.14160d0*CON_KCALTOK
      tclocal(2) =     -1.15703d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -1.23502d0*CON_KCALTOK
      tclocal(5) =     -1.58700d0*CON_KCALTOK
      tclocal(6) =    -27.78486d0*CON_KCALTOK
      tclocal(7) =     30.13150d0*CON_KCALTOK
      tclocal(8) =     10.55468d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -3.07925d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      4.69599d0*CON_KCALTOK
      tclocal(14) =      0.97123d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -8.65942d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -7.55028d0*CON_KCALTOK
      tclocal(1) =      2.25002d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.77190d0*CON_KCALTOK
      tclocal(5) =     -1.77130d0*CON_KCALTOK
      tclocal(6) =    -23.91165d0*CON_KCALTOK
      tclocal(7) =     29.49367d0*CON_KCALTOK
      tclocal(8) =      8.60849d0*CON_KCALTOK
      tclocal( 9) =    -10.17280d0*CON_KCALTOK
      tclocal(10) =     -5.43550d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.84626d0*CON_KCALTOK
      tclocal(13) =      3.38825d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -3.77167d0*CON_KCALTOK
      tclocal(17) =     -2.81261d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.05556d0*CON_KCALTOK
      tclocal(1) =      4.89365d0*CON_KCALTOK
      tclocal(2) =     -0.50300d0*CON_KCALTOK
      tclocal(3) =     14.45695d0*CON_KCALTOK
      tclocal(4) =     -4.71558d0*CON_KCALTOK
      tclocal(5) =      4.22966d0*CON_KCALTOK
      tclocal(6) =     -7.57439d0*CON_KCALTOK
      tclocal(7) =      9.63822d0*CON_KCALTOK
      tclocal(8) =      2.08364d0*CON_KCALTOK
      tclocal( 9) =      2.99525d0*CON_KCALTOK
      tclocal(10) =      1.51424d0*CON_KCALTOK
      tclocal(11) =      0.67055d0*CON_KCALTOK
      tclocal(12) =    -13.50863d0*CON_KCALTOK
      tclocal(13) =      2.71617d0*CON_KCALTOK
      tclocal(14) =      3.79840d0*CON_KCALTOK
      tclocal(15) =     15.35360d0*CON_KCALTOK
      tclocal(16) =      4.83429d0*CON_KCALTOK
      tclocal(17) =     -0.93715d0*CON_KCALTOK
      tclocal(18) =    -22.15911d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.34920d0*CON_KCALTOK
      tclocal(2) =      0.00907d0*CON_KCALTOK
      tclocal(3) =    -16.44449d0*CON_KCALTOK
      tclocal(4) =     -2.58394d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =     10.28010d0*CON_KCALTOK
      tclocal(7) =     15.86459d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      4.01612d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      7.56006d0*CON_KCALTOK
      tclocal(13) =     11.49839d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      8.58793d0*CON_KCALTOK
      tclocal(16) =     18.42219d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -21.10233d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.10605d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.07892d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     38.19848d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.06095d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -0.49534d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -1.75635d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -5.41999d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'op ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'op '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.50000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      3.12602d0*CON_KCALTOK
      tclocal(6) =     13.93419d0*CON_KCALTOK
      tclocal(7) =     21.19651d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =    -18.35542d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     14.50671d0*CON_KCALTOK
      tclocal(14) =      7.09755d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      7.60430d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -36.49242d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'op ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'op '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.10605d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.07892d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     38.19848d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.06095d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -0.49534d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -1.75635d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -5.41999d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'sp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'sp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.34920d0*CON_KCALTOK
      tclocal(2) =      0.00907d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      4.01612d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =     10.28010d0*CON_KCALTOK
      tclocal(7) =     15.86459d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =    -16.44449d0*CON_KCALTOK
      tclocal(10) =     -2.58394d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      8.58793d0*CON_KCALTOK
      tclocal(13) =     18.42219d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      7.56006d0*CON_KCALTOK
      tclocal(16) =     11.49839d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -21.10233d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'sp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'sp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'ct ')
      call twh_torstring(GLB_SET,type,4,'ct ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'ct '
      tnlocal(nsame,4) = 'ct '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.27873d0*CON_KCALTOK
      tclocal(1) =      1.12063d0*CON_KCALTOK
      tclocal(2) =      0.10066d0*CON_KCALTOK
      tclocal(3) =      0.26296d0*CON_KCALTOK
      tclocal(4) =     -2.49078d0*CON_KCALTOK
      tclocal(5) =      1.09210d0*CON_KCALTOK
      tclocal(6) =      1.62160d0*CON_KCALTOK
      tclocal(7) =      6.64254d0*CON_KCALTOK
      tclocal(8) =      0.70432d0*CON_KCALTOK
      tclocal( 9) =      0.51986d0*CON_KCALTOK
      tclocal(10) =      1.51208d0*CON_KCALTOK
      tclocal(11) =      0.11936d0*CON_KCALTOK
      tclocal(12) =     -2.45958d0*CON_KCALTOK
      tclocal(13) =      6.68147d0*CON_KCALTOK
      tclocal(14) =      0.83772d0*CON_KCALTOK
      tclocal(15) =     -0.13666d0*CON_KCALTOK
      tclocal(16) =      1.79326d0*CON_KCALTOK
      tclocal(17) =      0.83071d0*CON_KCALTOK
      tclocal(18) =    -14.93467d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.89504d0*CON_KCALTOK
      tclocal(1) =      0.72227d0*CON_KCALTOK
      tclocal(2) =     -0.19954d0*CON_KCALTOK
      tclocal(3) =      0.14469d0*CON_KCALTOK
      tclocal(4) =     -0.68318d0*CON_KCALTOK
      tclocal(5) =      0.05218d0*CON_KCALTOK
      tclocal(6) =      1.10400d0*CON_KCALTOK
      tclocal(7) =      0.89462d0*CON_KCALTOK
      tclocal(8) =     -0.11291d0*CON_KCALTOK
      tclocal( 9) =     -1.91404d0*CON_KCALTOK
      tclocal(10) =     -1.86574d0*CON_KCALTOK
      tclocal(11) =      0.11648d0*CON_KCALTOK
      tclocal(12) =     -3.15413d0*CON_KCALTOK
      tclocal(13) =      1.64833d0*CON_KCALTOK
      tclocal(14) =      0.77968d0*CON_KCALTOK
      tclocal(15) =     -1.82655d0*CON_KCALTOK
      tclocal(16) =      4.05999d0*CON_KCALTOK
      tclocal(17) =     -0.43907d0*CON_KCALTOK
      tclocal(18) =     -7.43865d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.18252d0*CON_KCALTOK
      tclocal(1) =      0.34951d0*CON_KCALTOK
      tclocal(2) =     -0.22523d0*CON_KCALTOK
      tclocal(3) =     -1.12030d0*CON_KCALTOK
      tclocal(4) =     -0.23011d0*CON_KCALTOK
      tclocal(5) =      0.48357d0*CON_KCALTOK
      tclocal(6) =     -0.22665d0*CON_KCALTOK
      tclocal(7) =      4.15260d0*CON_KCALTOK
      tclocal(8) =      0.03993d0*CON_KCALTOK
      tclocal( 9) =      0.07853d0*CON_KCALTOK
      tclocal(10) =     -0.48820d0*CON_KCALTOK
      tclocal(11) =      0.25209d0*CON_KCALTOK
      tclocal(12) =     -6.16087d0*CON_KCALTOK
      tclocal(13) =      2.47061d0*CON_KCALTOK
      tclocal(14) =     -0.17738d0*CON_KCALTOK
      tclocal(15) =     -1.40675d0*CON_KCALTOK
      tclocal(16) =      1.52928d0*CON_KCALTOK
      tclocal(17) =     -0.17046d0*CON_KCALTOK
      tclocal(18) =     -9.56337d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.12170d0*CON_KCALTOK
      tclocal(1) =      0.89386d0*CON_KCALTOK
      tclocal(2) =      0.02307d0*CON_KCALTOK
      tclocal(3) =     -2.34860d0*CON_KCALTOK
      tclocal(4) =     -0.37686d0*CON_KCALTOK
      tclocal(5) =      1.89587d0*CON_KCALTOK
      tclocal(6) =      0.74167d0*CON_KCALTOK
      tclocal(7) =      9.42302d0*CON_KCALTOK
      tclocal(8) =      0.38665d0*CON_KCALTOK
      tclocal( 9) =     -0.35988d0*CON_KCALTOK
      tclocal(10) =      3.82556d0*CON_KCALTOK
      tclocal(11) =      0.10137d0*CON_KCALTOK
      tclocal(12) =     -5.93130d0*CON_KCALTOK
      tclocal(13) =     -2.70149d0*CON_KCALTOK
      tclocal(14) =     -1.30015d0*CON_KCALTOK
      tclocal(15) =     -1.06195d0*CON_KCALTOK
      tclocal(16) =      1.11553d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -2.99982d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'s" ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 's" '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.86931d0*CON_KCALTOK
      tclocal(1) =      0.94769d0*CON_KCALTOK
      tclocal(2) =      0.15550d0*CON_KCALTOK
      tclocal(3) =     -2.60020d0*CON_KCALTOK
      tclocal(4) =      3.51795d0*CON_KCALTOK
      tclocal(5) =     -2.46453d0*CON_KCALTOK
      tclocal(6) =      1.01155d0*CON_KCALTOK
      tclocal(7) =      4.20649d0*CON_KCALTOK
      tclocal(8) =      1.73205d0*CON_KCALTOK
      tclocal( 9) =      0.84489d0*CON_KCALTOK
      tclocal(10) =      4.80600d0*CON_KCALTOK
      tclocal(11) =      3.50845d0*CON_KCALTOK
      tclocal(12) =     -1.46197d0*CON_KCALTOK
      tclocal(13) =     10.06669d0*CON_KCALTOK
      tclocal(14) =      0.14625d0*CON_KCALTOK
      tclocal(15) =     -1.83772d0*CON_KCALTOK
      tclocal(16) =      3.16545d0*CON_KCALTOK
      tclocal(17) =     -0.93361d0*CON_KCALTOK
      tclocal(18) =    -18.52175d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.05946d0*CON_KCALTOK
      tclocal(1) =     -0.31269d0*CON_KCALTOK
      tclocal(2) =     -0.02335d0*CON_KCALTOK
      tclocal(3) =     -7.21061d0*CON_KCALTOK
      tclocal(4) =      1.49105d0*CON_KCALTOK
      tclocal(5) =     -0.41584d0*CON_KCALTOK
      tclocal(6) =      1.21900d0*CON_KCALTOK
      tclocal(7) =      3.47446d0*CON_KCALTOK
      tclocal(8) =     -2.50597d0*CON_KCALTOK
      tclocal( 9) =     -1.52520d0*CON_KCALTOK
      tclocal(10) =      0.75636d0*CON_KCALTOK
      tclocal(11) =      0.66179d0*CON_KCALTOK
      tclocal(12) =     -9.66974d0*CON_KCALTOK
      tclocal(13) =      4.55274d0*CON_KCALTOK
      tclocal(14) =     -0.91118d0*CON_KCALTOK
      tclocal(15) =     -0.15612d0*CON_KCALTOK
      tclocal(16) =      1.84731d0*CON_KCALTOK
      tclocal(17) =     -1.12574d0*CON_KCALTOK
      tclocal(18) =    -11.70027d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.39765d0*CON_KCALTOK
      tclocal(1) =      1.72252d0*CON_KCALTOK
      tclocal(2) =     -0.86990d0*CON_KCALTOK
      tclocal(3) =     -7.95886d0*CON_KCALTOK
      tclocal(4) =      8.76775d0*CON_KCALTOK
      tclocal(5) =      1.08323d0*CON_KCALTOK
      tclocal(6) =      7.24992d0*CON_KCALTOK
      tclocal(7) =      8.30200d0*CON_KCALTOK
      tclocal(8) =     -0.90341d0*CON_KCALTOK
      tclocal( 9) =     -1.00376d0*CON_KCALTOK
      tclocal(10) =      1.27940d0*CON_KCALTOK
      tclocal(11) =     -1.45750d0*CON_KCALTOK
      tclocal(12) =     -4.30882d0*CON_KCALTOK
      tclocal(13) =     11.37146d0*CON_KCALTOK
      tclocal(14) =     -0.12169d0*CON_KCALTOK
      tclocal(15) =     -4.01741d0*CON_KCALTOK
      tclocal(16) =      1.33801d0*CON_KCALTOK
      tclocal(17) =     -1.69728d0*CON_KCALTOK
      tclocal(18) =    -17.71932d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.55691d0*CON_KCALTOK
      tclocal(1) =      2.47401d0*CON_KCALTOK
      tclocal(2) =     -0.50229d0*CON_KCALTOK
      tclocal(3) =      9.96567d0*CON_KCALTOK
      tclocal(4) =     -6.93805d0*CON_KCALTOK
      tclocal(5) =     -3.23503d0*CON_KCALTOK
      tclocal(6) =     17.60206d0*CON_KCALTOK
      tclocal(7) =      2.90902d0*CON_KCALTOK
      tclocal(8) =     -4.64335d0*CON_KCALTOK
      tclocal( 9) =     -4.21140d0*CON_KCALTOK
      tclocal(10) =     -3.35202d0*CON_KCALTOK
      tclocal(11) =     -0.37120d0*CON_KCALTOK
      tclocal(12) =      0.09344d0*CON_KCALTOK
      tclocal(13) =     -3.77505d0*CON_KCALTOK
      tclocal(14) =     -1.05862d0*CON_KCALTOK
      tclocal(15) =     -1.33657d0*CON_KCALTOK
      tclocal(16) =      0.60168d0*CON_KCALTOK
      tclocal(17) =     -1.49896d0*CON_KCALTOK
      tclocal(18) =    -21.25537d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'c` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'c` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.92132d0*CON_KCALTOK
      tclocal(1) =      1.21919d0*CON_KCALTOK
      tclocal(2) =      0.01518d0*CON_KCALTOK
      tclocal(3) =      4.44662d0*CON_KCALTOK
      tclocal(4) =      0.64289d0*CON_KCALTOK
      tclocal(5) =     -2.17288d0*CON_KCALTOK
      tclocal(6) =      5.55877d0*CON_KCALTOK
      tclocal(7) =      4.77446d0*CON_KCALTOK
      tclocal(8) =      4.05523d0*CON_KCALTOK
      tclocal( 9) =     -0.01382d0*CON_KCALTOK
      tclocal(10) =      2.11749d0*CON_KCALTOK
      tclocal(11) =      1.56619d0*CON_KCALTOK
      tclocal(12) =     -3.22511d0*CON_KCALTOK
      tclocal(13) =      5.39121d0*CON_KCALTOK
      tclocal(14) =     -2.09049d0*CON_KCALTOK
      tclocal(15) =     -2.18918d0*CON_KCALTOK
      tclocal(16) =      1.27470d0*CON_KCALTOK
      tclocal(17) =     -1.43933d0*CON_KCALTOK
      tclocal(18) =    -11.63641d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.85002d0*CON_KCALTOK
      tclocal(2) =     -0.62196d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -1.68408d0*CON_KCALTOK
      tclocal(5) =      0.55193d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      2.24893d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =     -2.00651d0*CON_KCALTOK
      tclocal(10) =     -1.49032d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      8.93165d0*CON_KCALTOK
      tclocal(13) =     -0.66039d0*CON_KCALTOK
      tclocal(14) =      0.15756d0*CON_KCALTOK
      tclocal(15) =      1.29003d0*CON_KCALTOK
      tclocal(16) =      1.62411d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -16.87728d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n` ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n` '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.12443d0*CON_KCALTOK
      tclocal(2) =     -0.50262d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.61140d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      6.46132d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =     -1.93840d0*CON_KCALTOK
      tclocal(10) =     -1.01405d0*CON_KCALTOK
      tclocal(11) =     -1.04341d0*CON_KCALTOK
      tclocal(12) =      3.04185d0*CON_KCALTOK
      tclocal(13) =      0.11856d0*CON_KCALTOK
      tclocal(14) =     -2.24173d0*CON_KCALTOK
      tclocal(15) =      4.42684d0*CON_KCALTOK
      tclocal(16) =      2.46497d0*CON_KCALTOK
      tclocal(17) =     -1.03669d0*CON_KCALTOK
      tclocal(18) =    -23.60411d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n` ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n` '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.16709d0*CON_KCALTOK
      tclocal(2) =     -0.45966d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -1.11493d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      2.34899d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      4.72582d0*CON_KCALTOK
      tclocal(10) =      0.42593d0*CON_KCALTOK
      tclocal(11) =      1.92557d0*CON_KCALTOK
      tclocal(12) =      5.24606d0*CON_KCALTOK
      tclocal(13) =      1.24318d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      1.09242d0*CON_KCALTOK
      tclocal(17) =      1.13988d0*CON_KCALTOK
      tclocal(18) =    -13.43199d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n` ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n` '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.53495d0*CON_KCALTOK
      tclocal(1) =      0.16497d0*CON_KCALTOK
      tclocal(2) =      0.05318d0*CON_KCALTOK
      tclocal(3) =      0.31097d0*CON_KCALTOK
      tclocal(4) =      0.08035d0*CON_KCALTOK
      tclocal(5) =      0.26021d0*CON_KCALTOK
      tclocal(6) =     -0.73574d0*CON_KCALTOK
      tclocal(7) =     -1.40107d0*CON_KCALTOK
      tclocal(8) =      0.90298d0*CON_KCALTOK
      tclocal( 9) =      0.05592d0*CON_KCALTOK
      tclocal(10) =      0.08066d0*CON_KCALTOK
      tclocal(11) =      0.22010d0*CON_KCALTOK
      tclocal(12) =      0.95275d0*CON_KCALTOK
      tclocal(13) =      1.10198d0*CON_KCALTOK
      tclocal(14) =      0.28044d0*CON_KCALTOK
      tclocal(15) =     -0.62147d0*CON_KCALTOK
      tclocal(16) =      1.23170d0*CON_KCALTOK
      tclocal(17) =     -0.11820d0*CON_KCALTOK
      tclocal(18) =     -8.43739d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n+ ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n+ '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.37453d0*CON_KCALTOK
      tclocal(1) =     -0.14001d0*CON_KCALTOK
      tclocal(2) =     -0.04311d0*CON_KCALTOK
      tclocal(3) =     -0.49432d0*CON_KCALTOK
      tclocal(4) =      0.70016d0*CON_KCALTOK
      tclocal(5) =      0.26687d0*CON_KCALTOK
      tclocal(6) =     -0.39586d0*CON_KCALTOK
      tclocal(7) =      0.79409d0*CON_KCALTOK
      tclocal(8) =      0.26896d0*CON_KCALTOK
      tclocal( 9) =      0.40436d0*CON_KCALTOK
      tclocal(10) =      0.44074d0*CON_KCALTOK
      tclocal(11) =     -0.17369d0*CON_KCALTOK
      tclocal(12) =     -2.20256d0*CON_KCALTOK
      tclocal(13) =      0.17201d0*CON_KCALTOK
      tclocal(14) =     -0.14641d0*CON_KCALTOK
      tclocal(15) =     -1.27766d0*CON_KCALTOK
      tclocal(16) =      0.74336d0*CON_KCALTOK
      tclocal(17) =     -0.06345d0*CON_KCALTOK
      tclocal(18) =    -11.13898d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n+ ')
      call twh_torstring(GLB_SET,type,4,'h+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n+ '
      tnlocal(nsame,4) = 'h+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.49995d0*CON_KCALTOK
      tclocal(1) =      0.31476d0*CON_KCALTOK
      tclocal(2) =     -0.12051d0*CON_KCALTOK
      tclocal(3) =      0.03448d0*CON_KCALTOK
      tclocal(4) =      0.19178d0*CON_KCALTOK
      tclocal(5) =     -0.25034d0*CON_KCALTOK
      tclocal(6) =      0.10422d0*CON_KCALTOK
      tclocal(7) =      0.23480d0*CON_KCALTOK
      tclocal(8) =     -1.73316d0*CON_KCALTOK
      tclocal( 9) =      0.32134d0*CON_KCALTOK
      tclocal(10) =     -0.07696d0*CON_KCALTOK
      tclocal(11) =      0.33039d0*CON_KCALTOK
      tclocal(12) =      0.98658d0*CON_KCALTOK
      tclocal(13) =      0.35887d0*CON_KCALTOK
      tclocal(14) =      0.52905d0*CON_KCALTOK
      tclocal(15) =     -1.05665d0*CON_KCALTOK
      tclocal(16) =      0.57993d0*CON_KCALTOK
      tclocal(17) =     -0.07109d0*CON_KCALTOK
      tclocal(18) =     -8.83696d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n+ ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n+ '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.67336d0*CON_KCALTOK
      tclocal(1) =     -0.12419d0*CON_KCALTOK
      tclocal(2) =     -0.22201d0*CON_KCALTOK
      tclocal(3) =     -0.24856d0*CON_KCALTOK
      tclocal(4) =      0.51535d0*CON_KCALTOK
      tclocal(5) =     -0.00821d0*CON_KCALTOK
      tclocal(6) =      0.33623d0*CON_KCALTOK
      tclocal(7) =      0.18945d0*CON_KCALTOK
      tclocal(8) =     -0.84435d0*CON_KCALTOK
      tclocal( 9) =     -0.01139d0*CON_KCALTOK
      tclocal(10) =      0.11922d0*CON_KCALTOK
      tclocal(11) =      0.13881d0*CON_KCALTOK
      tclocal(12) =     -0.82614d0*CON_KCALTOK
      tclocal(13) =      0.95977d0*CON_KCALTOK
      tclocal(14) =     -0.29398d0*CON_KCALTOK
      tclocal(15) =     -1.97712d0*CON_KCALTOK
      tclocal(16) =      0.88954d0*CON_KCALTOK
      tclocal(17) =     -0.37407d0*CON_KCALTOK
      tclocal(18) =    -14.93586d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n+ ')
      call twh_torstring(GLB_SET,type,4,'h+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n+ '
      tnlocal(nsame,4) = 'h+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      2.14959d0*CON_KCALTOK
      tclocal(1) =      0.67586d0*CON_KCALTOK
      tclocal(2) =      0.02562d0*CON_KCALTOK
      tclocal(3) =      0.24793d0*CON_KCALTOK
      tclocal(4) =      0.27268d0*CON_KCALTOK
      tclocal(5) =      0.19085d0*CON_KCALTOK
      tclocal(6) =      0.45438d0*CON_KCALTOK
      tclocal(7) =     -1.22773d0*CON_KCALTOK
      tclocal(8) =      0.85132d0*CON_KCALTOK
      tclocal( 9) =     -0.69324d0*CON_KCALTOK
      tclocal(10) =      0.45680d0*CON_KCALTOK
      tclocal(11) =     -0.15143d0*CON_KCALTOK
      tclocal(12) =      0.86815d0*CON_KCALTOK
      tclocal(13) =      0.65205d0*CON_KCALTOK
      tclocal(14) =      0.37011d0*CON_KCALTOK
      tclocal(15) =      1.53508d0*CON_KCALTOK
      tclocal(16) =      0.24958d0*CON_KCALTOK
      tclocal(17) =      0.46430d0*CON_KCALTOK
      tclocal(18) =     -9.51481d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n+ ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n+ '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.45190d0*CON_KCALTOK
      tclocal(1) =      0.38118d0*CON_KCALTOK
      tclocal(2) =     -0.01781d0*CON_KCALTOK
      tclocal(3) =     -0.78492d0*CON_KCALTOK
      tclocal(4) =      0.88845d0*CON_KCALTOK
      tclocal(5) =      0.42742d0*CON_KCALTOK
      tclocal(6) =      0.09009d0*CON_KCALTOK
      tclocal(7) =     -0.31135d0*CON_KCALTOK
      tclocal(8) =      0.17586d0*CON_KCALTOK
      tclocal( 9) =     -1.06056d0*CON_KCALTOK
      tclocal(10) =      0.27258d0*CON_KCALTOK
      tclocal(11) =     -0.08573d0*CON_KCALTOK
      tclocal(12) =     -2.48058d0*CON_KCALTOK
      tclocal(13) =      1.10757d0*CON_KCALTOK
      tclocal(14) =      0.22852d0*CON_KCALTOK
      tclocal(15) =      1.32228d0*CON_KCALTOK
      tclocal(16) =      1.17447d0*CON_KCALTOK
      tclocal(17) =      0.23317d0*CON_KCALTOK
      tclocal(18) =    -12.52141d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n+ ')
      call twh_torstring(GLB_SET,type,4,'h+ ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n+ '
      tnlocal(nsame,4) = 'h+ '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.83149d0*CON_KCALTOK
      tclocal(1) =      1.43689d0*CON_KCALTOK
      tclocal(2) =     -0.67997d0*CON_KCALTOK
      tclocal(3) =     -2.94731d0*CON_KCALTOK
      tclocal(4) =     -1.26732d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      2.37612d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     -6.99586d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -6.50184d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'n= ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'n= '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.28807d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     21.43939d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     15.94699d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      3.15121d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     14.15606d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.22074d0*CON_KCALTOK
      tclocal(1) =      1.94435d0*CON_KCALTOK
      tclocal(2) =     -0.14164d0*CON_KCALTOK
      tclocal(3) =     -1.73857d0*CON_KCALTOK
      tclocal(4) =     -0.80179d0*CON_KCALTOK
      tclocal(5) =     -0.27106d0*CON_KCALTOK
      tclocal(6) =    -11.16403d0*CON_KCALTOK
      tclocal(7) =     -4.85020d0*CON_KCALTOK
      tclocal(8) =     -3.85266d0*CON_KCALTOK
      tclocal( 9) =      6.45215d0*CON_KCALTOK
      tclocal(10) =     -7.11470d0*CON_KCALTOK
      tclocal(11) =     -1.62569d0*CON_KCALTOK
      tclocal(12) =     -7.20514d0*CON_KCALTOK
      tclocal(13) =      5.66013d0*CON_KCALTOK
      tclocal(14) =      1.27916d0*CON_KCALTOK
      tclocal(15) =      4.41320d0*CON_KCALTOK
      tclocal(16) =      3.14759d0*CON_KCALTOK
      tclocal(17) =      1.45927d0*CON_KCALTOK
      tclocal(18) =    -39.05669d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =     -2.06356d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -5.98038d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      7.28574d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -2.03487d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =    -20.00458d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -4.83929d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      1.54371d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.15856d0*CON_KCALTOK
      tclocal(1) =      3.36105d0*CON_KCALTOK
      tclocal(2) =     -0.38088d0*CON_KCALTOK
      tclocal(3) =     -1.68447d0*CON_KCALTOK
      tclocal(4) =     -0.69345d0*CON_KCALTOK
      tclocal(5) =      0.60351d0*CON_KCALTOK
      tclocal(6) =     -5.95895d0*CON_KCALTOK
      tclocal(7) =      4.82148d0*CON_KCALTOK
      tclocal(8) =      0.48631d0*CON_KCALTOK
      tclocal( 9) =     -1.32479d0*CON_KCALTOK
      tclocal(10) =     -5.07273d0*CON_KCALTOK
      tclocal(11) =      3.26773d0*CON_KCALTOK
      tclocal(12) =     -1.38563d0*CON_KCALTOK
      tclocal(13) =      6.05829d0*CON_KCALTOK
      tclocal(14) =     -0.31327d0*CON_KCALTOK
      tclocal(15) =     -3.50593d0*CON_KCALTOK
      tclocal(16) =      3.27119d0*CON_KCALTOK
      tclocal(17) =      1.24826d0*CON_KCALTOK
      tclocal(18) =    -16.35169d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.51958d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -2.89506d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      7.14605d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -4.73632d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -4.93512d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =    -14.00311d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -29.78865d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.42896d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.29726d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      6.62031d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.48035d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      1.81762d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -4.39331d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     10.93773d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.62208d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      3.89675d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     12.89311d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -2.89737d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      2.31577d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =    -10.58261d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -32.83899d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.28582d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      2.60867d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     13.97377d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.56318d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      5.61140d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -2.11396d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     10.66567d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.41895d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     10.19645d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      8.78676d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      2.23922d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     14.60749d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -5.95751d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -31.32088d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c+ ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c+ '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.17345d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      3.74308d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     20.86347d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      1.99828d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      3.60102d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      1.75949d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      7.19262d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c+ ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c+ '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      5.28994d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -2.31046d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     14.19178d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -6.16005d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      5.35508d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -9.12612d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -35.83511d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c= ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c= '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.95682d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      1.00454d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     10.98732d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -1.17833d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      1.94844d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -3.01418d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      4.06315d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c= ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c= '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.79064d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     16.07127d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     10.57250d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      3.24567d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -0.10587d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cl ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cl '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.55608d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     12.82789d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      9.41260d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.12475d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      8.22839d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cl ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cl '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.02126d0*CON_KCALTOK
      tclocal(1) =      3.64893d0*CON_KCALTOK
      tclocal(2) =      0.00589d0*CON_KCALTOK
      tclocal(3) =      0.64104d0*CON_KCALTOK
      tclocal(4) =     -2.13330d0*CON_KCALTOK
      tclocal(5) =      4.49891d0*CON_KCALTOK
      tclocal(6) =      0.04535d0*CON_KCALTOK
      tclocal(7) =      1.82227d0*CON_KCALTOK
      tclocal(8) =     -0.70990d0*CON_KCALTOK
      tclocal( 9) =      3.22324d0*CON_KCALTOK
      tclocal(10) =     -2.02888d0*CON_KCALTOK
      tclocal(11) =      0.11332d0*CON_KCALTOK
      tclocal(12) =     13.00052d0*CON_KCALTOK
      tclocal(13) =      7.90996d0*CON_KCALTOK
      tclocal(14) =      2.21686d0*CON_KCALTOK
      tclocal(15) =     -1.05062d0*CON_KCALTOK
      tclocal(16) =      9.80957d0*CON_KCALTOK
      tclocal(17) =      3.05859d0*CON_KCALTOK
      tclocal(18) =    -27.35507d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.02126d0*CON_KCALTOK
      tclocal(1) =      3.64893d0*CON_KCALTOK
      tclocal(2) =      0.00589d0*CON_KCALTOK
      tclocal(3) =      0.64104d0*CON_KCALTOK
      tclocal(4) =     -2.13330d0*CON_KCALTOK
      tclocal(5) =      4.49891d0*CON_KCALTOK
      tclocal(6) =      0.04535d0*CON_KCALTOK
      tclocal(7) =      1.82227d0*CON_KCALTOK
      tclocal(8) =     -0.70990d0*CON_KCALTOK
      tclocal( 9) =      3.22324d0*CON_KCALTOK
      tclocal(10) =     -2.02888d0*CON_KCALTOK
      tclocal(11) =      0.11332d0*CON_KCALTOK
      tclocal(12) =     13.00052d0*CON_KCALTOK
      tclocal(13) =      7.90996d0*CON_KCALTOK
      tclocal(14) =      2.21686d0*CON_KCALTOK
      tclocal(15) =     -1.05062d0*CON_KCALTOK
      tclocal(16) =      9.80957d0*CON_KCALTOK
      tclocal(17) =      3.05859d0*CON_KCALTOK
      tclocal(18) =    -27.35507d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.14077d0*CON_KCALTOK
      tclocal(1) =      1.11490d0*CON_KCALTOK
      tclocal(2) =     -0.17803d0*CON_KCALTOK
      tclocal(3) =      4.99328d0*CON_KCALTOK
      tclocal(4) =      0.75350d0*CON_KCALTOK
      tclocal(5) =     -2.13833d0*CON_KCALTOK
      tclocal(6) =     -2.54498d0*CON_KCALTOK
      tclocal(7) =      7.49732d0*CON_KCALTOK
      tclocal(8) =      6.39231d0*CON_KCALTOK
      tclocal( 9) =     -4.67034d0*CON_KCALTOK
      tclocal(10) =     -1.06476d0*CON_KCALTOK
      tclocal(11) =      4.54987d0*CON_KCALTOK
      tclocal(12) =    -10.14142d0*CON_KCALTOK
      tclocal(13) =     -5.58896d0*CON_KCALTOK
      tclocal(14) =      3.52537d0*CON_KCALTOK
      tclocal(15) =      2.08317d0*CON_KCALTOK
      tclocal(16) =     -6.11356d0*CON_KCALTOK
      tclocal(17) =      2.71815d0*CON_KCALTOK
      tclocal(18) =     -9.64721d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.14077d0*CON_KCALTOK
      tclocal(1) =      1.11490d0*CON_KCALTOK
      tclocal(2) =     -0.17803d0*CON_KCALTOK
      tclocal(3) =      4.99328d0*CON_KCALTOK
      tclocal(4) =      0.75350d0*CON_KCALTOK
      tclocal(5) =     -2.13833d0*CON_KCALTOK
      tclocal(6) =     -2.54498d0*CON_KCALTOK
      tclocal(7) =      7.49732d0*CON_KCALTOK
      tclocal(8) =      6.39231d0*CON_KCALTOK
      tclocal( 9) =     -4.67034d0*CON_KCALTOK
      tclocal(10) =     -1.06476d0*CON_KCALTOK
      tclocal(11) =      4.54987d0*CON_KCALTOK
      tclocal(12) =    -10.14142d0*CON_KCALTOK
      tclocal(13) =     -5.58896d0*CON_KCALTOK
      tclocal(14) =      3.52537d0*CON_KCALTOK
      tclocal(15) =      2.08317d0*CON_KCALTOK
      tclocal(16) =     -6.11356d0*CON_KCALTOK
      tclocal(17) =      2.71815d0*CON_KCALTOK
      tclocal(18) =     -9.64721d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.25828d0*CON_KCALTOK
      tclocal(1) =      0.41906d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.61949d0*CON_KCALTOK
      tclocal(4) =     -3.05898d0*CON_KCALTOK
      tclocal(5) =      0.08221d0*CON_KCALTOK
      tclocal(6) =      1.23962d0*CON_KCALTOK
      tclocal(7) =     -3.33121d0*CON_KCALTOK
      tclocal(8) =     -3.73618d0*CON_KCALTOK
      tclocal( 9) =     -3.29922d0*CON_KCALTOK
      tclocal(10) =      0.96586d0*CON_KCALTOK
      tclocal(11) =      0.44295d0*CON_KCALTOK
      tclocal(12) =      8.89284d0*CON_KCALTOK
      tclocal(13) =     -0.08980d0*CON_KCALTOK
      tclocal(14) =     -2.04757d0*CON_KCALTOK
      tclocal(15) =     -6.67993d0*CON_KCALTOK
      tclocal(16) =      4.54292d0*CON_KCALTOK
      tclocal(17) =      1.40803d0*CON_KCALTOK
      tclocal(18) =    -17.42309d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.03101d0*CON_KCALTOK
      tclocal(1) =      4.30487d0*CON_KCALTOK
      tclocal(2) =      0.07955d0*CON_KCALTOK
      tclocal(3) =     -3.31011d0*CON_KCALTOK
      tclocal(4) =     -9.97234d0*CON_KCALTOK
      tclocal(5) =      1.66007d0*CON_KCALTOK
      tclocal(6) =      9.58780d0*CON_KCALTOK
      tclocal(7) =      5.23124d0*CON_KCALTOK
      tclocal(8) =      2.99926d0*CON_KCALTOK
      tclocal( 9) =     -0.87849d0*CON_KCALTOK
      tclocal(10) =     -3.93098d0*CON_KCALTOK
      tclocal(11) =     -0.00202d0*CON_KCALTOK
      tclocal(12) =      7.97912d0*CON_KCALTOK
      tclocal(13) =      7.35321d0*CON_KCALTOK
      tclocal(14) =      1.91886d0*CON_KCALTOK
      tclocal(15) =      2.20485d0*CON_KCALTOK
      tclocal(16) =      5.71667d0*CON_KCALTOK
      tclocal(17) =      0.37888d0*CON_KCALTOK
      tclocal(18) =      5.95262d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.25828d0*CON_KCALTOK
      tclocal(1) =      0.41906d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.61949d0*CON_KCALTOK
      tclocal(4) =     -3.05898d0*CON_KCALTOK
      tclocal(5) =      0.08221d0*CON_KCALTOK
      tclocal(6) =      1.23962d0*CON_KCALTOK
      tclocal(7) =     -3.33121d0*CON_KCALTOK
      tclocal(8) =     -3.73618d0*CON_KCALTOK
      tclocal( 9) =     -3.29922d0*CON_KCALTOK
      tclocal(10) =      0.96586d0*CON_KCALTOK
      tclocal(11) =      0.44295d0*CON_KCALTOK
      tclocal(12) =      8.89284d0*CON_KCALTOK
      tclocal(13) =     -0.08980d0*CON_KCALTOK
      tclocal(14) =     -2.04757d0*CON_KCALTOK
      tclocal(15) =     -6.67993d0*CON_KCALTOK
      tclocal(16) =      4.54292d0*CON_KCALTOK
      tclocal(17) =      1.40803d0*CON_KCALTOK
      tclocal(18) =    -17.42309d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.51172d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.21517d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     18.08310d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -5.54739d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     12.62359d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      5.58993d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -7.76497d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'f  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'f  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.21961d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.25160d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     14.99841d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -0.88594d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     11.48589d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -1.82660d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -0.72357d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'f  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'f  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.51740d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.08553d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      9.49093d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      2.40139d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      1.68817d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      1.09433d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -1.72264d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.51740d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.08553d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      9.49093d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      2.40139d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      1.68817d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      3.11697d0*CON_KCALTOK
      tclocal(16) =      2.30906d0*CON_KCALTOK
      tclocal(17) =      0.51927d0*CON_KCALTOK
      tclocal(18) =     -1.72264d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.22074d0*CON_KCALTOK
      tclocal(1) =      1.94435d0*CON_KCALTOK
      tclocal(2) =     -0.14164d0*CON_KCALTOK
      tclocal(3) =     -1.73857d0*CON_KCALTOK
      tclocal(4) =     -0.80179d0*CON_KCALTOK
      tclocal(5) =     -0.27106d0*CON_KCALTOK
      tclocal(6) =    -11.16403d0*CON_KCALTOK
      tclocal(7) =     -4.85020d0*CON_KCALTOK
      tclocal(8) =     -3.85266d0*CON_KCALTOK
      tclocal( 9) =      6.45215d0*CON_KCALTOK
      tclocal(10) =     -7.11470d0*CON_KCALTOK
      tclocal(11) =     -1.62569d0*CON_KCALTOK
      tclocal(12) =     -7.20514d0*CON_KCALTOK
      tclocal(13) =      5.66013d0*CON_KCALTOK
      tclocal(14) =      1.27916d0*CON_KCALTOK
      tclocal(15) =      4.41320d0*CON_KCALTOK
      tclocal(16) =      3.14759d0*CON_KCALTOK
      tclocal(17) =      1.45927d0*CON_KCALTOK
      tclocal(18) =    -22.63363d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.22074d0*CON_KCALTOK
      tclocal(1) =      1.94435d0*CON_KCALTOK
      tclocal(2) =     -0.14164d0*CON_KCALTOK
      tclocal(3) =     -1.73857d0*CON_KCALTOK
      tclocal(4) =     -0.80179d0*CON_KCALTOK
      tclocal(5) =     -0.27106d0*CON_KCALTOK
      tclocal(6) =    -11.16403d0*CON_KCALTOK
      tclocal(7) =     -4.85020d0*CON_KCALTOK
      tclocal(8) =     -3.85266d0*CON_KCALTOK
      tclocal( 9) =      6.45215d0*CON_KCALTOK
      tclocal(10) =     -7.11470d0*CON_KCALTOK
      tclocal(11) =     -1.62569d0*CON_KCALTOK
      tclocal(12) =     -7.20514d0*CON_KCALTOK
      tclocal(13) =      5.66013d0*CON_KCALTOK
      tclocal(14) =      1.27916d0*CON_KCALTOK
      tclocal(15) =      4.41320d0*CON_KCALTOK
      tclocal(16) =      3.14759d0*CON_KCALTOK
      tclocal(17) =      1.45927d0*CON_KCALTOK
      tclocal(18) =    -22.63363d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.48362d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.54944d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     16.36806d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      1.45760d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -1.57318d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.32851d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -2.52986d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.15856d0*CON_KCALTOK
      tclocal(1) =      3.36105d0*CON_KCALTOK
      tclocal(2) =     -0.38088d0*CON_KCALTOK
      tclocal(3) =     -1.68447d0*CON_KCALTOK
      tclocal(4) =     -0.69345d0*CON_KCALTOK
      tclocal(5) =      0.60351d0*CON_KCALTOK
      tclocal(6) =     -5.95895d0*CON_KCALTOK
      tclocal(7) =      4.82148d0*CON_KCALTOK
      tclocal(8) =      0.48631d0*CON_KCALTOK
      tclocal( 9) =     -1.32479d0*CON_KCALTOK
      tclocal(10) =     -5.07273d0*CON_KCALTOK
      tclocal(11) =      3.26773d0*CON_KCALTOK
      tclocal(12) =     -1.38563d0*CON_KCALTOK
      tclocal(13) =      6.05829d0*CON_KCALTOK
      tclocal(14) =     -0.31327d0*CON_KCALTOK
      tclocal(15) =     -3.50593d0*CON_KCALTOK
      tclocal(16) =      3.27119d0*CON_KCALTOK
      tclocal(17) =      1.24826d0*CON_KCALTOK
      tclocal(18) =    -16.35169d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.06051d0*CON_KCALTOK
      tclocal(1) =      2.72297d0*CON_KCALTOK
      tclocal(2) =      0.48918d0*CON_KCALTOK
      tclocal(3) =     12.99527d0*CON_KCALTOK
      tclocal(4) =     -1.63574d0*CON_KCALTOK
      tclocal(5) =      8.83472d0*CON_KCALTOK
      tclocal(6) =     -7.93172d0*CON_KCALTOK
      tclocal(7) =     -3.93303d0*CON_KCALTOK
      tclocal(8) =     -6.83181d0*CON_KCALTOK
      tclocal( 9) =      6.54273d0*CON_KCALTOK
      tclocal(10) =    -11.52393d0*CON_KCALTOK
      tclocal(11) =      5.08658d0*CON_KCALTOK
      tclocal(12) =      3.66895d0*CON_KCALTOK
      tclocal(13) =     12.07491d0*CON_KCALTOK
      tclocal(14) =      3.82354d0*CON_KCALTOK
      tclocal(15) =      0.99361d0*CON_KCALTOK
      tclocal(16) =      8.89642d0*CON_KCALTOK
      tclocal(17) =      6.18353d0*CON_KCALTOK
      tclocal(18) =    -26.97102d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.63357d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -7.17155d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      7.74354d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -4.88818d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -3.49471d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -2.31293d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.38462d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     23.07667d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     18.86742d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      4.11834d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -15.52442d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.34357d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -8.48679d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     16.83182d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.24166d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     14.93197d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.52857d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     15.55674d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.06051d0*CON_KCALTOK
      tclocal(1) =      2.72297d0*CON_KCALTOK
      tclocal(2) =      0.48918d0*CON_KCALTOK
      tclocal(3) =     12.99527d0*CON_KCALTOK
      tclocal(4) =     -1.63574d0*CON_KCALTOK
      tclocal(5) =      8.83472d0*CON_KCALTOK
      tclocal(6) =     -7.93172d0*CON_KCALTOK
      tclocal(7) =     -3.93303d0*CON_KCALTOK
      tclocal(8) =     -6.83181d0*CON_KCALTOK
      tclocal( 9) =      6.54273d0*CON_KCALTOK
      tclocal(10) =    -11.52393d0*CON_KCALTOK
      tclocal(11) =      5.08658d0*CON_KCALTOK
      tclocal(12) =      3.66895d0*CON_KCALTOK
      tclocal(13) =     12.07491d0*CON_KCALTOK
      tclocal(14) =      3.82354d0*CON_KCALTOK
      tclocal(15) =      0.99361d0*CON_KCALTOK
      tclocal(16) =      8.89642d0*CON_KCALTOK
      tclocal(17) =      6.18353d0*CON_KCALTOK
      tclocal(18) =    -26.97102d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n+ ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n+ '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.90088d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      1.38969d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     16.11102d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -3.63296d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      1.66185d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      1.91554d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -0.57422d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n+ ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n+ '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.08560d0*CON_KCALTOK
      tclocal(1) =      2.62508d0*CON_KCALTOK
      tclocal(2) =     -0.29172d0*CON_KCALTOK
      tclocal(3) =     -2.57102d0*CON_KCALTOK
      tclocal(4) =      1.75871d0*CON_KCALTOK
      tclocal(5) =      1.65535d0*CON_KCALTOK
      tclocal(6) =      4.28903d0*CON_KCALTOK
      tclocal(7) =      3.62909d0*CON_KCALTOK
      tclocal(8) =      0.40635d0*CON_KCALTOK
      tclocal( 9) =     -1.17229d0*CON_KCALTOK
      tclocal(10) =     -3.53088d0*CON_KCALTOK
      tclocal(11) =     -1.83123d0*CON_KCALTOK
      tclocal(12) =      8.31065d0*CON_KCALTOK
      tclocal(13) =      6.06401d0*CON_KCALTOK
      tclocal(14) =     -0.32754d0*CON_KCALTOK
      tclocal(15) =     -9.32897d0*CON_KCALTOK
      tclocal(16) =      4.91947d0*CON_KCALTOK
      tclocal(17) =      1.24617d0*CON_KCALTOK
      tclocal(18) =     -9.59778d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00105d0*CON_KCALTOK
      tclocal(1) =      2.93499d0*CON_KCALTOK
      tclocal(2) =     -0.06558d0*CON_KCALTOK
      tclocal(3) =     14.39205d0*CON_KCALTOK
      tclocal(4) =      6.07815d0*CON_KCALTOK
      tclocal(5) =      7.12276d0*CON_KCALTOK
      tclocal(6) =     10.41197d0*CON_KCALTOK
      tclocal(7) =     15.77068d0*CON_KCALTOK
      tclocal(8) =      6.13202d0*CON_KCALTOK
      tclocal( 9) =     14.39205d0*CON_KCALTOK
      tclocal(10) =      6.07815d0*CON_KCALTOK
      tclocal(11) =      7.12276d0*CON_KCALTOK
      tclocal(12) =     -2.78899d0*CON_KCALTOK
      tclocal(13) =     -3.56781d0*CON_KCALTOK
      tclocal(14) =      5.02303d0*CON_KCALTOK
      tclocal(15) =     -9.92359d0*CON_KCALTOK
      tclocal(16) =     -9.76243d0*CON_KCALTOK
      tclocal(17) =      2.94042d0*CON_KCALTOK
      tclocal(18) =    -15.50940d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.08560d0*CON_KCALTOK
      tclocal(1) =      2.62508d0*CON_KCALTOK
      tclocal(2) =     -0.29172d0*CON_KCALTOK
      tclocal(3) =     -2.57102d0*CON_KCALTOK
      tclocal(4) =      1.75871d0*CON_KCALTOK
      tclocal(5) =      1.65535d0*CON_KCALTOK
      tclocal(6) =      4.28903d0*CON_KCALTOK
      tclocal(7) =      3.62909d0*CON_KCALTOK
      tclocal(8) =      0.40635d0*CON_KCALTOK
      tclocal( 9) =     -1.17229d0*CON_KCALTOK
      tclocal(10) =     -3.53088d0*CON_KCALTOK
      tclocal(11) =     -1.83123d0*CON_KCALTOK
      tclocal(12) =      8.31065d0*CON_KCALTOK
      tclocal(13) =      6.06401d0*CON_KCALTOK
      tclocal(14) =     -0.32754d0*CON_KCALTOK
      tclocal(15) =     -9.32897d0*CON_KCALTOK
      tclocal(16) =      4.91947d0*CON_KCALTOK
      tclocal(17) =      1.24617d0*CON_KCALTOK
      tclocal(18) =     -9.59778d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.63357d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -7.17155d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     21.00340d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -4.88818d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     18.54769d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     14.27360d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -22.46758d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nn ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nn '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.06051d0*CON_KCALTOK
      tclocal(1) =      2.72297d0*CON_KCALTOK
      tclocal(2) =      0.48918d0*CON_KCALTOK
      tclocal(3) =     12.99527d0*CON_KCALTOK
      tclocal(4) =     -1.63574d0*CON_KCALTOK
      tclocal(5) =      8.83472d0*CON_KCALTOK
      tclocal(6) =     -7.93172d0*CON_KCALTOK
      tclocal(7) =     -3.93303d0*CON_KCALTOK
      tclocal(8) =     -6.83181d0*CON_KCALTOK
      tclocal( 9) =      6.54273d0*CON_KCALTOK
      tclocal(10) =    -11.52393d0*CON_KCALTOK
      tclocal(11) =      5.08658d0*CON_KCALTOK
      tclocal(12) =      3.66895d0*CON_KCALTOK
      tclocal(13) =     12.07491d0*CON_KCALTOK
      tclocal(14) =      3.82354d0*CON_KCALTOK
      tclocal(15) =      0.99361d0*CON_KCALTOK
      tclocal(16) =      8.89642d0*CON_KCALTOK
      tclocal(17) =      6.18353d0*CON_KCALTOK
      tclocal(18) =    -26.97102d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nn ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nn '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.63357d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -7.17155d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      7.74354d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -4.88818d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -3.49471d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -2.31293d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nn ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nn '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.06051d0*CON_KCALTOK
      tclocal(1) =      2.72297d0*CON_KCALTOK
      tclocal(2) =      0.48918d0*CON_KCALTOK
      tclocal(3) =     12.99527d0*CON_KCALTOK
      tclocal(4) =     -1.63574d0*CON_KCALTOK
      tclocal(5) =      8.83472d0*CON_KCALTOK
      tclocal(6) =     -7.93172d0*CON_KCALTOK
      tclocal(7) =     -3.93303d0*CON_KCALTOK
      tclocal(8) =     -6.83181d0*CON_KCALTOK
      tclocal( 9) =     -1.32479d0*CON_KCALTOK
      tclocal(10) =     -5.07273d0*CON_KCALTOK
      tclocal(11) =      3.26773d0*CON_KCALTOK
      tclocal(12) =      3.66895d0*CON_KCALTOK
      tclocal(13) =     12.07491d0*CON_KCALTOK
      tclocal(14) =      3.82354d0*CON_KCALTOK
      tclocal(15) =     -3.50593d0*CON_KCALTOK
      tclocal(16) =      3.27119d0*CON_KCALTOK
      tclocal(17) =      1.24826d0*CON_KCALTOK
      tclocal(18) =    -26.97102d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nn ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nn '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.06941d0*CON_KCALTOK
      tclocal(1) =      3.46155d0*CON_KCALTOK
      tclocal(2) =     -0.57151d0*CON_KCALTOK
      tclocal(3) =      5.12313d0*CON_KCALTOK
      tclocal(4) =      3.68123d0*CON_KCALTOK
      tclocal(5) =      8.57607d0*CON_KCALTOK
      tclocal(6) =      1.69762d0*CON_KCALTOK
      tclocal(7) =     16.51112d0*CON_KCALTOK
      tclocal(8) =      1.29515d0*CON_KCALTOK
      tclocal( 9) =     -4.17516d0*CON_KCALTOK
      tclocal(10) =      4.44435d0*CON_KCALTOK
      tclocal(11) =     -1.69116d0*CON_KCALTOK
      tclocal(12) =     11.63241d0*CON_KCALTOK
      tclocal(13) =     16.32471d0*CON_KCALTOK
      tclocal(14) =      2.19385d0*CON_KCALTOK
      tclocal(15) =      2.72805d0*CON_KCALTOK
      tclocal(16) =     11.87338d0*CON_KCALTOK
      tclocal(17) =      3.95505d0*CON_KCALTOK
      tclocal(18) =    -28.67331d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.06941d0*CON_KCALTOK
      tclocal(1) =      3.46155d0*CON_KCALTOK
      tclocal(2) =     -0.57151d0*CON_KCALTOK
      tclocal(3) =      5.12313d0*CON_KCALTOK
      tclocal(4) =      3.68123d0*CON_KCALTOK
      tclocal(5) =      8.57607d0*CON_KCALTOK
      tclocal(6) =      1.69762d0*CON_KCALTOK
      tclocal(7) =     16.51112d0*CON_KCALTOK
      tclocal(8) =      1.29515d0*CON_KCALTOK
      tclocal( 9) =     -4.17516d0*CON_KCALTOK
      tclocal(10) =      4.44435d0*CON_KCALTOK
      tclocal(11) =     -1.69116d0*CON_KCALTOK
      tclocal(12) =     11.63241d0*CON_KCALTOK
      tclocal(13) =     16.32471d0*CON_KCALTOK
      tclocal(14) =      2.19385d0*CON_KCALTOK
      tclocal(15) =      2.72805d0*CON_KCALTOK
      tclocal(16) =     11.87338d0*CON_KCALTOK
      tclocal(17) =      3.95505d0*CON_KCALTOK
      tclocal(18) =    -28.67331d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'co ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'co '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.18617d0*CON_KCALTOK
      tclocal(1) =      3.39539d0*CON_KCALTOK
      tclocal(2) =     -0.25890d0*CON_KCALTOK
      tclocal(3) =      1.08202d0*CON_KCALTOK
      tclocal(4) =      3.85021d0*CON_KCALTOK
      tclocal(5) =      1.08402d0*CON_KCALTOK
      tclocal(6) =      4.26008d0*CON_KCALTOK
      tclocal(7) =      6.82008d0*CON_KCALTOK
      tclocal(8) =     -2.01709d0*CON_KCALTOK
      tclocal( 9) =      6.95794d0*CON_KCALTOK
      tclocal(10) =     -0.79778d0*CON_KCALTOK
      tclocal(11) =     -0.30757d0*CON_KCALTOK
      tclocal(12) =     -0.08713d0*CON_KCALTOK
      tclocal(13) =     -0.92276d0*CON_KCALTOK
      tclocal(14) =      0.02309d0*CON_KCALTOK
      tclocal(15) =      6.38736d0*CON_KCALTOK
      tclocal(16) =      4.54723d0*CON_KCALTOK
      tclocal(17) =      4.83943d0*CON_KCALTOK
      tclocal(18) =      2.55510d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.76207d0*CON_KCALTOK
      tclocal(1) =      2.99820d0*CON_KCALTOK
      tclocal(2) =     -0.63772d0*CON_KCALTOK
      tclocal(3) =      2.40423d0*CON_KCALTOK
      tclocal(4) =     -3.67656d0*CON_KCALTOK
      tclocal(5) =      3.18876d0*CON_KCALTOK
      tclocal(6) =     -5.41290d0*CON_KCALTOK
      tclocal(7) =     -0.53420d0*CON_KCALTOK
      tclocal(8) =      0.65941d0*CON_KCALTOK
      tclocal( 9) =     -3.64091d0*CON_KCALTOK
      tclocal(10) =     -3.05441d0*CON_KCALTOK
      tclocal(11) =     -0.30902d0*CON_KCALTOK
      tclocal(12) =     -7.12608d0*CON_KCALTOK
      tclocal(13) =      0.70382d0*CON_KCALTOK
      tclocal(14) =      1.10346d0*CON_KCALTOK
      tclocal(15) =     -7.83630d0*CON_KCALTOK
      tclocal(16) =      1.28845d0*CON_KCALTOK
      tclocal(17) =      1.42981d0*CON_KCALTOK
      tclocal(18) =    -16.23715d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.00604d0*CON_KCALTOK
      tclocal(1) =      2.84867d0*CON_KCALTOK
      tclocal(2) =      0.55423d0*CON_KCALTOK
      tclocal(3) =     -7.76920d0*CON_KCALTOK
      tclocal(4) =     -5.69610d0*CON_KCALTOK
      tclocal(5) =     -2.91541d0*CON_KCALTOK
      tclocal(6) =     -1.67001d0*CON_KCALTOK
      tclocal(7) =     -4.98350d0*CON_KCALTOK
      tclocal(8) =      7.99973d0*CON_KCALTOK
      tclocal( 9) =      3.33963d0*CON_KCALTOK
      tclocal(10) =     -1.55410d0*CON_KCALTOK
      tclocal(11) =      7.14706d0*CON_KCALTOK
      tclocal(12) =     14.53035d0*CON_KCALTOK
      tclocal(13) =     13.21181d0*CON_KCALTOK
      tclocal(14) =     -1.69322d0*CON_KCALTOK
      tclocal(15) =      2.09158d0*CON_KCALTOK
      tclocal(16) =     11.14065d0*CON_KCALTOK
      tclocal(17) =      4.72336d0*CON_KCALTOK
      tclocal(18) =     51.57005d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.39820d0*CON_KCALTOK
      tclocal(1) =      3.78168d0*CON_KCALTOK
      tclocal(2) =      1.17925d0*CON_KCALTOK
      tclocal(3) =      4.22255d0*CON_KCALTOK
      tclocal(4) =     -1.70760d0*CON_KCALTOK
      tclocal(5) =     -2.33567d0*CON_KCALTOK
      tclocal(6) =     -5.26288d0*CON_KCALTOK
      tclocal(7) =      5.97248d0*CON_KCALTOK
      tclocal(8) =     -7.21754d0*CON_KCALTOK
      tclocal( 9) =     16.45833d0*CON_KCALTOK
      tclocal(10) =    -11.01500d0*CON_KCALTOK
      tclocal(11) =      2.46553d0*CON_KCALTOK
      tclocal(12) =      0.32517d0*CON_KCALTOK
      tclocal(13) =     -0.44380d0*CON_KCALTOK
      tclocal(14) =      0.61470d0*CON_KCALTOK
      tclocal(15) =     -0.93663d0*CON_KCALTOK
      tclocal(16) =      0.69759d0*CON_KCALTOK
      tclocal(17) =     -0.23464d0*CON_KCALTOK
      tclocal(18) =    -26.66434d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.05273d0*CON_KCALTOK
      tclocal(1) =      2.93817d0*CON_KCALTOK
      tclocal(2) =     -1.14464d0*CON_KCALTOK
      tclocal(3) =     -4.28868d0*CON_KCALTOK
      tclocal(4) =     -1.96194d0*CON_KCALTOK
      tclocal(5) =      1.81428d0*CON_KCALTOK
      tclocal(6) =      4.40419d0*CON_KCALTOK
      tclocal(7) =      2.49092d0*CON_KCALTOK
      tclocal(8) =     -0.63771d0*CON_KCALTOK
      tclocal( 9) =     -3.06501d0*CON_KCALTOK
      tclocal(10) =     -1.73557d0*CON_KCALTOK
      tclocal(11) =      0.43597d0*CON_KCALTOK
      tclocal(12) =     -0.25485d0*CON_KCALTOK
      tclocal(13) =     -0.16845d0*CON_KCALTOK
      tclocal(14) =     -0.02611d0*CON_KCALTOK
      tclocal(15) =     -1.68274d0*CON_KCALTOK
      tclocal(16) =     -1.23328d0*CON_KCALTOK
      tclocal(17) =     -0.49191d0*CON_KCALTOK
      tclocal(18) =     -4.26044d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      5.59261d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      2.42176d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     26.14995d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -4.60150d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     18.22637d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -3.91881d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -16.78767d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'s" ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 's" '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.29709d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      2.60918d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     18.91990d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      2.08233d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      7.41993d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.31422d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     10.70056d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'s" ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 's" '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.27873d0*CON_KCALTOK
      tclocal(1) =      1.12063d0*CON_KCALTOK
      tclocal(2) =      0.10066d0*CON_KCALTOK
      tclocal(3) =      0.26296d0*CON_KCALTOK
      tclocal(4) =     -2.49078d0*CON_KCALTOK
      tclocal(5) =      1.09210d0*CON_KCALTOK
      tclocal(6) =      1.62160d0*CON_KCALTOK
      tclocal(7) =      6.64254d0*CON_KCALTOK
      tclocal(8) =      0.70432d0*CON_KCALTOK
      tclocal( 9) =      0.51986d0*CON_KCALTOK
      tclocal(10) =      1.51208d0*CON_KCALTOK
      tclocal(11) =      0.11936d0*CON_KCALTOK
      tclocal(12) =     -2.45958d0*CON_KCALTOK
      tclocal(13) =      6.68147d0*CON_KCALTOK
      tclocal(14) =      0.83772d0*CON_KCALTOK
      tclocal(15) =     -0.13666d0*CON_KCALTOK
      tclocal(16) =      1.79326d0*CON_KCALTOK
      tclocal(17) =      0.83071d0*CON_KCALTOK
      tclocal(18) =    -14.93467d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.14247d0*CON_KCALTOK
      tclocal(1) =      1.15143d0*CON_KCALTOK
      tclocal(2) =      0.08320d0*CON_KCALTOK
      tclocal(3) =     -1.61666d0*CON_KCALTOK
      tclocal(4) =     -3.40391d0*CON_KCALTOK
      tclocal(5) =      0.69603d0*CON_KCALTOK
      tclocal(6) =      0.73595d0*CON_KCALTOK
      tclocal(7) =      6.33789d0*CON_KCALTOK
      tclocal(8) =     -1.94840d0*CON_KCALTOK
      tclocal( 9) =      1.36019d0*CON_KCALTOK
      tclocal(10) =      0.78888d0*CON_KCALTOK
      tclocal(11) =      0.16499d0*CON_KCALTOK
      tclocal(12) =     -9.37341d0*CON_KCALTOK
      tclocal(13) =      3.04963d0*CON_KCALTOK
      tclocal(14) =     -1.22877d0*CON_KCALTOK
      tclocal(15) =      0.51555d0*CON_KCALTOK
      tclocal(16) =      0.77864d0*CON_KCALTOK
      tclocal(17) =     -0.87764d0*CON_KCALTOK
      tclocal(18) =     -7.72961d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.86931d0*CON_KCALTOK
      tclocal(1) =      0.94769d0*CON_KCALTOK
      tclocal(2) =      0.15550d0*CON_KCALTOK
      tclocal(3) =     -2.60020d0*CON_KCALTOK
      tclocal(4) =      3.51795d0*CON_KCALTOK
      tclocal(5) =     -2.46453d0*CON_KCALTOK
      tclocal(6) =      1.01155d0*CON_KCALTOK
      tclocal(7) =      4.20649d0*CON_KCALTOK
      tclocal(8) =      1.73205d0*CON_KCALTOK
      tclocal( 9) =      0.84489d0*CON_KCALTOK
      tclocal(10) =      4.80600d0*CON_KCALTOK
      tclocal(11) =      3.50845d0*CON_KCALTOK
      tclocal(12) =     -1.46197d0*CON_KCALTOK
      tclocal(13) =     10.06669d0*CON_KCALTOK
      tclocal(14) =      0.14625d0*CON_KCALTOK
      tclocal(15) =     -1.83772d0*CON_KCALTOK
      tclocal(16) =      3.16545d0*CON_KCALTOK
      tclocal(17) =     -0.93361d0*CON_KCALTOK
      tclocal(18) =    -18.52175d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.05946d0*CON_KCALTOK
      tclocal(1) =     -0.31269d0*CON_KCALTOK
      tclocal(2) =     -0.02335d0*CON_KCALTOK
      tclocal(3) =     -7.21061d0*CON_KCALTOK
      tclocal(4) =      1.49105d0*CON_KCALTOK
      tclocal(5) =     -0.41584d0*CON_KCALTOK
      tclocal(6) =      1.21900d0*CON_KCALTOK
      tclocal(7) =      3.47446d0*CON_KCALTOK
      tclocal(8) =     -2.50597d0*CON_KCALTOK
      tclocal( 9) =     -1.52520d0*CON_KCALTOK
      tclocal(10) =      0.75636d0*CON_KCALTOK
      tclocal(11) =      0.66179d0*CON_KCALTOK
      tclocal(12) =     -9.66974d0*CON_KCALTOK
      tclocal(13) =      4.55274d0*CON_KCALTOK
      tclocal(14) =     -0.91118d0*CON_KCALTOK
      tclocal(15) =     -0.15612d0*CON_KCALTOK
      tclocal(16) =      1.84731d0*CON_KCALTOK
      tclocal(17) =     -1.12574d0*CON_KCALTOK
      tclocal(18) =    -11.70027d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.39765d0*CON_KCALTOK
      tclocal(1) =      1.72252d0*CON_KCALTOK
      tclocal(2) =     -0.86990d0*CON_KCALTOK
      tclocal(3) =     -7.95886d0*CON_KCALTOK
      tclocal(4) =      8.76775d0*CON_KCALTOK
      tclocal(5) =      1.08323d0*CON_KCALTOK
      tclocal(6) =      7.24992d0*CON_KCALTOK
      tclocal(7) =      8.30200d0*CON_KCALTOK
      tclocal(8) =     -0.90341d0*CON_KCALTOK
      tclocal( 9) =     -1.00376d0*CON_KCALTOK
      tclocal(10) =      1.27940d0*CON_KCALTOK
      tclocal(11) =     -1.45750d0*CON_KCALTOK
      tclocal(12) =     -4.30882d0*CON_KCALTOK
      tclocal(13) =     11.37146d0*CON_KCALTOK
      tclocal(14) =     -0.12169d0*CON_KCALTOK
      tclocal(15) =     -4.01741d0*CON_KCALTOK
      tclocal(16) =      1.33801d0*CON_KCALTOK
      tclocal(17) =     -1.69728d0*CON_KCALTOK
      tclocal(18) =    -17.71932d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.63986d0*CON_KCALTOK
      tclocal(1) =      3.53760d0*CON_KCALTOK
      tclocal(2) =     -0.10549d0*CON_KCALTOK
      tclocal(3) =     -2.30644d0*CON_KCALTOK
      tclocal(4) =     -2.49751d0*CON_KCALTOK
      tclocal(5) =      2.06122d0*CON_KCALTOK
      tclocal(6) =      6.09782d0*CON_KCALTOK
      tclocal(7) =     12.66352d0*CON_KCALTOK
      tclocal(8) =      1.31170d0*CON_KCALTOK
      tclocal( 9) =      1.00475d0*CON_KCALTOK
      tclocal(10) =     -0.72763d0*CON_KCALTOK
      tclocal(11) =      0.04256d0*CON_KCALTOK
      tclocal(12) =     -6.93117d0*CON_KCALTOK
      tclocal(13) =      3.17139d0*CON_KCALTOK
      tclocal(14) =      0.99771d0*CON_KCALTOK
      tclocal(15) =      1.32993d0*CON_KCALTOK
      tclocal(16) =      1.60074d0*CON_KCALTOK
      tclocal(17) =     -0.62466d0*CON_KCALTOK
      tclocal(18) =     -9.61779d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      8.45054d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      1.23455d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     39.90289d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -18.39426d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -4.48632d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -1.67759d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -19.67213d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.10916d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     24.72213d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      5.62440d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      3.34566d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      7.18139d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -19.45697d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     32.27506d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -12.84867d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -8.83101d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =    -14.30701d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -22.00278d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c" ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c" '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      8.94184d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -5.81352d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     39.39101d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -12.09085d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.14073d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =    -11.61619d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -18.13190d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.58835d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -2.30750d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     34.30164d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -6.96077d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      1.46303d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -6.63592d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -35.35815d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.48653d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -2.49345d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     32.39576d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -8.03362d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -3.37732d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -5.42491d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -31.06250d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      8.94076d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -6.93414d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     41.17650d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -11.98409d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      9.26236d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =    -15.14057d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -17.65641d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c+ ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c+ '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      8.57714d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -4.65117d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     43.66953d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -12.89823d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      4.24550d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -7.59252d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -22.44985d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c= ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c= '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.53941d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     53.59150d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     13.70160d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     18.32332d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      2.70550d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cl ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cl '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10240d0*CON_KCALTOK
      tclocal(1) =      6.06300d0*CON_KCALTOK
      tclocal(2) =     -0.34381d0*CON_KCALTOK
      tclocal(3) =     -3.91736d0*CON_KCALTOK
      tclocal(4) =     -5.75943d0*CON_KCALTOK
      tclocal(5) =      1.19030d0*CON_KCALTOK
      tclocal(6) =     55.47661d0*CON_KCALTOK
      tclocal(7) =     12.30603d0*CON_KCALTOK
      tclocal(8) =     -0.93216d0*CON_KCALTOK
      tclocal( 9) =    -20.33975d0*CON_KCALTOK
      tclocal(10) =     -7.54007d0*CON_KCALTOK
      tclocal(11) =     -3.11424d0*CON_KCALTOK
      tclocal(12) =    -18.15219d0*CON_KCALTOK
      tclocal(13) =      7.41225d0*CON_KCALTOK
      tclocal(14) =      1.55575d0*CON_KCALTOK
      tclocal(15) =     12.57427d0*CON_KCALTOK
      tclocal(16) =     -3.35509d0*CON_KCALTOK
      tclocal(17) =      0.84449d0*CON_KCALTOK
      tclocal(18) =    -23.56553d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10240d0*CON_KCALTOK
      tclocal(1) =      6.06300d0*CON_KCALTOK
      tclocal(2) =     -0.34381d0*CON_KCALTOK
      tclocal(3) =     -3.91736d0*CON_KCALTOK
      tclocal(4) =     -5.75943d0*CON_KCALTOK
      tclocal(5) =      1.19030d0*CON_KCALTOK
      tclocal(6) =     55.47661d0*CON_KCALTOK
      tclocal(7) =     12.30603d0*CON_KCALTOK
      tclocal(8) =     -0.93216d0*CON_KCALTOK
      tclocal( 9) =    -20.33975d0*CON_KCALTOK
      tclocal(10) =     -7.54007d0*CON_KCALTOK
      tclocal(11) =     -3.11424d0*CON_KCALTOK
      tclocal(12) =    -18.15219d0*CON_KCALTOK
      tclocal(13) =      7.41225d0*CON_KCALTOK
      tclocal(14) =      1.55575d0*CON_KCALTOK
      tclocal(15) =     12.57427d0*CON_KCALTOK
      tclocal(16) =     -3.35509d0*CON_KCALTOK
      tclocal(17) =      0.84449d0*CON_KCALTOK
      tclocal(18) =    -23.56553d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.21825d0*CON_KCALTOK
      tclocal(1) =      7.11302d0*CON_KCALTOK
      tclocal(2) =     -0.04966d0*CON_KCALTOK
      tclocal(3) =    -16.22570d0*CON_KCALTOK
      tclocal(4) =    -21.00995d0*CON_KCALTOK
      tclocal(5) =     11.84145d0*CON_KCALTOK
      tclocal(6) =     65.28143d0*CON_KCALTOK
      tclocal(7) =     13.98449d0*CON_KCALTOK
      tclocal(8) =     -5.90249d0*CON_KCALTOK
      tclocal( 9) =    -10.47841d0*CON_KCALTOK
      tclocal(10) =     -3.50175d0*CON_KCALTOK
      tclocal(11) =     -6.39580d0*CON_KCALTOK
      tclocal(12) =    -25.54643d0*CON_KCALTOK
      tclocal(13) =     15.78673d0*CON_KCALTOK
      tclocal(14) =      2.43854d0*CON_KCALTOK
      tclocal(15) =      1.79540d0*CON_KCALTOK
      tclocal(16) =      6.33986d0*CON_KCALTOK
      tclocal(17) =      3.54517d0*CON_KCALTOK
      tclocal(18) =    -87.11735d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.37534d0*CON_KCALTOK
      tclocal(1) =      7.30275d0*CON_KCALTOK
      tclocal(2) =      0.07786d0*CON_KCALTOK
      tclocal(3) =     -0.46157d0*CON_KCALTOK
      tclocal(4) =      6.48393d0*CON_KCALTOK
      tclocal(5) =     -6.61708d0*CON_KCALTOK
      tclocal(6) =     45.94233d0*CON_KCALTOK
      tclocal(7) =     13.83163d0*CON_KCALTOK
      tclocal(8) =     -4.28317d0*CON_KCALTOK
      tclocal( 9) =     -8.83482d0*CON_KCALTOK
      tclocal(10) =     12.10557d0*CON_KCALTOK
      tclocal(11) =    -12.60398d0*CON_KCALTOK
      tclocal(12) =     -1.65306d0*CON_KCALTOK
      tclocal(13) =     16.61963d0*CON_KCALTOK
      tclocal(14) =     -4.21933d0*CON_KCALTOK
      tclocal(15) =     12.25186d0*CON_KCALTOK
      tclocal(16) =      6.66542d0*CON_KCALTOK
      tclocal(17) =     -4.53692d0*CON_KCALTOK
      tclocal(18) =    -28.89416d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =     10.64873d0*CON_KCALTOK
      tclocal(2) =     -3.82690d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -20.63452d0*CON_KCALTOK
      tclocal(5) =     17.61518d0*CON_KCALTOK
      tclocal(6) =     94.48789d0*CON_KCALTOK
      tclocal(7) =     22.65613d0*CON_KCALTOK
      tclocal(8) =    -22.80626d0*CON_KCALTOK
      tclocal( 9) =     53.89744d0*CON_KCALTOK
      tclocal(10) =     42.12952d0*CON_KCALTOK
      tclocal(11) =    -32.71372d0*CON_KCALTOK
      tclocal(12) =    -43.77643d0*CON_KCALTOK
      tclocal(13) =     -2.89012d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =    -47.91024d0*CON_KCALTOK
      tclocal(16) =    -38.14858d0*CON_KCALTOK
      tclocal(17) =     23.02462d0*CON_KCALTOK
      tclocal(18) =    -63.91177d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      9.44173d0*CON_KCALTOK
      tclocal(2) =     -5.48664d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -23.18543d0*CON_KCALTOK
      tclocal(5) =     16.12856d0*CON_KCALTOK
      tclocal(6) =     79.97757d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =     -8.57093d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     14.62184d0*CON_KCALTOK
      tclocal(11) =     -7.89922d0*CON_KCALTOK
      tclocal(12) =    -53.88348d0*CON_KCALTOK
      tclocal(13) =     -3.82603d0*CON_KCALTOK
      tclocal(14) =      8.36474d0*CON_KCALTOK
      tclocal(15) =    -32.25447d0*CON_KCALTOK
      tclocal(16) =    -25.03284d0*CON_KCALTOK
      tclocal(17) =     19.14937d0*CON_KCALTOK
      tclocal(18) =    -52.19081d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10240d0*CON_KCALTOK
      tclocal(1) =      6.06300d0*CON_KCALTOK
      tclocal(2) =     -0.34381d0*CON_KCALTOK
      tclocal(3) =     -3.91736d0*CON_KCALTOK
      tclocal(4) =     -5.75943d0*CON_KCALTOK
      tclocal(5) =      1.19030d0*CON_KCALTOK
      tclocal(6) =     55.47661d0*CON_KCALTOK
      tclocal(7) =     12.30603d0*CON_KCALTOK
      tclocal(8) =     -0.93216d0*CON_KCALTOK
      tclocal( 9) =    -20.33975d0*CON_KCALTOK
      tclocal(10) =     -7.54007d0*CON_KCALTOK
      tclocal(11) =     -3.11424d0*CON_KCALTOK
      tclocal(12) =    -18.15219d0*CON_KCALTOK
      tclocal(13) =      7.41225d0*CON_KCALTOK
      tclocal(14) =      1.55575d0*CON_KCALTOK
      tclocal(15) =     12.57427d0*CON_KCALTOK
      tclocal(16) =     -3.35509d0*CON_KCALTOK
      tclocal(17) =      0.84449d0*CON_KCALTOK
      tclocal(18) =    -23.56553d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.44245d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     65.15669d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     17.82864d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     22.16198d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -1.00170d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'f  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'f  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.98789d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.74313d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     37.50885d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -14.30748d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      4.39279d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -6.73226d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -17.77406d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.98789d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.74313d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     37.50885d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -14.30748d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      4.39279d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -6.73226d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -17.77406d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.27019d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      2.25921d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     36.86703d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -11.91271d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      9.31156d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -2.46046d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -21.43276d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.61644d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.30957d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     29.73646d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -7.75756d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      3.20807d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.89034d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -10.19179d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      7.53940d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      2.14405d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     42.21358d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     11.16292d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -15.74479d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.10916d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     24.72213d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      5.62440d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      3.34566d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      8.86023d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -17.15765d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     18.01261d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -16.17185d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =    -16.78555d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00760d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -16.94622d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.16830d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     60.24328d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     30.27569d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     14.33988d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      3.16777d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      7.64886d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -6.95768d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     58.46743d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -16.73631d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -0.67333d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -2.52771d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -20.21910d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'n+ ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'n+ '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.16552d0*CON_KCALTOK
      tclocal(1) =      6.10633d0*CON_KCALTOK
      tclocal(2) =     -0.40086d0*CON_KCALTOK
      tclocal(3) =     -1.91183d0*CON_KCALTOK
      tclocal(4) =     -4.71227d0*CON_KCALTOK
      tclocal(5) =      4.64328d0*CON_KCALTOK
      tclocal(6) =     42.09554d0*CON_KCALTOK
      tclocal(7) =     35.94446d0*CON_KCALTOK
      tclocal(8) =     -7.03218d0*CON_KCALTOK
      tclocal( 9) =    -16.10430d0*CON_KCALTOK
      tclocal(10) =    -18.27940d0*CON_KCALTOK
      tclocal(11) =      3.12360d0*CON_KCALTOK
      tclocal(12) =     -2.30827d0*CON_KCALTOK
      tclocal(13) =      7.62734d0*CON_KCALTOK
      tclocal(14) =      1.73865d0*CON_KCALTOK
      tclocal(15) =      4.01473d0*CON_KCALTOK
      tclocal(16) =      4.44395d0*CON_KCALTOK
      tclocal(17) =      5.73909d0*CON_KCALTOK
      tclocal(18) =    -45.19613d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.18784d0*CON_KCALTOK
      tclocal(1) =      5.74944d0*CON_KCALTOK
      tclocal(2) =     -1.81060d0*CON_KCALTOK
      tclocal(3) =      3.90207d0*CON_KCALTOK
      tclocal(4) =     -6.40864d0*CON_KCALTOK
      tclocal(5) =      2.25721d0*CON_KCALTOK
      tclocal(6) =     36.74109d0*CON_KCALTOK
      tclocal(7) =     31.38731d0*CON_KCALTOK
      tclocal(8) =     -2.85802d0*CON_KCALTOK
      tclocal( 9) =     -4.15351d0*CON_KCALTOK
      tclocal(10) =      0.10404d0*CON_KCALTOK
      tclocal(11) =     -0.85073d0*CON_KCALTOK
      tclocal(12) =     20.98515d0*CON_KCALTOK
      tclocal(13) =     23.21501d0*CON_KCALTOK
      tclocal(14) =     -9.71852d0*CON_KCALTOK
      tclocal(15) =      0.27026d0*CON_KCALTOK
      tclocal(16) =      9.80678d0*CON_KCALTOK
      tclocal(17) =      6.66992d0*CON_KCALTOK
      tclocal(18) =    -28.49270d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      8.86023d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -17.15765d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     18.01261d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -16.17185d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =    -16.78555d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00760d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -16.94622d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nn ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nn '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =     11.02646d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.39712d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     27.40584d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -11.91271d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -9.03275d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -2.46046d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -19.59726d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nn ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nn '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.61644d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.30957d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     29.73646d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -17.15765d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      3.20807d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =    -16.78555d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -10.19179d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nn ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nn '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.07952d0*CON_KCALTOK
      tclocal(1) =      5.78085d0*CON_KCALTOK
      tclocal(2) =     -0.18622d0*CON_KCALTOK
      tclocal(3) =    -24.02947d0*CON_KCALTOK
      tclocal(4) =     -6.37287d0*CON_KCALTOK
      tclocal(5) =      1.66849d0*CON_KCALTOK
      tclocal(6) =     39.49968d0*CON_KCALTOK
      tclocal(7) =     25.45245d0*CON_KCALTOK
      tclocal(8) =      1.00532d0*CON_KCALTOK
      tclocal( 9) =    -24.02947d0*CON_KCALTOK
      tclocal(10) =     -6.37287d0*CON_KCALTOK
      tclocal(11) =      1.66849d0*CON_KCALTOK
      tclocal(12) =     -7.88233d0*CON_KCALTOK
      tclocal(13) =     12.73951d0*CON_KCALTOK
      tclocal(14) =      1.40963d0*CON_KCALTOK
      tclocal(15) =     11.47688d0*CON_KCALTOK
      tclocal(16) =     -0.05033d0*CON_KCALTOK
      tclocal(17) =      0.77696d0*CON_KCALTOK
      tclocal(18) =    -16.19031d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.07952d0*CON_KCALTOK
      tclocal(1) =      5.78085d0*CON_KCALTOK
      tclocal(2) =     -0.18622d0*CON_KCALTOK
      tclocal(3) =    -24.02947d0*CON_KCALTOK
      tclocal(4) =     -6.37287d0*CON_KCALTOK
      tclocal(5) =      1.66849d0*CON_KCALTOK
      tclocal(6) =     39.49968d0*CON_KCALTOK
      tclocal(7) =     25.45245d0*CON_KCALTOK
      tclocal(8) =      1.00532d0*CON_KCALTOK
      tclocal( 9) =    -24.02947d0*CON_KCALTOK
      tclocal(10) =     -6.37287d0*CON_KCALTOK
      tclocal(11) =      1.66849d0*CON_KCALTOK
      tclocal(12) =     -7.88233d0*CON_KCALTOK
      tclocal(13) =     12.73951d0*CON_KCALTOK
      tclocal(14) =      1.40963d0*CON_KCALTOK
      tclocal(15) =     11.47688d0*CON_KCALTOK
      tclocal(16) =     -0.05033d0*CON_KCALTOK
      tclocal(17) =      0.77696d0*CON_KCALTOK
      tclocal(18) =    -16.19031d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.08324d0*CON_KCALTOK
      tclocal(1) =      4.38334d0*CON_KCALTOK
      tclocal(2) =     -0.28309d0*CON_KCALTOK
      tclocal(3) =     -6.81366d0*CON_KCALTOK
      tclocal(4) =    -31.94350d0*CON_KCALTOK
      tclocal(5) =      1.52291d0*CON_KCALTOK
      tclocal(6) =     60.63275d0*CON_KCALTOK
      tclocal(7) =     16.12044d0*CON_KCALTOK
      tclocal(8) =      0.01139d0*CON_KCALTOK
      tclocal( 9) =    -29.81630d0*CON_KCALTOK
      tclocal(10) =    -15.17859d0*CON_KCALTOK
      tclocal(11) =     -1.74000d0*CON_KCALTOK
      tclocal(12) =     28.05038d0*CON_KCALTOK
      tclocal(13) =     30.29008d0*CON_KCALTOK
      tclocal(14) =     -6.06805d0*CON_KCALTOK
      tclocal(15) =      8.31036d0*CON_KCALTOK
      tclocal(16) =     20.53851d0*CON_KCALTOK
      tclocal(17) =      2.79752d0*CON_KCALTOK
      tclocal(18) =    -20.48885d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.29348d0*CON_KCALTOK
      tclocal(1) =      7.94941d0*CON_KCALTOK
      tclocal(2) =      0.51909d0*CON_KCALTOK
      tclocal(3) =    -21.25451d0*CON_KCALTOK
      tclocal(4) =      2.74658d0*CON_KCALTOK
      tclocal(5) =     -3.82867d0*CON_KCALTOK
      tclocal(6) =     51.07684d0*CON_KCALTOK
      tclocal(7) =     21.26649d0*CON_KCALTOK
      tclocal(8) =    -15.51586d0*CON_KCALTOK
      tclocal( 9) =     -3.51061d0*CON_KCALTOK
      tclocal(10) =     22.69053d0*CON_KCALTOK
      tclocal(11) =    -15.29552d0*CON_KCALTOK
      tclocal(12) =     21.47278d0*CON_KCALTOK
      tclocal(13) =     14.74781d0*CON_KCALTOK
      tclocal(14) =    -15.04973d0*CON_KCALTOK
      tclocal(15) =      8.38004d0*CON_KCALTOK
      tclocal(16) =     11.94061d0*CON_KCALTOK
      tclocal(17) =     -0.00509d0*CON_KCALTOK
      tclocal(18) =      4.80478d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.29793d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -33.55749d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =     85.13625d0*CON_KCALTOK
      tclocal(7) =     35.80451d0*CON_KCALTOK
      tclocal(8) =    -12.93422d0*CON_KCALTOK
      tclocal( 9) =     44.43343d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     16.66613d0*CON_KCALTOK
      tclocal(13) =      7.14285d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     11.76895d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      5.38835d0*CON_KCALTOK
      tclocal(2) =      1.66577d0*CON_KCALTOK
      tclocal(3) =      6.62235d0*CON_KCALTOK
      tclocal(4) =    -39.73446d0*CON_KCALTOK
      tclocal(5) =     -9.89253d0*CON_KCALTOK
      tclocal(6) =     83.50647d0*CON_KCALTOK
      tclocal(7) =     19.60868d0*CON_KCALTOK
      tclocal(8) =    -12.95342d0*CON_KCALTOK
      tclocal( 9) =     34.78646d0*CON_KCALTOK
      tclocal(10) =     -0.99364d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     34.68258d0*CON_KCALTOK
      tclocal(13) =      6.55438d0*CON_KCALTOK
      tclocal(14) =     -9.26157d0*CON_KCALTOK
      tclocal(15) =     15.95467d0*CON_KCALTOK
      tclocal(16) =     30.18439d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.23721d0*CON_KCALTOK
      tclocal(1) =      5.48939d0*CON_KCALTOK
      tclocal(2) =     -0.45376d0*CON_KCALTOK
      tclocal(3) =    -19.62367d0*CON_KCALTOK
      tclocal(4) =      9.87993d0*CON_KCALTOK
      tclocal(5) =      6.20622d0*CON_KCALTOK
      tclocal(6) =    -28.80567d0*CON_KCALTOK
      tclocal(7) =     21.24309d0*CON_KCALTOK
      tclocal(8) =     -9.03072d0*CON_KCALTOK
      tclocal( 9) =     11.50587d0*CON_KCALTOK
      tclocal(10) =     -6.79374d0*CON_KCALTOK
      tclocal(11) =     -0.89703d0*CON_KCALTOK
      tclocal(12) =      2.33280d0*CON_KCALTOK
      tclocal(13) =     -0.83947d0*CON_KCALTOK
      tclocal(14) =     -1.63572d0*CON_KCALTOK
      tclocal(15) =     -6.71853d0*CON_KCALTOK
      tclocal(16) =      3.86032d0*CON_KCALTOK
      tclocal(17) =      0.85611d0*CON_KCALTOK
      tclocal(18) =    -24.74683d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'o  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'o  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.14359d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =     17.08255d0*CON_KCALTOK
      tclocal(6) =     46.34895d0*CON_KCALTOK
      tclocal(7) =     51.00625d0*CON_KCALTOK
      tclocal(8) =    -25.56989d0*CON_KCALTOK
      tclocal( 9) =     -8.08263d0*CON_KCALTOK
      tclocal(10) =    -24.43406d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =    -30.00980d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =     19.37609d0*CON_KCALTOK
      tclocal(15) =     -7.57418d0*CON_KCALTOK
      tclocal(16) =     -4.03854d0*CON_KCALTOK
      tclocal(17) =      8.26543d0*CON_KCALTOK
      tclocal(18) =    -13.78872d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'op ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'op '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.14359d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =     17.08255d0*CON_KCALTOK
      tclocal(6) =     46.34895d0*CON_KCALTOK
      tclocal(7) =     51.00625d0*CON_KCALTOK
      tclocal(8) =    -25.56989d0*CON_KCALTOK
      tclocal( 9) =     -8.08263d0*CON_KCALTOK
      tclocal(10) =    -24.43406d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =    -30.00980d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =     19.37609d0*CON_KCALTOK
      tclocal(15) =     -7.57418d0*CON_KCALTOK
      tclocal(16) =     -4.03854d0*CON_KCALTOK
      tclocal(17) =      8.26543d0*CON_KCALTOK
      tclocal(18) =    -13.78872d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'op ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'op '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      5.07687d0*CON_KCALTOK
      tclocal(2) =     -1.47129d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =     14.16754d0*CON_KCALTOK
      tclocal(6) =     17.34886d0*CON_KCALTOK
      tclocal(7) =     54.87574d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      2.26540d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     14.70551d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =     -8.87336d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'op ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'op '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =     10.19909d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -11.54411d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     52.68644d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -13.03999d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     11.91799d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -8.85359d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -18.23890d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'s" ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 's" '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      5.82375d0*CON_KCALTOK
      tclocal(2) =      0.73653d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -7.50892d0*CON_KCALTOK
      tclocal(5) =     -3.93669d0*CON_KCALTOK
      tclocal(6) =     14.10205d0*CON_KCALTOK
      tclocal(7) =     32.16866d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =     -5.30797d0*CON_KCALTOK
      tclocal(10) =    -26.43097d0*CON_KCALTOK
      tclocal(11) =     -6.28300d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =    -11.74811d0*CON_KCALTOK
      tclocal(14) =     -9.21681d0*CON_KCALTOK
      tclocal(15) =      6.72821d0*CON_KCALTOK
      tclocal(16) =     -1.92578d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -7.69337d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'sp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'sp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      5.82375d0*CON_KCALTOK
      tclocal(2) =      0.73653d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -7.50892d0*CON_KCALTOK
      tclocal(5) =     -3.93669d0*CON_KCALTOK
      tclocal(6) =     14.10205d0*CON_KCALTOK
      tclocal(7) =     32.16866d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =     -5.30797d0*CON_KCALTOK
      tclocal(10) =    -26.43097d0*CON_KCALTOK
      tclocal(11) =     -6.28300d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =    -11.74811d0*CON_KCALTOK
      tclocal(14) =     -9.21681d0*CON_KCALTOK
      tclocal(15) =      6.72821d0*CON_KCALTOK
      tclocal(16) =     -1.92578d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -7.69337d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'sp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'sp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.83390d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.08513d0*CON_KCALTOK
      tclocal(5) =      3.38557d0*CON_KCALTOK
      tclocal(6) =     10.58106d0*CON_KCALTOK
      tclocal(7) =     34.69605d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =     -5.98369d0*CON_KCALTOK
      tclocal(10) =     -5.30876d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      3.32692d0*CON_KCALTOK
      tclocal(14) =     -2.04487d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -2.29599d0*CON_KCALTOK
      tclocal(17) =     -1.24207d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'sp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'sp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.57451d0*CON_KCALTOK
      tclocal(1) =      1.39861d0*CON_KCALTOK
      tclocal(2) =      0.17479d0*CON_KCALTOK
      tclocal(3) =      1.46642d0*CON_KCALTOK
      tclocal(4) =     -1.82162d0*CON_KCALTOK
      tclocal(5) =      0.64339d0*CON_KCALTOK
      tclocal(6) =    -12.37527d0*CON_KCALTOK
      tclocal(7) =      2.82340d0*CON_KCALTOK
      tclocal(8) =     -1.02532d0*CON_KCALTOK
      tclocal( 9) =     -1.08955d0*CON_KCALTOK
      tclocal(10) =      1.86425d0*CON_KCALTOK
      tclocal(11) =     -0.20126d0*CON_KCALTOK
      tclocal(12) =     -2.25055d0*CON_KCALTOK
      tclocal(13) =      2.99571d0*CON_KCALTOK
      tclocal(14) =     -0.21198d0*CON_KCALTOK
      tclocal(15) =      2.75554d0*CON_KCALTOK
      tclocal(16) =      2.11375d0*CON_KCALTOK
      tclocal(17) =      0.03946d0*CON_KCALTOK
      tclocal(18) =    -11.63880d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.23846d0*CON_KCALTOK
      tclocal(1) =      0.73621d0*CON_KCALTOK
      tclocal(2) =      0.02564d0*CON_KCALTOK
      tclocal(3) =     -1.20040d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =     14.07372d0*CON_KCALTOK
      tclocal(7) =      5.58362d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -1.84049d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     -6.53282d0*CON_KCALTOK
      tclocal(13) =      4.76573d0*CON_KCALTOK
      tclocal(14) =     -0.90127d0*CON_KCALTOK
      tclocal(15) =      1.30114d0*CON_KCALTOK
      tclocal(16) =      0.06805d0*CON_KCALTOK
      tclocal(17) =      0.05077d0*CON_KCALTOK
      tclocal(18) =     -7.96267d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.18490d0*CON_KCALTOK
      tclocal(1) =      1.84254d0*CON_KCALTOK
      tclocal(2) =     -0.01920d0*CON_KCALTOK
      tclocal(3) =     -1.94727d0*CON_KCALTOK
      tclocal(4) =    -13.71715d0*CON_KCALTOK
      tclocal(5) =     -1.45349d0*CON_KCALTOK
      tclocal(6) =    -10.66467d0*CON_KCALTOK
      tclocal(7) =     -1.89282d0*CON_KCALTOK
      tclocal(8) =      2.85268d0*CON_KCALTOK
      tclocal( 9) =      1.25633d0*CON_KCALTOK
      tclocal(10) =     -0.65687d0*CON_KCALTOK
      tclocal(11) =     -0.41717d0*CON_KCALTOK
      tclocal(12) =     -1.72453d0*CON_KCALTOK
      tclocal(13) =     -1.75697d0*CON_KCALTOK
      tclocal(14) =     -0.11176d0*CON_KCALTOK
      tclocal(15) =      0.57353d0*CON_KCALTOK
      tclocal(16) =      0.42276d0*CON_KCALTOK
      tclocal(17) =      0.66627d0*CON_KCALTOK
      tclocal(18) =     -5.83153d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'p- ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'p- '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.09834d0*CON_KCALTOK
      tclocal(1) =      1.28532d0*CON_KCALTOK
      tclocal(2) =     -0.14947d0*CON_KCALTOK
      tclocal(3) =     -1.43992d0*CON_KCALTOK
      tclocal(4) =      6.63424d0*CON_KCALTOK
      tclocal(5) =     -0.40133d0*CON_KCALTOK
      tclocal(6) =     -5.70701d0*CON_KCALTOK
      tclocal(7) =      3.05887d0*CON_KCALTOK
      tclocal(8) =     -0.47794d0*CON_KCALTOK
      tclocal( 9) =      0.34974d0*CON_KCALTOK
      tclocal(10) =      0.53604d0*CON_KCALTOK
      tclocal(11) =     -0.45062d0*CON_KCALTOK
      tclocal(12) =      1.56745d0*CON_KCALTOK
      tclocal(13) =      3.29977d0*CON_KCALTOK
      tclocal(14) =      0.14051d0*CON_KCALTOK
      tclocal(15) =     -0.98378d0*CON_KCALTOK
      tclocal(16) =      2.25910d0*CON_KCALTOK
      tclocal(17) =     -0.32603d0*CON_KCALTOK
      tclocal(18) =    -21.58713d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.01017d0*CON_KCALTOK
      tclocal(1) =      0.08687d0*CON_KCALTOK
      tclocal(2) =     -0.10542d0*CON_KCALTOK
      tclocal(3) =     -5.97819d0*CON_KCALTOK
      tclocal(4) =      3.87826d0*CON_KCALTOK
      tclocal(5) =     -0.24943d0*CON_KCALTOK
      tclocal(6) =     15.90477d0*CON_KCALTOK
      tclocal(7) =     -0.71922d0*CON_KCALTOK
      tclocal(8) =     -2.14128d0*CON_KCALTOK
      tclocal( 9) =      0.03531d0*CON_KCALTOK
      tclocal(10) =     -0.80871d0*CON_KCALTOK
      tclocal(11) =      0.11266d0*CON_KCALTOK
      tclocal(12) =     -3.62691d0*CON_KCALTOK
      tclocal(13) =      2.25791d0*CON_KCALTOK
      tclocal(14) =     -0.95154d0*CON_KCALTOK
      tclocal(15) =     -1.88432d0*CON_KCALTOK
      tclocal(16) =      0.38325d0*CON_KCALTOK
      tclocal(17) =     -0.05725d0*CON_KCALTOK
      tclocal(18) =    -13.39170d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      1.20626d0*CON_KCALTOK
      tclocal(1) =      2.08720d0*CON_KCALTOK
      tclocal(2) =      0.19689d0*CON_KCALTOK
      tclocal(3) =      0.87925d0*CON_KCALTOK
      tclocal(4) =      2.82355d0*CON_KCALTOK
      tclocal(5) =      2.47918d0*CON_KCALTOK
      tclocal(6) =     -2.15314d0*CON_KCALTOK
      tclocal(7) =      7.74951d0*CON_KCALTOK
      tclocal(8) =      1.54209d0*CON_KCALTOK
      tclocal( 9) =     -5.23489d0*CON_KCALTOK
      tclocal(10) =      0.31844d0*CON_KCALTOK
      tclocal(11) =     -0.55612d0*CON_KCALTOK
      tclocal(12) =      4.99191d0*CON_KCALTOK
      tclocal(13) =      5.81273d0*CON_KCALTOK
      tclocal(14) =      1.32076d0*CON_KCALTOK
      tclocal(15) =     -0.16743d0*CON_KCALTOK
      tclocal(16) =      2.95431d0*CON_KCALTOK
      tclocal(17) =      0.40442d0*CON_KCALTOK
      tclocal(18) =    -25.14161d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.90445d0*CON_KCALTOK
      tclocal(1) =      1.21610d0*CON_KCALTOK
      tclocal(2) =      0.13973d0*CON_KCALTOK
      tclocal(3) =     -1.99726d0*CON_KCALTOK
      tclocal(4) =      1.08074d0*CON_KCALTOK
      tclocal(5) =      0.65711d0*CON_KCALTOK
      tclocal(6) =     16.00532d0*CON_KCALTOK
      tclocal(7) =      7.21276d0*CON_KCALTOK
      tclocal(8) =      1.63555d0*CON_KCALTOK
      tclocal( 9) =      1.21349d0*CON_KCALTOK
      tclocal(10) =     -0.10779d0*CON_KCALTOK
      tclocal(11) =     -0.69200d0*CON_KCALTOK
      tclocal(12) =     -4.24015d0*CON_KCALTOK
      tclocal(13) =      3.69584d0*CON_KCALTOK
      tclocal(14) =     -0.11987d0*CON_KCALTOK
      tclocal(15) =      0.19927d0*CON_KCALTOK
      tclocal(16) =      2.03351d0*CON_KCALTOK
      tclocal(17) =      0.15408d0*CON_KCALTOK
      tclocal(18) =    -14.43371d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'o  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'o  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      5.41566d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -5.16085d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     -2.88284d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      6.38572d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      5.24096d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -8.34688d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -30.37275d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'op ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'op '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.50000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =     -1.92472d0*CON_KCALTOK
      tclocal(6) =    -14.78112d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      9.06036d0*CON_KCALTOK
      tclocal(11) =     12.70801d0*CON_KCALTOK
      tclocal(12) =    -24.45041d0*CON_KCALTOK
      tclocal(13) =    -17.57623d0*CON_KCALTOK
      tclocal(14) =     11.55904d0*CON_KCALTOK
      tclocal(15) =    -18.72273d0*CON_KCALTOK
      tclocal(16) =    -15.90954d0*CON_KCALTOK
      tclocal(17) =     11.18271d0*CON_KCALTOK
      tclocal(18) =     -8.10749d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'op ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'op '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.50000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =     -1.92472d0*CON_KCALTOK
      tclocal(6) =    -14.78112d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      9.06036d0*CON_KCALTOK
      tclocal(11) =     12.70801d0*CON_KCALTOK
      tclocal(12) =    -24.45041d0*CON_KCALTOK
      tclocal(13) =    -17.57623d0*CON_KCALTOK
      tclocal(14) =     11.55904d0*CON_KCALTOK
      tclocal(15) =    -18.72273d0*CON_KCALTOK
      tclocal(16) =    -15.90954d0*CON_KCALTOK
      tclocal(17) =     11.18271d0*CON_KCALTOK
      tclocal(18) =     -8.10749d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'op ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'op '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.33214d0*CON_KCALTOK
      tclocal(2) =     -3.81549d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      2.50784d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     16.37369d0*CON_KCALTOK
      tclocal(8) =     11.52277d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =     -3.49586d0*CON_KCALTOK
      tclocal(12) =    -39.10426d0*CON_KCALTOK
      tclocal(13) =    -23.22887d0*CON_KCALTOK
      tclocal(14) =     15.21836d0*CON_KCALTOK
      tclocal(15) =    -36.65752d0*CON_KCALTOK
      tclocal(16) =    -27.78686d0*CON_KCALTOK
      tclocal(17) =     17.20320d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'op ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'op '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.50000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =     -1.92472d0*CON_KCALTOK
      tclocal(6) =    -14.78112d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      9.06036d0*CON_KCALTOK
      tclocal(11) =     12.70801d0*CON_KCALTOK
      tclocal(12) =    -24.45041d0*CON_KCALTOK
      tclocal(13) =    -17.57623d0*CON_KCALTOK
      tclocal(14) =     11.55904d0*CON_KCALTOK
      tclocal(15) =    -18.72273d0*CON_KCALTOK
      tclocal(16) =    -15.90954d0*CON_KCALTOK
      tclocal(17) =     11.18271d0*CON_KCALTOK
      tclocal(18) =     -8.10749d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'op ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'op '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.78379d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      2.24423d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      2.14667d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -22.45295d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'op ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'op '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.78379d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      2.24423d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      2.14667d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -22.45295d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'op ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'op '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.59513d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      2.86175d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      5.58512d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -17.14927d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'op ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'op '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      5.21676d0*CON_KCALTOK
      tclocal(2) =      1.46664d0*CON_KCALTOK
      tclocal(3) =     16.53261d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =    -11.49108d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     14.09302d0*CON_KCALTOK
      tclocal(11) =     11.17016d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -4.12087d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =     -3.33697d0*CON_KCALTOK
      tclocal(16) =    -13.75487d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -10.31435d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'op ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'op '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.72004d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     16.62125d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     33.11155d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =     48.42264d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =    -10.21041d0*CON_KCALTOK
      tclocal(17) =     11.34416d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'op ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'op '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.23608d0*CON_KCALTOK
      tclocal(1) =      0.22305d0*CON_KCALTOK
      tclocal(2) =      0.02443d0*CON_KCALTOK
      tclocal(3) =      1.39367d0*CON_KCALTOK
      tclocal(4) =     -0.57802d0*CON_KCALTOK
      tclocal(5) =      1.48373d0*CON_KCALTOK
      tclocal(6) =     -1.12763d0*CON_KCALTOK
      tclocal(7) =      2.01856d0*CON_KCALTOK
      tclocal(8) =      0.13374d0*CON_KCALTOK
      tclocal( 9) =     -0.40357d0*CON_KCALTOK
      tclocal(10) =      1.53544d0*CON_KCALTOK
      tclocal(11) =     -0.01334d0*CON_KCALTOK
      tclocal(12) =     -3.15776d0*CON_KCALTOK
      tclocal(13) =      2.04963d0*CON_KCALTOK
      tclocal(14) =      0.28089d0*CON_KCALTOK
      tclocal(15) =      1.41870d0*CON_KCALTOK
      tclocal(16) =      0.80638d0*CON_KCALTOK
      tclocal(17) =     -0.05714d0*CON_KCALTOK
      tclocal(18) =    -14.39357d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'s  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 's  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.27010d0*CON_KCALTOK
      tclocal(1) =     -0.04063d0*CON_KCALTOK
      tclocal(2) =      0.06483d0*CON_KCALTOK
      tclocal(3) =     -0.39581d0*CON_KCALTOK
      tclocal(4) =     -0.38678d0*CON_KCALTOK
      tclocal(5) =      0.35319d0*CON_KCALTOK
      tclocal(6) =     -0.24152d0*CON_KCALTOK
      tclocal(7) =     -0.76528d0*CON_KCALTOK
      tclocal(8) =     -0.19077d0*CON_KCALTOK
      tclocal( 9) =     -0.72234d0*CON_KCALTOK
      tclocal(10) =      0.65818d0*CON_KCALTOK
      tclocal(11) =      0.15225d0*CON_KCALTOK
      tclocal(12) =      4.55908d0*CON_KCALTOK
      tclocal(13) =     -0.95033d0*CON_KCALTOK
      tclocal(14) =     -0.27592d0*CON_KCALTOK
      tclocal(15) =      1.20136d0*CON_KCALTOK
      tclocal(16) =      2.48649d0*CON_KCALTOK
      tclocal(17) =      0.37716d0*CON_KCALTOK
      tclocal(18) =    -13.47431d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.13105d0*CON_KCALTOK
      tclocal(1) =      0.72659d0*CON_KCALTOK
      tclocal(2) =      0.26712d0*CON_KCALTOK
      tclocal(3) =     -0.34615d0*CON_KCALTOK
      tclocal(4) =      0.13419d0*CON_KCALTOK
      tclocal(5) =      0.68906d0*CON_KCALTOK
      tclocal(6) =     -0.17975d0*CON_KCALTOK
      tclocal(7) =     -0.46541d0*CON_KCALTOK
      tclocal(8) =      0.27119d0*CON_KCALTOK
      tclocal( 9) =     -0.95646d0*CON_KCALTOK
      tclocal(10) =      1.15936d0*CON_KCALTOK
      tclocal(11) =     -0.16683d0*CON_KCALTOK
      tclocal(12) =      6.09596d0*CON_KCALTOK
      tclocal(13) =     -0.57771d0*CON_KCALTOK
      tclocal(14) =     -0.30484d0*CON_KCALTOK
      tclocal(15) =      0.90227d0*CON_KCALTOK
      tclocal(16) =      4.34109d0*CON_KCALTOK
      tclocal(17) =      1.09899d0*CON_KCALTOK
      tclocal(18) =    -12.28509d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.51743d0*CON_KCALTOK
      tclocal(1) =      0.81656d0*CON_KCALTOK
      tclocal(2) =      0.25715d0*CON_KCALTOK
      tclocal(3) =     -0.90598d0*CON_KCALTOK
      tclocal(4) =     -0.97965d0*CON_KCALTOK
      tclocal(5) =      0.35347d0*CON_KCALTOK
      tclocal(6) =     -0.19168d0*CON_KCALTOK
      tclocal(7) =      0.16249d0*CON_KCALTOK
      tclocal(8) =      0.47810d0*CON_KCALTOK
      tclocal( 9) =     -1.62409d0*CON_KCALTOK
      tclocal(10) =      0.47333d0*CON_KCALTOK
      tclocal(11) =      0.19201d0*CON_KCALTOK
      tclocal(12) =      8.72806d0*CON_KCALTOK
      tclocal(13) =     -0.62151d0*CON_KCALTOK
      tclocal(14) =      1.11490d0*CON_KCALTOK
      tclocal(15) =      0.88227d0*CON_KCALTOK
      tclocal(16) =      2.58857d0*CON_KCALTOK
      tclocal(17) =     -0.23450d0*CON_KCALTOK
      tclocal(18) =    -21.84095d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.48597d0*CON_KCALTOK
      tclocal(1) =     -0.30217d0*CON_KCALTOK
      tclocal(2) =      0.18652d0*CON_KCALTOK
      tclocal(3) =     -1.36193d0*CON_KCALTOK
      tclocal(4) =     -0.27136d0*CON_KCALTOK
      tclocal(5) =      0.26506d0*CON_KCALTOK
      tclocal(6) =      0.32326d0*CON_KCALTOK
      tclocal(7) =     -0.68664d0*CON_KCALTOK
      tclocal(8) =     -0.24456d0*CON_KCALTOK
      tclocal( 9) =     -0.20289d0*CON_KCALTOK
      tclocal(10) =      0.53059d0*CON_KCALTOK
      tclocal(11) =      0.02891d0*CON_KCALTOK
      tclocal(12) =      2.94808d0*CON_KCALTOK
      tclocal(13) =     -0.57266d0*CON_KCALTOK
      tclocal(14) =      1.01841d0*CON_KCALTOK
      tclocal(15) =     -1.16965d0*CON_KCALTOK
      tclocal(16) =      4.51584d0*CON_KCALTOK
      tclocal(17) =     -0.09067d0*CON_KCALTOK
      tclocal(18) =     -5.53229d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.06103d0*CON_KCALTOK
      tclocal(1) =      0.36245d0*CON_KCALTOK
      tclocal(2) =     -0.24068d0*CON_KCALTOK
      tclocal(3) =      0.32416d0*CON_KCALTOK
      tclocal(4) =      0.34347d0*CON_KCALTOK
      tclocal(5) =      1.03362d0*CON_KCALTOK
      tclocal(6) =     -0.24412d0*CON_KCALTOK
      tclocal(7) =     -0.01276d0*CON_KCALTOK
      tclocal(8) =      0.13738d0*CON_KCALTOK
      tclocal( 9) =     -0.45392d0*CON_KCALTOK
      tclocal(10) =      1.31510d0*CON_KCALTOK
      tclocal(11) =     -0.20186d0*CON_KCALTOK
      tclocal(12) =      3.20405d0*CON_KCALTOK
      tclocal(13) =     -2.14365d0*CON_KCALTOK
      tclocal(14) =     -0.95651d0*CON_KCALTOK
      tclocal(15) =      0.45529d0*CON_KCALTOK
      tclocal(16) =      4.48804d0*CON_KCALTOK
      tclocal(17) =      0.61003d0*CON_KCALTOK
      tclocal(18) =     -6.57951d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.27070d0*CON_KCALTOK
      tclocal(1) =      0.52677d0*CON_KCALTOK
      tclocal(2) =     -0.08088d0*CON_KCALTOK
      tclocal(3) =      2.61911d0*CON_KCALTOK
      tclocal(4) =     -1.39312d0*CON_KCALTOK
      tclocal(5) =      0.53644d0*CON_KCALTOK
      tclocal(6) =     -0.01999d0*CON_KCALTOK
      tclocal(7) =      0.44505d0*CON_KCALTOK
      tclocal(8) =      0.10536d0*CON_KCALTOK
      tclocal( 9) =     -1.00672d0*CON_KCALTOK
      tclocal(10) =     -0.08298d0*CON_KCALTOK
      tclocal(11) =      1.32575d0*CON_KCALTOK
      tclocal(12) =      4.48711d0*CON_KCALTOK
      tclocal(13) =     -1.43500d0*CON_KCALTOK
      tclocal(14) =      0.13285d0*CON_KCALTOK
      tclocal(15) =     -0.45289d0*CON_KCALTOK
      tclocal(16) =      2.86804d0*CON_KCALTOK
      tclocal(17) =      0.46257d0*CON_KCALTOK
      tclocal(18) =    -18.55511d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.18475d0*CON_KCALTOK
      tclocal(1) =      0.39515d0*CON_KCALTOK
      tclocal(2) =      0.06502d0*CON_KCALTOK
      tclocal(3) =     -0.23951d0*CON_KCALTOK
      tclocal(4) =      0.41741d0*CON_KCALTOK
      tclocal(5) =      0.44195d0*CON_KCALTOK
      tclocal(6) =      0.70610d0*CON_KCALTOK
      tclocal(7) =     -1.15011d0*CON_KCALTOK
      tclocal(8) =      0.15306d0*CON_KCALTOK
      tclocal( 9) =     -0.15665d0*CON_KCALTOK
      tclocal(10) =      0.83772d0*CON_KCALTOK
      tclocal(11) =     -0.17978d0*CON_KCALTOK
      tclocal(12) =      4.81588d0*CON_KCALTOK
      tclocal(13) =     -0.64526d0*CON_KCALTOK
      tclocal(14) =      0.85116d0*CON_KCALTOK
      tclocal(15) =      1.13847d0*CON_KCALTOK
      tclocal(16) =      0.69986d0*CON_KCALTOK
      tclocal(17) =      0.15867d0*CON_KCALTOK
      tclocal(18) =    -12.89895d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.04342d0*CON_KCALTOK
      tclocal(1) =      0.53015d0*CON_KCALTOK
      tclocal(2) =     -0.35560d0*CON_KCALTOK
      tclocal(3) =      1.16822d0*CON_KCALTOK
      tclocal(4) =      0.50811d0*CON_KCALTOK
      tclocal(5) =      0.27025d0*CON_KCALTOK
      tclocal(6) =      0.54480d0*CON_KCALTOK
      tclocal(7) =     -0.50878d0*CON_KCALTOK
      tclocal(8) =     -0.40435d0*CON_KCALTOK
      tclocal( 9) =      0.55297d0*CON_KCALTOK
      tclocal(10) =      2.54921d0*CON_KCALTOK
      tclocal(11) =     -0.12896d0*CON_KCALTOK
      tclocal(12) =      6.25459d0*CON_KCALTOK
      tclocal(13) =     -0.88484d0*CON_KCALTOK
      tclocal(14) =     -0.81862d0*CON_KCALTOK
      tclocal(15) =     -1.07371d0*CON_KCALTOK
      tclocal(16) =      0.20315d0*CON_KCALTOK
      tclocal(17) =      0.22941d0*CON_KCALTOK
      tclocal(18) =    -11.11335d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'n  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'n  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.21535d0*CON_KCALTOK
      tclocal(1) =      0.27048d0*CON_KCALTOK
      tclocal(2) =     -0.44395d0*CON_KCALTOK
      tclocal(3) =      1.73122d0*CON_KCALTOK
      tclocal(4) =     -0.37830d0*CON_KCALTOK
      tclocal(5) =      0.57292d0*CON_KCALTOK
      tclocal(6) =     -0.69133d0*CON_KCALTOK
      tclocal(7) =     -0.52365d0*CON_KCALTOK
      tclocal(8) =     -0.59415d0*CON_KCALTOK
      tclocal( 9) =      1.62605d0*CON_KCALTOK
      tclocal(10) =      0.72936d0*CON_KCALTOK
      tclocal(11) =     -0.36641d0*CON_KCALTOK
      tclocal(12) =      7.02984d0*CON_KCALTOK
      tclocal(13) =     -1.11301d0*CON_KCALTOK
      tclocal(14) =     -0.09494d0*CON_KCALTOK
      tclocal(15) =     -1.84219d0*CON_KCALTOK
      tclocal(16) =      1.68317d0*CON_KCALTOK
      tclocal(17) =     -0.27625d0*CON_KCALTOK
      tclocal(18) =    -19.95390d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.23724d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -4.33700d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.67373d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      3.73806d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.11166d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -3.72662d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -32.20814d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c` ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'sp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c` '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'sp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.45208d0*CON_KCALTOK
      tclocal(1) =      1.20000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      4.17160d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.88364d0*CON_KCALTOK
      tclocal(6) =    -29.54916d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =     18.93905d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -2.18664d0*CON_KCALTOK
      tclocal(11) =      1.83028d0*CON_KCALTOK
      tclocal(12) =    -20.83713d0*CON_KCALTOK
      tclocal(13) =    -16.80435d0*CON_KCALTOK
      tclocal(14) =     14.36127d0*CON_KCALTOK
      tclocal(15) =    -22.73501d0*CON_KCALTOK
      tclocal(16) =    -16.29053d0*CON_KCALTOK
      tclocal(17) =     11.70823d0*CON_KCALTOK
      tclocal(18) =    -18.77487d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'sp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'sp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.45208d0*CON_KCALTOK
      tclocal(1) =      1.20000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      4.17160d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.88364d0*CON_KCALTOK
      tclocal(6) =    -29.54916d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =     18.93905d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -2.18664d0*CON_KCALTOK
      tclocal(11) =      1.83028d0*CON_KCALTOK
      tclocal(12) =    -20.83713d0*CON_KCALTOK
      tclocal(13) =    -16.80435d0*CON_KCALTOK
      tclocal(14) =     14.36127d0*CON_KCALTOK
      tclocal(15) =    -22.73501d0*CON_KCALTOK
      tclocal(16) =    -16.29053d0*CON_KCALTOK
      tclocal(17) =     11.70823d0*CON_KCALTOK
      tclocal(18) =    -18.77487d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'sp ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'sp '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -3.12890d0*CON_KCALTOK
      tclocal(1) =      2.50000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      1.35350d0*CON_KCALTOK
      tclocal(5) =      7.25953d0*CON_KCALTOK
      tclocal(6) =    -26.53370d0*CON_KCALTOK
      tclocal(7) =    -11.31796d0*CON_KCALTOK
      tclocal(8) =     14.11698d0*CON_KCALTOK
      tclocal( 9) =    -12.04108d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =     10.94827d0*CON_KCALTOK
      tclocal(12) =    -13.67621d0*CON_KCALTOK
      tclocal(13) =    -26.00040d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =    -96.40000d0*CON_KCALTOK
      tclocal(16) =    -61.56557d0*CON_KCALTOK
      tclocal(17) =     45.23009d0*CON_KCALTOK
      tclocal(18) =    -22.12262d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'sp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'sp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.45208d0*CON_KCALTOK
      tclocal(1) =      1.20000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      4.17160d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.88364d0*CON_KCALTOK
      tclocal(6) =    -29.54916d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =     18.93905d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -2.18664d0*CON_KCALTOK
      tclocal(11) =      1.83028d0*CON_KCALTOK
      tclocal(12) =    -20.83713d0*CON_KCALTOK
      tclocal(13) =    -16.80435d0*CON_KCALTOK
      tclocal(14) =     14.36127d0*CON_KCALTOK
      tclocal(15) =    -22.73501d0*CON_KCALTOK
      tclocal(16) =    -16.29053d0*CON_KCALTOK
      tclocal(17) =     11.70823d0*CON_KCALTOK
      tclocal(18) =    -18.77487d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'sp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'sp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.37287d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.72779d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -4.89891d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -6.95941d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -7.50041d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -32.06542d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'sp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'sp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.37287d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.72779d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -4.89891d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -6.95941d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -7.50041d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -32.06542d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'sp ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'sp '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      2.02638d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -9.16564d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -28.85110d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'sp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'sp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.17422d0*CON_KCALTOK
      tclocal(1) =      4.44047d0*CON_KCALTOK
      tclocal(2) =      1.63136d0*CON_KCALTOK
      tclocal(3) =     25.31599d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =     13.55166d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =     -6.18351d0*CON_KCALTOK
      tclocal( 9) =     -4.49945d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      2.43278d0*CON_KCALTOK
      tclocal(12) =      7.52124d0*CON_KCALTOK
      tclocal(13) =      9.16738d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -4.54030d0*CON_KCALTOK
      tclocal(17) =     -3.69552d0*CON_KCALTOK
      tclocal(18) =    -14.68984d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'sp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'sp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.17422d0*CON_KCALTOK
      tclocal(1) =      4.44047d0*CON_KCALTOK
      tclocal(2) =      1.63136d0*CON_KCALTOK
      tclocal(3) =     25.31599d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =     13.55166d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =     -6.18351d0*CON_KCALTOK
      tclocal( 9) =     -4.49945d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      2.43278d0*CON_KCALTOK
      tclocal(12) =      7.52124d0*CON_KCALTOK
      tclocal(13) =      9.16738d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -4.54030d0*CON_KCALTOK
      tclocal(17) =     -3.69552d0*CON_KCALTOK
      tclocal(18) =    -14.68984d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'sp ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'sp '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      8.31911d0*CON_KCALTOK
      tclocal(2) =      3.94364d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     17.81895d0*CON_KCALTOK
      tclocal(5) =      1.64967d0*CON_KCALTOK
      tclocal(6) =     -9.40201d0*CON_KCALTOK
      tclocal(7) =      1.64934d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =     28.56426d0*CON_KCALTOK
      tclocal(10) =    -11.22993d0*CON_KCALTOK
      tclocal(11) =     -9.87067d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     19.15489d0*CON_KCALTOK
      tclocal(14) =      7.91606d0*CON_KCALTOK
      tclocal(15) =    -11.01711d0*CON_KCALTOK
      tclocal(16) =    -24.79467d0*CON_KCALTOK
      tclocal(17) =     -3.72211d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cp ')
      call twh_torstring(GLB_SET,type,3,'sp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cp '
      tnlocal(nsame,3) = 'sp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.25039d0*CON_KCALTOK
      tclocal(1) =      0.76514d0*CON_KCALTOK
      tclocal(2) =     -0.14156d0*CON_KCALTOK
      tclocal(3) =     -0.76903d0*CON_KCALTOK
      tclocal(4) =     -0.41365d0*CON_KCALTOK
      tclocal(5) =     -0.23217d0*CON_KCALTOK
      tclocal(6) =     -0.25929d0*CON_KCALTOK
      tclocal(7) =      1.65826d0*CON_KCALTOK
      tclocal(8) =     -0.22540d0*CON_KCALTOK
      tclocal( 9) =     -0.76903d0*CON_KCALTOK
      tclocal(10) =     -0.41365d0*CON_KCALTOK
      tclocal(11) =     -0.23217d0*CON_KCALTOK
      tclocal(12) =      0.07657d0*CON_KCALTOK
      tclocal(13) =      1.22716d0*CON_KCALTOK
      tclocal(14) =     -0.31655d0*CON_KCALTOK
      tclocal(15) =      0.07657d0*CON_KCALTOK
      tclocal(16) =      1.22716d0*CON_KCALTOK
      tclocal(17) =     -0.31655d0*CON_KCALTOK
      tclocal(18) =     -8.35336d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.05589d0*CON_KCALTOK
      tclocal(1) =      0.95221d0*CON_KCALTOK
      tclocal(2) =     -0.19432d0*CON_KCALTOK
      tclocal(3) =     -0.34758d0*CON_KCALTOK
      tclocal(4) =     -0.78339d0*CON_KCALTOK
      tclocal(5) =      0.09350d0*CON_KCALTOK
      tclocal(6) =     -0.14613d0*CON_KCALTOK
      tclocal(7) =      1.45299d0*CON_KCALTOK
      tclocal(8) =      0.02996d0*CON_KCALTOK
      tclocal( 9) =      0.52413d0*CON_KCALTOK
      tclocal(10) =     -0.19494d0*CON_KCALTOK
      tclocal(11) =     -0.64109d0*CON_KCALTOK
      tclocal(12) =      0.10326d0*CON_KCALTOK
      tclocal(13) =     -0.02425d0*CON_KCALTOK
      tclocal(14) =      0.09770d0*CON_KCALTOK
      tclocal(15) =      0.02704d0*CON_KCALTOK
      tclocal(16) =      0.19558d0*CON_KCALTOK
      tclocal(17) =     -0.05615d0*CON_KCALTOK
      tclocal(18) =    -13.62600d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.08733d0*CON_KCALTOK
      tclocal(1) =      1.17678d0*CON_KCALTOK
      tclocal(2) =      0.04748d0*CON_KCALTOK
      tclocal(3) =     -0.88136d0*CON_KCALTOK
      tclocal(4) =     -1.10954d0*CON_KCALTOK
      tclocal(5) =     -0.18057d0*CON_KCALTOK
      tclocal(6) =      0.41107d0*CON_KCALTOK
      tclocal(7) =      0.25568d0*CON_KCALTOK
      tclocal(8) =     -0.27258d0*CON_KCALTOK
      tclocal( 9) =     -0.50787d0*CON_KCALTOK
      tclocal(10) =     -1.35831d0*CON_KCALTOK
      tclocal(11) =     -0.05719d0*CON_KCALTOK
      tclocal(12) =      0.18017d0*CON_KCALTOK
      tclocal(13) =     -0.18203d0*CON_KCALTOK
      tclocal(14) =      0.04331d0*CON_KCALTOK
      tclocal(15) =      0.02128d0*CON_KCALTOK
      tclocal(16) =      1.98248d0*CON_KCALTOK
      tclocal(17) =     -0.02949d0*CON_KCALTOK
      tclocal(18) =    -16.85982d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.07400d0*CON_KCALTOK
      tclocal(1) =      1.13802d0*CON_KCALTOK
      tclocal(2) =     -0.09335d0*CON_KCALTOK
      tclocal(3) =      0.55433d0*CON_KCALTOK
      tclocal(4) =     -1.19141d0*CON_KCALTOK
      tclocal(5) =      0.65523d0*CON_KCALTOK
      tclocal(6) =     -0.15313d0*CON_KCALTOK
      tclocal(7) =      1.86228d0*CON_KCALTOK
      tclocal(8) =      0.08344d0*CON_KCALTOK
      tclocal( 9) =     -0.49311d0*CON_KCALTOK
      tclocal(10) =      0.46931d0*CON_KCALTOK
      tclocal(11) =     -0.36870d0*CON_KCALTOK
      tclocal(12) =      0.00349d0*CON_KCALTOK
      tclocal(13) =     -0.05166d0*CON_KCALTOK
      tclocal(14) =     -0.01461d0*CON_KCALTOK
      tclocal(15) =      0.04357d0*CON_KCALTOK
      tclocal(16) =      0.19958d0*CON_KCALTOK
      tclocal(17) =      0.00842d0*CON_KCALTOK
      tclocal(18) =    -15.85352d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.21902d0*CON_KCALTOK
      tclocal(1) =      0.32496d0*CON_KCALTOK
      tclocal(2) =     -0.15413d0*CON_KCALTOK
      tclocal(3) =      0.62247d0*CON_KCALTOK
      tclocal(4) =     -1.13549d0*CON_KCALTOK
      tclocal(5) =      0.49749d0*CON_KCALTOK
      tclocal(6) =     -0.01248d0*CON_KCALTOK
      tclocal(7) =     -0.08080d0*CON_KCALTOK
      tclocal(8) =      0.16642d0*CON_KCALTOK
      tclocal( 9) =      0.09225d0*CON_KCALTOK
      tclocal(10) =     -0.28233d0*CON_KCALTOK
      tclocal(11) =     -0.43847d0*CON_KCALTOK
      tclocal(12) =     -0.31951d0*CON_KCALTOK
      tclocal(13) =     -0.11674d0*CON_KCALTOK
      tclocal(14) =      0.14461d0*CON_KCALTOK
      tclocal(15) =      0.08878d0*CON_KCALTOK
      tclocal(16) =     -0.08249d0*CON_KCALTOK
      tclocal(17) =     -0.24616d0*CON_KCALTOK
      tclocal(18) =     -0.76174d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.33106d0*CON_KCALTOK
      tclocal(1) =     -0.37242d0*CON_KCALTOK
      tclocal(2) =     -0.07863d0*CON_KCALTOK
      tclocal(3) =     -0.06226d0*CON_KCALTOK
      tclocal(4) =      0.24364d0*CON_KCALTOK
      tclocal(5) =      0.00424d0*CON_KCALTOK
      tclocal(6) =      0.14080d0*CON_KCALTOK
      tclocal(7) =     -0.04860d0*CON_KCALTOK
      tclocal(8) =      0.13508d0*CON_KCALTOK
      tclocal( 9) =     -0.06226d0*CON_KCALTOK
      tclocal(10) =      0.24364d0*CON_KCALTOK
      tclocal(11) =      0.00424d0*CON_KCALTOK
      tclocal(12) =      0.01116d0*CON_KCALTOK
      tclocal(13) =      0.04561d0*CON_KCALTOK
      tclocal(14) =      0.02190d0*CON_KCALTOK
      tclocal(15) =      0.01116d0*CON_KCALTOK
      tclocal(16) =      0.04561d0*CON_KCALTOK
      tclocal(17) =      0.02190d0*CON_KCALTOK
      tclocal(18) =     -5.79346d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.86250d0*CON_KCALTOK
      tclocal(1) =      1.91773d0*CON_KCALTOK
      tclocal(2) =      0.14733d0*CON_KCALTOK
      tclocal(3) =      0.18462d0*CON_KCALTOK
      tclocal(4) =     -0.16991d0*CON_KCALTOK
      tclocal(5) =      0.16243d0*CON_KCALTOK
      tclocal(6) =     -0.18800d0*CON_KCALTOK
      tclocal(7) =      1.31450d0*CON_KCALTOK
      tclocal(8) =     -0.25241d0*CON_KCALTOK
      tclocal( 9) =      0.11101d0*CON_KCALTOK
      tclocal(10) =      0.01028d0*CON_KCALTOK
      tclocal(11) =      0.12006d0*CON_KCALTOK
      tclocal(12) =      0.00374d0*CON_KCALTOK
      tclocal(13) =      0.00693d0*CON_KCALTOK
      tclocal(14) =      0.00168d0*CON_KCALTOK
      tclocal(15) =      0.01392d0*CON_KCALTOK
      tclocal(16) =     -0.01203d0*CON_KCALTOK
      tclocal(17) =      0.00927d0*CON_KCALTOK
      tclocal(18) =     -5.75418d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.09615d0*CON_KCALTOK
      tclocal(1) =      0.13767d0*CON_KCALTOK
      tclocal(2) =     -0.21083d0*CON_KCALTOK
      tclocal(3) =      0.17433d0*CON_KCALTOK
      tclocal(4) =      0.05890d0*CON_KCALTOK
      tclocal(5) =      0.13262d0*CON_KCALTOK
      tclocal(6) =      0.02500d0*CON_KCALTOK
      tclocal(7) =      0.14425d0*CON_KCALTOK
      tclocal(8) =     -0.06309d0*CON_KCALTOK
      tclocal( 9) =     -0.07931d0*CON_KCALTOK
      tclocal(10) =      0.19549d0*CON_KCALTOK
      tclocal(11) =     -0.04967d0*CON_KCALTOK
      tclocal(12) =     -0.00895d0*CON_KCALTOK
      tclocal(13) =      0.01134d0*CON_KCALTOK
      tclocal(14) =     -0.00656d0*CON_KCALTOK
      tclocal(15) =      0.00207d0*CON_KCALTOK
      tclocal(16) =      0.02284d0*CON_KCALTOK
      tclocal(17) =      0.00077d0*CON_KCALTOK
      tclocal(18) =     -5.21426d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.09726d0*CON_KCALTOK
      tclocal(1) =     -0.18410d0*CON_KCALTOK
      tclocal(2) =     -0.48873d0*CON_KCALTOK
      tclocal(3) =     -0.79688d0*CON_KCALTOK
      tclocal(4) =     -0.14973d0*CON_KCALTOK
      tclocal(5) =      0.75485d0*CON_KCALTOK
      tclocal(6) =      0.11397d0*CON_KCALTOK
      tclocal(7) =     -0.69666d0*CON_KCALTOK
      tclocal(8) =      0.06329d0*CON_KCALTOK
      tclocal( 9) =     -0.60286d0*CON_KCALTOK
      tclocal(10) =     -0.01678d0*CON_KCALTOK
      tclocal(11) =      0.23055d0*CON_KCALTOK
      tclocal(12) =     -0.02248d0*CON_KCALTOK
      tclocal(13) =      0.08955d0*CON_KCALTOK
      tclocal(14) =      0.01421d0*CON_KCALTOK
      tclocal(15) =      0.06993d0*CON_KCALTOK
      tclocal(16) =      0.01042d0*CON_KCALTOK
      tclocal(17) =      0.09405d0*CON_KCALTOK
      tclocal(18) =     -0.80354d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.61134d0*CON_KCALTOK
      tclocal(1) =      1.07288d0*CON_KCALTOK
      tclocal(2) =     -0.35922d0*CON_KCALTOK
      tclocal(3) =      0.03892d0*CON_KCALTOK
      tclocal(4) =      0.29757d0*CON_KCALTOK
      tclocal(5) =     -0.05391d0*CON_KCALTOK
      tclocal(6) =     -0.17091d0*CON_KCALTOK
      tclocal(7) =      0.01329d0*CON_KCALTOK
      tclocal(8) =      0.25971d0*CON_KCALTOK
      tclocal( 9) =      0.03892d0*CON_KCALTOK
      tclocal(10) =      0.29757d0*CON_KCALTOK
      tclocal(11) =     -0.05391d0*CON_KCALTOK
      tclocal(12) =     -0.00154d0*CON_KCALTOK
      tclocal(13) =     -0.00914d0*CON_KCALTOK
      tclocal(14) =      0.00223d0*CON_KCALTOK
      tclocal(15) =     -0.00154d0*CON_KCALTOK
      tclocal(16) =     -0.00914d0*CON_KCALTOK
      tclocal(17) =      0.00223d0*CON_KCALTOK
      tclocal(18) =     -6.57767d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.03550d0*CON_KCALTOK
      tclocal(1) =      1.17721d0*CON_KCALTOK
      tclocal(2) =     -0.21759d0*CON_KCALTOK
      tclocal(3) =      0.18095d0*CON_KCALTOK
      tclocal(4) =      0.16039d0*CON_KCALTOK
      tclocal(5) =      0.20877d0*CON_KCALTOK
      tclocal(6) =      0.08585d0*CON_KCALTOK
      tclocal(7) =      0.75639d0*CON_KCALTOK
      tclocal(8) =     -0.00563d0*CON_KCALTOK
      tclocal( 9) =      0.18760d0*CON_KCALTOK
      tclocal(10) =      0.00433d0*CON_KCALTOK
      tclocal(11) =      0.13975d0*CON_KCALTOK
      tclocal(12) =     -0.01267d0*CON_KCALTOK
      tclocal(13) =     -0.00727d0*CON_KCALTOK
      tclocal(14) =     -0.01020d0*CON_KCALTOK
      tclocal(15) =     -0.00456d0*CON_KCALTOK
      tclocal(16) =      0.00195d0*CON_KCALTOK
      tclocal(17) =     -0.00034d0*CON_KCALTOK
      tclocal(18) =     -6.75610d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.13227d0*CON_KCALTOK
      tclocal(1) =      1.24946d0*CON_KCALTOK
      tclocal(2) =      0.00571d0*CON_KCALTOK
      tclocal(3) =      0.09915d0*CON_KCALTOK
      tclocal(4) =      0.06330d0*CON_KCALTOK
      tclocal(5) =      0.08328d0*CON_KCALTOK
      tclocal(6) =      0.13714d0*CON_KCALTOK
      tclocal(7) =      0.67175d0*CON_KCALTOK
      tclocal(8) =     -0.03470d0*CON_KCALTOK
      tclocal( 9) =      0.04512d0*CON_KCALTOK
      tclocal(10) =     -0.11692d0*CON_KCALTOK
      tclocal(11) =      0.03597d0*CON_KCALTOK
      tclocal(12) =     -0.01304d0*CON_KCALTOK
      tclocal(13) =     -0.01349d0*CON_KCALTOK
      tclocal(14) =     -0.00927d0*CON_KCALTOK
      tclocal(15) =      0.00953d0*CON_KCALTOK
      tclocal(16) =      0.00318d0*CON_KCALTOK
      tclocal(17) =      0.00936d0*CON_KCALTOK
      tclocal(18) =     -1.18516d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.11305d0*CON_KCALTOK
      tclocal(1) =      0.19227d0*CON_KCALTOK
      tclocal(2) =     -0.33764d0*CON_KCALTOK
      tclocal(3) =      0.33852d0*CON_KCALTOK
      tclocal(4) =      0.41081d0*CON_KCALTOK
      tclocal(5) =      0.24289d0*CON_KCALTOK
      tclocal(6) =      0.07448d0*CON_KCALTOK
      tclocal(7) =      0.10855d0*CON_KCALTOK
      tclocal(8) =     -0.02478d0*CON_KCALTOK
      tclocal( 9) =      0.33852d0*CON_KCALTOK
      tclocal(10) =      0.41081d0*CON_KCALTOK
      tclocal(11) =      0.24289d0*CON_KCALTOK
      tclocal(12) =     -0.02282d0*CON_KCALTOK
      tclocal(13) =      0.03452d0*CON_KCALTOK
      tclocal(14) =     -0.01379d0*CON_KCALTOK
      tclocal(15) =     -0.02282d0*CON_KCALTOK
      tclocal(16) =      0.03452d0*CON_KCALTOK
      tclocal(17) =     -0.01379d0*CON_KCALTOK
      tclocal(18) =     -3.91119d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'op ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'op '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.06336d0*CON_KCALTOK
      tclocal(1) =      0.37523d0*CON_KCALTOK
      tclocal(2) =     -0.26833d0*CON_KCALTOK
      tclocal(3) =      0.12237d0*CON_KCALTOK
      tclocal(4) =      0.12233d0*CON_KCALTOK
      tclocal(5) =      0.09816d0*CON_KCALTOK
      tclocal(6) =     -0.00257d0*CON_KCALTOK
      tclocal(7) =      0.52226d0*CON_KCALTOK
      tclocal(8) =     -0.08119d0*CON_KCALTOK
      tclocal( 9) =      0.07984d0*CON_KCALTOK
      tclocal(10) =     -0.02190d0*CON_KCALTOK
      tclocal(11) =      0.09413d0*CON_KCALTOK
      tclocal(12) =     -0.01577d0*CON_KCALTOK
      tclocal(13) =      0.00220d0*CON_KCALTOK
      tclocal(14) =     -0.01113d0*CON_KCALTOK
      tclocal(15) =      0.00548d0*CON_KCALTOK
      tclocal(16) =      0.00739d0*CON_KCALTOK
      tclocal(17) =      0.00449d0*CON_KCALTOK
      tclocal(18) =     -0.30189d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'op ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'op '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.66753d0*CON_KCALTOK
      tclocal(1) =     -0.23255d0*CON_KCALTOK
      tclocal(2) =     -0.25808d0*CON_KCALTOK
      tclocal(3) =      0.42813d0*CON_KCALTOK
      tclocal(4) =     -0.16217d0*CON_KCALTOK
      tclocal(5) =      0.18799d0*CON_KCALTOK
      tclocal(6) =     -0.06261d0*CON_KCALTOK
      tclocal(7) =      0.03901d0*CON_KCALTOK
      tclocal(8) =     -0.04688d0*CON_KCALTOK
      tclocal( 9) =      0.42813d0*CON_KCALTOK
      tclocal(10) =     -0.16217d0*CON_KCALTOK
      tclocal(11) =      0.18799d0*CON_KCALTOK
      tclocal(12) =     -0.18896d0*CON_KCALTOK
      tclocal(13) =      0.14465d0*CON_KCALTOK
      tclocal(14) =      0.14792d0*CON_KCALTOK
      tclocal(15) =     -0.18896d0*CON_KCALTOK
      tclocal(16) =      0.14465d0*CON_KCALTOK
      tclocal(17) =      0.14792d0*CON_KCALTOK
      tclocal(18) =      0.01874d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'sp ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'cpb')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'sp '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'cpb'
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.14077d0*CON_KCALTOK
      tclocal(1) =      1.11490d0*CON_KCALTOK
      tclocal(2) =     -0.17803d0*CON_KCALTOK
      tclocal(3) =      4.99328d0*CON_KCALTOK
      tclocal(4) =      0.75350d0*CON_KCALTOK
      tclocal(5) =     -2.13833d0*CON_KCALTOK
      tclocal(6) =     -2.54498d0*CON_KCALTOK
      tclocal(7) =      7.49732d0*CON_KCALTOK
      tclocal(8) =      6.39231d0*CON_KCALTOK
      tclocal( 9) =     -4.67034d0*CON_KCALTOK
      tclocal(10) =     -1.06476d0*CON_KCALTOK
      tclocal(11) =      4.54987d0*CON_KCALTOK
      tclocal(12) =    -10.14142d0*CON_KCALTOK
      tclocal(13) =     -5.58896d0*CON_KCALTOK
      tclocal(14) =      3.52537d0*CON_KCALTOK
      tclocal(15) =      2.08317d0*CON_KCALTOK
      tclocal(16) =     -6.11356d0*CON_KCALTOK
      tclocal(17) =      2.71815d0*CON_KCALTOK
      tclocal(18) =     -9.64721d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.25828d0*CON_KCALTOK
      tclocal(1) =      0.41906d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.61949d0*CON_KCALTOK
      tclocal(4) =     -3.05898d0*CON_KCALTOK
      tclocal(5) =      0.08221d0*CON_KCALTOK
      tclocal(6) =      1.23962d0*CON_KCALTOK
      tclocal(7) =     -3.33121d0*CON_KCALTOK
      tclocal(8) =     -3.73618d0*CON_KCALTOK
      tclocal( 9) =     -3.29922d0*CON_KCALTOK
      tclocal(10) =      0.96586d0*CON_KCALTOK
      tclocal(11) =      0.44295d0*CON_KCALTOK
      tclocal(12) =      8.89284d0*CON_KCALTOK
      tclocal(13) =     -0.08980d0*CON_KCALTOK
      tclocal(14) =     -2.04757d0*CON_KCALTOK
      tclocal(15) =     -6.67993d0*CON_KCALTOK
      tclocal(16) =      4.54292d0*CON_KCALTOK
      tclocal(17) =      1.40803d0*CON_KCALTOK
      tclocal(18) =    -17.42309d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.03101d0*CON_KCALTOK
      tclocal(1) =      4.30487d0*CON_KCALTOK
      tclocal(2) =      0.07955d0*CON_KCALTOK
      tclocal(3) =     -3.31011d0*CON_KCALTOK
      tclocal(4) =     -9.97234d0*CON_KCALTOK
      tclocal(5) =      1.66007d0*CON_KCALTOK
      tclocal(6) =      9.58780d0*CON_KCALTOK
      tclocal(7) =      5.23124d0*CON_KCALTOK
      tclocal(8) =      2.99926d0*CON_KCALTOK
      tclocal( 9) =     -0.87849d0*CON_KCALTOK
      tclocal(10) =     -3.93098d0*CON_KCALTOK
      tclocal(11) =     -0.00202d0*CON_KCALTOK
      tclocal(12) =      7.97912d0*CON_KCALTOK
      tclocal(13) =      7.35321d0*CON_KCALTOK
      tclocal(14) =      1.91886d0*CON_KCALTOK
      tclocal(15) =      2.20485d0*CON_KCALTOK
      tclocal(16) =      5.71667d0*CON_KCALTOK
      tclocal(17) =      0.37888d0*CON_KCALTOK
      tclocal(18) =      5.95262d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.22074d0*CON_KCALTOK
      tclocal(1) =      1.94435d0*CON_KCALTOK
      tclocal(2) =     -0.14164d0*CON_KCALTOK
      tclocal(3) =     -1.73857d0*CON_KCALTOK
      tclocal(4) =     -0.80179d0*CON_KCALTOK
      tclocal(5) =     -0.27106d0*CON_KCALTOK
      tclocal(6) =    -11.16403d0*CON_KCALTOK
      tclocal(7) =     -4.85020d0*CON_KCALTOK
      tclocal(8) =     -3.85266d0*CON_KCALTOK
      tclocal( 9) =      6.45215d0*CON_KCALTOK
      tclocal(10) =     -7.11470d0*CON_KCALTOK
      tclocal(11) =     -1.62569d0*CON_KCALTOK
      tclocal(12) =     -7.20514d0*CON_KCALTOK
      tclocal(13) =      5.66013d0*CON_KCALTOK
      tclocal(14) =      1.27916d0*CON_KCALTOK
      tclocal(15) =      4.41320d0*CON_KCALTOK
      tclocal(16) =      3.14759d0*CON_KCALTOK
      tclocal(17) =      1.45927d0*CON_KCALTOK
      tclocal(18) =    -39.05669d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.53839d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      4.17293d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     23.75763d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      1.69182d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.23898d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.66640d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      3.19205d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.15856d0*CON_KCALTOK
      tclocal(1) =      3.36105d0*CON_KCALTOK
      tclocal(2) =     -0.38088d0*CON_KCALTOK
      tclocal(3) =     -1.68447d0*CON_KCALTOK
      tclocal(4) =     -0.69345d0*CON_KCALTOK
      tclocal(5) =      0.60351d0*CON_KCALTOK
      tclocal(6) =     -5.95895d0*CON_KCALTOK
      tclocal(7) =      4.82148d0*CON_KCALTOK
      tclocal(8) =      0.48631d0*CON_KCALTOK
      tclocal( 9) =     -1.32479d0*CON_KCALTOK
      tclocal(10) =     -5.07273d0*CON_KCALTOK
      tclocal(11) =      3.26773d0*CON_KCALTOK
      tclocal(12) =     -1.38563d0*CON_KCALTOK
      tclocal(13) =      6.05829d0*CON_KCALTOK
      tclocal(14) =     -0.31327d0*CON_KCALTOK
      tclocal(15) =     -3.50593d0*CON_KCALTOK
      tclocal(16) =      3.27119d0*CON_KCALTOK
      tclocal(17) =      1.24826d0*CON_KCALTOK
      tclocal(18) =    -16.35169d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.71252d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      1.81431d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     14.38935d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -9.77278d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -2.10420d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -8.65042d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -36.24387d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nn ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nn '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.72921d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.09517d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     13.92083d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -7.87706d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -2.41791d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -4.57322d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -1.41498d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nn ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nn '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.76207d0*CON_KCALTOK
      tclocal(1) =      2.99820d0*CON_KCALTOK
      tclocal(2) =     -0.63772d0*CON_KCALTOK
      tclocal(3) =      2.40423d0*CON_KCALTOK
      tclocal(4) =     -3.67656d0*CON_KCALTOK
      tclocal(5) =      3.18876d0*CON_KCALTOK
      tclocal(6) =     -5.41290d0*CON_KCALTOK
      tclocal(7) =     -0.53420d0*CON_KCALTOK
      tclocal(8) =      0.65941d0*CON_KCALTOK
      tclocal( 9) =     -3.64091d0*CON_KCALTOK
      tclocal(10) =     -3.05441d0*CON_KCALTOK
      tclocal(11) =     -0.30902d0*CON_KCALTOK
      tclocal(12) =     -7.12608d0*CON_KCALTOK
      tclocal(13) =      0.70382d0*CON_KCALTOK
      tclocal(14) =      1.10346d0*CON_KCALTOK
      tclocal(15) =     -7.83630d0*CON_KCALTOK
      tclocal(16) =      1.28845d0*CON_KCALTOK
      tclocal(17) =      1.42981d0*CON_KCALTOK
      tclocal(18) =    -16.23715d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.00604d0*CON_KCALTOK
      tclocal(1) =      2.84867d0*CON_KCALTOK
      tclocal(2) =      0.55423d0*CON_KCALTOK
      tclocal(3) =     -7.76920d0*CON_KCALTOK
      tclocal(4) =     -5.69610d0*CON_KCALTOK
      tclocal(5) =     -2.91541d0*CON_KCALTOK
      tclocal(6) =     -1.67001d0*CON_KCALTOK
      tclocal(7) =     -4.98350d0*CON_KCALTOK
      tclocal(8) =      7.99973d0*CON_KCALTOK
      tclocal( 9) =      3.33963d0*CON_KCALTOK
      tclocal(10) =     -1.55410d0*CON_KCALTOK
      tclocal(11) =      7.14706d0*CON_KCALTOK
      tclocal(12) =     14.53035d0*CON_KCALTOK
      tclocal(13) =     13.21181d0*CON_KCALTOK
      tclocal(14) =     -1.69322d0*CON_KCALTOK
      tclocal(15) =      2.09158d0*CON_KCALTOK
      tclocal(16) =     11.14065d0*CON_KCALTOK
      tclocal(17) =      4.72336d0*CON_KCALTOK
      tclocal(18) =     51.57005d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'nh ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'nh '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.05118d0*CON_KCALTOK
      tclocal(1) =      3.78317d0*CON_KCALTOK
      tclocal(2) =     -0.46307d0*CON_KCALTOK
      tclocal(3) =      2.40668d0*CON_KCALTOK
      tclocal(4) =     -7.11710d0*CON_KCALTOK
      tclocal(5) =      0.86266d0*CON_KCALTOK
      tclocal(6) =     -1.96071d0*CON_KCALTOK
      tclocal(7) =      9.81317d0*CON_KCALTOK
      tclocal(8) =     -1.81161d0*CON_KCALTOK
      tclocal( 9) =      0.03371d0*CON_KCALTOK
      tclocal(10) =     -0.19113d0*CON_KCALTOK
      tclocal(11) =      0.57037d0*CON_KCALTOK
      tclocal(12) =      0.29425d0*CON_KCALTOK
      tclocal(13) =      0.82213d0*CON_KCALTOK
      tclocal(14) =     -0.40578d0*CON_KCALTOK
      tclocal(15) =      0.01033d0*CON_KCALTOK
      tclocal(16) =      1.00393d0*CON_KCALTOK
      tclocal(17) =     -0.64947d0*CON_KCALTOK
      tclocal(18) =    -10.45300d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.49610d0*CON_KCALTOK
      tclocal(1) =      3.02988d0*CON_KCALTOK
      tclocal(2) =     -0.15539d0*CON_KCALTOK
      tclocal(3) =     -0.51597d0*CON_KCALTOK
      tclocal(4) =     -4.76622d0*CON_KCALTOK
      tclocal(5) =      0.66880d0*CON_KCALTOK
      tclocal(6) =     -1.74201d0*CON_KCALTOK
      tclocal(7) =     11.64419d0*CON_KCALTOK
      tclocal(8) =     -2.10687d0*CON_KCALTOK
      tclocal( 9) =      1.40127d0*CON_KCALTOK
      tclocal(10) =     -1.02090d0*CON_KCALTOK
      tclocal(11) =     -0.07525d0*CON_KCALTOK
      tclocal(12) =     -6.05526d0*CON_KCALTOK
      tclocal(13) =      1.55251d0*CON_KCALTOK
      tclocal(14) =      0.40965d0*CON_KCALTOK
      tclocal(15) =      0.93896d0*CON_KCALTOK
      tclocal(16) =      0.75435d0*CON_KCALTOK
      tclocal(17) =     -0.59335d0*CON_KCALTOK
      tclocal(18) =     -2.74229d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.55961d0*CON_KCALTOK
      tclocal(1) =      1.93490d0*CON_KCALTOK
      tclocal(2) =      0.22008d0*CON_KCALTOK
      tclocal(3) =     -0.47772d0*CON_KCALTOK
      tclocal(4) =      1.79172d0*CON_KCALTOK
      tclocal(5) =      0.86921d0*CON_KCALTOK
      tclocal(6) =      1.39786d0*CON_KCALTOK
      tclocal(7) =      9.38247d0*CON_KCALTOK
      tclocal(8) =     -1.20645d0*CON_KCALTOK
      tclocal( 9) =      0.20723d0*CON_KCALTOK
      tclocal(10) =      1.44435d0*CON_KCALTOK
      tclocal(11) =     -0.76629d0*CON_KCALTOK
      tclocal(12) =      0.84606d0*CON_KCALTOK
      tclocal(13) =      5.10783d0*CON_KCALTOK
      tclocal(14) =      0.85929d0*CON_KCALTOK
      tclocal(15) =      1.59939d0*CON_KCALTOK
      tclocal(16) =      2.14462d0*CON_KCALTOK
      tclocal(17) =      0.79777d0*CON_KCALTOK
      tclocal(18) =    -10.38781d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.94351d0*CON_KCALTOK
      tclocal(1) =      0.73102d0*CON_KCALTOK
      tclocal(2) =     -0.12279d0*CON_KCALTOK
      tclocal(3) =     -2.62854d0*CON_KCALTOK
      tclocal(4) =      3.33020d0*CON_KCALTOK
      tclocal(5) =      0.04887d0*CON_KCALTOK
      tclocal(6) =      2.24570d0*CON_KCALTOK
      tclocal(7) =      6.77400d0*CON_KCALTOK
      tclocal(8) =     -2.73703d0*CON_KCALTOK
      tclocal( 9) =     -1.14048d0*CON_KCALTOK
      tclocal(10) =     -0.27771d0*CON_KCALTOK
      tclocal(11) =     -0.26354d0*CON_KCALTOK
      tclocal(12) =     -4.83095d0*CON_KCALTOK
      tclocal(13) =      3.59220d0*CON_KCALTOK
      tclocal(14) =      0.65091d0*CON_KCALTOK
      tclocal(15) =      0.47487d0*CON_KCALTOK
      tclocal(16) =      0.73277d0*CON_KCALTOK
      tclocal(17) =      0.48079d0*CON_KCALTOK
      tclocal(18) =    -15.16028d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'nn ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'nn '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10240d0*CON_KCALTOK
      tclocal(1) =      6.06300d0*CON_KCALTOK
      tclocal(2) =     -0.34381d0*CON_KCALTOK
      tclocal(3) =     -3.91736d0*CON_KCALTOK
      tclocal(4) =     -5.75943d0*CON_KCALTOK
      tclocal(5) =      1.19030d0*CON_KCALTOK
      tclocal(6) =     55.47661d0*CON_KCALTOK
      tclocal(7) =     12.30603d0*CON_KCALTOK
      tclocal(8) =     -0.93216d0*CON_KCALTOK
      tclocal( 9) =    -20.33975d0*CON_KCALTOK
      tclocal(10) =     -7.54007d0*CON_KCALTOK
      tclocal(11) =     -3.11424d0*CON_KCALTOK
      tclocal(12) =    -18.15219d0*CON_KCALTOK
      tclocal(13) =      7.41225d0*CON_KCALTOK
      tclocal(14) =      1.55575d0*CON_KCALTOK
      tclocal(15) =     12.57427d0*CON_KCALTOK
      tclocal(16) =     -3.35509d0*CON_KCALTOK
      tclocal(17) =      0.84449d0*CON_KCALTOK
      tclocal(18) =    -23.56553d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =     10.64873d0*CON_KCALTOK
      tclocal(2) =     -3.82690d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -20.63452d0*CON_KCALTOK
      tclocal(5) =     17.61518d0*CON_KCALTOK
      tclocal(6) =     94.48789d0*CON_KCALTOK
      tclocal(7) =     22.65613d0*CON_KCALTOK
      tclocal(8) =    -22.80626d0*CON_KCALTOK
      tclocal( 9) =     53.89744d0*CON_KCALTOK
      tclocal(10) =     42.12952d0*CON_KCALTOK
      tclocal(11) =    -32.71372d0*CON_KCALTOK
      tclocal(12) =    -43.77643d0*CON_KCALTOK
      tclocal(13) =     -2.89012d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =    -47.91024d0*CON_KCALTOK
      tclocal(16) =    -38.14858d0*CON_KCALTOK
      tclocal(17) =     23.02462d0*CON_KCALTOK
      tclocal(18) =    -63.91177d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      8.45054d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      1.23455d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     39.90289d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -18.39426d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -4.48632d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -1.67759d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -19.67213d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.05994d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.50019d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     30.39831d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -3.55901d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      1.17341d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.37115d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -6.93932d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.08905d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -0.51511d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     26.51795d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -3.16878d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -0.28718d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.30405d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -11.65600d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.23238d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -3.17711d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     41.63848d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -1.76037d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -1.44853d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      2.57112d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -9.21541d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.16552d0*CON_KCALTOK
      tclocal(1) =      6.10633d0*CON_KCALTOK
      tclocal(2) =     -0.40086d0*CON_KCALTOK
      tclocal(3) =     -1.91183d0*CON_KCALTOK
      tclocal(4) =     -4.71227d0*CON_KCALTOK
      tclocal(5) =      4.64328d0*CON_KCALTOK
      tclocal(6) =     42.09554d0*CON_KCALTOK
      tclocal(7) =     35.94446d0*CON_KCALTOK
      tclocal(8) =     -7.03218d0*CON_KCALTOK
      tclocal( 9) =    -16.10430d0*CON_KCALTOK
      tclocal(10) =    -18.27940d0*CON_KCALTOK
      tclocal(11) =      3.12360d0*CON_KCALTOK
      tclocal(12) =     -2.30827d0*CON_KCALTOK
      tclocal(13) =      7.62734d0*CON_KCALTOK
      tclocal(14) =      1.73865d0*CON_KCALTOK
      tclocal(15) =      4.01473d0*CON_KCALTOK
      tclocal(16) =      4.44395d0*CON_KCALTOK
      tclocal(17) =      5.73909d0*CON_KCALTOK
      tclocal(18) =    -45.19613d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.07952d0*CON_KCALTOK
      tclocal(1) =      5.78085d0*CON_KCALTOK
      tclocal(2) =     -0.18622d0*CON_KCALTOK
      tclocal(3) =    -24.02947d0*CON_KCALTOK
      tclocal(4) =     -6.37287d0*CON_KCALTOK
      tclocal(5) =      1.66849d0*CON_KCALTOK
      tclocal(6) =     39.49968d0*CON_KCALTOK
      tclocal(7) =     25.45245d0*CON_KCALTOK
      tclocal(8) =      1.00532d0*CON_KCALTOK
      tclocal( 9) =    -24.02947d0*CON_KCALTOK
      tclocal(10) =     -6.37287d0*CON_KCALTOK
      tclocal(11) =      1.66849d0*CON_KCALTOK
      tclocal(12) =     -7.88233d0*CON_KCALTOK
      tclocal(13) =     12.73951d0*CON_KCALTOK
      tclocal(14) =      1.40963d0*CON_KCALTOK
      tclocal(15) =     11.47688d0*CON_KCALTOK
      tclocal(16) =     -0.05033d0*CON_KCALTOK
      tclocal(17) =      0.77696d0*CON_KCALTOK
      tclocal(18) =    -16.19031d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.08324d0*CON_KCALTOK
      tclocal(1) =      4.38334d0*CON_KCALTOK
      tclocal(2) =     -0.28309d0*CON_KCALTOK
      tclocal(3) =     -6.81366d0*CON_KCALTOK
      tclocal(4) =    -31.94350d0*CON_KCALTOK
      tclocal(5) =      1.52291d0*CON_KCALTOK
      tclocal(6) =     60.63275d0*CON_KCALTOK
      tclocal(7) =     16.12044d0*CON_KCALTOK
      tclocal(8) =      0.01139d0*CON_KCALTOK
      tclocal( 9) =    -29.81630d0*CON_KCALTOK
      tclocal(10) =    -15.17859d0*CON_KCALTOK
      tclocal(11) =     -1.74000d0*CON_KCALTOK
      tclocal(12) =     28.05038d0*CON_KCALTOK
      tclocal(13) =     30.29008d0*CON_KCALTOK
      tclocal(14) =     -6.06805d0*CON_KCALTOK
      tclocal(15) =      8.31036d0*CON_KCALTOK
      tclocal(16) =     20.53851d0*CON_KCALTOK
      tclocal(17) =      2.79752d0*CON_KCALTOK
      tclocal(18) =    -20.48885d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.29348d0*CON_KCALTOK
      tclocal(1) =      7.94941d0*CON_KCALTOK
      tclocal(2) =      0.51909d0*CON_KCALTOK
      tclocal(3) =    -21.25451d0*CON_KCALTOK
      tclocal(4) =      2.74658d0*CON_KCALTOK
      tclocal(5) =     -3.82867d0*CON_KCALTOK
      tclocal(6) =     51.07684d0*CON_KCALTOK
      tclocal(7) =     21.26649d0*CON_KCALTOK
      tclocal(8) =    -15.51586d0*CON_KCALTOK
      tclocal( 9) =     -3.51061d0*CON_KCALTOK
      tclocal(10) =     22.69053d0*CON_KCALTOK
      tclocal(11) =    -15.29552d0*CON_KCALTOK
      tclocal(12) =     21.47278d0*CON_KCALTOK
      tclocal(13) =     14.74781d0*CON_KCALTOK
      tclocal(14) =    -15.04973d0*CON_KCALTOK
      tclocal(15) =      8.38004d0*CON_KCALTOK
      tclocal(16) =     11.94061d0*CON_KCALTOK
      tclocal(17) =     -0.00509d0*CON_KCALTOK
      tclocal(18) =      4.80478d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.50000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =     -1.92472d0*CON_KCALTOK
      tclocal(6) =    -14.78112d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      9.06036d0*CON_KCALTOK
      tclocal(11) =     12.70801d0*CON_KCALTOK
      tclocal(12) =    -24.45041d0*CON_KCALTOK
      tclocal(13) =    -17.57623d0*CON_KCALTOK
      tclocal(14) =     11.55904d0*CON_KCALTOK
      tclocal(15) =    -18.72273d0*CON_KCALTOK
      tclocal(16) =    -15.90954d0*CON_KCALTOK
      tclocal(17) =     11.18271d0*CON_KCALTOK
      tclocal(18) =     -8.10749d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'op ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'op '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.82498d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      3.48705d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     10.50344d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -1.81789d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -0.90359d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      2.04216d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -19.99138d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'op ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'op '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.45208d0*CON_KCALTOK
      tclocal(1) =      1.20000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      4.17160d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.88364d0*CON_KCALTOK
      tclocal(6) =    -29.54916d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =     18.93905d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -2.18664d0*CON_KCALTOK
      tclocal(11) =      1.83028d0*CON_KCALTOK
      tclocal(12) =    -20.83713d0*CON_KCALTOK
      tclocal(13) =    -16.80435d0*CON_KCALTOK
      tclocal(14) =     14.36127d0*CON_KCALTOK
      tclocal(15) =    -22.73501d0*CON_KCALTOK
      tclocal(16) =    -16.29053d0*CON_KCALTOK
      tclocal(17) =     11.70823d0*CON_KCALTOK
      tclocal(18) =    -18.77487d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'sp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'sp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      3.97212d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      5.07665d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      9.48571d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -2.69880d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -4.94786d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      6.00890d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -26.31088d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'cpb')
      call twh_torstring(GLB_SET,type,3,'sp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'cpb'
      tnlocal(nsame,3) = 'sp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'ct ')
      call twh_torstring(GLB_SET,type,3,'ct ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'ct '
      tnlocal(nsame,3) = 'ct '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'ct ')
      call twh_torstring(GLB_SET,type,3,'ct ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'ct '
      tnlocal(nsame,3) = 'ct '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      0.00000d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'ct ')
      call twh_torstring(GLB_SET,type,3,'ct ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'ct '
      tnlocal(nsame,3) = 'ct '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.04732d0*CON_KCALTOK
      tclocal(1) =     -0.05395d0*CON_KCALTOK
      tclocal(2) =      0.04690d0*CON_KCALTOK
      tclocal(3) =      0.21470d0*CON_KCALTOK
      tclocal(4) =      0.21739d0*CON_KCALTOK
      tclocal(5) =     -0.08206d0*CON_KCALTOK
      tclocal(6) =      0.03514d0*CON_KCALTOK
      tclocal(7) =      0.03392d0*CON_KCALTOK
      tclocal(8) =     -0.19385d0*CON_KCALTOK
      tclocal( 9) =      0.07103d0*CON_KCALTOK
      tclocal(10) =     -0.07948d0*CON_KCALTOK
      tclocal(11) =     -0.20839d0*CON_KCALTOK
      tclocal(12) =      0.64805d0*CON_KCALTOK
      tclocal(13) =      0.75372d0*CON_KCALTOK
      tclocal(14) =     -0.56781d0*CON_KCALTOK
      tclocal(15) =      0.21517d0*CON_KCALTOK
      tclocal(16) =      1.47747d0*CON_KCALTOK
      tclocal(17) =      0.51052d0*CON_KCALTOK
      tclocal(18) =     -0.11556d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'n  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'n  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00899d0*CON_KCALTOK
      tclocal(1) =     -0.00663d0*CON_KCALTOK
      tclocal(2) =     -0.39958d0*CON_KCALTOK
      tclocal(3) =      0.58619d0*CON_KCALTOK
      tclocal(4) =     -0.14715d0*CON_KCALTOK
      tclocal(5) =      0.66498d0*CON_KCALTOK
      tclocal(6) =     -0.23897d0*CON_KCALTOK
      tclocal(7) =     -0.81143d0*CON_KCALTOK
      tclocal(8) =     -0.18042d0*CON_KCALTOK
      tclocal( 9) =     -0.09492d0*CON_KCALTOK
      tclocal(10) =      0.37473d0*CON_KCALTOK
      tclocal(11) =      0.05721d0*CON_KCALTOK
      tclocal(12) =      0.09748d0*CON_KCALTOK
      tclocal(13) =     -0.56149d0*CON_KCALTOK
      tclocal(14) =      0.92109d0*CON_KCALTOK
      tclocal(15) =     -1.82386d0*CON_KCALTOK
      tclocal(16) =      4.82027d0*CON_KCALTOK
      tclocal(17) =      1.91827d0*CON_KCALTOK
      tclocal(18) =     -4.12139d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'n  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'n  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.11972d0*CON_KCALTOK
      tclocal(1) =     -0.13196d0*CON_KCALTOK
      tclocal(2) =     -0.14644d0*CON_KCALTOK
      tclocal(3) =      0.17675d0*CON_KCALTOK
      tclocal(4) =      0.29238d0*CON_KCALTOK
      tclocal(5) =      0.11533d0*CON_KCALTOK
      tclocal(6) =     -0.13253d0*CON_KCALTOK
      tclocal(7) =     -0.30724d0*CON_KCALTOK
      tclocal(8) =     -0.00414d0*CON_KCALTOK
      tclocal( 9) =     -0.17152d0*CON_KCALTOK
      tclocal(10) =     -0.43941d0*CON_KCALTOK
      tclocal(11) =      0.22452d0*CON_KCALTOK
      tclocal(12) =      2.86696d0*CON_KCALTOK
      tclocal(13) =      0.12756d0*CON_KCALTOK
      tclocal(14) =     -0.27979d0*CON_KCALTOK
      tclocal(15) =      0.52547d0*CON_KCALTOK
      tclocal(16) =      2.98026d0*CON_KCALTOK
      tclocal(17) =     -0.02441d0*CON_KCALTOK
      tclocal(18) =      0.09036d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'n  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'n  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.24116d0*CON_KCALTOK
      tclocal(1) =      0.72225d0*CON_KCALTOK
      tclocal(2) =     -0.10166d0*CON_KCALTOK
      tclocal(3) =     -0.36521d0*CON_KCALTOK
      tclocal(4) =     -0.73097d0*CON_KCALTOK
      tclocal(5) =     -0.24729d0*CON_KCALTOK
      tclocal(6) =      0.09981d0*CON_KCALTOK
      tclocal(7) =      0.40130d0*CON_KCALTOK
      tclocal(8) =     -0.38994d0*CON_KCALTOK
      tclocal( 9) =     -1.05859d0*CON_KCALTOK
      tclocal(10) =      1.98022d0*CON_KCALTOK
      tclocal(11) =      0.74154d0*CON_KCALTOK
      tclocal(12) =     -3.28140d0*CON_KCALTOK
      tclocal(13) =     -2.24749d0*CON_KCALTOK
      tclocal(14) =     -0.17747d0*CON_KCALTOK
      tclocal(15) =      0.90686d0*CON_KCALTOK
      tclocal(16) =      4.32557d0*CON_KCALTOK
      tclocal(17) =      0.93281d0*CON_KCALTOK
      tclocal(18) =     -5.58600d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'n  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'n  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.40336d0*CON_KCALTOK
      tclocal(1) =      0.00803d0*CON_KCALTOK
      tclocal(2) =      0.04469d0*CON_KCALTOK
      tclocal(3) =      0.16863d0*CON_KCALTOK
      tclocal(4) =      3.08272d0*CON_KCALTOK
      tclocal(5) =      1.51547d0*CON_KCALTOK
      tclocal(6) =     -0.65537d0*CON_KCALTOK
      tclocal(7) =     -0.02300d0*CON_KCALTOK
      tclocal(8) =     -0.43926d0*CON_KCALTOK
      tclocal( 9) =      0.13422d0*CON_KCALTOK
      tclocal(10) =      0.09149d0*CON_KCALTOK
      tclocal(11) =     -0.16451d0*CON_KCALTOK
      tclocal(12) =      1.15037d0*CON_KCALTOK
      tclocal(13) =      2.34631d0*CON_KCALTOK
      tclocal(14) =      0.54433d0*CON_KCALTOK
      tclocal(15) =     -1.00246d0*CON_KCALTOK
      tclocal(16) =      1.29176d0*CON_KCALTOK
      tclocal(17) =      0.95743d0*CON_KCALTOK
      tclocal(18) =     -3.19413d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'n  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'n  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00899d0*CON_KCALTOK
      tclocal(1) =     -0.00663d0*CON_KCALTOK
      tclocal(2) =     -0.39958d0*CON_KCALTOK
      tclocal(3) =      0.58619d0*CON_KCALTOK
      tclocal(4) =     -0.14715d0*CON_KCALTOK
      tclocal(5) =      0.66498d0*CON_KCALTOK
      tclocal(6) =     -0.23897d0*CON_KCALTOK
      tclocal(7) =     -0.81143d0*CON_KCALTOK
      tclocal(8) =     -0.18042d0*CON_KCALTOK
      tclocal( 9) =     -0.09492d0*CON_KCALTOK
      tclocal(10) =      0.37473d0*CON_KCALTOK
      tclocal(11) =      0.05721d0*CON_KCALTOK
      tclocal(12) =      0.09748d0*CON_KCALTOK
      tclocal(13) =     -0.56149d0*CON_KCALTOK
      tclocal(14) =      0.92109d0*CON_KCALTOK
      tclocal(15) =     -1.82386d0*CON_KCALTOK
      tclocal(16) =      4.82027d0*CON_KCALTOK
      tclocal(17) =      1.91827d0*CON_KCALTOK
      tclocal(18) =     -4.12139d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'n  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'n  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.47139d0*CON_KCALTOK
      tclocal(1) =      0.57588d0*CON_KCALTOK
      tclocal(2) =      0.06384d0*CON_KCALTOK
      tclocal(3) =     -0.07992d0*CON_KCALTOK
      tclocal(4) =     -4.06635d0*CON_KCALTOK
      tclocal(5) =      2.20175d0*CON_KCALTOK
      tclocal(6) =      1.16902d0*CON_KCALTOK
      tclocal(7) =      0.27790d0*CON_KCALTOK
      tclocal(8) =     -0.57657d0*CON_KCALTOK
      tclocal( 9) =     -3.31350d0*CON_KCALTOK
      tclocal(10) =      2.21155d0*CON_KCALTOK
      tclocal(11) =     -1.41795d0*CON_KCALTOK
      tclocal(12) =      1.67651d0*CON_KCALTOK
      tclocal(13) =      0.19412d0*CON_KCALTOK
      tclocal(14) =      0.30033d0*CON_KCALTOK
      tclocal(15) =     -3.26118d0*CON_KCALTOK
      tclocal(16) =      5.46693d0*CON_KCALTOK
      tclocal(17) =     -3.76650d0*CON_KCALTOK
      tclocal(18) =     -8.35552d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'n  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'n  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.04732d0*CON_KCALTOK
      tclocal(1) =     -0.05395d0*CON_KCALTOK
      tclocal(2) =      0.04690d0*CON_KCALTOK
      tclocal(3) =     -1.17761d0*CON_KCALTOK
      tclocal(4) =      2.04941d0*CON_KCALTOK
      tclocal(5) =     -0.44554d0*CON_KCALTOK
      tclocal(6) =     -0.32890d0*CON_KCALTOK
      tclocal(7) =      0.00222d0*CON_KCALTOK
      tclocal(8) =     -1.08477d0*CON_KCALTOK
      tclocal( 9) =     -0.71425d0*CON_KCALTOK
      tclocal(10) =      0.82365d0*CON_KCALTOK
      tclocal(11) =     -0.14718d0*CON_KCALTOK
      tclocal(12) =      1.87907d0*CON_KCALTOK
      tclocal(13) =     -1.57910d0*CON_KCALTOK
      tclocal(14) =     -0.27992d0*CON_KCALTOK
      tclocal(15) =     -4.75137d0*CON_KCALTOK
      tclocal(16) =      3.62661d0*CON_KCALTOK
      tclocal(17) =      1.22977d0*CON_KCALTOK
      tclocal(18) =     -4.50000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'n  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'n  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.33741d0*CON_KCALTOK
      tclocal(1) =      0.54015d0*CON_KCALTOK
      tclocal(2) =     -0.49869d0*CON_KCALTOK
      tclocal(3) =      2.21368d0*CON_KCALTOK
      tclocal(4) =      2.01207d0*CON_KCALTOK
      tclocal(5) =      1.46808d0*CON_KCALTOK
      tclocal(6) =     -0.03633d0*CON_KCALTOK
      tclocal(7) =     -7.41464d0*CON_KCALTOK
      tclocal(8) =     -2.14043d0*CON_KCALTOK
      tclocal( 9) =     -1.16381d0*CON_KCALTOK
      tclocal(10) =     -1.01483d0*CON_KCALTOK
      tclocal(11) =     -0.55413d0*CON_KCALTOK
      tclocal(12) =      2.83910d0*CON_KCALTOK
      tclocal(13) =     -0.73591d0*CON_KCALTOK
      tclocal(14) =     -0.92538d0*CON_KCALTOK
      tclocal(15) =    -11.89348d0*CON_KCALTOK
      tclocal(16) =      1.35203d0*CON_KCALTOK
      tclocal(17) =     -2.62629d0*CON_KCALTOK
      tclocal(18) =    -12.97656d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'n  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'n  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.11972d0*CON_KCALTOK
      tclocal(1) =     -0.13196d0*CON_KCALTOK
      tclocal(2) =     -0.14644d0*CON_KCALTOK
      tclocal(3) =      0.19798d0*CON_KCALTOK
      tclocal(4) =     -0.04217d0*CON_KCALTOK
      tclocal(5) =     -0.18949d0*CON_KCALTOK
      tclocal(6) =      0.03327d0*CON_KCALTOK
      tclocal(7) =     -0.41938d0*CON_KCALTOK
      tclocal(8) =      0.77581d0*CON_KCALTOK
      tclocal( 9) =     -0.82319d0*CON_KCALTOK
      tclocal(10) =     -0.52203d0*CON_KCALTOK
      tclocal(11) =      1.18163d0*CON_KCALTOK
      tclocal(12) =      4.19438d0*CON_KCALTOK
      tclocal(13) =     -0.25928d0*CON_KCALTOK
      tclocal(14) =      0.29267d0*CON_KCALTOK
      tclocal(15) =     -3.56703d0*CON_KCALTOK
      tclocal(16) =      2.43221d0*CON_KCALTOK
      tclocal(17) =      0.19419d0*CON_KCALTOK
      tclocal(18) =     -2.05809d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'n  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'n  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.24116d0*CON_KCALTOK
      tclocal(1) =      0.72225d0*CON_KCALTOK
      tclocal(2) =     -0.10166d0*CON_KCALTOK
      tclocal(3) =      1.13931d0*CON_KCALTOK
      tclocal(4) =     -1.34592d0*CON_KCALTOK
      tclocal(5) =     -0.14086d0*CON_KCALTOK
      tclocal(6) =      0.20717d0*CON_KCALTOK
      tclocal(7) =      0.70954d0*CON_KCALTOK
      tclocal(8) =     -0.75880d0*CON_KCALTOK
      tclocal( 9) =     -2.44081d0*CON_KCALTOK
      tclocal(10) =      0.91837d0*CON_KCALTOK
      tclocal(11) =     -0.42141d0*CON_KCALTOK
      tclocal(12) =     -0.68172d0*CON_KCALTOK
      tclocal(13) =     -0.07373d0*CON_KCALTOK
      tclocal(14) =      0.20400d0*CON_KCALTOK
      tclocal(15) =     -7.81094d0*CON_KCALTOK
      tclocal(16) =      4.07025d0*CON_KCALTOK
      tclocal(17) =     -1.71178d0*CON_KCALTOK
      tclocal(18) =    -10.35921d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'n  ')
      call twh_torstring(GLB_SET,type,3,'s" ')
      call twh_torstring(GLB_SET,type,4,'o` ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'n  '
      tnlocal(nsame,3) = 's" '
      tnlocal(nsame,4) = 'o` '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.68426d0*CON_KCALTOK
      tclocal(1) =     16.41694d0*CON_KCALTOK
      tclocal(2) =      0.68855d0*CON_KCALTOK
      tclocal(3) =     -1.90878d0*CON_KCALTOK
      tclocal(4) =      2.94479d0*CON_KCALTOK
      tclocal(5) =     -0.90882d0*CON_KCALTOK
      tclocal(6) =     -4.41573d0*CON_KCALTOK
      tclocal(7) =     18.93605d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =     -1.90878d0*CON_KCALTOK
      tclocal(10) =      2.94479d0*CON_KCALTOK
      tclocal(11) =     -0.90882d0*CON_KCALTOK
      tclocal(12) =     -1.77517d0*CON_KCALTOK
      tclocal(13) =      8.02061d0*CON_KCALTOK
      tclocal(14) =     -0.18826d0*CON_KCALTOK
      tclocal(15) =     -1.77517d0*CON_KCALTOK
      tclocal(16) =      8.02061d0*CON_KCALTOK
      tclocal(17) =     -0.18826d0*CON_KCALTOK
      tclocal(18) =      3.43521d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'n= ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'n= '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =     18.88224d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'n= ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'n= '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.42649d0*CON_KCALTOK
      tclocal(1) =     16.74404d0*CON_KCALTOK
      tclocal(2) =     -1.12317d0*CON_KCALTOK
      tclocal(3) =     -2.10009d0*CON_KCALTOK
      tclocal(4) =      2.19568d0*CON_KCALTOK
      tclocal(5) =     -0.89526d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     14.56761d0*CON_KCALTOK
      tclocal(8) =     -4.38691d0*CON_KCALTOK
      tclocal( 9) =     -5.26640d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     -9.46275d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =     -3.18574d0*CON_KCALTOK
      tclocal(16) =      1.98019d0*CON_KCALTOK
      tclocal(17) =     -3.62082d0*CON_KCALTOK
      tclocal(18) =    -13.90012d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'n= ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'n= '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.54064d0*CON_KCALTOK
      tclocal(1) =     17.26681d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      1.39910d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      1.39910d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     -0.58234d0*CON_KCALTOK
      tclocal(13) =     -1.92145d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =     -0.58234d0*CON_KCALTOK
      tclocal(16) =     -1.92145d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     -9.78230d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'n= ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'n= '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -2.32740d0*CON_KCALTOK
      tclocal(1) =     18.11063d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      2.40141d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      8.92461d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =     -1.45763d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     -9.46275d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =     -0.99972d0*CON_KCALTOK
      tclocal(16) =      1.89270d0*CON_KCALTOK
      tclocal(17) =     -1.64977d0*CON_KCALTOK
      tclocal(18) =     -7.59843d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'n= ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'n= '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.05169d0*CON_KCALTOK
      tclocal(1) =     19.44674d0*CON_KCALTOK
      tclocal(2) =     -0.69701d0*CON_KCALTOK
      tclocal(3) =     -2.34317d0*CON_KCALTOK
      tclocal(4) =      3.48651d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =     -5.45693d0*CON_KCALTOK
      tclocal(7) =     11.41471d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =     -2.34317d0*CON_KCALTOK
      tclocal(10) =      3.48651d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =     -5.22972d0*CON_KCALTOK
      tclocal(13) =      3.29698d0*CON_KCALTOK
      tclocal(14) =     -0.88270d0*CON_KCALTOK
      tclocal(15) =     -5.22972d0*CON_KCALTOK
      tclocal(16) =      3.29698d0*CON_KCALTOK
      tclocal(17) =     -0.88270d0*CON_KCALTOK
      tclocal(18) =     -5.41409d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'n= ')
      call twh_torstring(GLB_SET,type,3,'n= ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'n= '
      tnlocal(nsame,3) = 'n= '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.59315d0*CON_KCALTOK
      tclocal(1) =     -2.52382d0*CON_KCALTOK
      tclocal(2) =     -0.75156d0*CON_KCALTOK
      tclocal(3) =      0.12739d0*CON_KCALTOK
      tclocal(4) =      1.49614d0*CON_KCALTOK
      tclocal(5) =     -0.00093d0*CON_KCALTOK
      tclocal(6) =     -0.73905d0*CON_KCALTOK
      tclocal(7) =    -11.50784d0*CON_KCALTOK
      tclocal(8) =     -2.33584d0*CON_KCALTOK
      tclocal( 9) =      0.12739d0*CON_KCALTOK
      tclocal(10) =      1.49614d0*CON_KCALTOK
      tclocal(11) =     -0.00093d0*CON_KCALTOK
      tclocal(12) =     -3.60715d0*CON_KCALTOK
      tclocal(13) =      0.04154d0*CON_KCALTOK
      tclocal(14) =     -1.67655d0*CON_KCALTOK
      tclocal(15) =     -3.60715d0*CON_KCALTOK
      tclocal(16) =      0.04154d0*CON_KCALTOK
      tclocal(17) =     -1.67655d0*CON_KCALTOK
      tclocal(18) =    -21.46361d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'na ')
      call twh_torstring(GLB_SET,type,3,'na ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'na '
      tnlocal(nsame,3) = 'na '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.81612d0*CON_KCALTOK
      tclocal(1) =     -3.13427d0*CON_KCALTOK
      tclocal(2) =     -0.38748d0*CON_KCALTOK
      tclocal(3) =     -0.31005d0*CON_KCALTOK
      tclocal(4) =      1.87707d0*CON_KCALTOK
      tclocal(5) =      0.24761d0*CON_KCALTOK
      tclocal(6) =      0.62834d0*CON_KCALTOK
      tclocal(7) =     -9.05034d0*CON_KCALTOK
      tclocal(8) =     -1.49877d0*CON_KCALTOK
      tclocal( 9) =     -4.38116d0*CON_KCALTOK
      tclocal(10) =      2.05000d0*CON_KCALTOK
      tclocal(11) =      1.46583d0*CON_KCALTOK
      tclocal(12) =     -7.38969d0*CON_KCALTOK
      tclocal(13) =      0.43762d0*CON_KCALTOK
      tclocal(14) =     -0.01091d0*CON_KCALTOK
      tclocal(15) =     -4.63041d0*CON_KCALTOK
      tclocal(16) =     -1.00398d0*CON_KCALTOK
      tclocal(17) =     -0.31234d0*CON_KCALTOK
      tclocal(18) =    -17.05311d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'na ')
      call twh_torstring(GLB_SET,type,3,'na ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'na '
      tnlocal(nsame,3) = 'na '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -1.19767d0*CON_KCALTOK
      tclocal(1) =     -3.81767d0*CON_KCALTOK
      tclocal(2) =     -0.23914d0*CON_KCALTOK
      tclocal(3) =     -3.14829d0*CON_KCALTOK
      tclocal(4) =      1.45929d0*CON_KCALTOK
      tclocal(5) =      0.33737d0*CON_KCALTOK
      tclocal(6) =     -0.84015d0*CON_KCALTOK
      tclocal(7) =     -9.44352d0*CON_KCALTOK
      tclocal(8) =     -0.54744d0*CON_KCALTOK
      tclocal( 9) =     -3.14829d0*CON_KCALTOK
      tclocal(10) =      1.45929d0*CON_KCALTOK
      tclocal(11) =      0.33737d0*CON_KCALTOK
      tclocal(12) =     -6.33773d0*CON_KCALTOK
      tclocal(13) =     -1.81411d0*CON_KCALTOK
      tclocal(14) =     -0.35410d0*CON_KCALTOK
      tclocal(15) =     -6.33773d0*CON_KCALTOK
      tclocal(16) =     -1.81411d0*CON_KCALTOK
      tclocal(17) =     -0.35410d0*CON_KCALTOK
      tclocal(18) =    -14.82895d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'na ')
      call twh_torstring(GLB_SET,type,3,'na ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'na '
      tnlocal(nsame,3) = 'na '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10303d0*CON_KCALTOK
      tclocal(1) =      3.29296d0*CON_KCALTOK
      tclocal(2) =      0.65786d0*CON_KCALTOK
      tclocal(3) =     -5.20804d0*CON_KCALTOK
      tclocal(4) =      9.50152d0*CON_KCALTOK
      tclocal(5) =      8.88436d0*CON_KCALTOK
      tclocal(6) =    -32.29330d0*CON_KCALTOK
      tclocal(7) =     17.10071d0*CON_KCALTOK
      tclocal(8) =     32.69798d0*CON_KCALTOK
      tclocal( 9) =      7.00922d0*CON_KCALTOK
      tclocal(10) =     -2.46427d0*CON_KCALTOK
      tclocal(11) =     -3.68879d0*CON_KCALTOK
      tclocal(12) =     -3.08771d0*CON_KCALTOK
      tclocal(13) =     16.05008d0*CON_KCALTOK
      tclocal(14) =      4.26405d0*CON_KCALTOK
      tclocal(15) =     -1.43886d0*CON_KCALTOK
      tclocal(16) =      9.43691d0*CON_KCALTOK
      tclocal(17) =      6.06227d0*CON_KCALTOK
      tclocal(18) =    -25.49177d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'nh ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'nh '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.09231d0*CON_KCALTOK
      tclocal(1) =      8.23605d0*CON_KCALTOK
      tclocal(2) =      1.08322d0*CON_KCALTOK
      tclocal(3) =     -3.81042d0*CON_KCALTOK
      tclocal(4) =     -0.55077d0*CON_KCALTOK
      tclocal(5) =      9.24568d0*CON_KCALTOK
      tclocal(6) =    -38.59937d0*CON_KCALTOK
      tclocal(7) =    -15.88707d0*CON_KCALTOK
      tclocal(8) =     19.95960d0*CON_KCALTOK
      tclocal( 9) =      6.57631d0*CON_KCALTOK
      tclocal(10) =     -1.86085d0*CON_KCALTOK
      tclocal(11) =     -8.59427d0*CON_KCALTOK
      tclocal(12) =      9.76107d0*CON_KCALTOK
      tclocal(13) =     21.69986d0*CON_KCALTOK
      tclocal(14) =     -2.23922d0*CON_KCALTOK
      tclocal(15) =    -24.08437d0*CON_KCALTOK
      tclocal(16) =     -2.24289d0*CON_KCALTOK
      tclocal(17) =     14.33058d0*CON_KCALTOK
      tclocal(18) =    -40.59531d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'nh ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'nh '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.10303d0*CON_KCALTOK
      tclocal(1) =      3.29296d0*CON_KCALTOK
      tclocal(2) =      0.65786d0*CON_KCALTOK
      tclocal(3) =     -5.20804d0*CON_KCALTOK
      tclocal(4) =      9.50152d0*CON_KCALTOK
      tclocal(5) =      8.88436d0*CON_KCALTOK
      tclocal(6) =    -32.29330d0*CON_KCALTOK
      tclocal(7) =     17.10071d0*CON_KCALTOK
      tclocal(8) =     32.69798d0*CON_KCALTOK
      tclocal( 9) =      7.00922d0*CON_KCALTOK
      tclocal(10) =     -2.46427d0*CON_KCALTOK
      tclocal(11) =     -3.68879d0*CON_KCALTOK
      tclocal(12) =     -3.08771d0*CON_KCALTOK
      tclocal(13) =     16.05008d0*CON_KCALTOK
      tclocal(14) =      4.26405d0*CON_KCALTOK
      tclocal(15) =     -1.43886d0*CON_KCALTOK
      tclocal(16) =      9.43691d0*CON_KCALTOK
      tclocal(17) =      6.06227d0*CON_KCALTOK
      tclocal(18) =    -25.49177d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'nh ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'nh '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      5.21694d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      5.79175d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     20.03994d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -11.98848d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      4.57948d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.00762d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -28.30798d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'nh ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'nh '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      5.21694d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      5.79175d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     20.03994d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -11.98848d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      4.57948d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     -0.00762d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -28.30798d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'nh ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'nh '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      5.00990d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      2.34549d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      5.58345d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     -8.32079d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     -4.73133d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      1.54494d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -26.92547d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'nh ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'nh '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00696d0*CON_KCALTOK
      tclocal(1) =      5.39416d0*CON_KCALTOK
      tclocal(2) =      0.54057d0*CON_KCALTOK
      tclocal(3) =     -5.96568d0*CON_KCALTOK
      tclocal(4) =     -1.06392d0*CON_KCALTOK
      tclocal(5) =     -0.40677d0*CON_KCALTOK
      tclocal(6) =     13.87537d0*CON_KCALTOK
      tclocal(7) =     24.81186d0*CON_KCALTOK
      tclocal(8) =     13.70316d0*CON_KCALTOK
      tclocal( 9) =     -7.13907d0*CON_KCALTOK
      tclocal(10) =    -12.37899d0*CON_KCALTOK
      tclocal(11) =     -5.73272d0*CON_KCALTOK
      tclocal(12) =     17.40886d0*CON_KCALTOK
      tclocal(13) =     14.77319d0*CON_KCALTOK
      tclocal(14) =      3.00827d0*CON_KCALTOK
      tclocal(15) =      5.47550d0*CON_KCALTOK
      tclocal(16) =     27.90795d0*CON_KCALTOK
      tclocal(17) =      9.14606d0*CON_KCALTOK
      tclocal(18) =     -8.00861d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'nh ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'nh '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00696d0*CON_KCALTOK
      tclocal(1) =      5.39416d0*CON_KCALTOK
      tclocal(2) =      0.54057d0*CON_KCALTOK
      tclocal(3) =     -5.96568d0*CON_KCALTOK
      tclocal(4) =     -1.06392d0*CON_KCALTOK
      tclocal(5) =     -0.40677d0*CON_KCALTOK
      tclocal(6) =     13.87537d0*CON_KCALTOK
      tclocal(7) =     24.81186d0*CON_KCALTOK
      tclocal(8) =     13.70316d0*CON_KCALTOK
      tclocal( 9) =     -7.13907d0*CON_KCALTOK
      tclocal(10) =    -12.37899d0*CON_KCALTOK
      tclocal(11) =     -5.73272d0*CON_KCALTOK
      tclocal(12) =     17.40886d0*CON_KCALTOK
      tclocal(13) =     14.77319d0*CON_KCALTOK
      tclocal(14) =      3.00827d0*CON_KCALTOK
      tclocal(15) =      5.47550d0*CON_KCALTOK
      tclocal(16) =     27.90795d0*CON_KCALTOK
      tclocal(17) =      9.14606d0*CON_KCALTOK
      tclocal(18) =     -8.00861d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'nh ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cpb')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'nh '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cpb'
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.32493d0*CON_KCALTOK
      tclocal(1) =      8.27169d0*CON_KCALTOK
      tclocal(2) =      2.26730d0*CON_KCALTOK
      tclocal(3) =      4.43913d0*CON_KCALTOK
      tclocal(4) =      4.69158d0*CON_KCALTOK
      tclocal(5) =      7.62106d0*CON_KCALTOK
      tclocal(6) =    -12.22822d0*CON_KCALTOK
      tclocal(7) =     -2.32073d0*CON_KCALTOK
      tclocal(8) =     29.98807d0*CON_KCALTOK
      tclocal( 9) =      7.07667d0*CON_KCALTOK
      tclocal(10) =      0.41433d0*CON_KCALTOK
      tclocal(11) =    -15.63790d0*CON_KCALTOK
      tclocal(12) =     10.68042d0*CON_KCALTOK
      tclocal(13) =     20.22235d0*CON_KCALTOK
      tclocal(14) =     14.42169d0*CON_KCALTOK
      tclocal(15) =    -23.84895d0*CON_KCALTOK
      tclocal(16) =     18.05960d0*CON_KCALTOK
      tclocal(17) =     27.13415d0*CON_KCALTOK
      tclocal(18) =      1.04814d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'nh ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'nh '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      6.57813d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -20.93435d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     19.30000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -20.93435d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     28.40382d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     28.40382d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -36.42702d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'np ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'np '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =     12.17490d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -48.58023d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     43.15572d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.52987d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     52.45256d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      8.14646d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =   -108.21271d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'np ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'np '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =     11.55398d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -18.20230d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     28.28769d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -26.80750d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     42.74496d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     23.90351d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -27.32161d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'np ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'np '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      7.94093d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -5.80711d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =    111.40153d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -25.50476d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'np ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'op ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'np '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'op '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      9.42548d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -15.71875d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     18.70196d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -23.38269d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'np ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'sp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'np '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'sp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =     12.17490d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -48.58023d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     43.15572d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.52987d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     52.45256d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      8.14646d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =   -108.21271d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'np ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'nh ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'np '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'nh '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =     12.18926d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -17.93631d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     49.43705d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -35.99905d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     16.04713d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     36.94224d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -97.31497d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'nh ')
      call twh_torstring(GLB_SET,type,2,'np ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'nh '
      tnlocal(nsame,2) = 'np '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =     17.74782d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -20.65957d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     32.29770d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =    -20.65957d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     33.00705d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =     33.00705d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =     27.11317d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'np ')
      call twh_torstring(GLB_SET,type,3,'np ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'np '
      tnlocal(nsame,3) = 'np '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.74829d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -16.86784d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     15.23519d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -13.38337d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'np ')
      call twh_torstring(GLB_SET,type,3,'op ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'np '
      tnlocal(nsame,3) = 'op '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      7.52964d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     -2.19746d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     11.28642d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =    -30.31447d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =      0.00000d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'np ')
      call twh_torstring(GLB_SET,type,3,'op ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'np '
      tnlocal(nsame,3) = 'op '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      4.74829d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =    -16.86784d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =     15.23519d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -13.38337d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cpb')
      call twh_torstring(GLB_SET,type,2,'np ')
      call twh_torstring(GLB_SET,type,3,'op ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cpb'
      tnlocal(nsame,2) = 'np '
      tnlocal(nsame,3) = 'op '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      8.77623d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =      0.00000d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =     66.27147d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -39.26680d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'np ')
      call twh_torstring(GLB_SET,type,3,'op ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'np '
      tnlocal(nsame,3) = 'op '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      1.72468d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     12.55477d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =     17.19386d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =    -18.06772d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      0.00000d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -47.39876d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'np ')
      call twh_torstring(GLB_SET,type,3,'sp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'np '
      tnlocal(nsame,3) = 'sp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =      7.88257d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     26.00769d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =      0.00000d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =    -32.15449d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =    -92.68162d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -44.94305d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'np ')
      call twh_torstring(GLB_SET,type,3,'sp ')
      call twh_torstring(GLB_SET,type,4,'np ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'np '
      tnlocal(nsame,3) = 'sp '
      tnlocal(nsame,4) = 'np '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.00000d0*CON_KCALTOK
      tclocal(1) =     12.30292d0*CON_KCALTOK
      tclocal(2) =      0.00000d0*CON_KCALTOK
      tclocal(3) =      0.00000d0*CON_KCALTOK
      tclocal(4) =     66.44578d0*CON_KCALTOK
      tclocal(5) =      0.00000d0*CON_KCALTOK
      tclocal(6) =      0.00000d0*CON_KCALTOK
      tclocal(7) =    -24.38398d0*CON_KCALTOK
      tclocal(8) =      0.00000d0*CON_KCALTOK
      tclocal( 9) =      0.00000d0*CON_KCALTOK
      tclocal(10) =      0.00000d0*CON_KCALTOK
      tclocal(11) =      0.00000d0*CON_KCALTOK
      tclocal(12) =      0.00000d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =      0.00000d0*CON_KCALTOK
      tclocal(15) =      0.00000d0*CON_KCALTOK
      tclocal(16) =      9.66775d0*CON_KCALTOK
      tclocal(17) =      0.00000d0*CON_KCALTOK
      tclocal(18) =    -51.12035d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'np ')
      call twh_torstring(GLB_SET,type,2,'np ')
      call twh_torstring(GLB_SET,type,3,'sp ')
      call twh_torstring(GLB_SET,type,4,'cp ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'np '
      tnlocal(nsame,2) = 'np '
      tnlocal(nsame,3) = 'sp '
      tnlocal(nsame,4) = 'cp '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.29303d0*CON_KCALTOK
      tclocal(1) =      0.68057d0*CON_KCALTOK
      tclocal(2) =     -0.11313d0*CON_KCALTOK
      tclocal(3) =     -0.45935d0*CON_KCALTOK
      tclocal(4) =     -1.10873d0*CON_KCALTOK
      tclocal(5) =     -0.27083d0*CON_KCALTOK
      tclocal(6) =      0.17518d0*CON_KCALTOK
      tclocal(7) =     -0.73302d0*CON_KCALTOK
      tclocal(8) =     -0.00101d0*CON_KCALTOK
      tclocal( 9) =     -6.05824d0*CON_KCALTOK
      tclocal(10) =      1.69084d0*CON_KCALTOK
      tclocal(11) =      0.37468d0*CON_KCALTOK
      tclocal(12) =     -0.76248d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =     -0.19175d0*CON_KCALTOK
      tclocal(15) =      0.70259d0*CON_KCALTOK
      tclocal(16) =      1.48819d0*CON_KCALTOK
      tclocal(17) =      0.69188d0*CON_KCALTOK
      tclocal(18) =     -9.79017d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'o  ')
      call twh_torstring(GLB_SET,type,3,'p  ')
      call twh_torstring(GLB_SET,type,4,'o  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'o  '
      tnlocal(nsame,3) = 'p  '
      tnlocal(nsame,4) = 'o  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.22275d0*CON_KCALTOK
      tclocal(1) =      1.07635d0*CON_KCALTOK
      tclocal(2) =     -0.04569d0*CON_KCALTOK
      tclocal(3) =     -3.01910d0*CON_KCALTOK
      tclocal(4) =     -1.30922d0*CON_KCALTOK
      tclocal(5) =     -0.09121d0*CON_KCALTOK
      tclocal(6) =      0.01766d0*CON_KCALTOK
      tclocal(7) =      1.19206d0*CON_KCALTOK
      tclocal(8) =     -0.17642d0*CON_KCALTOK
      tclocal( 9) =     -5.78843d0*CON_KCALTOK
      tclocal(10) =      0.57884d0*CON_KCALTOK
      tclocal(11) =     -0.26913d0*CON_KCALTOK
      tclocal(12) =      0.24477d0*CON_KCALTOK
      tclocal(13) =      0.00000d0*CON_KCALTOK
      tclocal(14) =     -0.14909d0*CON_KCALTOK
      tclocal(15) =      1.02249d0*CON_KCALTOK
      tclocal(16) =      2.78881d0*CON_KCALTOK
      tclocal(17) =     -0.04551d0*CON_KCALTOK
      tclocal(18) =    -12.75598d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'o  ')
      call twh_torstring(GLB_SET,type,3,'p  ')
      call twh_torstring(GLB_SET,type,4,'o- ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'o  '
      tnlocal(nsame,3) = 'p  '
      tnlocal(nsame,4) = 'o- '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      2.18489d0*CON_KCALTOK
      tclocal(1) =      1.92902d0*CON_KCALTOK
      tclocal(2) =      0.01356d0*CON_KCALTOK
      tclocal(3) =     -5.36892d0*CON_KCALTOK
      tclocal(4) =     -2.10947d0*CON_KCALTOK
      tclocal(5) =     -3.35883d0*CON_KCALTOK
      tclocal(6) =     -2.30134d0*CON_KCALTOK
      tclocal(7) =      2.52884d0*CON_KCALTOK
      tclocal(8) =     -2.30722d0*CON_KCALTOK
      tclocal( 9) =     -5.20845d0*CON_KCALTOK
      tclocal(10) =      0.38312d0*CON_KCALTOK
      tclocal(11) =     -0.70321d0*CON_KCALTOK
      tclocal(12) =      3.92024d0*CON_KCALTOK
      tclocal(13) =      2.67639d0*CON_KCALTOK
      tclocal(14) =     -0.77818d0*CON_KCALTOK
      tclocal(15) =      4.96988d0*CON_KCALTOK
      tclocal(16) =      5.88228d0*CON_KCALTOK
      tclocal(17) =     -1.37089d0*CON_KCALTOK
      tclocal(18) =    -28.00678d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'o  ')
      call twh_torstring(GLB_SET,type,3,'p- ')
      call twh_torstring(GLB_SET,type,4,'o- ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 'o  '
      tnlocal(nsame,3) = 'p- '
      tnlocal(nsame,4) = 'o- '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =      0.29723d0*CON_KCALTOK
      tclocal(1) =      0.41339d0*CON_KCALTOK
      tclocal(2) =      0.64034d0*CON_KCALTOK
      tclocal(3) =     -2.85805d0*CON_KCALTOK
      tclocal(4) =     -2.57968d0*CON_KCALTOK
      tclocal(5) =     -3.40452d0*CON_KCALTOK
      tclocal(6) =      4.56411d0*CON_KCALTOK
      tclocal(7) =     -0.26532d0*CON_KCALTOK
      tclocal(8) =      0.22932d0*CON_KCALTOK
      tclocal( 9) =     -4.88946d0*CON_KCALTOK
      tclocal(10) =      0.23973d0*CON_KCALTOK
      tclocal(11) =     -3.26764d0*CON_KCALTOK
      tclocal(12) =     -1.18518d0*CON_KCALTOK
      tclocal(13) =     -0.10803d0*CON_KCALTOK
      tclocal(14) =     -0.75169d0*CON_KCALTOK
      tclocal(15) =      0.14717d0*CON_KCALTOK
      tclocal(16) =      2.15427d0*CON_KCALTOK
      tclocal(17) =     -2.52034d0*CON_KCALTOK
      tclocal(18) =     -7.86046d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'cp ')
      call twh_torstring(GLB_SET,type,2,'o  ')
      call twh_torstring(GLB_SET,type,3,'p- ')
      call twh_torstring(GLB_SET,type,4,'o- ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'cp '
      tnlocal(nsame,2) = 'o  '
      tnlocal(nsame,3) = 'p- '
      tnlocal(nsame,4) = 'o- '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.16410d0*CON_KCALTOK
      tclocal(1) =      0.08926d0*CON_KCALTOK
      tclocal(2) =     -0.08768d0*CON_KCALTOK
      tclocal(3) =      1.28184d0*CON_KCALTOK
      tclocal(4) =      1.33466d0*CON_KCALTOK
      tclocal(5) =      1.49345d0*CON_KCALTOK
      tclocal(6) =      2.17508d0*CON_KCALTOK
      tclocal(7) =     -0.39472d0*CON_KCALTOK
      tclocal(8) =     -0.73230d0*CON_KCALTOK
      tclocal( 9) =     -3.68552d0*CON_KCALTOK
      tclocal(10) =     -0.24068d0*CON_KCALTOK
      tclocal(11) =     -0.02773d0*CON_KCALTOK
      tclocal(12) =      0.54572d0*CON_KCALTOK
      tclocal(13) =     -0.30409d0*CON_KCALTOK
      tclocal(14) =     -0.25175d0*CON_KCALTOK
      tclocal(15) =     -3.25990d0*CON_KCALTOK
      tclocal(16) =      1.43483d0*CON_KCALTOK
      tclocal(17) =     -0.30983d0*CON_KCALTOK
      tclocal(18) =     -2.74061d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'o  ')
      call twh_torstring(GLB_SET,type,3,'p- ')
      call twh_torstring(GLB_SET,type,4,'o- ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 'o  '
      tnlocal(nsame,3) = 'p- '
      tnlocal(nsame,4) = 'o- '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.90684d0*CON_KCALTOK
      tclocal(1) =     -3.38909d0*CON_KCALTOK
      tclocal(2) =     -0.65942d0*CON_KCALTOK
      tclocal(3) =      0.19282d0*CON_KCALTOK
      tclocal(4) =      0.07007d0*CON_KCALTOK
      tclocal(5) =     -0.12607d0*CON_KCALTOK
      tclocal(6) =      0.22671d0*CON_KCALTOK
      tclocal(7) =    -10.92156d0*CON_KCALTOK
      tclocal(8) =     -2.22252d0*CON_KCALTOK
      tclocal( 9) =      0.19282d0*CON_KCALTOK
      tclocal(10) =      0.07007d0*CON_KCALTOK
      tclocal(11) =     -0.12607d0*CON_KCALTOK
      tclocal(12) =     -0.88917d0*CON_KCALTOK
      tclocal(13) =      4.42063d0*CON_KCALTOK
      tclocal(14) =     -0.80576d0*CON_KCALTOK
      tclocal(15) =     -0.88917d0*CON_KCALTOK
      tclocal(16) =      4.42063d0*CON_KCALTOK
      tclocal(17) =     -0.80576d0*CON_KCALTOK
      tclocal(18) =    -53.23111d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'s  ')
      call twh_torstring(GLB_SET,type,3,'s  ')
      call twh_torstring(GLB_SET,type,4,'c  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 's  '
      tnlocal(nsame,3) = 's  '
      tnlocal(nsame,4) = 'c  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.35619d0*CON_KCALTOK
      tclocal(1) =     -3.38269d0*CON_KCALTOK
      tclocal(2) =     -0.34842d0*CON_KCALTOK
      tclocal(3) =      0.33177d0*CON_KCALTOK
      tclocal(4) =      1.15551d0*CON_KCALTOK
      tclocal(5) =     -0.09944d0*CON_KCALTOK
      tclocal(6) =      0.93792d0*CON_KCALTOK
      tclocal(7) =     -9.57986d0*CON_KCALTOK
      tclocal(8) =     -1.26075d0*CON_KCALTOK
      tclocal( 9) =     -0.07340d0*CON_KCALTOK
      tclocal(10) =      0.48637d0*CON_KCALTOK
      tclocal(11) =      0.21618d0*CON_KCALTOK
      tclocal(12) =     -2.86759d0*CON_KCALTOK
      tclocal(13) =      4.70258d0*CON_KCALTOK
      tclocal(14) =     -0.29652d0*CON_KCALTOK
      tclocal(15) =     -1.48883d0*CON_KCALTOK
      tclocal(16) =      3.63335d0*CON_KCALTOK
      tclocal(17) =      0.34988d0*CON_KCALTOK
      tclocal(18) =    -37.74289d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'c  ')
      call twh_torstring(GLB_SET,type,2,'s  ')
      call twh_torstring(GLB_SET,type,3,'s  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'c  '
      tnlocal(nsame,2) = 's  '
      tnlocal(nsame,3) = 's  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)

      type = type + 1
      call twh_torff(GLB_SET,type,ffname)
      call twh_torstyle(GLB_SET,type,5)
      call twh_loftor(GLB_SET,type,.true.)
      tclocal(-1) = 1.0d0
      tclocal(0) =     -0.12741d0*CON_KCALTOK
      tclocal(1) =     -3.51794d0*CON_KCALTOK
      tclocal(2) =     -0.26190d0*CON_KCALTOK
      tclocal(3) =      0.15271d0*CON_KCALTOK
      tclocal(4) =      0.17023d0*CON_KCALTOK
      tclocal(5) =      0.19542d0*CON_KCALTOK
      tclocal(6) =     -1.02693d0*CON_KCALTOK
      tclocal(7) =     -7.70273d0*CON_KCALTOK
      tclocal(8) =     -1.19188d0*CON_KCALTOK
      tclocal( 9) =      0.15271d0*CON_KCALTOK
      tclocal(10) =      0.17023d0*CON_KCALTOK
      tclocal(11) =      0.19542d0*CON_KCALTOK
      tclocal(12) =     -2.23283d0*CON_KCALTOK
      tclocal(13) =      2.74002d0*CON_KCALTOK
      tclocal(14) =     -0.68926d0*CON_KCALTOK
      tclocal(15) =     -2.23283d0*CON_KCALTOK
      tclocal(16) =      2.74002d0*CON_KCALTOK
      tclocal(17) =     -0.68926d0*CON_KCALTOK
      tclocal(18) =    -23.89908d0*CON_KCALTOK
      tclocal(19) = 0.0d0*CON_KCALTOK
      call twh_set_torcoeff(type,tclocal)
      call twh_torstring(GLB_SET,type,1,'h  ')
      call twh_torstring(GLB_SET,type,2,'s  ')
      call twh_torstring(GLB_SET,type,3,'s  ')
      call twh_torstring(GLB_SET,type,4,'h  ')
      nsame = 0
      nsame = nsame + 1
      tnlocal(nsame,1) = 'h  '
      tnlocal(nsame,2) = 's  '
      tnlocal(nsame,3) = 's  '
      tnlocal(nsame,4) = 'h  '
      call twh_set_tornames(type,nsame,tnlocal)


      ndihedtypes = type
      call twh_ffcheck(lfailure,3,type)
      if ( lfailure ) return

      return
      end

      subroutine twh_ffqmff_viii_angleangle(lfailure,ffname,naatypes)
c     ******************************************************************
c     * sets the angle-angle types                                     *
c     *                                                                *
c     * originally written 05-10-2006 by M.G. Martin                   *
c     * last modified 08-04-2011 by M.G. Martin                        *
c     ******************************************************************
      implicit none
c     --- variables passed to/from the subroutine
      character*(FFNAMELEN) ffname
      logical lfailure
      integer naatypes
c     --- local variables
      integer type,nsame
      double precision aaclocal
      dimension aaclocal(MINAACOEFF:MAXAACOEFF)
      character*(FFNAMELEN) aanlocal
      dimension aanlocal(MAXSAMEAA,4)

      type = 0

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.20109d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.48450d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'c" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.58698d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'c` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -10.49140d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'c- '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.20109d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'c= '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.53480d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.45217d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -12.16241d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'n+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -4.72319d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'n= '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.44196d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'na '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.22092d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.22092d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'nn '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.96863d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.02750d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 's  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -16.48768d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -5.23487d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.23027d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.96604d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      5.78858d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.23027d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'nn '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.45410d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c- '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     38.03453d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c- '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'n+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.70223d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c= '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -5.32601d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'co '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.22092d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'co '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.44550d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'co '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.50584d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.44016d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'ct '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.56111d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      1.13855d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      3.59901d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'n+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'n= '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.62565d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'na '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -4.88356d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -4.88356d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'nn '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      3.01460d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      1.10345d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 's  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -7.79268d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.66013d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.60463d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c" '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.97422d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c" '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      8.23566d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c" '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c" '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.45102d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c" '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.08331d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c" '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      5.57611d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c" '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.86265d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      5.22208d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.29029d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      8.23566d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'nn '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.95695d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'co '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      6.53387d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -5.36224d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.08331d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'nn '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      5.57611d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -19.22210d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c- '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.06001d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c- '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     29.77424d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c- '
      aanlocal(nsame,4) = 'n+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.62114d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c- '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     17.51961d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c- '
      aanlocal(nsame,4) = 'n+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.20109d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c= '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      1.13336d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c= '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -7.59873d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c= '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c= '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -9.58886d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'c= '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.73203d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cl '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'cl '
      aanlocal(nsame,4) = 'cl '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.49978d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cl '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'cl '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -4.17900d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'co '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.22092d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'co '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.61297d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'co '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.63797d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'co '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.74225d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'co '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -4.88356d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'co '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -3.49550d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'co '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -9.30701d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.56178d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cl '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'cl '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.07828d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'ct '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      5.07674d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'f  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'f  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -5.00712d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.01242d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'ct '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.32430d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'ct '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.82814d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'ct '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      6.94105d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'f  '
      aanlocal(nsame,4) = 'f  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.29908d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'f  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'f  '
      aanlocal(nsame,4) = 'f  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.13772d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      1.27814d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'c" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     15.15367d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'c` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      3.56441d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'c- '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.87681d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'c= '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.50377d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'co '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      6.44388d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.74948d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'ct '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.31838d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     14.99191d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     11.59812d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'n+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.85176d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'n= '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.82390d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'na '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.28921d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.28921d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'nn '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.55544d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      5.58826d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 's  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.97539d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.05273d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.39293d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.54535d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      1.56919d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'co '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.48518d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     18.98506d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.39293d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'nn '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.54535d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      3.94992d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c- '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.83682d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c- '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'n+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.52677d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c= '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.13265d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c= '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.11886d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'co '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.28921d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'co '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     10.38386d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'co '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      3.42858d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'ct '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.20616d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.27963d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'ct '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.36715d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      7.21445d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'n+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      3.72738d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'n= '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.82807d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'na '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      6.18253d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      6.18253d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'nn '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.82077d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.36443d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 's  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.53569d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     18.86510d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n= '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     36.76216d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      8.40132d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 's  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 's  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -4.51294d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      5.75917d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'c" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -28.40678d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'c` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -13.45416d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.38691d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -12.00925d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      3.95778d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -16.33138d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'n+ '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     17.14028d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'n+ '
      aanlocal(nsame,4) = 'c- '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     11.37277d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'n+ '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     15.81909d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c- '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'n+ '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'n+ '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -5.91527d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'n= '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     12.07832d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'n= '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.05525d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'n= '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      7.52525d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'n= '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      6.90840d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'na '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.94197d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'na '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -4.12744d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'na '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.72468d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.72468d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'co '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.00442d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.72468d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'co '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'co '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.00442d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'co '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      3.71325d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.72468d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'nn '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      5.75917d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'nn '
      aanlocal(nsame,4) = 'c` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.00442d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'nn '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.38691d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'nn '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      3.71325d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'nn '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -4.15624d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'c" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -7.70807d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'co '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.19635d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      7.49830d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -3.43485d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -3.43485d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -9.00302d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'co '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.47476d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.04900d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.82858d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 's  '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.23960d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 's  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.19463d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 's  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      1.08621d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 's  '
      aanlocal(nsame,4) = 's  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     16.22629d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 's" '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      3.60621d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 's" '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.52113d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c  '
      aanlocal(nsame,3) = 's" '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o  '
      aanlocal(nsame,2) = 'c" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c` '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c+ '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c+ '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c+ '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'c+ '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c+ '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c+ '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c+ '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o- '
      aanlocal(nsame,2) = 'c- '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o- '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o- '
      aanlocal(nsame,2) = 'c- '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o- '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c- '
      aanlocal(nsame,3) = 'o- '
      aanlocal(nsame,4) = 'o- '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c- '
      aanlocal(nsame,3) = 'o- '
      aanlocal(nsame,4) = 'o- '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'c= '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'c= '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'c= '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     10.55553d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c= '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     32.23052d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -8.60238d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'n= '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'n= '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'n= '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'c= '
      aanlocal(nsame,3) = 'n= '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.22092d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      1.66311d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.63797d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -5.77924d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -5.56350d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -38.18039d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.66013d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.95695d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -5.06898d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      1.56919d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'c` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     34.08296d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     18.27376d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -4.90824d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     19.26868d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     22.06343d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     31.92571d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     36.76216d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.72468d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -6.25881d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.74752d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      9.01014d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     15.71383d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -12.75416d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.43857d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.31663d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      7.49830d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     15.23510d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.36840d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.04900d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'co '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -23.77698d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     23.46985d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c" '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c" '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c" '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c" '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c" '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.16408d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.05935d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     13.15517d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -14.52094d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'op '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -13.96861d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'sp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      9.28144d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      9.88710d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -6.84570d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'op '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -9.45522d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'sp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -10.70563d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c+ '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     25.88817d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c+ '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     14.55129d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c+ '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     48.11251d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c+ '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     50.79081d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c+ '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -5.79916d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c= '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     24.62254d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c= '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     25.52847d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c= '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     32.75254d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c= '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     34.68235d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'c= '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cl '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cl '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cl '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cl '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -24.62322d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.61034d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -20.65738d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -8.35168d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -9.62163d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     29.01924d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'op '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     19.44912d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'sp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -15.29534d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c+ '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.08486d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c+ '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -6.12151d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c+ '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.03400d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c= '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -3.92070d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c= '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      3.26653d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c= '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cl '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cl '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cl '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -28.80482d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.52113d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'ct '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'f  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      7.55961d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'n` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      5.65345d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'n+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     30.70076d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'n= '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -15.46769d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nn '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -29.08574d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     26.64143d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'op '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 's  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -9.34557d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'sp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      7.55961d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'f  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'f  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     10.96315d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     16.83137d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     66.87612d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'op '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     55.19448d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'sp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.29752d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n+ '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     14.77371d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n+ '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -10.92405d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nn '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -38.16838d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -7.61023d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nn '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -30.32255d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'op '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      6.96514d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -20.78099d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'sp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      7.55961d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cpb'
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     16.83137d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cpb'
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     66.87612d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cpb'
      aanlocal(nsame,4) = 'op '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     55.19448d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'cpb'
      aanlocal(nsame,4) = 'sp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.33882d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'ct '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'f  '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'f  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'f  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'f  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     13.05265d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     13.05265d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'cpb'
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     13.50920d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      5.64051d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     16.87517d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'op '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     15.40099d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'sp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      5.64051d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     16.87517d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'op '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     15.40099d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'sp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -7.03795d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.74976d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      6.45062d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'op '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -5.76690d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'sp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'n` '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'n` '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'n` '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'n` '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.54160d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'n+ '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -6.68719d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'n+ '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -11.02763d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'n+ '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -9.45306d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'n+ '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      8.92400d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'n+ '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'n= '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.69532d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -10.93072d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     11.23132d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     18.56865d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c+ '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     57.12883d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c+ '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.49359d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c= '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     32.75960d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c= '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cl '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cl '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -52.63238d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'f  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      6.35663d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'n` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      6.21611d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'n+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.61334d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'nn '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -46.04578d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -7.45556d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'f  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -3.38406d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     33.21437d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     12.23008d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n+ '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nn '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.28704d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     18.35378d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nn '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nn '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nn '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nn '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'nn '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -6.35949d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     11.57924d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -20.47105d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.32330d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'op '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -6.71365d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'sp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.04851d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c+ '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     21.59915d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c+ '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -17.68270d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c+ '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.22025d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c= '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     19.14029d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c= '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -11.62266d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c= '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cl '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cl '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      3.25742d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'f  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      9.16629d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'n` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     10.73801d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'n+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     32.27164d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'nn '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -13.00821d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'op '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -14.39404d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     15.27906d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'sp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      9.16629d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'f  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     29.41256d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -3.79098d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'op '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'sp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     10.33684d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n+ '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -17.77895d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n+ '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'nn '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     45.83445d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.57076d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nn '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -17.21178d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     27.95152d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'op '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -5.17503d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'op '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'op '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     65.30466d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'op '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     33.32818d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'op '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     65.30466d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'op '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'op '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 's  '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.75517d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 's" '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      8.51570d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 's" '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      7.65589d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 's" '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -7.25655d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 's" '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     21.33701d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 's" '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     23.27094d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'sp '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.33864d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'sp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -65.39008d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'sp '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     52.03715d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'sp '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'sp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     52.03715d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'sp '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'cp '
      aanlocal(nsame,3) = 'sp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -3.88966d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'cpb'
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     20.98391d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nn '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.06372d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.03386d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     80.99145d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'op '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     66.87847d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'sp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     16.25053d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'nn '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -7.42993d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'cpb'
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     10.79230d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'cpb'
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     22.24997d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'cpb'
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -23.93238d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'cpb'
      aanlocal(nsame,4) = 'op '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.07268d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'cpb'
      aanlocal(nsame,4) = 'sp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'nh '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'cpb'
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     23.99090d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'cpb'
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -9.15376d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'cpb'
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      6.80468d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'nn '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'nh '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -19.55902d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'nn '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -6.56786d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'nn '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     10.84967d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'cpb'
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'nh '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -21.28686d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     34.74773d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'op '
      aanlocal(nsame,4) = 'cpb'
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     73.40099d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'cpb'
      aanlocal(nsame,3) = 'sp '
      aanlocal(nsame,4) = 'cpb'
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -7.51127d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'c" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'c` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -5.44599d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -8.85001d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -7.37874d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'c` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -15.69517d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c+ '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      3.01682d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c= '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -7.58696d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      5.46262d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c" '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -3.78087d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c" '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.39771d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c" '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -15.06742d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c" '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     21.75346d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'c` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.98092d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c` '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -3.47311d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c+ '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -8.70954d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c+ '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     15.66404d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c= '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -13.42872d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'c= '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -21.07853d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -7.63877d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -10.68477d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'c" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'c` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      7.25878d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'c+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -6.61276d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.16255d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -2.98546d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'c" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -7.46300d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c" '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'c` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -3.27022d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c` '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      6.10166d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c+ '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -9.99737d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c= '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -3.92415d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -5.44599d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 's" '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -15.86151d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 's" '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     13.77720d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 's" '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -8.46757d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 's" '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -15.86151d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'n  '
      aanlocal(nsame,3) = 's" '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o` '
      aanlocal(nsame,2) = 'n` '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'n` '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      1.72938d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n+ '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n+ '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.22633d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n+ '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      3.29898d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'n+ '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -3.64709d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h+ '
      aanlocal(nsame,2) = 'n+ '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n+ '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      3.39378d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n+ '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'h+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -6.31162d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h+ '
      aanlocal(nsame,2) = 'n+ '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'h+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      6.68831d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n+ '
      aanlocal(nsame,3) = 'h+ '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      8.04185d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n+ '
      aanlocal(nsame,3) = 'h+ '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.32063d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'n+ '
      aanlocal(nsame,3) = 'h+ '
      aanlocal(nsame,4) = 'h+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      4.69204d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'n+ '
      aanlocal(nsame,3) = 'h+ '
      aanlocal(nsame,4) = 'h+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.00000d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h+ '
      aanlocal(nsame,2) = 'n+ '
      aanlocal(nsame,3) = 'h+ '
      aanlocal(nsame,4) = 'h+ '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.49542d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'na '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -11.77781d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'na '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      5.76675d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'na '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'na '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -11.52875d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'na '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      2.89373d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'na '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'na '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.87812d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'na '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -7.15923d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'na '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     10.11158d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'na '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'na '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     14.49953d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'na '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'na '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      6.21107d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'na '
      aanlocal(nsame,3) = 'na '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.08981d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'na '
      aanlocal(nsame,3) = 'na '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      1.56335d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'na '
      aanlocal(nsame,3) = 'na '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     11.39648d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -10.36403d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'co '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -9.55370d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.76619d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.15864d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.76619d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     12.00108d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      9.28219d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.76619d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'cpb'
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     12.00108d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'cpb'
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.14276d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      0.14276d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'cpb'
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      5.02413d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      5.02413d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.70424d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'np '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -15.56391d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.22788d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -5.31855d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -1.22788d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -6.21327d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'nh '
      aanlocal(nsame,3) = 'np '
      aanlocal(nsame,4) = 'np '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -7.95198d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'nn '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -32.41907d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'nn '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'cpb'
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -8.34763d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'nn '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -35.87498d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'nn '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -11.30480d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'nn '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      8.79518d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'nn '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -14.20245d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'nn '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -0.17192d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'nn '
      aanlocal(nsame,3) = 'cpb'
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      3.62785d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'nn '
      aanlocal(nsame,3) = 'cpb'
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =    -13.64993d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 'nn '
      aanlocal(nsame,3) = 'cpb'
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     21.63920d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'nn '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     14.27158d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 'nn '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'cpb'
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     -3.10090d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 'nn '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =      1.24566d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cpb'
      aanlocal(nsame,2) = 'nn '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     40.18016d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o  '
      aanlocal(nsame,2) = 'p  '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'o- '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     67.43770d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o- '
      aanlocal(nsame,2) = 'p  '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'o- '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     24.47420d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o  '
      aanlocal(nsame,2) = 'p  '
      aanlocal(nsame,3) = 'o- '
      aanlocal(nsame,4) = 'o  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     79.94926d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o  '
      aanlocal(nsame,2) = 'p  '
      aanlocal(nsame,3) = 'o- '
      aanlocal(nsame,4) = 'o- '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     16.88806d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o- '
      aanlocal(nsame,2) = 'p- '
      aanlocal(nsame,3) = 'o  '
      aanlocal(nsame,4) = 'o- '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     11.74695d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o  '
      aanlocal(nsame,2) = 'p- '
      aanlocal(nsame,3) = 'o- '
      aanlocal(nsame,4) = 'o- '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     42.68981d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o- '
      aanlocal(nsame,2) = 'p- '
      aanlocal(nsame,3) = 'o- '
      aanlocal(nsame,4) = 'o- '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     52.35979d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     34.72976d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     63.94875d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     66.75040d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     89.27796d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o` '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'c  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     12.47297d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     31.42542d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     31.31363d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o` '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'cp '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     73.81797d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     81.35770d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     70.34304d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     94.78144d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o` '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'h  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     43.08492d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     23.90214d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     39.71268d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     66.60652d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'o` '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'n  '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     22.01490d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'c  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     12.78693d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'cp '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     26.90365d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     21.95011d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     14.37280d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'c  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     11.39788d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     10.90984d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'cp '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     33.24624d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'h  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     32.45804d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'n  '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     16.74507d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'h  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      type = type + 1
      call twh_aaff(GLB_SET,type,ffname)
      call twh_aastyle(GLB_SET,type,1)
      aaclocal(0) =     10.94628d0*CON_KCALTOK
      call twh_set_aacoeff(type,aaclocal)
      nsame = 0
      nsame = nsame + 1
      aanlocal(nsame,1) = 'n  '
      aanlocal(nsame,2) = 's" '
      aanlocal(nsame,3) = 'o` '
      aanlocal(nsame,4) = 'o` '
      call twh_set_aanames(type,nsame,aanlocal)

      naatypes = type
      call twh_ffcheck(lfailure,5,type)
      if ( lfailure ) return

      return
      end

      subroutine twh_ffqmff_viii_bond_increment(lfailure,ffname
     &     ,nbitypes)
c     ******************************************************************
c     * sets the bond increments                                       *
c     *                                                                *
c     * originally written 05-10-2006 by M.G. Martin                   *
c     * last modified 08-04-2011 by M.G. Martin                        *
c     ******************************************************************
      implicit none
c     --- variables passed to/from the subroutine
      character*(FFNAMELEN) ffname
      logical lfailure
      integer nbitypes
c     --- local variables
      integer type
      double precision qbivlocal
      character*(FFNAMELEN) qbinlocal
      dimension qbinlocal(2)

      type = 0

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'Br '
      qbinlocal(2) = 'Br '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'Ca '
      qbinlocal(2) = 'Ca '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'Cl '
      qbinlocal(2) = 'Cl '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'Cs '
      qbinlocal(2) = 'Cs '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'Cu '
      qbinlocal(2) = 'Cu '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'F  '
      qbinlocal(2) = 'F  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'Fe '
      qbinlocal(2) = 'Fe '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'I  '
      qbinlocal(2) = 'I  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'K  '
      qbinlocal(2) = 'K  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'Li '
      qbinlocal(2) = 'Li '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'Mg '
      qbinlocal(2) = 'Mg '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'Na '
      qbinlocal(2) = 'Na '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'Rb '
      qbinlocal(2) = 'Rb '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'Zn '
      qbinlocal(2) = 'Zn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'br '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.19200d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'c  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.08000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'c" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.08000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'c` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.02810d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'c- '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.08000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'c= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.05070d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'cl '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.08000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'cp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.01730d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'ct '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.20990d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'f  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.19780d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'h  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.01100d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'i  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.14220d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'n  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.24520d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'n+ '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.14220d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'n= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.06010d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'na '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.04380d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.14220d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.14220d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.08180d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.31400d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.31400d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.21560d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.04370d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00340d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00340d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'br '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'c  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'c" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'c` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.23000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'c- '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'c= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.01100d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'cl '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'co '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'cp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.04415d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'ct '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.13275d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'f  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.05300d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'h  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.22700d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'hc+'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.11200d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'i  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'n  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.20500d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'n+ '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.24050d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'n= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.08270d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'na '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.21080d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.21080d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.48020d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'nr '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.11330d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.39570d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.01100d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.06500d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.05000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.11800d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c  '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'c" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'c` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.13680d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'c- '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'c= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.10200d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'cl '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'co '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.06500d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'cp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.09270d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'ct '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.13000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'f  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.04560d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'h  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.12910d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'i  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'n  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.13310d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'n+ '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.03620d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'n= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.14220d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.03620d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.03620d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00300d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.39640d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.23960d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.01400d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c" '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'c` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.13680d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'c- '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'c= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.10200d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'cl '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.06500d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'cp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.09270d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'ct '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.13000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'f  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'h  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.12910d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'i  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'n  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.11690d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'n+ '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.03620d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'n= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.07000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.03620d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.03000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00300d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.23960d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.01400d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c` '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c+ '
      qbinlocal(2) = 'cp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.12680d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c+ '
      qbinlocal(2) = 'h  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.06800d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c+ '
      qbinlocal(2) = 'n  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.06800d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c+ '
      qbinlocal(2) = 'nr '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'c- '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.07470d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'cl '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.04240d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'cp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.04320d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'ct '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.22410d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'f  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.05300d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'h  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.01850d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'i  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.16070d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'n  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.25970d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'n+ '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.16070d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'n= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.01760d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.16070d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.16070d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.10120d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.32410d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.03370d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'o- '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.32410d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.08570d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.00870d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.12230d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c- '
      qbinlocal(2) = 's- '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 'c= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.10200d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 'cl '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.03000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 'cp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.08520d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 'ct '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.13000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 'f  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.12680d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 'h  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.06420d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 'i  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.07000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 'n  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.29890d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 'n+ '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.36000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 'n= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.15000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.35830d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.35830d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.03800d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.01200d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.07320d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.07320d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'c= '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'cl '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.10000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'cp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.03190d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'ct '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.15890d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'f  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.24040d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'h  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.06230d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'i  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.08970d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'n  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.18580d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'n+ '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.08970d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'n= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.01170d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'na '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.08540d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.08970d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.08970d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.03670d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.25850d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.25850d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.25440d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.08980d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.04570d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.04570d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cl '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.05300d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'co '
      qbinlocal(2) = 'h  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.08270d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'co '
      qbinlocal(2) = 'na '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.21080d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'co '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.11330d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'co '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'cp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'cpb'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.07904d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'ct '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.11600d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'f  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.14000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'h  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.06420d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'i  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.17460d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'n  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.20000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'n` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.29890d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'n+ '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.13800d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'n= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.16000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.04000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.18000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.15000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.39640d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.11000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.01200d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.07320d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.02000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cp '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cpb'
      qbinlocal(2) = 'cpb'
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.16000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cpb'
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cpb'
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.18000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cpb'
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.11000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cpb'
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.02000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cpb'
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.13800d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cr '
      qbinlocal(2) = 'n  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.17460d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'cr '
      qbinlocal(2) = 'nr '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 'ct '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.18730d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 'f  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.21833d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 'h  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.02810d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 'i  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.12040d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 'n  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.09920d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 'n+ '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.12040d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 'n= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.06360d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 'na '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.05680d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.09200d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.12040d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.06750d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.28740d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.28740d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.13350d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.05810d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.01350d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.01350d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'ct '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 'f  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.38230d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 'h  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.22340d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 'i  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.07310d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 'n  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00620d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 'n+ '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.07310d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 'n= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.14150d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 'na '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.22200d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.07310d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.07310d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.10770d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.08880d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.08880d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.38690d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.23800d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.20110d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.20110d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'f  '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 'h  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.19230d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 'i  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.49430d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.49430d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.03560d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.19000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.05000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.19320d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 'n  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.40000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 'n` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.28000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 'n= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.24870d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 'na '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.37000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.37000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.32780d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.40680d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 'nr '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.42410d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.39910d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h  '
      qbinlocal(2) = 'o? '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.28000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h+ '
      qbinlocal(2) = 'n+ '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.37000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'h+ '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'i  '
      qbinlocal(2) = 'i  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.15540d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'i  '
      qbinlocal(2) = 'n  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.26150d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'i  '
      qbinlocal(2) = 'n+ '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.15540d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'i  '
      qbinlocal(2) = 'n= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.07140d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'i  '
      qbinlocal(2) = 'na '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.03560d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'i  '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.15540d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'i  '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.15540d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'i  '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.09240d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'i  '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.32970d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'i  '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.32970d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'i  '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.21100d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'i  '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.03450d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'i  '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.01400d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'i  '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.01400d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'i  '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n  '
      qbinlocal(2) = 'n  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.08830d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n  '
      qbinlocal(2) = 'n+ '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n  '
      qbinlocal(2) = 'n= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.07420d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n  '
      qbinlocal(2) = 'na '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.16370d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n  '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n  '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n  '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.04270d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n  '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.16840d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n  '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.16840d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n  '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.33590d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n  '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.17530d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n  '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.15500d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n  '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.13460d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n  '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.33000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n` '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n+ '
      qbinlocal(2) = 'n+ '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.08830d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n+ '
      qbinlocal(2) = 'n= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.16500d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n+ '
      qbinlocal(2) = 'na '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.25440d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n+ '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.16500d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n+ '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.08830d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n+ '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.12500d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n+ '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.09180d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n+ '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.09180d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n+ '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.44940d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n+ '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.27550d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n+ '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.23410d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n+ '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.23410d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n+ '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n= '
      qbinlocal(2) = 'n= '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.07420d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n= '
      qbinlocal(2) = 'na '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.16370d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n= '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n= '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n= '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.04270d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n= '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.16840d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n= '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.16840d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n= '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.33590d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n= '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.17530d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n= '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.13460d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n= '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.13460d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'n= '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'na '
      qbinlocal(2) = 'na '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.09190d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'na '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.03710d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'na '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.07420d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'na '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.02540d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'na '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.23690d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'na '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.23690d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'na '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.25180d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'na '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.09660d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'na '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.05510d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'na '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.05510d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'na '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nh '
      qbinlocal(2) = 'nh '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.16370d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nh '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.15000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nh '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.10900d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nh '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.31480d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nh '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.31480d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nh '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.13750d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nh '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00460d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nh '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.04540d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nh '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.04540d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nh '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nn '
      qbinlocal(2) = 'nn '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nn '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.04270d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nn '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.16840d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nn '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.16840d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nn '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.33590d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nn '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.17530d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nn '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.13460d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nn '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.13460d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'nn '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'np '
      qbinlocal(2) = 'np '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.04270d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'np '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.16840d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'np '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.05000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'np '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.33590d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'np '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.17530d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'np '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.13460d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'np '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.13000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'np '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'o  '
      qbinlocal(2) = 'o  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.19620d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'o  '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.19620d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'o  '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.34800d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'o  '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.34800d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'o  '
      qbinlocal(2) = 'p- '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.11430d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'o  '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.07660d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'o  '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.07660d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'o  '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'o` '
      qbinlocal(2) = 'o` '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'o` '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.49330d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'o` '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.33860d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'o` '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.68000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'o` '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.30240d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'o` '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.31300d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'o- '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.31300d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'o- '
      qbinlocal(2) = 'p- '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'op '
      qbinlocal(2) = 'op '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.49330d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'op '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.33860d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'op '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.30240d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'op '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.30240d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'op '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =     -0.40000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'os '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'p  '
      qbinlocal(2) = 'p  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.16000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'p  '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.21060d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'p  '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.18240d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'p  '
      qbinlocal(2) = 's- '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.21060d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'p  '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 's  '
      qbinlocal(2) = 's  '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.04550d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 's  '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.04550d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 's  '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 's" '
      qbinlocal(2) = 's" '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 's" '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      type = type + 1
      call twh_qbiff(GLB_SET,type,ffname)
      qbivlocal =      0.00000d0
      call twh_qbivalue(GLB_SET,type,qbivlocal)
      qbinlocal(1) = 'sp '
      qbinlocal(2) = 'sp '
      call twh_set_qbinames(type,qbinlocal)

      nbitypes = type
      call twh_ffcheck(lfailure,6,type)
      if ( lfailure ) return

      return
      end
